# -*- coding: utf-8 -*-
from osv import osv
from osv import fields
import base64
from tools.translate import _
import binascii
from StringIO import StringIO
import csv
import time
from datetime import datetime
import xmlrpclib

class comprobante_contable(osv.osv):
    """
    Modulo para la carga de comprobantes contables.
    Crea una tabla en la base de datos para almacenar información con respecto
    al comprobante. La tabla es borrada al final del proceso de carga.
    """
    _name = "comprobante.contable"
    _description = "Account Move Common Report"
    
    _columns = {
        'user_name': fields.char('Usuario', size=64,help="Usuario de la base de datos"),
        'pwd_user': fields.char('Password', size=64,help="Contraseña de la base de datos"),
        'ref_des': fields.char('Descripcion del Comprobante',size=128),
        'file': fields.binary('Archivo CSV'),
        'f_name': fields.char('Nombre del CSV', size=128),
        'period_id': fields.many2one('account.period','Periodo del comprobante', required=True, select=True),
        'journal_id': fields.many2one('account.journal', 'Diario del comprobante', required=True, select=True),
        }
        
    def get_id_(self, cr, uid, values,obj,field):
        """
        Busca el id de un objeto que se le pase por [obj], donde se cumpla que la 
        variable [field] igual a la variable [values]
        """
        ids = self.pool.get(obj).search(cr,uid,[(str(field),'=',str(values))])
        if ids:
            return int(ids[0])
        else:
            return 0
    
    def upload_account_move(self, cr, uid, defaults, context=None):
        
        """
        Funcion para la carga del comprobante contable, en el proceso primero se
        sube el archivo a la base de datos para despues ser extraido, convertido
        en un archivo virtual que sera procesado en dos pasos, primero carga del
        account.move y despues account.move.line.
        """
        record_id = self.browse(cr, uid, defaults, context=context)
        
        # Se ejecuta un query para buscar el archivo que se guardo en la base
        #de datos        
        sql = 'select file, ref_des, period_id, journal_id from comprobante_contable where id=%s'%(defaults[0])
        cr.execute (sql)
        var_buffer = cr.fetchone()
        
        #cargo los datos del webservice
        username = record_id[0].user_name #the user
        pwd = record_id[0].pwd_user #the password of the user
        dbname = cr.dbname
        print username
        print pwd
        print dbname
        HOST= 'localhost' #'sisb.industriasdiana.gob.ve' #IP o Nombre del servidor
        PORT=8069 #Puerto del Servidor
        URL='http://%s:%d/xmlrpc/common'%(HOST,PORT)
        sock_common = xmlrpclib.ServerProxy (URL)
        #uidxml = sock_common.login(dbname, username, pwd)
        sock = xmlrpclib.ServerProxy('http://%s:%d/xmlrpc/object'%(HOST,PORT))
        
        
        #Se convierte el archivo de binario a lectura estandar y despues se
        #pasa a un archivo virtual
        compr_csv = binascii.a2b_base64(unicode(var_buffer[0]))
        arch_csv = StringIO(compr_csv)
        
        f_move = csv.DictReader(arch_csv, delimiter='|')
        dic = {}
        ids = []
        fallo = False
        vista = False
        for row in f_move:
            ids = sock.execute(dbname,uid,pwd,'account.account','search', [('code','=',row['cuenta'])])
            tipo = sock.execute(dbname,uid,pwd,'account.account','read',ids,['type'])
            if not ids:
                dic[str(row['cuenta'])] = row['descripcion']
                fallo = True
            elif tipo[0]['type']=='view':
                dic[str(row['cuenta'])] = row['descripcion']
                vista =True
        if vista == True:
            print "No se va a completar la operacion, las siguientes cuentas son tipo vista:"
            for d in dic.keys():
                print "Cuenta: %s \tNombre: %s"%(d,dic[d])
        if fallo == True:
            print "No se puede completar la operacion, las siguientes cuentas no existen en el catalogo:"
            for d in dic.keys():
                print "Cuenta: %s \tNombre: %s"%(d,dic[d])
            raise osv.except_osv(_('Error !'), _('El Comprobante no tiene una cuenta definida! %s')%(dic[d]))
            return {
                'name':("Importacion CSV"),
                'view_mode': 'form',
                'view_id': False,
                'view_type': 'form',
                'res_model': 'comprobante.contable',
                'type': 'ir.actions.act_window',
                'nodestroy': True,
                'target': 'new',
                'ref': 'comprobante_contable.view_comprobante_contable_end',
            }
        
        company_id = self.pool.get('res.company')._company_default_get(cr, uid, 'account.move', context=context)
        fecha = time.strftime('%Y-%m-%d')
        #Se carga el account.move
        moves_id = self.pool.get('account.move').search(cr,uid,[('journal_id','=',var_buffer[3]),('period_id','=', var_buffer[2])])
        if moves_id == []:
            moves = {
                'name': "/",
                'ref' : var_buffer[1],
                'to_check': 1,
                'journal_id': var_buffer[3],
                'period_id': var_buffer[2],
                'date': fecha,
                'state': 'draft',
                'company_id': int(company_id),
            }

            moves_id = sock.execute(dbname, uid, pwd, 'account.move', 'create', moves)
        if type(moves_id) == type([]):
            moves_id = moves_id[0]
        #Se vuelve a crear el archivo virtual para reiniciar la lectura
        #Se hace para la carga de los account.move.line
        arch_csv = StringIO(compr_csv)
        f_move = csv.DictReader(arch_csv, delimiter='|')
        obj_acm = self.pool.get('account.move.line')
        obj_aca = self.pool.get('account.account')
        i = 0
        lineas = 0
        for line in f_move:
            lineas += 1
        arch_csv = StringIO(compr_csv)
        f_move = csv.DictReader(arch_csv, delimiter='|')
        for mov in f_move:
            var_journal = var_buffer[3]
            var_period = var_buffer[2]
            var_move_id = moves_id
            var_account_id = self.get_id_(cr, uid, mov['cuenta'], 'account.account', 'code')
            var_name = obj_aca.read(cr,uid,var_account_id,['name'])
            cre = mov['haber'] if mov['haber'] != '' else '0.00' 
            deb = mov['debe'] if mov['debe'] != '' else '0.00'
            if var_account_id != 0:
                moves_lines = {
                    'journal_id': var_buffer[3],
                    'ref' : var_buffer[1],
                    'credit': cre,
                    'debit': deb,
                    'account_id': int(var_account_id),
                    'period_id': var_buffer[2], 
                    'move_id': int(var_move_id),
                    'name': var_name['name'][:63],
                    'state': 'draft',
                }
                i += 1
                if i == 1:
                    moves_lines_id = obj_acm.create(cr,uid,moves_lines,context=context)
                elif i == lineas:
                    moves_lines_id = obj_acm.create(cr,uid,moves_lines,context=context)
                else:
                    cr.execute("insert into account_move_line (account_id,name,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,ref,move_id,blocked,create_uid,create_date) values (%s,'%s','normal',%s,%s,NULL,%s,'%s',%s,%s,now(),now(),'Carga por Lotes',%s,False,%s,now())"%(moves_lines['account_id'],moves_lines['name'],moves_lines['journal_id'],int(company_id),moves_lines['credit'],moves_lines['state'],moves_lines['period_id'],moves_lines['debit'],moves_lines['move_id'],uid))
                
                cr.commit()
            else:
                raise osv.except_osv(_('Error !'), _('El Comprobante no tiene una cuenta definida! %s')%(mov['cuenta']))
        cr.execute('delete from comprobante_contable')
        
        return {
            'name':("Importacion CSV"),
            'view_mode': 'form',
            'view_id': False,
            'view_type': 'form',
            'res_model': 'comprobante.contable',
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'new',
            'ref': 'comprobante_contable.view_comprobante_contable_end',
        }

comprobante_contable()

class comprobante_contable_report(osv.osv):
    _name = "comprobante.contable.report"
    _description = "Reporte de Comprobantes Contables"
    
    def execute_report(self, cr, uid, values, context=None):
        return True

comprobante_contable_report()
    
