# -*- coding: utf-8 -*-
"""
Modulo utilizado para filtrar los albaranes, movimientos de inventario e inventarios fisicos en funcion a las ubicaciones permitidas al usuario,
para ello se debe asociar al almacen los grupos de usuarios que tendran acceso al mismo.
Adicionalmente se filtra la visualizacion de los productos segun las categorias permitidas en cada ubicacion,
para ello se debe asociar al almacen las categorias que tendran acceso desde el mismo.

Se crean campos funcionales para resolver este requerimiento, filtrando por el metodo search y read.

"""
from osv import fields, osv
import netsvc
import stock
import product
import time
from tools.translate import _

from datetime import datetime


class diana_stock_procurement(osv.osv):
    _name = 'diana.stock.procurement'
    _description = 'Abastecimiento Masivo'
        
    _columns = {
        'name':fields.char('Referencia de Abastecimiento', size=129),
        'date' : fields.date('Date', required=True),
        'responsible_id':fields.many2one('res.users', 'Responsible', ondelete='set null'),
        'procurement_order_ids' : fields.one2many('procurement.order', 'diana_stock_procurement_id', 'Procurement lines'),
        'state': fields.selection([('draft','Borrador'), ('exception','Excepcion'), ('done','Procesado'), ('cancel','Cancelado')], 'State', readonly=True),
        #~ 'global_notes': fields.text('Notas Generales'),
    }

    _defaults = {
        'name': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'diana.stock.procurement'),
        'date': lambda *args: time.strftime('%Y-%m-%d'),
        'responsible_id' : lambda self, cr, uid, context : uid,
        'state': 'draft',
    }
    
    #Metodo que se ejecuta al presionar el boton solicitar_abastecimiento
    def solicitar_abastecimiento(self, cr, uid, ids, context=None): 
        wf_service = netsvc.LocalService("workflow")
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context).login
        proc_obj = self.pool.get('procurement.order')
        pur_obj = self.pool.get('purchase.order')
        pur_line_obj = self.pool.get('purchase.order.line')
        pur_req_obj = self.pool.get('purchase.requisition')
        data_obj = self.pool.get('ir.model.data')
        p_id = 0
        
        for each_brw in self.browse(cr, uid, ids, context=context): #Devuelve objetos de tipo diana.stock.procurement

            if len(each_brw.procurement_order_ids) == 0 :
                raise osv.except_osv(_('Data Insufficient !'),_('Please entry lines!'))
            
            valida = True
            #~ general = each_brw.global_notes or ""
            
            for each in each_brw.procurement_order_ids: # Recorro las procurement.order contenidas en cada uno de los diana.stock.procurement
                p_id = each.id
                proc_obj.write(cr, uid, each.id, {
                    'name':'INT: '+str(user),
                    'procure_method':'make_to_order',
                    'origin': each_brw.name,
                    #~ 'global_notes':general,
                })
                wf_service.trg_validate(uid, 'procurement.order', each.id, 'button_confirm', cr) #Pasa el objeto procurement.order del estado draft al estado confirm
                wf_service.trg_validate(uid, 'procurement.order', each.id, 'button_check', cr) #Pasa el objeto procurement.order del estado confirm al estado check
                
                #Verificar que se ejecuto correctamente la solicitud
                estado = str(proc_obj.read(cr, uid, each.id, ['state'])['state'])
                if estado != 'running':
                    valida = False
                
                """
                El siguiente bloque consulta la solicitud de compra (purchase.requisition) y pedido de compra (purchase.order)
                que se acaban de generar para colocarle a dicho pedido de compra el documento de origen = nombre de la solicitud de compra
                """
        """
        Colocar el estado de la solicitud en aprobado si fue realizado con exito
        la misma, sino se coloca excepcion.
        """
        if valida:
            self.write(cr, uid, ids, {'state':'done'}, context=context)
        else:
            self.write(cr, uid, ids, {'state':'exception'}, context=context)
        
        """
        Lo siguiente se agrega en el context para que al abrir la ventana que retorna
        esta funcion, la misma se encuentre tambien con el filtro.
        """
        context['search_default_create_uid'] = uid
        
        return { 
            'view_type': 'form',
            'view_mode':'tree,form',
            'res_model': 'procurement.order',
            'type': 'ir.actions.act_window',
            'target': 'edit',
            'context': context,
            'res_id': p_id,
            'ref': 'procurement.procurement_tree_view'
        }



    def solicitar_servicio(self, cr, uid, ids, context=None):
        origin = self.read(cr,uid,ids,['name'])
        context.update({'diana_stock_procurement_id':ids, 'origin':origin})
        return { 
            'view_type': 'form',
            'view_mode':'form',
            'res_model': 'diana.stock.procurement.service',
            'type': 'ir.actions.act_window',
            'target': 'new',
            'context': context,
            'ref': 'diana_stock_procurement_service.servicio_form_view'
        }
        
diana_stock_procurement()

class diana_groups_location_rel(osv.osv):
    _name = 'diana.groups.location.rel'
    _description = "Diana Groups Locations Relations"

    _columns = {
        'groups_id': fields.many2one('res.groups', 'Allowed Groups' , required=True),
        'location_id': fields.many2one('stock.location', 'Groups Allowed Location' , required=True),
        }
    
    _sql_constraints = [('unique_group_location_relation','unique(groups_id, location_id)', _('The combination of group and location must be unique'))]
    
diana_groups_location_rel()

class diana_category_location_rel(osv.osv):
    _name = 'diana.category.location.rel'
    _description = "Diana Category Locations Relations"

    _columns = {
        'category_id': fields.many2one('product.category', 'Allowed Categories', required=True),
        'location_id': fields.many2one('stock.location', 'Groups Allowed Location', required=True),
        }
        
    _sql_constraints = [('unique_category_location_relation','unique(category_id, location_id)', _('The combination of category and location must be unique'))]
    
diana_category_location_rel()

class stock_location(osv.osv):
    _name = 'stock.location'
    _inherit = 'stock.location'
    _description = "Stock Location"

    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct slh.id as child_id from stock_location slp
            left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
            inner join diana_groups_location_rel dlr on dlr.location_id =slp.id
            where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)""", (uid,))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
                        
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if (uid <> 1):
            lista = self.lista_permitidos(cr, uid, context)     
            dic = 'id', 'in', lista
            args.append(dic)
        return super(stock_location, self).search(cr, uid, args, offset, limit, order, context, count)

    def _get_location_allowed(self, cr, uid, ids, field_names, arg=None, context=None):
        res = {}
        stock_location_ids = self.search(cr, uid, [], context=context)
        stock_location = self.browse(cr, uid, stock_location_ids)
        
        user_groups = []
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        
        for i,j in enumerate(user.groups_id): #Lista con todos los grupos a los que tiene acceso el usuario
            user_groups.append(user.groups_id[i].id)
        
        sua_ids = self.pool.get('diana.groups.location.rel').search(cr, uid, [('groups_id', 'in', user_groups)], context=context) #consulta de los registros diana.groups.location.rel que tiene un group_id al que tiene acceso el usuario
        sua_obj = self.pool.get('diana.groups.location.rel').browse(cr, uid, sua_ids)
        
        loc_list = []
        for s in sua_obj:
            child_ids = self.pool.get('stock.location').search(cr, uid, [('location_id', 'child_of', s.location_id.id)], context=context) # El grupo tiene permiso sobre el propio almacen y sobre todos sus hijos
            for item in child_ids:
                if not item in loc_list: 
                    loc_list.append(item)
        
        for item in stock_location:
            if item.id in loc_list:
                res[item.id] = 'Si'
            else:
                res[item.id] = 'No'
        return res
                
    _columns = {
        'groups_ids': fields.one2many('diana.groups.location.rel', 'location_id', 'Allowed Groups in Stock Location'),
        'category_ids': fields.one2many('diana.category.location.rel', 'location_id', 'Allowed Categories in Stock Location'),
        'stock_analytic_account_id': fields.many2one('account.analytic.account', 'Analytic Account in Stock Location'),
        }
    
    def _product_reserve(self, cr, uid, ids, product_id, product_qty, context=None, lock=False):
        """
        Attempt to find a quantity ``product_qty`` (in the product's default uom or the uom passed in ``context``) of product ``product_id``
        in locations with id ``ids`` and their child locations. If ``lock`` is True, the stock.move lines
        of product with id ``product_id`` in the searched location will be write-locked using Postgres's
        "FOR UPDATE NOWAIT" option until the transaction is committed or rolled back, to prevent reserving
        twice the same products.
        If ``lock`` is True and the lock cannot be obtained (because another transaction has locked some of
        the same stock.move lines), a log line will be output and False will be returned, as if there was
        not enough stock.

        :param product_id: Id of product to reserve
        :param product_qty: Quantity of product to reserve (in the uom passed in ``context``)
        :param lock: if True, the stock.move lines of product with id ``product_id`` in all locations (and children locations) with ``ids`` will
                     be write-locked using postgres's "FOR UPDATE NOWAIT" option until the transaction is committed or rolled back. This is
                     to prevent reserving twice the same products.
        :param context: context dictionary with 'uom' key mapped to the ID of the UoM to use to compute the product quantities
        :return: List of pairs (qty, location_id) with the (partial) quantities that can be taken in each location to
                 reach the requested product_qty (expressed in the requested uom), or False if not enough
                 products could be found, or the lock could not be obtained (and ``lock`` was True).
                 sum(qty) == ``product_qty``.
        """
        if context is None:
            context = {}
        location_ids = self.search(cr, uid, [('location_id', 'child_of', ids)])
        locations_tuple = tuple(location_ids)
        if lock and not self._try_lock_product_reserve(cr, uid, locations_tuple, product_id, product_qty, context=context):
            return False

        # Giant query to obtain triplets of (product_uom, product_qty, location_id) summing all relevant
        # stock moves quantities per location,  with incoming quantities taken positive,
        # and outgoing taken negative.
        cr.execute("""SELECT x.product_uom, SUM(x.coeff * x.product_qty) as product_qty, x.loc_id as location_id
                      FROM (
                          SELECT 1.0 as coeff, product_uom, location_dest_id as loc_id,
                                 sum(product_qty) AS product_qty
                              FROM stock_move
                              WHERE location_dest_id in %s AND
                                    location_id != location_dest_id AND
                                    product_id = %s AND
                                    state = 'done'
                              GROUP BY location_dest_id, product_uom
                          UNION
                          SELECT -1.0 as coeff, product_uom, location_id as loc_id,
                                 sum(product_qty) AS product_qty
                              FROM stock_move
                              WHERE location_id in %s AND
                                    location_id != location_dest_id AND
                                    product_id = %s AND
                                    state in ('done', 'assigned')
                              GROUP BY location_id, product_uom
                      ) AS x
                      GROUP BY x.loc_id, x.product_uom
                   """,
                   (locations_tuple, product_id, locations_tuple, product_id))
        sum_rows = cr.fetchall()

        qty_by_location = {}
        ProductUom = self.pool.get('product.uom')
        target_uom = context.get('uom')
        # Convert all UoMs into target UoM
        for uom_id, qty, loc_id in sum_rows:
            qty_by_location.setdefault(loc_id,0.0)
            qty_by_location[loc_id] += ProductUom._compute_qty(cr, uid, uom_id, qty, target_uom)

        # to compute final result we handle locations in the
        # order in which they were returned by the original search().
        result = []
        for loc_id in location_ids:
            if loc_id not in qty_by_location:
                #skip location without this product
                continue
            qty = qty_by_location[loc_id]
            if qty <= 0.0:
                continue
            qty = min(product_qty, qty)
            result.append((qty, loc_id))
            product_qty -= qty
            """
            Se realiza redondeo para evitar errores con la precision decimal
            de python.
            Se instancia decimal precision para usar la misma cantidad de
            digitos que se este usando en almacenes.
            """
            decimal_obj = self.pool.get('decimal.precision')
            product_qty = round(product_qty, decimal_obj.precision_get(cr, uid, 'Product UoM'))
            ####################################################################
            if product_qty <= 0.0:
                return result
        return False
    
        
stock_location()

class stock_picking(osv.osv):
    _name = 'stock.picking'
    _inherit = 'stock.picking'
    _description = "Stock Picking"

            
    def copy(self, cr, uid, id, defaults, context=None): #Sobreescritura del metodo copy para no permitir duplicar objetos stock_picking
        if not context is None:
            raise osv.except_osv(_('Acción Denegada!'),_('No se permite duplicar albaranes!'))
            return False
        else:
            return super(stock_picking, self).copy(cr, uid, id, defaults, context)
            
    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct sm.picking_id from stock_move sm 
                    inner join (
                    select distinct slp.id as parent_id, slh.id as child_id from stock_location slp
                    left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
                    inner join diana_groups_location_rel dlr on dlr.location_id = slp.id
                    where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)) 
                    as tbld on sm.location_id = tbld.child_id or sm.location_dest_id = tbld.child_id
                    inner join product_product p on sm.product_id =  p.id
                    inner join product_template pt on p.product_tmpl_id = pt.id
                    inner join (                
                    select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                    left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                    inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                    where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                    as tbld2 on pt.categ_id = tbld2.child_id""", (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
        
                
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if uid <> 1:
            lista = self.lista_permitidos(cr, uid, context)                  
            dic = 'id', 'in', lista
            args.append(dic)
        return super(stock_picking, self).search(cr, uid, args, offset, limit, order, context, count)
        
    def lista_productos_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct p.id from product_product p inner join product_template pt on p.product_tmpl_id = pt.id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_location_rel dcr on dcr.category_id =slp.id
                inner join diana_groups_location_rel dlr on dlr.location_id = dcr.location_id
                where dlr.groups_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld on pt.categ_id = tbld.child_id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld2 on pt.categ_id = tbld2.child_id """, (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
            
    def validar_lineas(self, cr, uid, ids, context=None):
        if uid == 1:
            return (True, '') 
        picking = self.browse(cr, uid, ids)
        lines = self.pool.get('stock.move').search(cr, 1, [('picking_id', '=', picking[0].id)], context=context)
        lines_ = self.pool.get('stock.move').search(cr, uid, [('picking_id', '=', picking[0].id)], context=context)
        if len(lines) <> len(lines_):
            return (False, 'No puede procesar el albaran. Existen líneas no permitidas al usuario actual!')
        else:
            productos_permitidos = self.lista_productos_permitidos(cr, uid, context)
            for item in lines_:
                line_obj = self.pool.get('stock.move').browse(cr, uid, item)
                if not line_obj.product_id.id in productos_permitidos:
                    return (False, 'No puede procesar el albaran. Existen líneas con productos no permitidos al usuario actual!')
            return (True, '') 
    
    def action_process(self, cr, uid, ids, context=None):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).action_process(cr, uid, ids, context)
            
    def force_assign(self, cr, uid, ids, *args):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context=None)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).force_assign(cr, uid, ids, *args)
            
    def draft_force_assign(self, cr, uid, ids, *args):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context=None)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).draft_force_assign(cr, uid, ids, *args)
            
    def action_cancel(self, cr, uid, ids, context=None):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).action_cancel(cr, uid, ids, context)
            
    def action_done(self, cr, uid, ids, context=None):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).action_done(cr, uid, ids, context)
            
    def action_move(self, cr, uid, ids, context=None):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).action_move(cr, uid, ids, context)
            
    def cancel_assign(self, cr, uid, ids, *args):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context=None)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).cancel_assign(cr, uid, ids, *args)
            
    def draft_validate(self, cr, uid, ids, context=None):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).draft_validate(cr, uid, ids, context)
            
    def action_assign(self, cr, uid, ids, *args):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context=None)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_picking, self).action_assign(cr, uid, ids, *args)
            
    def do_partial(self, cr, uid, ids, partial_datas, context=None):
        """ Makes partial picking and moves done.
        @param partial_datas : Dictionary containing details of partial picking
                          like partner_id, address_id, delivery_date,
                          delivery moves with product_id, product_qty, uom
        @return: Dictionary of values
        """
        
        #~ Forzar uid para que realice el calcul del costo promedio con todas las cantidades existentes
        #~ Es temporal, mientras se consigue una alternativa para filtrar almacenes por grupo 
        forced_uid = 1
        
        if context is None:
            context = {}
        else:
            context = dict(context)
        res = {}
        move_obj = self.pool.get('stock.move')
        product_obj = self.pool.get('product.product')
        currency_obj = self.pool.get('res.currency')
        uom_obj = self.pool.get('product.uom')
        sequence_obj = self.pool.get('ir.sequence')
        wf_service = netsvc.LocalService("workflow")
        for pick in self.browse(cr, forced_uid, ids, context=context):
            new_picking = None
            complete, too_many, too_few = [], [], []
            move_product_qty = {}
            prodlot_ids = {}
            product_avail = {}
            for move in pick.move_lines:
                if move.state in ('done', 'cancel'):
                    continue
                partial_data = partial_datas.get('move%s'%(move.id), {})
                
                #[FIX]-----------------------------
                if not partial_data.get('product_price'): 
                    price_unit = self.pool.get('product.product').browse(cr, forced_uid, partial_data.get('product_id')).standard_price
                    
                #[FIX]-----------------------------
                
                #Commented in order to process the less number of stock moves from partial picking wizard
                #assert partial_data, _('Missing partial picking data for move #%s') % (move.id)
                product_qty = partial_data.get('product_qty') or 0.0
                move_product_qty[move.id] = product_qty
                product_uom = partial_data.get('product_uom') or False

                #[FIX]-----------------------------
                #~ product_price = partial_data.get('product_price') or 0.0
                product_price = partial_data.get('product_price') or price_unit
                #[FIX]-----------------------------
                
                product_currency = partial_data.get('product_currency') or False
                prodlot_id = partial_data.get('prodlot_id') or False
                prodlot_ids[move.id] = prodlot_id
                if move.product_qty == product_qty:
                    complete.append(move)
                elif move.product_qty > product_qty:
                    too_few.append(move)
                else:
                    too_many.append(move)

                # Average price computation
                #FIX del [FIX]----------------------------------------
                # Se corrige para que actualice el precio estandar del producto al haber un albaran de entrada
                #[FIX]-----------------------------
                if (pick.type == 'in') and (move.product_id.cost_method == 'average'):
                #~ if (move.product_id.cost_method == 'average'):
                #[FIX]-----------------------------
                
                    product = product_obj.browse(cr, forced_uid, move.product_id.id)
                    move_currency_id = move.company_id.currency_id.id
                    context['currency_id'] = move_currency_id
                    qty = uom_obj._compute_qty(cr, forced_uid, product_uom, product_qty, product.uom_id.id)

                    if product.id in product_avail:
                        product_avail[product.id] += qty
                    else:
                        product_avail[product.id] = product.qty_available

                    if qty > 0:
                        new_price = currency_obj.compute(cr, forced_uid, product_currency,
                                move_currency_id, product_price)
                        new_price = uom_obj._compute_price(cr, forced_uid, product_uom, new_price,
                                product.uom_id.id)
                        if product.qty_available <= 0:
                            new_std_price = new_price
                        else:
                            # Get the standard price
                            amount_unit = product.price_get('standard_price', context)[product.id]
                            new_std_price = ((amount_unit * product_avail[product.id])\
                                + (new_price * qty))/(product_avail[product.id] + qty)
                        # Write the field according to price type field
                        product_obj.write(cr, uid, [product.id], {'standard_price': new_std_price})
                        # Record the values that were chosen in the wizard, so they can be
                        # used for inventory valuation if real-time valuation is enabled.
                        move_obj.write(cr, uid, [move.id],
                                {'price_unit': product_price,
                                 'price_currency_id': product_currency})
                
            
            for move in too_few:
                product_qty = move_product_qty[move.id]

                if not new_picking:
                    new_picking = self.copy(cr, uid, pick.id,
                            {
                                'name': sequence_obj.get(cr, uid, 'stock.picking.%s'%(pick.type)),
                                'move_lines' : [],
                                'state':'draft',
                            })
                if product_qty != 0:
                    defaults = {
                            'product_qty' : product_qty,
                            'product_uos_qty': product_qty, #TODO: put correct uos_qty
                            'picking_id' : new_picking,
                            'state': 'assigned',
                            'move_dest_id': False,
                    }
                    prodlot_id = prodlot_ids[move.id]
                    if prodlot_id:
                        defaults.update(prodlot_id=prodlot_id)
                    move_obj.copy(cr, uid, move.id, defaults)

                move_obj.write(cr, uid, [move.id],
                        {
                            'product_qty' : move.product_qty - product_qty,
                            'product_uos_qty':move.product_qty - product_qty, #TODO: put correct uos_qty
                        })

            if new_picking:
                move_obj.write(cr, uid, [c.id for c in complete], {'picking_id': new_picking})
            for move in complete:
                if prodlot_ids.get(move.id):
                    move_obj.write(cr, uid, [move.id], {'prodlot_id': prodlot_ids[move.id]})
            for move in too_many:
                product_qty = move_product_qty[move.id]
                defaults = {
                    'product_qty' : product_qty,
                    'product_uos_qty': product_qty, #TODO: put correct uos_qty
                }
                prodlot_id = prodlot_ids.get(move.id)
                if prodlot_ids.get(move.id):
                    defaults.update(prodlot_id=prodlot_id)
                if new_picking:
                    defaults.update(picking_id=new_picking)
                move_obj.write(cr, uid, [move.id], defaults)


            # At first we confirm the new picking (if necessary)
            if new_picking:
                wf_service.trg_validate(uid, 'stock.picking', new_picking, 'button_confirm', cr)
                # Then we finish the good picking
                self.write(cr, uid, [pick.id], {'backorder_id': new_picking})
                self.action_move(cr, uid, [new_picking])
                wf_service.trg_validate(uid, 'stock.picking', new_picking, 'button_done', cr)
                wf_service.trg_write(uid, 'stock.picking', pick.id, cr)
                delivered_pack_id = new_picking
            else:
                self.action_move(cr, uid, [pick.id])
                wf_service.trg_validate(uid, 'stock.picking', pick.id, 'button_done', cr)
                delivered_pack_id = pick.id
            
            ###########################registrar el precio en stock_move
            for move in pick.move_lines:
                #Si no hay un precio en el stock_move (no proviene de un albaran tipo in)
                #se registra el precio promedio que tiene el  producto
                price_move = self.pool.get('stock.move').read(cr,forced_uid,move.id,['price_unit'])['price_unit']
                partial_data = partial_datas.get('move%s'%(move.id), {})
                if partial_data and not partial_data.get('product_price', False):
                    product_uom = partial_data.get('product_uom') or False
                    product_price = self.pool.get('product.product').browse(cr, forced_uid, partial_data.get('product_id')).standard_price
                    product = product_obj.browse(cr, forced_uid, move.product_id.id)
                    move_currency_id = move.company_id.currency_id.id
                    product_currency = partial_data.get('product_currency') or False
                    new_price1 = currency_obj.compute(cr, forced_uid, product_currency,
                                move_currency_id, product_price)
                    new_price1 = uom_obj._compute_price(cr, forced_uid, product.uom_id.id, new_price1,
                                product_uom)
                    product_price = new_price1

                    # Record the values that were chosen in the wizard, so they can be
                    # used for inventory valuation if real-time valuation is enabled.
                    move_obj.write(cr, uid, [move.id],
                            {'price_unit': product_price,
                             'price_currency_id': product_currency})
                             
            ###############################################################
            
            delivered_pack = self.browse(cr, forced_uid, delivered_pack_id, context=context)
            res[pick.id] = {'delivered_picking': delivered_pack.id or False}

        return res
         
    _defaults = {
        'move_type' : 'one', #Colocar valor por defecto a move_type que se encuentra en la clase original, significa metodo de entrega todo junto en vez de entrega parcial
    }
                
stock_picking()


class stock_move(osv.osv):
    _name = 'stock.move'
    _inherit = 'stock.move'
    _description = "Stock Move"
    
    def copy(self, cr, uid, id, defaults, context=None): #Sobreescritura del metodo copy para no permitir duplicar objetos stock_move
        if not context is None:
            raise osv.except_osv(_('Acción Denegada!'),_('No se permite duplicar movimientos!'))
            return False
        else:
            return super(stock_move, self).copy(cr, uid, id, defaults, context)
            
    def _default_validate_create(self, cr, uid, context=None):
        if context is None:
            context = {}
        if (context.get('move_line') is None) | ((context.get('address_in_id') == False)):
            raise osv.except_osv(_('Acceso Denegado!'),_('No puede crear un movimiento!'))
        return super(stock_move, self)._default_location_source(cr, uid, context)
        
    def _default_location_destination_create(self, cr, uid, context=None):
        if context is None:
            context = {}
            
        result = super(stock_move, self)._default_location_destination(cr, uid, context)

        if uid <> 1 and result:
            #El metodo en ciertos casos retorna una tupla con el id de la ubicacion
            if isinstance(result,(tuple)):
                result = result[0]
            if result:
                location = self.pool.get('stock.location').search(cr, uid, [('id','=',result)], context=context)
                if not location:
                    return False
            else:
                return False
        return result
        
    _defaults={
        'location_id':_default_validate_create, #Esta funcion se usa para bloquear la creacion de movimientos sin cabecera, es decir desde movimientos de productos (enviar y recibir)
        'location_dest_id': _default_location_destination_create, #Esta funcion se sobreescribe para evitar colocar por defecto una ubicacion no permitida para el usuario
    }
    
    def read(self, cr, *args, **kwargs):
        dicc = {}
        records = super(stock_move, self).read(cr, *args, **kwargs)
        ok_records = []
        filtrado = False
        for r in records:
            if type(r) == type(dicc):
                if r.get('user_ids'):
                    filtrado=True
                    if r['user_ids'] == 'Si':        
                        ok_records.append(r)
        if filtrado & (int(args[0]) <> 1):
            return ok_records
        else:
            return records
            
    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct sm.id from stock_move sm 
                inner join (
                select distinct slp.id as parent_id, slh.id as child_id from stock_location slp
                left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_groups_location_rel dlr on dlr.location_id = slp.id
                where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)) 
                as tbld on sm.location_id = tbld.child_id or sm.location_dest_id = tbld.child_id
                inner join product_product p on sm.product_id =  p.id
                inner join product_template pt on p.product_tmpl_id = pt.id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld2 on pt.categ_id = tbld2.child_id""", (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
              
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if (uid <> 1):
            lista = self.lista_permitidos(cr, uid, context)     
            dic = 'id', 'in', lista
            args.append(dic)
        return super(stock_move, self).search(cr, uid, args, offset, limit, order, context, count)
        
    def onchange_location_id(self, cr, uid, ids, loc_id=False, origen=True):
        """ On change of location_id and location_dest_id.
        @param loc_id: Source location id
        @return: Dictionary of values
        """
        context = {}
        if bool(loc_id) & (uid <> 1):
            location = self.pool.get('stock.location')._get_location_allowed(cr, uid, loc_id, context)
            if not location.get(loc_id) is None:
                if location[loc_id] == 'No':
                    if origen:
                        return {'value':{'location_id': False},
                            'warning': {'title':'Atención','message':'Ubicación origen no permitido'}  }
                    else:
                        return {'value':{'location_dest_id': False},
                            'warning': {'title':'Atención','message':'Ubicación destino no permitido'}  }
            else:
                if origen:
                    return {'value':{'location_id': False},
                            'warning': {'title':'Atención','message':'Ubicación origen no permitido'}  }
                else:
                    return {'value':{'location_dest_id': False},
                        'warning': {'title':'Atención','message':'Ubicación destino no permitido'}  }
                
        if origen :
            result = {'location_id': loc_id}
        else:
            result = {'location_dest_id': loc_id}
        return {'value':result}
        
    def onchange_location_id_especial(self, cr, uid, ids, loc_id=False, origen=True):
        """ On change of location_id and location_dest_id.
        @param loc_id: Source location id
        @return: Dictionary of values
        """
        context = {}
        if bool(loc_id) & (uid <> 1):
            location = self.pool.get('stock.location')._get_location_allowed(cr, uid, loc_id, context)
            #consulto si es supplier
            force_uid = 1
            loc_supp = self.pool.get('stock.location').search(cr,force_uid, [('usage', '=', "supplier"),('id','=',loc_id)])
            if not location.get(loc_id) is None:
                if location[loc_id] == 'No':
                    if origen and not loc_supp:
                        return {'value':{'location_id': False},
                            'warning': {'title':'Atención','message':'Ubicación origen no permitido'}  }
                    elif not origen:
                        return {'value':{'location_dest_id': False},
                            'warning': {'title':'Atención','message':'Ubicación destino no permitido'}  }
            else:
                if origen and not loc_supp:
                    return {'value':{'location_id': False},
                            'warning': {'title':'Atención','message':'Ubicación origen no permitido'}  }
                elif not origen:
                    return {'value':{'location_dest_id': False},
                        'warning': {'title':'Atención','message':'Ubicación destino no permitido'}  }
                
        if origen :
            result = {'location_id': loc_id}
        else:
            result = {'location_dest_id': loc_id}
        return {'value':result}
        
    def _get_user_ids(self, cr, uid, ids, field_names, arg=None, context=None):
        res = {}
        stock_move = self.browse(cr, uid, ids)
        
        user_groups = []
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        
        for i,j in enumerate(user.groups_id): #Lista con todos los grupos a los que tiene acceso el usuario
            user_groups.append(user.groups_id[i].id)
        
        sua_ids = self.pool.get('diana.groups.location.rel').search(cr, uid, [('groups_id', 'in', user_groups)], context=context) #consulta de los registros diana.groups.location.rel que tiene un group_id al que tiene acceso el usuario
        sua_obj = self.pool.get('diana.groups.location.rel').browse(cr, uid, sua_ids)
        
        loc_list = []
        for s in sua_obj:
            child_ids = self.pool.get('stock.location').search(cr, uid, [('location_id', 'child_of', s.location_id.id)], context=context) # El grupo tiene permiso sobre el propio almacen y sobre todos sus hijos
            for item in child_ids:
                if not item in loc_list: 
                    loc_list.append(item)
        
        for item in stock_move:
            if (item.location_id.id in loc_list) | (item.location_dest_id.id in loc_list):
                res[item.id] = 'Si'
            else:
                res[item.id] = 'No'
        return res
        
    def _get_reference_accounting_values_for_valuation(self, cr, uid, move, context=None):
        """
        Return the reference amount and reference currency representing the inventory valuation for this move.
        These reference values should possibly be converted before being posted in Journals to adapt to the primary
        and secondary currencies of the relevant accounts.
        """
        product_uom_obj = self.pool.get('product.uom')
        currency_obj = self.pool.get('res.currency')

        # by default the reference currency is that of the move's company
        reference_currency_id = move.company_id.currency_id.id

        default_uom = move.product_id.uom_id.id
        qty = product_uom_obj._compute_qty(cr, uid, move.product_uom.id, move.product_qty, default_uom)
        
        # Buscar la moneda por defecto para el producto
        move_currency_id = move.price_currency_id and move.price_currency_id.id or \
            move.company_id.currency_id.id
        product_currency = move.company_id.currency_id.id
        product_uom = move.product_uom.id
        

        # if product is set to average price and a specific value was entered in the picking wizard,
        # we use it
        if move.product_id.cost_method == 'average' and move.price_unit:
            new_price = currency_obj.compute(cr, uid, product_currency,
                move_currency_id, move.price_unit)
            new_price = product_uom_obj._compute_price(cr, uid, product_uom, new_price,
                    default_uom)
            reference_amount = qty * new_price
            reference_currency_id = move.price_currency_id.id or reference_currency_id


        # Otherwise we default to the company's valuation price type, considering that the values of the
        # valuation field are expressed in the default currency of the move's company.
        else:
            if context is None:
                context = {}
            currency_ctx = dict(context, currency_id = move.company_id.currency_id.id)
            amount_unit = move.product_id.price_get('standard_price', currency_ctx)[move.product_id.id]
            reference_amount = amount_unit * qty or 1.0

        return reference_amount, reference_currency_id
        
    def onchange_product_id(self, cr, uid, ids, prod_id=False, loc_id=False,
                            loc_dest_id=False, address_id=False, name_onchange=None):
        result = super(stock_move, self).onchange_product_id(cr,uid,ids,prod_id,loc_id,loc_dest_id,address_id)
        
        #Para corregir el nombre del stock_move y asi generar correcto el nombre en el asiento
        if ids and prod_id:
            #Validar que no se permita cambiar el producto si el albaran proviene de algun documento de compra, venta o devolucion
            if isinstance(ids,(int,long)):
                ids = [ids]
            
            #Buscar la informacion para revertir el cambio
            move_brw = self.pool.get('stock.move').browse(cr, uid, ids)[0]
            
            
            #Evitar cambiar el producto si proviene de una orden de compra o de venta o de una devolucion o esta asociada a una hoja de carga
            purchase_brw = hasattr(move_brw.picking_id,'purchase_id') and move_brw.picking_id.purchase_id or False
            sale_brw = hasattr(move_brw.picking_id,'sale_id') and move_brw.picking_id.sale_id or False
            devoluciones_brw = hasattr(move_brw.picking_id,'devoluciones_id') and move_brw.picking_id.devoluciones_id or False
            hoja_carga_brw = hasattr(move_brw.picking_id,'hoja_carga_id') and move_brw.picking_id.hoja_carga_id or False
            if purchase_brw or sale_brw or devoluciones_brw or hoja_carga_brw:
                result['value']['product_id'] = move_brw.product_id.id
                if purchase_brw:
                    result['warning'] = {'title':'Atención','message':'No puede cambiar el producto. Proviene de una orden de compra.'}
                elif sale_brw:
                    result['warning'] = {'title':'Atención','message':'No puede cambiar el producto. Proviene de un pedido de venta.'}
                elif devoluciones_brw:
                    result['warning'] = {'title':'Atención','message':'No puede cambiar el producto. Proviene de una devolucion en venta.'}
                else:
                    result['warning'] = {'title':'Atención','message':'No puede cambiar el producto. Esta asociada a una hoja de carga.'}
                return result
                
            #validar si esta en estado reservado, el usuario debe cancelar la disponibilidad
            if move_brw.state == 'assigned':
                result['value']['product_id'] = move_brw.product_id.id
                result['warning'] = {'title':'Atención','message':'Debe cancelar la disponibilidad del producto antes de cambiarlo.'}
                return result
            
            #Modificar el name al cambiar el producto
            prod_brw = self.pool.get('product.product').browse(cr, uid, prod_id)
            result['value']['name'] = prod_brw.partner_ref
        
        return result
        
    _columns = {
            'user_ids' : fields.function(_get_user_ids, method=True, type='string', string='Users Allowed',),
            'parent_source':fields.related('location_id','location_id',type='many2one',relation='stock.location',string='Source Parent',readonly=True, store=False),
            'parent_dest':fields.related('location_dest_id','location_id',type='many2one',relation='stock.location',string='Destination Parent',readonly=True, store=False),
         }
                
stock_move()

class stock_inventory_line(osv.osv):
    _name = 'stock.inventory.line'
    _inherit = 'stock.inventory.line'
    _description = "Stock Inventory Line"

    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct sm.id from stock_inventory_line sm 
            inner join (
            select distinct slp.id as parent_id, slh.id as child_id from stock_location slp
            left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
            inner join diana_groups_location_rel dlr on dlr.location_id =slp.id
            where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)) 
            as tbld on sm.location_id = tbld.child_id
            inner join product_product p on sm.product_id =  p.id
            inner join product_template pt on p.product_tmpl_id = pt.id
            inner join (                
            select distinct slp.id as parent_id, slh.id as child_id from product_category slp
            left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
            inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
            where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
            as tbld2 on pt.categ_id = tbld2.child_id""", (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
                        
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if uid <> 1:
            lista = self.lista_permitidos(cr, uid, context)                       
            dic = 'id', 'in', lista
            args.append(dic)
        return super(stock_inventory_line, self).search(cr, uid, args, offset, limit, order, context, count)
        
    def onchange_location_id(self, cr, uid, ids, loc_id=False):
        """ On change of location_id and location_dest_id.
        @param loc_id: Source location id
        @return: Dictionary of values
        """
        context = {}
        if bool(loc_id) & (uid <> 1): 
            location = self.pool.get('stock.location')._get_location_allowed(cr, uid, loc_id, context)
            if location.get(loc_id) is None:
                return {'value':{'location_id': False},
                        'warning': {'title':'Atención','message':'Ubicación no permitida'}  }
            elif location[loc_id] == 'No':
                return {'value':{'location_id': False},
                        'warning': {'title':'Atención','message':'Ubicación no permitida'}  }
        return {'value':{'location_id': loc_id}}
                
stock_inventory_line()


class stock_inventory(osv.osv):
    _name = 'stock.inventory'
    _inherit = 'stock.inventory'
    _description = "Stock Inventory"

    def lista_productos_permitidos(self, cr, uid, ids, context=None):
        cr.execute("""select distinct p.id from stock_inventory si 
                    inner join stock_inventory_line spl on si.id = spl.inventory_id
                    inner join product_product p on spl.product_id =  p.id
                    inner join product_template pt on p.product_tmpl_id = pt.id
                    inner join (
                    select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                    left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                    inner join diana_category_location_rel dcr on dcr.category_id =slp.id
                    inner join diana_groups_location_rel dlr on dlr.location_id = dcr.location_id
                    where dlr.groups_id in (select gid from res_groups_users_rel where uid = %s)) as 
                    tbld on pt.categ_id = tbld.child_id
                    inner join (                
                    select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                    left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                    inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                    where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                    as tbld2 on pt.categ_id = tbld2.child_id """, (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
                
    def validar_lineas(self, cr, uid, ids, context=None):
        if uid == 1:
            return (True, '')
        inv = self.browse(cr, uid, ids)
        lines = self.pool.get('stock.inventory.line').search(cr, 1, [('inventory_id', '=', inv[0].id)], context=context)
        lines_ = self.pool.get('stock.inventory.line').search(cr, uid, [('inventory_id', '=', inv[0].id)], context=context)
        if len(lines) <> len(lines_):
            return (False, 'No puede procesar el inventario. Existen líneas de inventario no permitidas al usuario actual!')
        else:
            productos_permitidos = self.lista_productos_permitidos(cr, uid, ids, context)
            for item in lines_:
                line_obj = self.pool.get('stock.inventory.line').browse(cr, uid, item)
                if not line_obj.product_id.id in productos_permitidos:
                    return (False, 'No puede procesar el inventario. Existen líneas de inventario con productos no permitidos al usuario actual!')
            return (True, '') 
    
    def action_confirm(self, cr, uid, ids, context=None):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_inventory, self).action_confirm(cr, uid, ids, context)
            
    def action_done(self, cr, uid, ids, context=None):
        valor, mensaje = self.validar_lineas(cr, uid, ids, context)
        if not valor:
            raise osv.except_osv(_('Acción Denegada!'),_(mensaje))
        else:
            return super(stock_inventory, self).action_done(cr, uid, ids, context)
    
stock_inventory()


class product_product(osv.osv):
    _name = 'product.product'
    _inherit = 'product.product'
    _description = "Product Product"
    
    def copy(self, cr, uid, ids, defaults, context=None):
        raise osv.except_osv(('Error !'), ('No se puede duplicar productos'))
                
    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct p.id from product_product p inner join product_template pt on p.product_tmpl_id = pt.id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_location_rel dcr on dcr.category_id =slp.id
                inner join diana_groups_location_rel dlr on dlr.location_id = dcr.location_id
                where dlr.groups_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld on pt.categ_id = tbld.child_id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld2 on pt.categ_id = tbld2.child_id """, (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids                
                
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if uid <> 1:
            lista = self.lista_permitidos(cr, uid, context)                   
            dic = 'id', 'in', lista
            args.append(dic)
        return super(product_product, self).search(cr, uid, args, offset, limit, order, context, count)
    
product_product()

class procurement_order(osv.osv):
    _inherit = 'procurement.order'
    
    _columns = {
        'diana_stock_procurement_id' : fields.many2one('diana.stock.procurement','Abastecimiento Masivo'), #Apuntador al objeto diana.stock.procurement
        'create_uid' : fields.many2one('res.users','Usuario'),
    }
    
    def make_po(self, cr, uid, ids, context=None):
        res = super(procurement_order, self).make_po(cr, uid, ids, context=context)
        po_obj = self.pool.get('purchase.order')
        pol_obj = self.pool.get('purchase.order.line')
        for proc_id, po_id in res.items():
            procurement = self.browse(cr, uid, proc_id, context=context)
            if all ([procurement.requisition_id, procurement.purchase_id]):
                req = self.pool.get('purchase.requisition').browse(cr, uid, procurement.requisition_id.id) # Solicitud de compra (purchase.requisition)
                pur = self.pool.get('purchase.order').browse(cr, uid, procurement.purchase_id.id) # Pedido de compra (purchase.order)
                """ 
                Aca se actualiza el origen
                """
                values = {
                    'origin': req.name, 
                    'notes': procurement.note,
                    'state': 'cancel',
                    }
                if procurement.product_id.product_tmpl_id.type == 'service':
                    values['invoice_method']= 'order'
                    
                self.pool.get('purchase.order').write(cr, uid, po_id, values)
                """
                Agregar concepto de retencion de ISRL del producto a la solicitud 
                de presupuesto que se genera 
                """
                for i in pur.order_line:
                    pol_name = pol_obj.browse(cr, uid, i.id, context=context).name

                    """
                    Se debe realizar para optimizar el codigo, siempre es relacion de uno a uno, no hace falta verificar por nota
                    """
                    prl_id = self.pool.get('purchase.requisition.line').search(cr, uid, [('requisition_id','=',pur.requisition_id.id), ('product_id','=',i.product_id.id)])
                    prl_brw = self.pool.get('purchase.requisition.line').browse(cr, uid, prl_id)
                        
                    values = {
                        'concept_id': i.product_id and 
                            i.product_id.concept_id and 
                            i.product_id.concept_id.id or False,
                        'product_uom': prl_brw[0].product_uom_id.id,
                    }
                    
                    if i.product_id.product_tmpl_id.type == 'service':
                        values['name']= procurement.note or pol_name
                    
                    self.pool.get('purchase.order.line').write(cr, uid, i.id, values)
                    
                """
                Se consulta el warehouse de la ubicacion desde donde se realizo la solicitud de abastecimiento (procurement_order)
                para asignarle este warehouse a la solicitud de compra (purchase.requisition). Luego se actualiza la 
                solicitud de compra (purchase.requisition)
                """
                wh_ = self.pool.get('stock.warehouse').search(cr, uid, [('lot_input_id', '=', procurement.location_id.id)]) 
                
                self.pool.get('purchase.requisition').write(cr, uid, req.id, { 
                    'warehouse_id': wh_[0],
                    'origin': procurement.origin,
                    'description':procurement.note,
                })
                
                po_line_id = self.pool.get('purchase.requisition.line').search(cr, uid, [('requisition_id', '=', req.id)])
                self.pool.get('purchase.requisition.line').write(cr, uid, po_line_id, { 
                    'description':procurement.note,
                })
        return res
        
    def copy(self, cr, uid, id, defaults, context=None): #Sobreescritura del metodo copy para no permitir duplicar objetos stock_move
        raise osv.except_osv(_('Acción Denegada!'),_('No se permite duplicar solicitudes de abastecimiento!'))
        return False
    
    def _default_validate_create(self, cr, uid, context=None):
        if context is None:
            context = {}
        if not context.get('create', False):
            raise osv.except_osv(_('Acceso Denegado!'),_('No se permite crear solicitudes de abastecimiento desde esta opción!'))
        return 0
        
    _defaults={
        'product_qty':_default_validate_create, #Esta funcion se usa para bloquear la creacion de solicitudes de abastecimiento desde Lineas de Abastecimiento o Excepciones de abastecimiento
    }

    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct sm.id from procurement_order sm 
                inner join (
                select distinct slp.id as parent_id, slh.id as child_id from stock_location slp
                left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_groups_location_rel dlr on dlr.location_id =slp.id
                where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)) 
                as tbld on sm.location_id = tbld.child_id
                inner join product_product p on sm.product_id =  p.id
                inner join product_template pt on p.product_tmpl_id = pt.id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld2 on pt.categ_id = tbld2.child_id""", (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
              
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if uid <> 1:
            lista = self.lista_permitidos(cr, uid, context)     
            dic = 'id', 'in', lista
            args.append(dic)
        return super(procurement_order, self).search(cr, uid, args, offset, limit, order, context, count)
                
    def on_change_product_id(self, cr, uid, ids, product_id, uom=False):
        """ On change of product id.
        @param product_id: Changed Product id
        @param uom: Changed Product uom
        @return: Dictionary of values
        """
        context = {}   
        if product_id :
            product_uom = self.pool.get('product.product').browse(cr, uid, product_id)
            result = {'product_id': product_id, 'product_uom': product_uom.uom_id.id}
        else:
            result = {'product_id': product_id, 'product_uom': uom}
        return {'value':result}
        
    def onchange_location_id(self, cr, uid, ids, loc_id=False):
        """ On change of location_id and location_dest_id.
        @param loc_id: Source location id
        @return: Dictionary of values
        """
        context = {}
        if bool(loc_id) & (uid <> 1): 
            location = self.pool.get('stock.location')._get_location_allowed(cr, uid, loc_id, context)
            if location.get(loc_id) is None:
                return {'value':{'location_id': False},
                        'warning': {'title':'Atención','message':'Ubicación no permitida'}  }
            
            elif location[loc_id] == 'No':
                return {'value':{'location_id': False},
                        'warning': {'title':'Atención','message':'Ubicación no permitida'}  }
            
        wh_ = self.pool.get('stock.warehouse').search(cr, uid, [('lot_input_id', '=', loc_id)])
        if wh_ == []:
            return {'value':{'location_id': False},
                    'warning': {'title':'Atención','message':'La ubicación no tiene un almacén asociado'}  }
                        
        return {'value':{'location_id': loc_id}}
        
    """
    Se sobreescribe el metodo create de procurement.order para que se haga la validación de la cantidad (product_qty) de producto que se esta solicitando.
    Esta cantidad no debe ser ni igual ni menor que cero
    """ 
    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        
        record_id = None
        
        if vals.get('diana_stock_procurement_id'):
            dsp_brw = self.pool.get('diana.stock.procurement').browse(cr, uid, vals.get('diana_stock_procurement_id'))
            vals['origin'] = dsp_brw.name
            
        if float(vals['product_qty']) <= 0 :
            raise osv.except_osv(_('Data Insufficient !'),
                    _('Please check the Quantity in Procurement Order(s), it should not be less than 0!'))
        if vals['date_planned'] < time.strftime('%Y-%m-%d'):
            raise osv.except_osv(_('ERROR!'),_('Error date_planned!'))
            
        else :
            record_id = super(procurement_order, self).create(cr, uid, vals, context)

        return record_id

    def write(self, cr, user, ids, vals, context=None):
        if type(ids) <> list:
            ids = [ids]         
        record = super(procurement_order, self).write(cr, user, ids, vals, context)
        if record and vals.get('note'):
            procurement = self.browse(cr, user, ids, context=context)
            if all ([procurement[0].note, procurement[0].requisition_id, procurement[0].purchase_id]):
                prl_ids = self.pool.get('purchase.requisition.line').search(cr, user, [('requisition_id', '=', procurement[0].requisition_id.id)])
                pol_ids = self.pool.get('purchase.order.line').search(cr, user, [('order_id', '=', procurement[0].purchase_id.id)])
                values = {
                    'description': procurement[0].note,
                    }
                self.pool.get('purchase.requisition.line').write(cr, user, prl_ids, values)
                
                values = {
                    'notes': procurement[0].note,
                    }
                self.pool.get('purchase.order.line').write(cr, user, pol_ids, values)
            
        return record
        
procurement_order()

class stock_production_lot(osv.osv):
    _name = 'stock.production.lot'
    _inherit = 'stock.production.lot'
    _description = "Stock Production Lot"

    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct spl.id from stock_production_lot spl
                inner join product_product p on spl.product_id =  p.id
                inner join product_template pt on p.product_tmpl_id = pt.id
                inner join (
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_location_rel dcr on dcr.category_id =slp.id
                inner join diana_groups_location_rel dlr on dlr.location_id = dcr.location_id
                where dlr.groups_id in (select gid from res_groups_users_rel where uid = %s)) 
                as tbld on pt.categ_id = tbld.child_id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld2 on pt.categ_id = tbld2.child_id""", (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
                
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if uid <> 1:
            lista = self.lista_permitidos(cr, uid, context)                      
            dic = 'id', 'in', lista
            args.append(dic)
        return super(stock_production_lot, self).search(cr, uid, args, offset, limit, order, context, count)
    
stock_production_lot()

class stock_warehouse(osv.osv):
    _inherit = 'stock.warehouse'

    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""select distinct sm.id from stock_warehouse sm 
                inner join (
                select distinct slp.id as parent_id, slh.id as child_id from stock_location slp
                left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_groups_location_rel dlr on dlr.location_id =slp.id
                where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)) 
                as tbld on sm.lot_input_id = tbld.child_id or  sm.lot_stock_id = tbld.child_id 
                or sm.lot_output_id = tbld.child_id""", (uid,))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
                      
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if uid <> 1:
            lista = self.lista_permitidos(cr, uid, context)       
            dic = 'id', 'in', lista
            args.append(dic)
        return super(stock_warehouse, self).search(cr, uid, args, offset, limit, order, context, count)
    
    def onchange_location_id(self, cr, uid, ids, loc_id=False, tipo=1):
        """ On change of lot_input_id, lot_stock_id and lot_output_id.
        @param loc_id: Source lot_input_id, lot_stock_id and lot_output_id
        @param tipo: Source loc_id
        @return: Dictionary of values
        """
        context = {}
        if bool(loc_id) & (uid <> 1): 
            location = self.pool.get('stock.location')._get_location_allowed(cr, uid, loc_id, context)
            if bool(tipo == 1) and ((location.get(loc_id) is None) or (bool(location[loc_id] == 'No'))):
                return {'value':{'lot_input_id': False},
                        'warning': {'title':'Atención','message':'Ubicación no permitida'}  }
            elif bool(tipo == 2) and ((location.get(loc_id) is None) or (bool(location[loc_id] == 'No'))):
                return {'value':{'lot_stock_id': False},
                        'warning': {'title':'Atención','message':'Ubicación no permitida'}  }
            elif bool(tipo == 3) and ((location.get(loc_id) is None) or (bool(location[loc_id] == 'No'))):
                return {'value':{'lot_output_id': False},
                        'warning': {'title':'Atención','message':'Ubicación no permitida'}  }
        if tipo == 1:
            return {'value':{'lot_input_id': loc_id}}
        elif tipo == 2:
            return {'value':{'lot_stock_id': loc_id}}
        elif tipo == 3:
            return {'value':{'lot_output_id': loc_id}}
                    
stock_warehouse()

class make_procurement(osv.osv_memory):
    _inherit = 'make.procurement'
            
    def onchange_warehouse_id(self, cr, uid, ids, whs_id=False):
        """ On change of warehouse_id
        @param whs_id: Source warehouse_id
        """
        context = {}
        if bool(whs_id) & (uid <> 1): 
            warehouse_permitidos = self.pool.get('stock.warehouse').lista_permitidos(cr, uid, context)
            if whs_id not in warehouse_permitidos:
                return {'value':{'warehouse_id': False},
                        'warning': {'title':'Atención','message':'Almacén no permitido al Usuario'}  }
        
        return {'value':{'warehouse_id': whs_id}}
                    
make_procurement()

class product_category(osv.osv):
    
    _inherit = 'product.category'
    
    _columns = {
        'groups_ids': fields.many2many('res.groups', 'diana_category_groups_rel', 'category_id','group_id','Groups-Product Category Rel'),
    }
    
    def lista_permitidos(self, cr, uid, context=None):
        cr.execute("""
                select distinct pc.id, pc.name from product_category pc
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_location_rel dcr on dcr.category_id =slp.id
                inner join diana_groups_location_rel dlr on dlr.location_id = dcr.location_id
                where dlr.groups_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld on pc.id = tbld.child_id
                inner join (                
                select distinct slp.id as parent_id, slh.id as child_id from product_category slp
                left outer join product_category slh on  slh.parent_left between slp.parent_left and slp.parent_right
                inner join diana_category_groups_rel dcg on dcg.category_id = slp.id 
                where dcg.group_id in (select gid from res_groups_users_rel where uid = %s))
                as tbld2 on pc.id = tbld2.child_id""", (uid,uid))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
                      
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}
        if uid <> 1:
            lista = self.lista_permitidos(cr, uid, context)      
            dic = 'id', 'in', lista
            args.append(dic)
        return super(product_category, self).search(cr, uid, args, offset, limit, order, context, count)
    
product_category()

class res_groups(osv.osv):
    
    _inherit = 'res.groups'
    
    _columns = {
        'category_ids': fields.many2many('product.category', 'diana_category_groups_rel', 'group_id', 'category_id', 'Groups-Product Category Rel'),
    }
    
res_groups()

class stock_partial_picking(osv.osv_memory):
    
    _inherit = 'stock.partial.picking'
    
    def do_partial(self, cr, uid, ids, context=None):
        """ Funcion para validar que el coste sea mayor a cero
        """
        pick_obj = self.pool.get('stock.picking')
        uom_obj = self.pool.get('product.uom')

        picking_ids = context.get('active_ids', False)
        partial = self.browse(cr, uid, ids[0], context=context)
        partial_datas = {
            'delivery_date' : partial.date
        }

        for pick in pick_obj.browse(cr, uid, picking_ids, context=context):
            picking_type = self.get_picking_type(cr, uid, pick, context=context)
            moves_list = picking_type == 'in' and partial.product_moves_in or partial.product_moves_out

            for n in moves_list:
                if picking_type == 'in':
                    if n.cost <= 0:
                        raise osv.except_osv(_('mensaje_usuario'),_('Verifique que ningún coste sea menor o igual a cero'))
                if n.quantity <= 0:
                    raise osv.except_osv(_('mensaje_usuario'),_('Verifique que ninguna cantidad sea menor o igual a cero'))
    
        return super(stock_partial_picking, self).do_partial(cr, uid, ids, context)
        
    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False, submenu=False):
        result = super(stock_partial_picking, self).fields_view_get(cr, uid, view_id, view_type, context, toolbar, submenu)
        if result.get('arch', False):
            result['arch'] = result['arch'].replace('_Cancel','_Cancelar')
            result['arch'] = result['arch'].replace('_Validate','_Validar')
            
        return result
        
    def default_get(self, cr, uid, fields, context=None):
        """ To get default values for the object.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param fields: List of fields for which we want default values
        @param context: A standard dictionary
        @return: A dictionary which of fields with values.
        """
        if context is None:
            context = {}
        pick_obj = self.pool.get('stock.picking')
        uom_obj = self.pool.get('product.uom')
        res = super(stock_partial_picking, self).default_get(cr, uid, fields, context=context)
        for pick in pick_obj.browse(cr, uid, context.get('active_ids', []), context=context):
            if pick.type == 'in' and not pick.purchase_id:
                list_index = 0
                for item in res['product_moves_in']:
                    uom_prod_id = self.pool.get('product.product').read(cr,uid,res['product_moves_in'][list_index]['product_id'],['uom_id'])['uom_id'][0]
                    new_price = uom_obj._compute_price(cr, uid, uom_prod_id, res['product_moves_in'][list_index]['cost'],
                            res['product_moves_in'][list_index]['product_uom'])
                    res['product_moves_in'][list_index]['cost'] = new_price
                    list_index += 1
                    
        return res
        
stock_partial_picking()

#variable global para listar los productos que se pueden devolver
global_lista_devolver = []
class stock_return_picking(osv.osv_memory):
    _inherit = 'stock.return.picking'
        
    def fields_view_get(self, cr, uid, view_id=None, view_type='form',
                        context=None, toolbar=False, submenu=False):
        """
         Changes the view dynamically
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view.
        """
        res = super(stock_return_picking, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=False)
        record_id = context and context.get('active_id', False)
        active_model = context.get('active_model')
        if  active_model != 'stock.picking':
            return res
        if record_id:
            pick_obj = self.pool.get('stock.picking')
            pick = pick_obj.browse(cr, uid, record_id)
            return_history = {}
            res['fields'].clear()
            arch_lst=['<?xml version="1.0"?>', '<form string="%s">' % _('Return lines'), '<label string="%s" colspan="4"/>' % _('Provide the quantities of the returned products.')]
            for m in pick.move_lines:
                return_history[m.id] = 0
                for rec in m.move_history_ids2:
                    if rec.picking_id.type != 'internal' and rec.picking_id.state!='cancel':
                        return_history[m.id] += rec.product_qty
                    
                quantity = m.product_qty
                if m.state=='done' and quantity > return_history[m.id]:
                    arch_lst.append('<field name="return%s"/>\n<newline/>' % (m.id,))
                    res['fields']['return%s' % m.id]={'string':m.name, 'type':'float', 'required':True}
                    res.setdefault('returns', []).append(m.id)
            arch_lst.append('<field name="invoice_state"/>\n<newline/>')
            res['fields']['invoice_state']={'string':_('Invoicing'), 'type':'selection','required':True, 'selection':[('2binvoiced', _('To be refunded/invoiced')), ('none', _('No invoicing'))]}
            arch_lst.append('<group col="2" colspan="4">')
            arch_lst.append('<button icon="gtk-cancel" special="cancel" string="Cancelar" />')
            arch_lst.append('<button name="create_returns" string="Devolver" colspan="1" type="object" icon="gtk-apply" />')
            arch_lst.append('</group>')
            arch_lst.append('</form>')
            res['arch'] = '\n'.join(arch_lst)
            
        #codigo agregado para depurar el diccionario data, de tal forma  que no se intente procesar los productos ya devueltos
        if res.get('returns'):
            self.global_lista_devolver = res.get('returns')
        else:
            self.global_lista_devolver= []
        ###########################################################
        return res
    
    def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        if context is None:
            context = {}
        #~ res = super(stock_return_picking, self).view_init(cr, uid, fields_list, context=context)

        record_id = context and context.get('active_id', False)
        if record_id:
            pick_obj = self.pool.get('stock.picking')
            pick = pick_obj.browse(cr, uid, record_id, context=context)
            if pick.state not in ['done','confirmed','assigned']:
                raise osv.except_osv(_('Warning !'), _("You may only return pickings that are Confirmed, Available or Done!"))
            return_history = {}
            valid_lines = 0
            for m in [line for line in pick.move_lines]:
                if m.state == 'done':
                    return_history[m.id] = 0
                    for rec in m.move_history_ids2:
                        if rec.picking_id.type != 'internal' and rec.picking_id.state!='cancel':
                            return_history[m.id] += (rec.product_qty * rec.product_uom.factor)                        

                    if m.product_qty * m.product_uom.factor > return_history[m.id]:
                        valid_lines += 1
                        if 'return%s'%(m.id) not in self._columns:
                            self._columns['return%s'%(m.id)] = fields.float(string=m.name, required=True)
                        if 'invoice_state' not in self._columns:
                            self._columns['invoice_state'] = fields.selection([('2binvoiced', 'To be refunded/invoiced'), ('none', 'No invoicing')], string='Invoicing', required=True)

            if not valid_lines:
                raise osv.except_osv(_('Warning !'), _("There are no products to return (only lines in Done state and not fully returned yet can be returned)!"))
        
        return None # res
    
    def create_returns(self, cr, uid, ids, context=None):
        """ 
         Creates return picking.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: List of ids selected
         @param context: A standard dictionary
         @return: A dictionary which of fields with values.
        """
        if context is None:
            context = {} 
        record_id = context and context.get('active_id', False) or False
        move_obj = self.pool.get('stock.move')
        pick_obj = self.pool.get('stock.picking')
        uom_obj = self.pool.get('product.uom')
        wf_service = netsvc.LocalService("workflow")
    
        pick = pick_obj.browse(cr, uid, record_id, context=context)
        data = self.read(cr, uid, ids[0])
        new_picking = None
        date_cur = time.strftime('%Y-%m-%d %H:%M:%S')
        
        set_invoice_state_to_none = True
        returned_lines = 0
        
        #codigo agregado para depurar el diccionario data, de tal forma  que no se intente procesar los productos ya devueltos
        aux = data.keys()

        for variable in aux:
            if variable[:6] == 'return':
                valor = int(variable[6:])
                if valor not in self.global_lista_devolver:
                    data.pop(variable)
        ############################################
        
        
        for move in pick.move_lines:
            if not new_picking:
                if pick.type=='out':
                    new_type = 'in'
                elif pick.type=='in':
                    new_type = 'out'
                else:
                    new_type = 'internal'
                new_picking = pick_obj.copy(cr, uid, pick.id, {'name':'%s-return' % pick.name,
                        'move_lines':[], 'state':'draft', 'type':new_type,
                        'date':date_cur, 'invoice_state':data['invoice_state'],})
            new_location=move.location_dest_id.id
            if move.state=='done' and data.get('return%s' % move.id):
                new_qty = data['return%s' % move.id]
                returned_qty = move.product_qty
                
                for rec in move.move_history_ids2:
                    if rec.picking_id.type != 'internal' and rec.picking_id.state!='cancel':
                        returned_qty -= rec.product_qty
                    
                if returned_qty != new_qty:
                    set_invoice_state_to_none = False

                if new_qty:
                    # codigo agregado para validar que no se devuelva más de lo que puede devolver en el albaran
                    if new_qty > returned_qty:
                        raise osv.except_osv(_('Warning !'), _("No se puede procesar la devolucion por mas de lo que resta en el albaran! Cantidad restante : %s del producto %s"%(returned_qty, move.product_id.name)))
                    ########################################################################
                    
                    returned_lines += 1
                    new_move=move_obj.copy(cr, uid, move.id, {
                        'product_qty': new_qty,
                        'product_uos_qty': uom_obj._compute_qty(cr, uid, move.product_uom.id,
                            new_qty, move.product_uos.id),
                        'picking_id':new_picking, 'state':'draft',
                        'location_id':new_location, 'location_dest_id':move.location_id.id,
                        'date':date_cur, 'price_unit': 0.0})
                    move_obj.write(cr, uid, [move.id], {'move_history_ids2':[(4,new_move)]})

        if not returned_lines:
            raise osv.except_osv(_('Warning !'), _("Please specify at least one non-zero quantity!"))

        if set_invoice_state_to_none:
            pick_obj.write(cr, uid, [pick.id], {'invoice_state':'none'})
        wf_service.trg_validate(uid, 'stock.picking', new_picking, 'button_confirm', cr)
        pick_obj.force_assign(cr, uid, [new_picking], context)
        # Update view id in context, lp:702939
        view_list = {
                'out': 'view_picking_out_tree',
                'in': 'view_picking_in_tree',
                'internal': 'vpicktree',
            }
        data_obj = self.pool.get('ir.model.data')
        res = data_obj.get_object_reference(cr, uid, 'stock', view_list.get(new_type, 'vpicktree'))
        context.update({'view_id': res and res[1] or False})
        return {
            'domain': "[('id', 'in', ["+str(new_picking)+"])]",
            'name': 'Picking List',
            'view_type':'form',
            'view_mode':'tree,form',
            'res_model':'stock.picking',
            'type':'ir.actions.act_window',
            'context':context,
        }

stock_return_picking()
