# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _

class stock_location(osv.osv):
    _inherit = "stock.location"
    
    _columns = {
        'account_location_variant_id': fields.many2one('account.account', 'Cuenta Variante', help='Cuenta alternativa que se usara si no existe cuentas en la ubicacion, modificara el asiento contable'),
    }
    
stock_location()

class stock_move(osv.osv):
    _inherit = "stock.move"
    
    def _get_accounting_data_for_valuation(self, cr, uid, move, context=None):
        """
        Return the accounts and journal to use to post Journal Entries for the real-time
        valuation of the move.

        :param context: context dictionary that can explicitly mention the company to consider via the 'force_company' key
        :raise: osv.except_osv() is any mandatory account or journal is not defined.
        """
        product_obj=self.pool.get('product.product')
        accounts = product_obj.get_product_accounts(cr, uid, move.product_id.id, context)
        if move.location_id.valuation_out_account_id:
            acc_src = move.location_id.valuation_out_account_id.id
        elif move.location_id.account_location_variant_id and move.picking_id and move.picking_id.type == 'internal':
            acc_src = move.location_id.account_location_variant_id.id
        else:
            acc_src = accounts['stock_account_input']

        if move.location_dest_id.valuation_in_account_id:
            acc_dest = move.location_dest_id.valuation_in_account_id.id
        elif move.location_dest_id.account_location_variant_id and move.picking_id and move.picking_id.type == 'internal':
            acc_dest = move.location_dest_id.account_location_variant_id.id
        else:
            acc_dest = accounts['stock_account_output']
        
        """Este condicional se encarga de validar que solo se haga el intercambio de cuenta
        origen y cuenta destino, solo cuando sea un movimiento interno, se este usando la
        cuenta variante de la ubicacion origen/destino y ninguna de las ubicaciones tenga
        un cuenta de entrada o de salida"""
        if not move.location_id.valuation_out_account_id and not move.location_dest_id.valuation_in_account_id \
            and (move.location_id.account_location_variant_id or move.location_dest_id.account_location_variant_id):
            if move.picking_id and move.picking_id.type == 'internal':
                aux = acc_src
                acc_src = acc_dest
                acc_dest = aux
                
        acc_variation = accounts.get('property_stock_variation', False)
        journal_id = accounts['stock_journal']

        if acc_dest == acc_variation:
            raise osv.except_osv(_('Error!'),  _('Can not create Journal Entry, Output Account defined on this product and Variant account on category of this product are same.'))

        if acc_src == acc_variation:
            raise osv.except_osv(_('Error!'),  _('Can not create Journal Entry, Input Account defined on this product and Variant account on category of this product are same.'))

        if not acc_src:
            raise osv.except_osv(_('Error!'),  _('There is no stock input account defined for this product or its category: "%s" (id: %d)') % \
                                    (move.product_id.name, move.product_id.id,))
        if not acc_dest:
            raise osv.except_osv(_('Error!'),  _('There is no stock output account defined for this product or its category: "%s" (id: %d)') % \
                                    (move.product_id.name, move.product_id.id,))
        if not journal_id:
            raise osv.except_osv(_('Error!'), _('There is no journal defined on the product category: "%s" (id: %d)') % \
                                    (move.product_id.categ_id.name, move.product_id.categ_id.id,))
        if not acc_variation:
            raise osv.except_osv(_('Error!'), _('There is no inventory variation account defined on the product category: "%s" (id: %d)') % \
                                    (move.product_id.categ_id.name, move.product_id.categ_id.id,))
        return journal_id, acc_src, acc_dest, acc_variation
    
stock_move()
