# -*- coding: utf-8 -*-
from osv import osv, fields
import decimal_precision as dp
import netsvc
import pooler

class res_partner(osv.osv):
    _inherit = 'res.partner'
    
    def check_vat_xx(self, vat):
        return True
    
res_partner()

class account_journal(osv.osv):
    _inherit = "account.journal"
    _columns = {
        'code': fields.char('Code', size=64, required=True, \
         help="The code will be used to generate the numbers of the journal entries of this journal."),
    }

account_journal()


class account_aged_trial_balance(osv.osv_memory):
    _inherit = 'account.aged.trial.balance'
    
    _defaults = {
        'result_selection': lambda *a: 'supplier',
    }

account_aged_trial_balance()

class account_invoice_refund(osv.osv_memory):

    """Refunds invoice"""
    
    """Para sobre escribir el wizard de facturas rectificativas y poder usar 
    siempre por defecto 'Facturas rectificativas'.
    
    Tambien se hereda para colocar en 0.0 el Total al generar una factura
    rectificativa."""

    _inherit = "account.invoice.refund"
    _description = "Sobreescribir Factura Rectificativas"
    
    _defaults = {
        'filter_refund': 'refund',
    }

    def compute_refund(self, cr, uid, ids, mode='refund', context=None):
        """
        Super de compute_refund para colocar las facturas rectificativas con el
        monto check_total en 0
        """
        result = super(account_invoice_refund, self).compute_refund(cr, uid, ids, mode, context)
        inv_obj = self.pool.get('account.invoice')
        if result.get('domain', False):
            for res in result['domain']:
                if res[0] == 'id':
                    inv_obj.write(cr, uid, res[2], {'check_total':0.0}, context)
                    
        return result

account_invoice_refund()

class account_invoice(osv.osv):
    """Se hereda account.invoice para modificar el campo wh_iva_rate, creado 
    por l10n_ve_withholding_iva y colocarlo siempre en readonly, esto porque 
    ese campo se debe llenar por defecto segun el proveedor/cliente"""
    _inherit = "account.invoice"
    
    
    _columns = {
        'wh_iva_rate': fields.float('Wh rate', digits_compute= dp.get_precision('Withhold'), readonly=True, help="Withholding vat rate"),
    }
    
    
    def copy(self, cr, uid, id, default=None, context=None):
        if default is None:
            default = {}
        default = default.copy()
        default.update({'wh_src_id': False, 'wh_src_rate': 0.00})
        return super(account_invoice, self).copy(cr, uid, id, default, context)
    
    
    def validar_button_compute(self, cr, uid, ids):
        #Verificar lo registrado con el boton calcular
        if isinstance(ids,(int, long)):
            inv_id= [ids]
        else:
            inv_id= ids
        self.button_compute(cr, uid, inv_id, set_total=False)
        ai_brw = self.browse(cr, uid, ids)
        for inv in ai_brw:
            if inv.type in ('in_invoice', 'in_refund') and abs(inv.check_total - inv.amount_total) >= (inv.currency_id.rounding/2.0):
                raise osv.except_osv('Total errado !', 'Por favor verifique si ha presionado el boton calcular antes de validar la factura. Los totales pueden ser diferentes.')

        return True


    def validar_campos_obligatorios(self, cr, uid, ids):
        """Funcion para validar que la factura tenga el campo nro_ctrl y
        reference con alguna informacion, en caso de que por vista no se 
        active la obligatoriedad de los campos"""
        
        ai_brw = self.browse(cr, uid, ids)
        for ai in ai_brw:
            if (ai.type in ('in_invoice','in_refund')) and (not ai.nro_ctrl or not ai.reference):
                raise osv.except_osv('Error',"Falta por registrar campo 'Numero de Control' o 'Referencia' en la factura")
        return True

    def validar_account_invoice_refund(self, cr, uid, ids):
        """Funcion para validar que la factura rectificativa de proveedor no se le apruebe si el monto total
        de la factura rectificativa de proveedor es mayor a la factura origen"""
        ai_brw = self.browse(cr, uid, ids)
        for ai in ai_brw:
            if ai.parent_id and ai.type == 'in_refund':
                ai_rect_list = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',ai.parent_id.id),('type','=','in_refund')])
                ai_rect_brw = self.browse(cr, uid, ai_rect_list)
                monto_total = 0
                for fact_rec in ai_rect_brw:
                    monto_total += fact_rec.amount_total
                
                if (monto_total - ai.parent_id.amount_total) > 0.0001:
                    raise osv.except_osv('Error con Factura Rectificativa de Proveedor',"El monto total de la factura origen '%s', es menor que la factura rectificativa que intenta aprobar."%ai.parent_id.number)
                    
            elif ai.parent_id and ai.type == 'out_refund':
                ai_rect_list = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',ai.parent_id.id),('type','=','out_refund')])
                ai_rect_brw = self.browse(cr, uid, ai_rect_list)
                monto_total = 0
                for fact_rec in ai_rect_brw:
                    monto_total += fact_rec.amount_total

                if (monto_total - ai.parent_id.amount_total) > 0.0001:
                    raise osv.except_osv('Error con Factura Rectificativa de Cliente',"El monto total de la factura origen '%s', es menor que la factura rectificativa que intenta aprobar."%ai.parent_id.number)
            
        return True
    
    def validar_account_invoice_line(self, cr, uid, ids):
        """Funcion para validar que la factura no se le apruebe si unas de las
        lineas de la factura no tiene impuesto asociado o el precio unitario es
        menor o igual a 0"""
        ai_brw = self.browse(cr, uid, ids)
        
        for ail in ai_brw[0].invoice_line:
            if ail.price_unit <= 0.0:
                raise osv.except_osv('Error con Precio Unidad',"El precio unidad de '%s', debe tener un valor mayor a 0")
            if ail.invoice_line_tax_id == []:
                raise osv.except_osv('Error con Impuestos',"Se debe asignar impuesto a '%s', asi este sea impuesto exento"%ail.name)
        return True
        

    def action_date_assign(self, cr, uid, ids, *args):
        #Se hereda la función action_date_assign para llamar funcion de validacion
        self.validar_button_compute(cr, uid, ids)
        self.validar_campos_obligatorios(cr, uid, ids)
        self.validar_account_invoice_refund(cr, uid, ids)
        self.validar_account_invoice_line(cr, uid, ids)
        return super(account_invoice, self).action_date_assign(cr, uid, ids, args)

account_invoice()
