# -*- coding: utf-8 -*-

from osv import fields,osv
import tools
import decimal_precision as dp


class diana_out_picking(osv.osv):
    _name = "diana.out.picking"
    _description = "diana_out_picking"
    _auto = False
    _columns = {
        'picking_id': fields.many2one('stock.picking', 'Reference', select=True,states={'done': [('readonly', True)]}),
        'street': fields.char('Street', size=128),
        #~ 'street2': fields.char('Street2', size=128),
        'name': fields.char('Name', size=128, required=True, select=True),
        'address': fields.many2one('res.partner.address', 'Abasto', required=True),
        #~ 'address': fields.one2many('res.partner.address', 'partner_id', 'Contacts'),
        'product_id':fields.many2one('product.product', 'Product', readonly=True),
        'product_uom' : fields.many2one('product.uom', 'Reference UoM', required=True),
        #~ 'product_tmpl_id': fields.many2one('product.template', 'Product Template', required=True, ondelete="cascade"),
        'product_qty': fields.float('Quantity', digits_compute=dp.get_precision('Product UoM'), required=True,states={'done': [('readonly', True)]}),
        'weight': fields.float('Gross weight', help="The gross weight in Kg."),
        'weight_net': fields.float('Net weight', help="The net weight in Kg."),
        'partner_id': fields.many2one('res.partner', 'Partner Name', ondelete='set null', select=True, help="Keep empty for a private address, not related to partner."),
        'price_unit': fields.float('Unit Price', digits_compute= dp.get_precision('Account'), help="Technical field used to record the product cost set by the user during a picking confirmation (when average price costing method is used)"),
        'sp_state': fields.char('Street2', size=128),  
        'date': fields.datetime('Date', required=True, select=True, help="Move date: scheduled date until move is done, then date of actual move processing", readonly=True),
        'date_expected': fields.datetime('Scheduled Date', states={'done': [('readonly', True)]},required=True, select=True),
    }

    def init(self, cr):
        tools.sql.drop_view_if_exists(cr, 'diana_out_picking')
        cr.execute("""
            create or replace view diana_out_picking as (
                SELECT sm.id as id,
                        rpa.name as name,
                        rpa.id as address,
                        rpa.partner_id as partner_id,
                        sp.id as picking_id,
                        sm.product_id as product_id,
                        sm.product_uom as product_uom,
                        pt.weight as weight, 
                        pt.weight_net as weight_net,
                        sum(sm.product_qty) as product_qty,
                        sm.price_unit as price_unit,
                        
                        case when sp.state='done' then 'Realizado' else
                        case when sp.state='draft' then 'Borrador' else 
                        case when sp.state='cancel' then 'Cancelado' else sp.state END END END as sp_state,
                        rpa.street as street,
                        sm.date as date,
                        sm.date_expected as date_expected
             FROM stock_picking sp 
            inner join stock_move sm on sm.picking_id = sp.id
            inner join res_partner_address rpa on rpa.id = sp.address_id
            inner join product_product pp on pp.id = sm.product_id
            inner join product_template pt on pt.id = pp.product_tmpl_id
            inner join product_uom puom on puom.id = sm.product_uom
            where sm.location_dest_id = 128
            group by sm.id,
                        rpa.name,
                        rpa.id,
                        rpa.partner_id,
                        sp.id,
                        sm.product_id,
                        sm.product_uom,
                        pt.weight, 
                        pt.weight_net,
                        sm.price_unit,
                        sp_state,
                        rpa.street,
                        sm.date,
                        sm.date_expected
            )""")
diana_out_picking()
