#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import fields, osv
import netsvc
import stock
import product
import time
from tools.translate import _
import unicodedata

def elimina_tildes(s):
    """
    Funcion para eliminar las tildes de algun texto utilizando el modulo unicodedata.
    """
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn')).replace("'","")
    

class caja_reposicion(osv.osv):
    _name = 'caja.reposicion'
    _description = 'Reposicion de Caja Chica'
caja_reposicion()


class caja_chica_configuracion(osv.osv):
    _name = 'caja.chica.configuracion'
    _descripcion = 'Configuraciones de Caja Chica'
    
    
    _columns = {
        'name': fields.char('Referencia', size=128, readonly=True),
        'max_reposicion': fields.integer('U.T. por Caja Chica', required=True),
        'max_linea': fields.integer('U.T. por Linea de Reposicion', required=True),
        'product_categ': fields.many2one('product.category', 'Categoria de Servicios', required=True),
    }
    
    _defaults = {
        'name': 'Configuracion para Caja Chica',
    }
    
caja_chica_configuracion()


class caja_chica(osv.osv):
    _name = "caja.chica"
    _description = "Caja Chica"
    
    
    def _get_caja_en_uso(self, cr, uid, ids, name, args, context=None):
        """
        Metodo para campo funcional se encarga de verificar si que la caja
        chica se encuentra en uso.
        """
        if context is None:
            context = {}
        
        repo_obj = self.pool.get('caja.reposicion')
        
        res = {}
        for id in ids:
            res[id] = True
            repo_sea = repo_obj.search(cr, uid, [('caja_chica_id','=',id),('state','in',('draft','confirm','done'))], context=context)
            if repo_sea == []:
                res[id] = False

        return res
    
    
    def _get_configuracion(self, cr, uid, context=None):
        """
        Funcion para extraer la configuracion para la caja chica
        """
        if context is None:
            context = {}
        
        fecha_actual = time.strftime('%Y-%m-%d')
        
        config_obj = self.pool.get('caja.chica.configuracion')
        config_ser = config_obj.search(cr, uid,[])
        if config_ser:
            return config_ser[0]
        return False
    
    
    _columns = {
        'name': fields.char('Referencia', size=128, required=True),
        'user_ids': fields.many2many('res.users', 'caja_chica_res_users_rel',
                            'caja_chica_id', 'user_id', 'Responsables',
                            required=True),
        'amount': fields.float('Monto Total', required=True),
        'account_id': fields.many2one('account.account', 'Cuenta Contable', required=True),
        'company_id': fields.many2one('res.company', 'Company'),
        'journal_id': fields.many2one('account.journal', 'Diario de Pago', required=True),
        'journal_move_id': fields.many2one('account.journal', 'Diario de Gasto', required=True),
        'en_uso': fields.function(_get_caja_en_uso, method=True, string='En Uso', type='boolean'),
        'active': fields.boolean('Activa', help="Si este campo esta marcado la caja chica estara activa para ser usada"),
        'config': fields.many2one('caja.chica.configuracion', 'Configuracion', required=True),
    }
    
    _defaults = {
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'active': 1,
        'config': _get_configuracion,
    }
    
    
    def _check_monto_caja_chica(self, cr, uid, ids, context=None):
        """ Verificar que el monto de la caja chica, no exceda la cantidad de
        unidades permitidas configurado para caja chica.
        @return: True (bien) or False (mal)
        """
        config_obj = self.pool.get('caja.chica.configuracion')
        ut_obj = self.pool.get('l10n.ut')
        
        fecha_actual = time.strftime('%Y-%m-%d')
        ut_ser = ut_obj.search(cr, uid, [('date', '<=', fecha_actual)], order='date desc')
        
        if not ut_ser:
            raise osv.except_osv(('Error !'), ('No existe unidad tributaria creada'))
        
        for ca_ch in self.browse(cr, uid, ids):
            if not ca_ch.config:
                raise osv.except_osv(('Error !'), ('No existe configuracion para caja chica creada'))
            
            maximo_ut = float(ca_ch.config.max_reposicion)
            ut_brw = ut_obj.browse(cr, uid, [ut_ser[0]])[0]
            ut_amount = float(ut_brw.amount)
            monto_caja = float(ca_ch.amount)
            monto_permitido = maximo_ut * ut_amount
            if (monto_permitido - monto_caja) < 0:
                raise osv.except_osv(('Error !'), ('El saldo de la caja chica es superior a las %s UT permitida'%(maximo_ut)))
                return False

        return True
        
    _constraints = [(_check_monto_caja_chica, 'El saldo de la caja chica es superior al monto permitido para una caja chica', ['amount'])]
    
caja_chica()


class caja_reposicion_linea(osv.osv):
    _name = "caja.reposicion.linea"
    _description = "Lineas la Reposicion de Caja"
    

    _columns = {
        'name': fields.char('Referencia', size=128, required=True),
        'concepto': fields.char('Concepto', size=128, required=True),
        'caja_reposicion_id': fields.many2one('caja.reposicion', 'Caja Reposicion'),
        'amount': fields.float('Monto', required=True),
        'account_id': fields.many2one('account.account', 'Cuenta Contable', required=True),
        'analytic_id': fields.many2one('account.analytic.account', 'Cuenta Analitica'),
        'product_id': fields.many2one('product.product', 'Servicio', required=True),
        'date' : fields.date('Fecha de Emision', required=True),
        'state': fields.selection([('draft','Borrador'), ('done','Pagado'), 
                            ('cancel','Cancelado')], 'Estado', readonly=True),
    }
    
    _defaults = {
        'state': 'draft',
    }
    
    def _check_linea_reposicion(self, cr, uid, ids, context=None):
        """ Verificar que las lineas de la reposicion de caja chica,
        no exceda la cantidad de unidades permitidas configurado para caja chica.
        @return: True (bien) or False (mal)
        """
        if context is None:
            context = {}
        
        repo_obj = self.pool.get('caja.reposicion')
        config_obj = self.pool.get('caja.chica.configuracion')
        ut_obj = self.pool.get('l10n.ut')
        
        fecha_actual = time.strftime('%Y-%m-%d')
        ut_ser = ut_obj.search(cr, uid, [('date', '<=', fecha_actual)], order='date desc')
        config_ser = config_obj.search(cr, uid, [])
        
        if not ut_ser:
            raise osv.except_osv(('Error !'), ('No existe unidad tributaria creada'))
            
        for lin_rep in self.browse(cr, uid, ids):
            if lin_rep.amount <= 0:
                raise osv.except_osv(('Error !'), ('Debe colocar un monto mayor a 0'))
            
            ut_brw = ut_obj.browse(cr, uid, [ut_ser[0]])[0]
            max_line_total = 0
            config_brw = lin_rep.caja_reposicion_id.caja_chica_id and lin_rep.caja_reposicion_id.caja_chica_id.config or False
            if not config_brw:
                config_brw = config_obj.browse(cr, uid, [config_ser[0]])[0]
            if not config_brw:
                raise osv.except_osv(('Error !'), ('Se debe crear una configuracion para el manejo de caja chica.'))
                
            ut_amount = ut_brw.amount
            max_line_ut = config_brw.max_linea
            max_line_total = max_line_ut * ut_amount
            if lin_rep.amount - max_line_total > 0.0001:
                raise osv.except_osv(('Error !'), ('La linea "%s" excede las %s Unidades Tributarias. No puede ser aprobada la reposicion.'%(lin_rep.name,max_line_ut)))
                return False
                
        return True
        
    _constraints = [(_check_linea_reposicion, 'El monto de la linea excede las Unidades Tributarias permitidas. No puede ser aprobada la reposicion.', ['amount'])]
    
    
    def onchange_product_id(self, cr, uid, ids, product_id, context=None):
        """
        On_change para extraer la cuenta de gasto del producto.
        """
        if context is None:
            context = {}
        
        res = {}
        product_obj = self.pool.get('product.product')
        if product_id:
            product_brw = product_obj.browse(cr, uid, [product_id], context)[0]
            res.update({
                'account_id': product_brw.property_account_expense and product_brw.property_account_expense.id or False,
            })
            
        return {
            'value':res
        }
        
caja_reposicion_linea()


class caja_reposicion(osv.osv):
    _name = 'caja.reposicion'
    _description = 'Reposicion de Caja Chica'
    _order = "id desc"


    def _get_amount_total(self, cr, uid, ids, name, args, context=None):
        """
        Metodo para calcular el monto total de la reposicion
        """
        if context is None:
            context = {}
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        res = {}
        for id in ids:
            res[id] = 0.00
        
        for repo in repo_brw:
            for line in repo.reposicion_linea_ids:
                res[repo.id] = res[repo.id] + line.amount
                
        return res
    
    
    _columns = {
        'name': fields.char('Secuencia de movimiento', size=128, readonly=True),
        'ref': fields.char('Referencia de movimiento', size=128, required=True,
                        readonly=True,
                        states={'draft':[('readonly',False)]}),
        'reposicion_linea_ids': fields.one2many('caja.reposicion.linea', 
                            'caja_reposicion_id', 'Lineas Reposicion', 
                            states={'draft':[('readonly',False)], 
                                    'confirm':[('readonly',True)], 
                                    'done':[('readonly',True)], 
                                    'paid':[('readonly',True)], 
                                    'cancel':[('readonly',True)]}),
        'confirmado_por': fields.many2one('res.users', 'Confirmado por', readonly=True),
        'aprobado_por': fields.many2one('res.users', 'Aprobado por', readonly=True),
        'fecha_aprobado': fields.date('Fecha de Aprobacion', readonly=True),
        'fecha_confirmado': fields.date('Fecha Confirmado', readonly=True),
        'caja_chica_id': fields.many2one('caja.chica', 'Caja Chica', required= True,
                            states={'draft':[('readonly',False)], 
                                    'confirm':[('readonly',True)], 
                                    'done':[('readonly',True)], 
                                    'paid':[('readonly',True)], 
                                    'cancel':[('readonly',True)]}),
        'move_ids': fields.many2many('account.move.line', 'caja_reposicion_move_line_rel',
                            'reposicion_id', 'imputacion_id', 'Asientos Contable', 
                            readonly=True),
        'amount_total': fields.function(_get_amount_total, method=True, string='Monto Total'),
        'company_id': fields.many2one('res.company', 'Company'),
        'state': fields.selection([('draft','Borrador'), ('confirm','Confirmado'), 
                            ('done','Aprobado'), ('paid', 'Orden de Pago Creada'), ('final','Pagado'),('cancel','Rechazada')],
                            'Estado', readonly=True),
        #~ 'department_id': fields.many2one('hr.department','Departamento'),
        'period_id': fields.many2one('account.period', 'Periodo', readonly=True),
        'product_categ': fields.related('caja_chica_id','config','product_categ',type='many2one',relation='product.category',string='Categoria', store=True, readonly=True),
    }
    
    _defaults = {
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'state': 'draft',
    }
    
    
    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(('Error !'), ('No se permite duplicar una reposición'))

    
    def create(self, cr, uid, values, context=None):
        if context is None:
            context = {}
        
        sequence_obj = self.pool.get('ir.sequence')
        caja_obj = self.pool.get('caja.chica')
        
        if values.get('caja_chica_id', False):
            caja_brw = caja_obj.browse(cr, uid, [values['caja_chica_id']])[0]
            if caja_brw.en_uso:
                raise osv.except_osv(('Error !'), ('No se puede crear una reposicion para esta caja chica, ya existe una reposicion para %s'%caja_brw.name))
        
        values['name'] = sequence_obj.get(cr, uid, 'caja.reposicion')
        return super(caja_reposicion, self).create(cr, uid, values, context=context)


    def log_reposicion(self, cr, uid, repo_id, context=None):
        if context is None:
            context = {}
        
        repo_brw = self.browse(cr, uid, [repo_id])[0]
        name_ref = repo_brw.name + '/' + repo_brw.ref
        if context.get('estado', '') == 'confirm':
            mensaje = 'Se ha confirmado la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        elif context.get('estado', '') == 'done':
            mensaje = 'Se ha aprobado la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        elif context.get('estado', '') == 'paid':
            mensaje = 'Se ha creado orden de pago de la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        elif context.get('estado', '') == 'cancel':
            mensaje = 'Se ha cancelado la reposicion "' + name_ref + '" de la caja chica "' + repo_brw.caja_chica_id.name + '"'
        else:
            mensaje = 'Se modifico reposicion ' + name_ref

        self.log(cr, uid, repo_id, mensaje)


    def cancel_draft(self, cr, uid, ids, context=None):
        """
        Funcion para pasar a cancel una reposicion, solo cuando esta en draft
        """
        if context is None:
            context = {}
            
        self.write(cr, uid, ids, {'state':'cancel','confirmado_por': None,
        'fecha_confirmado': None,}, context=context)
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        return True
        
        
    def cancel_reposicion(self, cr, uid, ids, context=None):
        """
        Funcion para pasar a cancel una reposicion, solo cuando esta confirmada
        """
        if context is None:
            context = {}
        
        self.write(cr, uid, ids, {'state':'cancel','confirmado_por': None,
        'fecha_confirmado': None,}, context=context)
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        return True

        
    def confirmar_reposicion(self, cr, uid, ids, context=None):
        """
        Funcion para confirmar la reposicion de la caja chica.
        """
        if context is None:
            context = {}
        
        for repo in self.browse(cr, uid, ids):
            """
            Inicio de validacion de monto de reposicion no exceda monto de caja chica.
            """
            if (repo.amount_total - repo.caja_chica_id.amount) > 0.0001:
                raise osv.except_osv(('Error !'), ('La reposicion %s no se puede confirmar, excede el monto total de la caja chica (%s)'%(repo.name, repo.caja_chica_id.amount)))
            if not repo.reposicion_linea_ids:
                raise osv.except_osv(('Error !'), ('Debe agregar lineas a la reposicion antes de confirmar'))
            for line in repo.reposicion_linea_ids:
                if line.amount <= 0:
                    raise osv.except_osv(('Error !'), ('La linea "%s" debe tener un monto mayor a 0'%(line.name)))

        self.write(cr, uid, ids, {'state': 'confirm', 'confirmado_por': uid, 'fecha_confirmado': time.strftime('%Y-%m-%d')}, context=context)
        
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        
        return True
        
        
    def validar_reposicion(self, cr, uid, ids, context=None):
        """
        Aqui se aprueba y se generan los asientos contables al momento de aprobar
        la reposicion.
        """
        if context is None:
            context = {}
        
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        repo_line_obj = self.pool.get('caja.reposicion.linea')
        config_obj = self.pool.get('caja.chica.configuracion')
        obj_company = self.pool.get('res.company')
        ut_obj = self.pool.get('l10n.ut')
        
        fecha_actual = time.strftime('%Y-%m-%d')
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        
        """
        Inicio de validacion de las lineas de la caja chica para que no excedan
        el limite definido para las cajas chicas.
        """
        ut_ser = ut_obj.search(cr, uid, [('date', '<=', fecha_actual)], order='date desc')
        config_ser = config_obj.search(cr, uid, [])
        if not ut_ser:
            raise osv.except_osv(('Error !'), ('No existe unidad tributaria creada'))
        
        ut_brw = ut_obj.browse(cr, uid, [ut_ser[0]])[0]
        
        for repo in repo_brw:
            max_line_total = 0
            config_brw = repo.caja_chica_id.config
            if not config_brw:
                config_brw = config_obj.browse(cr, uid, [config_ser[0]])[0]
            if not config_brw:
                raise osv.except_osv(('Error !'), ('Se debe crear una configuracion para el manejo de caja chica.'))
            ut_amount = ut_brw.amount
            max_line_ut = config_brw.max_linea
            max_line_total = max_line_ut * ut_amount
            for line in repo.reposicion_linea_ids:
                if line.amount - max_line_total > 0.0001:
                    raise osv.except_osv(('Error !'), ('La linea "%s" excede las %s Unidades Tributarias. No puede ser aprobada la reposicion.'%(line.name,max_line_ut)))
                if line.account_id.type == 'view':
                    raise osv.except_osv(('Error !'), ('Imposible aprobar la reposicion. En la linea %s, se eligio una cuenta de tipo vista. (%s %s)' % (line.name, line.account_id.code, line.account_id.name)))
                    
        #Fin de la validacion
        
        for repo in repo_brw:
            caja_brw = repo.caja_chica_id
            move_dict = {}
            ref = elimina_tildes(repo.name + ' / ' + repo.ref)
            move_dict = {
                'ref': ref,
                'journal_id': caja_brw.journal_move_id and caja_brw.journal_move_id.id or None,
                'to_check': True,
            }
            move_id = move_obj.create(cr, uid, move_dict)
            move_brw = move_obj.browse(cr, uid, [move_id])[0]
            periodo_id = move_brw.period_id.id
            context.update({'period_id': periodo_id})
            
            move_line_ids = []
            company_id = obj_company._company_default_get(cr, uid, 'account.move', context=context)
            for line in repo.reposicion_linea_ids:
                move_line_dict = {}
                name = elimina_tildes(line.name + ' / ' + line.concepto)
                move_line_dict = {
                    'name': name[:64],
                    'ref': ref[:64],
                    #~ 'quantity': ,
                    #~ 'product_uom_id': ,
                    'product_id': line.product_id and line.product_id.id or None,
                    'debit': line.amount,
                    'credit': 0.00,
                    'account_id': line.account_id and line.account_id.id or None,
                    'move_id': move_id,
                    'journal_id': caja_brw.journal_move_id and caja_brw.journal_move_id.id or None,
                    'analytic_account_id': line.analytic_id and line.analytic_id.id or 'NULL',
                    'state': 'valid',
                    'period_id': periodo_id,
                }
                
                #Creacion de apunte contable para debito por medio de query
                cr.execute("insert into account_move_line (account_id,name,ref,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,product_id,analytic_account_id) values (%s,'%s','%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now(),%s,%s)"%(move_line_dict['account_id'],move_line_dict['name'],move_line_dict['ref'],move_line_dict['journal_id'],int(company_id),move_line_dict['credit'],move_line_dict['state'],move_line_dict['period_id'],move_line_dict['debit'],str(fecha_actual),move_line_dict['move_id'],uid,move_line_dict['product_id'],move_line_dict['analytic_account_id']))
                cr.commit()

                move_line_dict.update({'debit': 0.00, 'credit': line.amount, 'account_id': caja_brw.account_id and caja_brw.account_id.id or None})
                
                #Creacion de apunte contable para credito por medio de query
                cr.execute("insert into account_move_line (account_id,name,ref,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,product_id,analytic_account_id) values (%s,'%s','%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now(),%s,%s)"%(move_line_dict['account_id'],move_line_dict['name'],move_line_dict['ref'],move_line_dict['journal_id'],int(company_id),move_line_dict['credit'],move_line_dict['state'],move_line_dict['period_id'],move_line_dict['debit'],str(fecha_actual),move_line_dict['move_id'],uid,move_line_dict['product_id'],move_line_dict['analytic_account_id']))
                cr.commit()

            #Asentar el asiento contable de la caja chica.
            move_obj.button_validate(cr, uid, [move_id], context=context)
            
            #Unir lineas de asiento a la reposicion
            move_line_ids = [mo.id for mo in move_brw.line_id]
            self.write(cr, uid, [repo.id], {'move_ids': [(6, 0, move_line_ids)]}, context=context)
        
        self.write(cr, uid, ids, {'state': 'done', 'aprobado_por': uid, 
            'fecha_aprobado': fecha_actual, 'period_id': periodo_id}, context=context)
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)

        return True
        

    def voucher_reposicion(self, cr, uid, ids, context=None):
        """
        Crea un account.voucher para pagar la caja chica
        """
        if context is None:
            context = {}
        
        voucher_obj = self.pool.get('account.voucher')
        voucher_line_obj = self.pool.get('account.voucher.line')
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        
        for repo in repo_brw:
            journal_id = repo.caja_chica_id.journal_id and repo.caja_chica_id.journal_id.id or False
            account_id = repo.caja_chica_id.journal_id and repo.caja_chica_id.journal_id.default_credit_account_id.id or False
            partner_id = repo.company_id.partner_id.id
            
            voucher_dict = {
                'type': 'payment',
                'name': repo.name + ' / ' + repo.ref,
                'journal_id': journal_id,
                'account_id': account_id,
                #~ 'narration': 'NARRATION SERA EL MEMORIAL?',
                #~ 'currency_id': PUEDE SER QUE LO USE,
                'amount': repo.amount_total,
                'partner_id': partner_id,
                #~ 'comment': 'ESTE SERA EL MEMORIAL',
                'pay': 'check',
                #~ 'analytic_id': PUEDE SER QUE LO USE,
            }
            voucher_id = voucher_obj.create(cr, uid, voucher_dict)
            
            for line in repo.reposicion_linea_ids:
                
                vou_lin_dict = {
                    'voucher_id': voucher_id,
                    'partner_id': partner_id,
                    'name': line.name + ' / ' + line.concepto,
                    'account_id': repo.caja_chica_id.account_id.id,
                    #~ 'untax_amount': fields.float('Untax Amount'),
                    'amount': line.amount,
                    'type': 'dr',#fields.selection([('dr','Debit'),('cr','Credit')], 'Cr/Dr'),
                    #~ 'account_analytic_id': line.analytic_id.id,
                }
                vou_lin_id = voucher_line_obj.create(cr, uid, vou_lin_dict, context=context)
            # Mensaje Usuario
            message = 'Se genero orden de pago ' + repo.name + '/' + repo.ref
            voucher_obj.log(cr, uid, voucher_id, message, context=context)
        
        self.write(cr, uid, ids, {'state': 'paid'})
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        
        return True
        
        
    def action_cancel_draft(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        self.write(cr, uid, ids, {'state': 'draft', 'confirmado_por': None,
            'fecha_confirmado': None,}, context=context)
        
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
            
        return True
        
    
caja_reposicion()
