#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import fields, osv
import netsvc
import stock
import product
import time
from tools.translate import _


class caja_chica(osv.osv):
    _inherit = "caja.chica"
    
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica',
                            required=True),
    }
    
caja_chica()


class caja_reposicion_linea(osv.osv):
    _inherit = "caja.reposicion.linea"
    
    
    def _get_accion_especifica(self, cr, uid, context=None):
        if context is None:
            context = {}
            
        repo_obj = self.pool.get('caja.reposicion')
        if context.get('caja_reposicion_id', False):
             repo_brw = repo_obj.browse(cr, uid, [context['caja_reposicion_id']])[0]
             return repo_brw.caja_chica_id.accion_especifica_id.id
        
        return False
        
    
    _columns = {
        'enlace_id': fields.integer('Enlace ID'),
        'partida_id': fields.many2one('clasificador.presupuestario', 
                            'Partida Presupuestaria', required=True),
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica',
                            required=True),
    }
    
    _defaults = {
        'accion_especifica_id': _get_accion_especifica,
    }
    
    
    def onchange_product_id(self, cr, uid, ids, product_id, context=None):
        """
        On_change para extraer la cuenta de gasto y partida presupuesto de gasto
        del producto.
        """
        if context is None:
            context = {}
        
        res = {}
        product_obj = self.pool.get('product.product')
        if product_id:
            product_brw = product_obj.browse(cr, uid, [product_id], context)[0]
            res.update({
                'partida_id': product_brw.partida_egreso and product_brw.partida_egreso.id or False,
                'account_id': product_brw.property_account_expense and product_brw.property_account_expense.id or False,
            })
            
        return {
            'value':res
        }
        
caja_reposicion_linea()


class caja_reposicion(osv.osv):
    _inherit = 'caja.reposicion'


    _columns = {
        'imputacion_ids': fields.many2many('presupuesto.imputacion.linea', 
                            'caja_reposicion_imputacion_rel', 'reposicion_id', 
                            'imputacion_id', 'Imputaciones', readonly=True),
    }

        
    def validar_reposicion(self, cr, uid, ids, context=None):
        """
        Se realiza una herencia de la funcion validar_reposicion para crear las 
        imputaciones de comprometido y causado
        """
        if context is None:
            context = {}
            
        res = super(caja_reposicion, self).validar_reposicion(cr, uid, ids, context=context)
        
        repo_line_obj = self.pool.get('caja.reposicion.linea')
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        
        for repo in repo_brw:
            caja_brw = repo.caja_chica_id
            for line in repo.reposicion_linea_ids:
                """
                Desde aqui se llaman las funciones para la imputacion de la caja chica.
                """
                #Imputacion del comprometido
                val = self._imputar_gasto_reposicion(cr, uid, repo.id, line, 'comprometido')
                if val:
                    repo_line_obj.write(cr, uid, [line.id], {'enlace_id': val}, context=context)
                    self.write(cr, uid, [repo.id], {'imputacion_ids': [(4, val)]}, context=context)

                #Imputacion del causado
                val = self._imputar_gasto_reposicion(cr, uid, repo.id, line, 'causado_gasto', val, context=None)
                if val:
                    self.write(cr, uid, [repo.id], {'imputacion_ids': [(4, val)]}, context=context)

        return res


    def _imputar_gasto_reposicion(self, cr, uid, ids, line, status, enlace_id=False, context=None):
        """
        Funcion que crea los datos para la imputacion de la caja chica, comprometido
        y causado.
        Se crea diccionario para la imputacion presupuestaria.
        """
        if context is None:
            context = {}

        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        period_obj = self.pool.get('periodo.presupuestario')
        
        fecha = line.date
        periodo = period_obj.find(cr, uid, fecha, context=context)[0]
        acc_esp_id = line.accion_especifica_id.id
        partida = ''
        
        if not line.partida_id:
            raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el producto %s.'%line.product_id.name))
        else:
            partida = line.partida_id and line.partida_id.code or ''
            partida += ' '+ line.partida_id.name
        
        name_ref = line.caja_reposicion_id.name + '/' + line.caja_reposicion_id.ref + '/' + line.name + '/' + line.concepto
        
        imp_dict = {
            'ref': name_ref,
            'partida': partida,
            'partida_id': line.partida_id.id,
            'model': 'caja.reposicion.linea',
            'res_id': line.id,
            'fecha_imp': fecha,
            'amount': line.amount,
            'periodo': periodo,
            'accion_especifica_id': acc_esp_id,
            'status': status,
            'state': 'activo',
            'type': 'egreso',
            'type_create': 'automatica',
        }
        
        if not enlace_id:
            imp_id = imp_obj.create(cr, uid, imp_dict, context)
            imp_obj.write(cr, uid, [imp_id], {'enlace_id': imp_id}, context=context)
        else:
            imp_dict.update({'enlace_id': enlace_id})
            imp_id = imp_obj.create(cr, uid, imp_dict, context)
        
        return imp_id


    def voucher_reposicion(self, cr, uid, ids, context=None):
        """
        Crea un account.voucher para pagar la caja chica y agregar enlace_id
        para que no genere las imputaciones presupuestarias de comprometido y
        causado.
        """
        if context is None:
            context = {}
        
        voucher_obj = self.pool.get('account.voucher')
        voucher_line_obj = self.pool.get('account.voucher.line')
        
        repo_brw = self.browse(cr, uid, ids, context=context)
        
        for repo in repo_brw:
            journal_id = repo.caja_chica_id.journal_id and repo.caja_chica_id.journal_id.id or False
            account_id = repo.caja_chica_id.journal_id and repo.caja_chica_id.journal_id.default_credit_account_id.id or False
            partner_id = repo.company_id.partner_id.id
            
            voucher_dict = {
                'type': 'payment',
                'name': repo.name + ' / ' + repo.ref,
                'journal_id': journal_id,
                'account_id': account_id,
                #~ 'narration': 'NARRATION SERA EL MEMORIAL?',
                #~ 'currency_id': PUEDE SER QUE LO USE,
                'amount': repo.amount_total,
                'partner_id': partner_id,
                #~ 'comment': 'ESTE SERA EL MEMORIAL',
                'pay': 'check',
                #~ 'analytic_id': PUEDE SER QUE LO USE,
                'reposicion_id': repo.id,
            }
            voucher_id = voucher_obj.create(cr, uid, voucher_dict, context=context)
            
            for line in repo.reposicion_linea_ids:
                
                vou_lin_dict = {
                    'voucher_id': voucher_id,
                    'partner_id': partner_id,
                    'name': line.name + ' / ' + line.concepto,
                    'account_id': repo.caja_chica_id.account_id.id,
                    #~ 'untax_amount': fields.float('Untax Amount'),
                    'amount': line.amount,
                    'type': 'dr',#fields.selection([('dr','Debit'),('cr','Credit')], 'Cr/Dr'),
                    #~ 'account_analytic_id': line.analytic_id.id,
                    'accion_especifica_id': line.accion_especifica_id.id,
                    'partida_presupuesto': line.partida_id.id,
                    'enlace_id': line.enlace_id,
                }
                vou_lin_id = voucher_line_obj.create(cr, uid, vou_lin_dict)
            # Mensaje Usuario
            message = 'Se genero orden de pago ' + repo.name + '/' + repo.ref
            voucher_obj.log(cr, uid, voucher_id, message, context=context)
        
        self.write(cr, uid, ids, {'state': 'paid'})
        for repo in self.browse(cr, uid, ids):
            context.update({'estado': repo.state})
            self.log_reposicion(cr, uid, repo.id, context=context)
        
        return True
        
    
caja_reposicion()


class account_voucher(osv.osv):
    _inherit = "account.voucher"
    _name = "account.voucher"
    
    _columns = {
        'reposicion_id': fields.many2one('caja.reposicion', 'Caja Chica Reposicion'),
    }
    
    def _accion_imputar(self, cr , uid, ids, context=None):
        res = super(account_voucher,self)._accion_imputar(cr,uid,ids,context=context)
        
        if res:
            av_brw = self.browse(cr, uid, ids, context=context)[0]
            imp_obj = self.pool.get('presupuesto.imputacion.linea')
            repo_obj = self.pool.get('caja.reposicion')
            if av_brw.reposicion_id:
                user_forced = 1
                avl_ids = []
                for avl_id in av_brw.line_ids:
                    avl_ids.append(avl_id.id)
                av_ser = imp_obj.search(cr, uid, [('res_id', 'in', avl_ids),('model', '=', 'account.voucher.line'),('status','=','pagado')])
                for imp_vou_id in av_ser:
                    repo_obj.write(cr, user_forced, [av_brw.reposicion_id.id], {'imputacion_ids': [(4, imp_vou_id)]}, context=context)
                repo_obj.write(cr, user_forced, [av_brw.reposicion_id.id], {'state': 'final'}, context=context)
        
        return res
    
account_voucher()
