#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    SISB, Sistema Integral Simón Bolívar
#    Copyright (C) 2011-2013 Industrias Diana, C.A. (<http://www.industriasdiana.gob.ve>).
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################
from osv import osv
from osv import fields
from tools.translate import _
from tools import config
import time
import datetime
import decimal_precision as dp

class account_voucher(osv.osv):
    _inherit = "account.voucher"
    _name = "account.voucher"
    
    def bot_imputacion(self, cr, uid, imp_dict, context=None):
        """
        Funcion que se encarga de crear las imputaciones de gasto del
        estado comprometido y causado cuando la orden de pago se crea
        de forma manual y no tiene asociado una factura.
        return enlace_id, que es el id de la primera imputacion creada
        (comprometido).
        @imp_dict, es el diccionario con los datos de la imputacion que se creara
        """
        imp_obj = self.pool.get('presupuesto.imputacion.linea')
        #Comprometido
        imp_dict.update({'status': 'comprometido'})
        enlace_id = imp_obj.create(cr, uid, imp_dict, context=context)
        imp_obj.write(cr, uid, enlace_id, {'enlace_id': enlace_id})
        #Causado
        imp_dict.update({'status': 'causado_gasto', 'enlace_id': enlace_id})
        imp_obj.create(cr, uid, imp_dict, context=context)
        
        return enlace_id
    
    
    def bot_imputacion_ingreso(self, cr, uid, imp_dict, context=None):
        """
        Funcion que se encarga de crear las imputaciones de ingreso del
        estado devengado y liquidado cuando la orden de pago se crea de forma
        manual y no tiene asociado una factura.
        return enlace_id, que es el id de la primera imputacion creada
        (devengado).
        @imp_dict, es el diccionario con los datos de la imputacion que se creara
        """
        imp_obj = self.pool.get('presupuesto.imputacion.linea')
        #Devengado
        imp_dict.update({'status_ingreso': 'devengado'})
        enlace_id = imp_obj.create(cr, uid, imp_dict, context=context)
        imp_obj.write(cr, uid, enlace_id, {'enlace_id': enlace_id})
        #Liquidado
        imp_dict.update({'status_ingreso': 'liquidado', 'enlace_id': enlace_id})
        imp_obj.create(cr, uid, imp_dict, context=context)
         
        return enlace_id
    
    
    def _accion_imputar(self, cr, uid, ids, context=None):
        """
        Funcion que crea las imputaciones presupuestarias de la orden de pago.
        """
        ai_obj = self.pool.get('account.invoice')
        imp_obj = self.pool.get('presupuesto.imputacion.linea')
        period_obj = self.pool.get('periodo.presupuestario')
        voucher_line_obj = self.pool.get('account.voucher.line')
        fecha = time.strftime('%Y-%m-%d')
        
        av_obj = self.browse(cr, uid, ids, context=context)
        
        for avl in av_obj[0].line_ids:
            imp_dict = {}
            """
            Si orden de pago se realiza a partir de factura se llama dist_payment,
            de lo contrario se imputara el comprometido, causado y pago o devengado,
            liquidado y recaudado dependiendo del caso.
            """
            if avl.invoice_id:
                res = ai_obj.dist_payment(cr, uid, 'account.voucher.line', avl.id, avl.invoice_id.id, avl.amount, context['residuales'][str(avl.invoice_id.id)], context=None)
            else:
                ae_id = avl.accion_especifica_id and avl.accion_especifica_id.id or None
                partida_id = avl.partida_presupuesto and avl.partida_presupuesto.id or None
                partida = avl.partida_presupuesto and avl.partida_presupuesto.code or None
                partida_aux = avl.partida_presupuesto and avl.partida_presupuesto.code or ''
                
                if ae_id is None:
                    raise osv.except_osv(('Error !'), ('No hay accion especifica definida para la linea %s.'%avl.name))
                
                if partida is None:
                    raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%avl.name))
                else:
                    partida_id = avl.partida_presupuesto and avl.partida_presupuesto.id or None
                    name = avl.partida_presupuesto and avl.partida_presupuesto.name or ' '
                    partida_aux += ' '+name
                
                imp_dict = {
                    'ref': "Pago %s"%av_obj[0].number,
                    'partida': partida,
                    'partida_id': partida_id,
                    'model': 'account.voucher.line',
                    'res_id': avl.id,
                    'fecha_imp': fecha,
                    'amount': avl.amount,
                    'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                    'accion_especifica_id': ae_id,
                    'status': '',
                    'state': 'activo',
                    'type': 'egreso',
                    'type_create':'automatica',
                }
                
                """
                Si la orden de pago es de tipo 'payment' es una orden de pago
                del proveedor, se realiza las imputaciones para el gasto, sino
                se realizaran las imputaciones para el ingreso.
                """
                if avl.voucher_id.type == 'payment':
                    if not avl.enlace_id:
                        enlace_id = self.bot_imputacion(cr, uid, imp_dict, context=context)
                    else:
                        enlace_id = avl.enlace_id
                    imp_dict.update({'status': 'pagado', 'enlace_id': enlace_id})
                else:
                    if not avl.enlace_id:
                        imp_dict.update({'status': '', 'type': 'ingreso'})
                        enlace_id = self.bot_imputacion_ingreso(cr, uid, imp_dict, context=context)
                    else:
                        enlace_id = avl.enlace_id
                    imp_dict.update({'status_ingreso': 'recaudado', 'enlace_id': enlace_id})
                
                """
                Se marca la linea de la orden de pago con el enlace_id, para saber
                que la linea ya fue imputada. y tener una referencia de cual fue
                primera imputacion creada.
                """
                voucher_line_obj.write(cr, uid, avl.id, {'enlace_id': enlace_id})
                imp_id = imp_obj.create(cr, uid, imp_dict, context)
                
        return True
    
    
    def proforma_voucher(self, cr, uid, ids, context=None):
        """
        Herencia de la funcion proforma_voucher para realizar las imputaciones
        presupuestaria de un pago, una vez se realice el pago.
        """
        if context is None:
            context = {}
        
        """
        Se calcula el pendiente de cada factura que se le este haciendo un pago,
        antes de que se realice el pago, esto para evitar el cambie en el pendiente.
        El pendiente se envia por el context.
        """
        av_obj = self.browse(cr, uid, ids, context=context)
        residuales = {}
        for avl in av_obj[0].line_ids:
            if avl.invoice_id:
                residuales[str(avl.invoice_id.id)] = avl.invoice_id.residual
        
        res = super(account_voucher,self).proforma_voucher(cr,uid,ids,context=context)
        if res:
            context.update({'residuales': residuales})

            self._accion_imputar(cr,uid,ids,context=context)
        return res
    
    
    def eliminar_imputacion_pago(self, cr, uid, ids, context=None):
        """
        Funcion que se encarga de eliminar las imputaciones que esten asociadas
        a la linea de la orden de pago. Esto se realiza al cancelar una orden de pago.
        Se eliminan las imputaciones que tengan el mismo modelo y el res_id sea igual
        al id de la linea de pago.
        """
        pil_obj = self.pool.get('presupuesto.imputacion.linea')
        av_brw = self.browse(cr, uid, ids)
        
        for avl in av_brw[0].line_ids:
            pil_ids = pil_obj.search(cr, uid, [('model','=','account.voucher.line'),('res_id','=', avl.id)])
            pil_obj.unlink(cr, uid, pil_ids)
        
        return True
    
    
    def cancel_voucher(self, cr, uid, ids, context=None):
        """
        Herencia de cancel_voucher para eliminar imputaciones de una orden de pago
        cuando esta se cancele.
        """
        av_brw = self.browse(cr, uid, ids)
        number = av_brw[0].number
        res = super(account_voucher, self).cancel_voucher(cr, uid, ids, context=context)
        if res:
            self.eliminar_imputacion_pago(cr, uid, ids, context=context)
        return res
    
account_voucher()


class account_voucher_line(osv.osv):
    _inherit = "account.voucher.line"
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
        'partida_presupuesto': fields.many2one('clasificador.presupuestario','Partida Presupuestaria'),
        'enlace_id': fields.integer('ID del Enlace'),
    }
    
account_voucher_line()


class account_invoice_line(osv.osv):
    _inherit = "account.invoice.line"

    def _pagos_imputados(self, cr, uid, ids, field_names, args, context=None):
        """
        Para los campos funcionales restante_monto_2 y restante_iva_2, esta funcion
        calcula cantidad el monto pagado e iva pagado que se encuentra almacenado
        en la relacion many2many de monto_pago_ids y iva_pago_ids.
        """
        if context is None:
            context = {}
        res = {}
        inv_line_brw = self.browse(cr, uid, ids)
        for line in inv_line_brw:
            monto = 0.0
            iva = 0.0
            if 'restante_monto_2' in field_names:
                for imp_m in line.monto_pago_ids:
                    monto += imp_m.amount
            if 'restante_iva_2' in field_names:
                for imp_i in line.iva_pago_ids:
                    iva += imp_i.amount
            res[line.id] = {'restante_monto_2': monto, 'restante_iva_2': iva}
        return res


    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
        'enlace_id': fields.integer('ID del Enlace'),
        'partida_presupuesto': fields.many2one('clasificador.presupuestario','Partida Presupuestaria'),
        'restante_monto': fields.float('Rest. Monto', digits_compute=dp.get_precision('Account')),
        'restante_iva': fields.float('Rest. IVA', digits_compute=dp.get_precision('Account')),
        'monto_pago_ids': fields.many2many('presupuesto.imputacion.linea', 'presupuesto_imputacion_monto_pagado_ids', 'invoice_id', 'imputacion_id', 'Pagos del monto'),
        'iva_pago_ids': fields.many2many('presupuesto.imputacion.linea', 'presupuesto_imputacion_iva_pagado_ids', 'iva_id', 'imputacion_id', 'Pagos de IVA'),
        'restante_monto_2': fields.function(_pagos_imputados, digits_compute=dp.get_precision('Account'), method=True, string="Restante Monto", multi='restante_monto_2'),
        'restante_iva_2': fields.function(_pagos_imputados, digits_compute=dp.get_precision('Account'),  method=True, string="Restante IVA", multi='restante_monto_2'),
    }
    
    
    _defaults = {
        'restante_monto': 0.0,
        'restante_iva': 0.0,
    }
    
account_invoice_line()


class account_invoice(osv.osv):
    _inherit = "account.invoice"
    
    def validar_factura_rectificativa(self, cr, uid, ids):
        """
        Funcion para validar que la factura rectificativa no se apruebe sin una 
        factura padre.
        """
        ai_brw = self.browse(cr, uid, ids)
        
        for ai in ai_brw:
            if ai.type in ["in_refund", "out_refund"]:
                if not ai.parent_id:
                    raise osv.except_osv('Error Factura', "No se ha elegido una factura padre")
        return True


    def action_date_assign(self, cr, uid, ids, *args):
        """Se hereda la función action_date_assign para llamar funcion de validacion"""
        self.validar_factura_rectificativa(cr, uid, ids)
        return super(account_invoice, self).action_date_assign(cr, uid, ids, args)
    
    
    def load_lines(self, cr, uid, ids, parent_id, context=None):
        """
        Cuando se va crear una factura rectificativa se usa esta funcion que se
        encarga de extraer todas las lineas de la factura padre para agregar la
        accion especifica y partida presupuestaria de cada linea y de esta manera
        evitar errores al agregar los datos.
        """
        if context is None:
            context = {}
        res = False
        value = {}
        if context.get('type',False) in ['out_refund']: 
            return {'value':{}}
        else:
            if not ids:
                return {'value': {'parent_id': None}, 'warning': {'message': 'Debe guardar el registro antes de asignar la factura padre'}}
            if not parent_id:
                return True
            #Borro las lineas que se hayan cargado anteriormente para cargarlas nuevamente.
            unlink_ids = self.pool.get('account.invoice.line').search(cr, uid,[('invoice_id', 'in', ids)])
            unlink_res = self.pool.get('account.invoice.line').unlink(cr, uid, unlink_ids)
            
            inv_brw = self.browse(cr, uid, parent_id)
            for lines in inv_brw.invoice_line:
                tax_ids = []
                for tax_line in lines.invoice_line_tax_id:
                    tax_ids.append(tax_line.id)
                    
                line_dict = {
                    'product_id': lines.product_id and lines.product_id.id or None,
                    'uos_id': lines.uos_id and lines.uos_id.id or None,
                    'concept_id': lines.concept_id and lines.concept_id.id or None,
                    'quantity': lines.quantity,
                    'price_unit': lines.price_unit,
                    'discount': lines.discount,
                    'name': lines.name,
                    'accion_especifica_id': lines.accion_especifica_id.id or None,
                    'partida_presupuesto': lines.partida_presupuesto.id or None,
                    'account_id': lines.account_id and lines.account_id.id or None,
                    'account_analityc_id': lines.account_analytic_id and lines.account_analytic_id.id or None,
                    'invoice_line_tax_id': [(6,0,tax_ids)],
                    'invoice_id': ids[0],
                }
                value.update({'invoice_line': [(0, 0, line_dict)]})
                res = self.pool.get('account.invoice.line').create(cr, uid, line_dict)
            
        return {'value':value}
    
    
    def dist_payment(self, cr, uid, model, voucher_id, invoice_id, monto, residual, context=None):
        """
        Funcion que se encarga de calcular y guardar los montos restantes en cada 
        linea de factura.
        Se realizan las imputaciones para la orden de pago cuando tiene una factura
        asociada. Las imputaciones son por lineas, se realiza imputacion por linea e
        iva.
        Para establecer el monto que se pagara por linea se computa el porcentaje
        
        Parametros:
        @model, modelo del cual proviene la imputacion.
        @voucher_id, es un entero que representa el id de la linea de la orde de pago
        (account.voucher.line).
        @invoice_id, es un entero que representa el id de la linea de la factura
        (account.invoice.line).
        @monto, es el monto que se pagara de la factura.
        @residual, es el monto restante por pagar.
        """
        ail_obj = self.pool.get('account.invoice.line')
        imp_obj = self.pool.get('presupuesto.imputacion.linea')
        period_obj = self.pool.get('periodo.presupuestario')
        fecha = time.strftime('%Y-%m-%d')
        
        ai_brw = self.browse(cr,uid,[invoice_id])
        for ai_data in ai_brw:

            if residual > 0.0001:
                dist_percent = ((monto * 100) / residual)

                for ail in ai_data.invoice_line:
                    if ai_data.origin:
                        ref = "Pago factura %s al Prod: %s, Origen: %s"%(ai_data.number, ail.name, ai_data.origin)
                        ref_imp_com = "Pago Impuesto Compra %s del Prod: %s, Origen: %s"%(ai_data.number, ail.name, ai_data.origin)
                        ref_imp_ven = "Pago Impuesto Venta %s del Prod: %s, Origen: %s"%(ai_data.number, ail.name, ai_data.origin)
                    else:
                        ref = "Pago factura %s al Prod: %s"%(ai_data.number, ail.name)
                        ref_imp_com = "Pago Impuesto Compra %s del Prod: %s"%(ai_data.number, ail.name)
                        ref_imp_ven = "Pago Impuesto Venta %s del Prod: %s"%(ai_data.number, ail.name),
                    
                    res_ail = 0.0
                    
                    monto_restante = ail.restante_monto
                        
                    if monto_restante > 0:
                        payment = (monto_restante * (dist_percent / 100))

                        res_ail = monto_restante - payment
                        imp_dict = {}
                        if ai_brw[0].type == 'in_invoice':

                            partida = ail.partida_presupuesto and ail.partida_presupuesto.code or None
                            partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''
                            
                            if partida is None:
                                raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
                            else:
                               partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None
                               name = ail.partida_presupuesto and ail.partida_presupuesto.name or ' '
                               partida_aux += ' '+name
                               
                            imp_dict = {
                                'ref': ref,
                                'partida': partida,
                                'partida_id': partida_id,
                                'enlace_id': ail.enlace_id,
                                'model': model,
                                'res_id': voucher_id,
                                'fecha_imp': fecha,
                                'amount': payment,
                                'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                                'accion_especifica_id': ail.accion_especifica_id.id,
                                'status': 'pagado',
                                'state': 'activo',
                                'type': 'egreso',
                                'type_create':'automatica',
                            }
                            imp_id = imp_obj.create(cr,uid,imp_dict,context)
                            #Agrego la imputacion de pago realizada al campo many2many de account.invoice.line
                            ail_obj.write(cr, uid, ail.id, {'monto_pago_ids': [(4, imp_id)]})
                        elif ai_brw[0].type == 'out_invoice':
                            partida = ail.partida_presupuesto and ail.partida_presupuesto.id or None
                            partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''
                            
                            if partida is None:
                                raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
                            else:
                                partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None 
                                name = ail.partida_presupuesto and ail.partida_presupuesto.name or ' '
                                partida_aux += ' '+name
                                
                            imp_dict = {
                                'ref': ref,
                                'partida': partida,
                                'partida_id': partida_id,
                                'enlace_id': ail.enlace_id,
                                'model': model,
                                'res_id': voucher_id,
                                'fecha_imp': fecha,
                                'amount': payment,
                                'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                                'accion_especifica_id': ail.accion_especifica_id.id,
                                'status_ingreso': 'recaudado',
                                'state': 'activo',
                                'type': 'ingreso',
                                'type_create':'automatica',
                            }
                            imp_id = imp_obj.create(cr,uid,imp_dict,context)
                            ail_obj.write(cr, uid, ail.id, {'monto_pago_ids': [(4, imp_id)]})
                    res_tax = 0.0
                    restante_iva = ail.restante_iva
                    
                    if restante_iva > 0.0001:
                        imp_dict = {}
                        payment = (restante_iva * (dist_percent / 100))

                        res_tax = restante_iva - payment
                        partida = ail.invoice_line_tax_id[0].partida_impuesto.code or None
                        partida_aux = ail.invoice_line_tax_id[0].partida_impuesto.code or None
                        
                        if partida is None:
                            raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.invoice_line_tax_id[0].name))
                        else:
                            partida_id = ail.invoice_line_tax_id[0].partida_impuesto.id or None
                            name = ail.invoice_line_tax_id[0].partida_impuesto and ail.invoice_line_tax_id[0].partida_impuesto.name or ''
                            partida_aux += ' '+name
                            
                        if ai_brw[0].type == 'in_invoice':
                            imp_dict = {
                                'ref': ref_imp_com,
                                'partida': partida,
                                'partida_id': partida_id,
                                'enlace_id': ail.enlace_id,
                                'model': model,
                                'res_id': voucher_id,
                                'fecha_imp': fecha,
                                'amount': payment,
                                'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                                'accion_especifica_id': ail.accion_especifica_id.id,
                                'status': 'pagado',
                                'state': 'activo',
                                'type': 'egreso',
                                'type_create':'automatica',
                            }
                            imp_id = imp_obj.create(cr,uid,imp_dict,context)
                            #Agrego la imputacion de pago del iva realizado al campo many2many de account.invoice.line
                            ail_obj.write(cr, uid, ail.id, {'iva_pago_ids': [(4, imp_id)]})
                        elif ai_brw[0].type == 'out_invoice':
                            imp_dict = {
                                'ref': ref_imp_ven,
                                'partida': partida,
                                'partida_id': partida_id,
                                'enlace_id': ail.enlace_id,
                                'model': model,
                                'res_id': voucher_id,
                                'fecha_imp': fecha,
                                'amount': payment,
                                'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                                'accion_especifica_id': ail.accion_especifica_id.id,
                                'status_ingreso': 'recaudado',
                                'state': 'activo',
                                'type': 'ingreso',
                                'type_create': 'automatica',
                            }
                            imp_id = imp_obj.create(cr,uid,imp_dict,context)
                            ail_obj.write(cr, uid, ail.id, {'iva_pago_ids': [(4, imp_id)]})

                    ail_obj.write(cr, uid, ail.id, {'restante_monto': res_ail,'restante_iva': res_tax})
        return True
    
    
    def verificar_esdadiva(self, cr, uid, ids, nombre_campo, modelo, context=None):
        """
        Verifica si esta instalado el modulo de dadiva para saber si se ejecuta
        el proceso de dadiva.
        
        @nombre_campo, nombre del campo a consultar.
        @modelo, nombre del modelo a consultar.
        """
        fields_obj = self.pool.get('ir.model.fields')
        field = fields_obj.search(cr, uid, [('name', '=', nombre_campo), ('model', '=', modelo),])
        if field:
            return True
        else:
            return False
    
    
    def action_number(self, cr, uid, ids, context=None):
        """
        Herencia de action_number para realizar la imputacion de la factura.
        """
        number = super(account_invoice,self).action_number(cr,uid,ids,context=context)
        self.action_number_imputar(cr, uid, ids, context=context)
        return number


    def action_number_imputar(self, cr, uid, ids, context=None):
        """
        Funcion encargada de crear imputaciones por cada linea de la factura, junto
        a su impuesto, si tiene asociado alguno.
        """
        fecha = time.strftime('%Y-%m-%d')
        ai_brw = self.browse(cr, uid, ids, context=context)
        ail_obj = self.pool.get('account.invoice.line')
        period_obj = self.pool.get('periodo.presupuestario')
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        esdadiva = False
        
        if ai_brw:
            for ail in ai_brw[0].invoice_line:
                """
                Si la linea no tiene enlace_id quiere decir que la linea no ha
                sido imputada, y se debe crear una imputacion automatica, del
                comprometido o devengado dependiendo el tipo de factura.
                """
                if not ail.enlace_id:
                    if ai_brw[0].type == 'in_invoice':
                        enlace_id = self.bot_imputacion(cr, uid, ai_brw, ail, context=context)
                        ail_obj.write(cr, uid, ail.id, {'enlace_id': enlace_id})
                    if ai_brw[0].type == 'out_invoice':
                        enlace_id = self.bot_imputacion_ingreso(cr, uid, ai_brw, ail, context=context)
                        ail_obj.write(cr, uid, ail.id, {'enlace_id': enlace_id})
                    if ai_brw[0].type in ('in_refund', 'out_refund'):
                        enlace_id = None
                else:
                    enlace_id = ail.enlace_id
                
                #Armar la referencia para colocar en la imputacion
                if ai_brw[0].parent_id:
                    if ai_brw[0].type in ('in_invoice', 'out_invoice'):
                        referencia = 'NC/ND: %s de Factura: %s, Prod: %s'%(ai_brw[0].number, ai_brw[0].parent_id.number, ail.name)
                        ref_impuesto = "Impuesto al Prod: %s, NC/ND: %s de Factura: %s"%(ail.name, ai_brw[0].number, ai_brw[0].parent_id.number)
                    if ai_brw[0].type in ('in_refund', 'out_refund'):
                        referencia = 'Reverso NC/ND: %s, Factura Padre: %s, Prod: %s'%(ai_brw[0].number, ai_brw[0].parent_id.number, ail.name)
                        ref_impuesto = "Impuesto al Prod: %s, de Reverso NC/ND: %s, Factura Padre: %s"%(ail.name, ai_brw[0].number, ai_brw[0].parent_id.number)
                else:
                    if ai_brw[0].type in ('in_invoice', 'out_invoice'):
                        referencia = 'Factura %s, Prod: %s'%(ai_brw[0].number, ail.name)
                        ref_impuesto = "Impuesto al Prod: %s de Factura %s "%(ail.name, ai_brw[0].number)
                
                if ai_brw[0].origin:
                    origen = ', Origen: %s'%(ai_brw[0].origin)
                    referencia = referencia + origen
                    ref_impuesto = ref_impuesto + origen
                
                if ai_brw[0].type == 'in_invoice':
                    partida = ail.partida_presupuesto and ail.partida_presupuesto.code or None
                    partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''

                    if partida is None:
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
                    else:
                        partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None 
                        name = ail.partida_presupuesto and ail.partida_presupuesto.name or ''
                        partida_aux += ' '+name
                    
                    imp_dict = {
                        'ref': referencia,
                        'partida': partida,
                        'enlace_id': enlace_id,
                        'partida_id': partida_id,
                        'model': 'account.invoice.line',
                        'res_id': ail.id,
                        'fecha_imp': fecha,
                        'amount': ail.price_subtotal,
                        'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                        'accion_especifica_id': ail.accion_especifica_id.id,
                        'status': 'causado_gasto',
                        'state': 'activo',
                        'type': 'egreso',
                        'type_create':'automatica',
                    }
                    rest_line = ail.price_subtotal
                    ail_obj.write(cr,uid,ail.id,{'restante_monto': rest_line})
                    #Creo la imputacion de la factura
                    imp_id = imp_obj.create(cr,uid,imp_dict,context)
                    
                elif ai_brw[0].type == 'out_invoice':
                    partida = ail.partida_presupuesto and ail.partida_presupuesto.code or None
                    partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''
                    
                    if partida is None:
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
                    else:
                        partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None 
                        name = ail.partida_presupuesto and ail.partida_presupuesto.name or ''
                        partida_aux += ' '+name
                    
                    imp_dict = {
                        'ref': referencia,
                        'partida': partida,
                        'partida_id': partida_id,
                        'enlace_id': enlace_id,
                        'model': 'account.invoice.line',
                        'res_id': ail.id,
                        'fecha_imp': fecha,
                        'amount': ail.price_subtotal,
                        'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                        'accion_especifica_id': ail.accion_especifica_id.id,
                        'state': 'activo',
                        'type_create':'automatica',
                    }
                    
                    """Verificamos si la factura es una dadiva si es asi
                    entonces la imputacion se realiza como un gasto"""
                    dadiva_inv = False
                    esdadiva = False
                    esdadiva = self.verificar_esdadiva(cr, uid, ids, 'esdadiva', 'sale.order',context=context)
                    if esdadiva:
                        dadiva_inv = self.pool.get("sale.order").search(cr, uid, [('name','=',ai_brw[0].origin),('esdadiva','=',True)])
                    if  dadiva_inv:
                        imp_dict.update({'status': 'causado_gasto', 'type': 'egreso',})
                        #Inicializo el Restante de la linea
                        rest_line = ail.price_subtotal
                        ail_obj.write(cr,uid,ail.id,{'restante_monto': rest_line})
                        #Creo la imputacion de la factura
                        imp_id = imp_obj.create(cr,uid,imp_dict,context)
                        #Actualizo el diccionario e imputo pagado
                        imp_dict.update({'status':'pagado'})
                        imp_id = imp_obj.create(cr,uid,imp_dict,context)
                    else:
                        imp_dict.update({'status_ingreso': 'liquidado','type': 'ingreso',})
                        #Inicializo el Restante de la linea
                        rest_line = ail.price_subtotal
                        ail_obj.write(cr,uid,ail.id,{'restante_monto': rest_line})
                        #Creo la imputacion de la factura
                        imp_id = imp_obj.create(cr,uid,imp_dict,context)
                
                elif ai_brw[0].type == 'in_refund':
                    partida = ail.partida_presupuesto and ail.partida_presupuesto.code or None
                    partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''
                    
                    if partida is None:
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
                    else:
                        partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None 
                        partida += ' '+ail.partida_presupuesto.name
                    
                    imp_dict = {
                        'ref': referencia,
                        'partida': partida,
                        'enlace_id': enlace_id,
                        'partida_id': partida_id,
                        'model': 'account.invoice.line',
                        'res_id': ail.id,
                        'fecha_imp': fecha,
                        'amount': (ail.price_subtotal*(-1)),
                        'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                        'accion_especifica_id': ail.accion_especifica_id.id,
                        'status': 'comprometido',
                        'state': 'activo',
                        'type': 'egreso',
                        'type_create':'automatica',
                    }
                    rest_line = ail.price_subtotal
                    ail_obj.write(cr, uid, ail.id, {'restante_monto': rest_line}, context)
                    #Creo la imputacion de la factura
                    imp_id = imp_obj.create(cr, uid, imp_dict, context)
                    if not enlace_id:
                        enlace_id = imp_id
                        imp_dict.update({'enlace_id': enlace_id})
                        ail_obj.write(cr, uid, ail.id, {'enlace_id': enlace_id}, context)
                        imp_obj.write(cr, uid, imp_id, {'enlace_id': enlace_id}, context)
                    #Crea la Imputacion de Causado en Negativo.
                    imp_dict.update({'status': 'causado_gasto'})
                    imp_id = imp_obj.create(cr, uid, imp_dict, context)

                elif ai_brw[0].type == 'out_refund':
                    partida = ail.partida_presupuesto and ail.partida_presupuesto.code or None
                    partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''
                    
                    if partida is None:
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
                    else:
                        partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None 
                        partida += ' '+ail.partida_presupuesto.name
                    
                    imp_dict = {
                        'ref': referencia,
                        'partida': partida,
                        'enlace_id': enlace_id,
                        'partida_id': partida_id,
                        'model': 'account.invoice.line',
                        'res_id': ail.id,
                        'fecha_imp': fecha,
                        'amount': (ail.price_subtotal*(-1)),
                        'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                        'accion_especifica_id': ail.accion_especifica_id.id,
                        'status_ingreso':'devengado',
                        'state': 'activo',
                        'type': 'ingreso',
                        'type_create':'automatica',
                    }
                    rest_line = ail.price_subtotal
                    ail_obj.write(cr, uid, ail.id, {'restante_monto': rest_line}, context)
                    #Creo la imputacion de la factura
                    imp_id = imp_obj.create(cr, uid, imp_dict, context)
                    if not enlace_id:
                        enlace_id = imp_id
                        imp_dict.update({'enlace_id': enlace_id})
                        ail_obj.write(cr, uid, ail.id, {'enlace_id': enlace_id}, context)
                        imp_obj.write(cr, uid, imp_id, {'enlace_id': enlace_id}, context)
                    #Crea la Imputacion de Liquidada en Negativo.
                    imp_dict.update({'status_ingreso': 'liquidado'})
                    imp_id = imp_obj.create(cr, uid, imp_dict, context)
                    
                #Imputacion de los impuestos.
                if ai_brw[0].amount_tax > 0.0001:
                    tax_amount = 0.00
                    partida = None
                    for tax_line in ail.invoice_line_tax_id:
                        tax_amount += ail.price_subtotal * tax_line.amount
                                                
                        partida = tax_line.partida_impuesto and tax_line.partida_impuesto.code or None
                        partida_aux = tax_line.partida_impuesto and tax_line.partida_impuesto.code or ''
                       
                        if partida is None:
                            raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el impuesto %s.'%tax_line.name))
                        else:
                            partida_id = tax_line.partida_impuesto and tax_line.partida_impuesto.id or None
                            name = tax_line.partida_impuesto and tax_line.partida_impuesto.name or  ''
                            partida_aux += ' '+name
                    if tax_amount > 0.0001:
                        #Valida si es una factura rectificativa o una nota de debito.
                        if ai_brw[0].type in ('in_refund','out_refund'):
                            tax_amount = (tax_amount * -1)
                        
                        impuesto_dict = {
                            'ref': ref_impuesto,
                            'partida': partida,
                            'partida_id': partida_id,
                            'enlace_id': enlace_id,
                            'model': 'account.invoice.line',
                            'res_id': ail.id,
                            'fecha_imp': fecha,
                            'amount': tax_amount,
                            'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                            'accion_especifica_id':ail.accion_especifica_id.id,
                            'status': 'causado_gasto',
                            'state': 'activo',
                            'type': 'egreso',
                            'type_create':'automatica',
                        }
                        
                        if ai_brw[0].type in ('out_invoice', 'out_refund'):
                            impuesto_dict.update({'type': 'ingreso', 'status_ingreso': 'liquidado', 'status': ''})
                        
                        #Inicializo el Monto restante del IVA
                        ail_obj.write(cr,uid,ail.id,{'restante_iva':tax_amount})
                        #Creo la Imputacion de la linea
                        imp_id = imp_obj.create(cr,uid,impuesto_dict,context)
                        #Crea la Imputacion de comprometido cuando la factura sea out_refund
                        if ai_brw[0].type in ('in_refund', 'out_refund'):
                            if ai_brw[0].type == 'in_refund':
                                impuesto_dict.update({'status': 'comprometido'})
                            elif ai_brw[0].type == 'out_refund':
                                impuesto_dict.update({'status_ingreso': 'devengado', 'status': ''})
                            imp_id = imp_obj.create(cr, uid, impuesto_dict, context)
        else:
            raise osv.except_osv(('Accion invalida !'), ("No se puede realizar la imputacion presupuestaria debido a que no hay lineas de factura"))
        return True
        
        
    def bot_imputacion(self, cr, uid, ai_brw, ail, context=None):
        """
        Funcion que se encarga de crear las imputaciones de gasto del estado
        comprometido cuando la factura se crea de forma manual.
        @ai_brw, browse de account.invoice con la lista de facturas a imputar,
        solo se usa la primera y para obtener el tipo.
        @ail, browse de account.invoice.line de la linea que se va a imputar.
        @return enlace_id, que es el id de la imputacion creada (comprometido).
        """
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        period_obj = self.pool.get('periodo.presupuestario')
        fecha = time.strftime('%Y-%m-%d')
        
        #Validacion por si no se activa el requerido para accion_especifica_id de la vista
        acc_esp_id = ail.accion_especifica_id and ail.accion_especifica_id.id or None
        if acc_esp_id is None:
            raise osv.except_osv(('Error !'), ('No hay accion especifica definida para linea %s.'%ail.name))
            
        if ai_brw[0].type == 'in_invoice':
            partida = ail.partida_presupuesto and ail.partida_presupuesto.code or None
            partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''
            if partida is None:
                raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
            else:
                partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None 
                name = ail.partida_presupuesto and ail.partida_presupuesto.name or ''
                partida_aux += ' '+name
            
            #Validacion para mostrar la referencia correcta en las Notas de Debito
            if ai_brw[0].parent_id:
                referencia = 'Reverso ND: %s de Factura: %s Prod: %s'%(ai_brw[0].number,ai_brw[0].parent_id.number, ail.name)
            else:
                referencia = 'Factura %s Prod: %s'%(ai_brw[0].number, ail.name)
    
            imp_dict = {
                'ref': referencia,
                'partida': partida,
                'partida_id': partida_id,
                'model': 'account.invoice.line',
                'res_id': ail.id,
                'fecha_imp': fecha,
                'amount': ail.price_subtotal,
                'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                'accion_especifica_id': acc_esp_id,
                'status': 'comprometido',
                'state': 'activo',
                'type': 'egreso',
                'type_create':'automatica',
            }
            enlace_id = imp_obj.create(cr,uid,imp_dict,context)
            imp_obj.write(cr,uid,enlace_id,{'enlace_id':enlace_id},context)
        #Imputacion de los impuestos.
        if ai_brw[0].amount_tax > 0.0001 and ai_brw[0].type == 'in_invoice':
            tax_amount = 0.00
            partida = None
            for tax_line in ail.invoice_line_tax_id:
                tax_amount += ail.price_subtotal * tax_line.amount
                partida = tax_line.partida_impuesto and tax_line.partida_impuesto.code or None
                partida_aux = tax_line.partida_impuesto and tax_line.partida_impuesto.code or ''
               
                if partida is None:
                    raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el impuesto %s.'%tax_line.name))
                else:
                    partida_id = tax_line.partida_impuesto and tax_line.partida_impuesto.id or None
                    name = tax_line.partida_impuesto and tax_line.partida_impuesto.name or ''
                    partida_aux += ' '+name
            if tax_amount > 0.0001:
                impuesto_dict = {
                    'ref': "Impuesto de %s al Prod: %s"%(ai_brw[0].number,ail.name),
                    'partida': partida,
                    'partida_id': partida_id,
                    'enlace_id': enlace_id,
                    'model': 'account.invoice.line',
                    'res_id': ail.id,
                    'fecha_imp': fecha,
                    'amount': tax_amount,
                    'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                    'accion_especifica_id': acc_esp_id,
                    'status': 'comprometido',
                    'state': 'activo',
                    'type': 'egreso',
                    'type_create':'automatica',
                }
                #Creo la Imputacion de la linea
                imp_id = imp_obj.create(cr,uid,impuesto_dict,context)
                    
        return enlace_id
    
    
    def bot_imputacion_ingreso(self, cr, uid, ai_brw, ail, context=None):
        """
        Funcion que se encarga de crear las imputaciones de ingreso del estado
        devengado cuando la factura se crea de forma manual.
        @ai_brw, browse de account.invoice con la lista de facturas a imputar,
        solo se usa la primera y para obtener el tipo.
        @ail, browse de account.invoice.line de la linea que se va a imputar.
        @return enlace_id, que es el id de la imputacion creada (devengado).
        """
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        period_obj = self.pool.get('periodo.presupuestario')
        fecha = time.strftime('%Y-%m-%d')
        
        #Validacion por si no se activa el requerido para accion_especifica_id de la vista
        acc_esp_id = ail.accion_especifica_id and ail.accion_especifica_id.id or None
        if acc_esp_id is None:
            raise osv.except_osv(('Error !'), ('No hay accion especifica definida para linea %s.'%ail.name))
            
        if ai_brw[0].type == 'out_invoice':
            partida = ail.partida_presupuesto and ail.partida_presupuesto.code or None
            partida_aux = ail.partida_presupuesto and ail.partida_presupuesto.code or ''
            if partida is None:
                raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%ail.name))
            else:
                partida_id = ail.partida_presupuesto and ail.partida_presupuesto.id or None 
                name = ail.partida_presupuesto and ail.partida_presupuesto.name or ''
                partida_aux += ' '+name
            
            #Validacion para mostrar la referencia correcta en las Notas de Debito
            if ai_brw[0].parent_id:
                referencia = 'Reverso NC: %s de Factura: %s Prod: %s'%(ai_brw[0].number,ai_brw[0].parent_id.number, ail.name)
            else:
                referencia = 'Factura %s Prod: %s'%(ai_brw[0].number, ail.name)
    
            imp_dict = {
                'ref': referencia,
                'partida': partida,
                'partida_id': partida_id,
                'model': 'account.invoice.line',
                'res_id': ail.id,
                'fecha_imp': fecha,
                'amount': ail.price_subtotal,
                'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                'accion_especifica_id': acc_esp_id,
                'status_ingreso': 'devengado',
                'state': 'activo',
                'type': 'ingreso',
                'type_create':'automatica',
            }
            enlace_id = imp_obj.create(cr,uid,imp_dict,context)
            imp_obj.write(cr,uid,enlace_id,{'enlace_id':enlace_id},context)
        #Imputacion de los impuestos.
        if ai_brw[0].amount_tax > 0.0001 and ai_brw[0].type == 'out_invoice':
            tax_amount = 0.00
            partida = None
            for tax_line in ail.invoice_line_tax_id:
                tax_amount += ail.price_subtotal * tax_line.amount
                partida = tax_line.partida_impuesto and tax_line.partida_impuesto.code or None
                partida_aux = tax_line.partida_impuesto and tax_line.partida_impuesto.code or ''
               
                if partida is None:
                    raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el impuesto %s.'%tax_line.name))
                else:
                    partida_id = tax_line.partida_impuesto and tax_line.partida_impuesto.id or None
                    name = tax_line.partida_impuesto and tax_line.partida_impuesto.name or ''
                    partida_aux += ' '+name
            if tax_amount > 0.0001:
                impuesto_dict = {
                    'ref': "Impuesto de %s al Prod: %s"%(ai_brw[0].number,ail.name),
                    'partida': partida,
                    'partida_id': partida_id,
                    'enlace_id': enlace_id,
                    'model': 'account.invoice.line',
                    'res_id': ail.id,
                    'fecha_imp': fecha,
                    'amount': tax_amount,
                    'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                    'accion_especifica_id': acc_esp_id,
                    'status_ingreso': 'devengado',
                    'state': 'activo',
                    'type': 'ingreso',
                    'type_create':'automatica',
                }
                #Creo la Imputacion de la linea
                imp_id = imp_obj.create(cr,uid,impuesto_dict,context)
                    
        return enlace_id
        
        
    def _refund_cleanup_lines(self, cr, uid, lines):
        """
        Se hace herencia de la función _refund_cleanup_lines para modificar
        parametro de datos que se escribira al realizar una factura rectificativa
        por el wizard.
        """
        """
        Nota cuando se agregue la cabecera se debe realizar la misma validacion
        para el enlace_id, que sera al momento del cambio cabecera de las lineas
        de imputacion
        """
        res = super(account_invoice,self)._refund_cleanup_lines(cr, uid, lines)
        for line in res:
            if line[2].get('partida_presupuesto', False):
                line[2]['partida_presupuesto'] = line[2]['partida_presupuesto'][0]
            
            if line[2].get('accion_especifica_id', False):
                line[2]['accion_especifica_id'] = line[2]['accion_especifica_id'][0]
        
        return res

account_invoice()


class stock_picking(osv.osv):
    _inherit = "stock.picking"
    
    def action_invoice_create(self, cr, uid, ids, journal_id=False,
            group=False, type='out_invoice', context=None):
        """ Creates invoice based on the invoice state selected for picking.
        @param journal_id: Id of journal
        @param group: Whether to create a group invoice or not
        @param type: Type invoice to be created
        @return: Ids of created invoices for the pickings
        """
        if context is None:
            context = {}
        """Modifica la busqueda de las lineas para crear una factura"""
        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        address_obj = self.pool.get('res.partner.address')
        invoices_group = {}
        res = {}
        inv_type = type
        for picking in self.browse(cr, uid, ids, context=context):
            if picking.invoice_state != '2binvoiced':
                continue
            payment_term_id = False
            partner =  picking.address_id and picking.address_id.partner_id
            if not partner:
                raise osv.except_osv(_('Error, no partner !'),
                    _('Please put a partner on the picking list if you want to generate invoice.'))

            if not inv_type:
                inv_type = self._get_invoice_type(picking)

            if inv_type in ('out_invoice', 'out_refund'):
                account_id = partner.property_account_receivable.id
                payment_term_id = self._get_payment_term(cr, uid, picking)
            else:
                account_id = partner.property_account_payable.id

            address_contact_id, address_invoice_id = \
                    self._get_address_invoice(cr, uid, picking).values()
            address = address_obj.browse(cr, uid, address_contact_id, context=context)

            comment = self._get_comment_invoice(cr, uid, picking)
            if group and partner.id in invoices_group:
                invoice_id = invoices_group[partner.id]
                invoice = invoice_obj.browse(cr, uid, invoice_id)
                invoice_vals = {
                    'name': (invoice.name or '') + ', ' + (picking.name or ''),
                    'origin': (invoice.origin or '') + ', ' + (picking.name or '') + (picking.origin and (':' + picking.origin) or ''),
                    'comment': (comment and (invoice.comment and invoice.comment+"\n"+comment or comment)) or (invoice.comment and invoice.comment or ''),
                    'date_invoice':context.get('date_inv',False),
                    'user_id':uid
                }
                invoice_obj.write(cr, uid, [invoice_id], invoice_vals, context=context)
            else:
                invoice_vals = {
                    'name': picking.name,
                    'origin': (picking.name or '') + (picking.origin and (':' + picking.origin) or ''),
                    'type': inv_type,
                    'account_id': account_id,
                    'partner_id': address.partner_id.id,
                    'address_invoice_id': address_invoice_id,
                    'address_contact_id': address_contact_id,
                    'comment': comment,
                    'payment_term': payment_term_id,
                    'fiscal_position': partner.property_account_position.id,
                    'date_invoice': context.get('date_inv',False),
                    'company_id': picking.company_id.id,
                    'user_id':uid
                }
                cur_id = self.get_currency_id(cr, uid, picking)
                if cur_id:
                    invoice_vals['currency_id'] = cur_id
                if journal_id:
                    invoice_vals['journal_id'] = journal_id
                invoice_id = invoice_obj.create(cr, uid, invoice_vals,
                        context=context)
                invoices_group[partner.id] = invoice_id
            res[picking.id] = invoice_id
            for move_line in picking.move_lines:
                if move_line.state == 'cancel':
                    continue
                if move_line.scrapped:
                    # do no invoice scrapped products
                    continue
                origin = move_line.picking_id.name or ''
                if move_line.picking_id.origin:
                    origin += ':' + move_line.picking_id.origin
                if group:
                    name = (picking.name or '') + '-' + move_line.name
                else:
                    name = move_line.name

                if inv_type in ('out_invoice', 'out_refund'):
                    account_id = move_line.product_id.product_tmpl_id.\
                            property_account_income.id
                    if not account_id:
                        account_id = move_line.product_id.categ_id.\
                                property_account_income_categ.id
                    #Busqueda en las categorias padres de no existir la cuenta
                    if not account_id:
                        account_id = self.get_product_category_account(cr,uid,move_line.product_id.categ_id.id,"property_stock_account_output_categ",context=context)
                else:
                    account_id = move_line.product_id.product_tmpl_id.\
                            property_account_expense.id
                    if not account_id:
                        account_id = move_line.product_id.categ_id.\
                                property_stock_account_input_categ.id
                    #Busqueda en las categorias padres de no existir la cuenta
                    if not account_id:
                        account_id = self.get_product_category_account(cr,uid,move_line.product_id.categ_id.id,"property_stock_account_input_categ",context=context)

                price_unit = self._get_price_unit_invoice(cr, uid,
                        move_line, inv_type)
                discount = self._get_discount_invoice(cr, uid, move_line)
                tax_ids = self._get_taxes_invoice(cr, uid, move_line, inv_type)
                account_analytic_id = self._get_account_analytic_invoice(cr, uid, picking, move_line)

                #set UoS if it's a sale and the picking doesn't have one
                uos_id = move_line.product_uos and move_line.product_uos.id or False
                if not uos_id and inv_type in ('out_invoice', 'out_refund'):
                    uos_id = move_line.product_uom.id
                account_id = self.pool.get('account.fiscal.position').map_account(cr, uid, partner.property_account_position, account_id)
                if move_line.price_unit != 0 and price_unit != move_line.price_unit:
                    price_unit = move_line.price_unit
                
                """
                Se asigna la partida presupuestaria asociada al movimiento de albaran
                dependiendo si es factura de compra o venta se asocia un tipo de partida.
                Se asocia la accion especifica y enlace_id
                """
                partida = move_line.product_id.partida_egreso and move_line.product_id.partida_egreso.id or None
                if inv_type in ('out_invoice','out_refund'):
                    partida = move_line.product_id.partida_ingreso and move_line.product_id.partida_ingreso.id or None
                if partida is None:
                   raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el producto %s.'%move_line.product_id.name))
                accion_especifica = move_line.accion_especifica_id and move_line.accion_especifica_id.id or None
                invoice_line_id = invoice_line_obj.create(cr, uid, {
                    'name': name,
                    'origin': origin,
                    'invoice_id': invoice_id,
                    'uos_id': uos_id,
                    'product_id': move_line.product_id.id,
                    'account_id': account_id,
                    'price_unit': price_unit,
                    'discount': discount,
                    'quantity': move_line.product_uos_qty or move_line.product_qty,
                    'invoice_line_tax_id': [(6, 0, tax_ids)],
                    'account_analytic_id': account_analytic_id,
                    'note': move_line.note,
                    'enlace_id': move_line.enlace_id,
                    'accion_especifica_id': accion_especifica,
                    'partida_presupuesto': partida,
                }, context=context)
                self._invoice_line_hook(cr, uid, move_line, invoice_line_id)

            invoice_obj.button_compute(cr, uid, [invoice_id], context=context,
                    set_total=(inv_type in ('in_invoice', 'in_refund')))
            self.write(cr, uid, [picking.id], {
                'invoice_state': 'invoiced',
                }, context=context)
            self._invoice_hook(cr, uid, picking, invoice_id)
        self.write(cr, uid, res.keys(), {
            'invoice_state': 'invoiced',
            }, context=context)
        return res
    
stock_picking()


class account_wh_iva(osv.osv):
    _inherit = "account.wh.iva"
    
    def action_move_create(self, cr, uid, ids, context=None):
        """
        Se ejecuta la funcion que crea el asiento de la retencion de IVA, para
        llamar la funcion que crea la imputacion de retencion de IVA.
        """
        result = super(account_wh_iva,self).action_move_create(cr, uid, ids, context=context)
        self.action_move_create_imputar(cr, uid, ids, context=context)
        return result


    def action_move_create_imputar(self, cr, uid, ids, context=None):
        """
        Funcion que crea la imputacion por la retencion de IVA por cada linea
        de la factura se crea una imputacion. Como la retencion es un pago
        que se realizara o se realizo se hace la imputacion solo en el ultimo
        estado del ingreso o gasto.
        """
        fecha = time.strftime('%Y-%m-%d')
        period_obj = self.pool.get('periodo.presupuestario')
        ai_obj = self.pool.get('account.invoice')
        ail_obj = self.pool.get('account.invoice.line')
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        awhi_brw = self.browse(cr,uid,ids, context=context)
        
        if  awhi_brw:
            for wh_line in awhi_brw[0].wh_lines:
                #Aqui guardo el porcentaje de la retencion
                wh_number = awhi_brw[0].number
                invoice_id = wh_line.invoice_id.id

            ai_brw = ai_obj.browse(cr,uid,invoice_id)

            if ai_brw:
                for ail in ai_brw.invoice_line:
                #Aqui Busco la accion especifica en las lineas de la factura
                    monto = ail.price_subtotal
                    porce = ai_brw.wh_iva_rate
                    accion = ail.accion_especifica_id.id
                    for tax in ail.invoice_line_tax_id:
                        imp = tax.amount
                        partida = tax.partida_impuesto and tax.partida_impuesto.code or None
                        partida_aux = tax.partida_impuesto and tax.partida_impuesto.code or ''
                        
                        if partida is None:
                           raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el impuesto %s.'%tax.name))
                        else:
                            partida_id = tax.partida_impuesto and tax.partida_impuesto.id or None
                            name = tax.partida_impuesto and tax.partida_impuesto.name or ''
                            partida_aux += ' '+name
                    #Calculo de la retencion de IVA
                    amount = ((monto * imp) * (porce/100))
                    #Creo el dccionario para realizar la imputacion
                    if amount > 0.0001:
                        refer = 'Retencion de IVA: %s, Fact: %s del Prod: %s'%(awhi_brw[0].number,ai_brw.number,ail.name)
                        if ai_brw.type in ('in_refund','out_refund'):
                            amount = (amount * -1)
                            refer = 'Reverso ' + refer
                        imp_dict = {
                            'ref': refer,
                            'partida': partida,
                            'partida_id': partida_id,
                            'enlace_id': ail.enlace_id,
                            'model': 'account.invoice.line',
                            'res_id': ail.id,
                            'fecha_imp': fecha,
                            'amount': amount,
                            'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                            'accion_especifica_id': accion,
                            'status': 'pagado',
                            'state': 'activo',
                            'type': 'egreso',
                            'type_create': 'automatica',
                        }
                        if ai_brw.type in ('out_invoice', 'out_refund'):
                            imp_dict.update({'status_ingreso': 'recaudado', 'status': '', 'type': 'ingreso'})
                        
                        #Creo la imputacion
                        imp_id = imp_obj.create(cr,uid,imp_dict,context)
                        #Agrego la imputacion de pago de iva realizado al campo many2many de account.invoice.line
                        ail_obj.write(cr, uid, ail.id, {'iva_pago_ids': [(4, imp_id)]})
                        #Calculo el restante para el Impuesto
                        rest = (ail.restante_iva - amount)
                        ail_obj.write(cr, uid, ail.id, {'restante_iva': rest})
            else:
                raise osv.except_osv(('Accion invalida !'),("No se puede realizar la imputacion presupuestaria debido a que no hay una factura asociada a la retencion de IVA"))
        else:
            raise osv.except_osv(('Accion invalida !'),("No se puede realizar la imputacion presupuestaria debido a que no hay un documento de retencion de IVA asociada"))
        return True
        
account_wh_iva()

class islr_wh_doc(osv.osv):
    _inherit = "islr.wh.doc"
    
    def action_move_create(self, cr, uid, ids, *arg):
        """
        Se ejecuta la funcion que crea el asiento de la retencion de ISLR, para
        llamar la funcion que crea la imputacion de retencion de ISLR.
        """
        result = super(islr_wh_doc,self).action_move_create(cr, uid, ids, *arg)
        self.action_move_create_imputar(cr, uid, ids, *arg)
        return result
    
    
    def action_move_create_imputar(self, cr, uid, ids, *arg):
        """
        Funcion que crea la imputacion de la retencion de ISLR por cada linea
        de la factura se crea una imputacion. Como la retencion es un pago 
        que se realizara o se realizo se hace la imputacion solo en el ultimo 
        estado del ingreso o gasto.
        """
        fecha = time.strftime('%Y-%m-%d')
        period_obj = self.pool.get('periodo.presupuestario')
        ail_obj = self.pool.get('account.invoice.line')
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        
        awhi_brw = self.browse(cr,uid,ids, context=None)
        if awhi_brw:
            for wh_line in awhi_brw[0].concept_ids:
                #Aqui guardo el porcentaje de la retencion
                wh_number = awhi_brw[0].number
                invoice_id = wh_line.invoice_id.id
                for xml_lines in wh_line.xml_ids:
                    
                    amount = xml_lines.wh
                    partida = xml_lines.account_invoice_line_id and xml_lines.account_invoice_line_id.partida_presupuesto.code or None
                    partida_aux = xml_lines.account_invoice_line_id and xml_lines.account_invoice_line_id.partida_presupuesto.code or ''
                    
                    if partida is None:
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el concepto %s.'%xml_lines.concept_id.name))
                    else:
                        partida_id = xml_lines.account_invoice_line_id and xml_lines.account_invoice_line_id.partida_presupuesto.id or None
                        name = xml_lines.account_invoice_line_id.partida_presupuesto and xml_lines.account_invoice_line_id.partida_presupuesto.name or ''
                        partida_aux += ' '+name
                        
                    accion = xml_lines.account_invoice_line_id.accion_especifica_id and xml_lines.account_invoice_line_id.accion_especifica_id.id or None
                    price_subtotal = xml_lines.account_invoice_line_id.price_subtotal
                    ail_id =  xml_lines.account_invoice_line_id.id
                    #Creo el diccionario para realizar la imputacion
                    imp_dict = {
                            'ref': 'Retencion de ISLR: %s, Fact: %s del Prod: %s'%(awhi_brw[0].number, xml_lines.account_invoice_id.number, xml_lines.account_invoice_line_id.name),
                            'partida': partida,
                            'partida_id': partida_id,
                            'enlace_id': xml_lines.account_invoice_line_id.enlace_id,
                            'model': 'islr.xml.wh.line',
                            'res_id': xml_lines.id,
                            'fecha_imp': fecha,
                            'amount': amount,
                            'periodo': period_obj.find(cr, uid, fecha, context=None)[0],
                            'accion_especifica_id': accion,
                            'status': 'pagado',
                            'state': 'activo',
                            'type': 'egreso',
                            'type_create': 'automatica',
                        }
                    #Creo la imputacion
                    imp_id = imp_obj.create(cr,uid,imp_dict,context=None)
                    #Agrego la imputacion de pago realizada al campo many2many de account.invoice.line
                    ail_obj.write(cr, uid, ail_id, {'monto_pago_ids': [(4, imp_id)]})
                    #Aqui le resto la retencion a la linea
                    rest_line = (price_subtotal - amount)
                    ail_obj.write(cr, uid, ail_id, {'restante_monto':rest_line})
                    
        else:
            raise osv.except_osv(('Accion invalida !'),("No se puede realizar la imputacion presupuestaria debido a que no hay un documento de retencion de ISLR asociada"))
        return True
        
islr_wh_doc()
