#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################
from osv import osv
from osv import fields
from tools.translate import _
from tools import config
import calendar
import time
import datetime
import decimal_precision as dp
import string

class presupuesto_ejecutado(osv.osv):
    _name = "presupuesto.ejecutado"
    _description = "Presupuesto Ejecutado"
    _order = 'period_id'
    
    
    def _get_amount_total(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')
        query = """
            SELECT PE.id AS id, COALESCE(SUM(PF.amount),0) AS amount_total
            FROM presupuesto_ejecutado PE, presupuesto_formulacion PF
            WHERE PE.period_id = PF.period_id AND 
            PE.accion_especifica_id = PF.accion_especifica_id AND
            PE.id in %s
            GROUP BY PE.id
        """%(str_ids)
        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
        
        for id in ids:
            res[id] = 0.00
            
        for i in array:
            res[i['id']] = i['amount_total']
        return res
        
        
    def _get_amount_comprometido(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')
        query = """
            SELECT PE.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_comprometido
            FROM presupuesto_ejecutado PE, presupuesto_imputacion_linea PIL
            WHERE PE.period_id = PIL.periodo AND 
            PE.accion_especifica_id = PIL.accion_especifica_id AND
            PIL.status = 'comprometido' AND
            PE.id in %s
            GROUP BY PE.id
        """%(str_ids)
        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
        for id in ids:
            res[id] = 0.00
            
        for i in array:
            res[i['id']] = i['amount_comprometido']
            
        return res
    
    
    def _get_amount_causado(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')
        query = """
            SELECT PE.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_causado
            FROM presupuesto_ejecutado PE, presupuesto_imputacion_linea PIL
            WHERE PE.period_id = PIL.periodo AND 
            PE.accion_especifica_id = PIL.accion_especifica_id AND
            PIL.status = 'causado_gasto' AND
            PE.id in %s
            GROUP BY PE.id
        """%(str_ids)
        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
        for id in ids:
            res[id] = 0.00
            
        for i in array:
            res[i['id']] = i['amount_causado']
            
        return res
    
    
    def _get_amount_pagado(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')
        query = """
            SELECT PE.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_pagado
            FROM presupuesto_ejecutado PE, presupuesto_imputacion_linea PIL
            WHERE PE.period_id = PIL.periodo AND 
            PE.accion_especifica_id = PIL.accion_especifica_id AND
            PIL.status = 'pagado' AND
            PE.id in %s
            GROUP BY PE.id
        """%(str_ids)
        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
        for id in ids:
            res[id] = 0.00
            
        for i in array:
            res[i['id']] = i['amount_pagado']
            
        return res
        
        
    def _get_amount_residual(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        for id in id_brw:
            res[id.id] = id.amount_total - id.amount_comprometido
        return res
        
        
    #~ Aqui comienzan las funciones de los montos acumulados
    def _get_amount_comprometido_acumulado(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        acum = 0
        for id in id_brw:
            acum = id.amount_comprometido + acum
            res[id.id] = acum
        return res


    def _get_amount_total_acumulado(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        acum = 0
        for id in id_brw:
            acum = id.amount_total + acum
            res[id.id] = acum
        return res
    
    
    def _get_amount_residual_acumulado(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        acum = 0
        for id in id_brw:
            acum = (id.amount_total_acum - id.amount_comprometido_acum)
            res[id.id] = acum
        return res
    
    
    _columns = {
        'name': fields.char('Mes',size=64),
        'amount_total': fields.function(_get_amount_total, digits_compute=dp.get_precision('Account'), method=True, string='Monto Presupuestado'),
        'amount_total_acum': fields.function(_get_amount_total_acumulado, digits_compute=dp.get_precision('Account'), method=True, string='Acumulado Presupuestado'),
        'amount_comprometido': fields.function(_get_amount_comprometido, digits_compute=dp.get_precision('Account'), method=True, string='Monto Comprometido'),
        'amount_comprometido_acum': fields.function(_get_amount_comprometido_acumulado, digits_compute=dp.get_precision('Account'), method=True, string='Acumulado Comprometido'),
        'amount_causado': fields.function(_get_amount_causado, digits_compute=dp.get_precision('Account'), method=True, string='Monto Causado'),
        'amount_pagado': fields.function(_get_amount_pagado, digits_compute=dp.get_precision('Account'), method=True, string='Monto Pagado'),
        'amount_residual': fields.function(_get_amount_residual, digits_compute=dp.get_precision('Account'), method=True, string='Disponibilidad'),
        'amount_residual_acum': fields.function(_get_amount_residual_acumulado, digits_compute=dp.get_precision('Account'), method=True, string='Acumulado Disponibilidad'),
        'period_id': fields.many2one('periodo.presupuestario', 'Periodo Presupuestario', required=True),
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica', required=True),
    }
    
presupuesto_ejecutado()


class presupuesto_ejecutado_central(osv.osv):
    _name = "presupuesto.ejecutado.central"
    _description = "Presupuesto Ejecutado Global"
    _order = 'period_id'
    
    
    def _get_amount_total_central(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')

        res = {}
        for pec in self.browse(cr, uid, ids, context=context):
            res[pec.id] = 0.00
            str_ae_ids = str([x.id for x in pec.accion_central_id.child_ids]).replace('[','(').replace(']',')')
            query = """
                SELECT COALESCE(SUM(PF.amount),0) AS amount_total
                FROM presupuesto_ejecutado_central PEC, presupuesto_formulacion PF
                WHERE PEC.period_id = PF.period_id
                AND PEC.period_id = %s
                AND PEC.accion_central_id = %s
                AND PF.accion_especifica_id in %s
                AND PEC.id = %s
            """%(pec.period_id.id, pec.accion_central_id.id, str_ae_ids, pec.id)

            cr.execute(query)
            array = cr.dictfetchone()
            res[pec.id] = array.get('amount_total',0)

        return res
        
        
    def _get_amount_compometido_central(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')

        res = {}
        for pec in self.browse(cr, uid, ids, context=context):
            res[pec.id] = 0.00
            str_ae_ids = str([x.id for x in pec.accion_central_id.child_ids]).replace('[','(').replace(']',')')
            query = """
                SELECT COALESCE(SUM(PIL.amount),0) AS amount_execute
                FROM presupuesto_ejecutado_central PEC, presupuesto_imputacion_linea PIL
                WHERE PEC.period_id = PIL.periodo
                AND PEC.period_id = %s
                AND PEC.accion_central_id = %s
                AND PIL.accion_especifica_id in %s
                AND PIL.status = 'comprometido'
                AND PEC.id = %s
            """%(pec.period_id.id, pec.accion_central_id.id, str_ae_ids, pec.id)

            cr.execute(query)
            array = cr.dictfetchone()
            res[pec.id] = array.get('amount_execute',0)

        return res
        
        
    def _get_amount_causado_central(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')

        res = {}
        for pec in self.browse(cr, uid, ids, context=context):
            res[pec.id] = 0.00
            str_ae_ids = str([x.id for x in pec.accion_central_id.child_ids]).replace('[','(').replace(']',')')
            query = """
                SELECT COALESCE(SUM(PIL.amount),0) AS amount_execute
                FROM presupuesto_ejecutado_central PEC, presupuesto_imputacion_linea PIL
                WHERE PEC.period_id = PIL.periodo
                AND PEC.period_id = %s
                AND PEC.accion_central_id = %s
                AND PIL.accion_especifica_id in %s
                AND PIL.status = 'causado_gasto'
                AND PEC.id = %s
            """%(pec.period_id.id, pec.accion_central_id.id, str_ae_ids, pec.id)

            cr.execute(query)
            array = cr.dictfetchone()
            res[pec.id] = array.get('amount_execute',0)

        return res
        
        
    def _get_amount_pagado_central(self, cr, uid, ids, name, args, context=None):
        str_ids = str(ids).replace('[','(').replace(']',')')

        res = {}
        for pec in self.browse(cr, uid, ids, context=context):
            res[pec.id] = 0.00
            str_ae_ids = str([x.id for x in pec.accion_central_id.child_ids]).replace('[','(').replace(']',')')
            query = """
                SELECT COALESCE(SUM(PIL.amount),0) AS amount_execute
                FROM presupuesto_ejecutado_central PEC, presupuesto_imputacion_linea PIL
                WHERE PEC.period_id = PIL.periodo
                AND PEC.period_id = %s
                AND PEC.accion_central_id = %s
                AND PIL.accion_especifica_id in %s
                AND PIL.status = 'pagado'
                AND PEC.id = %s
            """%(pec.period_id.id, pec.accion_central_id.id, str_ae_ids, pec.id)

            cr.execute(query)
            array = cr.dictfetchone()
            res[pec.id] = array.get('amount_execute',0)

        return res
        
        
    def _get_amount_residual_central(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        for id in id_brw:
            res[id.id] = id.amount_total_central - id.amount_comprometido_central
        return res
    
    
    #~ Aqui comienzan las funciones de los montos acumulados
    def _get_amount_comprometido_acumulado_central(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        acum = 0
        for id in id_brw:
            acum = id.amount_comprometido_central + acum
            res[id.id] = acum
        return res


    def _get_amount_total_acumulado_central(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        acum = 0
        for id in id_brw:
            acum = id.amount_total_central + acum
            res[id.id] = acum
        return res
    
    
    def _get_amount_residual_acumulado_central(self, cr, uid, ids, name, args, context=None):
        res = {}
        id_brw = self.browse(cr, uid, ids, context=context)
        acum = 0
        for id in id_brw:
            acum = (id.amount_total_central_acum - id.amount_comprometido_central_acum)
            res[id.id] = acum
        return res
        
    _columns = {
        'name': fields.char('Mes',size=64),
        'amount_total_central': fields.function(_get_amount_total_central, digits_compute=dp.get_precision('Account'), method=True, string='Monto Presupuestado'),
        'amount_total_central_acum': fields.function(_get_amount_total_acumulado_central, digits_compute=dp.get_precision('Account'), method=True, string='Acumulado Presupuestado'),
        'amount_comprometido_central': fields.function(_get_amount_compometido_central, digits_compute=dp.get_precision('Account'), method=True, string='Monto Comprometido'),
        'amount_comprometido_central_acum': fields.function(_get_amount_comprometido_acumulado_central, digits_compute=dp.get_precision('Account'), method=True, string='Acumulado Comprometido'),
        'amount_causado_central': fields.function(_get_amount_causado_central, digits_compute=dp.get_precision('Account'), method=True, string='Monto Causado'),
        'amount_pagado_central': fields.function(_get_amount_pagado_central, digits_compute=dp.get_precision('Account'), method=True, string='Monto Pagado'),
        'amount_residual_central': fields.function(_get_amount_residual_central, digits_compute=dp.get_precision('Account'), method=True, string='Disponibilidad'),
        'amount_residual_central_acum': fields.function(_get_amount_residual_acumulado_central, digits_compute=dp.get_precision('Account'), method=True, string='Acumulado Disponibilidad'),
        'period_id': fields.many2one('periodo.presupuestario', 'Periodo Presupuestario', required=True),
        'accion_central_id': fields.many2one('accion.centralizada', 'Accion Central/Proyecto', required=True),
    }
    
presupuesto_ejecutado_central()


class presupuesto_formulacion_cabecera(osv.osv):
    _name = "presupuesto.formulacion.cabecera"
    _description = "Presupuesto Formulacion Cabecera"
    
    
    def _get_total_amount(self, cr, uid, ids, name, args, context=None):
        res = {}
        pfc_brw = self.browse(cr, uid, ids, context=context)
        for line in pfc_brw:
            res.update({line.id: 0.00})
        for line in pfc_brw:
            amount = 0.00
            for form in line.formulacion_ids:
                amount += form.amount
                res.update({line.id:amount})
        return res
    
    
    _columns = {
        'name': fields.char('Descripcion', size=64),
        'accion_centralizada_id': fields.many2one('accion.centralizada','Accion Central / Proyecto', required=True),
        'ejercicio_id': fields.many2one('ejercicio.presupuestario','Ejercicio Presupuestario', required=True),
        'formulacion_ids': fields.one2many('presupuesto.formulacion.detalle','pfc_id','Partidas'),
        'amount': fields.function(_get_total_amount, digits_compute=dp.get_precision('Account'), method=True, type='float', string='Monto'),

    }
    
    
    def write(self, cr, uid, ids, values, context=None):
        res = super(presupuesto_formulacion_cabecera, self).write(cr, uid, ids, values, context)
        return res
        
        
    def agregar_partida(self, cr, uid, ids, context=None):
        context.update({'pfc_id':ids[0], 'cbutton':True})
        return { 
            'name':_("Linea de Formulacion"),
            'view_type': 'form',
            'view_mode':'form',
            'res_model': 'presupuesto.formulacion.detalle',
            'type': 'ir.actions.act_window',
            'target': 'new',
            'context': context,
            'ref': 'presupuesto_formulacion_detalle.view_presupuesto_formulacion_detalle_form'
        }
    
    
    def dummy(self, cr, uid, ids, context=None):
        return True
        
presupuesto_formulacion_cabecera()


class presupuesto_formulacion_detalle(osv.osv):
    _name="presupuesto.formulacion.detalle"
    _description="Presupuesto Formulacion Detalle"
    _rec_name = 'partida_id'
    
    _columns = {
        #~ 'name': fields.char('Descripcion', size=64),
        'partida_id': fields.many2one('clasificador.presupuestario','Partida', required=True),
        'pfc_id': fields.many2one('presupuesto.formulacion.cabecera','Presupuesto', required=True, ondelete='cascade'),
        'accion_especifica_id': fields.many2one('accion.especifica','Accion Especifica', required=True),
        #~ 'dist_select': fields.selection([(0,'Manual'),(12,'Periodo'),(4,'Trimestre'),(2,'Semestre')],'Distribucion cada:', required= False),
        'dist_select': fields.selection([('manual','Manual'),('periodo','Periodo'),('trimestre','Trimestre'),('semestre','Semestre')],'Distribucion cada:', required= False),
        'amount': fields.float('Monto', digits_compute=dp.get_precision('Account')),
        'line_ids': fields.one2many('presupuesto.formulacion','detalle_id','Distribucion'),
        'state':fields.selection([
            ('draft','Borrador'),
            ('confirmed','Confirmado'),
            ('done','Done'),
        ],'State', select=True, readonly=True),
    }
    
    _defaults = {
        'dist_select': 'periodo',
        'state': 'draft',
    }
    
            
    def boton_guardar(self, cr, uid, ids, values, context=None):
        if not context:
            context = {}
        monto = 0.00
        monto_linea = 0.00
        for pfd in self.browse(cr, uid, ids, context):
            monto = pfd.amount
            for lines in pfd.line_ids:
                monto_linea += lines.amount
        if not monto:
            raise osv.except_osv(_('Aviso !'),_('Debe asignar un monto para la distribucion antes de guardar.'))
        if monto != monto_linea:
            raise osv.except_osv(_('Aviso !'),_('Debe realizar la distribucion antes de guardar.'))
        super(presupuesto_formulacion_detalle, self).write(cr, uid, ids, values, context=context)
        return {'type':'ir.actions.act_window_close'}
    
    
    def boton_editar(self, cr, uid, ids, context=None):
        """Funcion que llama la vista de purchase.order al hacer click sobre el boton
        de editar"""
        return {
            'name':_("Linea de Formulacion"),
            'view_type': 'form',
            'view_mode':'form',
            'res_model': 'presupuesto.formulacion.detalle',
            'type': 'ir.actions.act_window',
            'target': 'new',
            'context': context,
            'ref': 'presupuesto_formulacion_detalle.view_presupuesto_formulacion_detalle_form',
            'res_id': ids,
        }
    
    
    def boton_eliminar(self, cr, uid, ids, context=None):
        return self.unlink(cr, uid, ids, context=context)
    
        
    def validar_montos(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        monto = 0.00
        monto_linea = 0.00
        i = 0
        for pfd in self.browse(cr, uid, ids, context):
            monto = pfd.amount
            for lines in pfd.line_ids:
                monto_linea += lines.amount
                i = i + 1
        
        if not pfd.line_ids:
            return True
        
        if pfd.state == 'confirmed' and monto != monto_linea:
            return True
        
        if monto == 0.00:
            return False
        
        res = 0.00
        
        if monto_linea > 0.00:
            res = float(monto_linea) - float(monto)
            if round(res) == 0.00:
                return True
        
        return False
    

    _constraints = [
        (validar_montos, 'ERROR, El Monto no puede ser cero y debe coincidir con el total de la distribucion', ['amount']),
        ]
        
        
    def distribuir_presupuesto(self, cr, uid, ids, context=None):
        """
        El calculo para la distribucion se obtiene de las siguientes formulas:
        Formula de Frecuencia: FC=(CP/D).
        Donde: FC = Frecuencia, CP = Cant de Periodos, D = Division 
        Formula de Monto a Distribuir: PM = (MP/CP).
        Donde: PM = Partes del Monto, MP = Monto de la Partida, CP = Cant de Periodos) 
        """
        if context is None:
            context = {}
        
        pe_obj = self.pool.get('presupuesto.formulacion')
        per_obj = self.pool.get('periodo.presupuestario')
        ej_obj = self.pool.get('ejercicio.presupuestario')
        
        ej_act = ej_obj.find(cr, uid)
        ej_brw = ej_obj.browse(cr, uid, [ej_act])[0]
        per_ids = per_obj.search(cr, uid, [('ejercicio_id','=',ej_act)])
        line_ids = []
        dist_amount = 0.00
        cont = 1

        
        for pfd in self.browse(cr, uid, ids, context):
            
            if pfd.dist_select == 'manual':
                dist_select = 'periodo'
            else:
                dist_select = pfd.dist_select
            
            if dist_select == 'periodo':
                dist_select = 12
            elif dist_select == 'trimestre':
                dist_select = 4
            elif dist_select == 'semestre':
                dist_select = 2
                
            if  dist_select <= len(per_ids):
                porc = (100.00/float(dist_select))

                fc = (len(per_ids) / dist_select)
                dist_amount = (pfd.amount/dist_select)
            else:
                raise osv.except_osv(_('Aviso !'),_('La distribucion elegida no concuerda con la cantidad de periodos.'))
            
            for month in ej_brw.period_ids:
                sids = pe_obj.search(cr, uid, [('detalle_id','=',ids[0]),('period_id','=',month.id)])

                if sids:
                    res = pe_obj.unlink(cr, uid, sids, context)
                    
                formulado_id = pe_obj.create(cr, uid,{
                'name': string.upper(str(calendar.month_name[int(month.date_start[5:7])])),
                'period_id': month.id,
                'accion_especifica_id': pfd.accion_especifica_id.id,
                'partida_id':pfd.partida_id.id,
                'amount': dist_amount if cont == 1 else 0.00,
                'detalle_id': ids[0],
                'porcentaje':porc,
                }, context=context)

                line_ids.append(formulado_id)

                if cont >= fc:
                    cont = 1
                else:
                    cont += 1
                    
        self.write(cr, uid, ids, {'state':'confirmed', 'line_ids': [(6,0,line_ids)]}, context=context)
        return context
    
presupuesto_formulacion_detalle()


class presupuesto_formulacion(osv.osv):
    _name = "presupuesto.formulacion"
    _description = "Presupuesto Formulacion"
    
    _columns = {
        'name': fields.char('Descripcion', size=128),
        'partida_id': fields.many2one('clasificador.presupuestario', 'Partida', required=True),
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica', required=True),
        'period_id': fields.many2one('periodo.presupuestario', 'Periodo Presupuestario', required=True),
        'amount': fields.float('Monto', digits_compute=dp.get_precision('Account'), required=True),
        'detalle_id': fields.many2one('presupuesto.formulacion.detalle', 'Partida', ondelete='cascade'),
        'porcentaje': fields.float('Porc.', digits_compute=dp.get_precision('Account')),
    }
    
    
    def write(self, cr, uid, ids, values, context=None):

        res = self.calcular_porcentaje(cr, uid, ids, values, context=context)
        if res:
            return super(presupuesto_formulacion,self).write(cr, uid, ids, values, context=context)
        return False
    
    
    def calcular_porcentaje(self, cr, uid, ids, values, context=None):

        if values.get('porcentaje',False) and values.get('detalle_id',False):
            obj_pl = self.pool.get("presupuesto.formulacion.detalle")
            amount = 0.00
            pl = self.read(cr, uid, ids, ['detalle_id'], context=context)

            if pl:
                brw_pl = obj_pl.browse(cr, uid, [pl[0]['detalle_id'][0]], context=context)
                for reg in brw_pl:
                    amount = reg.amount
                values['amount'] = ((values['porcentaje'] * amount)/100) 

        return True
        
presupuesto_formulacion()
