#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################
from osv import osv
from osv import fields
from tools.translate import _
from tools import config
import time
import datetime
import decimal_precision as dp

class accion_centralizada(osv.osv):
    _name = "accion.centralizada"
    _description = "Accion Centralizada"
    
    _columns = {
        'name': fields.char('Nombre', size=128),
        'gerencia': fields.char('Gerencia', size=128),
        'codigo': fields.char('Codigo', size=128),
        #~ 'onapre': fields.char('Codigo Onapre', size=128),
        'child_ids': fields.one2many('accion.especifica', 'accion_central_id', 'Acciones Especificas'),
        'responsable': fields.many2one('res.users', 'Responsable'),
        'note': fields.text('Descripcion', size=256),
        'type': fields.selection([
            ('proyecto','Proyecto'),
            ('centralizada','Accion Central'),
            ]),
        'ejecutado_central_ids': fields.one2many('presupuesto.ejecutado.central','accion_central_id', 'Detalle Presupuestario'),
    }
    
    _defaults = {
      'type' : lambda self, cr, uid, context : context['type'] if context and 'type' in context else None,
    }
    
accion_centralizada()

class accion_especifica(osv.osv):
    _name = "accion.especifica"
    _description = "Accion Especifica"
    
    def _get_amount(self, cr, uid, ids, name, args, context=None):

        amount = 0
        res = {}
        for brw_ac in self.browse(cr,uid,ids):
            res[brw_ac.id] = 0
            amount = 0
            for line in brw_ac.imp_lines_ids:
                amount += line.amount
            res[brw_ac.id] = amount
        return res
    
    def name_get(self, cr, uid, ids, context=None):
        res = []
        for line in self.browse(cr, uid, ids, context=context):
            res.append((line.id, (line.accion_central_id.name or 'ERROR')+' / '+line.name))
        return res
        
    def name_search(self, cr, user, name, args=None, operator='ilike', context=None, limit=80):
        if args is None:
            args = []
        if context is None:
            context = {}
        ids = []
        if name:
            ids = self.search(cr, user, [('accion_central_id.name','ilike',name)]+ args, limit=limit)
        if not ids:
            ids = self.search(cr, user, [('name',operator,name)]+ args, limit=limit)
        return self.name_get(cr, user, ids, context=context)
    
    _columns = {
        'name': fields.char('Nombre', size=128),
        'accion_central_id': fields.many2one('accion.centralizada', 'Proyecto / Accion Centralizada'),
        'codigo': fields.char('Codigo', size=128),
        'note': fields.text('Descripcion', size=256),
        'responsable': fields.many2one('res.users', 'Responsable'),
        'imp_lines_ids': fields.one2many('presupuesto.imputacion.linea', 'accion_especifica_id', 'Lineas de Imputacion'),
        'ejercicio_id': fields.many2one('ejercicio.presupuestario', 'Ejercicio Presupuestario'),
        'ejecutado_id': fields.one2many('presupuesto.ejecutado', 'accion_especifica_id', 'Detalle Presupuestario'),
    }
    
accion_especifica()

class res_company(osv.osv):
    _inherit = "res.company"
    _columns = {
        'onapre':fields.char('Codigo Onapre',size=64),
    }
res_company()
