#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import fields, osv
from tools.translate import _
import decimal_precision as dp
import time

class clasificador_presupuestario(osv.osv):
    _name = "clasificador.presupuestario"
    _description = "Clasificador Presupuestario"
    _parent_name = "parent_id"
    _parent_store = True
    _order = "type,code"
    
    def _get_amount_presupuesto(self, cr, uid, ids, name, args, context=None):
        if context is None:
            context = {}
            
        ejercicio_obj = self.pool.get('ejercicio.presupuestario')
        
        query = """
            SELECT CP.id AS id, COALESCE(SUM(PF.amount),0) AS amount_presupuesto
            FROM clasificador_presupuestario CP, presupuesto_formulacion PF
            WHERE CP.id = PF.partida_id
            """
        
        if context.get('periods', False):
            periodos = " AND PF.period_id in %s"%(str(context.get('periods')).replace('[','(').replace(']',')'))
            query = query + periodos
        else:
            ejercicio_id = ejercicio_obj.find(cr, uid, context=context)
            period_ids = [x.id for x in ejercicio_obj.browse(cr, uid, ejercicio_id, context).period_ids]
            periodos = " AND PF.period_id in %s"%(str(period_ids).replace('[','(').replace(']',')'))
            query = query + periodos
            
        if context.get('accion_especifica_id', False):
            accion = " AND PF.accion_especifica_id in %s"%(str(context.get('accion_especifica_id')).replace('[','(').replace(']',')'))
            query = query + accion
        
        grupo = " GROUP BY CP.id"
        query = query + grupo
        
        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
        for id in ids:
            res[id] = 0.00
            
        for i in array:
            res[i['id']] = i['amount_presupuesto']
        
        for i in ids:
            query_hijo = """select cp_hijo.id as id from clasificador_presupuestario cp_padre
            left outer join clasificador_presupuestario cp_hijo on cp_hijo.parent_left between cp_padre.parent_left and cp_padre.parent_right
            where cp_padre.id = %s and cp_hijo.type != 'vista'"""%(i)

            cr.execute(query_hijo)
            array_hijo = cr.dictfetchall()

            
            for j in array_hijo:
                if i != j['id'] and res.get(j['id'],False):
                    res[i] += res[j['id']]
        
        return res
    
    
    def _get_amount_comprometido(self, cr, uid, ids, name, args, context=None):
        if context is None:
            context = {}
        
        ejercicio_obj = self.pool.get('ejercicio.presupuestario')

        query_comprometido = """
            SELECT CP.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_comprometido
            FROM clasificador_presupuestario CP, presupuesto_imputacion_linea PIL
            WHERE PIL.status = 'comprometido'
            AND CP.id = PIL.partida_id
        """
        
        query_devengado = """
            SELECT CP.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_devengado
            FROM clasificador_presupuestario CP, presupuesto_imputacion_linea PIL
            WHERE PIL.status_ingreso = 'devengado'
            AND CP.id = PIL.partida_id
        """
        
        if context.get('periods', False):
            periodos = " AND PIL.periodo in %s"%(str(context.get('periods')).replace('[','(').replace(']',')'))
            query_comprometido = query_comprometido + periodos
            query_devengado = query_devengado + periodos
        else:
            ejercicio_id = ejercicio_obj.find(cr, uid, context=context)
            period_ids = [x.id for x in ejercicio_obj.browse(cr, uid, ejercicio_id, context).period_ids]
            periodos = " AND PIL.periodo in %s"%(str(period_ids).replace('[','(').replace(']',')'))
            query_comprometido = query_comprometido + periodos
            query_devengado = query_devengado + periodos
            
        if context.get('accion_especifica_id',False):
            accion = " AND PIL.accion_especifica_id in %s"%(str(context.get('accion_especifica_id')).replace('[','(').replace(']',')'))
            query_comprometido = query_comprometido + accion
            query_devengado = query_devengado + accion
        grupo = " GROUP BY CP.id"
        query_comprometido = query_comprometido + grupo
        query_devengado = query_devengado + grupo

        query = """
            SELECT CP.id AS id,
                CASE WHEN gasto.amount_comprometido IS null THEN 
                CASE WHEN ingreso.amount_devengado IS null THEN 0 
                ELSE ingreso.amount_devengado end 
                ELSE gasto.amount_comprometido end AS amount_comprometido
            FROM clasificador_presupuestario CP
                LEFT OUTER JOIN
                (%s) gasto ON CP.id = gasto.id
                LEFT OUTER JOIN
                (%s) ingreso ON CP.id = ingreso.id
            ORDER BY CP.id 
        """%(query_comprometido, query_devengado)

        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
            
        for i in array:
            res[i['id']] = i['amount_comprometido']
        
        for i in ids:
            query_hijo = """select cp_hijo.id as id from clasificador_presupuestario cp_padre
            left outer join clasificador_presupuestario cp_hijo on cp_hijo.parent_left between cp_padre.parent_left and cp_padre.parent_right
            where cp_padre.id = %s and cp_hijo.type != 'vista'"""%(i)

            cr.execute(query_hijo)
            array_hijo = cr.dictfetchall()
            
            for j in array_hijo:
                if i != j['id'] and res.get(j['id'],False):
                    res[i] += res[j['id']]
            
        return res
    
    
    def _get_amount_causado(self, cr, uid, ids, name, args, context=None):
        if context is None:
            context = {}
            
        ejercicio_obj = self.pool.get('ejercicio.presupuestario')
        
        query_causado = """
            SELECT CP.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_causado
            FROM clasificador_presupuestario CP, presupuesto_imputacion_linea PIL
            WHERE PIL.status = 'causado_gasto'
            AND CP.id = PIL.partida_id
        """
        
        query_liquidado = """
            SELECT CP.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_liquidado
            FROM clasificador_presupuestario CP, presupuesto_imputacion_linea PIL
            WHERE PIL.status_ingreso = 'liquidado'
            AND CP.id = PIL.partida_id
        """
        
        if context.get('periods', False):
            periodos = " AND PIL.periodo in %s"%(str(context.get('periods')).replace('[','(').replace(']',')'))
            query_causado = query_causado + periodos
            query_liquidado = query_liquidado + periodos
        else:
            ejercicio_id = ejercicio_obj.find(cr, uid, context=context)
            period_ids = [x.id for x in ejercicio_obj.browse(cr, uid, ejercicio_id, context).period_ids]
            periodos = " AND PIL.periodo in %s"%(str(period_ids).replace('[','(').replace(']',')'))
            query_causado = query_causado + periodos
            query_liquidado = query_liquidado + periodos
            
        if context.get('accion_especifica_id', False):
            accion = " AND PIL.accion_especifica_id in %s"%(str(context.get('accion_especifica_id')).replace('[','(').replace(']',')'))
            query_causado = query_causado + accion
            query_liquidado = query_liquidado + accion
        
        grupo = " GROUP BY CP.id"
        query_causado = query_causado + grupo
        query_liquidado = query_liquidado + grupo

        query = """
            SELECT CP.id AS id,
                CASE WHEN gasto.amount_causado IS null THEN 
                CASE WHEN ingreso.amount_liquidado IS null THEN 0 
                ELSE ingreso.amount_liquidado end 
                ELSE gasto.amount_causado end AS amount_causado
            FROM clasificador_presupuestario CP
                LEFT OUTER JOIN
                (%s) gasto ON CP.id = gasto.id
                LEFT OUTER JOIN
                (%s) ingreso ON CP.id = ingreso.id
            ORDER BY CP.id 
        """%(query_causado, query_liquidado)
        
        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
            
        for i in array:
            res[i['id']] = i['amount_causado']
        
        
        for i in ids:
            query_hijo = """select cp_hijo.id as id from clasificador_presupuestario cp_padre
            left outer join clasificador_presupuestario cp_hijo on cp_hijo.parent_left between cp_padre.parent_left and cp_padre.parent_right
            where cp_padre.id = %s and cp_hijo.type != 'vista'"""%(i)

            cr.execute(query_hijo)
            array_hijo = cr.dictfetchall()

            
            for j in array_hijo:
                if i != j['id'] and res.get(j['id'],False):
                    res[i] += res[j['id']]
        
        return res
    
    
    def _get_amount_pagado(self, cr, uid, ids, name, args, context=None):
        if context is None:
            context = {}
        
        ejercicio_obj = self.pool.get('ejercicio.presupuestario')
        
        query_pagado = """
            SELECT CP.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_pagado
            FROM clasificador_presupuestario CP, presupuesto_imputacion_linea PIL
            WHERE PIL.status = 'pagado'
            AND CP.id = PIL.partida_id
        """
        
        query_recaudado = """
            SELECT CP.id AS id, COALESCE(SUM(PIL.amount),0) AS amount_recaudado
            FROM clasificador_presupuestario CP, presupuesto_imputacion_linea PIL
            WHERE PIL.status_ingreso = 'recaudado'
            AND CP.id = PIL.partida_id
        """
        
        if context.get('periods', False):
            periodos = " AND PIL.periodo in %s"%(str(context.get('periods')).replace('[','(').replace(']',')'))
            query_pagado = query_pagado + periodos
            query_recaudado = query_recaudado + periodos
        else:
            ejercicio_id = ejercicio_obj.find(cr, uid, context=context)
            period_ids = [x.id for x in ejercicio_obj.browse(cr, uid, ejercicio_id, context).period_ids]
            periodos = " AND PIL.periodo in %s"%(str(period_ids).replace('[','(').replace(']',')'))
            query_pagado = query_pagado + periodos
            query_recaudado = query_recaudado + periodos
        
        if context.get('accion_especifica_id',False):
            accion = " AND PIL.accion_especifica_id in %s"%(str(context.get('accion_especifica_id')).replace('[','(').replace(']',')'))
            query_pagado = query_pagado + accion
            query_recaudado = query_recaudado + accion
        grupo = " GROUP BY CP.id"
        query_pagado = query_pagado + grupo
        query_recaudado = query_recaudado + grupo

        query = """
            SELECT CP.id AS id,
                CASE WHEN gasto.amount_pagado IS null THEN 
                CASE WHEN ingreso.amount_recaudado IS null THEN 0 
                ELSE ingreso.amount_recaudado end 
                ELSE gasto.amount_pagado end AS amount_pagado
            FROM clasificador_presupuestario CP
                LEFT OUTER JOIN
                (%s) gasto ON CP.id = gasto.id
                LEFT OUTER JOIN
                (%s) ingreso ON CP.id = ingreso.id
            ORDER BY CP.id 
        """%(query_pagado, query_recaudado)
        
        cr.execute(query)
        array = cr.dictfetchall()
        res = {}
            
        for i in array:
            res[i['id']] = i['amount_pagado']
        
        for i in ids:
            query_hijo = """select cp_hijo.id as id from clasificador_presupuestario cp_padre
            left outer join clasificador_presupuestario cp_hijo on cp_hijo.parent_left between cp_padre.parent_left and cp_padre.parent_right
            where cp_padre.id = %s and cp_hijo.type != 'vista'"""%(i)

            cr.execute(query_hijo)
            array_hijo = cr.dictfetchall()
            
            for j in array_hijo:
                if i != j['id'] and res.get(j['id'],False):
                    res[i] += res[j['id']]
            
        return res
        
        
    def _get_amount_disponibilidad(self, cr, uid, ids, name, args, context=None):
        if context is None:
            context = {}
        res = {}
        id_brw = self.browse(cr, uid, ids)
        ids_dis = [x.id for x in id_brw]
        presupuesto = self._get_amount_presupuesto(cr, uid, ids_dis, 'presupuesto', None, context=context)
        comprometido = self._get_amount_comprometido(cr, uid, ids_dis, 'comprometido', None, context=context)
        for id in id_brw:
            res[id.id] = presupuesto[id.id] - comprometido[id.id]
        return res
    
    
    def _get_child_ids(self, cr, uid, ids, field_name, arg, context=None):
        result = {}
        for record in self.browse(cr, uid, ids, context=context):
            if record.child_parent_ids:
                result[record.id] = [x.id for x in record.child_parent_ids]
            else:
                result[record.id] = []
                
        return result
    
    
    _columns = {
        'name': fields.char('Nombre', size=512, required=True, select=True),
        'code': fields.char('Codigo', size=64, required=True),
        'type': fields.selection([
            ('vista', 'Vista'),
            ('otra', 'Regular'),
            ('ingreso', 'Ingreso'),
            ('gasto', 'Gasto'),
            ('cerrada', 'Cerrada'),
        ], 'Tipo de Cuenta', required=True),
        'parent_id':fields.many2one('clasificador.presupuestario', 'Padre', ondelete='cascade'),
        'child_parent_ids': fields.one2many('clasificador.presupuestario','parent_id','Hijos'),
        'child_id': fields.function(_get_child_ids, method=True, type='many2many', relation="clasificador.presupuestario", string="Partidas Hijas"),
        'active': fields.boolean('Activo', select=2, help="Si el campo active es False, la partida presupuestaria no podra recibir imputaciones."),
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'parent_left': fields.integer('Parent Left', select=1),
        'parent_right': fields.integer('Parent Right', select=1),
        'presupuesto': fields.function(_get_amount_presupuesto, digits_compute=dp.get_precision('Account'), method=True, string='Monto Presupuestado'),
        'comprometido': fields.function(_get_amount_comprometido, digits_compute=dp.get_precision('Account'), method=True, string='Monto Comprometido'),
        'causado': fields.function(_get_amount_causado, digits_compute=dp.get_precision('Account'), method=True, string='Monto Causado'),
        'pagado': fields.function(_get_amount_pagado, digits_compute=dp.get_precision('Account'), method=True, string='Monto Pagado'),
        'disponibilidad': fields.function(_get_amount_disponibilidad, digits_compute=dp.get_precision('Account'), method=True, string='Disponibilidad'),
    }
    
    
    def name_get(self, cr, uid, ids, context=None):
        res = []
        for line in self.browse(cr, uid, ids, context=context):
            res.append((line.id, (line.code or 'XXXX')+' '+line.name))
        return res
    
    
    def name_search(self, cr, user, name, args=None, operator='ilike', context=None, limit=80):
        if args is None:
            args = []
        if context is None:
            context = {}
        ids = []
        if name:
            ids = self.search(cr, user, [('code','ilike',name)]+ args, limit=limit)
        if not ids:
            ids = self.search(cr, user, [('name',operator,name)]+ args, limit=limit)
        return self.name_get(cr, user, ids, context=context)
    
    
    _defaults = {
        'active': True,
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
    }

clasificador_presupuestario()


class product_product(osv.osv):
    _inherit = "product.product"
    _name = "product.product"
    
    _columns = {
        'partida_egreso':fields.many2one('clasificador.presupuestario', 'Partida de Gasto'),
        'partida_ingreso':fields.many2one('clasificador.presupuestario', 'Partida de Ingreso'),
    }
    
product_product()

class account_tax(osv.osv):
    _inherit = 'account.tax'
    
    _columns = {
        'partida_impuesto': fields.many2one('clasificador.presupuestario', 'Partida de Impuesto'),
    }

account_tax()

class islr_wh_concept(osv.osv):
    _inherit = "islr.wh.concept"
    
    _columns = {
        'partida_islr': fields.many2one('clasificador.presupuestario', 'Partida de ISLR'),
    }
    
islr_wh_concept()
