#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################
import netsvc
import pooler
from osv import osv
from osv import fields
from tools.translate import _
from tools import config
import time
from datetime import datetime
import decimal_precision as dp
from dateutil.relativedelta import relativedelta
from osv.orm import browse_record, browse_null
import toolsd

class presupuesto_imputacion_linea(osv.osv):
    _name = "presupuesto.imputacion.linea"
    _description = "Presupuesto Imputacion Linea"
    _rec_name = 'ref'
    _order = "id desc"
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
        'fecha_imp': fields.date('Fecha Imputacion', required=True),
        'model': fields.char('Objeto', size=64, select=1),
        'res_id': fields.integer('ID Referencia', select=1),
        'ref': fields.char('referencia', size=256),
        'partida_id': fields.many2one('clasificador.presupuestario','Partida'),
        'enlace_id': fields.integer('Id del Enlace'),
        'partida': fields.char('Partida Presupuestaria', size=128),
        'amount': fields.float('Total Imputado', digits_compute=dp.get_precision('Account')),
        'periodo': fields.many2one('periodo.presupuestario', 'Periodo Presupuestario',
                                   required=True),
        'status': fields.selection([('precomprometido', 'PreComprometido'),
                        ('comprometido', 'Comprometido'),
                        ('causado_gasto', 'Causado'),
                        ('pagado', 'Pagado')], 'Status'),
        'status_ingreso': fields.selection([('devengado', 'Devengado'),
                        ('liquidado', 'Liquidado'),
                        ('recaudado', 'Recaudado')], 'Status'),
        'state': fields.selection([('activo', 'Activa'), ('cancel', 'Cancelado'),
                        ('transition', 'Transicion')],
                        'Estado', required=True, readonly=True),
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'type': fields.selection([('ingreso', 'Ingreso'),
                        ('egreso', 'Gasto')], 'Tipo', required=True),
        'type_create':fields.selection([('manual', 'Manual'),
                        ('automatica', 'Automatica')], 'Tipo Creacion'),
        'checkear': fields.selection([('check_ok', 'Chequeado'),
                        ('by_check', 'Por Chequear')], 'Chequear'),
        'ejercicio_id': fields.related('periodo', 'ejercicio_id', type='many2one',
                        relation='ejercicio.presupuestario', string='Ejercicio Presupuestario',
                        store=True),
    }
    
    _defaults = {
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'type': 'egreso',
        'state': 'activo',
        'fecha_imp': lambda *a: time.strftime('%Y-%m-%d'),
        'checkear': 'by_check',
        'type_create': 'manual',
    }


    def _check_type(self, cr, uid, ids, context=None):
        """ Verificar que el status de la imputacion se encuentre en el mismo tipo
        la imputacion.
         * tipo egreso corresponden el status: precomprometido, comprometido, causado_gasto
            y pagado
         * tipo ingreso corresponden el status_ingreso: devengado, liquidado, recauado
         
        @return: True (bien, no se activa el error) or False (mal, se activa el error)
        """
        for imp in self.browse(cr, uid, ids, context=context):
            if (imp.type == 'egreso') and (imp.status in ('precomprometido', 'comprometido', 'causado_gasto', 'pagado')):
                if imp.partida_id:
                    if imp.partida_id.type == 'gasto':
                        return True
                    else:
                        return False
                else:
                    return True
            elif (imp.type == 'ingreso') and (imp.status_ingreso in ('devengado','liquidado','recaudado')):
                if imp.partida_id:
                    if imp.partida_id.type == 'ingreso':
                        return True
                    else:
                        return False
        return False
    
    
    def _check_fecha_imp(self, cr, uid, ids, context=None):
        """ Verificar que la fecha de la imputacion se encuentre dentro del rango
            del periodo seleccionado.
        @return: True (bien) or False (mal)
        """
        periodo_obj = self.pool.get('periodo.presupuestario')
        
        for imp in self.browse(cr, uid, ids, context=context):
            if (imp.fecha_imp >= imp.periodo.date_start) and (imp.fecha_imp <= imp.periodo.date_stop):
                return True
        return False

    #~ _constraints = [(_check_type, 'El Status o la Partida Presupuestaria no corresponde con el Tipo de imputacion', ['type']),
                    #~ (_check_fecha_imp, 'La fecha de la imputacion no se encuentra dentro del rango del Periodo', ['fecha_imp'])]
    _constraints = [(_check_fecha_imp, 'La fecha de la imputacion no se encuentra dentro del rango del Periodo', ['fecha_imp'])]
    
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """
        Herencia del search para agregar filtro que busque periodo y ejercicio de presupuesto,
        se hace verificando patron que es enviado por el filtro de 'current_period' y
        'current_year'.
        """
        if context is None:
            context = {}
        
        period_obj = self.pool.get('periodo.presupuestario')
        fiscal_obj = self.pool.get('ejercicio.presupuestario')
        for arg in args:
            if arg[0] == 'periodo' and arg[2] == 'current_period':
                current_period = period_obj.find(cr, uid)[0]
                args.append(('periodo','in',[current_period]))
                break
            elif arg[0] == 'period_id' and arg[2] == 'current_year':
                current_year = fiscal_obj.find(cr, uid)
                args.append(['ejercicio_id','=',current_year])
                break
        for a in args:
            if a in [('periodo','in','current_period'),('period_id','in','current_year')]:
                args.remove(a)
        return super(presupuesto_imputacion_linea, self).search(cr, uid, args=args, offset=offset, limit=limit, order=order,
            context=context, count=count)
    
    
    def ref_imputacion(self, cr, uid, ids, context, origen, ref, pro_id, modelo):
        if context is None:
            context = {}
        
        imp_id = self.search(cr, uid, [('ref','=',origen), ('res_id','=',pro_id), ('model','=',modelo)])
        imp_brw = self.browse(cr, uid, imp_id, context=context)
        #~ self.write(cr, uid, imp_id, {'ref':ref}, context=context)
        
        return imp_brw


    def onchange_partida(self, cr, uid, ids, partida_id, context=None):
        res = {}
        if partida_id:
            par_brw = self.pool.get('clasificador.presupuestario').browse(cr, uid, [partida_id],context=context)[0]
            res['value'] = {}
            if par_brw:
                code = par_brw.code
                name = par_brw.name
                res['value']['partida'] = str(toolsd.elimina_tildes('%s %s'%(code, name)))
        return res


    def checkear_imputacion(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'checkear': 'check_ok'}, context=context)


    def onchange_fecha_periodo(self, cr, uid, ids, fecha, context=None):
        """
        On change para modificar el periodo, al momento de insertar una fecha de
        imputacion.
        """
        periodo_obj = self.pool.get('periodo.presupuestario')
        if not ids:
            ids = context.get('active_id', False)
        if fecha and ids:
            periodo = periodo_obj.search(cr, uid, [('date_start', '<=', fecha), ('date_stop', '>=', fecha)])
            if not periodo:
                return {'value': {'periodo': None}, 'warning': {'message': 'No existe un periodo presupuestario para la fecha'}}
            return {'value': {'periodo': periodo}}
        else:
            return {'value': {'periodo': None}}
        
presupuesto_imputacion_linea()


class purchase_order_line(osv.osv):
    _inherit = "purchase.order.line"
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
        'enlace_id':fields.integer('ID del Enlace'),
    }
    
purchase_order_line()


class stock_move(osv.osv):
    _inherit = "stock.move"
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
        'enlace_id':fields.integer('ID del Enlace'),
    }
stock_move()


class purchase_order(osv.osv):
    _inherit = "purchase.order"
    
    def buscar_origen(self, cr, uid, requisition, product_id):
        obj_pro = self.pool.get('procurement.order')
        obj_req = self.pool.get('purchase.requisition')
        
        origin = requisition.origin
        addr_id = obj_pro.search(cr, uid, [('origin','=',origin),('product_id','=',product_id.id)])

        while(addr_id == [] and origin):
            addr_id = obj_req.search(cr, uid, [('name','=',origin)])
            origin = obj_req.read(cr, uid, addr_id, ['origin'], context=context)[0]['origin'] or None
            addr_id = obj_pro.search(cr, uid, [('origin','=',origin),('product_id','=',product_id.id)])
            
        abast = obj_pro.read(cr, uid, addr_id, ['origin','id'], context=context)[0] or False

        return abast
    
    
    def wkf_confirm_order(self, cr, uid, ids, context=None):
        """
        Se Hereda la funcion wkf_confirm_order para imputar el estado comprometido
        por cada linea de la orden de comprar, esto se hace solo cuando la misma 
        es aprobada.
        """
        if context is None:
            context = {}
        
        res = super(purchase_order, self).wkf_confirm_order(cr, uid, ids, context=context)
        
        if res:
            po_obj = self.browse(cr, uid, ids, context=context)
            imp_obj = self.pool.get('presupuesto.imputacion.linea')
            period_obj = self.pool.get('periodo.presupuestario')
            order_line_obj = self.pool.get('purchase.order.line')
            
            fecha = time.strftime('%Y-%m-%d')
            periodo = period_obj.find(cr, uid, fecha, context=context)[0]
            impuesto = po_obj[0].amount_tax
            
            for pol in po_obj[0].order_line:
                imp_dict = {}
                
                qty = pol.product_qty
                price = pol.price_unit
                amount = (price * qty)
                
                acc_esp_id = pol.accion_especifica_id and pol.accion_especifica_id.id or None
                # Validacion por si no se activa el requerido para accion_especifica_id de la vista
                if acc_esp_id is None:
                    raise osv.except_osv(('Error !'), ('No hay accion especifica definida para linea %s.'%pol.name))
                
                partida = pol.product_id.partida_egreso and pol.product_id.partida_egreso.code or ''
                if partida == '':
                    raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%pol.name))
                else:
                    partida_id = pol.product_id.partida_egreso.id
                    partida += ' '+pol.product_id.partida_egreso.name
                name_ref = pol.order_id.name + ': ' + pol.name
                enlace_id = pol.enlace_id
                
                imp_dict = {
                    'ref': name_ref,
                    'partida': partida,
                    'partida_id': partida_id,
                    'enlace_id': enlace_id,
                    'model': 'purchase.order.line',
                    'res_id': pol.id,
                    'fecha_imp': fecha,
                    'amount': amount,
                    'periodo': periodo,
                    'accion_especifica_id': acc_esp_id,
                    'status': 'comprometido',
                    'state': 'activo',
                    'type': 'egreso',
                    'type_create': 'automatica',
                }
                
                imp_id = imp_obj.create(cr, uid, imp_dict, context)
                if not enlace_id:
                    imp_obj.write(cr, uid, imp_id, {'enlace_id': imp_id}, context)
                    order_line_obj.write(cr, uid, pol.id, {'enlace_id': imp_id}, context)
                    enlace_id = imp_id
                
                """
                Se agrega la imputacion del impuesto asociado a cada producto
                o servicio, si la orden de compra lo tiene. Se valida que exista
                impuesto total en la orden de comprar para evitar hacer procedimiento
                innecesario.
                """
                if impuesto:
                    tax_amount = 0.00
                    for tax_line in pol.taxes_id:
                        tax_amount += amount * tax_line.amount
            
                    if tax_amount:
                        partida = pol.taxes_id[0].partida_impuesto and pol.taxes_id[0].partida_impuesto.code or None
                        if partida is None:
                            raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el impuesto %s.'%pol.taxes_id[0].name))
                        else:
                            partida_id = pol.taxes_id[0].partida_impuesto.id
                            partida += ' '+pol.taxes_id[0].partida_impuesto.name
                        imp_dict = {}
                        name_ref = pol.order_id.name + ': ' + pol.name
                        imp_dict = {
                            'ref': 'Impuesto de %s'%name_ref,
                            'partida': partida,
                            'partida_id':partida_id,
                            'enlace_id': enlace_id,
                            'model': 'purchase.order.line',
                            'res_id': pol.id,
                            'fecha_imp': fecha,
                            'amount': tax_amount,
                            'periodo': periodo,
                            'accion_especifica_id': acc_esp_id,
                            'status': 'comprometido',
                            'state': 'activo',
                            'type': 'egreso',
                            'type_create':'automatica',
                        }
                        imp_id = imp_obj.create(cr, uid, imp_dict, context)
        return res


    def action_invoice_create(self, cr, uid, ids, *args):
        """
        Se herada la funcion action_invoice_create para colocar a los 
        account.invoice.line la accion especifica que viene desde la 
        purchase.order.line y asignar la partida presupuestario de 
        egreso que tiene el producto
        """
        res = super(purchase_order,self).action_invoice_create(cr, uid, ids, *args)
        
        ail_obj = self.pool.get("account.invoice.line")
        po_brw = self.browse(cr, uid, ids)
        
        for order in po_brw:
            prod = {}
            for order_line in order.order_line:
                if order_line.product_id.type == 'service':
                    ae_id = order_line.accion_especifica_id and order_line.accion_especifica_id.id or None
                    enlace_id = order_line.enlace_id or None
                    partida = order_line.product_id.partida_egreso and order_line.product_id.partida_egreso.id or None
                    clave = toolsd.elimina_tildes(order_line.name)
                    prod[clave] = [ae_id, enlace_id, partida]
            
            for inv in order.invoice_ids:
                ae_id = None,
                enlace_id = None
                partida = None
                for inv_lin in inv.invoice_line:
                    clave = toolsd.elimina_tildes(inv_lin.name)
                    ae_id, enlace_id, partida = prod.get(clave, (False,False,False))
                    ail_obj.write(cr,uid,inv_lin.id,{'accion_especifica_id': ae_id, 'partida_presupuesto': partida, 'enlace_id':enlace_id})
        return res


    def action_picking_create(self,cr, uid, ids, *args):
        """
        Se herada la funcion action_picking_create para colocar a los stock.move
        la accion especifica que viene desde la purchase.order.line
        """
        picking_id = super(purchase_order,self).action_picking_create(cr, uid, ids, *args)
        sm_obj = self.pool.get("stock.move")
        po_brw = self.browse(cr, uid, ids)
        for order in po_brw:
            for order_line in order.order_line:
                if order_line.product_id.type != 'service':
                    for move in order_line.move_ids:
                        ae_id = order_line.accion_especifica_id and order_line.accion_especifica_id.id or None
                        enlace_id = order_line.enlace_id
                        sm_obj.write(cr, uid, move.id, {'accion_especifica_id': ae_id, 'enlace_id': enlace_id,})
        return picking_id


    def procurement_backorder(self, cr, uid, po_obj, new_procurement, context=None):
        """
        Se herada la funcion procurement_backorder para cuando se genera una backorder
        de una orden de compra, se coloca a los purchase.requisition.line la accion 
        especifica y enlace_id que viene desde la purchase.requisition.line original
        """
        super(purchase_order,self).procurement_backorder(cr, uid, po_obj, new_procurement, context=context)
        
        if context is None:
            context = {}
        
        pur_req_obj = self.pool.get('purchase.requisition')
        pur_req_lin_obj = self.pool.get('purchase.requisition.line')
        
        origin_pr = po_obj[0].requisition_id.name
        
        prod = {}
        for line in po_obj[0].order_line:
            ae_id = line.accion_especifica_id and line.accion_especifica_id.id or None
            enlace_id = line.enlace_id or None
            if line.product_id.type == 'service':
                clave = toolsd.elimina_tildes(line.name)
                prod[clave] = [ae_id, enlace_id]
            else:
                prod[str(line.product_id.id)] = [ae_id, enlace_id]
        pur_req_ids = pur_req_obj.search(cr, uid, [('origin','=',origin_pr),('state','=','in_progress')], context=context)
        pur_req_brw = pur_req_obj.browse(cr, uid, pur_req_ids, context=context)
        
        for pur in pur_req_brw:
            for pur_lin in pur.line_ids:
                if pur_lin.product_id.type == 'service':
                    clave = toolsd.elimina_tildes(pur_lin.description)
                    ae_id, enlace_id = prod.get(clave, (False,False))
                else:
                    ae_id, enlace_id = prod.get(str(pur_lin.product_id.id), (False,False))
                if ae_id and enlace_id:
                    pur_req_lin_obj.write(cr, uid, pur_lin.id, {'accion_especifica_id': ae_id, 'enlace_id': enlace_id}, context)
    
purchase_order()


class procurement_order(osv.osv):
    _inherit = 'procurement.order'
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
    }
    
procurement_order()


class diana_stock_procurement_service(osv.osv_memory):
    _inherit = 'diana.stock.procurement.service'
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
    }
    
    def agregar_servicio(self, cr, uid, ids, context=None):
        """
        Se sobre escribe la funcion agregar_servicio, para incluir la seleccion
        de una accion especifica que sera unido a la solicitud de abastecimeinto.
        """
        service = self.browse(cr, uid, ids, context)
        
        for line in service:
                
            procurement_order = {
                'name': 'Orden de Servicio',
                'diana_stock_procurement_id': int(context['diana_stock_procurement_id'][0]), 
                'origin': context['origin'][0]['name'], 
                'product_id':  line.product_id.id,
                'product_qty': line.product_qty,
                'product_uom': line.product_uom.id,
                'location_id': line.location_id.id,
                'date_planned': line.date_planned,
                'accion_especifica_id': line.accion_especifica_id and line.accion_especifica_id.id or None,
                'note': line.note,
            }
            
        obj_procurement = self.pool.get('procurement.order')
        procurement_order_id = obj_procurement.create(cr,uid,procurement_order)
        context.update({'procurement_order_id':procurement_order_id})
        
        return { 
            'view_type': 'form',
            'view_mode':'form',
            'res_model': 'diana.stock.procurement',
            'type': 'ir.actions.act_window',
            'target': 'edit',
            'context': context,
            'res_id': int(context['diana_stock_procurement_id'][0]),
            'ref': 'diana_almacenes.abastecimiento_form_view',
        }
            
diana_stock_procurement_service()


class diana_stock_procurement(osv.osv):
    
    _name = 'diana.stock.procurement'
    _inherit = 'diana.stock.procurement'
    
    def solicitar_abastecimiento(self, cr, uid, ids, context=None):
        """
        Se Hereda la funcion solicitar_abastecimiento para imputar cada linea de
        la solicitud de abastecimiento masiva, para guardar lo que se esta pre-
        comprometiendo.
        """
        view = super(diana_stock_procurement,self).solicitar_abastecimiento(cr, uid, ids, context)
        obj_uom = self.pool.get("product.uom")
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        period_obj = self.pool.get('periodo.presupuestario')
        req_lin_obj = self.pool.get('purchase.requisition.line')
        fecha = time.strftime('%Y-%m-%d')
        
        if self.browse(cr, uid, ids, context=context)[0].state == 'done':
            for lines in self.browse(cr, uid, ids, context=context)[0].procurement_order_ids:
                imp_dict = {}
                qty = lines.product_qty
                price = lines.product_id.standard_price
                product_uom =  lines.product_id.uom_id.id
                abast_uom = lines.product_uom.id
                
                if lines.product_id.type != 'service':
                    qty = obj_uom._compute_qty(cr, uid, abast_uom, qty, product_uom)

                amount = (price * qty)
                if amount > 0.00:
                    ae_id = lines.accion_especifica_id and lines.accion_especifica_id.id or None
                    partida = lines.product_id.partida_egreso and lines.product_id.partida_egreso.code or ''
                    if partida == '':
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el producto %s.'%lines.product_id.name))
                    else:
                        partida_id = lines.product_id.partida_egreso.id
                        partida += ' '+ lines.product_id.partida_egreso.name
                    name_ref = lines.origin
                    if lines.product_id.type != 'service':
                        name_ref += ': [' + lines.product_id.default_code + '] ' + lines.product_id.name
                    else:
                        name_ref += ': ' + lines.note
                    imp_dict = {
                        'ref': name_ref,
                        'partida': partida,
                        'partida_id': partida_id,
                        'model': 'procurement.order',
                        'res_id': lines.id,
                        'fecha_imp': fecha,
                        'amount': amount,
                        'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                        'accion_especifica_id': ae_id,
                        'status': 'precomprometido',
                        'state': 'activo',
                        'type': 'egreso',
                        'type_create':'automatica',
                    }

                    imp_id = imp_obj.create(cr, uid, imp_dict, context)
                    imp_obj.write(cr, uid, imp_id, {'enlace_id': imp_id}, context)
                    
                    """
                    Se escribe la accion especifica de la linea de solicitud en la
                    linea de la solicitud de compra (purchase.requisition.line).
                    """
                    if lines.requisition_id.line_ids:
                        prl_id = lines.requisition_id.line_ids[0].id
                        req_lin_obj.write(cr, uid, prl_id, {'accion_especifica_id': ae_id, 'enlace_id': imp_id}, context)
        
        return view
            
diana_stock_procurement()


class purchase_requistion_line(osv.osv):
    _inherit = "purchase.requisition.line"
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
        'enlace_id': fields.integer('ID del Enlace'),
    }
    
purchase_requistion_line()


class purchase_requisition_partner(osv.osv_memory):
    _inherit = "purchase.requisition.partner"

    def create_order(self, cr, uid, ids, context=None):
        """
             To Create a purchase orders .

             @param self: The object pointer.
             @param cr: A database cursor
             @param uid: ID of the user currently logged in
             @param ids: the ID or list of IDs
             @param context: A standard dictionary
             @return: {}

        """
        if context is None:
            context = {}
        record_ids = context and context.get('active_ids', False)
        if record_ids:
            data =  self.browse(cr, uid, ids)
            company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
            order_obj = self.pool.get('purchase.order')
            order_line_obj = self.pool.get('purchase.order.line')
            partner_obj = self.pool.get('res.partner')
            tender_line_obj = self.pool.get('purchase.requisition.line')
            pricelist_obj = self.pool.get('product.pricelist')
            prod_obj = self.pool.get('product.product')
            tender_obj = self.pool.get('purchase.requisition')
            acc_pos_obj = self.pool.get('account.fiscal.position')
            partner_id = data[0].partner_id.id

            supplier_data = partner_obj.browse(cr, uid, partner_id, context=context)

            address_id = partner_obj.address_get(cr, uid, [partner_id], ['delivery'])['delivery']
            list_line=[]
            purchase_order_line={}
            for tender in tender_obj.browse(cr, uid, record_ids, context=context):
                for line in tender.line_ids:
                    partner_list = sorted([(partner.sequence, partner) for partner in  line.product_id.seller_ids if partner])
                    partner_rec = partner_list and partner_list[0] and partner_list[0][1] or False
                    #~ uom_id = line.product_id.uom_po_id and line.product_id.uom_po_id.id or False
                    """ FIX: Toma la medida del purchase requisition line """
                    uom_id = line.product_uom_id and line.product_uom_id.id or False
                    """ ----------------------------------- """

                    if tender.date_start:
                        newdate = datetime.strptime(tender.date_start, '%Y-%m-%d %H:%M:%S') - relativedelta(days=company.po_lead)
                    else:
                        newdate = datetime.today() - relativedelta(days=company.po_lead)
                    delay = partner_rec and partner_rec.delay or 0.0
                    if delay:
                        newdate -= relativedelta(days=delay)

                    partner = partner_rec and partner_rec.name or supplier_data
                    pricelist_id = partner.property_product_pricelist_purchase and partner.property_product_pricelist_purchase.id or False
                    price = pricelist_obj.price_get(cr, uid, [pricelist_id], line.product_id.id, line.product_qty, False, {'uom': uom_id})[pricelist_id]
                    product = prod_obj.browse(cr, uid, line.product_id.id, context=context)
                    location_id = self.pool.get('stock.warehouse').browse(cr, uid, [tender.warehouse_id.id], context)[0]['lot_input_id'].id

                    purchase_order_line= {
                            'name': product.partner_ref,
                            'product_qty': line.product_qty,
                            'product_id': line.product_id.id,
                            'product_uom': uom_id,
                            'price_unit': price,
                            'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                            #~ 'notes': product.description_purchase,
                            'notes': line.description,
                            'accion_especifica_id': line.accion_especifica_id.id,
                            'enlace_id': line.enlace_id,
                    }
                    if product.product_tmpl_id.type == 'service':
                        invoice_method = 'order'
                        purchase_order_line.update({
                            'name' : line.description,
                            })
                    else:
                        invoice_method = 'picking'
                    
                    taxes_ids = line.product_id.product_tmpl_id.supplier_taxes_id
                    taxes = acc_pos_obj.map_tax(cr, uid, partner.property_account_position, taxes_ids)
                    purchase_order_line.update({
                            'taxes_id': [(6,0,taxes)]
                        })
                    list_line.append(purchase_order_line)
                purchase_id = order_obj.create(cr, uid, {
                            'origin': tender.purchase_ids and tender.purchase_ids[0].origin or tender.name,
                            'partner_id': partner_id,
                            'partner_address_id': address_id,
                            'pricelist_id': pricelist_id,
                            'location_id': tender.purchase_ids and tender.purchase_ids[0].location_id.id or line.product_id.product_tmpl_id.property_stock_production.id,
                            'company_id': tender.company_id.id,
                            'fiscal_position': partner.property_account_position and partner.property_account_position.id or False,
                            'requisition_id':tender.id,
                            'notes':tender.description,
                            'warehouse_id':tender.warehouse_id.id and tender.warehouse_id.id ,
                            'location_id':location_id,
                            'company_id':tender.company_id.id,
                            'invoice_method' : invoice_method,
                })
                order_ids=[]
                for order_line in list_line:
                    order_line.update({
                            'order_id': purchase_id
                        })
                    order_line_obj.create(cr, uid, order_line, context)
        return {'type': 'ir.actions.act_window_close'}

purchase_requisition_partner()


class purchase_requisition_merge(osv.osv_memory):
    """
    Herencia para validar que no se realicen fusiones de pedidos de compra con
    orden de compra aprobada.
    """
    _inherit = "purchase.requisition.merge"
    
    def merge_requisitions(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        req_obj = self.pool.get('purchase.requisition')
        
        req_ids = context.get('active_ids',[])
        req_brws = req_obj.browse(cr, uid, req_ids, context)
        
        """
        Verificacion si alguna de las requisiciones tiene una orden de compra
        aprobada.
        """
        for req_brw in req_brws:
            for order in req_brw.purchase_ids:
                if order.state == 'approved':
                    raise osv.except_osv(_('ERROR'),_('No se puede fusionar Solicitudes de Compra que tengan una Orden de Compra Aprobada!'))
                    
        return super(purchase_requisition_merge,self).merge_requisitions(cr, uid, ids, context=context)
    
    
    def do_create_order(self, cr, uid, ids, context=None):
        """
             To Create a purchase orders .

             @param self: The object pointer.
             @param cr: A database cursor
             @param uid: ID of the user currently logged in
             @param ids: the ID or list of IDs
             @param context: A standard dictionary
             @return: {}

        """
        list_line, location_id, invoice_method, partner, partner_id, address_id, pricelist_id = self.create_purchase_merge_lines(cr, uid, ids, context)

        return self.create_purchase_merge_header(cr, uid, ids, list_line, location_id, invoice_method, partner, partner_id, address_id,pricelist_id,context)  
    
    
    def create_purchase_merge_lines(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        record_ids = [ids]
        
        if record_ids:
            data =  self.browse(cr, uid, ids, context)
            company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
            order_obj = self.pool.get('purchase.order')
            partner_obj = self.pool.get('res.partner')
            pricelist_obj = self.pool.get('product.pricelist')
            prod_obj = self.pool.get('product.product')
            tender_obj = self.pool.get('purchase.requisition')
            acc_pos_obj = self.pool.get('account.fiscal.position')

            partner_ids = []
            po_ids = order_obj.search(cr, uid, [('requisition_id', '=', ids)])
            for po in order_obj.browse(cr, uid, po_ids, context):
                partner_ids.append(po.partner_id.id)
            partner_ids = list(set(partner_ids))
            for partner_id in partner_ids:
                supplier_data = partner_obj.browse(cr, uid, partner_id, context=context)

                address_id = partner_obj.address_get(cr, uid, [partner_id], ['delivery'])['delivery']
                list_line=[]
                purchase_order_line={}
                for tender in tender_obj.browse(cr, uid, record_ids, context=context):
                    for line in tender.line_ids:
                        partner_list = sorted([(partner.sequence, partner) for partner in  line.product_id.seller_ids if partner])
                        partner_rec = partner_list and partner_list[0] and partner_list[0][1] or False
                        uom_id = line.product_uom_id.id
                        
                        if tender.date_start:
                            newdate = datetime.strptime(tender.date_start, '%Y-%m-%d %H:%M:%S') - relativedelta(days=company.po_lead)
                        else:
                            newdate = datetime.today() - relativedelta(days=company.po_lead)
                        delay = partner_rec and partner_rec.delay or 0.0
                        if delay:
                            newdate -= relativedelta(days=delay)

                        partner = partner_rec and partner_rec.name or supplier_data
                        pricelist_id = partner.property_product_pricelist_purchase and partner.property_product_pricelist_purchase.id or False
                        price = pricelist_obj.price_get(cr, uid, [pricelist_id], line.product_id.id, line.product_qty, False, {'uom': uom_id})[pricelist_id]
                        product = prod_obj.browse(cr, uid, line.product_id.id, context=context)
                        location_id = self.pool.get('stock.warehouse').browse(cr, uid, [tender.warehouse_id.id],context)[0]['lot_input_id'].id
                        
                        if line.product_id.product_tmpl_id.type == 'service':
                            invoice_method = 'order'
                        else:
                            invoice_method = 'picking'
                            
                        purchase_order_line= {
                                'name': product.partner_ref,
                                'product_qty': line.product_qty,
                                'product_id': line.product_id.id,
                                'product_uom': uom_id,
                                'price_unit': price,
                                'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                                'notes': line.description,
                                'accion_especifica_id': line.accion_especifica_id.id,
                                'enlace_id': line.enlace_id,
                        }
                        
                        if product.product_tmpl_id.type == 'service':
                            purchase_order_line['name'] = line.description
                        
                        taxes_ids = line.product_id.product_tmpl_id.supplier_taxes_id
                        taxes = acc_pos_obj.map_tax(cr, uid, partner.property_account_position, taxes_ids)
                        purchase_order_line.update({
                                'taxes_id': [(6,0,taxes)]
                            })
                        list_line.append(purchase_order_line)
        
        return list_line, location_id, invoice_method, partner, partner_id, address_id, pricelist_id


    def create_purchase_merge_header(self, cr, uid, ids, list_line, location_id, invoice_method, partner, partner_id, address_id, pricelist_id, context=None):
        
        tender_obj = self.pool.get('purchase.requisition')
        order_obj = self.pool.get('purchase.order')
        order_line_obj = self.pool.get('purchase.order.line')
        
        tender_brw = tender_obj.browse(cr, uid, [ids], context=context)
        for tender in tender_brw:
            purchase_id = order_obj.create(cr, uid, {
                        'origin': tender.name,
                        'partner_id': partner_id,
                        'partner_address_id': address_id,
                        'pricelist_id': pricelist_id,
                        'location_id': tender.purchase_ids and tender.purchase_ids[0].location_id.id or line.product_id.product_tmpl_id.property_stock_production.id,
                        'company_id': tender.company_id.id,
                        'fiscal_position': partner.property_account_position and partner.property_account_position.id or False,
                        'requisition_id':tender.id,
                        'notes':tender.description,
                        'warehouse_id':tender.warehouse_id.id and tender.warehouse_id.id ,
                        'location_id':location_id,
                        'company_id':tender.company_id.id,
                        'invoice_method':invoice_method,
            }, context)
            order_ids=[]
            for order_line in list_line:
                order_line.update({
                        'order_id': purchase_id
                    })
                order_line_obj.create(cr, uid, order_line, context)
                        
        return {
            'type': 'ir.actions.act_window_close',
            'active_id': ids
        }

purchase_requisition_merge()


class sale_order_line(osv.osv):
    _name = 'sale.order.line'
    _inherit = 'sale.order.line'
    
    _columns = {
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
        'partida_presupuesto': fields.many2one('clasificador.presupuestario','Partida Presupuestaria'),
        'enlace_id':fields.integer('ID del Enlace'),
    }
        
    def invoice_line_create(self, cr, uid, ids, context=None):
        create_ids = super(sale_order_line, self).invoice_line_create(cr, uid, ids, context=context)
        """Este codigo intera sobre los sale.order.line busca por el id
        del mismo en sale_order_line_invoice_rel para encontrar el id especifico
        de account.invoice.line asociado a la linea, para luego actualizar 
        la misma y poder arrastrar la partida, la accion especifica y el
        enlace_id, se valida que los ids de account.invoice.line coincidan
        con los actualizados """ 
        ail = self.pool.get('account.invoice.line')
        update_ids = []
        for line in self.browse(cr, uid, ids, context=context):
            sql = "SELECT invoice_id FROM sale_order_line_invoice_rel WHERE order_line_id = "+str(line.id)
            cr.execute(sql)
            line_inv_id = cr.dictfetchone()
            inv_line = {
            'accion_especifica_id':line.accion_especifica_id.id,
            'partida_presupuesto':line.partida_presupuesto.id,
            'enlace_id':line.enlace_id,
            }
            res = ail.write(cr, uid, line_inv_id['invoice_id'],inv_line)
            if res:
                update_ids.append(line_inv_id['invoice_id'])
        diff_ids = set(create_ids) - set(update_ids)
        if diff_ids:
            raise osv.except_osv(_('ERROR'),_('No se pueden generar las lineas de factura, las lineas creadas no coinciden con las actualizadas!'))
        
        return create_ids


    def product_id_change(self, cr, uid, ids, pricelist, product, qty=0,
            uom=False, qty_uos=0, uos=False, name='', partner_id=False,
            lang=False, update_tax=True, date_order=False, packaging=False, fiscal_position=False, flag=False):
        
        res = super(sale_order_line, self).product_id_change(cr, uid, ids, pricelist, product, qty,
            uom, qty_uos, uos, name, partner_id,
            lang, update_tax, date_order, packaging, fiscal_position, flag)
        
        product_obj = self.pool.get('product.product')
        if product:
            partida = product_obj.browse(cr, uid, product).partida_ingreso.id
            if partida:
                res['value'].update({'partida_presupuesto':partida})
        else:
            res['value'].update({'partida_presupuesto':''})
            
        return res
        
sale_order_line()


class sale_order(osv.osv):
    
    _name = 'sale.order'
    _inherit = 'sale.order'
    
    def action_wait(self, cr, uid, ids, *args):
        """
        Se Hereda la funcion action_wait que se ejecuta una vez se confirma la 
        orden de venta.
        """
        view = super(sale_order,self).action_wait(cr, uid, ids, *args)
        
        orders = self.browse(cr, uid, ids)
        res = self.verificar_esdadiva(cr, uid, ids, 'esdadiva', 'sale.order', context=None)
        for order in orders:
            if res and order.esdadiva:
                self.imputar_dadivas(cr, uid, ids, orders, context=None)
            else:
                self.imputar_ventas(cr, uid, ids, orders, context=None)
        return view


    def action_cancel(self, cr, uid, ids, context=None):
            res = super(sale_order, self).action_cancel(cr, uid, ids, context=context)
            imp_obj =  self.pool.get("presupuesto.imputacion.linea")
            if res:
                so_brw = self.browse(cr, uid, ids, context=context)
                if so_brw[0].order_line:
                    for line in so_brw[0].order_line:
                        enlace_id = line.enlace_id
                    if enlace_id:
                        del_list =  imp_obj.search(cr, uid,[('enlace_id','=',enlace_id)])
                        imp_obj.unlink(cr, uid, del_list)
            return res


    def verificar_esdadiva(self, cr, uid, ids, nombre_campo, modelo,context=None):
         fields_obj = self.pool.get('ir.model.fields')
         field = fields_obj.search(cr, uid, [('name', '=', nombre_campo), ('model', '=', modelo),])
         if field:
             return True
         else:
             return False


    def imputar_dadivas(self, cr, uid, ids, orders, context=None):
        
        obj_uom = self.pool.get("product.uom")
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        period_obj = self.pool.get('periodo.presupuestario')
        ord_lin_obj = self.pool.get('sale.order.line')
        fecha = time.strftime('%Y-%m-%d')
        
        for order in orders:
            if order.state != 'draft':
                name_ref = order.name
                for lines in order.order_line:
                    imp_dict = {}
                    amount = lines.price_subtotal
                    impuesto = lines.tax_id
                    
                    partida = order.shop_id.partida_id and order.shop_id.partida_id.code + ' ' + order.shop_id.partida_id.name or ''
                    partida_id = order.shop_id.partida_id and order.shop_id.partida_id.id or None
                    ae_id = order.shop_id.accion_especifica_id and order.shop_id.accion_especifica_id.id or None
                    
                    if partida == '':
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria de dadivas definida para la sucursal %s.'%order.shop_id.name))
                   
                    if lines.product_id.type != 'service':
                        name_ref = order.name+' '+': [' + lines.product_id.default_code + '] ' + lines.product_id.name
                    else:
                        notes_str = lines.notes or ''
                        name_ref = ': ' + notes_str
                        
                    imp_dict = {
                        'ref': name_ref,
                        'partida': partida,
                        'partida_id': partida_id,
                        'model': 'sale.order',
                        'res_id': lines.id,
                        'fecha_imp': fecha,
                        'amount': amount,
                        'periodo': period_obj.find(cr, uid, fecha)[0],
                        'accion_especifica_id': ae_id,
                        'status': 'comprometido',
                        'state': 'activo',
                        'type': 'egreso',
                        'type_create':'automatica',
                    }

                    imp_id = imp_obj.create(cr, uid, imp_dict, context)
                    imp_obj.write(cr, uid, imp_id,{'enlace_id': imp_id}, context)
                    ord_lin_obj.write(cr, uid, lines.id,{'enlace_id': imp_id}, context)
                    enlace_id = imp_id
                                
                    if impuesto:
                        tax_amount = 0.00
                        for tax_line in lines.tax_id:
                            tax_amount += amount * tax_line.amount
                
                        if tax_amount:
                            if partida is None:
                                raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el impuesto %s.'%lines.tax_id[0].name))

                            imp_dict = {}
                            name_ref = lines.order_id.name + ': ' + lines.name
                            imp_dict = {
                                'ref': 'Impuesto de %s'%name_ref,
                                'partida': partida,
                                'partida_id':partida_id,
                                'enlace_id': enlace_id,
                                'model': 'sale.order',
                                'res_id': lines.id,
                                'fecha_imp': fecha,
                                'amount': tax_amount,
                                'periodo': period_obj.find(cr, uid, fecha)[0],
                                'accion_especifica_id': ae_id,
                                'status': 'comprometido',
                                'state': 'activo',
                                'type': 'egreso',
                                'type_create':'automatica',
                            }
                            imp_id = imp_obj.create(cr, uid, imp_dict, context)

        return True


    def imputar_ventas(self,cr, uid, ids, orders, context=None):
        
        obj_uom = self.pool.get("product.uom")
        imp_obj = self.pool.get("presupuesto.imputacion.linea")
        period_obj = self.pool.get('periodo.presupuestario')
        ord_lin_obj = self.pool.get('sale.order.line')
        fecha = time.strftime('%Y-%m-%d')
        
        for order in orders:
            if order.state != 'draft':
                name_ref = order.name
                for lines in order.order_line:
                    imp_dict = {}
                    amount = lines.price_subtotal
                    impuesto = lines.tax_id
                    ae_id = lines.accion_especifica_id and lines.accion_especifica_id.id or None
                    partida = lines.product_id.partida_ingreso and lines.product_id.partida_ingreso.code or ''
                    if partida == '':
                        raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el producto %s.'%lines.product_id.name))
                    else:
                        partida_id = lines.product_id.partida_ingreso.id
                        partida += ' '+ lines.product_id.partida_ingreso.name
                    if lines.product_id.type != 'service':
                        name_ref = order.name+' '+': [' + lines.product_id.default_code + '] ' + lines.product_id.name
                    else:
                        notes_str = lines.notes or ''
                        name_ref = ': ' + notes_str
                    imp_dict = {
                        'ref': name_ref,
                        'partida': partida,
                        'partida_id': partida_id,
                        'model': 'sale.order',
                        'res_id': lines.id,
                        'fecha_imp': fecha,
                        'amount': amount,
                        'periodo': period_obj.find(cr, uid, fecha)[0],
                        'accion_especifica_id': ae_id,
                        'status_ingreso': 'devengado',
                        'state': 'activo',
                        'type': 'ingreso',
                        'type_create':'automatica',
                    }

                    imp_id = imp_obj.create(cr, uid, imp_dict, context)
                    imp_obj.write(cr, uid, imp_id,{'enlace_id':imp_id}, context)
                    ord_lin_obj.write(cr, uid, lines.id,{'enlace_id': imp_id}, context)
                                
                    if impuesto:
                        tax_amount = 0.00
                        for tax_line in lines.tax_id:
                            tax_amount += amount * tax_line.amount
                
                        if tax_amount:
                            partida = lines.tax_id[0].partida_impuesto and lines.tax_id[0].partida_impuesto.code or None
                            if partida is None:
                                raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para el impuesto %s.'%lines.tax_id[0].name))
                            else:
                                partida_id = lines.tax_id[0].partida_impuesto.id
                                partida += ' '+lines.tax_id[0].partida_impuesto.name
                            imp_dict = {}
                            name_ref = lines.order_id.name + ': ' + lines.name
                            imp_dict = {
                                'ref': 'Impuesto de %s'%name_ref,
                                'partida': partida,
                                'partida_id':partida_id,
                                'enlace_id': lines.enlace_id,
                                'model': 'sale.order',
                                'res_id': lines.id,
                                'fecha_imp': fecha,
                                'amount': tax_amount,
                                'periodo': period_obj.find(cr, uid, fecha)[0],
                                'accion_especifica_id': ae_id,
                                'status_ingreso': 'devengado',
                                'state': 'activo',
                                'type': 'ingreso',
                                'type_create':'automatica',
                            }
                            imp_id = imp_obj.create(cr, uid, imp_dict, context)
                            if not lines.enlace_id:
                                imp_obj.write(cr, uid, imp_id, {'enlace_id': imp_id}, context)
                                ord_lin_obj.write(cr, uid, lines.id, {'enlace_id': imp_id}, context)
            return True
sale_order()
