# -*- coding: utf-8 -*-
##############################################################################
#
#    SISB, Sistema Integral Simón Bolívar
#    Copyright (C) 2011-2013 Industrias Diana, C.A. (<http://www.industriasdiana.gob.ve>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import tools
from osv import fields,osv
import decimal_precision as dp

class presupuesto_imputacion_graph(osv.osv):
    _name = "presupuesto.imputacion.graph"
    _description = "Estadisticas Imputaciones Gasto"
    _auto = False
    
    _columns = {
        'accion_central_id': fields.many2one('accion.centralizada', 'Accion Central', readonly=True),
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica', readonly=True),
        #~ 'partida_id': fields.many2one('clasificador.presupuestario', 'Partida Presupuestaria', readonly=True),
        'ejercicio_id': fields.many2one('ejercicio.presupuestario', 'Ejercicio Presupuestaria', readonly=True),
        'period_id': fields.many2one('periodo.presupuestario', 'Periodo', readonly=True),
        'amount': fields.float('Total Imputado', digits_compute=dp.get_precision('Account'), readonly=True),
        'status': fields.selection([('precomprometido', '1 PreComprometido'),
                        ('comprometido', '2 Comprometido'),
                        ('causado_gasto', '3 Causado'),
                        ('pagado', '4 Pagado')], 'Status', readonly=True),
        'company_id': fields.many2one('res.company', 'Compania', readonly=True),
    }
    _order = 'period_id desc'
    
    def init(self, cr):
        tools.drop_view_if_exists(cr, 'presupuesto_imputacion_graph')
        cr.execute("""
            create or replace view presupuesto_imputacion_graph as (
                select min(pil.id) as id,
                    ep.id as ejercicio_id,
                    pil.periodo as period_id,
                    ac.id as accion_central_id,
                    pil.accion_especifica_id,
                    pil.status,
                    sum(pil.amount) as amount,
                    pil.company_id
                from presupuesto_imputacion_linea as pil,
                    ejercicio_presupuestario as ep,
                    periodo_presupuestario as pp,
                    accion_centralizada as ac,
                    accion_especifica as ae
                where pil.accion_especifica_id = ae.id
                    and ae.accion_central_id = ac.id
                    and pil.periodo = pp.id
                    and pp.ejercicio_id = ep.id
                    and pil.type = 'egreso'
                group by pil.periodo,
                    pil.accion_especifica_id,
                    pil.status,
                    pil.company_id,
                    ep.id,
                    ac.id
            )
        """)
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """
        Herencia del search para agregar filtro que busque periodo y ejercicio de presupuesto,
        se hace verificando patron que es enviado por el filtro de 'current_period' y
        'current_year'.
        """
        if context is None:
            context = {}
        
        period_obj = self.pool.get('periodo.presupuestario')
        fiscal_obj = self.pool.get('ejercicio.presupuestario')
        if context.get('year_actual', False) and context.get('_terp_view_name', False):
            current_year = fiscal_obj.find(cr, uid)
            args.append(['ejercicio_id','=',current_year])
        for arg in args:
            if arg[0] == 'period_id' and arg[2] == 'current_period':
                current_period = period_obj.find(cr, uid)[0]
                args.append(('period_id','in',[current_period]))
                break
            elif arg[0] == 'period_id' and arg[2] == 'current_year':
                current_year = fiscal_obj.find(cr, uid)
                args.append(['ejercicio_id','=',current_year])
                break
        for a in args:
            if a in [('period_id','in','current_period'),('period_id','in','current_year')]:
                args.remove(a)
        return super(presupuesto_imputacion_graph, self).search(cr, uid, args=args, offset=offset, limit=limit, order=order,
            context=context, count=count)

presupuesto_imputacion_graph()


class presupuesto_imputacion_graph_ingreso(osv.osv):
    _name = "presupuesto.imputacion.graph.ingreso"
    _description = "Estadisticas Imputaciones Ingreso"
    _auto = False
    
    _columns = {
        'accion_central_id': fields.many2one('accion.centralizada', 'Accion Central', readonly=True),
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica', readonly=True),
        #~ 'partida_id': fields.many2one('clasificador.presupuestario', 'Partida Presupuestaria', readonly=True),
        'ejercicio_id': fields.many2one('ejercicio.presupuestario', 'Ejercicio Presupuestaria', readonly=True),
        'period_id': fields.many2one('periodo.presupuestario', 'Periodo', readonly=True),
        'amount': fields.float('Total Imputado', digits_compute=dp.get_precision('Account'), readonly=True),
        'status_ingreso': fields.selection([('devengado', '1 Devengado'),
                        ('liquidado', '2 Liquidado'),
                        ('recaudado', '3 Recaudado')], 'Status', readonly=True),
        'company_id': fields.many2one('res.company', 'Compania', readonly=True),
    }
    _order = 'period_id desc'
    
    def init(self, cr):
        tools.drop_view_if_exists(cr, 'presupuesto_imputacion_graph_ingreso')
        cr.execute("""
            create or replace view presupuesto_imputacion_graph_ingreso as (
                select min(pil.id) as id,
                    ep.id as ejercicio_id,
                    pil.periodo as period_id,
                    ac.id as accion_central_id,
                    pil.accion_especifica_id,
                    pil.status_ingreso,
                    sum(pil.amount) as amount,
                    pil.company_id
                from presupuesto_imputacion_linea as pil,
                    ejercicio_presupuestario as ep,
                    periodo_presupuestario as pp,
                    accion_centralizada as ac,
                    accion_especifica as ae
                where pil.accion_especifica_id = ae.id
                    and ae.accion_central_id = ac.id
                    and pil.periodo = pp.id
                    and pp.ejercicio_id = ep.id
                    and pil.type = 'ingreso'
                group by pil.periodo,
                    pil.accion_especifica_id,
                    pil.status_ingreso,
                    pil.company_id,
                    ep.id,
                    ac.id
            )
        """)
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """
        Herencia del search para agregar filtro que busque periodo y ejercicio de presupuesto,
        se hace verificando patron que es enviado por el filtro de 'current_period' y
        'current_year'.
        """
        if context is None:
            context = {}
        
        period_obj = self.pool.get('periodo.presupuestario')
        fiscal_obj = self.pool.get('ejercicio.presupuestario')
        if context.get('year_actual', False) and context.get('_terp_view_name', False):
            current_year = fiscal_obj.find(cr, uid)
            args.append(['ejercicio_id','=',current_year])
        for arg in args:
            if arg[0] == 'period_id' and arg[2] == 'current_period':
                current_period = period_obj.find(cr, uid)[0]
                args.append(('period_id','in',[current_period]))
                break
            elif arg[0] == 'period_id' and arg[2] == 'current_year':
                current_year = fiscal_obj.find(cr, uid)
                args.append(['ejercicio_id','=',current_year])
                break
        for a in args:
            if a in [('period_id','in','current_period'),('period_id','in','current_year')]:
                args.remove(a)
        return super(presupuesto_imputacion_graph_ingreso, self).search(cr, uid, args=args, offset=offset, limit=limit, order=order,
            context=context, count=count)

presupuesto_imputacion_graph_ingreso()
