#!/usr/bin/python
# -*- coding: utf-8 -*-
###########################################################################
#    SISB, Sistema Integral Simón Bolívar
#    Copyright (C) 2011-2013 Industrias Diana, C.A. (<http://www.industriasdiana.gob.ve>).
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################
from time import strftime
from datetime import *
import datetime
from dateutil import relativedelta
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import logging
import base64
import xlwt
from xlwt import Workbook
from xlwt import Font
from xlwt import XFStyle
from xlwt import Borders
import addons
import os
import math
import binascii

class month_financial_entry_report(osv.osv):
    _name = "month.financial.entry.report"
    _rec_name = 'period_id'
    
    _columns = {
        'partida_id': fields.many2many('clasificador.presupuestario', 'month_financial_entry_report_rel', 'month_financial_id', 'clasificador_id', 'Partida',),
        'accion_ids': fields.many2many('accion.centralizada', 'month_financial_action_report_rel', 'month_financial_id', 'accion_id', 'Acciones / Proyectos'),
        'trimestre': fields.selection([('tri_1', 'Trimestre I'), ('tri_2', 'Trimestre II'), ('tri_3', 'Trimestre III'), ('tri_4', 'Trimestre IV')], "Trimestre"),
        'ejercicio_id': fields.many2one('ejercicio.presupuestario', 'Ejercicio'),
        'period_id': fields.many2one('periodo.presupuestario', 'Periodo'),
        'report_type': fields.selection([('general', 'Mensual por partida'), ('accion', 'Mensual por Accion/Proyecto'), ('0803', 'Formulario 0803'), ('0804', 'Formulario 0804'), ('0805', 'Formulario 0805'), ('0806', 'Formulario 0806')], "Tipo", required=True),
    }
    
    def generar_xls(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        report_brw = self.browse(cr, uid, ids, context=context)
        
        if not report_brw[0].partida_id and report_brw[0].report_type != 'accion':
            raise osv.except_osv(_('Aviso !'),_('Debe agregar alguna partida para poder generar el reporte'))
        
        if report_brw[0].report_type == 'general':
            self._get_month_financial_entry_report(cr, uid, ids, context)
        elif report_brw[0].report_type == 'accion':
            self._get_month_financial_action_entry_report(cr, uid, ids, context)
        elif report_brw[0].report_type in ['0804','0806']:
            self._get_trimestral_0804_financial_entry_report(cr, uid, ids, context)
            #~ raise osv.except_osv(_('Aviso !'),_('Este reporte no esta disponible en esta version'))
        elif report_brw[0].report_type in ['0803','0805']:
            self._get_trimestral_0803_financial_entry_report(cr, uid, ids, context)
            #~ raise osv.except_osv(_('Aviso !'),_('Este reporte no esta disponible en esta version'))
        return True
    
    
    def limpiar_lista_partidas(self, cr, uid, ids, context=None):
        """
        Funcion para limpiar las partidas que se encuentran en pantalla.
        """
        if not context:
            context = {}
            
        self.write(cr, uid, ids, {'partida_id': [(6,0,[])]}, context=context)
        return True
    
    
    def _get_partidas_hijas(self, cr, uid, partida_ids, context=None):
        """
        Funcion recursiva para obtener las hijas de las partidas.
        @partida_ids: Lista de browse de las partidas presupuestarias.
        @return: Lista de browse de partidas hijas. (no se incluye la que entra
            por partida_ids).
        """
        if not context:
            context = {}
        
        padres_e_hijas = []
        for partida in partida_ids:
            padres_e_hijas += partida.child_parent_ids
            padres_e_hijas += self._get_partidas_hijas(cr, uid, partida.child_parent_ids, context=context)
        
        return padres_e_hijas
    
    
    def _get_presupuesto_montos(self, cr, uid, list_partida, total=False, context=None):
        """
        Funcion que obtener los montos de presupuesto segun los periodos que se encuentren
        por el context.
        @list_partida: lista de ids de las partidas.
        @total: boleano para armar diccionario solo con presupuesto.
        @return: diccionario de diccionario, con llave usando id de la partida y
            la informacion de ejecucion (presupuesto, comprometido, causado y
            pagado).
        """
        if not context:
            context = {}
        
        clasi_presu = self.pool.get('clasificador.presupuestario')
        
        dict_partida = {}
        if total:
            for parti in clasi_presu.browse(cr, uid, list_partida, context=context):
                lista = {
                    'presupuesto': parti.presupuesto,
                }
                
                dict_partida.update({parti.id: lista})
        else:
            for parti in clasi_presu.browse(cr, uid, list_partida, context=context):
                lista = {
                    'presupuesto': parti.presupuesto,
                    'comprometido': parti.comprometido,
                    'causado': parti.causado,
                    'pagado': parti.pagado,
                }
                
                dict_partida.update({parti.id: lista})
        
        return dict_partida
    
    
    def _get_trimestral_0803_financial_entry_report(self, cr, uid, ids, context=None):
        if not context:
            context = {}
            
        clasi_presu = self.pool.get('clasificador.presupuestario')
        period_obj = self.pool.get("periodo.presupuestario")
        company = self.pool.get('res.company')
        
        report_brw = self.browse(cr, uid, ids, context=context)
        periods_id = period_obj.search(cr, uid, [('ejercicio_id','=', report_brw[0].ejercicio_id.id)], order='date_start')
        year = str(period_obj.read(cr, uid, periods_id, ['date_start'])[0]['date_start'][:4])
        
        if report_brw[0].trimestre in ['tri_1']:
            init = 0
            end = 3
            period = "TRIMESTRE I"
            nro = "1"
        elif report_brw[0].trimestre in ['tri_2']:
            init = 3
            end = 6
            period = "TRIMESTRE II"
            nro = "2"
        elif report_brw[0].trimestre in ['tri_3']:
            init = 6
            end = 9
            period = "TRIMESTRE III"
            nro = "3"
        elif report_brw[0].trimestre in ['tri_4']:
            init = 9
            end = 12
            period = "TRIMESTRE IV"
            nro = "4"
        
        #Armar Partida de escritura
        partida_ids = report_brw[0].partida_id
        partida_seleccionadas_ids = [x.id for x in partida_ids]
        
        if report_brw[0].report_type == '0803':
            pestana = 'REPORTE 0803'
            titulo = 'CONSOLIDADO DE EJECUCION TRIMESTRAL DE INGRESOS Y FUENTES FINANCIERAS'
            archivo = 'formulario_0803'
        elif report_brw[0].report_type == '0805':
            pestana = 'REPORTE 0805'
            titulo = 'EJECUCION TRIMESTRAL DE INGRESOS Y FUENTES FINANCIERAS'
            archivo = 'formulario_0805'
            partida_ids += self._get_partidas_hijas(cr, uid, partida_ids, context=context)
        
        partida_ids = list(set(partida_ids))
        lista_partida_id = [x.id for x in partida_ids]
        partida_ids = [{'code': x.code, 'name': x.name, 'id': x.id, 'type': x.type} for x in partida_ids]
        partida_ids.sort(key=lambda x: x['code'])
        
        context.update({'periods': periods_id})
        clasi_apro = self._get_presupuesto_montos(cr, uid, lista_partida_id, True, context=context)
        context.update({'periods': periods_id[init:end]})
        clasi_ejec = self._get_presupuesto_montos(cr, uid, lista_partida_id, context=context)
        context.update({'periods': periods_id[:end]})
        clasi_acum = self._get_presupuesto_montos(cr, uid, lista_partida_id, context=context)

        company_brw = company.browse(cr, uid, self.pool.get('res.company')._company_default_get(cr, uid, 'month.financial.entry.report', context=context))
        code, name, logo = company_brw.onapre, company_brw.partner_id.name, company_brw.logo
        
        wb = xlwt.Workbook()
        ws = wb.add_sheet(pestana, cell_overwrite_ok=True)

        title = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;')
        h1 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 320; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        h2 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        numbers_total = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;", num_format_str='#,##0')
        numbers_disp = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left dashed, right thin;", num_format_str='#,##0')
        numbers = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;", num_format_str='#,##0')
        text = xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;")
        numbers_disp_bold = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left dashed, right thin;", num_format_str='#,##0')
        numbers_bold = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;", num_format_str='#,##0')
        text_bold = xlwt.easyxf('font: name Arial, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;")
        for line in report_brw:
            ws.write_merge(0, 0, 0, 1, 'CODIGO PRESUPUESTARIO DEL ENTE: ', title)
            ws.write_merge(0, 0, 2, 12, code or 'NO DEFINIDO', title)
            ws.write_merge(1, 1, 0, 1, 'DENOMINACION DEL ENTE: ', title)
            ws.write_merge(1, 1, 2, 12, name or 'NO DEFINIDO', title)
            ws.write_merge(2, 2, 0, 1, 'ORGANO ADSCRITO: ', title)
            ws.write_merge(2, 2, 2, 5, 'MINISTERIO DEL PODER POPULAR PARA LA ALIMENTACION', title)
            ws.write_merge(3, 3, 0, 1, 'EJERCICIO PRESUPUESTARIO', title)
            ws.write_merge(3, 3, 2, 12, year, title)
            ws.write(2, 7, 'PERIODO PRESUPUESTARIO: '+period, title)
            ws.write_merge(4, 4, 0, 12, '', title)
            ws.write_merge(5, 5, 0, 12, titulo, h1)
            ws.write_merge(6, 6, 0, 12, '(EN BOLIVARES)', h2)
            ws.write_merge(7, 7, 0, 12, '', title)
            ws.write_merge(8, 9, 0, 0, 'PARTIDAS', h2)
            ws.write_merge(8, 9, 1, 1, 'DENOMINACION', h2)
            ws.write_merge(8, 9, 2, 2, 'PRESUPUESTO APROBADO', h2)
            ws.write_merge(8, 9, 3, 3, 'PRESUPUESTO MODIFICADO', h2)
            ws.write_merge(8, 9, 4, 4, 'PROGRAMADO EN EL TRIMESTRE NRO: '+nro, h2)
            ws.write_merge(8, 8, 5, 7,  'EJECUTADO EN EL TRIMESTRE NRO: '+nro, h2)
            ws.write(9, 5, 'DEVENGADO', h2)
            ws.write(9, 6, 'LIQUIDADO', h2)
            ws.write(9, 7, 'RECAUDADO', h2)
            ws.write_merge(8, 8, 8, 11,  'ACUMULADO AL TRIMESTRE NRO: '+nro, h2)
            ws.write(9, 8, 'PROGRAMADO', h2)
            ws.write(9, 9, 'DEVENGADO', h2)
            ws.write(9, 10, 'LIQUIDADO', h2)
            ws.write(9, 11, 'RECAUDADO', h2)
            ws.write_merge(8, 9, 12, 12, 'INGRESOS POR RECIBIR', h2)
            sumar_vista = []
            i = 11
            for details in partida_ids:
                if (clasi_apro[details['id']]['presupuesto'] > 0) or (clasi_ejec[details['id']]['presupuesto'] > 0) or (clasi_ejec[details['id']]['comprometido'] > 0) or (clasi_ejec[details['id']]['causado'] > 0) or (clasi_ejec[details['id']]['pagado'] > 0) or (clasi_acum[details['id']]['presupuesto'] > 0) or (clasi_acum[details['id']]['comprometido'] > 0) or (clasi_acum[details['id']]['causado'] > 0) or (clasi_acum[details['id']]['pagado'] > 0):
                    if details['type'] == 'vista':
                        if details['id'] in partida_seleccionadas_ids:
                            sumar_vista.append(i+1)
                        ws.write(i,0, details['code'], text_bold)
                        ws.write(i,1, details['name'], text_bold)
                        ws.write(i,2, round(clasi_apro[details['id']]['presupuesto']) if clasi_apro[details['id']]['presupuesto'] > 0 else '', numbers_bold)
                        ws.write(i,3, '', numbers_bold)
                        ws.write(i,4, round(clasi_ejec[details['id']]['presupuesto']) if clasi_ejec[details['id']]['presupuesto'] > 0 else '', numbers_bold)
                        ws.write(i,5, round(clasi_ejec[details['id']]['comprometido']) if clasi_ejec[details['id']]['comprometido'] > 0 else '', numbers_bold)
                        ws.write(i,6, round(clasi_ejec[details['id']]['causado']) if clasi_ejec[details['id']]['causado'] > 0 else '', numbers_bold)
                        ws.write(i,7, round(clasi_ejec[details['id']]['pagado']) if clasi_ejec[details['id']]['pagado'] > 0 else '', numbers_bold)
                        ws.write(i,8, round(clasi_acum[details['id']]['presupuesto']) if clasi_acum[details['id']]['presupuesto'] > 0 else '', numbers_bold)
                        ws.write(i,9, round(clasi_acum[details['id']]['comprometido']) if clasi_acum[details['id']]['comprometido'] > 0 else '', numbers_bold)
                        ws.write(i,10, round(clasi_acum[details['id']]['causado']) if clasi_acum[details['id']]['causado'] > 0 else '', numbers_bold)
                        ws.write(i,11, round(clasi_acum[details['id']]['pagado']) if clasi_acum[details['id']]['pagado'] > 0 else '', numbers_bold)
                        ws.write(i,12, xlwt.Formula("C%s-J%s"%(str(i+1),str(i+1))), numbers_disp_bold)
                    else:
                        ws.write(i,0, details['code'], text)
                        ws.write(i,1, details['name'], text)
                        ws.write(i,2, round(clasi_apro[details['id']]['presupuesto']) if clasi_apro[details['id']]['presupuesto'] > 0 else '', numbers)
                        ws.write(i,3, '', numbers)
                        ws.write(i,4, round(clasi_ejec[details['id']]['presupuesto']) if clasi_ejec[details['id']]['presupuesto'] > 0 else '', numbers)
                        ws.write(i,5, round(clasi_ejec[details['id']]['comprometido']) if clasi_ejec[details['id']]['comprometido'] > 0 else '', numbers)
                        ws.write(i,6, round(clasi_ejec[details['id']]['causado']) if clasi_ejec[details['id']]['causado'] > 0 else '', numbers)
                        ws.write(i,7, round(clasi_ejec[details['id']]['pagado']) if clasi_ejec[details['id']]['pagado'] > 0 else '', numbers)
                        ws.write(i,8, round(clasi_acum[details['id']]['presupuesto']) if clasi_acum[details['id']]['presupuesto'] > 0 else '', numbers)
                        ws.write(i,9, round(clasi_acum[details['id']]['comprometido']) if clasi_acum[details['id']]['comprometido'] > 0 else '', numbers)
                        ws.write(i,10, round(clasi_acum[details['id']]['causado']) if clasi_acum[details['id']]['causado'] > 0 else '', numbers)
                        ws.write(i,11, round(clasi_acum[details['id']]['pagado']) if clasi_acum[details['id']]['pagado'] > 0 else '', numbers)
                        ws.write(i,12, xlwt.Formula("C%s-J%s"%(str(i+1),str(i+1))), numbers_disp)
                    i += 1
                    
            ws.write(i+1, 0, '', xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;"))
            ws.write(i+1, 1, 'TOTAL', xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;"))
            ws.write(i+1, 2, xlwt.Formula('+'.join('C%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 3, xlwt.Formula('+'.join('D%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 4, xlwt.Formula('+'.join('E%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 5, xlwt.Formula('+'.join('F%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 6, xlwt.Formula('+'.join('G%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 7, xlwt.Formula('+'.join('H%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 8, xlwt.Formula('+'.join('I%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 9, xlwt.Formula('+'.join('J%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 10, xlwt.Formula('+'.join('K%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 11, xlwt.Formula('+'.join('L%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 12, xlwt.Formula("C%s-J%s"%(str(i+2),str(i+2))), numbers_total)
        wb.save('/tmp/reporte_trimestral_ingresos.xls')
        xls = open('/tmp/reporte_trimestral_ingresos.xls')
        return self._write_attachment_xls(cr, uid, ids[0], xls, archivo, context)
    
    
    def _get_trimestral_0804_financial_entry_report(self, cr, uid, ids, context=None):
        if not context:
            context = {}

        clasi_presu = self.pool.get('clasificador.presupuestario')
        period_obj = self.pool.get("periodo.presupuestario")
        company = self.pool.get('res.company')
        
        report_brw = self.browse(cr, uid, ids, context=context)
        periods_id = period_obj.search(cr, uid, [('ejercicio_id','=', report_brw[0].ejercicio_id.id)], order='date_start')
        year = str(period_obj.read(cr, uid, periods_id, ['date_start'])[0]['date_start'][:4])
        
        if report_brw[0].trimestre in ['tri_1']:
            init = 0
            end = 3
            period = "TRIMESTRE I"
            nro = "1"
        elif report_brw[0].trimestre in ['tri_2']:
            init = 3
            end = 6
            period = "TRIMESTRE II"
            nro = "2"
        elif report_brw[0].trimestre in ['tri_3']:
            init = 6
            end = 9
            period = "TRIMESTRE III"
            nro = "3"
        elif report_brw[0].trimestre in ['tri_4']:
            init = 9
            end = 12
            period = "TRIMESTRE IV"
            nro = "4"
        
        #Armar Partida de escritura
        partida_ids = report_brw[0].partida_id
        partida_seleccionadas_ids = [x.id for x in partida_ids]
        
        if report_brw[0].report_type == '0804':
            pestana = 'REPORTE 0804'
            titulo = 'CONSOLIDADO DE EJECUCION FINANCIERA TRIMESTRAL DE PROYECTO Y ACCION CENTRALIZADA POR PARTIDA'
            archivo = 'formulario_0804'
        elif report_brw[0].report_type == '0806':
            pestana = 'REPORTE 0806'
            titulo = 'EJECUCION TRIMESTRAL DE GASTOS Y APLICACIONES FINANCIERAS'
            archivo = 'formulario_0806'
            partida_ids += self._get_partidas_hijas(cr, uid, partida_ids, context=context)
        
        partida_ids = list(set(partida_ids))
        lista_partida_id = [x.id for x in partida_ids]
        partida_ids = [{'code': x.code, 'name': x.name, 'id': x.id, 'type': x.type} for x in partida_ids]
        partida_ids.sort(key=lambda x: x['code'])
        
        context.update({'periods': periods_id})
        clasi_apro = self._get_presupuesto_montos(cr, uid, lista_partida_id, True, context=context)
        context.update({'periods': periods_id[init:end]})
        clasi_ejec = self._get_presupuesto_montos(cr, uid, lista_partida_id, context=context)
        context.update({'periods': periods_id[:end]})
        clasi_acum = self._get_presupuesto_montos(cr, uid, lista_partida_id, context=context)
        
        company_brw = company.browse(cr, uid, self.pool.get('res.company')._company_default_get(cr, uid, 'month.financial.entry.report', context=context))
        code, name, logo = company_brw.onapre, company_brw.partner_id.name, company_brw.logo
        
        wb = xlwt.Workbook()
        ws = wb.add_sheet(pestana, cell_overwrite_ok=True)
        title = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;')
        h1 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 320; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        h2 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        numbers_total = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;", num_format_str='#,##0')
        numbers_disp = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left dashed, right thin;", num_format_str='#,##0')
        numbers = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;", num_format_str='#,##0')
        text = xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;")
        numbers_disp_bold = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left dashed, right thin;", num_format_str='#,##0')
        numbers_bold = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;", num_format_str='#,##0')
        text_bold = xlwt.easyxf('font: name Arial, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;")
        for line in report_brw:
            ws.write_merge(0, 0, 0, 1, 'CODIGO PRESUPUESTARIO DEL ENTE: ', title)
            ws.write_merge(0, 0, 2, 12, code or 'NO DEFINIDO', title)
            ws.write_merge(1, 1, 0, 1, 'DENOMINACION DEL ENTE: ', title)
            ws.write_merge(1, 1, 2, 12, name or 'NO DEFINIDO', title)
            ws.write_merge(2, 2, 0, 1, 'ORGANO ADSCRITO: ', title)
            ws.write_merge(2, 2, 2, 5, 'MINISTERIO DEL PODER POPULAR PARA LA ALIMENTACION', title)
            ws.write_merge(3, 3, 0, 1, 'EJERCICIO PRESUPUESTARIO', title)
            ws.write_merge(3, 3, 2, 12, year, title)
            ws.write(2, 7, 'PERIODO PRESUPUESTARIO: '+period, title)
            ws.write_merge(4, 4, 0, 12, '', title)
            ws.write_merge(5, 5, 0, 12, titulo, h1)
            ws.write_merge(6, 6, 0, 12, '(EN BOLIVARES)', h2)
            ws.write_merge(7, 7, 0, 12, '', title)
            ws.write_merge(8, 9, 0, 0, 'PARTIDAS', h2)
            ws.write_merge(8, 9, 1, 1, 'DENOMINACION', h2)
            ws.write_merge(8, 9, 2, 2, 'PRESUPUESTO APROBADO', h2)
            ws.write_merge(8, 9, 3, 3, 'PRESUPUESTO MODIFICADO', h2)
            ws.write_merge(8, 9, 4, 4, 'PROGRAMADO EN EL TRIMESTRE NRO: '+nro, h2)
            ws.write_merge(8, 8, 5, 7,  'EJECUTADO EN EL TRIMESTRE NRO: '+nro, h2)
            ws.write(9, 5, 'COMPROMISO', h2)
            ws.write(9, 6, 'CAUSADO', h2)
            ws.write(9, 7, 'PAGADO', h2)
            ws.write_merge(8, 8, 8, 11,  'ACUMULADO AL TRIMESTRE NRO: '+nro, h2)
            ws.write(9, 8, 'PROGRAMADO', h2)
            ws.write(9, 9, 'COMPROMISO', h2)
            ws.write(9, 10, 'CAUSADO', h2)
            ws.write(9, 11, 'PAGADO', h2)
            ws.write_merge(8, 9, 12, 12, 'DISPONIBILIDAD PRESUPUESTARIA', h2)
            sumar_vista = []
            i = 11
            for details in partida_ids:
                if (clasi_apro[details['id']]['presupuesto'] > 0) or (clasi_ejec[details['id']]['presupuesto'] > 0) or (clasi_ejec[details['id']]['comprometido'] > 0) or (clasi_ejec[details['id']]['causado'] > 0) or (clasi_ejec[details['id']]['pagado'] > 0) or (clasi_acum[details['id']]['presupuesto'] > 0) or (clasi_acum[details['id']]['comprometido'] > 0) or (clasi_acum[details['id']]['causado'] > 0) or (clasi_acum[details['id']]['pagado'] > 0):
                    if details['type'] == 'vista':
                        if details['id'] in partida_seleccionadas_ids:
                            sumar_vista.append(i+1)
                        ws.write(i,0, details['code'], text_bold)
                        ws.write(i,1, details['name'], text_bold)
                        ws.write(i,2, round(clasi_apro[details['id']]['presupuesto']) if clasi_apro[details['id']]['presupuesto'] > 0 else '', numbers_bold)
                        ws.write(i,3, '', numbers_bold)
                        ws.write(i,4, round(clasi_ejec[details['id']]['presupuesto']) if clasi_ejec[details['id']]['presupuesto'] > 0 else '', numbers_bold)
                        ws.write(i,5, round(clasi_ejec[details['id']]['comprometido']) if clasi_ejec[details['id']]['comprometido'] > 0 else '', numbers_bold)
                        ws.write(i,6, round(clasi_ejec[details['id']]['causado']) if clasi_ejec[details['id']]['causado'] > 0 else '', numbers_bold)
                        ws.write(i,7, round(clasi_ejec[details['id']]['pagado']) if clasi_ejec[details['id']]['pagado'] > 0 else '', numbers_bold)
                        ws.write(i,8, round(clasi_acum[details['id']]['presupuesto']) if clasi_acum[details['id']]['presupuesto'] > 0 else '', numbers_bold)
                        ws.write(i,9, round(clasi_acum[details['id']]['comprometido']) if clasi_acum[details['id']]['comprometido'] > 0 else '', numbers_bold)
                        ws.write(i,10, round(clasi_acum[details['id']]['causado']) if clasi_acum[details['id']]['causado'] > 0 else '', numbers_bold)
                        ws.write(i,11, round(clasi_acum[details['id']]['pagado']) if clasi_acum[details['id']]['pagado'] > 0 else '', numbers_bold)
                        ws.write(i,12, xlwt.Formula("C%s-J%s"%(str(i+1),str(i+1))), numbers_disp_bold)
                    else:
                        ws.write(i,0, details['code'], text)
                        ws.write(i,1, details['name'], text)
                        ws.write(i,2, round(clasi_apro[details['id']]['presupuesto']) if clasi_apro[details['id']]['presupuesto'] > 0 else '', numbers)
                        ws.write(i,3, '', numbers)
                        ws.write(i,4, round(clasi_ejec[details['id']]['presupuesto']) if clasi_ejec[details['id']]['presupuesto'] > 0 else '', numbers)
                        ws.write(i,5, round(clasi_ejec[details['id']]['comprometido']) if clasi_ejec[details['id']]['comprometido'] > 0 else '', numbers)
                        ws.write(i,6, round(clasi_ejec[details['id']]['causado']) if clasi_ejec[details['id']]['causado'] > 0 else '', numbers)
                        ws.write(i,7, round(clasi_ejec[details['id']]['pagado']) if clasi_ejec[details['id']]['pagado'] > 0 else '', numbers)
                        ws.write(i,8, round(clasi_acum[details['id']]['presupuesto']) if clasi_acum[details['id']]['presupuesto'] > 0 else '', numbers)
                        ws.write(i,9, round(clasi_acum[details['id']]['comprometido']) if clasi_acum[details['id']]['comprometido'] > 0 else '', numbers)
                        ws.write(i,10, round(clasi_acum[details['id']]['causado']) if clasi_acum[details['id']]['causado'] > 0 else '', numbers)
                        ws.write(i,11, round(clasi_acum[details['id']]['pagado']) if clasi_acum[details['id']]['pagado'] > 0 else '', numbers)
                        ws.write(i,12, xlwt.Formula("C%s-J%s"%(str(i+1),str(i+1))), numbers_disp)
                    i += 1

            ws.write(i+1, 0, '', xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;"))
            ws.write(i+1, 1, 'TOTAL', xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;"))
            ws.write(i+1, 2, xlwt.Formula('+'.join('C%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 3, xlwt.Formula('+'.join('D%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 4, xlwt.Formula('+'.join('E%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 5, xlwt.Formula('+'.join('F%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 6, xlwt.Formula('+'.join('G%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 7, xlwt.Formula('+'.join('H%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 8, xlwt.Formula('+'.join('I%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 9, xlwt.Formula('+'.join('J%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 10, xlwt.Formula('+'.join('K%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 11, xlwt.Formula('+'.join('L%s'% x for x in sumar_vista)), numbers_total)
            ws.write(i+1, 12, xlwt.Formula("C%s-J%s"%(str(i+2),str(i+2))), numbers_total)
        wb.save('/tmp/reporte_trimestral.xls')
        xls = open('/tmp/reporte_trimestral.xls')
        return self._write_attachment_xls(cr, uid, ids[0], xls, archivo, context)

    
    def _get_month_financial_entry_report(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        report_brw = self.browse(cr, uid, ids, context=context)
        period_obj = self.pool.get("periodo.presupuestario")
        year = str(period_obj.read(cr, uid, [report_brw[0].period_id.id], ['date_start'])[0]['date_start'][:4])
        
        #Armar Partida de escritura
        partida_ids = report_brw[0].partida_id
        partida_seleccionadas_ids = [x.id for x in partida_ids]
        
        partida_ids = list(set(partida_ids))
        lista_partida_id = [x.id for x in partida_ids]
        partida_ids = [{'code': x.code, 'name': x.name, 'id': x.id, 'type': x.type} for x in partida_ids]
        partida_ids.sort(key=lambda x: x['code'])
        
        period_ids = period_obj.search(cr, uid, [('ejercicio_id','=',report_brw[0].period_id.ejercicio_id.id)],order="date_start")
        context.update({'periods':period_ids})
        clasi_pro = self._get_presupuesto_montos(cr, uid, lista_partida_id, True, context=context)
        period_ant_ids = period_obj.search(cr, uid, [('ejercicio_id','=',report_brw[0].period_id.ejercicio_id.id), ('date_start', '<=', report_brw[0].period_id.date_start)],order="date_start")
        context.update({'periods': period_ant_ids})
        #~ context.update({'periods': [report_brw[0].period_id.id]})
        clasi = self._get_presupuesto_montos(cr, uid, lista_partida_id, context=context)
        
        
        company = self.pool.get('res.company')
        company_brw = company.browse(cr, uid, self.pool.get('res.company')._company_default_get(cr, uid, 'month.financial.entry.report', context=context))
        code, name, logo = company_brw.onapre, company_brw.partner_id.name, company_brw.logo
        wb = xlwt.Workbook()
        ws = wb.add_sheet('PRESUPUESTO FINANCIERO',cell_overwrite_ok=True)
        title = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;')
        h1 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 320; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        h2 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        numbers_total = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;", num_format_str='#,##0')
        numbers_disp = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left dashed, right thin;", num_format_str='#,##0')
        numbers = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;", num_format_str='#,##0')
        text = xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;")
            
        for line in report_brw: 
            ws.write_merge(0, 0, 0, 1, 'CODIGO PRESUPUESTARIO DEL ENTE: ', title)
            ws.write_merge(0, 0, 2, 7, code or 'NO DEFINIDO', title)
            ws.write_merge(1, 1, 0, 1, 'DENOMINACION DE LA EMPRESA: ', title)
            ws.write_merge(1, 1, 2, 7, name or 'NO DEFINIDO', title)
            ws.write_merge(2, 2, 0, 1, 'ORGANO ADSCRITO: ', title)
            ws.write_merge(2, 2, 2, 5, 'MINISTERIO DEL PODER POPULAR PARA LA ALIMENTACION', title)
            ws.write_merge(3, 3, 0, 1, 'PERIODO PRESUPUESTARIO', title)
            ws.write_merge(3, 3, 2, 7, year, title)
            ws.write(2, 7, 'MES: '+line.period_id.name, title)
            ws.write_merge(4, 4, 0, 7, '', title)
            ws.write_merge(5, 5, 0, 7, 'EJECUCION FINANCIERA MENSUAL DE LOS ENTES ADSCRITOS POR PARTIDAS', h1)
            ws.write_merge(6, 6, 0, 7, '(EN BOLIVARES)', h2)
            ws.write_merge(7, 7, 0, 7, '', title)
            ws.write_merge(8, 9, 0, 0, 'PARTIDAS', h2)
            ws.write_merge(8, 9, 1, 1, 'DENOMINACION', h2)
            ws.write_merge(8, 9, 2, 2, 'PRESUPUESTO APROBADO', h2)
            ws.write_merge(8, 9, 3, 3, 'PRESUPUESTO MODIFICADO', h2)
            ws.write_merge(8, 8, 4, 6,  'EJECUTADO MENSUAL', h2)
            ws.write(9, 4, 'COMPROMISO', h2)
            ws.write(9, 5, 'CAUSADO', h2)
            ws.write(9, 6, 'PAGADO', h2)
            ws.write_merge(8, 9, 7, 7, 'DISPONIBILIDAD PRESUPUESTARIA', h2)
            i = 11
            for details in partida_ids:
                ws.write(i,0, details['code'], text)
                ws.write(i,1, details['name'], text)
                ws.write(i,2, round(clasi_pro[details['id']]['presupuesto']) if clasi_pro[details['id']]['presupuesto'] > 0 else '', numbers)
                ws.write(i,3, '', numbers)
                ws.write(i,4, round(clasi[details['id']]['comprometido']) if clasi[details['id']]['comprometido'] > 0 else '', numbers)
                ws.write(i,5, round(clasi[details['id']]['causado']) if clasi[details['id']]['causado'] > 0 else '', numbers)
                ws.write(i,6, round(clasi[details['id']]['pagado']) if clasi[details['id']]['pagado'] > 0 else '', numbers)
                ws.write(i,7, xlwt.Formula("C%s-E%s"%(str(i+1),str(i+1))), numbers_disp)
                i += 1
                
            ws.write(i+1, 0, '', xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;"))
            ws.write(i+1, 1, 'TOTAL', xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;"))
            ws.write(i+1, 2, xlwt.Formula("SUM(C12:C%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 3, xlwt.Formula("SUM(D12:D%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 4, xlwt.Formula("SUM(E12:E%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 5, xlwt.Formula("SUM(F12:F%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 6, xlwt.Formula("SUM(G12:G%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 7, xlwt.Formula("C%s-E%s)"%(str(i+2),str(i+2))), numbers_total)
        archivo = 'ejecucion_financiera_mensual'
        wb.save('/tmp/ppto_partida.xls')
        xls = open('/tmp/ppto_partida.xls')
        return self._write_attachment_xls(cr, uid, ids[0], xls, archivo, context)
    
    
    def _get_accion_central_mensual(self, cr, uid, report_brw, context=None):
        cls_obj = self.pool.get('clasificador.presupuestario')
        period_obj = self.pool.get("periodo.presupuestario")
        
        report_brw = report_brw[0]
        action_central_ids = [x.child_ids for x in report_brw.accion_ids if x.type == 'centralizada']
        action_central_ids = [z.id for z in reduce(lambda x,y:x+y,action_central_ids)]

        period_ids = period_obj.search(cr, uid, [('ejercicio_id','=',report_brw.period_id.ejercicio_id.id)],order="date_start")
        period_id = period_obj.search(cr, uid, [('ejercicio_id','=',report_brw.period_id.ejercicio_id.id), ('date_start', '<=', report_brw.period_id.date_start)],order="date_start")
        #~ period_id = [report_brw.period_id.id]
        
        dict_acc = [{'name': 'Direccion y coordinacion de los gastos de los trabajadores'},
            {'name': 'Gestion Administrativa'},
            {'name': 'Prevencion y Proteccion Social'},
            {'name': 'Proteccion y Atencion Integral a las Familias y Personas en Refugios en Casos de Emergencias o Desatres', 'presupuesto': 0, 'comprometido': 0, 'causado': 0, 'pagado': 0},
        ]
        
        search_id = cls_obj.search(cr, uid, [('code','in',['401000000','402000000','403000000','404000000', '405000000','408000000','411000000','407010199','407010300','407010201','407010102','407010103','407010104','407010105','407010107','407010113'])])
        context.update({'periods': period_ids, 'accion_especifica_id': action_central_ids,})
        pres_cla = cls_obj.browse(cr, uid, search_id, context)
        
        for partida in pres_cla:
            if partida.code in ['401000000']:
                if not dict_acc[0].get('presupuesto', False):
                    dict_acc[0]['presupuesto'] = 0
                dict_acc[0]['presupuesto'] = dict_acc[0]['presupuesto'] + round(partida.presupuesto)
                dict_acc[0]['comprometido'] = 0
                dict_acc[0]['causado'] = 0
                dict_acc[0]['pagado'] = 0
                    
            if partida.code in ['402000000','403000000','404000000', '405000000','408000000','411000000','407010199','407010300','407010201']:
                if not dict_acc[1].get('presupuesto', False):
                    dict_acc[1]['presupuesto'] = 0
                dict_acc[1]['presupuesto'] = dict_acc[1]['presupuesto'] + round(partida.presupuesto)
                dict_acc[1]['comprometido'] = 0
                dict_acc[1]['causado'] = 0
                dict_acc[1]['pagado'] = 0
            
            if partida.code in ['407010102','407010103','407010104','407010105','407010107','407010113']:
                if not dict_acc[2].get('presupuesto', False):
                    dict_acc[2]['presupuesto'] = 0
                dict_acc[2]['presupuesto'] = dict_acc[2]['presupuesto'] + round(partida.presupuesto)
                dict_acc[2]['comprometido'] = 0
                dict_acc[2]['causado'] = 0
                dict_acc[2]['pagado'] = 0
        
        context.update({'periods': period_id, 'accion_especifica_id': action_central_ids,})
        acc_data = cls_obj.browse(cr, uid, search_id, context)
        
        for partida in acc_data:
            if partida.code in ['401000000']:
                dict_acc[0]['comprometido'] = dict_acc[0]['comprometido'] + partida.comprometido
                dict_acc[0]['causado'] = dict_acc[0]['causado'] + partida.causado
                dict_acc[0]['pagado'] = dict_acc[0]['pagado'] + partida.pagado
            
            if partida.code in ['402000000', '403000000', '404000000', '405000000', '408000000', '411000000', '407010199', '407010300', '407010201']:
                dict_acc[1]['comprometido'] = dict_acc[1]['comprometido'] + partida.comprometido
                dict_acc[1]['causado'] = dict_acc[1]['causado'] + partida.causado
                dict_acc[1]['pagado'] = dict_acc[1]['pagado'] + partida.pagado
            
            if partida.code in ['407010102','407010103','407010104','407010105','407010107','407010113']:
                dict_acc[2]['comprometido'] = dict_acc[2]['comprometido'] + partida.comprometido
                dict_acc[2]['causado'] = dict_acc[2]['causado'] + partida.causado
                dict_acc[2]['pagado'] = dict_acc[2]['pagado'] + partida.pagado
            
        return dict_acc
    
    
    def _get_proyecto_mensual(self, cr, uid, report_brw, context=None):
        cls_obj = self.pool.get('clasificador.presupuestario')
        period_obj = self.pool.get("periodo.presupuestario")
        
        report_brw = report_brw[0]
        action_central_ids = [x.child_ids for x in report_brw.accion_ids if x.type == 'proyecto']
        action_central_ids = [z.id for z in reduce(lambda x,y:x+y,action_central_ids)]
        
        period_ids = period_obj.search(cr, uid, [('ejercicio_id','=',report_brw.period_id.ejercicio_id.id)],order="date_start")
        period_id = period_obj.search(cr, uid, [('ejercicio_id','=',report_brw.period_id.ejercicio_id.id), ('date_start', '<=', report_brw.period_id.date_start)],order="date_start")
        #~ period_id = [report_brw.period_id.id]
        
        
        search_id = cls_obj.search(cr, uid, [('code', '=', '0')])
        search_id = cls_obj.search(cr, uid, [('parent_id', '=', search_id[0]), ('code', 'ilike', '4')])
        
        list_proy = []
        for proy in report_brw.accion_ids:
            if proy.type == 'proyecto':
                dict_proy = {
                    'name': proy.name,
                    'presupuesto': 0,
                    'comprometido': 0,
                    'causado': 0,
                    'pagado': 0,
                }
                
                accions = [x.id for x in proy.child_ids]
                context.update({'periods': period_ids, 'accion_especifica_id': accions})
                pres_data = cls_obj.browse(cr, uid, search_id, context)
                for pres in pres_data:
                    dict_proy['presupuesto'] = dict_proy['presupuesto'] + pres.presupuesto
                
                context.update({'periods': period_id})
                acc_data = cls_obj.browse(cr, uid, search_id, context)
                for status_data in acc_data:
                    dict_proy['comprometido'] = dict_proy['comprometido'] + status_data.comprometido
                    dict_proy['causado'] = dict_proy['causado'] + status_data.causado
                    dict_proy['pagado'] = dict_proy['pagado'] + status_data.pagado
                list_proy.append(dict_proy)

        return list_proy
    
    
    def _get_month_financial_action_entry_report(self, cr, uid, ids, context=None):
        if not context:
            context = {}
        cls_obj = self.pool.get('clasificador.presupuestario')
        period_obj = self.pool.get("periodo.presupuestario")
        
        report_brw = self.browse(cr, uid, ids, context=context)
        year = str(period_obj.read(cr, uid, [report_brw[0].period_id.id], ['date_start'])[0]['date_start'][:4])
        action_central_ids = [x.child_ids for x in report_brw[0].accion_ids if x.type == 'centralizada']
        action_central_ids = [z.id for z in reduce(lambda x,y:x+y,action_central_ids)]
        
        company = self.pool.get('res.company')
        company_brw = company.browse(cr, uid, self.pool.get('res.company')._company_default_get(cr, uid, 'month.financial.entry.report', context=context))
        code, name, logo = company_brw.onapre, company_brw.partner_id.name, company_brw.logo
        
        wb = xlwt.Workbook()
        ws = wb.add_sheet('PRESUPUESTO FINANCIERO',cell_overwrite_ok=True)
        title = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;')
        h1 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 320; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        h2 = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 200; align: wrap on, vert centre, horiz center; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;")
        numbers_total = xlwt.easyxf('font: name Times New Roman, colour black, bold on, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left thin, right thin;", num_format_str='#,##0')
        numbers_disp = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top thin, bottom thin, left dashed, right thin;", num_format_str='#,##0')
        numbers = xlwt.easyxf('font: name Times New Roman, colour black, height 160; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;", num_format_str='#,##0')
        text = xlwt.easyxf('font: name Arial, colour black, height 200; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;")
        text_bold = xlwt.easyxf('font: name Arial, colour black, bold on, height 200; pattern: pattern solid, fore_colour white;' "borders: top dashed, bottom dashed, left dashed, right dashed;")
            
        for line in report_brw: 
            ws.write_merge(0, 0, 0, 1, 'CODIGO PRESUPUESTARIO DEL ENTE: ', title)
            ws.write_merge(0, 0, 2, 7, code or 'NO DEFINIDO', title)
            ws.write_merge(1, 1, 0, 1, 'DENOMINACION DE LA EMPRESA: ', title)
            ws.write_merge(1, 1, 2, 7, name or 'NO DEFINIDO', title)
            ws.write_merge(2, 2, 0, 1, 'ORGANO ADSCRITO: ', title)
            ws.write_merge(2, 2, 2, 5, 'MINISTERIO DEL PODER POPULAR PARA LA ALIMENTACION', title)
            ws.write_merge(3, 3, 0, 1, 'PERIODO PRESUPUESTARIO', title)
            ws.write_merge(3, 3, 2, 7, year, title)
            ws.write(2, 7, 'MES: '+line.period_id.name, title)
            ws.write_merge(4, 4, 0, 7, '', title)
            ws.write_merge(5, 5, 0, 7, 'EJECUCION FINANCIERA MENSUAL DE LOS ENTES ADSCRITOS POR ESTRUCTURA PRESUPUESTARIA', h1)
            ws.write_merge(6, 6, 0, 7, '(EN BOLIVARES)', h2)
            ws.write_merge(7, 7, 0, 7, '', title)
            ws.write_merge(8, 9, 0, 0, 'CODIGO', h2)
            ws.write_merge(8, 9, 1, 1, 'ESTRUCTURA', h2)
            ws.write_merge(8, 9, 2, 2, 'PRESUPUESTO APROBADO', h2)
            ws.write_merge(8, 9, 3, 3, 'PRESUPUESTO MODIFICADO', h2)
            ws.write_merge(8, 8, 4, 6,  'EJECUTADO MENSUAL', h2)
            ws.write(9, 4, 'COMPROMISO', h2)
            ws.write(9, 5, 'CAUSADO', h2)
            ws.write(9, 6, 'PAGADO', h2)
            ws.write_merge(8, 9, 7, 7, 'DISPONIBILIDAD PRESUPUESTARIA', h2)
            ws.write(11, 1, 'ACCIONES CENTRALIZADAS', text_bold)
            i = 12
            #~ group = []
            acc_lis = self._get_accion_central_mensual(cr, uid, report_brw, context)
            for acc in acc_lis:
                ws.write(i,0, '', text)
                ws.write(i,1, acc['name'], text)
                ws.write(i,2, round(acc['presupuesto']) if acc['presupuesto'] > 0 else '', numbers)
                ws.write(i,3, '', numbers)
                ws.write(i,4, round(acc['comprometido']) if acc['comprometido'] > 0 else '', numbers)
                ws.write(i,5, round(acc['causado']) if acc['causado'] > 0 else '', numbers)
                ws.write(i,6, round(acc['pagado']) if acc['pagado'] > 0 else '', numbers)
                ws.write(i,7, xlwt.Formula("C%s-E%s"%(str(i+1),str(i+1))), numbers_disp)
                i += 1
                
            ws.write(i, 1, 'PROYECTOS', text_bold)
            i += 1
            proy_lis = self._get_proyecto_mensual(cr, uid, report_brw, context)
            for proy in proy_lis:
                ws.write(i,0, '', text)
                ws.write(i,1, proy['name'], text)
                ws.write(i,2, round(proy['presupuesto']) if proy['presupuesto'] > 0 else '', numbers)
                ws.write(i,3, '', numbers)
                ws.write(i,4, round(proy['comprometido']) if proy['comprometido'] > 0 else '', numbers)
                ws.write(i,5, round(proy['causado']) if proy['causado'] > 0 else '', numbers)
                ws.write(i,6, round(proy['pagado']) if proy['pagado'] > 0 else '', numbers)
                ws.write(i,7, xlwt.Formula("C%s-E%s"%(str(i+1),str(i+1))), numbers_disp)
                i += 1
            ws.write(i+1, 0, '', text_bold)
            ws.write(i+1, 1, 'TOTAL', text_bold)
            ws.write(i+1, 2, xlwt.Formula("SUM(C12:C%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 3, xlwt.Formula("SUM(D12:D%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 4, xlwt.Formula("SUM(E12:E%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 5, xlwt.Formula("SUM(F12:F%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 6, xlwt.Formula("SUM(G12:G%s)"%str(i+1)), numbers_total)
            ws.write(i+1, 7, xlwt.Formula("C%s-E%s"%(str(i+2),str(i+2))), numbers_total)
        archivo = "ejecucion_financiera_por_partida"
        wb.save('/tmp/ppto_accion_proyecto.xls')
        xls = open('/tmp/ppto_accion_proyecto.xls')
        return self._write_attachment_xls(cr, uid, ids[0], xls, archivo, context)
    
    
    def _write_attachment_xls(self, cr, uid, ids, root, name, context=None):
        '''
        Encrypt xls, save it to the db and view it on the client as an attachment
        '''

        fecha = datetime.datetime.now().strftime('%Y-%m-%d')
        name = name + ' - ' + fecha + '.xls'
        id_att = self.pool.get('ir.attachment').create(cr, uid, {
            'name': name,
            'voucher_support': name,
            'res_name': name,
            'datas': base64.encodestring(root.read()),
            'datas_fname': name,
            'res_model': 'month.financial.entry.report',
            'res_id': str(ids),
            }, context=context)
        return id_att
        
month_financial_entry_report()
