# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv

class presupuesto_clasificador_list(osv.osv_memory):
    """
    Wizard para seleccionar la informacion del clasificador presupuestario
    """
    _name = "presupuesto.clasificador.list"
    _description = "Presupuesto Clasificador Lista"
    
    _columns = {
        'clasificador': fields.many2one('clasificador.presupuestario', 'Clasificador Presupuestario', required=True),
        'fiscalyear': fields.many2one('ejercicio.presupuestario', 'Ejercicio Presupuestario',
                                    help = 'Keep empty for all open fiscal years'),
        'periods': fields.many2many('periodo.presupuestario', 'clasificador_periodo_wizard', 'pre_cla_id', 'period_id', 'Periodos Presupuestarios'),
        'target_move': fields.selection([('sin_period', 'No Usar Periodos'),
                                         ('con_period', 'Usar Periodos'),
                                        ], 'Periodos', required = True),
        'accion_especifica_id': fields.many2one('accion.especifica', 'Accion Especifica'),
    }
    
    
    def presupuesto_armar_vista(self, cr, uid, ids, context=None):
        mod_obj = self.pool.get('ir.model.data')
        act_obj = self.pool.get('ir.actions.act_window')
        fy_obj = self.pool.get('ejercicio.presupuestario')
        ae_obj = self.pool.get('accion.especifica')
        if context is None:
            context = {}
        data = self.read(cr, uid, ids, [], context=context)[0]
        result = mod_obj.get_object_reference(cr, uid, 'l10n_ve_presupuesto', 'clasificador_list_action')
        id = result and result[1] or False
        result = act_obj.read(cr, uid, [id], context=context)[0]
        
        result['context'] = {}
        if data['target_move'] == 'con_period':
            result['context']['periods'] = list(set(data['periods']))
        
        if data['accion_especifica_id']:
            result['name'] += ':' + ae_obj.read(cr, uid, [data['accion_especifica_id']], context=context)[0]['name']
            result['context']['accion_especifica_id'] = [data['accion_especifica_id']]
            
        if data['fiscalyear']:
            result['name'] += ':' + fy_obj.read(cr, uid, [data['fiscalyear']], context=context)[0]['code']
            result['context']['periods'] = list(set([i.id for i in fy_obj.browse(cr, uid, data['fiscalyear']).period_ids]))
            
        return result


    _defaults = {
        'target_move': 'sin_period',
        'clasificador': lambda self, cr, uid, c: self.pool.get('clasificador.presupuestario').search(cr, uid, [('parent_id','=',False)])[0],
    }

presupuesto_clasificador_list()
