# -*- encoding: utf-8 -*-
##############################################################################
# Copyright (c) 2011 OpenERP Venezuela (http://openerp.com.ve)
# All Rights Reserved.
# Programmed by: Israel Fermín Montilla  <israel@openerp.com.ve>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################
from osv import osv
from osv import fields
import sys
from tools.translate import _
import decimal_precision as dp
import netsvc

class sisb_account_invoice_wizard(osv.osv_memory):
    """
    Calcular ISLR en las facturas menores o iguales a 395
    """
    _name = "sisb.account.invoice.wizard"
    
    _description = "Invoice ISLR Calc"
    _columns = {
       'amount_untaxed_islr': fields.float('Base Price', required=True, digits_compute= dp.get_precision('Account')),
       'concept_id': fields.many2one('islr.wh.concept', 'Withholding Concept', required=True),
       'second': fields.boolean('Second Concept', help="If the second field is set to True,  it will allow you store a second concept."),
       'second_amount_untaxed_islr': fields.float('Second Base Price', digits_compute= dp.get_precision('Account')),
       'second_concept_id': fields.many2one('islr.wh.concept', 'Second Withholding Concept'),
    }
    
    def islr_open(self, cr, uid, ids, context=None):
        
        #obtener monto base de la factura
        data_islr = self.read(cr, uid, ids, [],context=context)[0]
        base_amount = self.pool.get('account.invoice').browse(cr, uid, context['active_id'], context=context).amount_untaxed
        #obtener la información de las lineas de la factura
        acc_inv_line_id = self.pool.get('account.invoice.line').search(cr, uid, [('invoice_id', '=', context['active_id'])], context=context)
        acc_inv_line = self.pool.get('account.invoice.line').browse(cr, uid, acc_inv_line_id, context=context)
        
        if context['active_id'] > 395:
            raise osv.except_osv(_('mensaje_usuario'),_('Este tipo de cálculo no aplica a esta factura. Esto es debido a que la factura no ha sido importada del sistema CKF.'))
        elif len(acc_inv_line) > 1:
            raise osv.except_osv(_('mensaje_usuario'),_('No se permite procesar en facturas con más de una línea a facturar.'))
        elif data_islr['amount_untaxed_islr'] + data_islr['second_amount_untaxed_islr'] <= 0 or data_islr['amount_untaxed_islr'] + data_islr['second_amount_untaxed_islr'] > base_amount:
            raise osv.except_osv(_('mensaje_usuario'),_('Debe registrar un moto menor o igual al monto base registrado en la factura. Por favor revisar.'))
        else:            
            #Crear account.invoice.line en base a la original
            # y asignarle monto indicado por el usuario y el concepto  indicado 
            # la linea original se actualiza a "No Aplica Retencion"
            # si hay un segundo concepto se debe crear account.invoice.line en base a la original
            # y asignarle monto indicado por el usuario y el concepto  indicado 

            
            data_islr_cp = {
                'price_unit':base_amount - data_islr['amount_untaxed_islr'] - data_islr['second_amount_untaxed_islr'],
                'price_subtotal':base_amount - data_islr['amount_untaxed_islr'] - data_islr['second_amount_untaxed_islr'],
                'concept_id': 1,
                }
            resultado = self.pool.get('account.invoice.line').copy(cr,uid,acc_inv_line_id[0],data_islr_cp,context=context)
            if resultado:
                #Actualizar monto de la linea original
                data_origen = {
                    'price_unit':data_islr['amount_untaxed_islr'],
                    'price_subtotal':data_islr['amount_untaxed_islr'],
                    'concept_id': data_islr['concept_id'],
                }
                resultado2 = self.pool.get('account.invoice.line').write(cr,uid,acc_inv_line_id[0],data_origen,context=context)
                if bool(resultado2):
                    if bool(data_islr['second']):
                        data_origen = {
                            'price_unit':data_islr['second_amount_untaxed_islr'],
                            'price_subtotal':data_islr['second_amount_untaxed_islr'],
                            'concept_id': data_islr['second_concept_id'],
                        }
                        resultado3 = self.pool.get('account.invoice.line').copy(cr,uid,resultado,data_origen,context=context)
                        if not bool(resultado3):
                            raise osv.except_osv(_('mensaje_usuario'),_('Hubo problemas en el proceso de crear la segunda retencion, contactar al administrador del sistema.'))
                else:
                    raise osv.except_osv(_('mensaje_usuario'),_('Hubo problemas en el proceso de crear la retencion, contactar al administrador del sistema.')) 
            else:
                raise osv.except_osv(_('mensaje_usuario'),_('Hubo problemas en el proceso, contactar al administrador del sistema.'))            
            
            #mandar a ejecutar los procesos preestablecidos
            #~ Sub-flujos existentes
            #~ Or
            #~ wh_router - islr_router - Condicion: check_invoice_type()
            #~ wh_router - Withholding Vat - Condicion: check_wh_apply()
            
            #~ Transicion wh_router - islr_router: check_invoice_type()
            
            #~ XOr
            #~ islr_router - islr_xml - Condicion: check_wh_islr_xml()
            #~ islr_router - Withholding Islr - Condicion: check_wh_islr_apply()
            
            #~ Xor
            #~ Withholding Islr - paid - Condicion: test_paid()
            
            #~ Xor
            #~ Withholding Vat - paid - Condicion: test_paid()
            
            #Mandar a ejecutar el workflow
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'account.invoice', context['active_id'], 'invoice_open', cr)
            
            return {'type': 'ir.actions.act_window_close'}

sisb_account_invoice_wizard()
