#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################################
#    SISB, Sistema Integral Simón Bolívar
#    Copyright (C) 2011-2013 Industrias Diana, C.A. (<http://www.industriasdiana.gob.ve>).
#    All Rights Reserved
###############Credits######################################################################
#    Developed by: Industrias Diana, C.A.
############################################################################################

import time
from osv import osv, fields
import decimal_precision as dp
from tools.translate import _

#TODO: Merge this code with the module sisb_banking
class account_advances_config(osv.osv):
    _name = 'account.advances.config'
    
    _columns = {
        'name': fields.char('Ref', size=64),
        'journal_id': fields.many2one('account.journal','Journal'),
        'sequence_id': fields.many2one('ir.sequence', 'Entry Sequence', help="This field contains the informatin related to the numbering of the journal entries of this journal.", required=True),
        'sequence_payable_id': fields.many2one('ir.sequence', 'Entry Sequence Advance Payable', help="Este campo contiene informacion relacionada al numero de secuencia de anticipos para cuentas por pagar", required=True),
    }
    
account_advances_config()


class account_advances(osv.osv):
    _name = 'account.advances'

    def copy(self, cr, uid, ids, defaults, context=None):
        raise osv.except_osv(_('Error !'), _('No esta permitido duplicar el documento'))
        return True


    def dummy(self, cr, uid, context=None):
        return True


    def _get_journal(self, cr, uid, context=None):
        """Funcion que trae el diario configurado para
        los anticipos """
        journal_obj = self.pool.get('account.advances.config')
        journal_ids = journal_obj.search(cr, uid, [])
        if not journal_ids:
            raise osv.except_osv(_('Error !'), _('Debe verificar que exista configuracion para los anticipos'))
        journal_brw = journal_obj.browse(cr, uid, journal_ids, context)
        return journal_brw[0].journal_id.id
    
    
    def _get_account(self, cr, uid, context=None):
        """Funcion que trae la cuenta del diario configurado para
        los anticipos"""
        journal_obj = self.pool.get('account.advances.config')
        journal_ids = journal_obj.search(cr, uid, [])
        if not journal_ids:
            raise osv.except_osv(_('Error !'), _('Debe verificar que exista configuracion para los anticipos'))
        journal_brw = journal_obj.browse(cr, uid, journal_ids, context)
        if context.get('type', 'cobrar') == 'cobrar':
            return journal_brw[0].journal_id.default_credit_account_id.id
        else:
            return journal_brw[0].journal_id.default_debit_account_id.id


    def _get_period(self, cr, uid, context=None):
        """ Funcion que trae el periodo en curso """
        period_obj = self.pool.get('account.period')
        return period_obj.find(cr, uid)[0]


    def _compute_lines(self, cr, uid, ids, name, args, context=None):
        """ Esta Funcion busca los apuntes que se han conciliado con
        la(s) factura(s) que pertenecen al diario de anticipos"""
        result = {}
        invoice_ids = []
        move_linea_ids = []
        aac_conf = self.pool.get('account.advances.config')
        aac_conf_id = aac_conf.search(cr, uid, [])
        aac_brw = aac_conf.browse(cr, uid, aac_conf_id, context=context)[0]
        journal = aac_brw.journal_id.id

        amortization_brw = self.browse(cr, uid, ids, context)
        for header in amortization_brw:
            move_id = header.move_id and header.move_id.id or None
            for line in header.linea_ids: 
                invoice_ids.append(line.invoice_id.id)
            invoice_brw = self.pool.get('account.invoice').browse(cr, uid, invoice_ids, context)
            for invoice in invoice_brw:
                lines = []
                if invoice.move_id:
                    for m in invoice.move_id.line_id:
                        temp_lines = []
                        if m.reconcile_id:
                            temp_lines = [x.id for x in m.reconcile_id.line_id if x.journal_id.id == journal and x.move_id.id == move_id]
                        elif m.reconcile_partial_id:
                            temp_lines = [x.id for x in m.reconcile_partial_id.line_partial_ids if x.journal_id.id == journal and x.move_id.id == move_id]
                        if temp_lines:
                            lines.append(temp_lines)

                if lines:
                    lines = list(set(reduce(lambda x,y: x+y, lines)))
                    move_linea_ids.append(lines)

            if move_linea_ids:
                move_linea_ids = list(set(reduce(lambda x,y: x+y, move_linea_ids)))

            result[header.id] = move_linea_ids
        return result


    def _compute_amount(self, cr, uid, ids, name, args, context=None):
        """ Esta funcion calcula el saldo restante de los anticipos agregados
        contra el monto de las facturas a aplicar el anticipo"""
        precision_obj = self.pool.get('decimal.precision')
        result = {}
        move_amount = 0.00
        invo_amount = 0.00
        anticipo_brw = self.browse(cr, uid, ids, context=context)
        for header in anticipo_brw:
            for move in header.move_line_ids:
                if header.type == 'cobrar':
                    move_amount += move.credit
                else:
                    move_amount += move.debit
            for line in header.linea_ids:
                invo_amount += line.amount
            result[header.id] = round(move_amount - invo_amount, precision_obj.precision_get(cr, uid, 'Account'))
        return result


    _order = 'id desc'
    
    _columns = {
        'name': fields.char("Ref.", size=120),
        'partner_id': fields.many2one('res.partner', 'Partner', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'period_id': fields.many2one('account.period', 'Periodo', readonly=True, states={'draft':[('readonly',False)]}),
        'journal_id': fields.many2one('account.journal', 'Diario', readonly=True, states={'draft':[('readonly',False)]}),
        'date_advances': fields.date('Fecha Registro'),
        'date_process': fields.date('Fecha Procesado'),
        'linea_ids': fields.one2many('account.advances.lines', 'advances_id',
            'Lineas', ondelete='cascade',readonly=True, states={'draft':[('readonly',False)]}),
        'move_line_ids': fields.many2many('account.move.line','advances_move_lines_rel','advances_id','move_line_id','Move Lines', readonly=True, states={'draft':[('readonly',False)]}),
        'payment_ids': fields.function(_compute_lines, method=True, relation='account.move.line', type="many2many", string='Payments'),
        'account_id': fields.many2one('account.account', 'Cuenta Contable', readonly=True),
        'type': fields.selection([('pagar','CxP'),('cobrar','CxC')], 'Tipo de Anticipo'),
        'state': fields.selection([
                ('draft','Draft'),
                ('concilied','Concilied'),
                ('cancel','Cancelled')
                ],'State', select=True, readonly=True),
        'residual_amount': fields.function(_compute_amount, digits_compute=dp.get_precision('Account'), method=True, type="float",
            string='Monto Restante del Anticipo'),
        'move_id': fields.many2one('account.move', 'Asiento Contable'),
    }


    _defaults = {
        'state': 'draft',
        'date_advances': lambda *a: time.strftime('%Y-%m-%d'),
        'journal_id': _get_journal,
        'period_id': _get_period,
        'account_id': _get_account,
    }


    def _check_fecha_anticipo(self, cr, uid, ids, context=None):
        """ Verificar que la fecha del anticipo se encuentre dentro del rango
            del periodo seleccionado.
            @return: True (bien) or False (mal)
        """
        for adv in self.browse(cr, uid, ids, context=context):
            if (adv.date_advances >= adv.period_id.date_start) and (adv.date_advances <= adv.period_id.date_stop):
                return True
        return False
    

    _constraints = [(_check_fecha_anticipo, 'La fecha del anticipo no se encuentra dentro del rango del Periodo', ['date_advances'])]


    def _validar_retencion(self, cr, uid, ids, context=None):
        """Valida que las facturas hayan sido procesadas su retencion
        """
        iva_inv = []
        islr_inv = []
        doc_brw = self.browse(cr, uid, ids, context=context)
        for line in doc_brw:
            if not line.linea_ids:
                raise osv.except_osv(_('Error !'), _('Debe agregar facturas'))
            for inv in line.linea_ids:
                if inv.invoice_id.wh_iva_id and inv.invoice_id.wh_iva_id.state <> 'done':
                    iva_inv.append(inv.invoice_id.number)
                if inv.invoice_id.islr_wh_doc_id and inv.invoice_id.islr_wh_doc_id.state <> 'done':
                    islr_inv.append(inv.invoice_id.number)
        if iva_inv:
            raise osv.except_osv(_('Integrity Error !'), _('Las Facturas %s posee retenciones de IVA sin aprobar'%iva_inv))
        if islr_inv:
            raise osv.except_osv(_('Integrity Error !'), _('Las Facturas %s posee retenciones de ISLR sin aprobar'%islr_inv))
 
        return True


    def apply_advances(self, cr, uid, ids, context=None):
        """ Funcion que busca y valida que las facturas agregadas
        a las lineas del anticipos posean notas de credito para
        luego comenzar a conciliarlas """
        if context is None:
            context = {}
        
        self._validar_retencion(cr, uid, ids, context=context)
        self._validar_facturas_rectificativas(cr, uid, ids, context=context)
        linea_ids = []
        doc_brw = self.browse(cr, uid, ids, context)
        for doc in doc_brw:
            type_anticipo = doc.type
            if doc.residual_amount >= 0.00:
                if doc.linea_ids:
                    if not self._process_advances(cr, uid, doc, context):
                        raise osv.except_osv(_('Error !'), _('The reconciliation process is not complete, please check the documents'))
                else:
                    raise osv.except_osv(_('Integrity Error !'), _('You can not process a write without lines')) 
            else:
                raise osv.except_osv(_('Integrity Error !'), _('El Documento de anticipo tiene un monto errado'))
        
        if type_anticipo == 'cobrar':
            number = self.pool.get('ir.sequence').get(cr, uid, 'account.advances.code')
        else:
            number = self.pool.get('ir.sequence').get(cr, uid, 'account.advances.payable.code')
            
        self.write(cr, uid, ids, {'state': 'concilied', 'date_process': time.strftime('%Y-%m-%d'), 'name': number})
        return True


    def _process_advances(self, cr, uid, doc, context=None):
        """ Funcion que se encarga de la conciliacion
        automatica de los anticipos, si la misma
        es parcial crea un asiento intermedio y hace una
        conciliacion parcial de lo contrario ejecuta
        la conciliacion por el monto completo. """
        if context is None:
            context = {}
        am_obj = self.pool.get("account.move")
        aml_obj = self.pool.get("account.move.line")
        seq_obj = self.pool.get('ir.sequence')
        ai_obj = self.pool.get('account.invoice')
        aj_obj = self.pool.get('account.journal')
        
        inv_amount_total = 0.00
        move_amount_total = 0.00
        diff_move_amount = 0.00
        date_reg = ''
        period_id = ''
        partner_id = ''
        lineas_dict = {}
        invoice_ids = []
        conciliation_ids = []
        advances_ids = []
        move_line_ids = []

        #TODO: improve the code loops
        type_anticipo = doc.type
        date_reg = doc.date_advances 
        period_id = doc.period_id.id
        partner_id = doc.partner_id.id
        account_anti_id = doc.account_id.id
        
        for doc_line in doc.linea_ids:
            inv_amount_total += doc_line.amount
            invoice_ids.append(doc_line.invoice_id.id)
            lineas_dict.update({doc_line.invoice_id.id: doc_line.amount})
            
        for move_line in doc.move_line_ids:
            if type_anticipo == 'cobrar':
                move_amount_total += move_line.credit
            else:
                move_amount_total += move_line.debit
        
        move_amount_total_copy = move_amount_total
        
        #TODO: improve this block with the field account_id to the advances
        aac_conf = self.pool.get('account.advances.config')
        aac_conf_id = aac_conf.search(cr, uid, [])
        aac_brw = aac_conf.browse(cr, uid, aac_conf_id, context=context)
        for journal in aac_brw:
            journal_sequence_id = journal.journal_id.sequence_id.id
            journal_id = journal.journal_id.id
            if type_anticipo == 'cobrar':
                account_id = journal.journal_id.default_credit_account_id.id
            else:
                account_id = journal.journal_id.default_debit_account_id.id
        
        """Se realiza una resta para determinar
        si es necesario hacer un apunte nuevo 
        para el asiento intermedio"""
        if (move_amount_total - inv_amount_total) > 0.0001:
            diff_move_amount = (move_amount_total - inv_amount_total)

        move_id = am_obj.create(cr, uid, {
                'name': seq_obj.get_id(cr, uid, journal_sequence_id),
                'journal_id': journal_id,
                'date': time.strftime('%Y-%m-%d'),
                'ref': 'Anticipo Asiento Intermedio',
                'period_id': period_id,
                }, context)
        
        #TODO: improve the code for the purchase advances
        """Busco los IDS de los asientos de la
        cuenta por cobrar/pagar"""
        ai_brw = ai_obj.browse(cr, uid, invoice_ids, context)
        for inv in ai_brw:
            for move in [inv.move_id]:
                for lines in move.line_id:
                    if type_anticipo == 'cobrar':
                        if lines.account_id.id == inv.account_id.id and lines.debit > 0.00:
                            account_inv_id = lines.account_id.id
                            conciliation_ids.append(lines.id) 
                    elif type_anticipo == 'pagar':
                        if lines.account_id.id == inv.account_id.id and lines.credit > 0.00:
                            account_inv_id = lines.account_id.id
                            conciliation_ids.append(lines.id)
                
            residual = lineas_dict[inv.id] if lineas_dict[inv.id] <= move_amount_total else move_amount_total
            aml_id = aml_obj.create(cr, uid, {
                        'name': 'Anticipo Fact. %s'%(inv.number),
                        'debit': residual if type_anticipo == 'pagar' else 0.00,
                        'credit': residual if type_anticipo == 'cobrar' else 0.00,
                        'invoice_id': inv.id,
                        'account_id': account_inv_id, 
                        'move_id': move_id,
                        'journal_id': journal_id,
                        'period_id': period_id,
                        'partner_id': partner_id, 
                        'currency_id': False,
                        'date': time.strftime('%Y-%m-%d'), 
                        'date_maturity': time.strftime('%Y-%m-%d'),
                        }, context=None)
            move_amount_total -= residual
            #~ Concilia la linea de compensacion del debito contra la cxc de facturas.
            conciliation_ids.append(aml_id)
            move_line_ids.append(conciliation_ids)
            conciliation_ids = []
        
        if diff_move_amount > 0.0001:
            aml_id = aml_obj.create(cr, uid, {
                'name': 'Saldo Restante de Anticipo',
                'debit': diff_move_amount if type_anticipo == 'pagar' else 0.00,
                'credit': diff_move_amount if type_anticipo == 'cobrar' else 0.00,
                'account_id': account_anti_id, 
                'move_id': move_id,
                'journal_id': journal_id,
                'period_id': period_id,
                'partner_id': partner_id, 
                'currency_id': False,
                'date': time.strftime('%Y-%m-%d'), 
                'date_maturity': time.strftime('%Y-%m-%d'),
                }, context=None)
        
        #TODO: improve the code for the purchase advances
        for move_line in doc.move_line_ids:
            conciliation_ids.append(move_line.id)
            aml_id = aml_obj.create(cr, uid, {
                'name': 'Anticipo %s'%(move_line.name),
                'debit': move_line.credit if type_anticipo == 'cobrar' else 0.00,
                'credit': move_line.debit if type_anticipo == 'pagar' else 0.00,
                'account_id': move_line.account_id.id, 
                'move_id': move_id,
                'journal_id': journal_id,
                'period_id': period_id,
                'partner_id': partner_id, 
                'currency_id': False,
                'date': time.strftime('%Y-%m-%d'), 
                'date_maturity': time.strftime('%Y-%m-%d'),
                }, context=None)
            conciliation_ids.append(aml_id)
            move_line_ids.append(conciliation_ids)
            conciliation_ids = []
        
        for line in move_line_ids:
            aml_obj.reconcile_partial(cr, uid, line, 'auto', context=context)
        
        self.write(cr, uid, doc.id, {'move_id': move_id}, context=context)

        return True
    
    
    def rpc_fix_advances(self, cr, uid, ids, context=None):
        doc_brw = self.browse(cr, uid, ids, context)
        for doc in doc_brw:
            self.rpc_fix_move_lines_advances(cr, uid, doc, context=context)
        return True
    
    
    def rpc_fix_move_lines_advances(self, cr, uid, doc, context=None):
        """ Funcion utilizada para corregir los anticipos con 
        asientos erroneos """
        if context is None:
            context = {}
        am_obj = self.pool.get("account.move")
        aml_obj = self.pool.get("account.move.line")
        amr_obj = self.pool.get("account.move.reconcile")
        seq_obj = self.pool.get('ir.sequence')
        ai_obj = self.pool.get('account.invoice')
        aj_obj = self.pool.get('account.journal')
        
        inv_amount_total = 0.00
        move_amount_total = 0.00
        diff_move_amount = 0.00
        date_reg = ''
        period_id = ''
        partner_id = ''
        invoice_ids = []
        conciliation_ids = []
        advances_ids = []
        move_line_ids = []
        move_concil_ids = ''
        amount_dict = {}
        print "Cargando datos principales"
        #TODO: improve the code loops
        type_anticipo = doc.type
        date_reg = doc.date_advances 
        period_id = doc.period_id.id
        partner_id = doc.partner_id.id
        account_anti_id = doc.account_id.id
        print "Carga Terminada"
        
        for doc_line in doc.linea_ids:
            inv_amount_total += doc_line.amount
            invoice_ids.append(doc_line.invoice_id.id)
            amount_dict[doc_line.invoice_id.id] = doc_line.amount
            print "Invoice Number %s Monto: %s"%(doc_line.invoice_id.number, doc_line.amount)
        
        for move_line in doc.move_line_ids:
            if type_anticipo == 'cobrar':
                move_amount_total += move_line.credit
            else:
                move_amount_total += move_line.debit
        
        move_amount_total_copy = move_amount_total
        
        #TODO: improve this block with the field account_id to the advances
        aac_conf = self.pool.get('account.advances.config')
        aac_conf_id = aac_conf.search(cr, uid, [])
        aac_brw = aac_conf.browse(cr, uid, aac_conf_id, context=context)
        for journal in aac_brw:
            journal_sequence_id = journal.journal_id.sequence_id.id
            journal_id = journal.journal_id.id
            if type_anticipo == 'cobrar':
                account_id = journal.journal_id.default_credit_account_id.id
            else:
                account_id = journal.journal_id.default_debit_account_id.id
        
        """Desconcilio todos las lineas del
        movimiento contable que se crearon con el codigo antiguo"""
        old_move_ids = []
        payment_dict = self.read(cr, uid, doc.id, ['payment_ids'],context=context)
        print payment_dict
        for pay in aml_obj.browse(cr, uid, payment_dict['payment_ids'], context=context):
            old_move_ids.append(pay.id)
            move_id = pay.move_id.id
                
        print "Este es el ID del Asiento ", move_id
        print "IDS Asientos viejos: ", old_move_ids
        #~ print error
        aml_ids = aml_obj.search(cr, uid, [('move_id','=',move_id)])
        aml_brw = aml_obj.browse(cr, uid, aml_ids, context=context)
        for m in aml_brw:
            if m.reconcile_id:
                amr_obj.unlink(cr, uid, m.reconcile_id.id)
                print "Desconcilio Total ", m.reconcile_id.id 
            if m.reconcile_partial_id:
                amr_obj.unlink(cr, uid, m.reconcile_partial_id.id)
                print "Desconcilio Parcial ",m.reconcile_partial_id.id 
            
            if type_anticipo == 'cobrar':
                if m.debit > 0.00:
                    move_concil_ids = m.id 
            elif type_anticipo == 'pagar':
                if m.credit > 0.00:
                    move_concil_ids = m.id 
                    
        #~ Elimina las lineas viejas del anticipo
        aml_obj.unlink(cr, uid, old_move_ids)
        print "Lineas Eliminadas con exito IDS: ",old_move_ids
        print "Anticipo Referencia ", doc.name
        #TODO: improve the code for the purchase advances
        """Busco los IDS de los asientos de la
        cuenta por cobrar/pagar"""
        ai_brw = ai_obj.browse(cr, uid, invoice_ids, context)
        for inv in ai_brw:
            for move in [inv.move_id]:
                for lines in move.line_id:
                    if type_anticipo == 'cobrar':
                        if lines.account_id.id == inv.account_id.id and lines.debit > 0.00:
                            account_inv_id = lines.account_id.id
                            conciliation_ids.append(lines.id) 
                    elif type_anticipo == 'pagar':
                        if lines.account_id.id == inv.account_id.id and lines.credit > 0.00:
                            account_inv_id = lines.account_id.id
                            conciliation_ids.append(lines.id)
                
            residual = amount_dict[inv.id] #~ if amount_dict[inv.id] <= move_amount_total else move_amount_total
            mov_dict = {
                        'name': 'Anticipo Fact. %s'%(inv.number),
                        'debit': residual if type_anticipo == 'pagar' else 0.00,
                        'credit': residual if type_anticipo == 'cobrar' else 0.00,
                        'invoice_id': inv.id,
                        'account_id': account_inv_id, 
                        'move_id': move_id,
                        'journal_id': journal_id,
                        'period_id': period_id,
                        'partner_id': partner_id, 
                        'currency_id': False,
                        'date': time.strftime('%Y-%m-%d'), 
                        'date_maturity': time.strftime('%Y-%m-%d'),
                        }
                        
            print "Asiento ",mov_dict
            aml_id = aml_obj.create(cr, uid, mov_dict, context=None)
            print "Creado Asiento de Fact: ",inv.number 
            #~ Concilia la linea de compensacion del debito contra la cxc de facturas.
            conciliation_ids.append(aml_id)
            move_line_ids.append(conciliation_ids)
            conciliation_ids = []
        
        print "Comenzando proceso de Conciliacion de Account_move"
        #TODO: improve the code for the purchase advances
        for move_line in doc.move_line_ids:
            conciliation_ids.append(move_line.id)
            conciliation_ids.append(move_concil_ids)
            print "Estos son los IDS a Conciliar ", conciliation_ids
            move_line_ids.append(conciliation_ids)
            conciliation_ids = []
        
        for line in move_line_ids:
            aml_obj.reconcile_partial(cr, uid, line, 'auto', context=context)
        
        return True
              
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        """
        Herencia del search para agregar filtro que busque periodo, se hace
        verificando patron que es enviado por el filtro de 'current_period'.
        """
        if context is None:
            context = {}
        
        period_obj = self.pool.get('account.period')
        
        for arg in args:
            if arg[0] == 'period_id' and arg[2] == 'current_period':
                current_period = period_obj.find(cr, uid)[0]
                args.append(('period_id','in',[current_period]))
                break
        
        for a in args:
            if a in [('period_id','in','current_period')]:
                args.remove(a)
        return super(account_advances, self).search(cr, uid, args=args, offset=offset, limit=limit, order=order,
            context=context, count=count)
    
    
    def _validar_facturas_rectificativas(self, cr, uid, ids, context=None):
        """
        Funcion para validar si las facturas que se van a procesar tienen
        nota de debito/credito sin conciliar.
        """
        invoice_obj = self.pool.get('account.invoice')
        
        for anticipo in self.browse(cr, uid, ids, context=context):
            facturas_rec = []
            facturas = []
            for line in anticipo.linea_ids:
                if (line.amount - line.invoice_id.residual) > 0.001:
                    facturas.append(line.invoice_id.number)
                if invoice_obj.search(cr, uid, [('parent_id', '=', line.invoice_id.id), ('state', '!=', 'paid')]):
                    facturas_rec.append(line.invoice_id.number)
            
            if facturas_rec:
                raise osv.except_osv(_('Integrity Error !'), _('Las Facturas %s poseen notas rectificativas sin conciliar'%facturas_rec))
            
            if facturas:
                raise osv.except_osv(_('Integrity Error !'), _('En las siguientes Facturas %s se debe modificar el monto porque es mayor al residual de la factura'%facturas))
                
        return True
    
account_advances()


class account_advances_lines(osv.osv):
    _name = 'account.advances.lines'
    
    def _check_amount(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        aal_brw = self.browse(cr, uid, ids, context) 
        for val in aal_brw:
            if val.amount > 0:
                return True
            else:
                return False
    
    
    def _check_unique_invoice(self, cr, uid, ids, context=None):
        """ Verificar que la factura sea unica en el documento de anticipo.
            @return: True (bien) or False (mal)
        """
        if context is None:
            context = {}
        for adv_lin in self.browse(cr, uid, ids, context=context):
            #Validar que la factura no se encuentre en el mismo Anticipo
            adv_list = self.search(cr, uid, [('invoice_id', '=', adv_lin.invoice_id.id), ('advances_id', '=', adv_lin.advances_id.id)])
            if len(adv_list) > 1:
                return False
            #Validar que la factura no se encuentre en otro anticipo que se encuentre en estado borrador
            adv_list = self.search(cr, uid, [('invoice_id', '=', adv_lin.invoice_id.id), ('advances_id', '!=', adv_lin.advances_id.id)])
            for anti in adv_list:
                ant_brw = self.browse(cr, uid, anti, context=context)
                if ant_brw.advances_id.state == 'draft':
                    raise osv.except_osv(_('Error !'),_('No se puede agregar la factura, esta se encuentra en otro Anticipo en "Borrador"'))
                    return False

        return True


    _rec_name = "invoice_id"

    _columns = {
        'invoice_id':fields.many2one('account.invoice','Factura'),
        'advances_id':fields.many2one('account.advances','Anticipo'),
        'amount': fields.float("Monto", required=True),
    }
    
    _constraints = [(_check_amount, "El monto de la linea no puede ser cero",['amount']),
        (_check_unique_invoice, 'La factura no puede ser repetida en el documento', ['invoice_id']),]


    def onchange_invoice(self, cr, uid, ids, invoice_id, context=None):
        res = {'value': {'amount': 0.00}}

        if invoice_id:
            inv_brw = self.pool.get("account.invoice").browse(cr, uid, [invoice_id], context)
            for inv in inv_brw:
                amount = inv.residual
            if amount:
                res['value']['amount'] = amount
        return res
    
    
    def onchange_amount_limit(self, cr, uid, ids, invoice_id, amount, context=None):
        if context is None:
            context = {}

        result = {}
        inv_obj = self.pool.get('account.invoice')
        if invoice_id:
            inv_brw = inv_obj.browse(cr, uid, invoice_id, context)
            if inv_brw.residual < amount:
                result.update({'amount': inv_brw.residual})
                warning = {
                    'title': 'Error',
                    'message': 'El valor no puede ser mayor al monto restante en la factura'
                }
                return {'value': result, 'warning': warning}
                
        return {'value': result}

account_advances_lines()


class account_bank_statement_line(osv.osv):
    _inherit = 'account.bank.statement.line'
     
    def onchange_type(self, cr, uid, line_id, partner_id, type, context=None):
        res = {'value': {}}
        obj_partner = self.pool.get('res.partner')
        res = super(account_bank_statement_line, self).onchange_type(cr, uid, line_id, partner_id, type, context)
        if type == 'advance' or type == 'advance_payable':
            aac_conf = self.pool.get('account.advances.config')
            aac_conf_id = aac_conf.search(cr, uid, [])
            if not aac_conf_id:
                raise osv.except_osv(_('Error !'), _('Debe verificar que exista configuracion para los anticipos'))
            aac_brw = aac_conf.browse(cr, uid, aac_conf_id, context=context)
            for journal in aac_brw:
                account_id = journal.journal_id.default_credit_account_id.id or False
                if type == 'advance_payable':
                    account_id = journal.journal_id.default_debit_account_id.id or False
            res['value']['account_id'] = account_id
        return res
    
account_bank_statement_line()
