#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################################
#    SISB, Sistema Integral Simón Bolívar
#    Copyright (C) 2011-2013 Industrias Diana, C.A. (<http://www.industriasdiana.gob.ve>).
#    All Rights Reserved
###############Credits######################################################################
#    Developed by: Industrias Diana, C.A.
############################################################################################

import time
from osv import osv, fields
import decimal_precision as dp
from tools.translate import _


class account_advances(osv.osv):
    _inherit = 'account.advances'


    def _process_advances(self, cr, uid, doc, context=None):
        """
        Funcion para crear la imputacion al crear un anticipo.
        """
        if context is None:
            context = {}
        ai_obj = self.pool.get("account.invoice") 
        """
        Se calcula el pendiente de cada factura que se le este haciendo un pago,
        antes de que se realice el pago, esto para evitar el cambie en el pendiente.
        El pendiente se envia por el context.
        """
        residuales = {}
        anticipo_invoice = []

        for line in doc.linea_ids: 
            residuales[str(line.invoice_id.id)] = line.invoice_id.residual
            anticipo_invoice.append({'invoice': line.invoice_id.id, 'amount': line.amount})

        result = super(account_advances, self)._process_advances(cr, uid, doc, context=context)
        
        if result and anticipo_invoice:
            for inv_dict in anticipo_invoice:
                res = ai_obj.dist_payment(cr, uid, 'account.advances', doc.id, inv_dict['invoice'], inv_dict['amount'], residuales[str(inv_dict['invoice'])], context=None)
        else:
            raise osv.except_osv(_('Integrity Error'),('Ha Ocurrido un'+\
                ' error al intentar crear las imputaciones'))
        
        return result
              
account_advances()


class account_voucher(osv.osv):
    _inherit = "account.voucher"

    
    def _no_imputados(self, cr, uid, ids, context=None):
        """
        Busca todas las lineas que tienen una cuenta que no debe ser imputada.
        @return: lista de diccionario con el modelo y id que no se imputara
        """
        user_id = 1
        aac_conf = self.pool.get('account.advances.config')
        aac_conf_id = aac_conf.search(cr, user_id, [])
        account_list = []
        aac_brw = aac_conf.browse(cr, user_id, aac_conf_id, context=context)
        if aac_brw:
            aac_brw = aac_brw[0]
            account_list.append(aac_brw.journal_id.default_credit_account_id.id)
            account_list.append(aac_brw.journal_id.default_debit_account_id.id)
        voucher_list = []
        av_obj = self.browse(cr, uid, ids, context=context)
        for avl in av_obj[0].line_ids:
            if not avl.invoice_id and avl.account_id.id in account_list:
                voucher_list.append({
                    'res_id': avl.id,
                    'model': 'account.voucher.line'
                    }
                )
        return voucher_list


    def _accion_imputar_voucher_line(self, cr, uid, ids, context=None):
        """
        Funcion que crea las imputaciones presupuestarias de la orden de pago.
        """
        ai_obj = self.pool.get('account.invoice')
        avl_obj = self.pool.get('account.voucher.line')
        imp_obj = self.pool.get('presupuesto.imputacion.linea')
        period_obj = self.pool.get('periodo.presupuestario')
        voucher_line_obj = self.pool.get('account.voucher.line')
        fecha = time.strftime('%Y-%m-%d')
        
        avl = avl_obj.browse(cr, uid, ids, context=context)
        
        if avl:
            imp_dict = {}
            """
            Si orden de pago se realiza a partir de factura se llama dist_payment,
            de lo contrario se imputara el comprometido, causado y pago o devengado,
            liquidado y recaudado dependiendo del caso.
            """
            if avl.invoice_id:
                res = ai_obj.dist_payment(cr, uid, 'account.voucher.line', avl.id, avl.invoice_id.id, avl.amount, context['residuales'][str(avl.invoice_id.id)], context=None)
            else:
                ae_id = avl.accion_especifica_id and avl.accion_especifica_id.id or None
                partida_id = avl.partida_presupuesto and avl.partida_presupuesto.id or None
                partida = avl.partida_presupuesto and avl.partida_presupuesto.code or None
                partida_aux = avl.partida_presupuesto and avl.partida_presupuesto.code or ''
                
                if ae_id is None:
                    raise osv.except_osv(('Error !'), ('No hay accion especifica definida para la linea %s.'%avl.name))
                
                if partida is None:
                    raise osv.except_osv(('Error !'), ('No hay partida presupuestaria definida para la linea %s.'%avl.name))
                else:
                    partida_id = avl.partida_presupuesto and avl.partida_presupuesto.id or None
                    name = avl.partida_presupuesto and avl.partida_presupuesto.name or ' '
                    partida_aux += ' '+name
                
                imp_dict = {
                    'ref': "Pago %s"%avl.voucher_id.number,
                    'partida': partida,
                    'partida_id': partida_id,
                    'model': 'account.voucher.line',
                    'res_id': avl.id,
                    'fecha_imp': fecha,
                    'amount': avl.amount,
                    'periodo': period_obj.find(cr, uid, fecha, context=context)[0],
                    'accion_especifica_id': ae_id,
                    'status': '',
                    'state': 'activo',
                    'type': 'egreso',
                    'type_create':'automatica',
                }
                
                """
                Si la orden de pago es de tipo 'payment' es una orden de pago
                del proveedor, se realiza las imputaciones para el gasto, sino
                se realizaran las imputaciones para el ingreso.
                """
                if avl.voucher_id.type == 'payment':
                    if not avl.enlace_id:
                        enlace_id = self.bot_imputacion(cr, uid, imp_dict, context=context)
                    else:
                        enlace_id = avl.enlace_id
                    imp_dict.update({'status': 'pagado', 'enlace_id': enlace_id})
                else:
                    if not avl.enlace_id:
                        imp_dict.update({'status': '', 'type': 'ingreso'})
                        enlace_id = self.bot_imputacion_ingreso(cr, uid, imp_dict, context=context)
                    else:
                        enlace_id = avl.enlace_id
                    imp_dict.update({'status_ingreso': 'recaudado', 'enlace_id': enlace_id})
                
                """
                Se marca la linea de la orden de pago con el enlace_id, para saber
                que la linea ya fue imputada. y tener una referencia de cual fue
                primera imputacion creada.
                """
                voucher_line_obj.write(cr, uid, avl.id, {'enlace_id': enlace_id})
                imp_id = imp_obj.create(cr, uid, imp_dict, context)
                
        return True


    def _accion_imputar(self, cr, uid, ids, context=None):
        """
        Herencia de _accion_imputar para buscar las lineas del account_voucher
        que no se deben imputar por usar cuenta de anticipo
        """
        if context is None:
            context = {}
        voucher_list = self._no_imputados(cr, uid, ids, context=context)
        if voucher_list:
            av_obj = self.browse(cr, uid, ids, context=context)
            ids_no_impu = [x['res_id'] for x in voucher_list]
            for avl in av_obj[0].line_ids:
                if avl.id not in ids_no_impu:
                    res = self._accion_imputar_voucher_line(cr, uid, [avl.id], context=context)
            return True
        else:
            return super(account_voucher, self)._accion_imputar(cr, uid, ids, context=context)

account_voucher()
