# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from lxml import etree
import netsvc
from osv import osv, fields
import decimal_precision as dp
from tools.translate import _

class account_bank_statement(osv.osv):
    _inherit = 'account.bank.statement'

    def create_move_from_st_line(self, cr, uid, st_line_id, company_currency_id, st_line_number, context=None):
        if context is None:
            context = {}
        mov_id = super(account_bank_statement, self).create_move_from_st_line(cr, uid, st_line_id,\
                company_currency_id, st_line_number, context=context)
        account_move_obj = self.pool.get('account.move')
        account_move_line_obj = self.pool.get('account.move.line')
        account_bank_statement_line_obj = self.pool.get('account.bank.statement.line')
        st_line = account_bank_statement_line_obj.browse(cr, uid, \
                st_line_id, context=context)
        st = st_line.statement_id
        ml_ids = account_move_line_obj.search(cr, uid, [('statement_id', '=', st.id)])
        ml_brw_ids = account_move_line_obj.browse(cr, uid, ml_ids, context=context)
        for ml in ml_brw_ids:
            mov_id = ml.move_id.id
        res = account_move_obj.write(cr, uid, [mov_id], {'ref': st_line.name,}, context=context)
        move_brw  = account_move_obj.browse(cr, uid, [mov_id], context=context)
        for m in move_brw:
            for l in m.line_id:
                val = {
                'name': st_line.ref,
                'date': st_line.date,
                'ref': st_line.name, }
                cr.execute("UPDATE account_move_line SET name = '%s', date = '%s', ref = '%s' WHERE id = %s"%(val["name"],val["date"],val["ref"], l.id))
                cr.commit
                #~ account_move_line_obj.write(cr, uid, [l.id], val, context=context)
        return mov_id

    def write(self, cr, uid, ids, vals, context=None):
        """Se sobrecarga el metodo write para colocar
        las funciones para actualizar el Libro Diario en
        el pago cuando se guarde el extracto bancario """
        res = super(account_bank_statement, self).write(cr, uid, ids, vals, context=context)
        av = self.pool.get('account.voucher')
        for statement in self.browse(cr, uid, ids, context):
            for line in statement.line_ids:
                try:
                    if line.voucher_id:
                        av.write(cr, uid, [line.voucher_id.id],{'account_id':\
                                statement.journal_id.default_debit_account_id.id, \
                                'journal_id':statement.journal_id.id,\
                                'memory':line.ref, 'ref':line.name,\
                                'pay':'asiento'},
                                context=context)
                except:
                    raise osv.except_osv(_('Error !'),
                        _('No se puede guardar el extracto uno o mas de sus referencias a los pagos ha sido eliminada.'))
        return res


    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(_('Advertencia!'),
          _('No se permite duplicar este documento!'))


    def unlink(self, cr, uid, ids, context=None):
        """ Funcion de Borrado modificada para la eliminacion del account voucher 
        generado por la lina del extrato bancario """
        av = self.pool.get("account.voucher")
        unlink_ids = []
        eb_brw = self.browse(cr, uid, ids, context)
        for exban in eb_brw:
            for exban_lines in exban.line_ids:
                 unlink_ids.append(exban_lines.voucher_id.id)
            if unlink_ids:
                av.unlink(cr, uid, unlink_ids, context)
        return super(account_bank_statement, self).unlink(cr, uid, ids, context)


    def button_confirm_bank(self, cr, uid, ids, context=None):
         """ Sobrecarga del metodo para validar que no se confirme si no existen
         lineas de detalles para el extracto bancario """
         av = self.pool.get('account.voucher')
         bank_brw = self.browse(cr, uid, ids, context)
         for lines in bank_brw:
            if not lines.line_ids:
                 raise osv.except_osv(_('Error !'),
                        _('No puede confirmar un extrato \
                            bancario sin lineas de movimiento.'))
            for line in lines.line_ids:
                if line.type in ['general'] and line.invoice_id:
                    raise osv.except_osv(_('Error !'),\
                            _('No puede confirmar un extracto con pagos de \
                            factura en tipo General, el tipo debe ser \
                            Proveedor o Cliente'))
                if line.type in ['customer'] and line.invoice_id.type not in ['out_invoice']:
                    raise osv.except_osv(_('Error !'),\
                            _('El tipo de pago cliente es erroneo para una de las \
                                facturas, verifique que el pago corresponda con \
                                el tipo de factura'))
                if line.type in ['supplier'] and line.invoice_id.type not in ['in_invoice']:
                    raise osv.except_osv(_('Error !'),\
                            _('El tipo de pago Proveedor es erroneo para una de las \
                                facturas, verifique que el pago corresponda con \
                                el tipo de factura'))
                if line.type in ['advance'] and line.invoice_id:
                    raise osv.except_osv(_('Error !'),\
                            _('No puede confirmar un anticipo cliente con una factura \
                                asociada, el anticipo no debe tener factura'))
                if line.type in ['advance_payable'] and line.invoice_id:
                     raise osv.except_osv(_('Error !'),\
                            _('No puede confirmar un anticipo cliente con una factura \
                                asociada, el anticipo no debe tener factura'))
                if line.type in ['returned_check'] and line.invoice_id:
                     raise osv.except_osv(_('Error !'),\
                            _('No puede confirmar un cheque devuelto con una factura \
                                asociada, el cheque no debe tener factura'))
            for line in lines.line_ids:
                if line.voucher_id:
                    av.write(cr, uid, [line.voucher_id.id], \
                    {'account_id':lines.journal_id.default_debit_account_id.id,\
                    'journal_id':lines.journal_id.id, \
                    'pay':'asiento', 'bank_confirm':True,}, context=context)
         return super(account_bank_statement, self).button_confirm_bank(cr,\
                 uid, ids, context=context)

account_bank_statement()


class account_voucher(osv.osv):
    _inherit = "account.voucher"
    """Se agrega esta columna para que 
    el extracto se guarde una sola vez
    cuando se ejecuten los worflow."""
    _columns = {
        'bank_confirm': fields.boolean("Confirmado"),
    }
    
    _defaults = {
        'bank_confirm':False,
    }
    
account_voucher()


class account_bank_statement_line(osv.osv):
    _inherit = 'account.bank.statement.line'
    
    """ Se modifico el campo voucher_id para asignarle el borrado
    de tipo cascade para al momento de eliminarlo tambien eliminar los
    account.voucher generados por el extracto"""
    _columns = {
        'date_ref': fields.date('Fecha de Referencia'),
        'voucher_id': fields.many2one('account.voucher', 'Payment', ondelete='cascade'),
        'invoice_id': fields.many2one('account.invoice', 'Factura'),
        'type': fields.selection([
            ('supplier','Supplier'),
            ('customer','Customer'),
            ('general','General'),
            ('advance','Anticipo CxC'),
            #~ ('advance_payable','Anticipo CxP'),
            ('returned_check','Cheque Devuelto'),
            ], 'Type', required=True),
        'state': fields.related('statement_id','state',type='char',string='Estado',readonly=True),
        'period_id': fields.related('statement_id','period_id',type='many2one', relation='account.period', string='Periodo',readonly=True),
    }
    

    def _check_amount(self, cr, uid, ids, context=None):
        """Se sobre escribe la funcion _check_amount para cambiar
        el comportamiento y evitar que valide si los montos son diferentes
        para asi poder aplicar la modificacion para realizar pagos parciales"""
        for obj in self.browse(cr, uid, ids, context=context):
            if obj.voucher_id:
                if not (abs(obj.amount) == obj.voucher_id.amount):
                    return True
        return True

    def _check_transaction(self, cr, uid, ids, context=None):
        """ Chequea que los depositos no se dupliquen 
        en otro extracto bancario """
        abl = self.browse(cr, uid, ids, context=context)
        for line in abl:
            name = line.name
            statement_id = line.statement_id.id
            journal_id = line.statement_id.journal_id.id
            SQL = """SELECT absl.id, absl.name line_name, abs.name as statement_name
            FROM account_bank_statement_line absl 
            INNER JOIN account_bank_statement abs ON abs.id = absl.statement_id
            WHERE abs.journal_id = %s and absl.name = '%s' and statement_id <>
            %s"""%(journal_id, name, statement_id)
            cr.execute(SQL)
            res = cr.dictfetchone()
            if res:
                raise  osv.except_osv(_('Integrity Error'),('La Transaccion %s ya ' \
                        'esta duplicada en el extracto ' \
                        '%s'%(res['line_name'],res['statement_name'])))
                return False
            return True

    _constraints = [
        (_check_amount, 'The amount of the voucher must be the ' \
                'same amount as the one on the statement line', ['amount']),
        (_check_transaction, 'Transaccion duplicada en otro extracto ' \
                'bancario',['name'])
    ]
    
    
    def write(self, cr, uid, ids, values, context=None):
        """Se sobreescribe la funcion write para actualizar el monto
        del pago creado por el monto del pago parcial en la linea del
        extracto bancario"""
        if isinstance(ids, (int, long)):
            ids = [ids]
        res = super(account_bank_statement_line,self).write(cr, uid, ids, values, context)
        av = self.pool.get("account.voucher")
        voucher_id = self.read(cr, uid, ids,['voucher_id','amount',\
                'statement_id','type','name','ref'])
        statement_id = voucher_id[0]['statement_id'][0]
        values.update({'type': voucher_id[0]['type']})
        if values.get("type", False) in ['supplier','customer']:
            invoice_id = self.read(cr, uid, ids, ['invoice_id'])
            if not invoice_id:
                raise osv.except_osv(_('Error !'),
                _('Si el tipo del extracto es cliente o proveedor la factura es requerida.'))
            if not voucher_id[0]['voucher_id']:
                raise osv.except_osv(_('Error !'),
                _('Si el tipo del extracto es cliente o proveedor el pago asociado es requerido.'))
                
            values.update({'amount': voucher_id[0]['amount'] or 0.00})
            values.update({'voucher_id': voucher_id[0]['voucher_id'][0]})
            values.update({'ref':voucher_id[0]['ref'],'name':voucher_id[0]['name'],})
             
            if values.get("voucher_id", False):
                av_line = av.browse(cr, uid, [values["voucher_id"]], context)
                write_amount = {}
                for line in av_line:
                    if not line.bank_confirm:
                        for vline in line.line_ids:
                            write_amount = {'amount':values["amount"],\
                                    'pay':'asiento',\
                                    'reference':values["name"],\
                                    'name': values["ref"], \
                            'line_ids':[(1,vline.id,{'amount':values["amount"],})]}
                        av.write(cr, uid,  [line.id], write_amount, context=context)
        return res
        
        
    def on_change_populate_statement(self, cr, uid, ids, invoice_ids, journal_id, company_id, currency_id, period_id, context=None):
        """ Funcion OnChange para extraer los datos necesarios para la creacion
        de las lineas del extracto bancario.
        @invoice_ids: Es el id de la factura seleccionada en la grilla.
        @journal_id: Corresponde al id del parent del formulario.
        @company_id: Corresponde a la compañia en al que se ejecuta la operacion
        @currency_id: Es el id de la moneda utilizada en la operacion.
        @period_id: El id del perido en el que se esta realizando dicha operacion.
        @line_ids: Es el id del asiento de la factura relacionada a la linea
        del movimiento bancario. """
        
        line_obj = self.pool.get('account.move.line')
        statement_obj = self.pool.get('account.bank.statement')
        statement_line_obj = self.pool.get('account.bank.statement.line')
        currency_obj = self.pool.get('res.currency')
        voucher_obj = self.pool.get('account.voucher')
        voucher_line_obj = self.pool.get('account.voucher.line')
        line_date = time.strftime('%Y-%m-%d')
        
        if not (invoice_ids, journal_id, company_id, currency_id, period_id):
            return {'value':{}}
        
        statement = {
        'journal_id': journal_id, 
        'company_id': company_id, 
        'currency': currency_id, 
        'period_id': period_id,
        }
        
        journal_brw = self.pool.get("account.journal").browse(cr, uid, [statement['journal_id']],context)
        
        for account_default in journal_brw:
            statement.update({'default_credit_account_id':account_default.default_credit_account_id.id,})
        
        line_ids = []

        if context is None:
            context = {}
        if isinstance(invoice_ids, (int, long)):
            invoice_ids = [invoice_ids]
        if not isinstance(statement, (dict)):
            statement = dict(statement)

        inv_brw = self.pool.get("account.invoice")
        inv_data = inv_brw.browse(cr, uid, invoice_ids, context=context)
        for inv_header in inv_data:
            for move_header in [inv_header.move_id]:
                for move_line in move_header.line_id:
                    line_ids.append(move_line.id)
        
        line_ids = line_obj.search(cr, uid, [('account_id.type','in',['receivable','payable']),('reconcile_id','=',False), ('journal_id.type','in',['sale','purchase']), ('state', '=', 'valid'), ('id','in',line_ids)])
        
        if not line_ids:
            return True

        # for each selected move lines
        for line in line_obj.browse(cr, uid, line_ids, context=context):
            voucher_res = {}
            ctx = context.copy()
            #  take the date for computation of currency => use payment date
            ctx['date'] = line_date
            amount = 0.0

            if line.debit > 0:
                amount = line.debit
            elif line.credit > 0:
                amount = -line.credit

            if line.amount_currency:
                amount = currency_obj.compute(cr, uid, line.currency_id.id,
                    statement['currency'], line.amount_currency, context=ctx)
            elif (line.invoice and line.invoice.currency_id.id <> statement['currency']):
                amount = currency_obj.compute(cr, uid, line.invoice.currency_id.id,
                    statement['currency'], amount, context=ctx)
            context.update({'move_line_ids': [line.id]})
            result = voucher_obj.onchange_partner_id(cr, uid, [], partner_id=line.partner_id.id, journal_id=statement['journal_id'], price=abs(amount), currency_id= statement['currency'], ttype=(amount < 0 and 'payment' or 'receipt'), date=time.strftime('%Y-%m-%d'), context=context)
            voucher_res = { 'type':(amount < 0 and 'payment' or 'receipt'),
                            'name': line.name,
                            'partner_id': line.partner_id.id,
                            'journal_id': statement['journal_id'],
                            'account_id': result.get('account_id', statement['default_credit_account_id']), # improve me: statement.journal_id.default_credit_account_id.id
                            'company_id':statement['company_id'],
                            'currency_id':statement['currency'],
                            'date':time.strftime('%Y-%m-%d'),
                            'amount':abs(amount),
                            'period_id':statement['period_id']}
            voucher_id = voucher_obj.create(cr, uid, voucher_res, context=context)

            voucher_line_dict =  {}
            if result['value']['line_ids']:
                for line_dict in result['value']['line_ids']:
                    move_line = line_obj.browse(cr, uid, line_dict['move_line_id'], context)
                    if line.move_id.id == move_line.move_id.id:
                        voucher_line_dict = line_dict

            if voucher_line_dict:
                voucher_line_dict.update({'voucher_id': voucher_id})
                voucher_line_obj.create(cr, uid, voucher_line_dict, context=context)

            #Updated the amount of voucher in case of partially paid invoice
            amount_res = voucher_line_dict.get('amount_unreconciled',amount)
            voucher_obj.write(cr, uid, voucher_id, {'amount':amount_res}, context=context)
            if line.journal_id.type == 'sale':
                type = 'customer'
            elif line.journal_id.type == 'purchase':
                type = 'supplier'
            else:
                type = 'general'

            statement_line = {
                #~ 'name': line.name or '?',
                'amount': amount > 0 and amount_res or (amount_res * -1),
                'type': type,
                'partner_id': line.partner_id.id,
                'account_id': line.account_id.id,
                #~ 'ref': line.ref,
                'voucher_id': voucher_id,
                'date': time.strftime('%Y-%m-%d'), #time.strftime('%Y-%m-%d'), #line.date_maturity or,
            }
        return {'value':statement_line}
        
        
    #~ TODO: Crear metodo que borre el pago al cambiar el tipo de linea.
    def onchange_type(self, cr, uid, line_id, partner_id, type, context=None):
        res = {'value': {}}
        obj_partner = self.pool.get('res.partner')
        res = super(account_bank_statement_line, self).onchange_type(cr, uid, \
                line_id, partner_id, type, context=context)
        if type == 'returned_check':
                absc_conf = self.pool.get('account.bank.statement.config')
                absc_conf_id = absc_conf.search(cr, uid, [])
                absc_brw = absc_conf.browse(cr, uid, absc_conf_id, context=context)
                for config in absc_brw:
                    account_id = config.account_id.id or False
                res['value']['account_id'] = account_id
        return res
        
account_bank_statement_line()


class account_statement_from_invoice_lines(osv.osv_memory):
    """
    Generate Entries by Statement from Invoices
    """
    _inherit = "account.statement.from.invoice.lines"

    def populate_statement(self, cr, uid, ids, context=None):
        """Funcion que pertenece al wizard de account_voucher en el
        archivo account_statement_from_invoice.py se sobrescribe para
        agregar el campo de invoice_id al momento de crear la linea
        del extracto bancario"""
        if context is None:
            context = {}
        statement_id = context.get('statement_id', False)
        if not statement_id:
            return {'type': 'ir.actions.act_window_close'}
        data =  self.read(cr, uid, ids, context=context)[0]
        line_ids = data['line_ids']
                
        if not line_ids:
            return {'type': 'ir.actions.act_window_close'}

        line_obj = self.pool.get('account.move.line')
        statement_obj = self.pool.get('account.bank.statement')
        statement_line_obj = self.pool.get('account.bank.statement.line')
        currency_obj = self.pool.get('res.currency')
        voucher_obj = self.pool.get('account.voucher')
        voucher_line_obj = self.pool.get('account.voucher.line')
        line_date = time.strftime('%Y-%m-%d')
        statement = statement_obj.browse(cr, uid, statement_id, context=context)

        # for each selected move lines
        for line in line_obj.browse(cr, uid, line_ids, context=context):
            voucher_res = {}
            ctx = context.copy()
            #  take the date for computation of currency => use payment date
            ctx['date'] = line_date
            amount = 0.0

            if line.debit > 0:
                amount = line.debit
            elif line.credit > 0:
                amount = -line.credit

            if line.amount_currency:
                amount = currency_obj.compute(cr, uid, line.currency_id.id,
                    statement.currency.id, line.amount_currency, context=ctx)
            elif (line.invoice and line.invoice.currency_id.id <> statement.currency.id):
                amount = currency_obj.compute(cr, uid, line.invoice.currency_id.id,
                    statement.currency.id, amount, context=ctx)
           
            context.update({'move_line_ids': [line.id]})
            result = voucher_obj.onchange_partner_id(cr, uid, [], partner_id=line.partner_id.id, journal_id=statement.journal_id.id, price=abs(amount), currency_id= statement.currency.id, ttype=(amount < 0 and 'payment' or 'receipt'), date=time.strftime('%Y-%m-%d'), context=context)
            voucher_res = { 'type':(amount < 0 and 'payment' or 'receipt'),
                            'name': line.name,
                            'partner_id': line.partner_id.id,
                            'journal_id': statement.journal_id.id,
                            'account_id': result.get('account_id', statement.journal_id.default_credit_account_id.id), # improve me: statement.journal_id.default_credit_account_id.id
                            'company_id':statement.company_id.id,
                            'currency_id':statement.currency.id,
                            'date':line.date,
                            'amount':abs(amount),
                            'period_id':statement.period_id.id}
            voucher_id = voucher_obj.create(cr, uid, voucher_res, context=context)

            voucher_line_dict =  {}
            if result['value']['line_ids']:
                for line_dict in result['value']['line_ids']:
                    move_line = line_obj.browse(cr, uid, line_dict['move_line_id'], context)
                    if line.move_id.id == move_line.move_id.id:
                        voucher_line_dict = line_dict

            if voucher_line_dict:
                voucher_line_dict.update({'voucher_id': voucher_id})
                voucher_line_obj.create(cr, uid, voucher_line_dict, context=context)

            #Updated the amount of voucher in case of partially paid invoice
            amount_res = voucher_line_dict.get('amount_unreconciled',amount)
            voucher_obj.write(cr, uid, voucher_id, {'amount':amount_res}, context=context)
                
            if line.journal_id.type == 'sale':
                type = 'customer'
            elif line.journal_id.type == 'purchase':
                type = 'supplier'
            else:
                type = 'general'
            statement_line_obj.create(cr, uid, {
                'name': line.name or '?',
                'amount': amount > 0 and amount_res or (amount_res * -1),
                'type': type,
                'partner_id': line.partner_id.id,
                'account_id': line.account_id.id,
                'statement_id': statement_id,
                'ref': line.ref,
                'voucher_id': voucher_id,
                'invoice_id': line.invoice.id,
                'date': time.strftime('%Y-%m-%d'), #time.strftime('%Y-%m-%d'), #line.date_maturity or,
            }, context=context)
        return {'type': 'ir.actions.act_window_close'}

account_statement_from_invoice_lines()


class account_bank_statement_config(osv.osv):
    _name = 'account.bank.statement.config'
    
    _columns = {
        'name': fields.char('Descripcion', size=64),
        'account_id':fields.many2one('account.account', 'Account', required=True,),
    }
    
account_bank_statement_config()


class account_voucher(osv.osv):
    _inherit = "account.voucher"
    
    def cancel_voucher(self, cr, uid, ids, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]
        return super(account_voucher, self).cancel_voucher(cr, uid, ids, context=context)
    
account_voucher()
