#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import fields, osv
import netsvc
import datetime
import base64
import unicodedata

class registro_pago(osv.osv):
    logger = netsvc.Logger()
    _name = "registro.pago"
    _rec_name = "reference"

    _columns = {
        'reference': fields.char('Referencia', size=64, select=True, required=True),
        'invoice_ids': fields.many2many('account.invoice', 'rel_registro_pago_inv', 
            'registro_id', 'invoice_id', 'Facturas'),
        'bank_id': fields.many2one('res.bank', 'Banco'),
        'partner_id': fields.many2one('res.partner', 'Partner Company', required=True),
        'account_bank_id': fields.many2one('res.partner.bank', 'Cuenta Bancaria'),
        'total': fields.float('Total'),
        'pago_linea_ids': fields.one2many('registro.pago.linea', 'pago_id', 'Lineas'),
        'tipo': fields.selection([
            ('pago', 'Pago'),
            ('cobranza', 'Cobranza'),
            ], 'Tipo Registro', readonly=True, select=True),
        'state': fields.selection([
            ('draft', 'Borrador'),
            ('done', 'Archivo Creado'),
            ('cancel', 'Cancelado')
            ], 'Estado', readonly=True, select=True),
    }

    _defaults = {
        'reference': lambda self, cr, uid, context: '/',
        'total': lambda self, cr, uid, context: 0.0,
        'state': 'draft',
    }
    
    
    def _write_attachment(self, cr, uid, id, root, vals, context=None):
        """
        Encrypt txt, save it to the db and view it on the client as an attachment
        """
        if context is None:
            context = {}
        fecha = datetime.datetime.now().strftime('%Y-%m-%d')
        name = vals['bank'] + '-' + vals['ref'] + '-' + fecha + '.txt'

        id_att = self.pool.get('ir.attachment').create(cr, uid, {
            'name': name,
            'res_name': name,
            'datas': base64.encodestring(root),
            'datas_fname': name,
            'res_model': 'registro.pago',
            'res_id': str(id),
        }, context=context)
        
        return True


    #~ def _get_atributo(self, )


    def generate_file(self, cr, uid, ids, context=None):
        """
        Primero se verifican que todas las lineas tengan cuenta bancaria.
        Luego, se toma la definicion de cabecera/detalle del archivo para el
        banco seleccionado.
        """

        if context is None:
            context = {}

        inv_obj = self.pool.get('account.invoice')
        bank_obj = self.pool.get('res.bank')
        
        
        company_brw = None
        reg_brw = self.browse(cr, uid, ids, context= context)[0]
        try:
            company_brw = reg_brw.partner_id.company_id
        except Exception, e:
            company_brw = self.pool.get('res.company').browse(cr, uid, 1)
        
        invoice_count = 0
        # Verificar lineas
        for lin in reg_brw.pago_linea_ids:
            invoice_count = invoice_count + 1
            if not lin.account_bank_id:
                msg = "Verifique que todas las lineas tienen una cuenta bancaria. Verificar (%s)"%(lin.invoice_id.number)
                raise osv.except_osv(('Error!'),(msg))
            #~ if lin.invoice_id.residual <= 0.00:
                #~ msg = "Verificar (%s), la factura tiene un monto menor o igual a 0, no se le realizara transferencia."%(lin.invoice_id.number)
                #~ raise osv.except_osv(('Error!'),(msg))
        
        bank_brw = bank_obj.browse(cr, uid, [reg_brw.bank_id.id], context=context)[0]
        if reg_brw.tipo == 'pago':
            hdr_eval_str = str(bank_brw.pago_txt_hdr).rstrip()
            dtl_eval_str = str(bank_brw.pago_txt_detail).rstrip()
        else:
            hdr_eval_str = str(bank_brw.cobranza_txt_hdr).rstrip()
            dtl_eval_str = str(bank_brw.cobranza_txt_detail).rstrip()

        hdr_eval_globals = {
            'total': round(reg_brw.total, 2),
            'line_count': invoice_count,
            'company': company_brw,
            'ref': reg_brw.reference,
            'today': datetime.datetime.now().strftime('%Y-%m-%d'),
            'bank': bank_brw,
            'cuenta_bancaria': reg_brw.account_bank_id.acc_number,
        }
        
        res = ''
        try:
            res += eval(hdr_eval_str, hdr_eval_globals) + '\n'
            for lin in reg_brw.pago_linea_ids:
                dtl_eval_globals = {
                    'invoice': lin.invoice_id,
                    'name': ''.join((c for c in unicodedata.normalize('NFD', lin.invoice_id.partner_id.name) if unicodedata.category(c) != 'Mn')),
                    'cuenta_cliente': lin.account_bank_id.acc_number,
                    'email': lin.invoice_id.address_invoice_id.email or '',
                    #~ 'banco_local': _get_atributo,
                }
                res += eval(dtl_eval_str, dtl_eval_globals) + '\n'
        except Exception, e:
            raise osv.except_osv(_('Alert!'),_(e))
        
        values = {
            'bank': bank_brw.name,
            'ref': reg_brw.reference
        }

        self._write_attachment(cr, uid, reg_brw.id, res, values, context=context)
        
        self.write(cr, uid, [reg_brw.id], {'state': 'done'}, context=context)
        return True


    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
        if ('reference' not in vals) or (vals.get('reference') == '/'):
            vals['reference'] = self.pool.get('ir.sequence').get(cr, user, 'registro_pago_tseq')

        new_id = super(registro_pago, self).create(cr, user, vals, context)

        if not type(new_id) == type([]):
            new_id = [new_id]

        return new_id[0]

registro_pago()


class registro_pago_linea(osv.osv):
    logger = netsvc.Logger()
    _name = "registro.pago.linea"
    _rec_name = "invoice_id"
    _order = "invoice_id"

    _columns = {
        'invoice_id': fields.many2one('account.invoice', 'Factura'),
        'partner_id': fields.related('invoice_id','partner_id', type='many2one', relation='res.partner', string='Partner', readonly=True),
        'amount_total': fields.related('invoice_id','amount_total', type='float', string='Total', readonly=True),
        'residual': fields.related('invoice_id','residual', type='float', string='Pendiente', readonly=True),
        'reference': fields.related('invoice_id','reference', type='char', string='Referencia Factura', readonly=True),
        'date_invoice': fields.related('invoice_id','date_invoice', type='date', string='Fecha Factura', readonly=True),
        'date_due': fields.related('invoice_id','date_due', type='date', string='Fecha Vencimiento', readonly=True),
        'account_bank_id': fields.many2one('res.partner.bank', 'Cuenta Bancaria'),
        'pago_id': fields.many2one('registro.pago', 'Registro de Pago'),
    }
    
    _sql_constraints = [
        ('registro_linea_invoice_uniq', 'unique (invoice_id)', 'La factura ya ha sido procesada')
    ]
    
registro_pago_linea()
