#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import fields, osv
import tools
from tools.translate import _
import netsvc
import datetime
import base64
import unicodedata

class wizard_cobranza_en_linea(osv.osv_memory):
    _name = "wizard.cobranza.en.linea"

    def to_cobranza(self, cr, uid, values, context=None):
        inv_obj = self.pool.get('account.invoice')
        mod_obj = self.pool.get('ir.model.data')
        reg_lin_obj = self.pool.get('registro.pago.linea')
        
        invoice_ids = context.get('active_ids', [])
        res_lin_ids = reg_lin_obj.search(cr, uid, [('invoice_id', 'in', invoice_ids)])
        list_inv = [x.invoice_id.id for x in reg_lin_obj.browse(cr, uid, res_lin_ids, context=context)]
        invoice_ids = list(set(invoice_ids)-set(list_inv))
        inv_brw = inv_obj.browse(cr, uid, invoice_ids, context=context)
        invoices = []
        for inv in inv_brw:
            if inv.state == 'open':
                invoices.append(inv.id)
        
        if invoices == []:
            raise osv.except_osv(('Error!'),('Ninguna de las facturas seleccionadas se les puede crear una transferencia'))
        
        wizard = {
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'cobranza.en.linea',
            'type': 'ir.actions.act_window',
            'target': 'new',
            'domain': str([('invoice_ids', '=', invoices)]),
        }
        
        inv_brw = inv_obj.browse(cr, uid, invoices, context=context)
        inv_types = list(set([i.type for i in inv_brw]))
        
        if len(inv_types) > 1:
            raise osv.except_osv(('Error!'),('No se puede combinar diferentes tipos de facturas'))
        elif inv_types == ['in_invoice']:
            model_data_ids = mod_obj.search(cr, uid, [('model', '=', 'ir.ui.view'),('name', '=', 'view_cobro_en_linea_pago')], context=context)
            resource_id = mod_obj.read(cr, uid, model_data_ids, fields=['res_id'], context=context)[0]['res_id']
            wizard.update({'views': [(resource_id, 'form')], 'context': {'invoices': invoices, 'tipo': 'pago'}})
        elif inv_types == ['out_invoice']:
            model_data_ids = mod_obj.search(cr, uid, [('model', '=', 'ir.ui.view'),('name', '=', 'view_cobro_en_linea_cobranza')], context=context)
            resource_id = mod_obj.read(cr, uid, model_data_ids, fields=['res_id'], context=context)[0]['res_id']
            wizard.update({'views': [(resource_id,'form')], 'context': {'invoices': invoices, 'tipo': 'cobranza'}})
        else:
            raise osv.except_osv(('Error!'),('No se puede usar notas de credito o debito para este proceso'))
        
        return wizard

wizard_cobranza_en_linea()


class cobranza(osv.osv_memory):
    logger = netsvc.Logger()
    _name = "cobranza.en.linea"

    _columns = {
        'invoice_ids': fields.one2many('account.invoice', 'id'),
        'bank_id': fields.many2one('res.bank',),
        'account_bank_id': fields.many2one('res.partner.bank', 'Cuenta Bancaria'),
        'partner_id': fields.many2one('res.partner', 'Partner Company', required=True),
    }
    
    _defaults = {
        'partner_id': lambda self,cr,uid,c: self.pool.get('res.company').browse(cr, uid, self.pool.get('res.company')._company_default_get(cr, uid, 'cobranza.en.linea', context=c)).partner_id.id,
    }
    
    def generate_registro(self, cr, uid, ids, context=None):
        """
        Genera un registro de pago con las facturas y cuentas bancarias
        asociadas que se puedan usar si existen.
        """
        if context is None:
            context = {}
        
        inv_obj = self.pool.get('account.invoice')
        registro_brw = self.pool.get('registro.pago')
        registro_lin_brw = self.pool.get('registro.pago.linea')
        
        cobranza_brw = self.browse(cr, uid, ids)[0]
        total = sum([x.residual for x in inv_obj.browse(cr, uid, context.get('invoices',[]), context=context)])
        registro_id = registro_brw.create(cr, uid, 
            {   
                'reference': '/',
                'invoice_ids': [(6, 0, context.get('invoices',[]))],
                'bank_id': cobranza_brw.bank_id.id,
                'partner_id': cobranza_brw.partner_id.id,
                'account_bank_id': cobranza_brw.account_bank_id.id,
                'tipo': context.get('tipo', ''),
                'total': total,
            })

        for inv in context.get('invoices',[]):
            registro_lin_brw.create(cr, uid, {
                    'invoice_id': inv,
                    'pago_id': registro_id,
                })
        
        return {
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'registro.pago',
            'type': 'ir.actions.act_window',
            #~ 'target': 'edit',
            'target': 'current',
            'res_id': registro_id,
            'active_id': registro_id,
            'ref': 'sisb_banking.view_registro_pago_form',
        }

cobranza()
