#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################################
#    SISB, Sistema Integral Simón Bolívar
#    Copyright (C) 2011-2013 Industrias Diana, C.A. (<http://www.industriasdiana.gob.ve>).
#    All Rights Reserved
###############Credits######################################################################
#    Developed by: Industrias Diana, C.A.
############################################################################################

import time
from lxml import etree
import netsvc
from osv import osv, fields
import decimal_precision as dp
from tools.translate import _

class sisb_compensaciones_config(osv.osv):
    _name = "sisb.compensaciones.config"

    """ Clase sisb_compensaciones_config:
    Este modelo se utiliza para la configuracion de compensaciones contables
    """
    _columns = {
            'name': fields.char("Nombre",size=64),
            'journal_id': fields.many2one("account.journal","Diario"),
            'account_id': fields.many2one("account.account","Cuenta"),
            }

sisb_compensaciones_config()

class sisb_compensaciones(osv.osv):
    _name = "sisb.compensaciones"
    
    """ Clase sisb_compensaciones:
    Este modelo se utiliza para las compensaciones contables y la
    conciliacion automatica entre las mismas.

    Funciones:
    - validar_lineas_cxc.
    - validar_lineas_cxp.
    - validar_facturas.
    - crear_asientos.
    - procesar_compensacion.
    - _compute_lines.
    - _get_period.
    - _get_journal.

    """
    def _compute_lines_cxc(self, cr, uid, ids, name, args, context=None):
        """ Esta Funcion busca los asientos que se han reconciliado con
        la factura"""
        result = {}
        invoice_ids = []
        linea_ids = []
        move_linea_ids = []
        # ~ Extrae el Diario 
        conf_obj = self.pool.get('sisb.compensaciones.config') 
        conf_id = conf_obj.search(cr, uid,[])
        journal = conf_obj.read(cr, uid, conf_id, ['journal_id'])
        journal = journal[0]['journal_id'][0]

        # ~ Busqueda de asientos conciliados
        compensacion_brw = self.browse(cr, uid, ids, context)
        for header in compensacion_brw:
            move_id = header.move_id.id or False
            for line in header.sale_invoice_ids: 
                invoice_ids.append(line.id)
            invoice_brw = self.pool.get('account.invoice').browse(cr,\
                        uid, invoice_ids, context=None)
            for invoice in invoice_brw:
                lines = []
                if invoice.move_id:
                    for m in invoice.move_id.line_id:
                        temp_lines = []
                        if m.reconcile_id:
                            temp_lines = [x.id for x in m.reconcile_id.line_id if x.journal_id.id == journal and x.move_id.id == move_id]
                        elif m.reconcile_partial_id:
                            temp_lines = [x.id for x in m.reconcile_partial_id.line_partial_ids if x.journal_id.id == journal and x.move_id.id == move_id]
                        if temp_lines:
                            lines.append(temp_lines)
                if lines:
                    lines = list(set(reduce(lambda x,y: x+y, lines)))
                    move_linea_ids.append(lines)

            if move_linea_ids:
                move_linea_ids = list(set(reduce(lambda x,y: x+y, move_linea_ids)))

            result[header.id] = move_linea_ids
        return result

   
    def _compute_lines_cxp(self, cr, uid, ids, name, args, context=None):
        """ Esta Funcion busca los asientos que se han reconciliado con
        la factura"""
        result = {}
        invoice_ids = []
        linea_ids = []
        move_linea_ids = []
        move_id = ''
        # ~ Extrae el Diario 
        conf_obj = self.pool.get('sisb.compensaciones.config') 
        conf_id = conf_obj.search(cr, uid,[])
        journal = conf_obj.read(cr, uid, conf_id, ['journal_id'])
        journal = journal[0]['journal_id'][0]

        # ~ Busqueda de asientos conciliados
        compensacion_brw = self.browse(cr, uid, ids, context)
        for header in compensacion_brw:
            move_id = header.move_id.id or False
            for line in header.purchase_invoice_ids: 
                invoice_ids.append(line.id)
            invoice_brw = self.pool.get('account.invoice').browse(cr,\
                        uid, invoice_ids, context=None)
            for invoice in invoice_brw:
                lines = []
                if invoice.move_id:
                    for m in invoice.move_id.line_id:
                        temp_lines = []
                        if m.reconcile_id:
                            temp_lines = [x.id for x in m.reconcile_id.line_id if x.journal_id.id == journal and x.move_id.id == move_id]
                        elif m.reconcile_partial_id:
                            temp_lines = [x.id for x in m.reconcile_partial_id.line_partial_ids if x.journal_id.id == journal and x.move_id.id == move_id]
                        if temp_lines:
                            lines.append(temp_lines)
                if lines:
                    lines = list(set(reduce(lambda x,y: x+y, lines)))
                    move_linea_ids.append(lines)

            if move_linea_ids:
                move_linea_ids = list(set(reduce(lambda x,y: x+y, move_linea_ids)))

            result[header.id] = move_linea_ids
        return result


    def _get_period(self, cr, uid, context=None):
        """ Funcion que trae el periodo en curso """
        period_obj = self.pool.get('account.period')
        return period_obj.find(cr, uid)[0]

    def _get_journal(self, cr, uid, context=None):
        """Funcion que trae el diario configurado para
        las amortizaciones """
        journal_obj = self.pool.get('sisb.compensaciones.config')
        journal_ids = journal_obj.search(cr, uid, [])
        journal_brw = journal_obj.browse(cr, uid, journal_ids, context)
        return journal_brw[0].journal_id.id

    _order = "id desc"

    _columns = {
            'name': fields.char("Ref",size=64),
            'purchase_invoice_ids': fields.many2many('account.invoice','sisb_compensaciones_purchase_rel','invoice_id','purchase_invoice_ids','Facturas de Compras', readonly=True, states={'draft':[('readonly',False)]}),
            'sale_invoice_ids': fields.many2many('account.invoice','sisb_compensaciones_sale_rel','invoice_id','sale_invoice_ids','Facturas de Compras', readonly=True, states={'draft':[('readonly',False)]}),
            'journal_id': fields.many2one("account.journal","Diario"),
            'period_id': fields.many2one("account.period", "Periodo"),
            'partner_id': fields.many2one("res.partner", "Partner"),
            'payment_ids': fields.function(_compute_lines_cxp, method=True, relation='account.move.line', type="many2many", string='Payments'),
            'payment_cxc_ids': fields.function(_compute_lines_cxc, method=True, relation='account.move.line', type="many2many", string='Payments'),
            'move_id': fields.many2one("account.move","Asiento Contable"),
            'amount': fields.float('Monto'),
            'state': fields.selection([
            ('draft','Draft'),
            ('concilied','Concilied'),
            ('cancel','Cancelled')
            ],'State', select=True, readonly=True),
            }

    _defaults = {
            'state':'draft', 
            'period_id': _get_period,
            'journal_id': _get_journal,
            'name':'/',
            }

    def validar_lineas_cxc(self, cr, uid, ids, context=None):
        """Valida que existan facturas cxc, si no existen
        devuelve False, de lo contrario devuelve True.
        """
        iva_inv = []
        islr_inv = []
        for line in self.browse(cr, uid, ids, context=context):
            if not line.sale_invoice_ids:
                return False
            for inv in line.sale_invoice_ids:
                #for iva in [inv.wh_iva_id]:
                if inv.wh_iva_id and inv.wh_iva_id.state <> 'done':
                    iva_inv.append(inv.number)
                #for islr in inv.islr_wh_doc_id:
                if inv.islr_wh_doc_id and inv.islr_wh_doc_id.state <> 'done':
                    islr_inv.append(inv.number)
        if iva_inv:
            raise osv.except_osv(_('Integrity Error !'), _('Las Facturas %s posee retenciones de IVA sin aprobar'%iva_inv))
        if islr_inv:
            raise osv.except_osv(_('Integrity Error !'), _('Las Facturas %s posee retenciones de ISLR sin aprobar'%islr_inv))
 
        return True

    def validar_lineas_cxp(self, cr, uid, ids, context=None):
        """Valida que existan facturas cxp, si no existen
        devuelve False, de lo contrario devuelve True.
        """
        iva_inv = []
        islr_inv = []
        doc_brw = self.browse(cr, uid, ids, context=context)
        for line in doc_brw:
            if not line.purchase_invoice_ids:
                return False
            for inv in line.purchase_invoice_ids:
                #for iva in [inv.wh_iva_id]:
                if inv.wh_iva_id and inv.wh_iva_id.state <> 'done':
                    iva_inv.append(inv.number)
                #for islr in inv.islr_wh_doc_id:
                if inv.islr_wh_doc_id and inv.islr_wh_doc_id.state <> 'done':
                    islr_inv.append(inv.number)
        if iva_inv:
            raise osv.except_osv(_('Integrity Error !'), _('Las Facturas %s posee retenciones de IVA sin aprobar'%iva_inv))
        if islr_inv:
            raise osv.except_osv(_('Integrity Error !'), _('Las Facturas %s posee retenciones de ISLR sin aprobar'%islr_inv))

        return True


    def validar_rectificativas(self, cr, uid, ids, context=None):
        rect_ids = []
        inv_ids = []
        nc_ids = []
        doc_brw = self.browse(cr, uid, ids, context)
        for line in doc_brw:
            inv_ids = [ln.id for ln in line.purchase_invoice_ids]
            inv_ids += [ln.id for ln in line.sale_invoice_ids]
            rect_ids = self.pool.get('account.invoice').search(cr, uid,\
                    [('parent_id','in',inv_ids),\
                    ('type','in',['out_refund','in_refund']),\
                    ('state','=','draft')])
            if rect_ids:
                return rect_ids
        return False


    def validar_facturas(self, cr, uid, ids, context=None):
       """Valida si hay facturas sobrantes en el documento
       de compensaciones 
       """
       tipo =  ""
       amount_pur_inv = 0.00
       amount_sal_inv = 0.00
       doc_brw = self.browse(cr, uid, ids, context)
       for line in doc_brw:
           for pur_inv in line.purchase_invoice_ids:
               amount_pur_inv += pur_inv.residual
           for sal_inv in line.sale_invoice_ids:
               amount_sal_inv += sal_inv.residual

       if amount_pur_inv > amount_sal_inv:
           residual = amount_pur_inv - amount_sal_inv
           tipo = 'purchase'
       elif amount_sal_inv > amount_pur_inv:
           residual = amount_sal_inv - amount_pur_inv
           tipo = 'sale'
       if tipo:
           if tipo == 'purchase':
               for line in doc_brw:
                   for pur_inv in line.purchase_invoice_ids:
                       if pur_inv.residual <= residual:
                           raise osv.except_osv(_('Integrity Error !'), _('La Factura de Compras Nro %s sobra en el documento'%pur_inv.number))
           else:
               for line in doc_brw:
                   for sal_inv in line.sale_invoice_ids:
                       if sal_inv.residual <= residual:
                           raise osv.except_osv(_('Integrity Error !'), _('La Factura de Ventas Nro %s sobra en el documento'%sal_inv.number))
       return True

    def procesar_compensacion(self, cr, uid, ids, context=None):
        """Procesa y ejecuta los precesos necesarios para la
        conciliacion automatica de las compensaciones. """
        rect = self.validar_rectificativas(cr, uid, ids, context=context)
        if rect:
            raise osv.except_osv(_('Error'), _('Las facturas poseen \
                    notas de credito sin aplicar'))
        if not self.validar_lineas_cxp(cr, uid, ids, context=context):
            raise osv.except_osv(_('Integrity Error !'), _('You can not \
                    process a write without CXP lines')) 
        if not self.validar_lineas_cxc(cr, uid, ids, context=context):
            raise osv.except_osv(_('Integrity Error !'), _('You can not \
                    process a write without CXC lines'))
        self.validar_facturas(cr, uid, ids, context=context) #Valida que no sobren facturas
        doc_brw = self.browse(cr, uid, ids, context=context)

        if doc_brw:
            for line in doc_brw:
                if len(line.purchase_invoice_ids) > 30:
                   raise osv.except_osv(_('Integrity Error!'), _('Solo puede \
                         compensar un maximo de 30 facturas por pagar, por \
                         documento'))

                if len(line.sale_invoice_ids) > 30:
                    raise osv.except_osv(_('Integrity Error!'), _('Solo puede \
                        compensar un maximo de 30 facturas por pagar, por \
                        documento'))

            if not self.crear_asientos(cr, uid, doc_brw, context=context):
                raise osv.except_osv(_('Integrity Error!'), _('El proceso de \
                        conciliacion no ha sido exitoso'))

        number = self.pool.get('ir.sequence').get(cr, uid,'sisb.compensaciones.code')
        self.write(cr, uid, ids, {'state':'concilied','name':number,})
        return True

    def crear_asientos(self, cr, uid, doc, context=None):
        """ Genera los asientos contables 
        para las amortizaciones """
        am_obj = self.pool.get("account.move")
        aml_obj = self.pool.get("account.move.line")
        seq_obj = self.pool.get('ir.sequence')
        ai_obj = self.pool.get('account.invoice')
        aj_obj = self.pool.get('account.journal')
        
        purchase_amount_total = 0.00
        sale_amount_total = 0.00
        diff_move_amount = 0.00
        comp_amount = 0.00
        date_reg = ''
        period_id = ''
        partner_id = ''
        purchase_inv_ids = []
        concil_ids = []
        move_line_ids = []
        sale_inv_ids = []
        result_dict = []
        comp_id = ''

        #Busca el diario configurado para las compensaciones 
        aac_conf = self.pool.get('sisb.compensaciones.config')
        aac_conf_id = aac_conf.search(cr, uid, [])
        aac_brw = aac_conf.browse(cr, uid, aac_conf_id, context=context)
        for journal in aac_brw:
            journal_id = journal.journal_id.id
            journal_sequence_id = journal.journal_id.sequence_id.id

        #TODO: improve the code loops
        for doc_header in doc: 
            period_id = doc_header.period_id.id
            partner_id = doc_header.partner_id.id
            comp_id = doc_header.id

            for doc_line in doc_header.purchase_invoice_ids:
                purchase_amount_total += doc_line.residual
                purchase_inv_ids.append(doc_line.id)

            for doc_line in doc_header.sale_invoice_ids:
                sale_amount_total += doc_line.residual
                sale_inv_ids.append(doc_line.id)

        if purchase_amount_total < sale_amount_total:
            res_pur_total = purchase_amount_total
            res_sal_total = purchase_amount_total
        #~ Se agrega solo para el saldo de la compensacion.
            comp_amount = purchase_amount_total
        else:
            res_sal_total = sale_amount_total
            res_pur_total = sale_amount_total
            comp_amount = sale_amount_total

        #~ Se crea la cabecera de la compensacion.
        move_id = am_obj.create(cr, uid, {
                'name': seq_obj.get_id(cr, uid, journal_sequence_id),
                'journal_id': journal_id,
                'date':  time.strftime('%Y-%m-%d'),
                'ref': 'Compensacion Asiento Intermedio',
                'period_id': period_id or False,
                }, context)

        #TODO: Resumir la creacion del asiento en una funcion.
        """Busco los IDS de los asientos de la
        cuenta por cobrar asociada a la factura
        de CXP """
        ai_brw = ai_obj.browse(cr, uid, purchase_inv_ids, context)
        for inv in ai_brw:
            for move in [inv.move_id]:
                for lines in move.line_id:
                    if lines.account_id.id == inv.account_id.id and lines.credit > 0.00:
                        purchase_account_inv_id = lines.account_id.id
                        concil_ids.append(lines.id)
            aml_id = aml_obj.create(cr, uid, {
                        'name': 'Compensacion de Fact. CXP',
                        'debit': inv.residual if inv.residual <= res_pur_total else res_pur_total,
                        'credit': 0.00,
                        'invoice_id': inv.id,
                        'account_id': purchase_account_inv_id, 
                        'move_id': move_id,
                        'journal_id': journal_id,
                        'period_id': period_id,
                        'partner_id': partner_id, 
                        'currency_id': False,
                        'date': time.strftime('%Y-%m-%d'), 
                        'date_maturity': time.strftime('%Y-%m-%d'),
                        }, context=None)
            #~ Diccionario para distribuir la imputacion
            result_dict.append({'id':inv.id,'monto': inv.residual if \
                    inv.residual <= res_pur_total else res_pur_total,})
            res_pur_total -= inv.residual
            #~ Concilia la linea de compensacion del debito contra la cxc de facturas.
            concil_ids.append(aml_id)
            move_line_ids.append(concil_ids)
            concil_ids = []

        """Busco los IDS de los asientos de la
        cuenta por cobrar asociada a la factura
        de CXC """
        ai_brw = ai_obj.browse(cr, uid, sale_inv_ids, context)
        for inv in ai_brw:
            for move in [inv.move_id]:
                for lines in move.line_id:
                    if lines.account_id.id == inv.account_id.id and lines.debit > 0.00:
                        sale_account_inv_id = lines.account_id.id
                        concil_ids.append(lines.id)
            aml_id = aml_obj.create(cr, uid, {
                        'name': 'Compensacion de Fact. CXC',
                        'debit': 0.00 ,
                        'credit': inv.residual if inv.residual <= res_sal_total else res_sal_total,
                        'account_id': sale_account_inv_id, 
                        'move_id': move_id,
                        'invoice_id': inv.id,
                        'journal_id': journal_id,
                        'period_id': period_id,
                        'partner_id': partner_id, 
                        'currency_id': False,
                        'date': time.strftime('%Y-%m-%d'), 
                        'date_maturity': time.strftime('%Y-%m-%d'),
                        }, context=None)
            #~ Diccionario para distribuir la imputacion
            result_dict.append({'id':inv.id,'monto': inv.residual if \
                    inv.residual <= res_sal_total else res_sal_total,})
            res_sal_total -= inv.residual
            concil_ids.append(aml_id)
            move_line_ids.append(concil_ids)
            concil_ids = []

        for line in move_line_ids:
            aml_obj.reconcile_partial(cr, uid, line, 'auto',context=context)

        #~ Guarda el monto compensado
        if comp_id:
            self.write(cr, uid, comp_id, {'amount':comp_amount, 'move_id':move_id,}, context=context) 

        return am_obj.button_validate(cr, uid, [move_id], context), result_dict

    def copy(self, cr, uid, ids, defaults, context=None):
        """ Sobreescritura del metodo copy del orm del
        modelo de compensaciones para evitar duplicar 
        los documentos """
        raise osv.except_osv(_('Error !'), _('No se permite duplicar el documento'))
        return True

sisb_compensaciones()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
