#!/usr/bin/python
# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############Credits########################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

import time
from report import report_sxw
from osv import osv
import pooler
from datetime import datetime, timedelta
import datetime as dt

class compensaciones_parser(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(compensaciones_parser, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'time': time,
            'user': self.pool.get('res.users').browse(cr, uid, uid, context),
            'get_empresa': self.get_empresa,
            'get_montos_compensados': self.get_montos_compensados,
            'func_total_cxp': self.func_total_cxp,
            'func_total_cxc': self.func_total_cxc,
            'residual_cxp': self.residual_cxp,
            'residual_cxc': self.residual_cxc,
            
        })
        self.total_cxp = 0.00
        self.total_cxc = 0.00
        self.total_res_cxc = 0.00
        self.total_res_cxp = 0.00
        
    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
    
    def get_montos_compensados(self, invoice_id, move_id):
        conf_obj = self.pool.get('sisb.compensaciones.config') 
        aml_obj = self.pool.get('account.move.line')
        move_linea_ids = []
        type_inv = ''
        result = {}
    
        conf_id = conf_obj.search(self.cr, self.uid,[])
        journal = conf_obj.read(self.cr,self.uid, conf_id, ['journal_id'])
        journal = journal[0]['journal_id'][0]
        invoice_brw = self.pool.get('account.invoice').browse(self.cr, self.uid, [invoice_id], context=None)
        for invoice in invoice_brw:
            lines = []
            type_inv = invoice.type
            if invoice.move_id:
                for m in invoice.move_id.line_id:
                    temp_lines = []
                    if m.reconcile_id:
                        temp_lines = [x.id for x in m.reconcile_id.line_id if x.journal_id.id == journal and x.move_id.id == move_id]
                    elif m.reconcile_partial_id:
                        temp_lines = [x.id for x in m.reconcile_partial_id.line_partial_ids if x.journal_id.id == journal and x.move_id.id == move_id]
                    if temp_lines:
                        lines.append(temp_lines)
            if lines:
                lines = list(set(reduce(lambda x,y: x+y, lines)))
                move_linea_ids.append(lines)

            if type_inv == 'in_invoice':
                self.total_res_cxp += invoice.residual
            elif type_inv == 'out_invoice':
                self.total_res_cxc += invoice.residual
                
        if move_linea_ids:
            move_linea_ids = list(set(reduce(lambda x,y: x+y, move_linea_ids)))

        aml_brw = aml_obj.browse(self.cr, self.uid, move_linea_ids, context=None)

        for mov in aml_brw:
            if type_inv == 'in_invoice':
                result.update({'monto':mov.debit})
                self.total_cxp +=  mov.debit
            elif type_inv == 'out_invoice':
                result.update({'monto':mov.credit})
                self.total_cxc +=  mov.credit
        return result
    
    def get_final_date(self):
        return datetime.now().strftime('%d-%m-%Y')
    
    def get_fecha(self, date):
        if str(date) == '':
            return False
        new = datetime.strptime(date, '%Y-%m-%d')
        return str(new.strftime('%d-%m-%Y'))
        
    def func_total_cxc(self):
        return self.total_cxc
        
    def func_total_cxp(self):
        return self.total_cxp
        
    def residual_cxc(self):
        return self.total_res_cxc
        
    def residual_cxp(self):
        return self.total_res_cxp
        
       
report_sxw.report_sxw('report.sisb.compensaciones',
    'sisb.compensaciones',
    'sisb_compensaciones/report/reporte_sisb_compensaciones.rml',
    parser=compensaciones_parser)
