# -*- coding: utf-8 -*-
from osv import osv
from osv import fields
import base64
from tools.translate import _
import binascii
from StringIO import StringIO
import csv
import time
from datetime import datetime
import xmlrpclib

class comprobante_contable(osv.osv):
    """
    Modulo para la carga de comprobantes contables.
    Crea una tabla en la base de datos para almacenar información con respecto
    al comprobante. La tabla es borrada al final del proceso de carga.
    """
    _name = "comprobante.contable"
    _description = "Account Move Common Report"
    
    _columns = {
        'ref_des': fields.char('Descripcion del Comprobante',size=128),
        'file': fields.binary('Archivo CSV'),
        'f_name': fields.char('Nombre del CSV', size=128),
        'period_id': fields.many2one('account.period','Periodo del comprobante', required=True, select=True),
    }
        
    def get_id_(self, cr, uid, values,obj,field):
        """
        Busca el id de un objeto que se le pase por [obj], donde se cumpla que la 
        variable [field] igual a la variable [values]
        """
        ids = self.pool.get(obj).search(cr,uid,[(str(field),'=',str(values))])
        if ids:
            return int(ids[0])
        else:
            return 0
    
    def upload_account_move(self, cr, uid, defaults, context=None):
        """
        Funcion para la carga del comprobante contable, en el proceso primero se
        sube el archivo a la base de datos para despues ser extraido, convertido
        en un archivo virtual que sera procesado en dos pasos, primero carga del
        account.move y despues account.move.line.
        """
        if context is None:
            context = {}
            
        record_id = self.browse(cr, uid, defaults, context=context)
        
        #Modelos necesarios
        obj_company = self.pool.get('res.company')
        obj_journal = self.pool.get('account.journal')
        obj_period = self.pool.get('account.period')
        obj_moves = self.pool.get('account.move')
        obj_acm = self.pool.get('account.move.line')
        obj_aca = self.pool.get('account.account')
        
        file_in = record_id[0].file
        
        #Se convierte el archivo de binario a lectura estandar y despues se
        #pasa a un archivo virtual
        compr_csv = binascii.a2b_base64(unicode(file_in))
        arch_csv = StringIO(compr_csv)
        
        #INICIAL VALIDACION DEL ARCHIVO
        f_move = csv.DictReader(arch_csv, delimiter='|')
        lineas = 0
        dic = {}
        dic_vista = {}
        dic_deb_cre = []
        ids = []
        fallo = False
        vista = False
        debe_haber = False
        try:
            for row in f_move:
                lineas += 1
                """Se asigna cada cabecera a una variable para validar que este
                en el formato correcto"""
                cuenta = row['cuenta']
                cc = row['cc']
                descripcion = row['descripcion']
                debe = str(row['debe']).replace(' ','')
                if debe == '':
                    debe = '0.00'
                haber = str(row['haber']).replace(' ','')
                if haber == '':
                    haber = '0.00'
                ids = obj_aca.search(cr,uid,[('code','=',cuenta)])
                tipo = obj_aca.read(cr,uid,ids,['type'])
                if not ids:
                    dic[str(cuenta)] = descripcion
                    fallo = True
                elif tipo[0]['type']=='view':
                    dic_vista[str(cuenta)] = descripcion
                    vista =True
                elif float(debe) > 0 and float(haber) > 0:
                    linea = lineas+1
                    dic_deb_cre.append(linea)
                    debe_haber = True
        except Exception, e:
            raise osv.except_osv(_('Error !'), _("Verificar que el archivo sea un CSV.\nLa cabecera tenga el formato: cuenta, cc, descripcion, debe y haber, y este se encuentre separado por '|'.\n Error con: %s")%(e))
            
        if fallo:
            print "No se puede completar la operacion, las siguientes cuentas no existen en el catalogo:"
            for d in dic.keys():
                print "Cuenta: %s \tNombre: %s"%(d,dic[d])
            raise osv.except_osv(_('Error !'), _("El Comprobante no tiene una cuenta definida en el plan contable!. Cuenta no encontrada: %s")%(dic.keys()[0]))
        if vista:
            print "No se va a completar la operacion, las siguientes cuentas son tipo vista:"
            for d in dic_vista.keys():
                print "Cuenta: %s \tNombre: %s"%(d,dic_vista[d])
            raise osv.except_osv(_('Error !'), _("El Comprobante tiene una o mas cuentas de tipo 'Vista'!. Cuenta con error: %s")%(dic_vista.keys()[0]))
        if debe_haber:
            print "No se puede completar la operacion, una de las lineas contiene saldo tanto por el debe como por el haber:"
            for d in dic_deb_cre:
                print "Linea con error: %s"%(d)
            raise osv.except_osv(_('Error !'), _("El Comprobante contiene una o mas lineas que tienen saldo tanto por el debe como por el haber. Linea con error: %s")%(dic_deb_cre[0]))
        #TERMINA VALIDACION DEL ARCHIVO
        
        #Valores que siempre seran igual
        company_id = obj_company._company_default_get(cr, uid, 'account.move', context=context)
        var_diario = obj_journal.search(cr,uid,[('name','=','Diario General'),('type','=','general')])[0]
        var_period = record_id[0].period_id.id
        var_ref = record_id[0].ref_des
        fecha = obj_period.read(cr,uid,var_period,['date_stop'])['date_stop']
        
        #Se carga el account.move UNICA
        moves = {
            'name': "/",
            'ref' : var_ref,
            'to_check': 1,
            'journal_id': var_diario,
            'period_id': var_period,
            'date': fecha,
            'state': 'draft',
            'company_id': int(company_id),
        }
        moves_id = obj_moves.create(cr, uid, moves,context=context)
        
        if type(moves_id) == type([]):
            moves_id = moves_id[0]
        
        #Se vuelve a crear el archivo virtual para reiniciar la lectura
        #Se hace para la carga de los account.move.line
        arch_csv = StringIO(compr_csv)
        f_move = csv.DictReader(arch_csv, delimiter='|')
        
        i = 0
        for mov in f_move:
            
            #Cambiar centro de costos 300 por 100 (antes 030), se hace por diferencial con el CKF
            if str(mov['cc']) == '300':
                mov['cc'] = '100'
            ####################################################################
            
            var_move_id = moves_id
            var_account_id = self.get_id_(cr, uid, mov['cuenta'], 'account.account', 'code')
            var_analytic = self.get_id_(cr,uid,str(mov['cc']),'account.analytic.account','code')
            var_name = mov['descripcion']
            cre = str(mov['haber']).replace(' ','')
            if cre == '':
                cre = '0.00'
            deb = str(mov['debe']).replace(' ','')
            if deb == '':
                deb = '0.00'
            
            """
            Por si la excepcion no fue tomado se hace esta validacion, causa un
            error la carga si la linea contiene saldo tanto en el debito como en
            el credito.
            """
            if float(cre) > 0:
                deb = '0.00'
            else:
                cre = '0.00'
            
            if var_account_id != 0:
                if var_analytic == 0:
                    moves_lines = {
                        'journal_id': var_diario,
                        'credit': float(cre),
                        'debit': float(deb),
                        'account_id': int(var_account_id),
                        'period_id': var_period, 
                        'move_id': int(var_move_id),
                        'name': var_name[:63],
                        'state': 'draft',
                    }
                    i += 1
                    if i == 1:
                        moves_lines_id = obj_acm.create(cr,uid,moves_lines,context=context)
                    elif i == lineas:
                        moves_lines_id = obj_acm.create(cr,uid,moves_lines,context=context)
                    else:
                        cr.execute("insert into account_move_line (account_id,name,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date) values (%s,'%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now())"%(moves_lines['account_id'],moves_lines['name'],moves_lines['journal_id'],int(company_id),moves_lines['credit'],moves_lines['state'],moves_lines['period_id'],moves_lines['debit'],str(fecha),moves_lines['move_id'],uid))
                        cr.commit()
                else:
                    moves_lines = {
                        'journal_id': var_diario,
                        'credit': float(cre),
                        'debit': float(deb),
                        'account_id': int(var_account_id),
                        'period_id': var_period, 
                        'move_id': int(var_move_id),
                        'name': var_name[:63],
                        'state': 'draft',
                        'analytic_account_id': var_analytic,
                    }
                    i += 1
                    if i == 1:
                        moves_lines_id = obj_acm.create(cr,uid,moves_lines,context=context)
                    elif i == lineas:
                        moves_lines_id = obj_acm.create(cr,uid,moves_lines,context=context)
                    else:
                        cr.execute("insert into account_move_line (account_id,name,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,analytic_account_id) values (%s,'%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s',now(),%s,False,%s,now(),%s)"%(moves_lines['account_id'],moves_lines['name'],moves_lines['journal_id'],int(company_id),moves_lines['credit'],moves_lines['state'],moves_lines['period_id'],moves_lines['debit'],str(fecha),moves_lines['move_id'],uid,moves_lines['analytic_account_id']))
                        cr.commit()
            else:
                raise osv.except_osv(_('Error !'), _('El Comprobante no tiene una cuenta definida! %s')%(mov['cuenta']))
        cr.execute('delete from comprobante_contable')
        
        return {
            'name':("Importacion CSV"),
            'view_mode': 'form',
            'view_id': False,
            'view_type': 'form',
            'res_model': 'comprobante.contable',
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'new',
            'ref': 'view_comprobante_contable_end',
        }

comprobante_contable()

class comprobante_contable_report(osv.osv):
    _name = "comprobante.contable.report"
    _description = "Reporte de Comprobantes Contables"
    
    def execute_report(self, cr, uid, values, context=None):
        return True

comprobante_contable_report()
