# -*- coding: utf-8 -*-
##############################################################################
#
#    SISB, Sistema Integral Simon Bolivar
#    Copyright (C) 2012-2013 Industrias Diana, C.A. 
#    (<http://www.industriasdiana.gob.ve>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from suds.client import Client
import logging
import time
from lxml import etree
import netsvc
from osv import osv, fields
import decimal_precision as dp
from tools.translate import _

class res_partner_contractor_config(osv.osv):
    _name = "res.partner.contractor.config"
    
    _columns = {
    'name': fields.char("Nombre Servicio",size=64),
    'wsdl_login': fields.char("WSDL Login",size=256),
    'wsdl_url': fields.char("WSDL Url",size=256),
    'username': fields.char("Usuario", size=64),
    'password': fields.char("Password", size=64),
    'state': fields.selection([
            ('active','Activa'),
            ('inactive','Inactiva'),
            ],'State', select=True, readonly=True),
    }
    _defaults = {
    'state':'active',
    }
res_partner_contractor_config()


class res_partner_contractor_info(osv.osv):
    _name = "res.partner.contractor.info"
    
    _recname = 'contractor_number'
    
    _columns = {
        'parnert_id': fields.many2one('res.partner','Partner', required=True,),
        'contractor_number': fields.char('Nro RNC', size=128),
        'contractor_form': fields.char('Nro Planilla', size=64),
        'contractor_level': fields.char('Nivel Financiero', size=64),
        'date_register': fields.date("Fecha de Inscripcion"),
        'date_expiration': fields.date("Fecha de Vencimiento"),
        'capital_paid': fields.float("Capital pagado"),
        'capital_subscribed': fields.float("Capital Suscrito"),
        'finantial_level': fields.char("Calificacion finaciera",size=64),
        'finantial_level_estimated': fields.float("Capacidad financiera Estimada"),
        'extra_info': fields.char("Informacion",size=512),
        'url_view': fields.char("URL", size=128),
    }
    
res_partner_contractor_info()

class res_partner(osv.osv):
    _inherit = 'res.partner'
    
    _columns = {
        'contractor_ids': fields.one2many('res.partner.contractor.info','parnert_id','Registro RNC'),
    }

    def get_contractor_rnc(self, cr, uid, ids, context=None):
        """
        Funcion que ejecuta consultas al web service del RCN
        y actualiza los datos en la tabla.
        """
        
        conf_obj = self.pool.get("res.partner.contractor.config")
        conf_ids = conf_obj.search(cr, uid, [])
        conf_brw = conf_obj.browse(cr, uid, conf_ids, context=context)
        
        login = []
        rnc_id = None
        
        for row in self.browse(cr, uid, ids):
            vat = row.vat[2:]
            for line in row.contractor_ids:
                rnc_id = line.id
        
        
        for config in conf_brw:
            if config.state == 'active':
                wsdl_url = config.wsdl_url
                wsdl_login = config.wsdl_login
                login.append(config.username)
                login.append(config.password)
            else:
                raise osv.except_osv(_('Integrity Error !'), _('You dont have WSDL service configured'))
                
        try:
            client = Client(wsdl_login)
            session = client.service.autenticacion(login[0],login[1])

            if (session.codigoRespuesta == 'EXITO'):
                token = session.content
            else:
                 raise osv.except_osv(_('Error !'), _('WSDL Service: %s',session.codigoRespuesta))


            client = Client(wsdl_url)
            session = client.service.doFindContratistaWSByRif(vat,login[0],token)
            
        except Exception, e:
            raise osv.except_osv(_('Error !'), _('WSDL Service: %s'%e))
            
        if (session.codigoRespuesta == 'EXITO'):
            content = session.content
            if content:
                numcertrnc = session.content.contratista.numcertrnc
                contractor_dict = {
                    "contractor_number": str(content.contratista.numcertrnc),
                    "contractor_form":  str(content.accionistas[0].id.procesoId),
                    "contractor_level": content.anafinancierre.nivelfinanciero.descniv,
                    "date_register": content.contratista.fecinscrncAt,
                    "date_expiration": content.contratista.fecvencrncAt,
                    "capital_paid": content.regmercantile.cappagado,
                    "capital_subscribed": content.regmercantile.capsusc,
                    "finantial_level": content.anafinancierre.califfec,
                    "finantial_level_estimated": content.anafinancierre.capfec,
                    "extra_info": content.contratista.edocontratista.infoadic,
                    "url_view": 'http://rncenlinea.snc.gob.ve/planilla/index/'+str(content.accionistas[0].id.procesoId)+'?anafinan=N&anafinanpub=Y&login=N&mostrar=INF',
                }

                if rnc_id:
                    self.pool.get('res.partner.contractor.info').unlink(cr, uid, rnc_id, context=context)
                    contractor_id = self.write(cr, uid, ids, {'contractor_ids':[(0,0,contractor_dict)]}, context=context)
                else:
                    contractor_id = self.write(cr, uid, ids, {'contractor_ids':[(0,0,contractor_dict)]}, context=context)
            else:
                raise osv.except_osv(_('Error !'), _('WSDL Service: No se encuentra registrado en el Registro Nacional de Contratistas'))
        else:
            raise osv.except_osv(_('Error !'), _('WSDL Service: %s',session.codigoRespuesta))
        
        return contractor_id

res_partner()
