# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from datetime import datetime
import time
from tools.translate import _
import binascii
import tools

class crm_case_section(osv.osv):
    """Sales Team"""
    _inherit = 'crm.case.section'
    
    def _validate_user_group(self, cr, uid, ids, context=None):
        crm_brw = self.browse(cr, uid, ids, context)
        members_ids = [] 
        memb_ids = [] 
        for crm_ids in crm_brw[0].member_ids:
            members_ids.append(crm_ids.id)
        team_ids = self.search(cr, uid,[('id','not in',ids)])
        team_brw = self.browse(cr, uid, team_ids, context)
        for team in team_brw:
            for list_members in team.member_ids:
                memb_ids.append(list_members.id)
        if set(members_ids).intersection(memb_ids):
            return False
        else:
            return True
        
    _constraints = [( _validate_user_group, 'Error: Ha agregado un usuario que ya esta en otro grupo', ['member_ids'])]

crm_case_section()

class sale_order(osv.osv):
    _inherit = 'sale.order'

    _defaults = {
        'user_id': False,
    }
    
    #Buscar el equipo de venta del usuario
    def find_section_id(self, cr, uid, user_id):
        val = {}
        #Busca en equipo de venta
        team_id = self.pool.get('crm.case.section').search(cr, uid, [('member_ids','in',[user_id])])
        if not team_id:
            #Busca en usuario
            team_id = self.pool.get('res.users').browse(cr, uid, user_id).context_section_id \
                and self.pool.get('res.users').browse(cr, uid, user_id).context_section_id.id or False
            if not team_id:
                val.update({'section_id': 0})
            else:
                val.update({'section_id': team_id})
        else:
            val.update({'section_id': team_id[0]})
        return val
    
    #~ OnChange para Actualizar la lista de Equipos
    def onchange_salesman(self, cr, uid, ids, user_id, shop_id=None, context=None):
        val = {}
        if user_id:
            #para validar permiso para seleccionar el vendedor de la sucursal
            user_perm = self.pool.get('res.users').search(cr, uid, [('id','=',user_id)], context={'shop_id': shop_id})
            if not user_perm:
                warning = {'title': 'Vendedor no permitido!',
                            'message':"Debe seleccionar un vendedor asociado a la sucursal."
                            }
                return {'value': {'user_id': False, 'section_id': 0}, 'warning': warning}
            #Busca en equipo de venta
            val = self.find_section_id(cr, uid, user_id)
            return {'value': val}
        else:
            return {'value': {'section_id': 0}}
            
    #~ OnChange del Cliente 
    def onchange_partner_id(self, cr, uid, ids, part):
        val = {}
        val = super(sale_order, self).onchange_partner_id(cr, uid, ids, part)

        if val['value'].get('user_id',False):
            #Se asigna el vendedor del cliente o se deja vacio para que el usuario escoja el vendedor de acuerdo a la sucursal
            brw_part = self.pool.get('res.partner').browse(cr, uid, part)
            val['value']['user_id'] = brw_part.user_id and brw_part.user_id.id or False
            if val['value']['user_id']:
                #Busca en equipo de venta
                val_dep = self.find_section_id(cr, uid, val['value']['user_id'])
                val['value'].update(val_dep)
                if val['value'].has_key('section_id'):
                    val['value']['section_id'] = brw_part.section_id and brw_part.section_id.id or False
                else:
                    val['value'].update({'section_id':0})
            else:
                val['value'].update({'section_id':0})
        else:
            val['value'].update({'section_id':0})

        return val

    def onchange_shop_id(self, cr, uid, ids, shop_id):
        v = {}
        v = super(sale_order, self).onchange_shop_id(cr, uid, ids, shop_id)

        if v.get('value',False):
            v['value'].update({'user_id':False})
            v['value'].update({'partner_id':False})
            v['value'].update({'partner_invoice_id':False})
            v['value'].update({'partner_order_id':False})
            v['value'].update({'partner_shipping_id':False})


        return v
        
sale_order()

class salesmen_geolocation(osv.osv):
    _name = "salesmen.geolocation"
    
    _columns = {
    'name':fields.char("Nombre Zona",size=64),
    'description':fields.text("Descripcion de la Zona",size=256),
    'salesmen_ids':fields.many2many('res.users', 'salesmen_geolocation_rel', 'salesmen_geolocation_id', 'salesmen_member_id', 'Geolocation Members'),
    'geostages_ids':fields.one2many('salesmen.geostages','salesmen_id','Stages'),
    'note':fields.text("Notas",size=128),
    }
    
salesmen_geolocation()

class salesmen_geostages(osv.osv):
    _name = "salesmen.geostages"
    
    _columns = {
    'name':fields.char("Nombre Punto",size=64),
    'salesmen_id':fields.many2one('salesmen.geolocation','Partida',ondelete='cascade'),
    }

salesmen_geostages()

class res_users(osv.osv):
    _inherit = 'res.users'
    
    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if context is None:
            context = {}
        if context.get('shop_id', False):
            shop_address = False
            brw_shop_address = self.pool.get('sale.shop').browse(cr, uid, context['shop_id'])
            if brw_shop_address.warehouse_id.partner_address_id:
                shop_address= brw_shop_address.warehouse_id.partner_address_id.id
            
            if shop_address:
                args.append(('address_id','=',shop_address))
            else:
                raise osv.except_osv(_('Error de Configuracion'),_('Debe registrar la direccion de la sucursal en el almacen asociado.'))
                return []

        return super(res_users, self).search(cr, uid, args, offset, limit, order, context, count)
    
res_users()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
