# -*- coding: utf-8 -*-
"""
Módulo para dadivas
"""
from osv import fields, osv
import time
from tools.translate import _
import logging
import netsvc

class sale_order(osv.osv):
    _inherit = 'sale.order'


    _columns = {
        'esdadiva': fields.boolean('Es dádiva', readonly=False, help="Es dádiva."),
        }

    _defaults = {
        'picking_policy': 'one',
        'order_policy': 'manual',
    }
    

    def esdadiva(self, cr, uid, ids, valor, shop, context=None):
        
        result = {}
        user_obj = self.pool.get('res.users')
        shop_obj = self.pool.get('sale.shop')
        user_brw = user_obj.browse(cr, uid, [uid])
        if shop:
            shop_brw = shop_obj.browse(cr, uid, [shop])
        else:
            shop_brw = []
        project_id = False
        price_list = False
        for value in user_brw:
            partner_id = value.company_id.partner_id.id
        for value in shop_brw:
            project_id = value.project_id and value.project_id.id or False
            price_list = value.pricelist_id and value.pricelist_id.id or False
            payment_term_id = value.payment_default_id and value.payment_default_id.id or False
        if valor:
            
            if not partner_id:
                return {'value': {'partner_invoice_id': False, 'partner_shipping_id': False, 'partner_order_id': False, 'payment_term': False, 'fiscal_position': False}}

            addr = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery', 'invoice', 'contact'])
            part = self.pool.get('res.partner').browse(cr, uid, partner_id)
            pricelist = part.property_product_pricelist and part.property_product_pricelist.id or False
            payment_term = part.property_payment_term and part.property_payment_term.id or False
            fiscal_position = part.property_account_position and part.property_account_position.id or False
            
            #Para registrar el plazo de pago por defecto de la sucursal en caso de no tener
            #plazo de pago en el cliente
            if not payment_term:
                payment_term = payment_term_id
                
            #~ dedicated_salesman = part.user_id and part.user_id.id or uid
            val = {
                'partner_invoice_id': addr['invoice'],
                'partner_order_id': addr['invoice'],
                'partner_shipping_id': addr['invoice'],
                'payment_term': payment_term,
                'fiscal_position': fiscal_position,
                #~ 'user_id': dedicated_salesman,
                'partner_id': partner_id,
                'project_id': project_id,
                'picking_policy': 'one',
                'order_policy': 'manual',
                'invoice_quantity': 'order',
            }
            if pricelist:
                val['pricelist_id'] = pricelist
            result = {'value':val}
        else:
            result = {'value': {
                'fiscal_position': False,
                #~ 'user_id': 0,
                'partner_shipping_id': 0,
                'payment_term': False,
                'partner_order_id': 0,
                'partner_invoice_id': 0,
                'pricelist_id': price_list,
                'partner_id': 0,
                'project_id': project_id,
                'partner_invoice_id': 0,
                'partner_order_id': 0,
                'partner_shipping_id': 0,
                }
            }
        return result
         
    def action_invoice_reconcile(self, cr, uid, ids, context=None):
        currency_pool = self.pool.get("res.currency")
        seq_obj = self.pool.get('ir.sequence')
        wf_service = netsvc.LocalService("workflow")
        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        am_obj = self.pool.get('account.move')
        aml_obj = self.pool.get('account.move.line')
        
        if context is None:
            context = {}
            
        """Se buscan todos los IDS de sale.order donde dadiva sea True
        para procesar automaticamente las facturas asociadas """
        dad_ids = self.search(cr, uid, [('id','in',ids),('esdadiva','=',True)])
        
        if dad_ids:
            """ Con los ids de sale.order que son dadivas se
            recorre y se hace una busqueda de las facturas que
            se generaron por los pedidos de venta, para luego
            recorrer las facturas correspondientes y luego
            procesarlas por el workflow"""
            for sale in self.browse(cr, uid, dad_ids):
                inv_ids = invoice_obj.search(cr, uid, [('origin','=',sale.name)])
                inv_line_ids = []
                for invoice in invoice_obj.browse(cr, uid, inv_ids):
                    for line in invoice.invoice_line:
                        inv_line_ids.append(line.id)
                    #Validar configuracion
                    cuenta_dadivas = invoice.journal_id.sale_shop_id and \
                                    invoice.journal_id.sale_shop_id.account_dadivas and \
                                    invoice.journal_id.sale_shop_id.account_dadivas.id or False
                    if not cuenta_dadivas:
                        raise osv.except_osv(_('Error !'), _('Debe configurar la cuenta de dadiva en la sucursal.'))
                    
                    journal_dadivas = invoice.journal_id.sale_shop_id and \
                                    invoice.journal_id.sale_shop_id.journal_dadiva_id and \
                                    invoice.journal_id.sale_shop_id.journal_dadiva_id.sequence_id and \
                                    invoice.journal_id.sale_shop_id.journal_dadiva_id.sequence_id.id or False
                    if not journal_dadivas:
                        raise osv.except_osv(_('Error !'), _('Debe configurar correctamente el diario de pago de dadiva en la sucursal.'))
                    
                    wf_service.trg_validate(uid, 'account.invoice', invoice.id, 'invoice_open', cr)
            """ Se concilian los asientos de la factura """
            moves_ids = []
            for sale in self.browse(cr, uid, dad_ids):
                inv_ids = invoice_obj.search(cr, uid, [('origin','=',sale.name)])
                inv_line_ids = []
                reconcil_ids = []
                for inv in invoice_obj.browse(cr, uid, inv_ids):
                    
                    context_multi_currency = context.copy()
                    context_multi_currency.update({'date': inv.date_invoice})
                    
                    name = seq_obj.get_id(cr, uid, inv.journal_id.sale_shop_id.journal_dadiva_id.sequence_id.id)
                    
                    if not inv.reference:
                        ref = name.replace('/','')
                    else:
                        ref = inv.reference
                    """ Buscamos la cuenta Analitica del Asiento """
                    analytic_account_id = 0
                    for move_line in inv.move_id.line_id:
                        if inv.account_id.id == move_line.account_id.id:
                            analytic_account_id = move_line.analytic_account_id.id 
                    """ Fin Bloque de Cuenta Analitica """        
                    move = {
                        'name': name,
                        'journal_id': inv.journal_id.sale_shop_id.journal_dadiva_id.id,
                        'date': inv.date_invoice,
                        'ref': ref,
                        'period_id': inv.period_id and inv.period_id.id or False
                    }
                    move_id = am_obj.create(cr, uid, move)
                    moves_ids.append(move_id)
                    #create the first line manually
                    company_currency = inv.journal_id.sale_shop_id.journal_dadiva_id.company_id.currency_id.id
                    current_currency = inv.currency_id.id
                    debit = 0.0
                    credit = 0.0
                    #create the first line of the voucher
                    debit = inv.amount_total
                    move_line = {
                        'name': inv.name or '/',
                        'debit': debit,
                        'credit': credit,
                        'account_id': cuenta_dadivas,
                        'move_id': move_id,
                        'journal_id': inv.journal_id.sale_shop_id.journal_dadiva_id.id,
                        'period_id': inv.period_id.id,
                        'partner_id': inv.partner_id.id,
                        'currency_id': company_currency <> current_currency and  current_currency or False,
                        'date': inv.date_invoice,
                        'date_maturity': inv.date_due,
                        'analytic_account_id': analytic_account_id,
                    }
                    account_debit = aml_obj.create(cr, uid, move_line)
                    credit = inv.amount_total
                    debit = 0.0
                    move_line = {
                        'name': inv.name or '/',
                        'debit': debit,
                        'credit': credit,
                        'account_id': inv.account_id.id,
                        'move_id': move_id,
                        'journal_id': inv.journal_id.sale_shop_id.journal_dadiva_id.id,
                        'period_id': inv.period_id.id,
                        'partner_id': inv.partner_id.id,
                        'currency_id': company_currency <> current_currency and  current_currency or False,
                        'date': inv.date_invoice,
                        'date_maturity': inv.date_due,
                        'analytic_account_id': analytic_account_id,
                    }
                    
                    account_credit = aml_obj.create(cr, uid, move_line)

                    reconcil_ids = [x.id for x in inv.move_id.line_id if x.account_id.id == inv.account_id.id]
                    reconcil_ids.append(account_credit)
                    """ Se concilian los asientos de la factura """
                    isok = am_obj.button_validate(cr, uid, moves_ids, context)
                    isok = aml_obj.reconcile_partial(cr, uid, reconcil_ids,'manual', context)
        return True
        
    def onchange_shop_id(self, cr, uid, ids, shop_id):
        v = {}
        v = super(sale_order, self).onchange_shop_id(cr, uid, ids, shop_id)
        if v.get('value',False):
            v['value'].update({'esdadiva':False})

        return v
        
    def onchange_partner_id(self, cr, uid, ids, part, shop_id=False):
        val = {}
        val = super(sale_order, self).onchange_partner_id(cr, uid, ids, part, shop_id)
        if val.get('value',False):
            val['value'].update({'esdadiva':False})

        return val
        
        
sale_order()

class sale_shop(osv.osv):
    _inherit = "sale.shop"

    _columns = {
        'stock_out_dadivas': fields.many2one('stock.location', 'Ubicacion de Salida para Dadivas', help="Se debe indicar la ubicacion de salida para los productos entregados por dadivas o dotaciones."), 
        'account_dadivas': fields.many2one('account.account', 'Cuenta Contable para Dadivas', help="Se debe indicar la cuenta contable para los productos entregados por dadivas o dotaciones."), 
        'journal_dadiva_id': fields.many2one('account.journal', 'Diario para el Pago de Dadivas', help="Se debe indicar el diario de pago por dadivas o dotaciones."), 
    }

sale_shop()


class stock_move(osv.osv):
    _inherit = 'stock.move'
    
    
    def _chain_compute(self, cr, uid, moves, context=None):
        #validar si pasa por el proceso de dadivas
        pasar_x_dadiva = False
        for m in moves:
            pick_dadiva = m.picking_id.sale_id and m.picking_id.sale_id.esdadiva or False
            pick_partner = m.picking_id and m.picking_id.address_id and m.picking_id.address_id.partner_id.id
            pick_company =  m.picking_id and  m.picking_id.company_id.partner_id.id or False
            if m.location_dest_id.chained_location_type == 'customer' and pick_dadiva and pick_partner == pick_company:
                pasar_x_dadiva = True
            else:
                pasar_x_dadiva = False
                break
        #########     
           
        res = super(stock_move, self)._chain_compute(cr, uid, moves, context)
        
        #Ajustar la ubicacion destino si es dadiva
        if pasar_x_dadiva:
            result = {}
            for item in res.values():
                for mov_item in item:
                    result_dest, l_chained_auto_packing, l_chained_delay, l_chained_journal_id, l_chained_company_id, l_chained_picking_type = mov_item[1]
                    nuevo_dest = mov_item[0].picking_id.sale_id.shop_id.stock_out_dadivas, l_chained_auto_packing, l_chained_delay, l_chained_journal_id, l_chained_company_id, l_chained_picking_type
                    
                    result.setdefault(mov_item[0].picking_id, [])
                    result[mov_item[0].picking_id].append( (mov_item[0], nuevo_dest) )

            return result
        ###############
        
        return res
    
stock_move()
