#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp


class sisb_devoluciones(osv.osv):
    _name = 'sisb.devoluciones'
    _descripcion = 'Returns'
    _order = "id desc"
    
    def _get_type(self, cr, uid, context=None):
        if context is None:
            context = {}
        return context.get('type', 'out_refund')
        
        
    def _get_devoluciones_linea(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('sisb.devoluciones.linea').browse(cr, uid, ids, context=context):
            result[line.devoluciones_id.id] = True
        return result.keys()
        
        
    def _amount_line_tax(self, cr, uid, line, context=None):
        val = 0.0
        for c in self.pool.get('account.tax').compute_all(cr, uid, line.invoice_line_tax_id, line.price_unit * (1-(line.discount or 0.0)/100.0), line.quantity, line.devoluciones_address_invoice_id.id, line.product_id, line.devoluciones_partner_id)['taxes']:
            val += c.get('amount', 0.0)
        return val


    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        cur_obj = self.pool.get('res.currency')
        res = {}
        for devolucion in self.browse(cr, uid, ids, context=context):
            res[devolucion.id] = {
                'amount_untaxed': 0.0,
                'amount_tax': 0.0,
                'amount_total': 0.0,
            }
            val = val1 = 0.0
            cur = devolucion.currency_id
            for line in devolucion.devoluciones_linea:
                val1 += line.price_subtotal
                val += self._amount_line_tax(cr, uid, line, context=context)
            res[devolucion.id]['amount_tax'] = cur_obj.round(cr, uid, cur, val)
            res[devolucion.id]['amount_untaxed'] = cur_obj.round(cr, uid, cur, val1)
            res[devolucion.id]['amount_total'] = res[devolucion.id]['amount_untaxed'] + res[devolucion.id]['amount_tax']
        return res
    
    
    _columns = {
        'name': fields.char('Reference', size=64, required=True,
            readonly=True, states={'draft': [('readonly', False)]}, select=True),
        'shop_id': fields.many2one('sale.shop', 'Shop', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'journal_id': fields.many2one('account.journal', 'Journal', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'state': fields.selection([
            ('draft', 'Draft'),
            ('confirmed', 'Confirmed'),
            ('done', 'Done'),
            ('cancel', 'Cancel')
            ], 'Estate', readonly=True, help="Indicates the status of the return on sales.", select=True),
        'type': fields.selection([
            ('out_refund','Customer Refund'),
            ('in_refund','Supplier Refund'),
            ],'Type', readonly=True, select=True, change_default=True),
        'date_devolucion': fields.datetime('Return Date', states={'draft':[('readonly',False)]}, readonly=True, select=True, required=True),
        'partner_id': fields.many2one('res.partner', 'Partner', change_default=True, readonly=True, required=True, states={'draft':[('readonly',False)]}),
        'address_id': fields.many2one('res.partner.address', 'Address', readonly=True, required=True, states={'draft':[('readonly',False)]}, help="Address of partner"),
        'parent_invoice_id':fields.many2one('account.invoice', 'Parent Invoice', readonly=True, states={'draft':[('readonly',False)], 'confirmed':[('readonly',False)]}, help='It is parent invoice to apply invoice refund.'),
        'currency_id': fields.many2one('res.currency', 'Currency', required=True, readonly=True, states={'draft':[('readonly',False)]}),
        'devoluciones_linea': fields.one2many('sisb.devoluciones.linea', 'devoluciones_id', 'Return Line', readonly=True, states={'draft': [('readonly', False)]}),
        'amount_untaxed': fields.function(_amount_all, method=True, digits_compute=dp.get_precision('Account'), string='Untaxed',
            store={
                'sisb.devoluciones': (lambda self, cr, uid, ids, c={}: ids, ['devoluciones_linea'], 20),
                'sisb.devoluciones.linea': (_get_devoluciones_linea, ['price_unit','quantity','discount','invoice_line_tax_id'], 20),
            },
            multi='all'),
        'amount_tax': fields.function(_amount_all, method=True, digits_compute=dp.get_precision('Account'), string='Tax',
            store={
                'sisb.devoluciones': (lambda self, cr, uid, ids, c={}: ids, ['devoluciones_linea'], 20),
                'sisb.devoluciones.linea': (_get_devoluciones_linea, ['price_unit','quantity','discount','invoice_line_tax_id'], 20),
            },
            multi='all'),
        'amount_total': fields.function(_amount_all, method=True, digits_compute=dp.get_precision('Account'), string='Total',
            store={
                'sisb.devoluciones': (lambda self, cr, uid, ids, c={}: ids, ['devoluciones_linea'], 20),
                'sisb.devoluciones.linea': (_get_devoluciones_linea, ['price_unit','quantity','discount','invoice_line_tax_id'], 20),
            },
            multi='all'),
        'invoice_refund_ids': fields.many2many('account.invoice', 'sisb_devoluciones_invoice_refund_rel', 'devoluciones_id', 'invoice_id', 'Invoices', readonly=True, help="This is the list of invoice that have been generated from the returns document."),
        'picking_ids': fields.one2many('stock.picking', 'devoluciones_id', 'Albaranes relacionados', readonly=True, help="This is the list of picking that have been generated from the returns document."),
        'company_id': fields.related('shop_id','company_id',type='many2one',relation='res.company',string='Company',store=True,readonly=True),
    }
    
    
    _defaults = {
        'name':'/',
        'type': _get_type,
        'state': 'draft',
        'date_devolucion': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'company_id': lambda self,cr,uid,c: self.pool.get('res.company')._company_default_get(cr, uid, 'sisb.devoluciones', context=c),
    }
    
    
    def validaciones_registros(self, values, context=None):
        if context is None:
            context = {}
            
        return True
    
    
    def create(self, cr, uid, values, context=None):

        self.validaciones_registros(values, context=context)

        res = super(sisb_devoluciones, self).create(cr, uid, values, context)
        values = {
            'name':self.pool.get('ir.sequence').get(cr, uid, 'dev_vta_tseq')
            }
        seq = self.write(cr, uid, res, values, context)
        
        if not seq:
            raise osv.except_osv(_('Error!'),_('There were errors in the process of creating the sequence. Contact webmaster!'))
    
        return res
        
        
    def write(self, cr, user, ids, vals, context=None):
        #Validar cambio de cliente habiendo lineas registradas
        #Si hubo cambio de partner_id y ya habia registrado lineas de devolucion
        #no permitir cambios
        
        if vals.get('partner_id',False):
            dev_brow = self.browse(cr,user,ids,context=context)
            for item in dev_brow:
                for dev_line in item.devoluciones_linea:
                    if dev_line.invoice_lines:
                        raise osv.except_osv(_('Error!'),_('There change of customer. Must: 1st delete lines in the return associated invoices and then hit save and edit to register!'))
        
        self.validaciones_registros(vals, context=context)
        
        record = super(sisb_devoluciones, self).write(cr, user, ids, vals, context)
        
        return record


    def copy(self, cr, uid, id, defaults, context=None):
        raise osv.except_osv(_('Action Denied!'),_('Are not permitted duplicate!'))
        return False
        
        
    def onchange_shop_id(self, cr, uid, ids, shop_id):
        currency_id = False
        diario_rectifica = False
        if shop_id:
            sucursal = self.pool.get('sale.shop').browse(cr, uid, shop_id)
            diario_rectifica = sucursal.refund_journal_id and sucursal.refund_journal_id.id or False
            if diario_rectifica:
                #verificar el permiso en el diario configurado en la sucursal
                journal_refund_access = self.pool.get('account.journal').browse(cr, uid, diario_rectifica)
                shop_access = journal_refund_access.sale_shop_id and journal_refund_access.sale_shop_id.id or False
                if shop_access == shop_id:
                    currency_id = journal_refund_access.currency and journal_refund_access.currency.id or journal_refund_access.company_id.currency_id.id
                else:
                    diario_rectifica = False
                    currency_id = False
                
        result = {'value': {
            'journal_id': diario_rectifica,
            'parent_invoice_id': False,
            'currency_id': currency_id,
            }
        }
        return result
        
        
    def onchange_journal_id(self, cr, uid, ids, journal_id=False):
        currency_id = False
        if journal_id:
            journal = self.pool.get('account.journal').browse(cr, uid, journal_id)
            currency_id = journal.currency and journal.currency.id or journal.company_id.currency_id.id
            
        result = {'value': {
                'currency_id': currency_id,
                }
            }
        return result
        
        
    def onchange_partner_id(self, cr, uid, ids, part):
        if not part:
            return {'value': {'address_id': False}}
            
        address_obj = self.pool.get('res.partner.address')

        #si existe un contacto, verificar si es el unico conctacto, de lo contrario asignar False
        contacto = address_obj.search(cr,uid,[('partner_id','=',part),('type','in',('delivery','invoice'))])
        if len(contacto) == 1:
            val = {
            'address_id': contacto,
            }
        else:
            val = {
            'address_id': False,
            }
        
        return {'value': val}
        
        
    def valida_cancel(self, cr, uid, ids, context=None):
        """Condiciones para pasar a cancelado"""
        if context is None:
            context = {}

        for item in self.browse(cr, uid, ids, context=context):
            if item.state in ('confirmed'):
                #verificar si el(los) albaran(es) asociado(s) está(n) cancelado(s)
                for pick in item.picking_ids:
                    if pick.state != 'cancel':
                        raise osv.except_osv(_('Action Denied!'),_('The return cannot be canceled! First you must canceling all pickings.'))
            elif item.state in ('done'):
                raise osv.except_osv(_('Action Denied!'),_('The return can not be canceled!'))
        return True
        
        
    def action_cancel(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_cancel(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'cancel'}, context=context)
        return True
    
    
    def action_picking_create(self, cr, uid, ids, tipo='out_refund', context=None):
        wf_service = netsvc.LocalService("workflow")
        pick_obj = self.pool.get('stock.picking')
        picking_id = False
        move_obj = self.pool.get('stock.move')
        company = self.pool.get('res.users').browse(cr, uid, uid).company_id
        for dev in self.browse(cr, uid, ids, context={}):
            output_id = dev.shop_id.return_stock_location_id and dev.shop_id.return_stock_location_id.id or False
            if not output_id:
                raise osv.except_osv(_('Invalid action !'), _('No location configured return in the shop or do not have permission to access the location.'))

            picking_id = False
            for line in dev.devoluciones_linea:
                proc_id = False
                date_planned = datetime.now()
                date_planned = (date_planned - timedelta(days=company.security_lead)).strftime('%Y-%m-%d %H:%M:%S')

                move_id = False
                if line.product_id and line.product_id.product_tmpl_id.type in ('product', 'consu'):
                    if tipo == 'out_refund':
                        location_id = dev.partner_id.property_stock_customer.id
                    else:
                        location_id = dev.partner_id.property_stock_supplier.id
                        
                    if not picking_id:
                        pick_name = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.in')

                        picking_id = self.pool.get('stock.picking').create(cr, uid, {
                            'name': pick_name,
                            'origin': dev.name,
                            'type': 'in',
                            'state': 'auto',
                            'move_type': 'one',
                            'devoluciones_id': dev.id,
                            'address_id': dev.address_id and dev.address_id.id or False,
                            #~ 'note': order.note,
                            'invoice_state': 'none',
                            'company_id': dev.company_id.id,
                        })
                    
                    move_id = self.pool.get('stock.move').create(cr, uid, {
                        'name': self.pool.get('product.product').name_get(cr, uid, [line.product_id.id])[0][1][:64],
                        'picking_id': picking_id,
                        'product_id': line.product_id.id,
                        'date': date_planned,
                        'date_expected': date_planned,
                        'product_qty': line.quantity,
                        'product_uom': (line.uos_id and line.uos_id.id)\
                                or line.product_uom.id,
                        'product_uos_qty': line.quantity,
                        'product_uos': (line.uos_id and line.uos_id.id)\
                                or line.product_uom.id,
                        #~ 'product_packaging': line.product_packaging.id,
                        'address_id': dev.parent_invoice_id.address_shipping_id and dev.parent_invoice_id.address_shipping_id.id or False,
                        'location_id': location_id,
                        'location_dest_id': output_id,
                        'devoluciones_linea_id': line.id,
                        'tracking_id': False,
                        'state': 'draft',
                        #~ #'state': 'waiting',
                        #~ 'note': line.notes,
                        'company_id': dev.company_id.id,
                    })
            
            val = {}

            if picking_id:
                wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
                pick_obj.force_assign(cr, uid, [picking_id], context)
        
        return True
    
    
    def valida_confirm(self, cr, uid, ids, context=None):
        """Condiciones para pasar a confirmado"""
        if context is None:
            context = {}
        
        #Validar que tiene lineas asociadas
        dev_brw = self.browse(cr, uid, ids)
        config_num_max_factura = dev_brw[0].company_id.lines_invoice or 0
        lista_dev = [x.id for x in dev_brw[0].devoluciones_linea]
        
        if not lista_dev:
            raise osv.except_osv(_('Action Denied!'),_('Before confirming must indicate the returned Products.!'))
        
        #Validar el numero maximo de lineas en una factura
        if len(lista_dev) > config_num_max_factura:
            raise osv.except_osv(_('Action Denied!'),_('The maximum allowed number of lines is: %s')%str(config_num_max_factura))
        
        return True
        
        
    def action_confirm(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_confirm(cr, uid, ids, context=context):
            #Crear el albaran asociado
            
            if self.action_picking_create(cr, uid, ids, 'out_refund', context=context):
                self.write(cr, uid, ids, {'state':'confirmed'}, context=context)
                
                return True
            else:
                raise osv.except_osv(_('Action Denied!'),_('Process failed. Contact the administrator!'))
        else:
            return False

        
    def button_reset_taxes(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        return True
        
        
    def validar_devolucion_invoice_refund(self, cr, uid, ids, invoice_parent_id, tipo='out_refund', context=None):
        """Funcion para validar que la factura rectificativa de proveedor no se registre si el monto total
        de la factura rectificativa es mayor a la factura origen"""
        if context is None:
            context = {}
            
        dev_brw = self.browse(cr, uid, ids)
        inv_brw = self.pool.get('account.invoice').browse(cr, uid, invoice_parent_id)

        if (dev_brw.amount_total - inv_brw.amount_total) > 0.0001:
            raise osv.except_osv('Error with parent invoice Selected',"The total amount of the invoice source '%s', is less than the return that tries to register."%inv_brw.number)
        else:
            #verificar si no hay notas rectificativas ya asociada a la factura padre
            ai_rect_list = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',invoice_parent_id),('type','=',tipo)])
            ai_rect_brw = self.pool.get('account.invoice').browse(cr, uid, ai_rect_list)
            monto_total = 0
            for fact_rec in ai_rect_brw:
                monto_total += fact_rec.amount_total
            
            if (monto_total - inv_brw.amount_total) > 0.0001:
                raise osv.except_osv('Error with parent invoice Selected',"The total amount of the invoice source '%s', is less than the return that tries to register. Invoice refund exist."%inv_brw.number)
                
            
        return True
        
        
    def _make_invoice(self, cr, uid, dev, lines, tipo='out_refund', context=None):
        journal_obj = self.pool.get('account.journal')
        inv_obj = self.pool.get('account.invoice')
        obj_invoice_line = self.pool.get('account.invoice.line')
        
        if context is None:
            context = {}

        journal_id = dev.journal_id and dev.journal_id.id or False
        if not journal_id:
            raise osv.except_osv(_('Error !'),
                _('There is no sales journal defined for this return: "%s"') % (dev.name))
        
        if tipo == 'out_refund':
            a = dev.partner_id.property_account_receivable.id
        else:
            a = dev.partner_id.property_account_payable.id
            
        invoiced_dev_linea_ids = self.pool.get('sisb.devoluciones.linea').search(cr, uid, [('devoluciones_id', '=', dev.id)], context=context)
        from_line_invoice_ids = []
        for invoiced_dev_linea_id in self.pool.get('sisb.devoluciones.linea').browse(cr, uid, invoiced_dev_linea_ids, context=context):
            for invoice_line_id in invoiced_dev_linea_id.invoice_refund_lines:
                if invoice_line_id.invoice_id.id not in from_line_invoice_ids:
                    from_line_invoice_ids.append(invoice_line_id.invoice_id.id)
        for preinv in dev.invoice_refund_ids:
            if preinv.state not in ('cancel',) and preinv.id not in from_line_invoice_ids:
                for preline in preinv.invoice_line:
                    inv_line_id = obj_invoice_line.copy(cr, uid, preline.id, {'invoice_id': False, 'price_unit': -preline.price_unit})
                    lines.append(inv_line_id)
                    
        #~ pay_term = dev.parent_invoice_id.payment_term and dev.parent_invoice_id.payment_term.id or False
        
        inv = {
            'name': 'Rectifica a ' + dev.parent_invoice_id.reference,
            'origin': dev.name,
            'type': tipo,
            'reference': dev.parent_invoice_id.name,
            'account_id': a,
            'partner_id': dev.partner_id.id,
            'journal_id': journal_id,
            'address_invoice_id': dev.address_id.id,
            'address_contact_id': dev.parent_invoice_id.address_contact_id and dev.parent_invoice_id.address_contact_id.id or False,
            'invoice_line': [(6, 0, lines)],
            'currency_id': dev.currency_id.id,
            #~ 'comment': dev.parent_invoice_id.note,
            #~ 'payment_term': pay_term,
            'fiscal_position': dev.partner_id.property_account_position and dev.partner_id.property_account_position.id or False,
            'date_invoice': time.strftime('%Y-%m-%d'),
            'company_id': dev.company_id.id,
            'user_id': dev.parent_invoice_id.user_id and dev.parent_invoice_id.user_id.id or False,
            'parent_id': dev.parent_invoice_id.id,
            'address_shipping_id': dev.parent_invoice_id.address_shipping_id.id,
        }
        
        inv_id = inv_obj.create(cr, uid, inv, context=context)
        inv_obj.button_compute(cr, uid, [inv_id], context=context)
        
        return inv_id
        
        
    def action_invoice_create(self, cr, uid, ids, tipo='out_refund', context=None):
        invoice_ids = []

        invoice = self.pool.get('account.invoice')
        obj_sisb_devoluciones_linea = self.pool.get('sisb.devoluciones.linea')
        if context is None:
            context = {}

        for d in self.browse(cr, uid, ids, context=context):
            inv_id = False
            invoices = {}

            lines = []
            for line in d.devoluciones_linea:
                lines.append(line.id)
            created_lines = obj_sisb_devoluciones_linea.invoice_line_create(cr, uid, lines)
            if created_lines:
                invoices.setdefault(d.partner_id.id, []).append((d, created_lines))
                
            for val in invoices.values():
                for dev, il in val:
                    inv_id = self._make_invoice(cr, uid, dev, il, tipo, context=context)
                    invoice_ids.append(inv_id)
                    self.write(cr, uid, [dev.id], {'state': 'done'})
                    
                    cr.execute('insert into sisb_devoluciones_invoice_refund_rel (devoluciones_id,invoice_id) values (%s,%s)', (dev.id, inv_id))

        return invoice_ids
        
    
    def valida_done(self, cr, uid, ids, context=None):
        """Condiciones para pasar a confirmado"""
        if context is None:
            context = {}
        
        #Validar Factura Padre
        for item in self.browse(cr, uid, ids, context=context):
            if not item.parent_invoice_id:
                raise osv.except_osv(_('Action Denied!'),_('To approve the document must indicate the parent invoice!'))

            #Verificar saldo de la factura y compararlo con el total de la devolucion
            self.validar_devolucion_invoice_refund(cr,uid,item.id,item.parent_invoice_id.id,'out_refund', context=context)
            
        return True
    
    
    def action_done(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        aml_obj = self.pool.get('account.move.line')
        wf_service = netsvc.LocalService("workflow")
        
        act_obj = self.pool.get('ir.actions.act_window')
        mod_obj = self.pool.get('ir.model.data')
        model_pool = self.pool.get('account.invoice')
        
        result = {}
        
        if self.valida_done(cr, uid, ids, context=context):
            #Registrar factura rectificativa asociada
            invoice_refund = self.action_invoice_create(cr, uid, ids, 'out_refund', context=context)
            if invoice_refund:
                #procesar las facturas
                for invoice in invoice_refund:
                    wf_service.trg_validate(uid, 'account.invoice', invoice, 'invoice_open', cr)
                
                """ Se concilian los asientos de la factura rectificativa y la factura padre """
                reconcil_ids = []
                for invoice in invoice_refund:
                    reconcil_ids = []
                    inv_refund = model_pool.browse(cr, uid, invoice, context=context)
                    for line_m_r in inv_refund.move_id.line_id:
                        if line_m_r.account_id == inv_refund.account_id:
                            reconcil_ids.append(line_m_r.id)
                            break
                    
                    for line_m_i in inv_refund.parent_id.move_id.line_id:
                        if line_m_i.account_id == inv_refund.account_id:
                            reconcil_ids.append(line_m_i.id)
                            break

                    """ Se concilian los asientos de la factura """
                    if reconcil_ids:
                        isok = aml_obj.reconcile_partial(cr, uid, reconcil_ids,'manual', context)
                
                
                #retornar en la ventana de factura rectificativa
                result = mod_obj.get_object_reference(cr, uid, 'account', 'action_invoice_tree3')
                id = result and result[1] or False
                result = act_obj.read(cr, uid, id, context=context)
                invoice_domain = eval(result['domain'])
                invoice_domain.append(('id', 'in', invoice_refund))
                result['domain'] = invoice_domain
                return result
            else:
                raise osv.except_osv(_('Action Denied!'),_('Process failed. Contact the administrator!'))

        else:
            return result
        
sisb_devoluciones()


class sisb_devoluciones_linea(osv.osv):
    _name = "sisb.devoluciones.linea"
    _description = "Returns Lines"
    
    def _amount_line(self, cr, uid, ids, prop, unknow_none, unknow_dict):
        res = {}
        tax_obj = self.pool.get('account.tax')
        cur_obj = self.pool.get('res.currency')
        for line in self.browse(cr, uid, ids):
            price = line.price_unit * (1-(line.discount or 0.0)/100.0)
            taxes = tax_obj.compute_all(cr, uid, line.invoice_line_tax_id, price, line.quantity, product=line.product_id, address_id=line.devoluciones_address_invoice_id, partner=line.devoluciones_partner_id)
            res[line.id] = taxes['total']
            if line.devoluciones_id:
                cur = line.devoluciones_id.currency_id
                res[line.id] = cur_obj.round(cr, uid, cur, res[line.id])
        return res
    
    
    _columns = {
        'devoluciones_id': fields.many2one('sisb.devoluciones', 'Return Reference', required=True, ondelete='cascade', select=True, readonly=True, states={'draft':[('readonly',False)]}),
        'name': fields.char('Reference', size=128),
        'invoice_id':fields.many2one('account.invoice', 'Invoice', readonly=True, states={'draft':[('readonly',False)]}),
        'invoice_lines': fields.many2one('account.invoice.line', 'Invoice Lines', readonly=True, states={'draft':[('readonly',False)]}),
        'product_id': fields.many2one('product.product', 'Product', domain=[('sale_ok', '=', True)], required=True, change_default=True),
        'quantity': fields.float('Quantity', digits=(16, 2), required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'uos_id': fields.many2one('product.uom', 'Unit', ondelete='set null'),
        'price_unit': fields.float('Price Unit', required=True, digits_compute= dp.get_precision('Sale Price'), readonly=True, states={'draft': [('readonly', False)]}),
        'price_subtotal': fields.function(_amount_line, method=True, string='Subtotal', type="float", digits_compute= dp.get_precision('Account'), store=True),
        'cost_unit': fields.float('Cost Unit', digits_compute= dp.get_precision('Account'), help="Technical field used to record the product cost set by the user during a picking confirmation (when average price costing method is used)"),
        'discount': fields.float('Discount (%)', digits_compute= dp.get_precision('Account')),
        'invoice_line_tax_id': fields.many2many('account.tax', 'sisb_devoluciones_line_tax', 'devoluciones_linea_id', 'tax_id', 'Taxes', domain=[('parent_id','=',False)]),
        'invoice_refund_lines': fields.many2many('account.invoice.line', 'sisb_devoluciones_linea_invoice_refund_rel', 'devoluciones_linea_id', 'invoice_id', 'Invoice Refund Lines', readonly=True),
        'move_ids': fields.one2many('stock.move', 'devoluciones_linea_id', 'Stock Moves', readonly=True),
        'devoluciones_partner_id': fields.related('invoice_id', 'partner_id', type='many2one', relation='res.partner', store=True, string='Cliente'),
        'devoluciones_address_invoice_id': fields.related('invoice_id', 'address_invoice_id', type='many2one', relation='res.partner.address', store=True, string='Address Invoice Partner'),
        'company_id': fields.related('devoluciones_id', 'company_id', type='many2one', relation='res.company', string='Company', store=True, readonly=True),
    }
    
    
    _sql_constraints = [
        ('dev_invoice_line', 'unique(devoluciones_id,invoice_lines)', 'The line of the invoice must be attached only once in the document!'),
        ('dev_qty_line', 'CHECK (quantity>0.00)','The amount must be greater than "0"!'),
    ]
    
    
    def validaciones_registros(self, values, ids=False, context=None):
        if context is None:
            context = {}
        if values.get('quantity',0.00) <= 0.00:
            raise osv.except_osv(_('Error!'), _('The quantity must be greater than zero "0"!'))
            
        if values.get('cost_unit',0.00) <= 0.00:
            raise osv.except_osv(_('Error!'), _('The cost must be greater than zero "0"!'))
                
        return True
    
    
    def fill_taxes_line(self, cr, uid, values, context=None):
        taxes = []
        invoice_line_id = values.get('invoice_lines', False)
        if invoice_line_id:
            inv_line = self.pool.get('account.invoice.line').browse(cr, uid, invoice_line_id,context=context)
            taxes = [x.id for x in inv_line.invoice_line_tax_id]
        else:
            product_id = values.get('product_id', False)
            if product_id:
                product = self.pool.get('product.product').browse(cr, uid, product_id,context=context)
                taxes = [x.id for x in product.taxes_id]

        return taxes
        
        
    def create(self, cr, uid, values, context=None):
        
        self.validaciones_registros(values, context=context)
        
        taxes_line = self.fill_taxes_line(cr, uid, values, context)
        values['invoice_line_tax_id'] = [(6,0, taxes_line)]
        
        res = super(sisb_devoluciones_linea, self).create(cr, uid, values, context)
        
        return res
        
        
    def write(self, cr, user, ids, vals, context=None):
        
        self.validaciones_registros(vals, ids=ids, context=context)
        
        taxes_line = self.fill_taxes_line(cr, user, vals, context)
        vals['invoice_line_tax_id'] = [(6,0, taxes_line)]
        
        record = super(sisb_devoluciones_linea, self).write(cr, user, ids, vals, context)
        
        return record
        
        
    def asignar_datos_linea(self, cr, uid, ids, invoice_lines, ref_dev, context=None):
        result = {}
        warning = {}

        if isinstance(invoice_lines,(int,long)):
            invoice_lines = [invoice_lines]
        
        if isinstance(ids,(int,long)):
            ids = [ids]
        
        if invoice_lines:
            inv_line = self.pool.get('account.invoice.line').browse(cr, uid, invoice_lines)[0]
            product = inv_line.product_id and inv_line.product_id.id or False
            name = ref_dev
            cantidad = 0
            unidad = inv_line.uos_id and inv_line.uos_id.id or False
            costo = 0
            #Costo promedio del producto al momento de facturar
            linea_asiento = inv_line.invoice_id.move_id and inv_line.invoice_id.move_id.id or False
            producto = inv_line.product_id and inv_line.product_id.id or False
            cuenta_retorno = inv_line.invoice_id.journal_id.sale_shop_id.return_stock_location_id and inv_line.invoice_id.journal_id.sale_shop_id.return_stock_location_id.account_location_variant_id and inv_line.invoice_id.journal_id.sale_shop_id.return_stock_location_id.account_location_variant_id.id or False
            if linea_asiento and producto and cuenta_retorno:
                costo_move = self.pool.get('account.move.line').search(cr, uid,  [('move_id', '=',linea_asiento),('product_id','=',producto),('account_id','=',cuenta_retorno)])
                if costo_move:
                    credito = self.pool.get('account.move.line').browse(cr, uid, costo_move[0]).credit
                    quantity = self.pool.get('account.move.line').browse(cr, uid, costo_move[0]).quantity
                    if quantity:
                        costo_unit = credito/quantity
                        #convertir si tiene moneda distinta
                        cur_obj = self.pool.get('res.currency')
                        company_currency = inv_line.invoice_id.company_id.currency_id.id
                        if inv_line.invoice_id.currency_id.id != company_currency:
                            costo = cur_obj.compute(cr, uid, company_currency, inv_line.invoice_id.currency_id.id, costo_unit, context={'date': inv_line.invoice_id.date_invoice})
                        else:
                            costo = costo_unit
                    else:
                        warning = {
                        'title': _('Error in configuration!'),
                        'message':
                            _('Verify the quantity of product in the invoice.!')
                        }
                else:
                    warning = {
                    'title': _('Error in configuration!'),
                    'message':
                        _('Check the seat of the bill, has not reflected the account for dispatch.!')
                    }
            else:
                warning = {
                'title': _('Error in configuration!'),
                'message':
                    _('Verify if the location of return is configured correctly.!')
                }
            
            result.update({'warning': warning})

            descuento = inv_line.discount
            precio = inv_line.price_unit
            
            prod_tax = [x.id for x in inv_line.invoice_line_tax_id]
            
            result.update( {'value': {
                'product_id': product,
                'name': name,
                'quantity': cantidad,
                'uos_id': unidad,
                'cost_unit': costo,
                'discount': descuento,
                'price_unit': precio,
                'invoice_line_tax_id': [(6,0, prod_tax)]
                #~ 'invoice_line_tax_id': inv_line.invoice_line_tax_id
                }
            })

        return result
        
        
    def onchange_invoice_id(self, cr, uid, ids, invoice_id, ref_dev, context=None):
        print "onchange_invoice_id",context
        """condicion que se encuentra en el domain"""
        invoice_line_ids = self.pool.get('account.invoice.line').search(cr, uid, [('invoice_id', '=', invoice_id),('product_id','!=',False),('product_id.type','!=','service')])
        if  len(invoice_line_ids) == 1:
            #Asignar todos los datos requeridos de la linea de la factura
            result = self.asignar_datos_linea(cr, uid, ids, invoice_line_ids, ref_dev, context=context)
            result['value']['invoice_lines'] = invoice_line_ids
        else:
            result = {'value': {
                'invoice_lines': False,
                'product_id': False,
                'name': False,
                'quantity': 0,
                'uos_id': False,
                'cost_unit': 1,
                'discount': False,
                'price_unit': 1,
                'invoice_line_tax_id': [(6,0, [])]
                }
            }

        return result
        
        
    def onchange_invoice_lines_id(self, cr, uid, ids, invoice_lines, ref_dev, context=None):
        print "onchange_invoice_lines_id",context
        if invoice_lines:
            result = self.asignar_datos_linea(cr, uid, ids, invoice_lines, ref_dev, context=context)
        else:
            result = {'value': {
                'product_id': False,
                'name': False,
                'quantity': 0,
                'uos_id': False,
                'cost_unit': 1,
                'discount': 0.00,
                'price_unit': 1,
                'invoice_line_tax_id': [(6,0, [])]
                }
            }
        
        return result
        
        
    def onchange_product_id(self, cr, uid, ids, product_id):
        """consultar datos del producto"""
        if product_id:
            producto = self.pool.get('product.product').browse(cr, uid, product_id)
            
            prod_tax = [x.id for x in producto.taxes_id]

            result = {'value': {
                'invoice_id': False,
                'invoice_lines': False,
                'uos_id': producto.uom_id.id,
                'cost_unit': producto.standard_price,
                'discount': 0.00,
                'price_unit': producto.list_price,
                'invoice_line_tax_id': [(6,0, prod_tax)]
                #~ 'invoice_line_tax_id': producto.taxes_id
                }
            }
        else:
            
            result = {'value': {
                'name': False,
                'quantity': 0,
                'uos_id': False,
                'cost_unit': 1,
                'discount': 0.00,
                'price_unit': 1,
                'invoice_line_tax_id': [(6,0, [])]
                }
            }
        return result
        
        
    def onchange_quantity(self, cr, uid, ids, invoice_lines, quantity, context=None):
        print "onchange_quantity",context
        result = {}
        warning = {}
        if quantity:
            if isinstance(invoice_lines,(int,long)):
                invoice_lines = [invoice_lines]
            
            if isinstance(ids,(int,long)):
                ids = [ids]
            
            if invoice_lines:
                inv_line = self.pool.get('account.invoice.line').browse(cr, uid, invoice_lines)[0]
                #Cantidad del producto al momento de facturar
                cantidad = inv_line.quantity
                if quantity > cantidad:
                    warning = {
                    'title': _('Error in configuration!'),
                    'message':
                        _('Verify the quantity of product in the invoice. Quantity invoiced: %s')%cantidad
                    }
                
                if warning:
                    result.update( {'value': {
                        'quantity': 0.00,
                        }
                    })
                    
                result.update({'warning': warning})
                

        return result
        
        
    def onchange_uos_id(self, cr, uid, ids, invoice_lines, uos_id, context=None):
        print "onchange_uos_id",context
        result = {}
        warning = {}

        if isinstance(invoice_lines,(int,long)):
            invoice_lines = [invoice_lines]
        
        if isinstance(ids,(int,long)):
            ids = [ids]
        
        if invoice_lines:
            inv_line = self.pool.get('account.invoice.line').browse(cr, uid, invoice_lines)[0]
            #Cantidad del producto al momento de facturar
            unidad_venta = inv_line.uos_id and inv_line.uos_id.id or False
            unidad_venta_name = inv_line.uos_id and inv_line.uos_id.name or ''
            if not uos_id or uos_id != unidad_venta:
                warning = {
                'title': _('Error in configuration!'),
                'message':
                    _('Verify the unit of product in the invoice. Unit of product: %s')%unidad_venta_name
                }
            
            if warning:
                result.update( {'value': {
                    'uos_id': unidad_venta,
                    }
                })
                
            result.update({'warning': warning})

        return result
        
        
    def get_product_category_account(self,cr, uid, categ_id, fieldname,context=None):
        if context is None:
            context = {}
        categ_brw=self.pool.get('product.category').browse(cr, uid, categ_id, context=context)
        account_id = getattr(categ_brw,fieldname) and getattr(categ_brw,fieldname).id or False
        if not account_id and categ_brw.parent_id:
            account_id = self.get_product_category_account(cr, uid, categ_brw.parent_id.id, fieldname,context=context)
        return account_id
        
        
    def invoice_line_create(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
            
        create_ids = []
        for line in self.browse(cr, uid, ids, context=context):
            if line.product_id:
                acc_contra = False
                a = line.product_id.product_tmpl_id.property_account_income.id
                if not a:
                    a = line.product_id.categ_id.property_account_income_categ.id
                if not a:
                    a = self.get_product_category_account(cr,uid,line.product_id.categ_id.id,"property_account_income_categ",context=context)
                if not a:
                    raise osv.except_osv(_('Error !'),
                            _('There is no income account defined ' \
                                    'for this product: "%s" (id:%d)') % \
                                    (line.product_id.name, line.product_id.id,))
                                    
                #Se asigna contrapartida si se encuentra registrada
                for ctas in line.devoluciones_id.shop_id.account_income_ids:
                    if ctas.return_account_id.id == a:
                        acc_contra = ctas.return_contrapartida_account_id.id
                        break
                        
                if not acc_contra:
                    cuen =  self.pool.get('account.account').browse(cr, uid, a, context=context)
                    raise osv.except_osv(_('Error !'),
                            _('There is no income account (%s %s) defined ' \
                                    'for this shop: "%s"') % \
                                    (cuen.code, cuen.name, line.devoluciones_id.shop_id.name))
                            
            else:
                raise osv.except_osv(_('Error!'),
                            _('There is no income product defined.'))
            
            inv_id = self.pool.get('account.invoice.line').create(cr, uid, {
                'name': self.pool.get('product.product').name_get(cr, uid, [line.product_id.id])[0][1][:128],
                'origin': line.devoluciones_id.name,
                'account_id': acc_contra,
                'price_unit': line.price_unit,
                'quantity': line.quantity,
                'discount': line.discount,
                'uos_id': line.uos_id.id,
                'product_id': line.product_id.id or False,
                'invoice_line_tax_id': [(6, 0, [x.id for x in line.invoice_line_tax_id])],
                #~ 'note': line.notes,
                'account_analytic_id': line.devoluciones_id.shop_id.project_id and line.devoluciones_id.shop_id.project_id.id or False,
            }, context=context)
            cr.execute('insert into sisb_devoluciones_linea_invoice_refund_rel (devoluciones_linea_id,invoice_id) values (%s,%s)', (line.id, inv_id))
            create_ids.append(inv_id)
            
        return create_ids
    
sisb_devoluciones_linea()


class stock_picking(osv.osv):
    _inherit = 'stock.picking'
    
    _columns = {
        'devoluciones_id': fields.many2one('sisb.devoluciones', 'Return', ondelete='set null', select=True),
    }
    
    _defaults = {
        'devoluciones_id': False
    }
    
stock_picking()


class stock_move(osv.osv):
    _inherit = 'stock.move'
    
    _columns = {
        'devoluciones_linea_id': fields.many2one('sisb.devoluciones.linea', 'Return Line', ondelete='set null', select=True, readonly=True),
    }
    
    
    def _get_reference_accounting_values_for_valuation(self, cr, uid, move, context=None):
        """
        Overrides the default stock valuation to take into account the currency that was specified
        on the Devolution Document in case the valuation data was not directly specified during picking
        confirmation.
        """
        reference_amount, reference_currency_id = super(stock_move, self)._get_reference_accounting_values_for_valuation(cr, uid, move, context=context)
        if move.product_id.cost_method != 'average' or not move.price_unit:
            # no average price costing or cost not specified during picking validation, we will
            # plug the devolution line values if they are found.
            if move.devoluciones_linea_id and move.picking_id.devoluciones_id.currency_id:
                reference_amount, reference_currency_id = move.devoluciones_linea_id.cost_unit * move.product_qty, move.picking_id.devoluciones_id.currency_id.id
        return reference_amount, reference_currency_id
    
stock_move()


class sisb_dev_account_contrapartida_rel(osv.osv):
    _name = 'sisb.dev.account.contrapartida.rel'
    _descripcion = 'Return Accounts'
    
    _columns = {
        'shop_id': fields.many2one('sale.shop', 'Shop', required=True),
        'return_account_id': fields.many2one('account.account', 'Income Account', required=True), 
        'return_contrapartida_account_id': fields.many2one('account.account', 'Return Income Account', required=True), 
    }
    
sisb_dev_account_contrapartida_rel()


class sale_shop(osv.osv):
    _inherit = "sale.shop"
    #para indicar la ubicacion de devoluciones
    _columns = {
        'return_stock_location_id': fields.many2one('stock.location', 'Return Stock Location'), 
        'account_income_ids': fields.one2many('sisb.dev.account.contrapartida.rel', 'shop_id', 'Return Account in Invoice'),
    }

sale_shop()


class stock_partial_picking(osv.osv_memory):
    
    _inherit = 'stock.partial.picking'
        
    def default_get(self, cr, uid, fields, context=None):
        """ To get default values for the object.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param fields: List of fields for which we want default values
        @param context: A standard dictionary
        @return: A dictionary which of fields with values.
        """
        if context is None:
            context = {}
        pick_obj = self.pool.get('stock.picking')
        res = super(stock_partial_picking, self).default_get(cr, uid, fields, context=context)
        for pick in pick_obj.browse(cr, uid, context.get('active_ids', []), context=context):
            has_product_cost = (pick.type == 'in' and pick.devoluciones_id)
            for m in pick.move_lines:
                if m.state in ('done','cancel') :
                    continue
                if has_product_cost and m.product_id.cost_method == 'average' and m.devoluciones_linea_id:
                    # We use the original DEV unit purchase price as the basis for the cost, expressed
                    # in the currency of the PO (i.e the DEV's currency)
                    list_index = 0
                    for item in res['product_moves_in']:
                        if item['move_id'] == m.id:
                            res['product_moves_in'][list_index]['cost'] = m.devoluciones_linea_id.cost_unit
                            res['product_moves_in'][list_index]['currency'] = m.picking_id.devoluciones_id.currency_id.id
                        list_index += 1
        return res
        
stock_partial_picking()


class account_invoice(osv.osv):
    _inherit = "account.invoice"
    
    _columns = {
        'devoluciones_ids': fields.many2many('sisb.devoluciones', 'sisb_devoluciones_invoice_refund_rel', 'invoice_id', 'devoluciones_id', 'Devoluciones', help='This is the list of returns that generated this invoice'),
    }

account_invoice()


class account_invoice_line(osv.osv):
    _inherit = "account.invoice.line"
    
    _columns = {
        'devoluciones_lines': fields.many2many('sisb.devoluciones.linea', 'sisb_devoluciones_linea_invoice_refund_rel', 'invoice_id', 'devoluciones_linea_id', 'Devoluciones Lineas', readonly=True),
    }
    
    
    def move_line_get(self, cr, uid, invoice_id, context=None):
        """Se sobreescribe para registrar el asiento de costo enfuncion al
        docuemnto de devolucion en caso de existir"""
        res = []
        
        inv = self.pool.get('account.invoice').browse(cr, uid, invoice_id, context=context)
        company_currency = inv.company_id.currency_id.id
        def sisb_get_price(cr, uid, inv, company_currency,i_line):
            cur_obj = self.pool.get('res.currency')
            if inv.currency_id.id != company_currency:
                price = cur_obj.compute(cr, uid, company_currency, inv.currency_id.id, i_line.product_id.product_tmpl_id.standard_price * i_line.quantity, context={'date': inv.date_invoice})
            else:
                price = i_line.product_id.product_tmpl_id.standard_price * i_line.quantity
            return price
                
        if inv.type in ('out_invoice','out_refund'):
            ##################
            #move_line_get original
            res = []
            tax_obj = self.pool.get('account.tax')
            cur_obj = self.pool.get('res.currency')
            if context is None:
                context = {}
            inv = self.pool.get('account.invoice').browse(cr, uid, invoice_id, context=context)
            company_currency = inv.company_id.currency_id.id

            for line in inv.invoice_line:
                mres = self.move_line_get_item(cr, uid, line, context)
                if not mres:
                    continue
                res.append(mres)
                tax_code_found= False
                for tax in tax_obj.compute_all(cr, uid, line.invoice_line_tax_id,
                        (line.price_unit * (1.0 - (line['discount'] or 0.0) / 100.0)),
                        line.quantity, inv.address_invoice_id.id, line.product_id,
                        inv.partner_id)['taxes']:

                    if inv.type in ('out_invoice', 'in_invoice'):
                        tax_code_id = tax['base_code_id']
                        tax_amount = line.price_subtotal * tax['base_sign']
                    else:
                        tax_code_id = tax['ref_base_code_id']
                        tax_amount = line.price_subtotal * tax['ref_base_sign']

                    if tax_code_found:
                        if not tax_code_id:
                            continue
                        res.append(self.move_line_get_item(cr, uid, line, context))
                        res[-1]['price'] = 0.0
                        res[-1]['account_analytic_id'] = False
                    elif not tax_code_id:
                        continue
                    tax_code_found = True

                    res[-1]['tax_code_id'] = tax_code_id
                    res[-1]['tax_amount'] = cur_obj.compute(cr, uid, inv.currency_id.id, company_currency, tax_amount, context={'date': inv.date_invoice})
                
            ####################
            #Asiento del costo
            
            for i_line in inv.invoice_line:
                if i_line.product_id and i_line.product_id.type != 'service':
                    if inv.type == 'out_invoice':
                        # debit account dacc will be the output account
                        # first check the product, if empty check the category                
                                
                        #ojo este comportamiento esta sobreescrito para facturas del modulo de ventas
                        
                        # Se selecciona la cuenta variante de la ubicacion de salida
                        # del almacen de la sucursal
                        
                        dacc = inv.journal_id.sale_shop_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id.id or False
                        
                        if not dacc:
                            raise osv.except_osv(_('Error !'),
                                    _('No hay cuenta de variacion en la ubicacion de salida de la sucursal asociada al journal' \
                                            ' para este producto: "%s" (id:%d)') % \
                                            (i_line.product_id.name, i_line.product_id.id,))
                        
                        #costo actual del prodcuto
                        price_unit_act = i_line.product_id.product_tmpl_id.standard_price
                        price_act = sisb_get_price(cr, uid, inv, company_currency, i_line)

                        #####

                    else:
                        # = out_refund
                        # debit account dacc will be the input account
                        # first check the product, if empty check the category             
                        
                        #ojo este comportamiento esta sobreescrito para facturas del modulo de ventas           
                       
                        # Se selecciona la cuenta variante de la ubicacion de salida
                        # del almacen de la sucursal
                        dacc = inv.journal_id.sale_shop_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id.id or False
                        
                        
                        if not dacc:
                            raise osv.except_osv(_('Error !'),
                                    _('No hay cuenta de variacion en la ubicacion de salida de la sucursal asociada al journal ' \
                                            'para este producto: "%s" (id:%d)') % \
                                            (i_line.product_id.name, i_line.product_id.id,))
                                            
                                            
                        #costo de la devolucion si aplica
                        inv_dev = pricelist_version_ids = self.pool.get('product.pricelist').search(cr, uid, [], context=context)
                        if i_line.devoluciones_lines:
                            price_unit_act = i_line.devoluciones_lines[0].cost_unit
                            if inv.currency_id.id != company_currency:
                                price_act = cur_obj.compute(cr, uid, company_currency, inv.currency_id.id, price_unit_act * i_line.devoluciones_lines[0].quantity, context={'date': inv.date_invoice})
                            else:
                                price_act = price_unit_act * i_line.devoluciones_lines[0].quantity
                        else:                        
                            price_unit_act = i_line.product_id.product_tmpl_id.standard_price
                            price_act = sisb_get_price(cr, uid, inv, company_currency, i_line)
                            
                        #####
                                            
                    # in both cases the credit account cacc will be the expense account
                    # first check the product, if empty check the category                                    
                    cacc = i_line.product_id.product_tmpl_id.property_account_expense and i_line.product_id.product_tmpl_id.property_account_expense.id
                    if not cacc:
                        cacc = i_line.product_id.categ_id.property_account_expense_categ and i_line.product_id.categ_id.property_account_expense_categ.id
                    
                    if not cacc:
                            cacc = self.get_product_category_account(cr,uid,i_line.product_id.categ_id.id,"property_account_expense_categ",context=context)
                            
                    if not cacc:
                        raise osv.except_osv(_('Error !'),
                                _('There is no account expense defined ' \
                                        'for this product: "%s" (id:%d)') % \
                                        (i_line.product_id.name, i_line.product_id.id,))
                                        
                    ####
                    
                    
                    if dacc and cacc:
                        res.append({
                            'type': 'src',
                            'name': i_line.name[:64],
                            'price_unit': price_unit_act,
                            'quantity': i_line.quantity,
                            'price': price_act,
                            'account_id': dacc,
                            'product_id': i_line.product_id.id,
                            'uos_id': i_line.uos_id.id,
                            'account_analytic_id': i_line.account_analytic_id.id,
                            'taxes': i_line.invoice_line_tax_id,
                            })
                        
                        res.append({
                            'type': 'src',
                            'name': i_line.name[:64],
                            'price_unit': price_unit_act,
                            'quantity': i_line.quantity,
                            'price': -1 * price_act,
                            'account_id': cacc,
                            'product_id': i_line.product_id.id,
                            'uos_id': i_line.uos_id.id,
                            'account_analytic_id': i_line.account_analytic_id.id,
                            'taxes': i_line.invoice_line_tax_id,
                            })

        else:
            
            res = super(account_invoice_line,self).move_line_get(cr, uid, invoice_id, context=context)

        return res
        
account_invoice_line()
