#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp


class sisb_devoluciones(osv.osv):
    _name = 'sisb.devoluciones'
    _inherit = 'sisb.devoluciones'
    
    def action_invoice_create(self, cr, uid, ids, tipo='out_refund', context=None):
        
        if context is None:
            context = {}
            
        if isinstance(ids, (int, long)):
            ids = [ids]
            
        if not ids:
            return {}

        res = super(sisb_devoluciones, self).action_invoice_create(cr, uid,  ids, tipo, context)

        #Actualizar las lineas de las facturas creadas, colocando la accion especifica y la partida presupuestaria en sus lineas
        if res:
            facturas_creadas = list(set(res))
            inv_obj = self.pool.get('account.invoice')
            inv_line_obj = self.pool.get('account.invoice.line')
            
            devolucion_brw = self.browse(cr, uid, ids[0])
            
            #Consultar Accion Especifica
            accion_esp = devolucion_brw.shop_id.accion_especifica_id.id or False
            
            #Diccionario de partida devolucion
            dict_partida = {}
            for ret_acc in devolucion_brw.shop_id.account_income_ids:
                if not dict_partida.get(ret_acc.return_contrapartida_account_id.id, False):
                    dict_partida[ret_acc.return_contrapartida_account_id.id] = ret_acc.return_partida_id.id
            
            #Consutar facturas creadas
            for fact in inv_obj.browse(cr, uid, facturas_creadas):
                for linea_fact in fact.invoice_line:
                    #Consultar cuenta de la linea para agregar partida presupuestaria
                    if dict_partida.get(linea_fact.account_id.id, False):
                        producto_partida = dict_partida[linea_fact.account_id.id]
                    else:
                        raise osv.except_osv(_('Error!'),
                            _('You need budget account in the shop to make the return.'))
                            
                    #Actualizar
                    values = {
                        'accion_especifica_id': accion_esp,
                        'partida_presupuesto': producto_partida,
                    }
                    line_update = inv_line_obj.write(cr, uid, linea_fact.id, values, context)
                    
        return res
    
        
sisb_devoluciones()


class sisb_dev_account_contrapartida_rel(osv.osv):
    _inherit = 'sisb.dev.account.contrapartida.rel'
    
    _columns = {
        'return_partida_id': fields.many2one('clasificador.presupuestario', 'Return Budget Account', required=True),
    }
    
sisb_dev_account_contrapartida_rel()

