# -*- encoding: utf-8 -*-
###############Credits#########################################################	
#    Finance by: Industrias Diana, C.A. http://www.industriasdiana.gob.ve	
###############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

from osv import osv
from osv import fields
import sys
from tools.translate import _
import time
import netsvc

class purchase_sales_book_wizard(osv.osv_memory):
    _name = "sales.book.wizard"
    _inherit = "sales.book.wizard"
    
    def _check_email_server(self, cr, uid, ids, context=None):
        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),
                                        ('state','=','confirm')], context=False)
        if not smtp_server:
            raise osv.except_osv(_('Acción Denegada!'), _('Debe configurar un servidor de correos para generar el reporte'))
        return True
      
    def _check_user_email_adress(self, cr, uid, ids, context=None):
        email = self.pool.get('res.users').browse(cr, uid, uid).user_email
        if not email:
            raise osv.except_osv(_('Acción Denegada!'), _('El usuario debe tener una cuenta de correo asociada para recibir el reporte'))
        return email
    
    def _valid_report(self, cr, uid, ids,  book_type, date_start, date_end, context=None):
        """
        Devuelve True en caso de que existan datos en el rango de fechas
        seleccionados para la generacion del reporte y False en caso contrario.
        """
        if book_type == 'compras':
            report = self.pool.get("fiscal.reports.purchase")
        else:
            report = self.pool.get("fiscal.reports.sale")
        
        lines = report.search(cr, uid,['&', ('ai_date_invoice', '>=', date_start), ('ai_date_invoice', '<=', date_end),], count=True)
        if lines > 0:
            return True
        else:
            return False
    def create_scheduled_report(self, cr, uid, ids, context=None):
        """
        Crea una accion planificada que ejecutara un metodo para generar
        el reporte y enviarlo por correo electronico. Se envia una notificacion
        interna para indicar al usuario que se generara el reporte. Solo se
        permite crear la accion planificada si existe un servidor de correos
        configurado y el usuario posee un email asociado.
        """
        
        self._check_email_server(cr, uid, ids, context)
        email = self._check_user_email_adress(cr, uid, ids, context)
        
        data = {}
        data['ids'] = [1]
        data['form'] = self.read(cr, uid, ids, ['date_start',  'date_end',  'control_start', 'control_end', 'type'])[0]    
        book_type = 'compras' if data['form']['type'] == 'purchase' else 'ventas'
        data['model'] = "fiscal.reports.purchase.purchase_seniat"
        
        if self._valid_report(cr, uid, ids, book_type, data['form']['date_start'], data['form']['date_end'], context):
            subject = "Libro de %s SENIAT desde %s hasta %s" % (book_type, data['form']['date_start'], data['form']['date_end'])        
            request_message = "Se esta generando el reporte de libro de %s. Al finalizar se enviara a su direccion de correo '%s'" % (book_type, email)
            email_message = "Se adjunta el reporte de libro de %s" % (book_type)
            
            cron_parameters = dict()
            cron_parameters['data'] = data
            cron_parameters['subject'] = subject
            cron_parameters['message'] = email_message
            cron_parameters['file_name'] = subject
            cron_parameters['uid'] = uid
            vals = {
                'name': subject,
                'active': True,
                'user_id': uid,
                'interval_number': 1,
                'interval_type':'minutes',
                'numbercall': 1,
                'doall': True,
                'model': 'sales.book.wizard',
                'function': '_send_report_by_email',
                'args': '(' + str(cron_parameters) + ',)',
            }
            self.pool.get('ir.cron').create(cr, uid, vals, context)
            self._send_request(cr, uid, ids, subject, request_message, context)
            return {'type':'ir.actions.act_window_close'}
        else:
            raise osv.except_osv(_('Acción Denegada!'), _('No existen datos para el reporte en el rango de fechas seleccionado'))
    
    def _send_report_by_email(self, cr, uid, parameters, context=None):
        """
        Metodo que envia un correo electronico con el reporte de libro de
        compras/ventas
        """
        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),
                                        ('state','=','confirm')], context=False)
        subject = parameters['subject']
        message = parameters['message']
        data = parameters['data']
        file_name = parameters['file_name']
        report = self._create_report(cr, uid, data['ids'], data['model'], file_name, data)
        attachments = report[0] and [report[1]] or []
        if attachments == []:
            message = 'Ocurrio un error al generar el reporte.'

        try:
            if smtp_server:
                email_to = self.pool.get('res.users').browse(cr, uid, parameters['uid']).user_email
                if email_to:
                    smtp_server_id = smtp_server[0]
                    server_name = smtp_client.browse(cr, uid, smtp_server_id).server    
                    email = {
                            'server_id': smtp_server_id,
                            'name': (subject),
                            'server': server_name,
                            'body': (message),
                            'to': email_to,
                            'from': uid,
                            'attachments': attachments
                            }
                    smtp_client.send_email(cr, uid, smtp_server_id, email['to'], email['name'], email['body'], email['attachments'])
        finally:
            #Se elimina la accion planificada
            cron = self.pool.get('ir.cron')
            accion_planificada = cron.search(cr, uid, [('name','=', subject)])
            cron.unlink(cr, uid, accion_planificada)
                
    def _send_request(self, cr, uid, ids, subject, message, context=None):
        """
        Metodo que envia una notificacion interna de OpenERP
        """
        notificaciones = {
                'name': (subject),
                'act_to': uid,
                'act_from': 1,
                'body': (message),
                'active': True,
                'priority': '2',
                'state': 'waiting'
            }
        request_obj = self.pool.get('res.request')
        request_obj.create(cr, uid, notificaciones, context)
        return True
    
    def _create_report(self, cr, uid, res_ids, report_name=False, file_name=False, data=False):
        try:
            ret_file_name = '/tmp/'+file_name+'.pdf'
            service = netsvc.LocalService("report."+report_name);
            (result, format) = service.create(cr, uid, res_ids, data, {})
            fp = open(ret_file_name, 'wb+');
            fp.write(result);
            fp.close();
        except Exception,e:
            print 'Exception in create report:',e
            return (False, str(e))
        return (True, ret_file_name)

purchase_sales_book_wizard()
