#########################################################################
#                                                                       #
# Copyright (C) 2009  Sharoon Thomas                                    #
#                                                                       #
#This program is free software: you can redistribute it and/or modify   #
#it under the terms of the GNU General Public License as published by   #
#the Free Software Foundation, either version 3 of the License, or      #
#(at your option) any later version.                                    #
#                                                                       #
#This program is distributed in the hope that it will be useful,        #
#but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#GNU General Public License for more details.                           #
#                                                                       #
#You should have received a copy of the GNU General Public License      #
#along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#########################################################################
from osv import osv, fields
import datetime
from dateutil.relativedelta import *
import time

class fms_calendar(osv.osv):
    _name = "fms.calendar"
    _description = "An event recorder for Fleet Manager"
    
    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['desc', 'vehicle', 'type'], context)
        res = []
        for record in reads:
            name = ''
#            if record['desc']:
#                name = record['desc']
            if record['vehicle']:
                name += " [" + record['vehicle'][1] + ']'
            if record['type']:
                name += ' / ' + record['type'][1]
            res.append((record['id'], name))
        return res
    
    def _name_get_fnc(self, cr, uid, ids, prop, unknow_none, context):
        res = self.name_get(cr, uid, ids, context)
        return dict(res)
    
    _order = "datestart"
    
    def _onchange_startdate(self, cr, uid, ids, datestart, dateend):
        if datestart:
            if dateend == False:
                return {'value':{'dateend':datestart}}
        
    _columns = {
        'desc':fields.char('Short Description', size=20, required=True),
        'name': fields.function(_name_get_fnc, method=True, type="char", string='Name'),
        'vehicle':fields.many2one('fms.vehicles', string="Vehicle", domain="[('active','=','1')]", required=True),
        'type': fields.many2one('fms.parameters', string='Event Type', domain="[('type','=','event')]"),
        'datestart': fields.date('Event Start', required=True),
        'dateend': fields.date('Event End', required=True),
        'alertdate': fields.datetime('Alert alarm'),
        'alertuser': fields.many2one('res.users', string="Send alert to"),
        'notes':fields.text('Notes')
                }
    
fms_calendar()

class fms_fuelling(osv.osv):
    _name = "fms.fuelling"
    _description = "Fuelling management in FMS"
    
    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['invno', 'vendor', 'vehicle'], context)
        res = []
        for record in reads:
            name = ''
            if record['invno']:
                name = record['invno']
            if record['vendor']:
                name += " [" + record['vendor'][1] 
            if record['vehicle']:
                name += "/" + record['vehicle'][1]
            if record['vendor']:
                name += "]" 
            res.append((record['id'], name))
        return res
    
    def _name_get_fnc(self, cr, uid, ids, prop, unknow_none, context):
        res = self.name_get(cr, uid, ids, context)
        return dict(res)    
    
    def _get_rate(self, cr, uid, ids, prop, unknow_none, context):
        if ids:
            reads = self.read(cr, uid, ids, ['quantity', 'sub_total'], context)
            res = []
            for record in reads:
                res.append((record['id'], record['sub_total'] / record['quantity']))
            return dict(res)
    
    def _get_total(self, cr, uid, ids, prop, unknow_none, context):
        if ids:
            reads = self.read(cr, uid, ids, ['misc_amt', 'sub_total'], context)
            res = []
            for record in reads:
                res.append((record['id'], record['sub_total'] + record['misc_amt']))
            return dict(res)
    
    def _get_last_mileage(self, cr, uid, ids, prop, unknow_none, context):
        if ids:
            reads = self.browse(cr, uid, ids, context)
            res = []
            for record in reads:
                curr_odo = record.odometer
                curr_qty = record.quantity
                cr.execute("select odometer from fms_fuelling where vehicle=" + str(record.vehicle.id) + " and invdate<='" + str(record.invdate) + "' and id<>" + str(record.id) + ";")
                prev_odo = cr.fetchall()
                prev_odo.sort()
                if prev_odo:
                    res.append((record['id'], (curr_odo - prev_odo[-1][0]) / curr_qty))
                else:
                    res.append((record['id'], 0))
            return dict(res)
        
    _order = 'invdate'
    
    _columns = {
        'name':fields.function(_name_get_fnc, method=True, type="char", size=100, string='Description'),
        'vendor':fields.many2one('res.partner', 'Supplier', domain="[('supplier','=','1')]", required=True),
        'invno':fields.char('Invoice No', size=20, required=True),
        'invdate':fields.date('Invoice Date', required=True),
        'vehicle':fields.many2one('fms.vehicles', 'Vehicle', domain="[('active','=','1')]", required=True),
        'odometer':fields.integer('Odometer', required=True),
        'quantity':fields.float('Fuel Quantity', digits=(10, 3), required=True),
        'rate':fields.function(_get_rate, method=True, type="float", string="Rate"),
        'sub_total':fields.float(string="Sub Total", digits=(10, 2), required=True),
        'misc_amt':fields.float('Other bill items', digits=(10, 2), help="Enter any other debits or credits in the bill"),
        'total':fields.function(_get_total, method=True, type="float", string="Total Amount"),
        'narration':fields.char('Narration', size=100),
        'mileage':fields.function(_get_last_mileage, method=True, type="float", string="Mileage from last billing"),
        'driver':fields.many2one('hr.employee', string='Driver', domain="[('is_driver','=','1')]")
                }
    _defaults = {
        'quantity':lambda * a:1,
        'sub_total':lambda * a:1,
        'misc_amt':lambda * a:0,
        'odometer':lambda * a:0,
                 }
fms_fuelling()

class fms_pm(osv.osv):
    _name = "fms.pm"
    _description = "Preventive Maintainance System"
    
    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['vehicle', 'type'], context)
        res = []
        for record in reads:
            name = ''
            if record['vehicle']:
                name += record['vehicle'][1]
            if record['type']:
                name += "/" + record['type'][1]
            res.append((record['id'], name))
        return res
    
    def _name_get_fnc(self, cr, uid, ids, prop, unknow_none, context):
        res = self.name_get(cr, uid, ids, context)
        return dict(res)
    
    def _days_next_due(self, cr, uid, ids, prop, unknow_none, context):
        if ids:
            reads = self.browse(cr, uid, ids, context)
            res = []
            for record in reads:
                if (record.meter == "days") or (record.meter == 'first'):
                    interval = datetime.timedelta(days=record.days_interval)
                    last_done = record.days_last_done
                    last_done = datetime.datetime.fromtimestamp(time.mktime(time.strptime(last_done, "%Y-%m-%d")))
                    next_due = last_done + interval
                    res.append((record.id, next_due.strftime("%Y-%m-%d")))
                else:
                    res.append((record.id, False))
            return dict(res)
    
    def _days_due(self, cr, uid, ids, prop, unknow_none, context):
        if ids:
            reads = self.browse(cr, uid, ids, context)
            res = []
            for record in reads:
                if (record.meter == "days") or (record.meter == 'first'):
                    interval = datetime.timedelta(days=record.days_interval)
                    last_done = record.days_last_done
                    last_done = datetime.datetime.fromtimestamp(time.mktime(time.strptime(last_done, "%Y-%m-%d")))
                    next_due = last_done + interval
                    NOW = datetime.datetime.now()
                    due_days = next_due - NOW
                    res.append((record.id, due_days.days))
                else:
                    res.append((record.id, False))
            return dict(res)

    def _dist_next_due(self, cr, uid, ids, prop, unknow_none, context):
        if ids:
            reads = self.browse(cr, uid, ids, context)
            res = []
            for record in reads:
                if (record.meter == "distance") or (record.meter == 'first'):
                    next_due = record.dist_interval + record.dist_last_done
                    res.append((record.id, next_due))
                else:
                    res.append((record.id, False))
            return dict(res)

    def _dist_due(self, cr, uid, ids, prop, unknow_none, context):
        if ids:
            reads = self.browse(cr, uid, ids, context)
            res = []
            for record in reads:
                if (record.meter == "distance") or (record.meter == 'first'):
                    next_due = record.dist_interval + record.dist_last_done
                    last_known_odo = self.pool.get('fms.vehicles')._get_last_odometer(cr, uid, [record.vehicle.id])
                    if last_known_odo:
                        last_known_odo = last_known_odo[record.vehicle.id]
                    else:
                        last_known_odo = 0
                    res.append((record.id, next_due - last_known_odo))
                else:
                    res.append((record.id, False))
            return dict(res)
    
    def _get_state(self, cr, uid, ids, prop, unknow_none, context):
        res = {}
        if ids:
            reads = self.browse(cr, uid, ids, context)
            for record in reads:
                if record.meter == 'first':
                    if (record.days_left <= 0) or (record.dist_left <= 0):
                        res[record.id] = 'Overdue'
                    elif (record.days_left <= record.days_warn_period) or (record.dist_left <= record.dist_warn_period):
                        res[record.id] = 'Approaching'
                    else:
                        res[record.id] = 'Safe'
                elif record.meter == 'days':
                    if (record.days_left <= 0):
                        res[record.id] = 'Overdue'
                    elif (record.days_left <= record.days_warn_period):
                        res[record.id] = 'Approaching'
                    else:
                        res[record.id] = 'Safe'
                else:
                    if (record.dist_left <= 0):
                        res[record.id] = 'Overdue'
                    elif (record.dist_left <= record.dist_warn_period):
                        res[record.id] = 'Approaching'
                    else:
                        res[record.id] = 'Safe'
            return res
                    
        
    #_order = 'dist_left,days_left'
    _columns = {
       'name':fields.function(_name_get_fnc, method=True, type="char", size=100, string='Description'),
       'type':fields.many2one('fms.parameters', 'PM Type', domain="[('type','=','pmtype')]"),
       'vehicle':fields.many2one('fms.vehicles', 'Vehicle', domain="[('active','=','1')]"),
       'meter':fields.selection([
                ('days', 'Days'),
                ('distance', 'Distance'),
                ('first', 'First of Either')
                                 ], 'Meter for PM', required=True),
        'reccurant':fields.boolean('Recurrant ?', help="Tick this if the preventive maintainance is continously repeating."),
        #Days 
        'days_interval':fields.integer('Interval'),
        'days_last_done':fields.date('Last done on'),
        'days_next_due':fields.function(_days_next_due, method=True, type="date", string='Next Due date'),
        'days_warn_period':fields.integer('Alert Days'),
        'days_left':fields.function(_days_due, method=True, type="integer", string='Due days'),
        #Distance
        'dist_interval':fields.integer('Interval'),
        'dist_last_done':fields.integer('Last done on'),
        'dist_next_due':fields.function(_dist_next_due, method=True, type="integer", string='Next Due odometer'),
        'dist_warn_period':fields.integer('Alert Odometer'),
        'dist_left':fields.function(_dist_due, method=True, type="integer", string='Due distance'),
        #Prop
        'state':fields.function(_get_state, method=True, type="char", string='Status')
                }
    _defaults = {
        'meter': lambda * a:'distance',
        'reccurant': lambda * a:True,
                 }
fms_pm()
