#########################################################################
#                                                                       #
# Copyright (C) 2009  Sharoon Thomas                                    #
#                                                                       #
#This program is free software: you can redistribute it and/or modify   #
#it under the terms of the GNU General Public License as published by   #
#the Free Software Foundation, either version 3 of the License, or      #
#(at your option) any later version.                                    #
#                                                                       #
#This program is distributed in the hope that it will be useful,        #
#but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#GNU General Public License for more details.                           #
#                                                                       #
#You should have received a copy of the GNU General Public License      #
#along with this program.  If not, see <http://www.gnu.org/licenses/>.  #
#########################################################################
from osv import osv,fields

class fms_parameters(osv.osv):
    _name = "fms.parameters"
    _description = "Generic parameters for Fleet Manager"
    _order = "type"
    _columns = {
        'name':fields.char('Parameter Value',size=200,required=True),
        'type':fields.selection([
                ('fuels','Fuels'),
                ('model','Models'),
                ('make','Make'),
                ('event','Event'),
                ('category','Vehicle Category'),
                ('pmtype','Prev. Maintainance'),
                ('lcsclass', 'License class'),
                ],'Type',required=True)
                }
fms_parameters()

class fms_vehicles(osv.osv):
    _name = "fms.vehicles"
    _description = "Stores Record of all vehicles"
    
    def _get_mileage(self, cr, uid, ids, prop, unknow_none, context):
        res = {}
        for id in ids:
            fuel_obj = self.pool.get('fms.fuelling')
            fuel_ids = fuel_obj.search(cr,uid,[('vehicle','=',id)])
            readings = fuel_obj.read(cr,uid,fuel_ids,['mileage'])
            total = 0
            for record in readings:
                total += record['mileage']
            try:
                mileage = total / (len(readings)-1)
            except:
                mileage = 0
            res[id]=mileage
        return res
    
    def _get_last_odometer(self,cr,uid,ids,ctx={}):
        res = {}
        for id in ids:
            query = "SELECT odometer FROM fms_fuelling where vehicle=%s ORDER BY invdate DESC limit 1" %id
            cr.execute(query)
            result = cr.fetchone()
            if result:
                res[id] = result[0]
            else:
                res[id] = 0
        return res
    _columns = {
        'name': fields.char('Vehicle #',size=20,required=True),
        'make': fields.many2one('fms.parameters',string='Make',domain="[('type','=','make')]"),#,type="selection")
        'model': fields.many2one('fms.parameters',string='Model',domain="[('type','=','model')]"),
        'chassisno': fields.char('Chassis #',size=50),
        'purchaseprice': fields.float('Purchase Price',digits=(10,3)),
        'fueltype': fields.many2one('fms.parameters',string='Fuel Type',domain="[('type','=','fuels')]"),
        'fuelcap': fields.float('Fuel Capacity',digits=(10,3)),
        'avg_mileage':fields.function(_get_mileage,method=True,type="float",string="Avg. Mileage"),
        'fullmass': fields.float('Full Mass',digits=(10,3)),
        'loadcap': fields.float('Loading Capacity',digits=(10,3)),
        'notyres': fields.integer('No of Tyres',help="Tyres Including spare wheel"),
        'active': fields.boolean('Active',help="Select if the vehicle is active"),
        'events': fields.one2many('fms.calendar','vehicle','Events related to Vehicle'),
        'state':fields.selection([
                        ('available','Available For Use'),
                        ('availablewc','Available (Work Complete)'),
                        ('awaitinghp','Awaiting High Priority'),
                        ('awaitinglp','Awaiting Low Priority'),
                        ('awaitingpm','Awaiting for PM'),
                        ('repair','Currently being repaired'),
                        ('waiting','Waiting for Parts'),
                        ('outofsc','Out of Service')
                                  ],'Status',required=True),
        'tag_plate' : fields.char('Tag plate', size=20, required=True),
    }
    _defaults = {
        'active':lambda *a: True,
        'state':lambda *a: 'available'
                 }
fms_vehicles()
