# -*- coding: utf-8 -*-
"""
Módulo utilizado para incluir la hoja de carga en el módulo de ventas de SISB.
El documento permite asociar los albaranes a despachar en un camion asi como registrar la capacidad del vehiculo.
A patir del documento de la hoja de carga se podrá facturar el flete del camion.
Se puede crear la hoja de carga a partir de un conjunto de albaranes seleccionados.
"""

import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp

#----------------------------------------------------------
#  Transportista
#----------------------------------------------------------

class hoja_carga_transportista(osv.osv):
    _name = "hoja.carga.transportista"
    _description = "Transportista"
    _order = "name"

    _columns = {
        'name': fields.char('Nombre de la Transportista', size=64, select=True, required=True),
        'active': fields.boolean('Active', help="Si el campo activo esta en falso, permite ocultar la transportista sin eliminarla."),
        'partner_id': fields.many2one('res.partner', 'Transportista', required=True),
        'product_id': fields.many2one('product.product', 'Servicio de flete', required=True),
    }

    _defaults = {
        'active': lambda *args:1
    }
hoja_carga_transportista()

#---------------------------------------------------------
#  Rutas
#---------------------------------------------------------

class trucks_areas(osv.osv):
    _name = "trucks.areas"

    _columns = {
    'name':fields.char("Nombre Zona",size=128),
    'shop_id': fields.many2one('sale.shop', 'Sucursal', required=True),
    'childs_ids': fields.one2many("trucks.routes","parent_id","Rutas"),
    'rates_ids': fields.one2many("rates.routes","trucks_areas_id","Tarifas"),
    'state':fields.selection([
                        ('activate','Activada'),
                        ('deactivate','Desactivada')
                                  ],'Status',required=True)
    }
trucks_areas()

class trucks_routes(osv.osv):
    _name = "trucks.routes"

    _columns = {
    'name': fields.char("Ruta",size=128),
    'parent_id': fields.many2one("trucks.areas","Zona"),
    }
trucks_routes()

class trucks_types(osv.osv):
    _name = "trucks.types"

    _columns = {
    'name': fields.char('Tipo',size=64),
    'load': fields.float('Carga(KGS)'),
    'trucks_vehicles_id': fields.one2many("trucks.vehicles","truck_type","Camiones")
    }
trucks_types()

class rates_routes(osv.osv):
    _name="rates.routes"
    _description="Tarifas"

    _columns = {
    'name': fields.char("Tarifa", size=64),
    'trucks_type': fields.many2one("trucks.types","Tipo Camion"),
    'rate': fields.float("Monto Flete"),
    'trucks_areas_id': fields.many2one("trucks.areas","Zona"),
    }
rates_routes()

class trucks_vehicles(osv.osv):
    _name = "trucks.vehicles"

    _columns = {
    'name': fields.char("Nombre Vehiculo", size=64),
    'registration': fields.char("Placa vehiculo", size=64),
    #~ 'company': fields.many2one('res.partner', 'Transportista', required=True),
    #~ 'location': fields.many2one('sale.shop', 'Sucursal', required=True),
    #~ 'trucks_areas_id': fields.many2one("trucks.areas","Zona"),
    'company': fields.many2one('hoja.carga.transportista', 'Transportista'),
    'year': fields.char("Año",size=64),
    'make': fields.char("Fabrica",size=64),
    'model': fields.char("Modelo",size=64),
    'truck_type': fields.many2one("trucks.types","Tipo Camion"),
    'max_load': fields.float("Carga Maxima"),
    'serial': fields.char("Serial",size=64),
    'volumen': fields.float("Volumen de Carga"),
    'gps_device': fields.one2many("trucks.gps.devices","vehicle_id","GPS"),
    'ownership':fields.selection([
                        ('own','Propio'),
                        ('contracted','Contratado')
                                  ],'Servicio',required=True),
    'state':fields.selection([
                        ('available','Disponible'),
                        ('outofsc','Fuera de Servicio')
                                  ],'Estado',required=True),
    }

    def _check_capacidad(self, cr, uid, ids, context=None):
        obj = self.browse(cr, uid, ids[0], context=context)
        if obj.truck_type.load < obj.max_load:
            return False
        return True

    _constraints = [
        (_check_capacidad, 'La capacidad del camion no corresponde al tipo de camion seleccionado. ', ['max_load']),
    ]

trucks_vehicles()

class trucks_gps_devices(osv.osv):
    _name = 'trucks.gps.devices'

    _columns = {
    'name': fields.char("Nombre GPS", size=64),
    'imei_esn': fields.char("Numero IMEI/ESN", size=64),
    'vehicle_id': fields.many2one("trucks.vehicles","Camion"),
    'state':fields.selection([
                        ('online','En Linea'),
                        ('offline','Fuera de linea')
                                  ],'Status',required=True)
    }

trucks_gps_devices()

#----------------------------------------------------------
# Hoja de Carga
#----------------------------------------------------------

class hoja_de_carga(osv.osv):
    _name = "hoja.de.carga"
    _description = "Hoja de carga"
    _order = "id desc"
    

    #Validaciones antes de crear/actualizar una hoja de carga
    def validar_hoja_de_carga(self, cr, user, ids, vals, context=None):
        if context is None:
            context = {}

        res=[]
        #validar la existencia de albaranes en la hoja de carga
        if vals.get('picking_ids'):
            op,val,lista = vals['picking_ids'][0]
            if lista == []:
                raise osv.except_osv(_('Error al registrar!'), _('Debe incluir los albaranes en la hoja de carga.'))
            else:
                res = self.search(cr, user, [('picking_ids', 'in', lista), ('state', '!=', 'cancel'), ('id','not in',ids)])
                if res:
                    texto = ''
                    for line in self.browse(cr, user, res, context=context):
                        texto += line.name + '  '
                    raise osv.except_osv(_('Error al registrar!'), _("Hay albaranes asociados en otras hojas de cargas: %s ") % (texto))

        #Validar si la hoja de carga es facturable, debe tener asociado una transportista
        state_hc = self.validar_campo(cr, user, ids, vals, 'hoja_carga_state', context=context)
        partner = self.validar_campo(cr, user, ids, vals, 'transportista_id', context=context)

        if state_hc and state_hc != 'none' and not partner:
            raise osv.except_osv(_('Error al registrar!'), _("El alabarán debe tener asociado una transportista. Su estado es para facturar. "))


        #Valida si hay asociado un vehiculo, el peso debe corresponder con el configurado en el camion
        vehiculo_id = self.validar_campo(cr, user, ids, vals, 'vehiculo', context=context)
        capacidad_v = self.validar_campo(cr, user, ids, vals, 'capacidad_vehiculo', context=context)

        if vehiculo_id and state_hc and state_hc != 'none':
            if type(vehiculo_id) != int:
                vehiculo_id = vehiculo_id.id
            capacidad = self.pool.get('trucks.vehicles').browse(cr, user, vehiculo_id).max_load
            if capacidad_v != capacidad:
                raise osv.except_osv(_('Error al registrar!'), _("El capacidad del vehiculo no corresponde con la configuracion del vehiculo seleccionado."))

        return True

    def validar_campo(self, cr, user, ids, vals, campo, context=None):
        if context is None:
            context = {}

        value = ''
        result = False

        if type(ids) == type([]): #si esta editando
            value = vals.get(campo, 'no')
            if value == 'no':
                result = getattr(self.browse(cr, user, ids[0], context=context),campo)
            else:
                result = vals.get(campo)
        else: #esta creando
            result = vals.get(campo, False)

        return result
        
    def depurar_m2m(self, picking_ids):
        #*** Por el bug de openerp web con m2m a stock_picking al seleccionar directamente el mismo registro
        lista = list(set(picking_ids[0][2]))
        picking_ids = [(6,0,lista)]

        return picking_ids

    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
            
        if vals.get('picking_ids',False):
            vals['picking_ids'] = self.depurar_m2m(vals['picking_ids'])
  
        self.validar_hoja_de_carga(cr, user, 0, vals, context)

        if ('name' not in vals) or (vals.get('name') == '/'):
            vals['name'] = self.pool.get('ir.sequence').get(cr, user, 'hoja_de_carga_tseq')
        
        
        new_id = super(hoja_de_carga, self).create(cr, user, vals, context)

        if not type(new_id) == type([]):
            new_id = [new_id]

        for item in self.browse(cr, user, new_id, context=context):
            for pick in item.picking_ids:
                self.pool.get('stock.picking').write(cr, user, pick.id, {'hoja_carga_id':item.id})

        return new_id[0]

    def write(self, cr, user, ids, vals, context=None):
        if context is None:
            context = {}
        
        if vals.get('picking_ids',False):
            vals['picking_ids'] = self.depurar_m2m(vals['picking_ids'])
            
        self.validar_hoja_de_carga(cr, user, ids, vals, context)

        record = super(hoja_de_carga, self).write(cr, user, ids, vals, context)
        
        if not type(ids) == type([]):
            up_ids = [ids]
        else:
            up_ids = ids
            
        for item in self.browse(cr, user, up_ids, context=context):
            list_pick = []
            for pick in item.picking_ids:
                self.pool.get('stock.picking').write(cr, user, pick.id, {'hoja_carga_id':item.id})
                list_pick.append(pick.id)
            
            #Para desasociar los albaranes eliminados de la hoja de carga, 
            #de este modo pueda estar disponibles los albaranes desasociados en otra hoja de carga.
            pick_hc_id = self.pool.get('stock.picking').search(cr, user,[('hoja_carga_id','=',item.id)])
            
            #Si está cancelado se desasocia todos los albaranes
            if item.state == 'cancel':
                self.pool.get('stock.picking').write(cr, user, pick_hc_id, {'hoja_carga_id':False})
            else:
                pic_asoc = set(list_pick)
                pic_dele = set(pick_hc_id)
                resta = pic_dele - pic_asoc
                self.pool.get('stock.picking').write(cr, user, list(resta), {'hoja_carga_id':False})
            #########################################################################################
            
            
        return record

    #para obtener las líneas de los albaranes asociados
    def _get_lines(self, cr, uid, ids, name, arg, context=None):
        #~ Forzar uid para que realice el calcul del costo promedio con todas las cantidades existentes
        #~ Es temporal, mientras se consigue una alternativa para filtrar almacenes por grupo
        uid = 1

        if context is None:
            context = {}

        res = {}
        for hoja_carga in self.browse(cr, uid, ids, context=context):
            id = hoja_carga.id
            res[id] = []
            if not hoja_carga.picking_ids:
                continue
            data_lines = [x.move_lines for x in hoja_carga.picking_ids]
            l=[]
            for sm_det in data_lines:
                for sm in sm_det:
                    l.append(sm.id)
                    res[id] = l
        return res

    #Para obtener el peso total de todos los productos de los albaranes asociados
    def _peso_total(self, cr, uid, ids, name, arg, context=None):
        if context is None:
            context = {}

        res = {}
        uom_obj = self.pool.get('product.uom')
        obj_move = self.pool.get('stock.move')

        list_moves = self._get_lines(cr, uid, ids, name, arg, context=context)
        for hoja_carga in self.browse(cr, uid, ids, context=context):
            id = hoja_carga.id
            res[id] = 0.00
            if list_moves.has_key(id):
                for item in obj_move.browse(cr, uid, list_moves[id], context=context):
                    if item.product_id.weight > 0.00:
                        converted_qty = item.product_qty

                        if item.product_uom.id <> item.product_id.uom_id.id:
                            converted_qty = uom_obj._compute_qty(cr, uid, item.product_uom.id, item.product_qty, item.product_id.uom_id.id)

                        res[id] += (converted_qty * item.product_id.weight)
        return res

    #Para obtener las facturas registradas a los albaranes asociados
    def _get_picking_invoice(self, cr, uid, ids, name, arg, context=None):
        if context is None:
            context = {}

        res = {}
        for hoja_carga in self.browse(cr, uid, ids, context=context):
            id = hoja_carga.id
            res[id] = []
            if not hoja_carga.picking_ids:
                continue
            data_lines = [x.sisb_invoice_ids for x in hoja_carga.picking_ids]

            l=[]
            for invoices in data_lines:
                for il in invoices:
                    l.append(il.id)
                    res[id] = l
        return res

    _columns = {
        'name': fields.char('Referencia', size=64, select=True, required=True),
        'shop_id': fields.many2one('sale.shop', 'Origen', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'state': fields.selection([
            ('draft', 'Draft'),
            ('confirmed', 'Confirmed'),
            ('done', 'Done'),
            ('cancel', 'Cancelled'),
            ('exception', 'Exception'),
            ], 'State', readonly=True, select=True,
            help="* Draft: En la espera de registrar los albaranes a despachar en el camion\n"\
                 "* Confirmed: Confirmado los albaranes a despachar en el camion\n"\
                 "* Done: Ha sido pesado y enviado en el camion\n"\
                 "* Cancelled: Ha sido cancelado\n"\
                 "* Exception: Ha sido pasado a estado de excepcion"),
        'date': fields.datetime('Fecha Creado', help="Fecha Creado", select=True, readonly=True),
        'date_done': fields.datetime('Fecha Realizado', help="Fecha Realizado", readonly=True),
        'transportista_id': fields.many2one('hoja.carga.transportista', 'Transportista', readonly=True,  states={'draft':[('readonly',False)]}),
        'hoja_carga_state': fields.selection([
            ("invoiced", "Invoiced"),
            ("2binvoiced", "To Be Invoiced"),
            ("none", "Not Applicable")], "Control de Factura",
            select=True, required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'company_id': fields.many2one('res.company', 'Company', required=True, select=True),
        'picking_ids': fields.many2many('stock.picking', 'stock_picking_hoja_carga_rel', 'hoja_carga_id', 'picking_id', 'Asociated Picking', readonly=True, states={'draft': [('readonly', False)]}),
        'ruta': fields.many2one('trucks.areas', 'Ruta', select=True, readonly=True,  states={'draft':[('readonly',False)]}),
        'tarifa': fields.many2one('rates.routes', 'Tarifa', select=True, readonly=True,  states={'draft':[('readonly',False)]}),
        'tipo_camion': fields.many2one('trucks.types', 'Tipo de camion', select=True, readonly=True,  states={'draft':[('readonly',False)]}),
        'vehiculo': fields.many2one('trucks.vehicles', 'Vehiculo', select=True, readonly=True,  states={'draft':[('readonly',False)]}),
        'conductor': fields.char('Conductor', size=64, readonly=True,  states={'draft':[('readonly',False)]}),
        'note': fields.text('Notes', readonly=True,  states={'draft':[('readonly',False)]}),
        'stock_move_lines':fields.function(_get_lines, method=True, type='many2many', relation='stock.move', string='Stock Move Lines'),
        'peso_total':fields.function(_peso_total, method=True, type='float', string='Peso total (Kg)', digits_compute=dp.get_precision('Stock Weight'), store = False),
        'picking_invoice_ids':fields.function(_get_picking_invoice, method=True, type='many2many', relation='account.invoice', string='Stock Picking Invoices'),
        'capacidad_vehiculo':fields.float('Capacidad del Vehiculo', digits_compute=dp.get_precision('Stock Weight'), states={'done': [('readonly', True)]}),
        'invoice_ids': fields.many2many('account.invoice', 'hoja_carga_invoice_rel', 'hoja_carga_id', 'invoice_id', 'Invoices'),
        'precintos': fields.char('Precintos', size=255, states={'done': [('readonly', True)]}),
        'peso_romana': fields.float('Peso de la Romana', digits_compute=dp.get_precision('Stock Weight'), states={'done': [('readonly', True)]}),
        'ticket_romana': fields.char('Ticket de la Romana', size=100, states={'done': [('readonly', True)]}),
    }
    _defaults = {
        'name': lambda self, cr, uid, context: '/',
        'state': 'draft',
        'hoja_carga_state': '2binvoiced',
        'date': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'hoja.de.carga', context=c),
    }

    def onchange_tipo_camion(self, cr, uid, ids, tipo_camion, tarifa, vehiculo, capacidad_vehiculo):
        result = self.onchange_tarifa(cr, uid, ids, tarifa, vehiculo, capacidad_vehiculo, True)
        result.update({'warning': {'message': 'No debe cambiar manualmente esta registro.'}})
        return result
    
    def onchange_shop_id(self, cr, uid, ids, shop_id, transportista_id, ruta, tarifa, vehiculo):
        result = {}

        result = {'value': {
            'ruta': False,
            'tarifa': False,
            'vehiculo': False,
            'capacidad_vehiculo': 0.00,
            'tipo_camion': False,
            }
        }
        return result

    def onchange_transportista_id(self, cr, uid, ids, transportista_id, ruta, tarifa, vehiculo):
        result = {}

        result = {'value': {
            'vehiculo': False,
            'capacidad_vehiculo': 0.00,
            'tipo_camion': False,
            }
        }
        return result

    def onchange_ruta(self, cr, uid, ids, ruta, tarifa, vehiculo):
        result = {}

        result = {'value': {
            'tarifa': False,
            'vehiculo': False,
            'capacidad_vehiculo': 0.00,
            'tipo_camion': False,
            }
        }
            
        return result

    def onchange_picking_ids(self, cr, uid, ids, picking_ids):
        result = {}
        acum = 0.0
        #*** Por el bug de openerp web con m2m a stock_picking al seleccionar directamente el mismo registro
        pickingids = list(set(picking_ids[0][2]))
        picking_obj = self.pool.get('stock.picking')
        for pick in picking_obj.browse(cr, uid, pickingids):
            acum += pick.peso_bruto
        result = {'value': {
            'peso_total': acum,
            }
        }
        return result

    def onchange_tarifa(self, cr, uid, ids, tarifa, vehiculo, capacidad_vehiculo, saltar = False):
        result = {}
        if saltar:
            if tarifa:
                trucks_type_id = self.pool.get('rates.routes').browse(cr, uid, tarifa).trucks_type.id
            else:
                trucks_type_id = False
                
            result = {'value': {
                    'vehiculo': vehiculo,
                    'capacidad_vehiculo': capacidad_vehiculo,
                    'tipo_camion':trucks_type_id,
                    }
                }
        else:
            if tarifa:
                trucks_type_id = self.pool.get('rates.routes').browse(cr, uid, tarifa).trucks_type.id
                result = {'value': {
                    'vehiculo': False,
                    'capacidad_vehiculo': 0.00,
                    'tipo_camion':trucks_type_id,
                    }
                }
            else:
                result = {'value': {
                    'vehiculo': False,
                    'capacidad_vehiculo': 0.00,
                    'tipo_camion':False,
                    }
                }
        return result

    def onchange_vehiculo(self, cr, uid, ids, vehiculo):
        result = {}
        if vehiculo:
            capacidad = self.pool.get('trucks.vehicles').browse(cr, uid, vehiculo).max_load
            result = {'value': {
                'capacidad_vehiculo': capacidad,
                }
            }
        else:
            result = {'value': {
                'capacidad_vehiculo': 0.00,
                }
            }
        return result

    def copy(self, cr, uid, id, default=None, context=None):
        raise osv.except_osv(_('Acción Denegada!'),_('No se permite duplicar hoja de cargas!'))
        return False

    #~ Procesos

    #Condiciones para pasar a confirmado
    def valida_confirm(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        #validar que las facturas de los albaranes para facturar existan y estén en edo done
        for hoja_carga in self.browse(cr, uid, ids, context=context):
            if not hoja_carga.picking_ids:
                raise osv.except_osv(_('Acción Denegada!'),_('La hoja de carga no tiene asociado albaranes!'))

            #lista de las facturas asociadas
            data_lines = []

            #Buscar en cada albarán las facturas asociadas si el albarán está para facturar
            for x in hoja_carga.picking_ids:
                if x.state != 'done':
                    raise osv.except_osv(_('Acción Denegada!'),_('Hay albaranes sin procesar!'))
                elif x.invoice_state != 'none' and x.sisb_invoice_ids == []:
                    raise osv.except_osv(_('Acción Denegada!'),_('Hay albaranes facturables sin facturas asociadas!'))
                elif x.invoice_state != 'none':
                    data_lines.append(x.sisb_invoice_ids)

            #Buscar en cada factura asociada si está en open
            for ai_list in data_lines:
                for ai_x_alb in ai_list:
                    if self.pool.get('account.invoice').browse(cr, uid, ai_x_alb.id, context=context).state != 'open':
                        raise osv.except_osv(_('Acción Denegada!'),_('Hay albaranes facturables con facturas sin completar!'))

        return True

    #Cambiar la hoja de carga a estado Confirmado
    def action_confirm(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_confirm(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'confirmed'}, context=context)
            return True
        else:
            return False

    #Condiciones para pasar a cancelado
    def valida_cancel(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        for item in self.browse(cr, uid, ids, context=context):
            if item.state == 'done':
                raise osv.except_osv(_('Acción Denegada!'),_('La hoja de carga no no se puede cancelar!'))
        return True

    #Cambiar la hoja de carga a estado Cancelado
    def action_cancel(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_cancel(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'cancel'}, context=context)
        return True

    #Condiciones para pasar a excepcion
    def valida_excepcion(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        for item in self.browse(cr, uid, ids, context=context):
            if item.state in ('draft','cancel','done'):
                raise osv.except_osv(_('Acción Denegada!'),_('La hoja de carga no se puede pasar a estado de excepcion!'))
        return True

    #Cambiar la hoja de carga a estado Excepcion
    def action_excepcion(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_excepcion(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'exception'}, context=context)
        return True

    #Condiciones para pasar a realizado
    def valida_done(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        for hoja_carga in self.browse(cr, uid, ids, context=context):
            if hoja_carga.capacidad_vehiculo <= 0 or hoja_carga.capacidad_vehiculo < hoja_carga.peso_total:
                raise osv.except_osv(_('Acción Denegada!'),_('El peso sobrepasa la capacidad del vehiculo!'))
        resultado = self.valida_confirm(cr, uid, ids, context)
        return resultado

    #Cambiar la hoja de carga a estado Realizado
    def action_done(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_done(cr, uid, ids, context=context):
            for item in self.browse(cr, uid, ids, context=context):
                for pick in item.picking_ids:
                    self.pool.get('stock.picking').write(cr, uid, pick.id, {'hoja_carga_id':item.id})

            self.write(cr, uid, ids, {'state':'done', 'date_done': time.strftime('%Y-%m-%d %H:%M:%S')}, context=context)
        else:
            raise osv.except_osv(_('Acción Denegada!'),_('Hubo error en el proceso. Por favor contactar al administrador!'))

        return True

    #Sólo las hojas de carga en estado confirmado se podrán pasar a borrador
    def valida_restart(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        for item in self.browse(cr, uid, ids, context=context):
            if not item.state in ('confirmed','exception'):
                return False
        return True

    #Cambiar la hoja de carga a estado Borrador
    def action_restart(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        if self.valida_restart(cr, uid, ids, context=context):
            self.write(cr, uid, ids, {'state':'draft'}, context=context)
        else:
            raise osv.except_osv(_('Error al actualizar!'), _('La hoja de carga debe estar en estado confirmado o en excepcion para pasarla a borrador.'))
        return True

    #~ Para crear factura

    def _get_address_invoice(self, cr, uid, hoja_c):
        """ Gets invoice address of a partner
        @return {'contact': address, 'invoice': address} for invoice
        """
        partner_obj = self.pool.get('res.partner')
        partner = hoja_c.transportista_id.partner_id
        return partner_obj.address_get(cr, uid, [partner.id],
                ['contact', 'invoice'])

    def _get_comment_invoice(self, cr, uid, hoja_c):
        """
        @return: comment string for invoice
        """
        return hoja_c.note or ''

    def _get_price_unit_invoice(self, cr, uid, hoja_c, type):
        """ Obtener el precio unitario para facturar, tomar en cuenta
        que el precio del flete es segun la distancia recorrida en el proceso de despacho
        @param hoja_c: Hoja de carga
        @param type: Tipo de Factura (para este modelo es tipo in_invoice)
        @return: El precio total por la hoja de carga
        """
        return hoja_c.tarifa.rate

    def _get_discount_invoice(self, cr, uid, hoja_c):
        '''Retorna el descuento de  la hoja de carga'''
        return 0.0

    def _get_account_analytic_invoice(self, cr, uid, hoja_c):
        return False

    def action_invoice_create(self, cr, uid, ids, journal_id=False,
            group=False, type='in_invoice', context=None):
        """ Crea las facturas de las hojas de carga para facturar
        @param journal_id: Id del diario
        @param group: Si se agrupapor transportista o no
        @param type: Tipo de factura que debe crearse
        @return: Ids de la factura creada para la(s) hoja(s) de carga
        """

        if context is None:
            context = {}

        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        address_obj = self.pool.get('res.partner.address')
        stock_move_obj = self.pool.get('stock.move')

        res = {}
        invoices_group = {}

        inv_type = type
        for hc in self.browse(cr, uid, ids, context=context):
            if hc.hoja_carga_state != '2binvoiced':
                continue
            payment_term_id = False
            partner =  hc.transportista_id and hc.transportista_id.partner_id or False
            if not partner:
                raise osv.except_osv(_('Error, no hay empresa transportista !'),
                    _('Por favor registrar la empresa transportista para generar la factura.'))

            if not inv_type:
                inv_type = 'in_invoice'

            if inv_type == 'in_invoice':
                account_id = partner.property_account_payable.id
            else:
                raise osv.except_osv(_('Error!'),
                    _('Se debe generar una factura a proveedor a la empresa transportista.'))

            address_contact_id, address_invoice_id = \
                    self._get_address_invoice(cr, uid, hc).values()
            address = address_obj.browse(cr, uid, address_contact_id, context=context)

            comment = self._get_comment_invoice(cr, uid, hc)

            if group and partner.id in invoices_group:
                invoice_id = invoices_group[partner.id]
                invoice = invoice_obj.browse(cr, uid, invoice_id)
                invoice_vals = {
                    'name': (invoice.name or '') + ', ' + (hc.name or ''),
                    'origin': (invoice.origin or '') + ', ' + (hc.name or ''),
                    'comment': (comment and (invoice.comment and invoice.comment+"\n"+comment or comment)) or (invoice.comment and invoice.comment or ''),
                    'date_invoice':context.get('date_inv',False),
                    'user_id':uid
                }
                invoice_obj.write(cr, uid, [invoice_id], invoice_vals, context=context)
            else:
                invoice_vals = {
                    'name': hc.name,
                    'origin': (hc.name or ''),
                    'type': inv_type,
                    'account_id': account_id,
                    'partner_id': address.partner_id.id,
                    'address_invoice_id': address_invoice_id,
                    'address_contact_id': address_contact_id,
                    'comment': comment,
                    'payment_term': payment_term_id,
                    'fiscal_position': partner.property_account_position.id,
                    'date_invoice': context.get('date_inv',False),
                    'company_id': hc.company_id.id,
                    'user_id':uid
                }

                if journal_id:
                    invoice_vals['journal_id'] = journal_id
                invoice_id = invoice_obj.create(cr, uid, invoice_vals,
                        context=context)
                invoices_group[partner.id] = invoice_id

            res[hc.id] = invoice_id

            #*****OJO: Obtener el precio, descuento y cuenta analitica de alguna parte *****
            price_unit = self._get_price_unit_invoice(cr, uid, hc, inv_type)
            discount = self._get_discount_invoice(cr, uid, hc)
            account_analytic_id = self._get_account_analytic_invoice(cr, uid, hc)

            #************************************************************************************

            account_id = hc.transportista_id.product_id.product_tmpl_id\
                    .property_account_expense.id
            if not account_id:
                account_id = hc.transportista_id.product_id.categ_id\
                        .property_account_expense.id

            taxes_ids = [x.id for x in hc.transportista_id.product_id.supplier_taxes_id]

            origin = (hc.name or '')

            invoice_line_id = invoice_line_obj.create(cr, uid, {
                'name': hc.ruta.name + ' - ' + origin,
                'origin': origin,
                'invoice_id': invoice_id,
                'uom_id': hc.transportista_id.product_id.uom_id.id,
                'product_id': hc.transportista_id.product_id.id,
                'account_id': account_id,
                'price_unit': price_unit,
                'discount': discount,
                'quantity': 1,
                'invoice_line_tax_id': [(6, 0,taxes_ids)],
                'account_analytic_id': account_analytic_id,
            }, context=context)

            invoice_obj.button_compute(cr, uid, [invoice_id], context=context,
                    set_total=(inv_type in ('in_invoice', 'in_refund')))

            self.write(cr, uid, [hc.id], {
                'hoja_carga_state': 'invoiced',
                'invoice_ids': [[6, 0, [invoice_id]]],
                }, context=context)

        return res

hoja_de_carga()

#----------------------------------------------------------
# Stock Picking
#----------------------------------------------------------

class stock_picking(osv.osv):
    _inherit = 'stock.picking'
    
    def _get_city(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        if not isinstance(ids, list):
            ids = [ids]
        picking_brw = self.browse(cr, uid, ids)
        for p in picking_brw:
            res[p.id] = p.address_id.city
        return res
            
    def search_city(self, cr, uid, obj, name, args, context=None):
        addr_ids = self.pool.get('res.partner.address').search(cr, uid, [('city', 'ilike', '%'+str(args[0][2])+'%'),])
        ids = self.search(cr, uid, [('address_id', 'in', addr_ids)])
        return [('id','in',ids)]

    def _cal_peso_picking(self, cr, uid, ids, name, args, context=None):
        uid = 1
        res = {}
        stock_move_obj = self.pool.get("stock.move")
        for pick in ids:
            acum_weigth = 0.0
            acum_net = 0.0
            stock_move_ids = stock_move_obj.search(cr, uid, [('picking_id','=', pick)], context=context)
            stock_move_brw = stock_move_obj.browse(cr, uid, stock_move_ids, context=context)
            for move in stock_move_brw:
                acum_weigth += move.peso_bruto
                acum_net += move.peso_neto

            res[pick] =  {
                            'peso_bruto': acum_weigth,
                            'peso_neto': acum_net,
                            }

        return res

    _columns = {
        'sisb_invoice_ids': fields.many2many('account.invoice', 'hoja_carga_picking_invoice_rel', 'picking_id', 'invoice_id', 'Invoices'),
        'hoja_carga_id': fields.many2one('hoja.de.carga', 'Hoja de Carga Definitiva', ondelete='set null', select=True),
        'peso_bruto': fields.function(_cal_peso_picking, method=True, type='float', string='Peso Bruto (Kg)', digits_compute= dp.get_precision('Picking Weight'), multi='_cal_peso_picking', store = True),
        'peso_neto': fields.function(_cal_peso_picking, method=True, type='float', string='Peso Neto (Kg)', digits_compute= dp.get_precision('Picking Weight'), multi='_cal_peso_picking', store = False),
        'ciudad': fields.function(_get_city, fnct_search=search_city, method=True, type='char', size=128, store=False, string='Ciudad'),
    }

    _defaults = {
        'hoja_carga_id': False
    }
    
    def _invoice_hook(self, cr, uid, picking, invoice_id):
        '''Call after the creation of the invoice'''
        picking_obj = self.pool.get('stock.picking')
        picking_obj.write(cr, uid, [picking.id], {'sisb_invoice_ids': [[4,invoice_id]]})
        return
        

    #Metodo para crear hojas de carga a partir de los albaranes
    def action_hoja_carga_create(self, cr, uid, ids, shop_id=False,
        hoja_carga_state='none', transportista_id=False, context=None):

        if context is None:
            context = {}

        hc_obj = self.pool.get('hoja.de.carga')

        res = {}

        hc_vals = {
            'shop_id': shop_id,
            'transportista_id': transportista_id,
            'hoja_carga_state': hoja_carga_state,
            'picking_ids': [(6, 0,ids)],
            'note': 'Creado desde albaranes.',
        }

        hoja_carga_id = hc_obj.create(cr, uid, hc_vals,
                context=context)

        for item in ids:
            res[item] = hoja_carga_id

        return res
        
        
    def copy(self, cr, uid, id, defaults, context=None): 
        #Sobreescritura del metodo copy para borrar hoja_carga_id y las facturas asociadas en sisb_invoice_ids
        defaults['sisb_invoice_ids'] = []
        defaults['hoja_carga_id'] = ""
        return super(stock_picking, self).copy(cr, uid, id, defaults, context)

stock_picking()

class stock_move(osv.osv):
    _inherit = 'stock.move'

    def _cal_peso_move(self, cr, uid, ids, name, args, context=None):
        uid = 1
        res = {}
        uom_obj = self.pool.get('product.uom')
        for move in self.browse(cr, uid, ids, context=context):
            weight = weight_net = 0.00
            if move.product_id.weight > 0.00:
                converted_qty = move.product_qty

                if move.product_uom.id <> move.product_id.uom_id.id:
                    converted_qty = uom_obj._compute_qty(cr, uid, move.product_uom.id, move.product_qty, move.product_id.uom_id.id)

                weight = (converted_qty * move.product_id.weight)

                if move.product_id.weight_net > 0.00:
                    weight_net = (converted_qty * move.product_id.weight_net)

            res[move.id] =  {
                            'peso_bruto': weight,
                            'peso_neto': weight_net,
                            }
        return res

    #Para obtener la referencia de las facturas registradas a los albaranes asociados por cada sotock_move
    def _get_invoice_number_move_id(self, cr, uid, ids, name, args, context=None):
        uid = 1
        res = {}
        for move in self.browse(cr, uid, ids, context=context):
            #Si hay facturas desde albaran se las trae
            picking_invoice  = move.picking_id and move.picking_id.sisb_invoice_ids or []
            lista_inv = ''
            for inv in picking_invoice:
                numero_fact = inv.number or ''
                lista_inv += numero_fact + ' '
                
            if not lista_inv:
                #buscar la factura en el pedido de venta
                if move.picking_id.sale_id:
                    sale_order_invoice_id = move.picking_id.sale_id.invoice_ids
                    for inv in sale_order_invoice_id:
                        numero_fact = inv.number or ''
                        lista_inv += numero_fact + ' '

            res[move.id] = lista_inv.strip()

        return res

    #columnas utilizadas para el reporte de hoja de carga
    _columns = {
        'peso_bruto': fields.function(_cal_peso_move, method=True, type='float', string='Peso Bruto (Kg)', digits_compute= dp.get_precision('Stock Weight'), multi='_cal_peso_move', store = False),
        'peso_neto': fields.function(_cal_peso_move, method=True, type='float', string='Peso Neto (Kg)', digits_compute= dp.get_precision('Stock Weight'), multi='_cal_peso_move', store = False),
        'number_invoice': fields.function(_get_invoice_number_move_id, method=True, type='string', string='Peso neto', store = False),
        }

stock_move()
