# -*- coding: utf-8 -*-
##############################################################################
#
# Facturar Hoja Carga, permite facturar agrupado por transportista
#
##############################################################################

from osv import fields, osv

from tools.translate import _


class hoja_carga_invoice(osv.osv_memory):
    #Metodo para obtener los diarios de compras
    def _get_journal_id(self, cr, uid, context=None):
        if context is None:
            context = {}

        model = context.get('active_model')
        if not model or model != 'hoja.de.carga':
            return []

        model_pool = self.pool.get(model)
        journal_obj = self.pool.get('account.journal')
        res_ids = context and context.get('active_ids', [])
        vals = []
        browse_hoja_carga = model_pool.browse(cr, uid, res_ids, context=context)
        
        for hc in browse_hoja_carga:
            journal_type = ('purchase',)
            value = journal_obj.search(cr, uid, [('type','in',journal_type )])
            for jr_type in journal_obj.browse(cr, uid, value, context=context):
                t1 = jr_type.id,jr_type.name
                if t1 not in vals:
                    vals.append(t1)
        if not vals:
            raise osv.except_osv(_('Mensaje !'), _('No hay diario configurado para facturas de proveedor!'))
        
        return vals


    _name = "hoja.carga.invoice"
    _description = "Facturar hoja de carga"

    _columns = {
        'journal_id': fields.selection(_get_journal_id, 'Diario destino',required=True),
        'group': fields.boolean("Agrupado por transportista"),
        'invoice_date': fields.date('Fecha de la Facttura'),
    }

    def view_init(self, cr, uid, fields_list, context=None):
        if context is None:
            context = {}
        res = super(hoja_carga_invoice, self).view_init(cr, uid, fields_list, context=context)
        hoja_carga_obj = self.pool.get('hoja.de.carga')
        count = 0
        active_ids = context.get('active_ids',[])
        #No se va a permitir facturar en un mismo grupo hojas de carga de diferentes sucursales
        sucursal_ant = False
        for hc in hoja_carga_obj.browse(cr, uid, active_ids, context=context):
            if sucursal_ant != hc.shop_id and sucursal_ant:
                raise osv.except_osv(_('Mensaje !'), _('Intenta facturar hojas de cargas de diferentes sucursales.'))
            sucursal_ant = hc.shop_id
            if hc.hoja_carga_state != '2binvoiced':
                count += 1
            elif hc.state != 'done':
                count += 1
        if len(active_ids) >= 1 and count:
            raise osv.except_osv(_('Mensaje !'), _('Hay hojas de cargas que no requieren facturarse o su estado no es realizado.'))

        return res

    def open_invoice(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        invoice_ids = []
        data_pool = self.pool.get('ir.model.data')
        res = self.create_invoice(cr, uid, ids, context=context)
        invoice_ids += res.values()
        inv_type = context.get('inv_type', False)
        action_model = False
        action = {}
        if not invoice_ids:
            raise osv.except_osv(_('Error'), _('No se pudo crear factura'))
        if inv_type == "in_invoice":
            action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree2")
        else:
            raise osv.except_osv(_('Error'), _('No se puede crear facturas que no sean al proveedor'))
        if action_model:
            action_pool = self.pool.get(action_model)
            action = action_pool.read(cr, uid, action_id, context=context)
            action['domain'] = "[('id','in', ["+','.join(map(str,invoice_ids))+"])]"
        return action

    def create_invoice(self, cr, uid, ids, context=None):
        journal_mapping_type_inv = {
            'sale': 'out_invoice',
            'purchase': 'in_invoice',
            'sale_refund': 'out_refund',
            'purchase_refund': 'in_refund',
        }
        if context is None:
            context = {}
        hoja_carga_pool = self.pool.get('hoja.de.carga')
        invoice_data_obj = self.read(cr, uid, ids, ['journal_id', 'group', 'invoice_date'])
        journal_pool = self.pool.get('account.journal')

        context['date_inv'] = invoice_data_obj[0]['invoice_date']
        active_ids = context.get('active_ids', [])
        active_hoja_carga = hoja_carga_pool.browse(cr, uid, context.get('active_id',False), context=context)
        journal_type = journal_pool.browse(cr, uid, invoice_data_obj[0]['journal_id']).type
        inv_type = journal_mapping_type_inv[journal_type]
        context['inv_type'] = inv_type
        res = hoja_carga_pool.action_invoice_create(cr, uid, active_ids,
              journal_id = invoice_data_obj[0]['journal_id'],
              group = invoice_data_obj[0]['group'],
              type = inv_type,
              context=context)
        return res

hoja_carga_invoice()
