# -*- coding: utf-8 -*-
##############################################################################
#
# Facturar Hoja Carga, permite facturar agrupado por transportista
#
##############################################################################

from osv import fields, osv

from tools.translate import _


class picking_hoja_carga(osv.osv_memory):

    _name = "picking.hoja.carga"
    _description = "Crear hoja de carga"

    _columns = {
        'shop_id': fields.many2one('sale.shop', 'Sucursal', required=True),
        'hoja_carga_state': fields.selection([
            ("invoiced", "Invoiced"),
            ("2binvoiced", "To Be Invoiced"),
            ("none", "Not Applicable")], "Control de Factura",
            select=True, required=True),
        'transportista_id':  fields.many2one('hoja.carga.transportista', 'Transportista'),
    }

    def view_init(self, cr, uid, fields_list, context=None):
        if context is None:
            context = {}
        res = super(picking_hoja_carga, self).view_init(cr, uid, fields_list, context=context)
        picking_obj = self.pool.get('stock.picking')
        active_ids = context.get('active_ids',[])
        for pick in picking_obj.browse(cr, uid, active_ids, context=context):
            pick_asociado = pick.hoja_carga_id or None

            if not pick.type in ('internal','out'):
                raise osv.except_osv(_('Mensaje !'), _('Se crean hojas de cargas a partir de albaranes internos o de salida. Por favor revisar.'))
            elif pick.state in ('cancel'):
                raise osv.except_osv(_('Mensaje !'), _('Ha seleccionado un albaran en estado cancelado.'))
            elif not (pick_asociado == None):
                raise osv.except_osv(_('Mensaje !'), _('Ha seleccionado un albaran registrado en otra hoja de carga.'))
        return res

    def open_hoja_carga(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        hoja_carga_ids = []
        data_pool = self.pool.get('ir.model.data')
        res = self.create_hoja_carga(cr, uid, ids, context=context)
        lista = list(set(res.values()))
        hoja_carga_ids += lista #res.values()
        action_model = False
        action = {}
        if not hoja_carga_ids:
            raise osv.except_osv(_('Error'), _('No se pudo crear hoja de carga'))
        action_model,action_id = data_pool.get_object_reference(cr, uid, 'sisb_hoja_de_carga', "action_hoja_de_carga_form")
        
        if action_model:
            action_pool = self.pool.get(action_model)
            action = action_pool.read(cr, uid, action_id, context=context)
            action['domain'] = "[('id','in', ["+','.join(map(str,hoja_carga_ids))+"])]"

        return action

    def create_hoja_carga(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        stock_picking_pool = self.pool.get('stock.picking')
        hc_data_obj = self.read(cr, uid, ids, ['shop_id', 'hoja_carga_state', 'transportista_id'])

        active_ids = context.get('active_ids', [])
        active_stock_picking = stock_picking_pool.browse(cr, uid, context.get('active_id',False), context=context)
        
        res = stock_picking_pool.action_hoja_carga_create(cr, uid, active_ids,
              shop_id = hc_data_obj[0]['shop_id'],
              hoja_carga_state = hc_data_obj[0]['hoja_carga_state'],
              transportista_id =  hc_data_obj[0]['transportista_id'],
              context=context)
        return res

picking_hoja_carga()
