# -*- coding: utf-8 -*-
"""
Módulo utilizado para incluir presupuesto en la hoja de carga del SISB.
Al facturar la hoja de carga se registre la partida presupuestaria y la accion especifica.
"""

import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _
import decimal_precision as dp


#----------------------------------------------------------
# Hoja de Carga
#----------------------------------------------------------

class hoja_de_carga(osv.osv):
    _name = 'hoja.de.carga'
    _inherit = 'hoja.de.carga'
   

    def action_invoice_create(self, cr, uid, ids, journal_id=False,
            group=False, type='in_invoice', context=None):
        """ Crea las facturas de las hojas de carga para facturar
        @param journal_id: Id del diario
        @param group: Si se agrupapor transportista o no
        @param type: Tipo de factura que debe crearse
        @return: Ids de la factura creada para la(s) hoja(s) de carga
        """

        if context is None:
            context = {}
            
        if isinstance(ids, (int, long)):
            ids = [ids]
            
        if not ids:
            return {}

        res = super(hoja_de_carga, self).action_invoice_create(cr, uid,  ids, journal_id, group, type, context)
        
        inv_obj = self.pool.get('account.invoice')
        inv_line_obj = self.pool.get('account.invoice.line')
        suc_obj = self.pool.get('sale.shop')
        
        #Actualizar las lineas de las facturas creadas, colocando la accion especifica y la partida presupuestaria en sus lineas
        facturas_creadas = list(set(res.values()))
        
        #Consultar Accion Especifica
        accion_esp = self.browse(cr, uid, ids[0]).shop_id.accion_especifica_id.id or False
        
        #Consutar facturas creadas
        for fact in inv_obj.browse(cr, uid, facturas_creadas):
            for linea_fact in fact.invoice_line:
                #Consultar producto
                producto_partida = linea_fact.product_id.partida_egreso and linea_fact.product_id.partida_egreso.id or False
                #Actualizar
                values = {
                    'accion_especifica_id': accion_esp,
                    'partida_presupuesto': producto_partida,
                }
                line_update = inv_line_obj.write(cr, uid, linea_fact.id, values, context)

        return res

hoja_de_carga()
