# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import time
from datetime import datetime
from dateutil.relativedelta import relativedelta
from osv import fields, osv
import netsvc
import pooler
from osv.orm import browse_record, browse_null
from tools.translate import _

class purchase_requisition_merge(osv.osv_memory):
    _name = "purchase.requisition.merge"
    _description = "Purchase Requisition Merge"

    def fields_view_get(self, cr, uid, view_id=None, view_type='form',
                        context=None, toolbar=False, submenu=False):
        """
         Changes the view dynamically
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view.
        """

        if context is None:
            context={}
        res = super(purchase_requisition_merge, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=False)
        if context.get('active_model','') == 'purchase.requisition' and len(context['active_ids']) < 2:
            raise osv.except_osv(_('Warning'),
            _('Please select multiple order to merge in the list view.'))
        return res
        
    def merge_requisitions(self, cr, uid, ids, context=None):
        """
             @param self: The object pointer.
             @param cr: A database cursor
             @param uid: ID of the user currently logged in
             @param ids: the ID or list of IDs
             @param context: A standard dictionary
        """
        types = []
        for item in context.get('active_ids',[]):
            prl_id = self.pool.get('purchase.requisition.line').search(cr, uid, [('requisition_id', '=', item)])
            product = self.pool.get('purchase.requisition.line').browse(cr, uid, prl_id)[0].product_id
            if not (product.product_tmpl_id.type in types):
                types.append(product.product_tmpl_id.type)
        
        if len(types) > 1:
            raise osv.except_osv(_('ERROR'),
                _('No se permite fusionar Solicitudes de Compra de Productos con Solicitudes Compra de Servicios!'))

        po_obj = self.pool.get('purchase.requisition')
        if context is None:
            context = {}

        res = po_obj.do_merge(cr, uid, context.get('active_ids',[]), context)
        if res :
            self.do_create_order(cr, uid, res, context)
        
        return {
            'domain': "[('id', '=', " + str(res) + ")]",
            'name': 'Purchase Requisitions',
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'purchase.requisition',
            'view_id': False,
            'type': 'ir.actions.act_window',
            'search_view_id': res
        }
    
    def do_create_order(self, cr, uid, ids, context=None):
        """
             To Create a purchase orders .

             @param self: The object pointer.
             @param cr: A database cursor
             @param uid: ID of the user currently logged in
             @param ids: the ID or list of IDs
             @param context: A standard dictionary
             @return: {}

        """
        if context is None:
            context = {}
        
        record_ids = [ids]
        
        if record_ids:
            data =  self.read(cr, uid, ids)
            company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
            order_obj = self.pool.get('purchase.order')
            order_line_obj = self.pool.get('purchase.order.line')
            partner_obj = self.pool.get('res.partner')
            #~ tender_line_obj = self.pool.get('purchase.requisition.line')
            pricelist_obj = self.pool.get('product.pricelist')
            prod_obj = self.pool.get('product.product')
            tender_obj = self.pool.get('purchase.requisition')
            acc_pos_obj = self.pool.get('account.fiscal.position')
            #~ partner_id = data[0]['partner_id']

            partner_ids = []
            po_ids = order_obj.search(cr, uid, [('requisition_id', '=', ids)])
            for po in order_obj.browse(cr, uid, po_ids):
                partner_ids.append(po.partner_id.id)
            partner_ids = list(set(partner_ids))
            for partner_id in partner_ids:
                supplier_data = partner_obj.browse(cr, uid, partner_id, context=context)

                address_id = partner_obj.address_get(cr, uid, [partner_id], ['delivery'])['delivery']
                list_line=[]
                purchase_order_line={}
                for tender in tender_obj.browse(cr, uid, record_ids, context=context):
                    for line in tender.line_ids:
                        partner_list = sorted([(partner.sequence, partner) for partner in  line.product_id.seller_ids if partner])
                        partner_rec = partner_list and partner_list[0] and partner_list[0][1] or False
                        uom_id = line.product_uom_id.id
                        
                        if tender.date_start:
                            newdate = datetime.strptime(tender.date_start, '%Y-%m-%d %H:%M:%S') - relativedelta(days=company.po_lead)
                        else:
                            newdate = datetime.today() - relativedelta(days=company.po_lead)
                        delay = partner_rec and partner_rec.delay or 0.0
                        if delay:
                            newdate -= relativedelta(days=delay)

                        partner = partner_rec and partner_rec.name or supplier_data
                        pricelist_id = partner.property_product_pricelist_purchase and partner.property_product_pricelist_purchase.id or False
                        price = pricelist_obj.price_get(cr, uid, [pricelist_id], line.product_id.id, line.product_qty, False, {'uom': uom_id})[pricelist_id]
                        product = prod_obj.browse(cr, uid, line.product_id.id, context=context)
                        location_id = self.pool.get('stock.warehouse').read(cr, uid, [tender.warehouse_id.id], ['lot_input_id'])[0]['lot_input_id'][0]
                        
                        if line.product_id.product_tmpl_id.type == 'service':
                            invoice_method = 'order'
                        else:
                            invoice_method = 'picking'
                            
                        purchase_order_line= {
                                'name': product.partner_ref,
                                'product_qty': line.product_qty,
                                'product_id': line.product_id.id,
                                'product_uom': uom_id,
                                'price_unit': price,
                                'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                                #~ 'notes': product.description_purchase,
                                'notes': line.description,
                        }
                        
                        if product.product_tmpl_id.type == 'service':
                            purchase_order_line['name'] = line.description
                        
                        taxes_ids = line.product_id.product_tmpl_id.supplier_taxes_id
                        taxes = acc_pos_obj.map_tax(cr, uid, partner.property_account_position, taxes_ids)
                        purchase_order_line.update({
                                'taxes_id': [(6,0,taxes)]
                            })
                        list_line.append(purchase_order_line)
                        
                    
                    purchase_id = order_obj.create(cr, uid, {
                                'origin': tender.name,
                                'partner_id': partner_id,
                                'partner_address_id': address_id,
                                'pricelist_id': pricelist_id,
                                'location_id': tender.purchase_ids and tender.purchase_ids[0].location_id.id or line.product_id.product_tmpl_id.property_stock_production.id,
                                'company_id': tender.company_id.id,
                                'fiscal_position': partner.property_account_position and partner.property_account_position.id or False,
                                'requisition_id':tender.id,
                                'notes':tender.description,
                                'warehouse_id':tender.warehouse_id.id and tender.warehouse_id.id ,
                                'location_id':location_id,
                                'company_id':tender.company_id.id,
                                'invoice_method':invoice_method,
                    })
                    order_ids=[]
                    for order_line in list_line:
                        order_line.update({
                                'order_id': purchase_id
                            })
                        order_line_obj.create(cr,uid,order_line)
                        
        #~ return {'type': 'ir.actions.act_window_close'}
        return {
            'type': 'ir.actions.act_window_close',
            'active_id': ids
        }


purchase_requisition_merge()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
