#-*- coding:utf-8 -*-

import time
from datetime import date
from datetime import datetime
from datetime import timedelta
from dateutil import relativedelta

import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import decimal_precision as dp
import math
from tools.safe_eval import safe_eval as eval
import logging
import unicodedata


def elimina_tildes(s):
    """
    Funcion para eliminar las tildes de algun texto utilizando el modulo unicodedata.
    """
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))

class hr_contract(osv.osv):
    """ 
    Employee contract based on the visa, work permits
    allows to configure different Salary structure
    """

    _inherit = 'hr.contract'
    _description = 'Employee Contract'

        
    def send_email_notification(self, cr, uid, subject, message, correos, context=None):
        """
        Metodo que envia un correo electronico al grupo de emails especificados
        en correos
        """
        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),
                                        ('state','=','confirm')], context=False)
        if smtp_server:
            email_to = correos.split(',')
            if email_to:
                smtp_server_id = smtp_server[0]
                server_name = smtp_client.browse(cr, uid, smtp_server_id).server
                email = {
                        'server_id': smtp_server_id,
                        'name': (subject),
                        'server': server_name,
                        'body': (message),
                        'to': email_to,
                        'from': uid,
                        }
                smtp_client.send_email(cr, uid, smtp_server_id, email['to'], email['name'], email['body'])

    def send_request(self, cr, uid, subject, message, context=None):
        """
        Metodo que envia una notificacion interna de OpenERP a un usuario cuando
        ocurre algun evento de hr_process
        """
        notificaciones = {
                'name': (subject),
                'act_to': uid,
                'act_from': uid,
                'body': (message),
                'active': True,
                'priority': '2',
                'state': 'waiting'
            }
        request_obj = self.pool.get('res.request')
        request_obj.create(cr, uid, notificaciones, context)
        
    def end_contract(self, cr, uid, ids=None, context=None):
        ''' Runs through scheduler.
        Metodo usado para realizar verificar los contratos de empleados
        que culminaran en los proximos dias
        '''
        if context is None:
            context = {}
        
        parameter_obj = self.pool.get('hr.parameters')
        contract_obj = self.pool.get('hr.contract')
        process_obj = self.pool.get('hr.process')
        
        param_end_contract_id = parameter_obj.search(cr, uid, [('type', '=', 'notify_end_contract')])
        if not param_end_contract_id:
            return False

        correos = parameter_obj.browse(cr, uid, param_end_contract_id)[0].correos
        usuario = parameter_obj.browse(cr, uid, param_end_contract_id)[0].user_id
        dias_antelacion = parameter_obj.browse(cr, uid, param_end_contract_id)[0].days_end_contract
        date_review = datetime.now() + timedelta(days=dias_antelacion)
        
        contract_ids = contract_obj.search(cr, uid, [('date_end', '>', datetime.now()),('date_end', '<=', date_review)])
        contract_brw = contract_obj.browse(cr, uid, contract_ids)
        
        subject = u'Contratos por vencer'
        message_email = u'Empleados a los que se le vencerá el contrato en los próximos %s días: <br/>'%(dias_antelacion)
        message_request = u'Empleados a los que se le vencerá el contrato en los próximos %s días: \n'%(dias_antelacion)
        for contract in contract_brw:
            message_email = message_email + elimina_tildes(contract.employee_id.name) + ' ' + contract.employee_id.cod_number + '<br/>'
            message_request = message_request + elimina_tildes(contract.employee_id.name) + ' ' + contract.employee_id.cod_number + '\n'
        self.send_email_notification(cr, uid, subject, message_email, correos, context)
        self.send_request(cr, usuario.id, subject, message_request, context=None)
        
        return True
        
hr_contract()
