#-*- coding:utf-8 -*-

import time
from datetime import date
from datetime import datetime
from datetime import timedelta
from dateutil import relativedelta
import calendar
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import decimal_precision as dp
import math
from tools.safe_eval import safe_eval as eval
import logging

class contrib_register(osv.osv):
    '''
    Contribution Register
    '''

    _name = 'hr.contribution.register'
    _description = 'Contribution Register'

    _columns = {
        'company_id':fields.many2one('res.company', 'Company', required=False),
        'name':fields.char('Name', size=256, required=True, readonly=False),
        'register_line_ids':fields.one2many('hr.payslip.line', 'register_id', 'Register Line', readonly=True),
        'note': fields.text('Description'),
    }
    _defaults = {
        'company_id': lambda self, cr, uid, context: \
                self.pool.get('res.users').browse(cr, uid, uid,
                    context=context).company_id.id,
    }

contrib_register()

class hr_salary_rule_category(osv.osv):
    """
    HR Salary Rule Category
    """

    _name = 'hr.salary.rule.category'
    _description = 'Salary Rule Category'
    _columns = {
        'name':fields.char('Name', size=64, required=True, readonly=False),
        'code':fields.char('Code', size=64, required=True, readonly=False),
        'parent_id':fields.many2one('hr.salary.rule.category', 'Parent', help="Linking a salary category to its parent is used only for the reporting purpose."),
        'children_ids': fields.one2many('hr.salary.rule.category', 'parent_id', 'Children'),
        'note': fields.text('Description'),
        'company_id':fields.many2one('res.company', 'Company', required=False),
    }

    _defaults = {
        'company_id': lambda self, cr, uid, context: \
                self.pool.get('res.users').browse(cr, uid, uid,
                    context=context).company_id.id,
    }

hr_salary_rule_category()

class one2many_mod2(fields.one2many):

    def get(self, cr, obj, ids, name, user=None, offset=0, context=None, values=None):
        if context is None:
            context = {}
        if not values:
            values = {}
        res = {}
        for id in ids:
            res[id] = []
        ids2 = obj.pool.get(self._obj).search(cr, user, [(self._fields_id,'in',ids), ('appears_on_payslip', '=', True)], limit=self._limit)
        for r in obj.pool.get(self._obj)._read_flat(cr, user, ids2, [self._fields_id], context=context, load='_classic_write'):
            res[r[self._fields_id]].append( r['id'] )
        return res

class hr_payslip_run(osv.osv):

    _name = 'hr.payslip.run'
    _description = 'Payslip Batches'
    _columns = {
        'name': fields.char('Name', size=64, required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'slip_ids': fields.one2many('hr.payslip', 'payslip_run_id', 'Payslips', required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'state': fields.selection([
            ('draft', 'Draft'),
            ('close', 'Close'),
        ], 'State', select=True, readonly=True),
        'date_start': fields.date('Date From', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'date_end': fields.date('Date To', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'credit_note': fields.boolean('Credit Note', readonly=True, states={'draft': [('readonly', False)]}, help="If its checked, indicates that all payslips generated from here are refund payslips."),
    }
    _defaults = {
        'state': 'draft',
        'date_start': lambda *a: time.strftime('%Y-%m-01'),
        'date_end': lambda *a: str(datetime.now() + relativedelta.relativedelta(months=+1, day=1, days=-1))[:10],
    }

    def draft_payslip_run(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'draft'}, context=context)

    def close_payslip_run(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'close'}, context=context)

hr_payslip_run()

class hr_payroll_structure(osv.osv):
    """
    Salary structure used to defined
    - Basic
    - Allowances
    - Deductions
    """

    _name = 'hr.payroll.structure'
    _description = 'Estructura'
    _columns = {
        'name':fields.char('Name', size=256, required=True),
        'code':fields.char('Reference', size=64, required=True),
        'company_id':fields.many2one('res.company', 'Company', required=True),
        'note': fields.text('Description'),
        'parent_id':fields.many2one('hr.payroll.structure', 'Parent'),
        'children_ids':fields.one2many('hr.payroll.structure', 'parent_id', 'Children'),
        'hr_contract_ids':fields.one2many('hr.contract', 'payroll_structure_id', 'Contratos'),
    }

    def _get_parent(self, cr, uid, context=None):
        obj_model = self.pool.get('ir.model.data')
        res = False
        data_id = obj_model.search(cr, uid, [('model', '=', 'hr.payroll.structure'), ('name', '=', 'structure_base')])
        if data_id:
            res = obj_model.browse(cr, uid, data_id[0], context=context).res_id
        return res

    _defaults = {
        'company_id': lambda self, cr, uid, context: \
                self.pool.get('res.users').browse(cr, uid, uid,
                    context=context).company_id.id,
        'parent_id': _get_parent,
    }

    def copy(self, cr, uid, id, default=None, context=None):
        """
        Create a new record in hr_payroll_structure model from existing one
        @param cr: cursor to database
        @param user: id of current user
        @param id: list of record ids on which copy method executes
        @param default: dict type contains the values to be override during copy of object
        @param context: context arguments, like lang, time zone

        @return: returns a id of newly created record
        """
        if not default:
            default = {}
        default.update({
            'code': self.browse(cr, uid, id, context=context).code + "(copy)",
            'company_id': self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id.id
        })
        return super(hr_payroll_structure, self).copy(cr, uid, id, default, context=context)

    def get_all_rules(self, cr, uid, structure_ids, employee, period_id, context=None):
        """
        @param structure_ids: list of structure
        @return: returns a list of tuple (id, sequence) of rules that are maybe to apply
        """

        all_rules = []
        for struct in self.browse(cr, uid, structure_ids, context=context):
            for concept in struct.concept_ids:
                if concept.type == 'fijo' or concept.type == 'aporte_patronal':
                    period_brw = self.pool.get('hr.period').browse(cr, uid, period_id)
                    date_end = concept.date_end >= period_brw.date_start if concept.date_end else True
                    if concept.date_start <= period_brw.date_end and date_end:
                        salary_rule_ids = []
                        for salary_rule in concept.salary_rule_ids:
                            #Si la regla aplica para una nomina en especial
                            if salary_rule.hr_payroll_ids and employee.hr_payroll_id in salary_rule.hr_payroll_ids:
                                salary_rule_ids.append(salary_rule)
                            #Si la regla no tiene hr_payroll_ids aplica para todas las nominas
                            elif not salary_rule.hr_payroll_ids:
                                salary_rule_ids.append(salary_rule)
                        all_rules += self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, salary_rule_ids, context=context)
        return all_rules

    def _get_parent_structure(self, cr, uid, struct_ids, context=None):
        if not struct_ids:
            return []
        parent = []
        for struct in self.browse(cr, uid, struct_ids, context=context):
            if struct.parent_id:
                parent.append(struct.parent_id.id)
        if parent:
            parent = self._get_parent_structure(cr, uid, parent, context)
        return parent + struct_ids

hr_payroll_structure()

class hr_contract(osv.osv):
    """
    Employee contract based on the visa, work permits
    allows to configure different Salary structure
    """
        
    _inherit = 'hr.contract'
    _description = 'Employee Contract'
        
    _columns = {
        'payroll_structure_id': fields.many2one('hr.payroll.structure', 'Nomina Estructura'),
        'schedule_pay': fields.selection([
            ('monthly', 'Monthly'),
            ('quarterly', 'Quarterly'),
            ('semi-annually', 'Semi-annually'),
            ('annually', 'Annually'),
            ('weekly', 'Weekly'),
            ('bi-weekly', 'Bi-weekly'),
            ('bi-monthly', 'Bi-monthly'),
            ], 'Scheduled Pay', select=True),
        'wage_type_id': fields.many2one('hr.contract.wage.type', 'Wage Type'),
        'wage': fields.float('Wage', digits=(16,2)),
        'type_id': fields.many2one('hr.contract.type', "Contract Type"),
        'termination_reason': fields.selection([('renuncia','Renuncia'), ('despido_justificado','Despido justificado'), 
                            ('despido_injustificado','Despido injustificado'),('terminacion_contrato','Terminación de contrato'),
                            ('abandono_trabajo','Abandono del trabajo')],
                                'Motivo de terminación', size=64),
    }

    _defaults = {
        'schedule_pay': 'monthly',
    }

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        employee_brw = self.pool.get('hr.employee').browse(cr, uid, vals['employee_id'], context)
        contract_brw = employee_brw.contract_ids
        for contract in contract_brw:
            if not contract.date_end:
                raise osv.except_osv(_('Acción Denegada!'), _('El trabajador tiene un contrato vigente'))
            if vals['date_start'] <= contract.date_end:
                raise osv.except_osv(_('Acción Denegada!'), _('La fecha de inicio del nuevo contrato debe ser mayor \
                                                        a la fecha fin del contrato anterior'))
        if employee_brw.state != 'activo':
            self.pool.get('hr.employee').write(cr, uid, vals['employee_id'], {'state':'borrador'}, context)
        return super(hr_contract, self).create(cr, uid, vals, context)
    
    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = [ids]
        for id in ids:
            contract_brw = self.browse(cr, uid, id, context)

            date_end = vals['date_end'] if 'date_end' in vals else contract_brw.date_end
            if ('termination_reason' in vals and vals['termination_reason'] and not date_end):
                raise osv.except_osv(_('Acción Denegada!'), _('Debe ingresar Motivo de terminación y Fecha de finalización del contrato'))
            if 'termination_reason' in vals and vals['termination_reason']:
                if contract_brw.employee_id.state <> 'activo':
                    raise osv.except_osv(_('Acción Denegada!'), _('El empleado debe estar activo para poder dar finalización al contrato'))
                
                payslip_review = self.pool.get('hr.payslip').search(cr, uid, [('employee_id','=',contract_brw.employee_id.id),
                            ('hr_process_id.state','<>','done')])
                if payslip_review:
                    raise osv.except_osv(_('ERROR!'),_('El empleado tiene una hoja de pago en un proceso por aprobar, se debe verificar!'))
                    
                self.pool.get('hr.employee').write(cr, uid, contract_brw.employee_id.id, {'state':'egresado'}, context)
        return super(hr_contract, self).write(cr, uid, ids, vals, context)
        
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = [ids]
        for id in ids:
            contract_brw = self.browse(cr, uid, id, context) 
            if not contract_brw.date_end:
                self.pool.get('hr.employee').write(cr, uid, contract_brw.employee_id.id, {'state':'borrador'}, context)
        return super(hr_contract, self).unlink(cr, uid, ids, context)
            
    def get_all_structures(self, cr, uid, contract_ids, context=None):
        """
        @param contract_ids: list of contracts
        @return: the structures linked to the given contracts, ordered by hierachy (parent=False first, then first level children and so on) and without duplicata
        """
        all_structures = []
        structure_ids = [contract.payroll_structure_id.id for contract in self.browse(cr, uid, contract_ids, context=context)]
        return list(set(self.pool.get('hr.payroll.structure')._get_parent_structure(cr, uid, structure_ids, context=context)))

hr_contract()

class hr_payslip(osv.osv):
    '''
    Pay Slip
    '''

    _name = 'hr.payslip'
    _description = 'Pay Slip'
    _logger = logging.getLogger('hr.payslip')


    def _get_lines_salary_rule_category(self, cr, uid, ids, field_names, arg=None, context=None):
        result = {}
        if not ids: return result
        for id in ids:
            result.setdefault(id, [])
        cr.execute('''SELECT pl.slip_id, pl.id FROM hr_payslip_line AS pl \
                    LEFT JOIN hr_salary_rule_category AS sh on (pl.category_id = sh.id) \
                    WHERE pl.slip_id in %s \
                    GROUP BY pl.slip_id, pl.sequence, pl.id ORDER BY pl.sequence''',(tuple(ids),))
        res = cr.fetchall()
        for r in res:
            result[r[0]].append(r[1])
        return result

    _columns = {
        'payroll_structure_id': fields.many2one('hr.payroll.structure', 'Estructura', readonly=True, states={'draft': [('readonly', False)]}, help='Defines the rules that have to be applied to this payslip, accordingly to the contract chosen. If you let empty the field contract, this field isn\'t mandatory anymore and thus the rules applied will be all the rules set on the structure of all contracts of the employee valid for the chosen period'),
        'name': fields.char('Description', size=64, required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'number': fields.char('Reference', size=64, required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'employee_id': fields.many2one('hr.employee', 'Employee', required=True, readonly=True, states={'draft': [('readonly', False)]}),
        'date_from': fields.date('Date From', readonly=True, states={'draft': [('readonly', False)]}, required=True),
        'date_to': fields.date('Date To', readonly=True, states={'draft': [('readonly', False)]}, required=True),
        'state': fields.selection([
            ('draft', 'Draft'),
            ('review','Review'),
            ('except','Exception'),
            ('done', 'Done'),
            ('cancel', 'Rejected'),
        ], 'State', select=True, readonly=True,
            help='* When the payslip is created the state is \'Draft\'.\
            \n* If the payslip is under verification, the state is \'Review\'. \
            \n* If the payslip is in exception, the state is \'Exception\'. \
            \n* If the payslip is confirmed then state is set to \'Done\'.\
            \n* When user cancel payslip the state is \'Rejected\'.'),
        'line_ids': fields.one2many('hr.payslip.line', 'slip_id', 'Payslip Line', required=False, readonly=True, states={'draft': [('readonly', False)]}, domain=[('appears_on_payslip', '=', True)]),
        'company_id': fields.many2one('res.company', 'Company', required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'worked_days_line_ids': fields.one2many('hr.payslip.worked_days', 'payslip_id', 'Payslip Worked Days', required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'input_line_ids': fields.one2many('hr.payslip.input', 'payslip_id', 'Payslip Inputs', required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'paid': fields.boolean('Made Payment Order ? ', required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'note': fields.text('Description', readonly=True, states={'draft':[('readonly',False)]}),
        'contract_id': fields.many2one('hr.contract', 'Contract', required=False, readonly=True, states={'draft': [('readonly', False)]}),
        'details_by_salary_rule_category': fields.function(_get_lines_salary_rule_category, method=True, type='one2many', relation='hr.payslip.line', string='Details by Salary Rule Category'),
        'credit_note': fields.boolean('Credit Note', help="Indicates this payslip has a refund of another", readonly=True, states={'draft': [('readonly', False)]}),
        'payslip_run_id': fields.many2one('hr.payslip.run', 'Payslip Batches', readonly=True, states={'draft': [('readonly', False)]}),
    }
    _defaults = {
        'state': 'draft',
        'credit_note': False,
        'company_id': lambda self, cr, uid, context: \
                self.pool.get('res.users').browse(cr, uid, uid,
                    context=context).company_id.id,
    }

    def _check_dates(self, cr, uid, ids, context=None):
        for payslip in self.browse(cr, uid, ids, context=context):
            if payslip.date_from > payslip.date_to:
                return False
        return True

    _constraints = [(_check_dates, "Payslip 'Date From' must be before 'Date To'.", ['date_from', 'date_to'])]

    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        company_id = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id.id
        default.update({
            'line_ids': [],
            'move_ids': [],
            'move_line_ids': [],
            'company_id': company_id,
            'period_id': False,
            'basic_before_leaves': 0.0,
            'basic_amount': 0.0
        })
        return super(hr_payslip, self).copy(cr, uid, id, default, context=context)

    def cancel_sheet(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'cancel'}, context=context)

    def process_sheet(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'paid': True, 'state': 'done'}, context=context)

    def hr_verify_sheet(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'review'}, context=context)

    def refund_sheet(self, cr, uid, ids, context=None):
        mod_obj = self.pool.get('ir.model.data')
        wf_service = netsvc.LocalService("workflow")
        for payslip in self.browse(cr, uid, ids, context=context):
            id_copy = self.copy(cr, uid, payslip.id, {'credit_note': True, 'name': _('Refund: ')+payslip.name}, context=context)
            self.compute_sheet(cr, uid, [id_copy], context=context) #Calculo de las reglas asociadas al payslip
            wf_service.trg_validate(uid, 'hr.payslip', id_copy, 'hr_verify_sheet', cr)
            wf_service.trg_validate(uid, 'hr.payslip', id_copy, 'process_sheet', cr)

        form_id = mod_obj.get_object_reference(cr, uid, 'sisb_nomina', 'view_hr_payslip_form')
        form_res = form_id and form_id[1] or False
        tree_id = mod_obj.get_object_reference(cr, uid, 'sisb_nomina', 'view_hr_payslip_tree')
        tree_res = tree_id and tree_id[1] or False
        return {
            'name':_("Refund Payslip"),
            'view_mode': 'tree, form',
            'view_id': False,
            'view_type': 'form',
            'res_model': 'hr.payslip',
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'current',
            'domain': "[('id', 'in', %s)]" % [id_copy],
            'views': [(tree_res, 'tree'), (form_res, 'form')],
            'context': {}
        }

    def check_done(self, cr, uid, ids, context=None):
        return True

    #TODO move this function into hr_contract module, on hr.employee object
    def get_contract(self, cr, uid, employee, date_from, date_to, context=None):
        """
        @param employee: browse record of employee
        @param date_from: date field
        @param date_to: date field
        @return: returns the ids of all the contracts for the given employee that need to be considered for the given dates
        """
        contract_obj = self.pool.get('hr.contract')
        clause = []
        #a contract is valid if it ends between the given dates
        clause_1 = ['&',('date_end', '<=', date_to),('date_end','>=', date_from)]
        #OR if it starts between the given dates
        clause_2 = ['&',('date_start', '<=', date_to),('date_start','>=', date_from)]
        #OR if it starts before the date_from and finish after the date_end (or never finish)
        clause_3 = [('date_start','<=', date_from),'|',('date_end', '=', False),('date_end','>=', date_to)]
        clause_final =  [('employee_id', '=', employee.id),'|','|'] + clause_1 + clause_2 + clause_3
        contract_ids = contract_obj.search(cr, uid, clause_final, context=context)
        return contract_ids

    def _check_unique_codes(self, cr, uid, ids, context=None):
        self._logger.info('Chequeo lineas repetidas')
        self.write(cr, uid, ids, {'state': 'review'})
        process = self.browse(cr, uid, ids[0], context).hr_process_id
        if process.type <> 'guarderias': 
            cr.execute("select hpl.slip_id from hr_payslip_line hpl \
                        join hr_concept hc on hpl.concept_id = hc.id \
                        where hpl.process_id in (select id from hr_process where id = %s)\
                        and hc.id not in (select concept_id from hr_concept_loan)\
                        group by hpl.concept_id, hpl.employee_id, hpl.slip_id \
                        having count(*) > 1 \
                        order by hpl.slip_id"%(process.id))
            ids_except = cr.fetchall()
            ids_except =[x[0] for x in ids_except]
            if ids_except:
                self.write(cr, uid, ids_except, {'state': 'except'})
        return True
        
    #Calculo de las reglas asociadas al payslip
    def compute_sheet(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        slip_line_pool = self.pool.get('hr.payslip.line')

        process = self.browse(cr, uid, ids[0], context=context).hr_process_id #Usado para saber que tipo de proceso se esta ejecutando
        context['procesar'] = True #Para que en la sobreescritura del unlink permita eliminar la linea para recalcularla

        for payslip in self.browse(cr, uid, ids, context=context):
            number = payslip.number or ''
            #delete old payslip lines
            old_slipline_ids = slip_line_pool.search(cr, uid, [('slip_id', '=', payslip.id),('variable', '=', False)], context=context)
            if old_slipline_ids:
                slip_line_pool.unlink(cr, uid, old_slipline_ids, context=context)
            if payslip.contract_id:
                #set the list of contract for which the rules have to be applied
                contract_ids = [payslip.contract_id.id]
            else:
                #if we don't give the contract, then the rules to apply should be for all current contracts of the employee
                contract_ids = self.get_contract(cr, uid, payslip.employee_id, payslip.date_from, payslip.date_to, context=context)

            try:
                get_lines = getattr(self, 'get_payslip_lines_' + process.type)
                lines = get_lines(cr, uid, contract_ids, payslip.id, context=context)
            except Exception, e:
                #Se envia la excepcion a hr_process
                self.write(cr, uid, payslip.id, {'state': 'except'})
                raise osv.except_osv(_('ERROR!'), e.value)

            for line in lines:
                if line['amount'] > 0.0:
                    save_on_database = False
                    if 'salary_rule_id' in line:
                        rule = self.pool.get('hr.salary.rule').browse(cr, uid, line['salary_rule_id'])
                        save_on_database = rule.save_on_database
                    else: #Es un attribute
                        attribute = self.pool.get('hr.attribute').browse(cr, uid, line['attribute_id'])
                        save_on_database = attribute.save_on_database
                    if save_on_database: #Se debe guardar en BD
                        line_ids = slip_line_pool.create(cr, uid, line, context)
            
            self._logger.info('Se ha calculado: %s', payslip.name)
        return True

    def calcular_payslips(self, cr, uid, ids, context=None):
        self.compute_sheet(cr, uid, ids, context)
        self._check_unique_codes(cr, uid, ids, context=None)
        return True
        
    def get_worked_day_lines(self, cr, uid, contract_ids, date_from, date_to, context=None):
        """
        @param contract_ids: list of contract id
        @return: returns a list of dict containing the input that should be applied for the given contract between date_from and date_to
        """
        def was_on_leave(employee_id, datetime_day, context=None):
            res = False
            day = datetime_day.strftime("%Y-%m-%d")
            holiday_ids = self.pool.get('hr.holidays').search(cr, uid, [('state','=','validate'),('employee_id','=',employee_id),('type','=','remove'),('date_from','<=',day),('date_to','>=',day)])
            if holiday_ids:
                res = self.pool.get('hr.holidays').browse(cr, uid, holiday_ids, context=context)[0].holiday_status_id.name
            return res

        res = []
        for contract in self.pool.get('hr.contract').browse(cr, uid, contract_ids, context=context):
            if not contract.working_hours:
                #fill only if the contract as a working schedule linked
                continue
            attendances = {
                 'name': _("Normal Working Days paid at 100%"),
                 'sequence': 1,
                 'code': 'WORK100',
                 'number_of_days': 0.0,
                 'number_of_hours': 0.0,
                 'contract_id': contract.id,
            }
            leaves = {}
            day_from = datetime.strptime(date_from,"%Y-%m-%d")
            day_to = datetime.strptime(date_to,"%Y-%m-%d")
            nb_of_days = (day_to - day_from).days + 1
            for day in range(0, nb_of_days):
                working_hours_on_day = self.pool.get('resource.calendar').working_hours_on_day(cr, uid, contract.working_hours, day_from + timedelta(days=day), context)
                if working_hours_on_day:
                    #the employee had to work
                    leave_type = was_on_leave(contract.employee_id.id, day_from + timedelta(days=day), context=context)
                    if leave_type:
                        #if he was on leave, fill the leaves dict
                        if leave_type in leaves:
                            leaves[leave_type]['number_of_days'] += 1.0
                            leaves[leave_type]['number_of_hours'] += working_hours_on_day
                        else:
                            leaves[leave_type] = {
                                'name': leave_type,
                                'sequence': 5,
                                'code': leave_type,
                                'number_of_days': 1.0,
                                'number_of_hours': working_hours_on_day,
                                'contract_id': contract.id,
                            }
                    else:
                        #add the input vals to tmp (increment if existing)
                        attendances['number_of_days'] += 1.0
                        attendances['number_of_hours'] += working_hours_on_day
            leaves = [value for key,value in leaves.items()]
            res += [attendances] + leaves
        return res

    def get_inputs(self, cr, uid, contract_ids, date_from, date_to, context=None):
        res = []
        contract_obj = self.pool.get('hr.contract')
        rule_obj = self.pool.get('hr.salary.rule')

        structure_ids = contract_obj.get_all_structures(cr, uid, contract_ids, context=context)
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, context=context)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

        for contract in contract_obj.browse(cr, uid, contract_ids, context=context):
            for rule in rule_obj.browse(cr, uid, sorted_rule_ids, context=context):
                if rule.input_ids:
                    for input in rule.input_ids:
                        inputs = {
                             'name': input.name,
                             'code': input.code,
                             'contract_id': contract.id,
                        }
                        res += [inputs]
        return res

    def habiles(self, fecha_inicio, fecha_fin=None, days=None, count=False, holidays=[]):
        """
        @param fecha_inicio: datetime.date
        @param fecha_fin: datetime.date
        @param days: integer
        @param holidays: list(datetime.date) contiene feriados
        @return: lista de dias habiles contenidos en un periodo de tiempo,
        o en n cantidad de dias desde la fecha de inicio. En caso de recibir
        count True devuelve la cantidad de dias.
        """
        res = []
        diff = 0
        if fecha_fin:
            diff = (fecha_fin - fecha_inicio).days + 1
        elif days:
            diff = days
        while diff > 0:
            if fecha_fin:
                diff = diff - 1
            if fecha_inicio.weekday() not in [5,6] and datetime.strftime(fecha_inicio,"%Y-%m-%d") not in holidays:
                res.append(fecha_inicio)
                if not fecha_fin and days:
                    diff = diff - 1
            fecha_inicio = fecha_inicio + timedelta(days=1)
        return res if not count else len(res)
        
    def get_payslip_lines(self, cr, uid, contract_ids, payslip_id, context):
        def _sum_salary_rule_category(localdict, category, amount):
            if category.parent_id:
                localdict = _sum_salary_rule_category(localdict, category.parent_id, amount)
                localdict['categories'].dict[category.code] = category.code in localdict['categories'].dict and localdict['categories'].dict[category.code] + amount or amount
            return localdict

        class BrowsableObject(object):
            def __init__(self, pool, cr, uid, employee_id, dict):
                self.pool = pool
                self.cr = cr
                self.uid = uid
                self.employee_id = employee_id
                self.dict = dict

            def __getattr__(self, attr):
                return attr in self.dict and self.dict.__getitem__(attr) or 0.0

        class InputLine(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(amount) as sum\
                            FROM hr_payslip as hp, hr_payslip_input as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()[0]
                return res or 0.0

        class WorkedDays(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def _sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(number_of_days) as number_of_days, sum(number_of_hours) as number_of_hours\
                            FROM hr_payslip as hp, hr_payslip_worked_days as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done'\
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                return self.cr.fetchone()

            def sum(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[0] or 0.0

            def sum_hours(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[1] or 0.0

        class Payslips(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""

            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                self.cr.execute("SELECT sum(case when hp.credit_note = False then (pl.total) else (-pl.total) end)\
                            FROM hr_payslip as hp, hr_payslip_line as pl \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pl.slip_id AND pl.code = %s",
                            (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()
                return res and res[0] or 0.0

        #we keep a dict with the result because a value can be overwritten by another rule with the same code
        result_dict = {}
        rules = {}
        categories_dict = {}
        categories_concept_dict = {}
        parameters_dict = {}
        payroll_dict = {}
        period_dict = {}
        blacklist = []
        payslip_obj = self.pool.get('hr.payslip')
        inputs_obj = self.pool.get('hr.payslip.worked_days')
        obj_rule = self.pool.get('hr.salary.rule')
        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)
        worked_days = {}
        for worked_days_line in payslip.worked_days_line_ids:
            worked_days[worked_days_line.code] = worked_days_line
        inputs = {}
        for input_line in payslip.input_line_ids:
            inputs[input_line.code] = input_line

        categories_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, categories_dict)
        input_obj = InputLine(self.pool, cr, uid, payslip.employee_id.id, inputs)
        worked_days_obj = WorkedDays(self.pool, cr, uid, payslip.employee_id.id, worked_days)
        rules_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, rules)
        get_method = getattr(self.pool.get('hr.employee'), '_get')
        get_concept = getattr(self.pool.get('hr.concept'), 'get_cal_concept')

        payroll_dict[payslip.employee_id.hr_payroll_id.code] = True or 0.0
        employee = payslip.employee_id
        period = list(set(payslip.hr_process_id.hr_period_ids) & set(employee.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
        feriados = self.pool.get('hr.holidays.calendar').search(cr, uid, 
                                [('date','>=',datetime(datetime.strptime(period.date_start,"%Y-%m-%d").year,01,01)),
                                ('date','<=',datetime(datetime.strptime(period.date_end,"%Y-%m-%d").year,12,31))])
        feriados = [f.date for f in self.pool.get('hr.holidays.calendar').browse(cr, uid, feriados)]

        localdict = {
            'categories': categories_obj,
            'rules': rules_obj,
            'payslip': payslip,
            'worked_days': worked_days_obj,
            'inputs': input_obj,
            'atributo': get_method,
            'historial': get_concept,
            'param': [cr, uid],
            'concepto': {},
            'cantidad_horas': {},
            'categoria_concepto': {},
            'nomina': payroll_dict,
            'datetime': datetime,
            'feriados': feriados,
        }

        edad = (datetime.now() - datetime.strptime(employee.birthday,"%Y-%m-%d")).days/365
        localdict.update({'empleado': employee,
                'sexo': 'Femenino' if employee.gender == 'female' else 'Masculino',
                'edad': edad,
                'jornada_dia': employee.shift_id.jor_day,
                'jornada_semana': employee.shift_id.jor_week,})

        rules_dict = {}
        hours_dict = {}

        # Busqueda del parametro que indica la cantidad de dias para perder descanso. Si el parametro no existe
        # al empleado se le calculara el descanso para que luego verifiquen -----------------------------
        parameter_obj = self.pool.get('hr.parameters')
        param_perder_descanso_id = parameter_obj.search(cr, uid, [('type', '=', 'perdida_descanso'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_perder_descanso_id:
            param_perder_descanso_id = parameter_obj.search(cr, uid, [('type', '=', 'perdida_descanso'),('payroll_id', '=', None)])

        if param_perder_descanso_id:
            pierde_descanso = parameter_obj.browse(cr, uid, param_perder_descanso_id)[0].value
            pierde_descanso = pierde_descanso*employee.shift_id.jor_day #Para llevarlo a horas
            localdict['pierde_descanso'] = pierde_descanso
        else:
            localdict['pierde_descanso'] = 1000 #Si no tiene lo calcula igual para que luego verifiquen

        #Incluir en el localdict categories los montos de los conceptos variables
        for line in payslip.line_ids:
            rules_dict[line.code] = line.total
            localdict['concepto'] = rules_dict

            ''' Para el concepto de Descanso se debe tener la cuanta de las horas en la semana que el emplaedo falto,
                para ello se agrega un diccionario en el cual se lleva la cuenta de la cantidad en horas por concepto.
                Luego en la condicion del concepto Descanso se pregunta si esta cantidad de horas es mayor
                a la cantidad que se suministra en los parametro para la perdida del descanso
            '''
            if line.salary_rule_id.concept_id.type_amount and line.salary_rule_id.concept_id.type_amount == 'hours':
                hours = line.quantity
            else:
                hours = line.quantity*employee.shift_id.jor_day
            hours_dict[line.code] = hours
            localdict['cantidad_horas'] = hours_dict

            if line.salary_rule_id:
                if line.salary_rule_id.concept_id.category_id.code not in categories_concept_dict:
                    categories_concept_dict[line.salary_rule_id.concept_id.category_id.code] = line.total or 0.0
                else:
                    categories_concept_dict[line.salary_rule_id.concept_id.category_id.code] = categories_concept_dict[line.salary_rule_id.concept_id.category_id.code] + line.total
            localdict['categories_concept'] = categories_concept_dict
            if line.attribute_id:
                if line.attribute_id.concept_id.category_id.code not in categories_concept_dict:
                    categories_concept_dict[line.attribute_id.concept_id.category_id.code] = line.total or 0.0
                else:
                    categories_concept_dict[line.attribute_id.concept_id.category_id.code] = categories_concept_dict[line.attribute_id.concept_id.category_id.code] + line.total
            localdict['categoria_concepto'] = categories_concept_dict

        structure_ids = []
        structure_brw = [j for j in employee.hr_payroll_id.hr_payroll_structure_ids]
        for str_brw in structure_brw: #El tipo de la estructura debe ser del mismo tipo del proceso
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)
        if not structure_ids:
            raise osv.except_osv(_('ERROR!'), _('No hay una estructura de tipo %s definida para la nomina %s')
                            %(payslip.hr_process_id.type, employee.hr_payroll_id.name))

        #Agregar parametros de la nomina
        for parameter in employee.hr_payroll_id.parameters_ids:
            parameters_dict[parameter.type] = parameter.value or 0.0
        localdict.update({'parametros': parameters_dict,})

        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina-----------------------------
        param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo Concepto Salario')
                                                %(employee.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id
        
        #Busqueda del concepto que se definio como Concepto para registro de fecha para calculos de prestaciones-----------------------------
        param_fp_id = parameter_obj.search(cr, uid, [('type', '=', 'fecha_prest')])
        concept_fp_id = 0
        if param_fp_id:
            concept_fp_id = parameter_obj.browse(cr, uid, param_fp_id)[0].concept_prest_id
            
        #Calculo de los lunes en el periodo actual (para seguro social obligatorio) y regimen prestacional
        if not employee.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'), _('La nomina %s no tiene calendario de pago. Debe asignarle un calendario de pago \
                            antes de ejecutar cualquier proceso')
                            %(employee.hr_payroll_id.name))

        mondays_in_period = 1
        if period.type == 'mensual':
            mondays_in_period = period.hr_month_id.monday_first_fortnight if period.period_in_month == 1 else period.hr_month_id.monday_second_fortnight
        localdict.update({'lunes': mondays_in_period})
        
        #Numero de periodos en el mes anterior
        periodos_mes_ant = 0
        if period.hr_month_id.month_year == 1:
            year_ant = self.pool.get('hr.year').search(cr, uid,[('year','=',period.hr_month_id.hr_year_id.year-1)])
            if year_ant:
                if isinstance(year_ant,list):
                    year_ant = year_ant[0]
                month_ant = self.pool.get('hr.month').search(cr, uid, [('month_year','=',12),('hr_year_id','=',year_ant)])
        else:
            month_ant = self.pool.get('hr.month').search(cr, uid, [('month_year','=',period.hr_month_id.month_year-1),
                                            ('hr_year_id','=',period.hr_month_id.hr_year_id.id)])
        
        if month_ant:
            if isinstance(month_ant,list):
                month_ant = month_ant[0]
            periodos_mes_ant_ids = self.pool.get('hr.period').search(cr, uid, [('hr_month_id','=',month_ant),('type','=',period.type)])
            periodos_mes_ant = len(periodos_mes_ant_ids)
        
        localdict['periodos_mes_ant'] = periodos_mes_ant
        localdict['ultimo_periodo'] = period.last_period_month
        localdict['periodo'] = period
        #Verificar con cada empleado si esta de aniversario
        if not employee.contract_id:
            raise osv.except_osv(_('ERROR!'), _('El empleado %s no tiene un contrato registrado')
                            %(employee.name))
        period_end_year = datetime.strptime(period.date_end,"%Y-%m-%d").year
            
        contract_year = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").year
        contract_month = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").month
        contract_day = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").day
                        
        if contract_year < period_end_year:
            nueva_f = datetime(period_end_year, contract_month, contract_day)
            localdict['aniversario'] = True if (nueva_f >= datetime.strptime(period.date_start,"%Y-%m-%d") and
                        nueva_f <= datetime.strptime(period.date_end,"%Y-%m-%d")) else False
        else:
            localdict['aniversario'] = False
        
        #Fecha de ingreso segun fecha de prestaciones
        attr_date_prest = list(set(concept_fp_id.attribute_ids) & set(payslip.employee_id.attribute_ids)) if concept_fp_id else False
        date_prest = datetime.strptime(attr_date_prest[0].date_value,"%Y-%m-%d") if attr_date_prest \
                        else datetime.strptime(payslip.employee_id.start_rel_lab,"%Y-%m-%d")
                        
        if date_prest.year < period_end_year:
            nueva_f = datetime(period_end_year, date_prest.month, date_prest.day)
            
            period_start = datetime.strptime(period.date_start,"%Y-%m-%d")
            period_end = datetime.strptime(period.date_end,"%Y-%m-%d")
            
            if period.period_in_month == 1:
                period_start = datetime(period_start.year, period_start.month, 01)
            elif period.last_period_month:
                last_day = calendar.monthrange(period_start.year,period_start.month)[1]
                period_end = datetime(period_start.year, period_start.month, last_day)
                
            localdict['aniversario_prest'] = True if (nueva_f >= period_start and nueva_f <= period_end) else False
        else:
            localdict['aniversario_prest'] = False
                            
        #get the rules of the structure and thier children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)
        #get the attributes of the structure
        attribute_ids = self.pool.get('hr.payroll.structure').get_all_attributes(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)
        
        #run the rules by sequence
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

        #Dias trabajados por el empleado en el periodo a pagar
        contract_old_1 = contract_old_2 = 0
        contract_old_1 = self.pool.get('hr.contract').search(cr, uid, [('employee_id','=',employee.id),('date_start','<=',period.date_start),
                                                                ('date_end','<=',employee.contract_id.date_start)], order='date_start desc')
        if not contract_old_1:
            contract_old_2 = self.pool.get('hr.contract').search(cr, uid, [('employee_id','=',employee.id),('date_start','>=',period.date_start),
                                                                ('date_end','<=',employee.contract_id.date_start)], order='date_start desc')
        
        if contract_old_1 and isinstance(contract_old_1, list):
            contract_old_1 = contract_old_1[0]
        if contract_old_2 and isinstance(contract_old_2, list):
            contract_old_2 = contract_old_2[0]
            
        contract_date_end = period.date_end <= employee.contract_id.date_end if employee.contract_id.date_end else True
        
        if (period.date_start >= employee.contract_id.date_start and contract_date_end) or contract_old_1:
            if period.type == 'diario':
                dias_trabajados = 5
            else:
                dias_trabajados = 15
            #Se debe verificar si el empleado acaba de regresar de vacaciones para calcular en numero de dias que se le cancelaran
            if employee.hr_payroll_id.out_in_holidays:
                hol_employee_id = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id', '=', employee.id),
                                        ('date_end', '>', period.date_start),('date_end', '<', period.date_end),
                                        ('state', '=', 'done')])
                if hol_employee_id:
                    hol_employee_brw = self.pool.get('hr.employee.holidays').browse(cr, uid, hol_employee_id[0])
                    dias = datetime.strptime(period.date_end,"%Y-%m-%d") - datetime.strptime(hol_employee_brw.date_end,"%Y-%m-%d")
                    dias_trabajados = dias.days
        else:
            dias_contrato_ant = 0
            if contract_old_2:
                contract_old_brw_2 = self.pool.get('hr.contract').browse(cr, uid, contract_old_2)
                dias_contrato_ant = datetime.strptime(contract_old_brw_2.date_end,"%Y-%m-%d") - datetime.strptime(contract_old_brw_2.date_start,"%Y-%m-%d")
                dias_contrato_ant = dias_contrato_ant.days
            if period.date_start < employee.contract_id.date_start:
                dias = datetime.strptime(period.date_end,"%Y-%m-%d") - datetime.strptime(employee.contract_id.date_start,"%Y-%m-%d")
                dias_trabajados = dias.days + 1 + dias_contrato_ant
            elif employee.contract_id.date_end:
                dias = datetime.strptime(employee.contract_id.date_end,"%Y-%m-%d") - datetime.strptime(period.date_start,"%Y-%m-%d")
                dias_trabajados = dias.days + 1 + dias_contrato_ant

        if not employee.attribute_ids:
            raise osv.except_osv(_('ERROR!'), _('El empleado %s no tiene atributos')%(employee.name))
                            
        for attribute in employee.attribute_ids:
            if attribute.concept_id.id == concept_salary_brw.id:
                rules_dict['SUELDO'] = attribute.amount
                localdict['concepto'] = rules_dict
        
        for attribute in attribute_ids:
            key = attribute.code
            if not key in result_dict:
                result_dict[key] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': employee.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip_id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = attribute.amount or 0.0
                localdict['concepto'] = rules_dict
            else:
                result_dict[key+str(attribute.id)] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': employee.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip_id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = rules_dict[attribute.code] + (attribute.amount or 0.0)
                localdict['concepto'] = rules_dict
            
            if attribute.concept_id.category_id.code not in categories_concept_dict:
                categories_concept_dict[attribute.concept_id.category_id.code] = attribute.amount or 0.0
            else:
                categories_concept_dict[attribute.concept_id.category_id.code] = categories_concept_dict[attribute.concept_id.category_id.code] + attribute.amount
                localdict['categoria_concepto'] = categories_concept_dict

        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
            key = rule.code
            localdict['result'] = None
            localdict['result_qty'] = 1.0
            #check if the rule can be applied
            if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                #compute the amount of the rule
                amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)

                #check if there is already a rule computed with that code
                previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                #set/overwrite the amount computed for this rule in the localdict
                tot_rule = amount * qty * rate / 100.0
                localdict[rule.code] = tot_rule
                rules[rule.code] = rule
                #sum the amount for its salary category
                localdict = _sum_salary_rule_category(localdict, rule.category_id, tot_rule - previous_amount)
                #create/overwrite the rule in the temporary results

                if rule.code == '0100':
                    qty = dias_trabajados
                
                if period.type == 'diario' and rule.code == '1000':
                    qty = 2

                result_dict[key] = {
                    'salary_rule_id': rule.id,
                    'name': rule.name,
                    'code': key,
                    'category_id': rule.category_id.id,
                    'sequence': rule.sequence,
                    'appears_on_payslip': rule.appears_on_payslip,
                    'condition_select': rule.condition_select,
                    'condition_python': rule.condition_python,
                    'condition_range': rule.condition_range,
                    'condition_range_min': rule.condition_range_min,
                    'condition_range_max': rule.condition_range_max,
                    'amount_select': rule.amount_select,
                    'amount_fix': rule.amount_fix,
                    'amount_python_compute': rule.amount_python_compute,
                    'amount_percentage': rule.amount_percentage,
                    'amount_percentage_base': rule.amount_percentage_base,
                    'register_id': rule.register_id.id,
                    'amount': amount,
                    'employee_id': employee.id,
                    'quantity': qty,
                    'rate': rate,
                    'slip_id': payslip_id,
                    'state': 'draft',
                }
                if rule.code == concept_salary_brw.code:
                    rules_dict['SUELDO'] = amount*qty or 0.0
                else:
                    rules_dict[rule.code] = amount*qty or 0.0
                localdict['concepto'] = rules_dict

                ''' Para el concepto de Descanso se debe tener la cuanta de las horas en la semana que el emplaedo falto,
                para ello se agrega un diccionario en el cual se lleva la cuenta de la cantidad en horas por concepto.
                Luego en la condicion del concepto Descanso se pregunta si esta cantidad de horas es mayor
                a la cantidad que se suministra en los parametro para la perdida del descanso
                '''
                if rule.concept_id.type_amount and rule.concept_id.type_amount == 'hours':
                    hours = qty
                else:
                    hours = qty*employee.shift_id.jor_day
                hours_dict[rule.code] = hours
                localdict['cantidad_horas'] = hours_dict

                if rule.concept_id.category_id.code not in categories_concept_dict:
                    categories_concept_dict[rule.concept_id.category_id.code] = amount or 0.0
                else:
                    categories_concept_dict[rule.concept_id.category_id.code] = categories_concept_dict[rule.concept_id.category_id.code] + amount
                localdict['categoria_concepto'] = categories_concept_dict
            else:
                #blacklist this rule and its children
                blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result
    
    def get_payslip_lines_nomina(self, cr, uid, contract_ids, payslip_id, context):
        return self.get_payslip_lines(cr, uid, contract_ids, payslip_id, context)
    
    def get_payslip_lines_anticipos(self, cr, uid, contract_ids, payslip_id, context):
        return self.get_payslip_lines(cr, uid, contract_ids, payslip_id, context)
    
    def get_payslip_lines_pago_especial(self, cr, uid, contract_ids, payslip_id, context):
        return self.get_payslip_lines(cr, uid, contract_ids, payslip_id, context)
    
    def calculo_acumulados(self, cr, uid, payslip, context):
        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'),
              _('La nomina %s no tiene calendario de pago. Debe asignarle un \
                calendario de pago antes de ejecutar cualquier proceso') % (payslip.employee_id.hr_payroll_id.name))

        concept_obj = self.pool.get('hr.concept')
        parameter_obj = self.pool.get('hr.parameters')

        concepts = [
                ('sso', ' Retencion S.S.O.', 'Concepto S.S.O.'),
                ('islr', ' Retencion I.S.L.R.', 'Concepto I.S.L.R.'),
                ('rpe', ' Regimen Prestacional de Empleo', 'Concepto R.P.E.'),
                ('faov', ' Retencion F.A.O.V.', 'Concepto F.A.O.V.'),
                ('utilidades', 's que inciden en Utilidades', 'Conceptos que inciden en Utilidades'),
                ('devengado', 's que inciden en I.S.R.L. para Devengado', 'Conceptos que inciden en I.S.R.L. para Devengado'),
            ]

        # Consigue el período y la nómina anterior
        period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))
        if period:
            period = period[0]
        previous_payslip_id = self.pool.get('hr.payslip').search(cr, uid, [('id','<>',payslip.id),
                                ('type_process', 'in', ['nomina', 'vacaciones',]),
                                ('employee_id', '=', payslip.employee_id.id), ('hr_process_id.state', '=', 'done'),
                                ('hr_process_id.date_done','<=', datetime.now().strftime('%Y-%m-%d'))], order='date_from desc')
        if previous_payslip_id:
            previous_payslip_id = previous_payslip_id[0]
        # Revisar búsqueda
        previous_payslip = self.pool.get('hr.payslip').browse(cr, uid, previous_payslip_id)

        values = {}

        for cpt in concepts:
            """
            FIXME: Estandarizar los nombres/tipos de los parametros para evitar
            estas comparaciones y hacer mas simple el metodo
            """
            args = 'concepto_' + cpt[0]
            key = cpt[0] + '_acumulado'
            brw_key = 'concept_' + cpt[0] + '_id'

            if cpt[0] == 'utilidades':
                args = 'conceptos_util'
                brw_key = 'concept_util_ids'

            if cpt[0] == 'devengado':
                args = 'conceptos_deven'
                brw_key = 'concept_deven_ids'

            param_id = parameter_obj.search(cr, uid, [('type', '=', args),
              ('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])

            if not param_id:
                param_id = parameter_obj.search(cr, uid, [('type', '=', args),('payroll_id', '=', None)])
            if not param_id:
                raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para concepto%s para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo %s')
                            %(cpt[1], payslip.employee_id.hr_payroll_id.name, cpt[2]))

            concept_brw = parameter_obj.browse(cr, uid, param_id)[0][brw_key]
            if cpt[0] not in ['utilidades', 'devengado']:
                if period.period_in_year == 1 and payslip.type_process == 'nomina':
                    values[key] = 0
                else:
                    try:
                        values[key] = previous_payslip[cpt[0] + '_acumulado']
                    except Exception, e:
                        values[key] = 0
                for line in payslip.line_ids:
                    if line.code == concept_brw.code:
                        values[key] = values[key] + line.total
            else:
                self._logger.debug('Calculando acumulado %s..' % cpt[0])
                if period.period_in_year == 1 and payslip.type_process == 'nomina':
                    values[key] = 0
                else:
                    try:
                        values[key] = previous_payslip[cpt[0] + '_acumulado']
                    except Exception, e:
                        values[key] = 0
                for line in payslip.line_ids:
                    if line.code in [concept.code for concept in concept_brw]:
                        for concept in concept_brw:
                            if line.code == concept.code:
                                values[key] = (values[key] + line.total) if concept.type_calc == 'allocation' \
                                  else (values[key] - line.total)

        res = self.pool.get('hr.payslip').write(cr, uid, payslip.id, values)

        """
        TODO: incluir esta porción de código en el mismo ciclo anterior.
        """
        # Busqueda del concepto que se definio como Concepto de anticipo de prestaciones-----------------------------
        param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_ant_pres_id:
            param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', None)])
        if not param_ant_pres_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Concepto anticipo prestaciones para la nomina %s. Ir a \
                    la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                    registrar un parametro de tipo Concepto anticipo prestaciones')
                    %(payslip.employee_id.hr_payroll_id.name))

        concept_ant_prest_id = parameter_obj.browse(cr, uid, param_ant_pres_id)[0].concept_ant_presta_id

        # Calculo del saldo de prestamos
        self._logger.debug('Calculando prestamos..')
        conceptos_prestamos = []
        total_saldo = 0.0
        concept_prest_ids = self.pool.get('hr.concept.loan').search(cr, uid, [])
        #Para cada uno de los conceptos que se difenieron como conceptos de prestamo verificar que no sean prestamo de anticipo
        #de prestaciones
        for concept_prest in self.pool.get('hr.concept.loan').browse(cr, uid, concept_prest_ids):
            if concept_prest.concept_id <> concept_ant_prest_id:
                conceptos_prestamos.append(concept_prest.concept_id.id)
        loan_ids = self.pool.get('hr.loan').search(cr, uid, [('employee_id', '=',payslip.employee_id.id),('active','=',True)])

        for loan in self.pool.get('hr.loan').browse(cr, uid, loan_ids):
            if loan.concept_id.concept_id.id in conceptos_prestamos:
                total_saldo = total_saldo + loan.remaining_debt

        return self.pool.get('hr.payslip').write(cr, uid, payslip.id,
                            {
                            'saldo_prestamo':total_saldo})

    def calculo_acumulado_payslip(self, cr, uid, slip_id):
        """
        Calcula el devengado generado en el payslip actual (Se usa en el script de recalculo de acumulados)
        """
        payslip = self.browse(cr, uid, slip_id)
        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'),
              _('La nomina %s no tiene calendario de pago. Debe asignarle un \
                calendario de pago antes de ejecutar cualquier proceso') % (payslip.employee_id.hr_payroll_id.name))

        concept_obj = self.pool.get('hr.concept')
        parameter_obj = self.pool.get('hr.parameters')

        concepts = [
                ('sso', ' Retencion S.S.O.', 'Concepto S.S.O.'),
                ('islr', ' Retencion I.S.L.R.', 'Concepto I.S.L.R.'),
                ('rpe', ' Regimen Prestacional de Empleo', 'Concepto R.P.E.'),
                ('faov', ' Retencion F.A.O.V.', 'Concepto F.A.O.V.'),
                ('utilidades', 's que inciden en Utilidades', 'Conceptos que inciden en Utilidades'),
                ('devengado', 's que inciden en I.S.R.L. para Devengado', 'Conceptos que inciden en I.S.R.L. para Devengado'),
            ]

        values = {}

        for cpt in concepts:
            """
            FIXME: Estandarizar los nombres/tipos de los parametros para evitar
            estas comparaciones y hacer mas simple el metodo
            """
            args = 'concepto_' + cpt[0]
            key = cpt[0] + '_acumulado'
            brw_key = 'concept_' + cpt[0] + '_id'

            if cpt[0] == 'utilidades':
                args = 'conceptos_util'
                brw_key = 'concept_util_ids'

            if cpt[0] == 'devengado':
                args = 'conceptos_deven'
                brw_key = 'concept_deven_ids'

            param_id = parameter_obj.search(cr, uid, [('type', '=', args),
              ('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])

            if not param_id:
                param_id = parameter_obj.search(cr, uid, [('type', '=', args),('payroll_id', '=', None)])
            if not param_id:
                raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para concepto%s para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo %s')
                            %(cpt[1], payslip.employee_id.hr_payroll_id.name, cpt[2]))

            concept_brw = parameter_obj.browse(cr, uid, param_id)[0][brw_key]
            if cpt[0] not in ['utilidades', 'devengado']:
                for line in payslip.line_ids:
                    if line.code == concept_brw.code:
                        values[key] = line.total
            else:
                self._logger.debug('Calculando acumulado %s..' % cpt[0])
                for line in payslip.line_ids:
                    if line.code in [concept.code for concept in concept_brw]:
                        for concept in concept_brw:
                            if line.code == concept.code:
                                values[key] = (values.get(key, 0) + line.total) if concept.type_calc == 'allocation' \
                                  else (values.get(key, 0) - line.total)
        return values
        
    def get_payslip_lines_vacaciones(self, cr, uid, contract_ids, payslip_id, context):
        """
            Método para el cálculo de la nómina de  vacaciones
        """
        parameter_obj = self.pool.get('hr.parameters')
        payslip_obj = self.pool.get('hr.payslip')
        concept_obj = self.pool.get('hr.concept')
        obj_rule = self.pool.get('hr.salary.rule')
        obj_holidays_employee = self.pool.get('hr.employee.holidays')
        get_method = getattr(self.pool.get('hr.employee'), '_get')
        get_concept = getattr(self.pool.get('hr.concept'), 'get_cal_concept')
        parameters_dict = {}
        categories_concept_dict = {}
        blacklist = []
        result_dict = {}
        rules = {}
        rules_dict = {}

        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)
        param_concept_holidays_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_vac'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_concept_holidays_id:
            param_concept_holidays_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_vac'),('payroll_id', '=', None)])
        if not param_concept_holidays_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el Concepto promedio sueldo vacaciones para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto promedio sueldo vacaciones')
                            %(payslip.employee_id.hr_payroll_id.name))

        concept_holidays_brw = parameter_obj.browse(cr, uid, param_concept_holidays_id)[0].concept_holidays_id

        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina
        param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto Salario')
                            %(payslip.employee_id.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id

        #Busqueda del concepto que se definio como concepto Bono Vacacional
        param_hol_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_bono_vac'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_hol_id:
            param_hol_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_bono_vac'),('payroll_id', '=', None)])
        if not param_hol_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto Bono Vacacional para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto Bono Vacacional')
                            %(payslip.employee_id.hr_payroll_id.name))
                            
        concept_bon_hol_brw = parameter_obj.browse(cr, uid, param_hol_id)[0].concept_bon_hol_id
                            
        #Busqueda del concepto que se definio como Dias de disfrute de vacaciones
        param_dias_vac_id = parameter_obj.search(cr, uid, [('type', '=', 'dias_disfrute_vacaciones'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_dias_vac_id:
            param_dias_vac_id = parameter_obj.search(cr, uid, [('type', '=', 'dias_disfrute_vacaciones'),('payroll_id', '=', None)])
        if not param_dias_vac_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Dias de disfrute de vacaciones %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Dias de disfrute de vacaciones')
                            %(payslip.employee_id.hr_payroll_id.name))

        dias_vac = int(parameter_obj.browse(cr, uid, param_dias_vac_id)[0].value)
        
        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'), _('La nomina %s no tiene calendario de pago. Debe asignarle un calendario de pago \
                            antes de ejecutar cualquier proceso')%(payslip.employee_id.hr_payroll_id.name))

        if not payslip.employee_id.contract_id:
            raise osv.except_osv(_('ERROR!'), _('El empleado %s no tiene un contrato registrado')%(payslip.employee_id.name))

        period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
        period_start_year = datetime.strptime(period.date_start,"%Y-%m-%d").year
        contract_date = datetime.strptime(payslip.employee_id.start_rel_lab,"%Y-%m-%d")
        contract_year = contract_date.year

        #Se cancelan 1 dia adicional por año a partir del 2º año
        dias_adicionales = period_start_year-contract_year-1 if period_start_year-contract_year > 1 else 0
        dias_adicionales = dias_adicionales if dias_adicionales < 15 else 15
        
        #Calculo del periodo que le toca al empleado salir de vacaciones (fecha de inicio y fecha de finalizacion)
        fecha_desde = datetime.strptime(period.date_end,"%Y-%m-%d") + timedelta(days=1)
        fecha_hasta = self.pool.get('hr.holidays.calendar').calculate_period_holidays(cr, uid, fecha_desde, dias_vac, dias_adicionales)

        vals_employee_holidays = {
            'name': 'Vacaciones '+ payslip.employee_id.name + ' / ' +str(period.hr_month_id.hr_year_id.year-1)+'-'+str(period.hr_month_id.hr_year_id.year),
            'employee_id': payslip.employee_id.id,
            'date_start': fecha_desde,
            'date_end': fecha_hasta,
            'hr_process_id': payslip.hr_process_id.id,
            'corresp_to_start': datetime((datetime.strptime(period.date_end,"%Y-%m-%d") - timedelta(days=365)).year,
                    contract_date.month,
                    contract_date.day),
            'corresp_to_end': datetime(datetime.strptime(period.date_end,"%Y-%m-%d").year,
                    contract_date.month,
                    contract_date.day) - timedelta(days=1),
            'type_holidays_status': 'Vacaciones Legales',
        }

        #Se guarda el periodo en el que el empleado va a estar de vacaciones
        old_ = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id','=',payslip.employee_id.id),
            ('corresp_to_start', '=', vals_employee_holidays['corresp_to_start']),('corresp_to_end', '=', vals_employee_holidays['corresp_to_end'])])
        if old_: #Si ya existe un periodo de vacaciones creado con el mismo nombre, este se debe sobreescribir
            self.pool.get('hr.employee.holidays').write(cr, uid, old_, vals_employee_holidays, context)
        else: #De lo contario se crea
            self.pool.get('hr.employee.holidays').create(cr, uid, vals_employee_holidays, context)

        localdict = {
            'payslip': payslip,
            'param': [cr, uid], # Prueba de get_cal_concept
            'dias_adicionales': dias_adicionales,
            'atributo': get_method,
            'historial': get_concept, # Prueba de get_cal_concept
            'categoria_concepto': {},
            'concepto': {},
            'periodo': period,
        }
        localdict.update({'lunes': 1})

        edad = (datetime.now() - datetime.strptime(payslip.employee_id.birthday,"%Y-%m-%d")).days/365
        localdict.update({'empleado': payslip.employee_id,
                'sexo': 'Femenino' if payslip.employee_id.gender == 'female' else 'Masculino',
                'edad': edad,
                'jornada_dia': payslip.employee_id.shift_id.jor_day,
                'jornada_semana': payslip.employee_id.shift_id.jor_week,})

        #Agregar parametros de la nomina
        for parameter in payslip.employee_id.hr_payroll_id.parameters_ids:
            parameters_dict[parameter.type] = parameter.value or 0.0
        localdict.update({'parametros': parameters_dict,})

        structure_ids = []
        structure_brw = [j for j in payslip.employee_id.hr_payroll_id.hr_payroll_structure_ids]
        for str_brw in structure_brw: #El tipo de la estructura debe ser del mismo tipo del proceso
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)
        if not structure_ids:
            raise osv.except_osv(_('ERROR!'), _('No hay una estructura de tipo %s definida para la nomina %s')
                            %(payslip.hr_process_id.type, payslip.employee_id.hr_payroll_id.name))

        #get the rules of the structure and their children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]
        #get the attributes of the structure
        attribute_ids = self.pool.get('hr.payroll.structure').get_all_attributes(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)

        for attribute in payslip.employee_id.attribute_ids:
            if attribute.concept_id.id == concept_salary_brw.id:
                rules_dict['SUELDO'] = attribute.amount
                localdict['concepto'] = rules_dict

        for attribute in attribute_ids:
            key = attribute.code
            if not key in result_dict:
                result_dict[key] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = attribute.amount or 0.0
                localdict['concepto'] = rules_dict
            else:
                result_dict[key+str(attribute.id)] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = rules_dict[attribute.code] + (attribute.amount or 0.0)
                localdict['concepto'] = rules_dict

        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
            key = rule.code
            localdict['result'] = None
            localdict['result_qty'] = 1.0
            #check if the rule can be applied
            if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                #compute the amount of the rule
                amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)

                #check if there is already a rule computed with that code
                previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                #set/overwrite the amount computed for this rule in the localdict
                tot_rule = amount * qty * rate / 100.0
                localdict[rule.code] = tot_rule
                rules[rule.code] = rule

                if rule.code == 'DA':
                    qty = localdict['dias_adicionales']

                if rule.code == concept_bon_hol_brw.code:
                    qty = parameters_dict['vacaciones']

                result_dict[key] = {
                    'salary_rule_id': rule.id,
                    'name': rule.name,
                    'code': key,
                    'category_id': rule.category_id.id,
                    'sequence': rule.sequence,
                    'appears_on_payslip': rule.appears_on_payslip,
                    'condition_select': rule.condition_select,
                    'condition_python': rule.condition_python,
                    'condition_range': rule.condition_range,
                    'condition_range_min': rule.condition_range_min,
                    'condition_range_max': rule.condition_range_max,
                    'amount_select': rule.amount_select,
                    'amount_fix': rule.amount_fix,
                    'amount_python_compute': rule.amount_python_compute,
                    'amount_percentage': rule.amount_percentage,
                    'amount_percentage_base': rule.amount_percentage_base,
                    'register_id': rule.register_id.id,
                    'amount': amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': qty,
                    'rate': rate,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }

                if rule.code == concept_salary_brw.code:
                    rules_dict['SUELDO'] = amount*qty or 0.0
                else:
                    rules_dict[rule.code] = amount*qty or 0.0
                localdict['concepto'] = rules_dict

                if rule.concept_id.category_id.code not in categories_concept_dict:
                    categories_concept_dict[rule.concept_id.category_id.code] = amount or 0.0
                else:
                    categories_concept_dict[rule.concept_id.category_id.code] = categories_concept_dict[rule.concept_id.category_id.code] + amount
                localdict['categoria_concepto'] = categories_concept_dict

            else:
                #blacklist this rule and its children
                blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result

    def get_payslip_lines_utilidades(self, cr, uid, contract_ids, payslip_id, context):
        """
            Método para el cálculo de la nómina de utilidades
        """
        parameter_obj = self.pool.get('hr.parameters')
        payslip_obj = self.pool.get('hr.payslip')
        concept_obj = self.pool.get('hr.concept')
        obj_rule = self.pool.get('hr.salary.rule')
        obj_holidays_employee = self.pool.get('hr.employee.holidays')
        process_obj = self.pool.get('hr.process')
        get_method = getattr(self.pool.get('hr.employee'), '_get')
        get_concept = getattr(self.pool.get('hr.concept'), 'get_cal_concept')
        parameters_dict = {}
        categories_concept_dict = {}
        blacklist = []
        result_dict = {}
        rules = {}
        rules_dict = {}

        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)

        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina-----------------------------
        param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo concepto_sueldo')
                            %(payslip.employee_id.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id

        #Busqueda del concepto que se definio como concepto que acumula la utilidad bruta pagada-----------------------------
        param_concept_util_bruta_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_util_bruta'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_concept_util_bruta_id:
            param_concept_util_bruta_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_util_bruta'),('payroll_id', '=', None)])
        if not param_concept_util_bruta_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto que acumula la utilidad bruta pagada %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto utilidad bruta pagada')
                            %(payslip.employee_id.hr_payroll_id.name))

        concept_util_bruta_brw = parameter_obj.browse(cr, uid, param_concept_util_bruta_id)[0].concept_util_bruta_id

        #Busqueda del concepto que se definio como concepto Utilidades-----------------------------
        param_util_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_bono_util'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_util_id:
            param_util_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_bono_util'),('payroll_id', '=', None)])
        if not param_util_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto Utilidades para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto Utilidades')
                            %(payslip.employee_id.hr_payroll_id.name))

        concept_util_brw = parameter_obj.browse(cr, uid, param_util_id)[0].concept_util_id
        #-------------------------------------------------------------------------------------------------------------------------

        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'), _('La nomina %s no tiene calendario de pago. Debe asignarle un calendario de pago \
                            antes de ejecutar cualquier proceso')
                            %(payslip.employee_id.hr_payroll_id.name))

        period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]

        process = payslip.hr_process_id
        payroll_ids = [p.id for p in process.hr_payroll_ids]
        year = datetime.strptime(process.date_end,"%Y-%m-%d").year
        start = datetime(year, 01, 01)
        end = datetime(year, 12, 31)
        date_start = datetime.strptime(process.date_start,"%Y-%m-%d")
        date_end = datetime.strptime(process.date_end,"%Y-%m-%d")

        if date_start == start and date_end == end:
            if period.type == 'diario':
                num_periods = 52 #Correspondientes a las 52 semanas del año
            else:
                num_periods = 24 #Correspondientes a las 24 quincenas del año
            periodos_estimados = 0
        else:
            periods_old = self.pool.get('hr.period').search(cr, uid, [('type','=', period.type),('date_start','>=', date_start),
                                ('date_end','<=', date_end),('state','=','close')]) + \
            self.pool.get('hr.period').search(cr, uid, [('type','=', period.type),('date_start','<', date_start),
                                ('date_end','>=', date_start),('state','=','close')]) + \
            self.pool.get('hr.period').search(cr, uid, [('type','=', period.type),('date_start','<=', date_end),
                                ('date_end','>', date_end),('state','=','close')])
            num_periods = len(periods_old)
            periodos_estimados = 12 - period.hr_month_id.month_year + 1

        #Verificar si hay pago de utilidades en el mismo año de pago del proceso actual

        process_old_ids = process_obj.search(cr, uid, [('type', '=', 'utilidades'), ('date_end', '>=', start), ('date_end', '<=', end),
                                ('hr_payroll_ids', '=', payroll_ids), ('state', '=', 'done')])

        utilidades_canceladas = 0.0
        for process_old in process_obj.browse(cr, uid, process_old_ids):
            payslip_old_id = payslip_obj.search(cr, uid, [('hr_process_id', '=', process_old.id), ('employee_id', '=', payslip.employee_id.id)],
                                                context=context)
            payslip_line_ids = self.pool.get('hr.payslip.line').search(cr, uid, [('slip_id','=', payslip_old_id)])
            for line in self.pool.get('hr.payslip.line').browse(cr, uid, payslip_line_ids):
                if line.code == concept_util_bruta_brw.code:
                    utilidades_canceladas = utilidades_canceladas + line.total
        
        localdict = {
            'payslip': payslip,
            'param': [cr, uid], # Prueba de get_cal_concept
            'utilidades_canceladas': utilidades_canceladas,
            'atributo': get_method,
            'historial': get_concept, # Prueba de get_cal_concept
            'categoria_concepto': {},
            'concepto': {},
            'periodos_estimados': periodos_estimados,
            'num_periodos': num_periods,
            'periodo': period,
        }
        
        edad = (datetime.now() - datetime.strptime(payslip.employee_id.birthday,"%Y-%m-%d")).days/365
        localdict.update({'empleado': payslip.employee_id,
                'sexo': 'Femenino' if payslip.employee_id.gender == 'female' else 'Masculino',
                'edad': edad,
                'jornada_dia': payslip.employee_id.shift_id.jor_day,
                'jornada_semana': payslip.employee_id.shift_id.jor_week,})

        #Agregar parametros de la nomina
        for parameter in payslip.employee_id.hr_payroll_id.parameters_ids:
            parameters_dict[parameter.type] = parameter.value or 0.0
        localdict.update({'parametros': parameters_dict,})

        structure_ids = []
        structure_brw = [j for j in payslip.employee_id.hr_payroll_id.hr_payroll_structure_ids]
        for str_brw in structure_brw: #El tipo de la estructura debe ser del mismo tipo del proceso
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)
        if not structure_ids:
            raise osv.except_osv(_('ERROR!'), _('No hay una estructura de tipo %s definida para la nomina %s')
                                %(payslip.hr_process_id.type, payslip.employee_id.hr_payroll_id.name))

        #get the rules of the structure and their children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]
        #get the attributes of the structure
        attribute_ids = self.pool.get('hr.payroll.structure').get_all_attributes(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)

        # Calculo del sueldo mensual del empleado para obtener el monto estimado de los meses futuros que se tienen que cancelar
        for attribute in payslip.employee_id.attribute_ids:
            if attribute.concept_id.id == concept_salary_brw.id:
                if period.type == 'diario':
                    localdict['sueldo_mensual'] = attribute.amount*30
                else:
                    localdict['sueldo_mensual'] = attribute.amount

        for attribute in attribute_ids:
            key = attribute.code
            if not key in result_dict:
                result_dict[key] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = attribute.amount or 0.0
                localdict['concepto'] = rules_dict
            else:
                result_dict[key+str(attribute.id)] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = rules_dict[attribute.code] + (attribute.amount or 0.0)
                localdict['concepto'] = rules_dict
            
        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
            key = rule.code
            localdict['result'] = None
            localdict['result_qty'] = 1.0
            #check if the rule can be applied
            if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                #compute the amount of the rule
                amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)

                #check if there is already a rule computed with that code
                previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                #set/overwrite the amount computed for this rule in the localdict
                tot_rule = amount * qty * rate / 100.0
                localdict[rule.code] = tot_rule
                rules[rule.code] = rule

                if rule.code == concept_util_brw.code:
                    qty = parameters_dict['utilidades']

                result_dict[key] = {
                    'salary_rule_id': rule.id,
                    'name': rule.name,
                    'code': key,
                    'category_id': rule.category_id.id,
                    'sequence': rule.sequence,
                    'appears_on_payslip': rule.appears_on_payslip,
                    'condition_select': rule.condition_select,
                    'condition_python': rule.condition_python,
                    'condition_range': rule.condition_range,
                    'condition_range_min': rule.condition_range_min,
                    'condition_range_max': rule.condition_range_max,
                    'amount_select': rule.amount_select,
                    'amount_fix': rule.amount_fix,
                    'amount_python_compute': rule.amount_python_compute,
                    'amount_percentage': rule.amount_percentage,
                    'amount_percentage_base': rule.amount_percentage_base,
                    'register_id': rule.register_id.id,
                    'amount': amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': qty,
                    'rate': rate,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }

                if rule.code == concept_util_brw.code:
                    rules_dict[concept_util_brw.code] = amount*qty or 0.0
                else:
                    rules_dict[rule.code] = amount or 0.0
                localdict['concepto'] = rules_dict

                if rule.concept_id.category_id.code not in categories_concept_dict:
                    categories_concept_dict[rule.concept_id.category_id.code] = amount or 0.0
                else:
                    categories_concept_dict[rule.concept_id.category_id.code] = categories_concept_dict[rule.concept_id.category_id.code] + amount
                localdict['categoria_concepto'] = categories_concept_dict
            else:
                #blacklist this rule and its children
                blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result

    def get_payslip_lines_prestaciones(self, cr, uid, contract_ids, payslip_id, context):
        """
            Método para el cálculo de prestaciones.
            El proceso esta diseñado para ser corrido el ultimo periodo de cada mes
        """

        parameter_obj = self.pool.get('hr.parameters')
        payslip_obj = self.pool.get('hr.payslip')
        concept_obj = self.pool.get('hr.concept')
        obj_rule = self.pool.get('hr.salary.rule')
        obj_holidays_employee = self.pool.get('hr.employee.holidays')
        process_obj = self.pool.get('hr.process')
        get_method = getattr(self.pool.get('hr.employee'), '_get')
        get_concept = getattr(self.pool.get('hr.concept'), 'get_cal_concept')
        get_anticipo = getattr(self.pool.get('hr.concept'), 'get_cal_concept_anticipos')
        parameters_dict = {}
        categories_concept_dict = {}
        blacklist = []
        result_dict = {}
        rules = {}
        rules_dict = {}

        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)

        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina-----------------------------------------
        param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parámetro definido para el concepto salario para la nómina %s. Ir a \
                            la pestaña Configuración -> Configuraciones de nómina -> Parámetros de nómina y \
                            registrar un parámetro de tipo concepto_sueldo')
                            %(payslip.employee_id.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id

        #Busqueda del concepto que se definio como Fecha máxima para cálculo días adicionales de prestaciones-----------------------------
        param_date_prest_id = parameter_obj.search(cr, uid, [('type', '=', 'date_prest'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_date_prest_id:
            param_date_prest_id = parameter_obj.search(cr, uid, [('type', '=', 'date_prest'),('payroll_id', '=', None)])
        if not param_date_prest_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un párametro definido para Fecha máxima para cálculo días adicionales de prestaciones \
                            para la nómina %s. Ir a la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Fecha máxima para cálculo días adicionales de prestaciones')
                            %(payslip.employee_id.hr_payroll_id.name))

        date_prest = parameter_obj.browse(cr, uid, param_date_prest_id)[0].date_prest #Fecha máxima para cálculo días adicionales de prestaciones
        date_prest = datetime.strptime(date_prest,"%Y-%m-%d")
        max_year = parameter_obj.browse(cr, uid, param_date_prest_id)[0].max_year #Número de años máximo para acumulado de días adicionales de prestaciones
        #---------------------------------------------------------------------------------------------------------------------------------
        
        #Busqueda del concepto que se definio como Concepto Dias adicionales prestaciones-----------------------------
        param_dap_id = parameter_obj.search(cr, uid, [('type', '=', 'dias_adic_prest')])
        if not param_dap_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un párametro definido para Concepto Dias adicionales prestaciones. \
                            Ir a la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto Dias adicionales prestaciones'))

        concept_dap_id = parameter_obj.browse(cr, uid, param_dap_id)[0].concept_prest_id
        #---------------------------------------------------------------------------------------------------------------------------------
        
        #Busqueda del concepto que se definio como Concepto para registro de fecha para calculos de prestaciones-----------------------------
        param_fp_id = parameter_obj.search(cr, uid, [('type', '=', 'fecha_prest')])
        concept_fp_id = 0
        if param_fp_id:
            concept_fp_id = parameter_obj.browse(cr, uid, param_fp_id)[0].concept_prest_id
        #---------------------------------------------------------------------------------------------------------------------------------
        
        #Busqueda del concepto que se definio como Concepto calculo de prestaciones mensual-----------------------------
        param_prest_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_dias_prest')])
        if not param_prest_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un párametro definido para Concepto calculo de prestaciones mensual. \
                            Ir a la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto calculo de prestaciones mensual'))

        prest = parameter_obj.browse(cr, uid, param_prest_id)[0]
        concept_5d_id = prest.concept_prest_id
        dias_prest = prest.value
        #---------------------------------------------------------------------------------------------------------------------------------            
                    
        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'), _('La nómina %s no tiene calendario de pago. Debe asignarle un calendario de pago \
                            antes de ejecutar cualquier proceso')%(payslip.employee_id.hr_payroll_id.name))

        period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]

        # Fijacion del numero de periodo que se van a consultar segun el tipo de nomina del empleado
        if period.type == 'diario':
            num_periods = period.hr_month_id.weeks #Correspondientes al numero de semanas del mes
        else:
            num_periods = 2 #Correspondientes a las 2 quincenas del mes

        # Calculo de la antiguedad en meses del empleado para ser usado en el concepto de prestaciones que se debe calcular cada 3 meses
        if not payslip.employee_id.contract_id:
            raise osv.except_osv(_('ERROR!'), _('El empleado %s no tiene un contrato. El contrato es obligatorio para \
                            todo empleado')%(payslip.employee_id.name))

        contract_date = datetime.strptime(payslip.employee_id.start_rel_lab,"%Y-%m-%d")
        period_date_end = datetime.strptime(period.date_end,"%Y-%m-%d")
        
        attr_date_prest = list(set(concept_fp_id.attribute_ids) & set(payslip.employee_id.attribute_ids)) if concept_fp_id else False
        date_prest = datetime.strptime(attr_date_prest[0].date_value,"%Y-%m-%d") if attr_date_prest \
                        else contract_date
        num_meses = ((period_date_end.year * 12) + period_date_end.month) - ((date_prest.year * 12) + date_prest.month)
        num_dias = (math.ceil(num_meses/12)+1)*2 if (math.ceil(num_meses/12)+1)*2 <= 30 else 30
        
        # Calculo del numero de periodos a consultar para el concepto de prestaciones trimestrales (Se debe calcular cuantos periodos tiene ese trimestre)
        if num_meses <> 0 and num_meses%3 == 0: # Solo si se cumple el trimestre para el empleado
            if period.hr_month_id.month_year >= 3: # Si el mes en el que se esta calculando es por lo menos el mes tres se calculan los periodos de
                                                    # ese mes y de los 2 meses anteriores
                month_ant_ids = self.pool.get('hr.month').search(cr, uid, [('hr_year_id','=', period.hr_month_id.hr_year_id.id),
                                    ('month_year', 'in',(period.hr_month_id.month_year-1,period.hr_month_id.month_year-2))])
                month_ant_brw = self.pool.get('hr.month').browse(cr, uid, month_ant_ids)

                num_periodos_trimestre = 0
                for period_ in period.hr_month_id.hr_period_ids:
                    if period_.type == period.type:
                        num_periodos_trimestre = num_periodos_trimestre + 1
                for period_ in month_ant_brw[0].hr_period_ids:
                    if period_.type == period.type:
                        num_periodos_trimestre = num_periodos_trimestre + 1
                for period_ in month_ant_brw[1].hr_period_ids:
                    if period_.type == period.type:
                        num_periodos_trimestre = num_periodos_trimestre + 1

            else: # El periodo actual es del mes 1 o 2 del año es necesario consultar dependiendo del caso los periodos de uno o dos ultimos
                    # meses del año anterior
                year_ant_id = self.pool.get('hr.year').search(cr, uid, [('year', '=', period.hr_month_id.hr_year_id.year-1)])
                if not year_ant_id:
                    num_periodos_trimestre = period.period_in_year #ESTO SE COLOCA PROVISIONAL PORQUE NO EXISTEN HISTORICOS, SE DEBE QUITAR Y DEJAR EL RAISE
                    #~ raise osv.except_osv(_('ERROR!'), _('Es necesario tomar datos de los períodos del año pasado el cual \
                                    #~ no existe!'))
                elif period.hr_month_id.month_year == 2:
                    month_ant_ids_1 = self.pool.get('hr.month').search(cr, uid, [('hr_year_id','=', period.hr_month_id.hr_year_id.id),
                                    ('month_year', '=', period.hr_month_id.month_year-1)])
                    month_ant_brw_1 = self.pool.get('hr.month').browse(cr, uid, month_ant_ids_1)
                    month_ant_ids_2 = self.pool.get('hr.month').search(cr, uid, [('hr_year_id','=', year_ant_id),
                                    ('month_year', '=', 12)])
                    month_ant_brw_2 = self.pool.get('hr.month').browse(cr, uid, month_ant_ids_2)
                    
                    if isinstance(month_ant_brw_1, list):
                         month_ant_brw_1 = month_ant_brw_1[0]
                    if isinstance(month_ant_brw_2, list):
                         month_ant_brw_2 = month_ant_brw_2[0]

                    num_periodos_trimestre = 0
                    for period_ in period.hr_month_id.hr_period_ids:
                        if period_.type == period.type:
                            num_periodos_trimestre = num_periodos_trimestre + 1
                    for period_ in month_ant_brw_1.hr_period_ids:
                        if period_.type == period.type:
                            num_periodos_trimestre = num_periodos_trimestre + 1
                    for period_ in month_ant_brw_2.hr_period_ids:
                        if period_.type == period.type:
                            num_periodos_trimestre = num_periodos_trimestre + 1

                elif period.hr_month_id.month_year == 1:
                    month_ant_ids = self.pool.get('hr.month').search(cr, uid, [('hr_year_id','=', year_ant_id),
                                    ('month_year', 'in', (11,12))])
                    month_ant_brw = self.pool.get('hr.month').browse(cr, uid, month_ant_ids)

                    num_periodos_trimestre = 0
                    for period_ in period.hr_month_id.hr_period_ids:
                        if period_.type == period.type:
                            num_periodos_trimestre = num_periodos_trimestre + 1
                    for period_ in month_ant_brw[0].hr_period_ids:
                        if period_.type == period.type:
                            num_periodos_trimestre = num_periodos_trimestre + 1
                    for period_ in month_ant_brw[1].hr_period_ids:
                        if period_.type == period.type:
                            num_periodos_trimestre = num_periodos_trimestre + 1

        else:
            num_periodos_trimestre = 0

        # Si al empleado se le cancelo vacaciones en el mes actual, calcular el numero de periodos hacia atras que deben ser
        # consultados para obtener el acumulado de Bono vacacional mes a mes que ya se ha cancelado
        num_periodos_vacaciones = 0
        hol_employee_ids = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id','=', payslip.employee_id.id),('state','=','done'),
                                        ('prest_calc','=',False)])
        if hol_employee_ids:
            payslip_holidays_brw = self.pool.get('hr.employee.holidays').browse(cr, uid, hol_employee_ids)[0]
            periods_old = self.pool.get('hr.period').search(cr, uid, [('type','=', period.type),('date_start','>=', payslip_holidays_brw.corresp_to_start),
                                ('date_end','<=', payslip_holidays_brw.corresp_to_end),('state','=','close')]) + \
                self.pool.get('hr.period').search(cr, uid, [('type','=', period.type),('date_start','<', payslip_holidays_brw.corresp_to_start),
                                ('date_end','>=', payslip_holidays_brw.corresp_to_start),('state','=','close')]) + \
                self.pool.get('hr.period').search(cr, uid, [('type','=', period.type),('date_start','<=', payslip_holidays_brw.corresp_to_end),
                                ('date_end','>', payslip_holidays_brw.corresp_to_end),('state','=','close')])
            num_periodos_vacaciones = len(periods_old)+1 #Se le suma uno para que no deje por fuera el primer periodo al momento de consultar el historico del concepto
        # Si al empleado estara de aniversario el prox mes se debe calcular el numero de dias adicionales que le corresponden por antiguedad, en base
        # al salario intergral promedio del año inmediatamente anterior, esto es, el de los doce meses anteriores a la fecha aniversario del trabajador
        # Ejemplo: Fecha de inicio en la empresa: 18 de julio de 2008.
        # Fecha aniversario: 18 de julio de 2012.
        # Año para el promedio: El transcurrido entre el 18 de julio 2011 hasta el 18 de julio 2012.
        num_periodos_dias_adic = 0

        if period.type == 'diario':
            num_periodos_dias_adic = 52
        elif period.type == 'mensual':
            num_periodos_dias_adic = 24

        # Busqueda de la tasa de interes para el calculo de intereses sobre prestaciones. Debe ser la tasa de interes correspondiente al mes anterior
        # al que se esta calculando
        mes_ant_id = False
        if period.hr_month_id.month_year > 1:
            mes_ant_id = self.pool.get('hr.month').search(cr, uid, [('month_year', '=', period.hr_month_id.month_year-1),
                                                                ('hr_year_id', '=', period.hr_month_id.hr_year_id.id)])
        else:
            year_old_id = self.pool.get('hr.year').search(cr, uid, [('year', '=', period.hr_month_id.hr_year_id.year-1)])
            if year_old_id:
                mes_ant_id = self.pool.get('hr.month').search(cr, uid, [('month_year', '=', 12),('hr_year_id', '=', year_old_id)])
        if mes_ant_id:
            mes_ant = self.pool.get('hr.month').browse(cr, uid, mes_ant_id)
            if not mes_ant[0].interes_prest:
                raise osv.except_osv(_('ERROR!'), _('No se ha registrado la tasa de interes de prestaciones del mes %s')
                            %(mes_ant[0].name))
            interes_prest = mes_ant[0].interes_prest
        else:
            interes_prest = 0

        localdict = {
            'math': math,
            'num_meses':num_meses,
            'payslip': payslip,
            'param': [cr, uid], # Prueba de get_cal_concept
            'atributo': get_method,
            'historial': get_concept, # Prueba de get_cal_concept
            'anticipo': get_anticipo, # Prueba de get_anticipo
            'categoria_concepto': {},
            'concepto': {},
            'periodo': period,
            'tasa_interes': interes_prest,
            'num_periodos': num_periods,
            'num_periodos_trimestre': num_periodos_trimestre,
            'num_periodos_vacaciones': num_periodos_vacaciones,
            'num_periodos_dias_adic': num_periodos_dias_adic,
            'datetime': datetime,
        }


        edad = (datetime.now() - datetime.strptime(payslip.employee_id.birthday,"%Y-%m-%d")).days/365
        localdict.update({'empleado': payslip.employee_id,
                'sexo': 'Femenino' if payslip.employee_id.gender == 'female' else 'Masculino',
                'edad': edad,
                'jornada_dia': payslip.employee_id.shift_id.jor_day,
                'jornada_semana': payslip.employee_id.shift_id.jor_week,})

        #Agregar parametros de la nomina
        for parameter in payslip.employee_id.hr_payroll_id.parameters_ids:
            parameters_dict[parameter.type] = parameter.value or 0.0
        localdict.update({'parametros': parameters_dict,})

        structure_ids = []
        structure_brw = [j for j in payslip.employee_id.hr_payroll_id.hr_payroll_structure_ids]
        for str_brw in structure_brw: #El tipo de la estructura debe ser del mismo tipo del proceso
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)
        if not structure_ids:
            raise osv.except_osv(_('ERROR!'), _('No hay una estructura de tipo %s definida para la nomina %s')
                                %(payslip.hr_process_id.type, payslip.employee_id.hr_payroll_id.name))

        #get the rules of the structure and their children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]
        #get the attributes of the structure
        attribute_ids = self.pool.get('hr.payroll.structure').get_all_attributes(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)

        # Calculo del sueldo mensual del empleado para obtener el monto estimado de los meses futuros que se tienen que cancelar
        for attribute in payslip.employee_id.attribute_ids:
            if attribute.concept_id.id == concept_salary_brw.id:
                if period.type == 'diario':
                    localdict['sueldo_mensual'] = attribute.amount*30
                else:
                    localdict['sueldo_mensual'] = attribute.amount

        for attribute in attribute_ids:
            key = attribute.code
            if not key in result_dict:
                result_dict[key] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = attribute.amount or 0.0
                localdict['concepto'] = rules_dict
            else:
                result_dict[key+str(attribute.id)] = {
                    'attribute_id': attribute.id,
                    'name': attribute.name,
                    'code': attribute.code,
                    'appears_on_payslip': attribute.appears_on_payslip,
                    'amount': attribute.amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': 1.00,
                    'rate': 100.00,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }
                rules_dict[attribute.code] = rules_dict[attribute.code] + (attribute.amount or 0.0)
                localdict['concepto'] = rules_dict

        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
            key = rule.code
            localdict['result'] = None
            localdict['result_qty'] = 1.0
            #check if the rule can be applied
            if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                #compute the amount of the rule
                amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)

                #check if there is already a rule computed with that code
                previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                #set/overwrite the amount computed for this rule in the localdict
                tot_rule = amount * qty * rate / 100.0
                localdict[rule.code] = tot_rule
                rules[rule.code] = rule
                
                if rule.code == concept_dap_id.code:
                    qty = num_dias
                    
                elif rule.code == concept_5d_id.code:
                    qty = dias_prest
                    
                result_dict[key] = {
                    'salary_rule_id': rule.id,
                    'name': rule.name,
                    'code': key,
                    'category_id': rule.category_id.id,
                    'sequence': rule.sequence,
                    'appears_on_payslip': rule.appears_on_payslip,
                    'condition_select': rule.condition_select,
                    'condition_python': rule.condition_python,
                    'condition_range': rule.condition_range,
                    'condition_range_min': rule.condition_range_min,
                    'condition_range_max': rule.condition_range_max,
                    'amount_select': rule.amount_select,
                    'amount_fix': rule.amount_fix,
                    'amount_python_compute': rule.amount_python_compute,
                    'amount_percentage': rule.amount_percentage,
                    'amount_percentage_base': rule.amount_percentage_base,
                    'register_id': rule.register_id.id,
                    'amount': amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': qty,
                    'rate': rate,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }

                if rule.code == concept_dap_id.code or rule.code == concept_5d_id.code:
                    rules_dict[rule.code] = amount*qty or 0.0
                else:
                    rules_dict[rule.code] = amount or 0.0
                
                localdict['concepto'] = rules_dict

                if rule.concept_id.category_id.code not in categories_concept_dict:
                    categories_concept_dict[rule.concept_id.category_id.code] = amount or 0.0
                else:
                    categories_concept_dict[rule.concept_id.category_id.code] = categories_concept_dict[rule.concept_id.category_id.code] + amount
                localdict['categoria_concepto'] = categories_concept_dict

            else:
                #blacklist this rule and its children
                blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result

    def get_payslip_lines_guarderias(self, cr, uid, contract_ids, payslip_id, context):
        """
        Método para el cálculo de pago de guarderias
        """

        parameter_obj = self.pool.get('hr.parameters')
        payslip_obj = self.pool.get('hr.payslip')
        concept_obj = self.pool.get('hr.concept')
        obj_rule = self.pool.get('hr.salary.rule')
        blacklist = []
        result_dict = {}
        rules = {}
        rules_dict = {}

        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)

        param_guarderias_id = parameter_obj.search(cr, uid, [('type', '=', 'guarderia')])
        if not param_guarderias_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el monto a pagar por guardería. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Monto máximo a pagar por Guardería'))

        param_guarderias_brw = parameter_obj.browse(cr, uid, param_guarderias_id)[0]

        localdict = {
            'monto_guarderia': 0.0,
        }

        monto_guarderia = {}
        for employee_family in payslip.employee_id.employee_family_ids:
            if employee_family.relationship == 'hijo_a' and employee_family.guarderia_id:
                '''
                Si el monto de la guarderia es <= que lo que se paga por guarderia entonces se cancela
                el monto de la guarderia sino se cancela el monto maximo establecido
                '''
                if employee_family.guarderia_amount <= param_guarderias_brw.value: #Si el monto de la guarderia <= que lo que se paga por guarderia
                    monto_guarderia[employee_family] = employee_family.guarderia_amount
                else:
                    monto_guarderia[employee_family] = param_guarderias_brw.value

        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'), _('La nomina %s no tiene calendario de pago. Debe asignarle un calendario de pago \
                            antes de ejecutar cualquier proceso')
                            %(payslip.employee_id.hr_payroll_id.name))

        period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
        structure_ids = []
        structure_brw = [j for j in payslip.employee_id.hr_payroll_id.hr_payroll_structure_ids]
        for str_brw in structure_brw: #El tipo de la estructura debe ser del mismo tipo del proceso
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)
        if not structure_ids:
            raise osv.except_osv(_('ERROR!'), _('No hay una estructura de tipo %s definida para la nomina %s')
                                %(payslip.hr_process_id.type, payslip.employee_id.hr_payroll_id.name))

        #get the rules of the structure and their children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
            localdict['result'] = None
            localdict['result_qty'] = 1.0
            #check if the rule can be applied
            if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                for hijo in monto_guarderia:
                    key = rule.code+'/'+hijo.name
                    localdict['monto_guarderia'] = monto_guarderia[hijo]
                    #compute the amount of the rule
                    amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)

                    #check if there is already a rule computed with that code
                    previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                    #set/overwrite the amount computed for this rule in the localdict
                    tot_rule = amount * qty * rate / 100.0
                    localdict[rule.code] = tot_rule
                    rules[rule.code] = rule

                    result_dict[key] = {
                        'salary_rule_id': rule.id,
                        'name': rule.name+' / '+hijo.name,
                        'code': key,
                        'category_id': rule.category_id.id,
                        'sequence': rule.sequence,
                        'appears_on_payslip': rule.appears_on_payslip,
                        'condition_select': rule.condition_select,
                        'condition_python': rule.condition_python,
                        'condition_range': rule.condition_range,
                        'condition_range_min': rule.condition_range_min,
                        'condition_range_max': rule.condition_range_max,
                        'amount_select': rule.amount_select,
                        'amount_fix': rule.amount_fix,
                        'amount_python_compute': rule.amount_python_compute,
                        'amount_percentage': rule.amount_percentage,
                        'amount_percentage_base': rule.amount_percentage_base,
                        'register_id': rule.register_id.id,
                        'amount': amount,
                        'employee_id': payslip.employee_id.id,
                        'quantity': qty,
                        'rate': rate,
                        'slip_id': payslip.id,
                        'state': 'draft',
                        'employee_family_id': hijo.id,
                    }
            else:
                #blacklist this rule and its children
                blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result

    def get_payslip_lines_bono_alimentacion(self, cr, uid, contract_ids, payslip_id, context):
        """
        Metodo para el calculo de pago de bono de alimentacion
        """
        parameter_obj = self.pool.get('hr.parameters')
        payslip_obj = self.pool.get('hr.payslip')
        concept_obj = self.pool.get('hr.concept')
        obj_rule = self.pool.get('hr.salary.rule')
        blacklist = []
        result_dict = {}
        rules = {}
        rules_dict = {}

        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)

        localdict = {
            'param': [cr, uid],
            'payslip': payslip,
            'empleado': payslip.employee_id,
            'mes': payslip.hr_process_id.month_id,
            'concepto': {},
            'datetime':datetime,
        }

        #Incluir en el localdict categories los montos de los conceptos variables
        for line in payslip.line_ids:
            rules_dict[line.code] = line.total
            localdict['concepto'] = rules_dict

        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'), _('La nomina %s no tiene calendario de pago. Debe asignarle un calendario de pago \
                            antes de ejecutar cualquier proceso')
                            %(payslip.employee_id.hr_payroll_id.name))

        period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
        structure_ids = []
        structure_brw = [j for j in payslip.employee_id.hr_payroll_id.hr_payroll_structure_ids]
        for str_brw in structure_brw: #El tipo de la estructura debe ser del mismo tipo del proceso
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)
        if not structure_ids:
            raise osv.except_osv(_('ERROR!'), _('No hay una estructura de tipo %s definida para la nomina %s')
                            %(payslip.hr_process_id.type, payslip.employee_id.hr_payroll_id.name))

        localdict['periodo'] = period

        #get the rules of the structure and their children
        rule_ids = self.pool.get('hr.payroll.structure').get_all_rules(cr, uid, structure_ids, payslip.employee_id, period.id, context=context)
        sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

        for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
            localdict['result'] = None
            localdict['result_qty'] = 1.0
            #check if the rule can be applied
            if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                key = rule.code
                #compute the amount of the rule
                amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)

                #check if there is already a rule computed with that code
                previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                #set/overwrite the amount computed for this rule in the localdict
                tot_rule = amount * qty * rate / 100.0
                localdict[rule.code] = tot_rule
                rules[rule.code] = rule

                result_dict[key] = {
                    'salary_rule_id': rule.id,
                    'name': rule.name,
                    'code': key,
                    'category_id': rule.category_id.id,
                    'sequence': rule.sequence,
                    'appears_on_payslip': rule.appears_on_payslip,
                    'condition_select': rule.condition_select,
                    'condition_python': rule.condition_python,
                    'condition_range': rule.condition_range,
                    'condition_range_min': rule.condition_range_min,
                    'condition_range_max': rule.condition_range_max,
                    'amount_select': rule.amount_select,
                    'amount_fix': rule.amount_fix,
                    'amount_python_compute': rule.amount_python_compute,
                    'amount_percentage': rule.amount_percentage,
                    'amount_percentage_base': rule.amount_percentage_base,
                    'register_id': rule.register_id.id,
                    'amount': amount,
                    'employee_id': payslip.employee_id.id,
                    'quantity': qty,
                    'rate': rate,
                    'slip_id': payslip.id,
                    'state': 'draft',
                }
                rules_dict[rule.code] = amount*qty or 0.0
                localdict['concepto'] = rules_dict

            else:
                #blacklist this rule and its children
                blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]

        result = [value for code, value in result_dict.items()]
        return result

    def onchange_employee_id(self, cr, uid, ids, date_from, date_to, employee_id=False, contract_id=False, context=None):
        empolyee_obj = self.pool.get('hr.employee')
        contract_obj = self.pool.get('hr.contract')
        worked_days_obj = self.pool.get('hr.payslip.worked_days')
        input_obj = self.pool.get('hr.payslip.input')

        if context is None:
            context = {}
        #delete old worked days lines
        old_worked_days_ids = ids and worked_days_obj.search(cr, uid, [('payslip_id', '=', ids[0])], context=context) or False
        if old_worked_days_ids:
            worked_days_obj.unlink(cr, uid, old_worked_days_ids, context=context)

        #delete old input lines
        old_input_ids = ids and input_obj.search(cr, uid, [('payslip_id', '=', ids[0])], context=context) or False
        if old_input_ids:
            input_obj.unlink(cr, uid, old_input_ids, context=context)


        #defaults
        res = {'value':{
                  'line_ids':[],
                  'input_line_ids': [],
                  'worked_days_line_ids': [],
                  #'details_by_salary_head':[], TODO put me back
                  'name':'',
                  'contract_id': False,
                  'payroll_structure_id': False,
                  }
            }
        if (not employee_id) or (not date_from) or (not date_to):
            return res
        ttyme = datetime.fromtimestamp(time.mktime(time.strptime(date_from, "%Y-%m-%d")))
        employee_id = empolyee_obj.browse(cr, uid, employee_id, context=context)
        res['value'].update({
                'name': _('Salary Slip of %s for %s') % (employee_id.get_name(cr, uid, employee_id.id), tools.ustr(ttyme.strftime('%B-%Y'))),
                'company_id': employee_id.company_id.id
        })

        if not context.get('contract', False):
            #fill with the first contract of the employee
            contract_ids = self.get_contract(cr, uid, employee_id, date_from, date_to, context=context)
            res['value'].update({
                'payroll_structure_id': contract_ids and contract_obj.read(cr, uid, contract_ids[0], ['payroll_structure_id'],
                                                        context=context)['payroll_structure_id'][0] or False,
                'contract_id': contract_ids and contract_ids[0] or False,
            })
        else:
            if contract_id:
                #set the list of contract for which the input have to be filled
                contract_ids = [contract_id]
                #fill the structure with the one on the selected contract
                contract_record = contract_obj.browse(cr, uid, contract_id, context=context)
                res['value'].update({
                            'payroll_structure_id': contract_record.payroll_structure_id.id,
                            'contract_id': contract_id
                })
            else:
                #if we don't give the contract, then the input to fill should be for all current contracts of the employee
                contract_ids = self.get_contract(cr, uid, employee_id, date_from, date_to, context=context)
                if not contract_ids:
                    return res

        #computation of the salary input
        worked_days_line_ids = self.get_worked_day_lines(cr, uid, contract_ids, date_from, date_to, context=context)
        input_line_ids = self.get_inputs(cr, uid, contract_ids, date_from, date_to, context=context)
        res['value'].update({
                'worked_days_line_ids': worked_days_line_ids,
                'input_line_ids': input_line_ids,
        })
        return res

    def onchange_contract_id(self, cr, uid, ids, date_from, date_to, employee_id=False, contract_id=False, context=None):
        #TODO it seems to be the mess in the onchanges, we should have onchange_employee => onchange_contract => doing all the things
        if context is None:
            context = {}
        res = {'value':{
             'line_ids': [],
             'name': '',
             }
              }
        context.update({'contract': True})
        if not contract_id:
            res['value'].update({'payroll_structure_id': False})
        return self.onchange_employee_id(cr, uid, ids, date_from=date_from, date_to=date_to, employee_id=employee_id, contract_id=contract_id, context=context)

hr_payslip()

class hr_payslip_worked_days(osv.osv):
    '''
    Payslip Worked Days
    '''

    _name = 'hr.payslip.worked_days'
    _description = 'Payslip Worked Days'
    _columns = {
        'name': fields.char('Description', size=256, required=True),
        'payslip_id': fields.many2one('hr.payslip', 'Pay Slip', required=True, ondelete='cascade', select=True),
        'sequence': fields.integer('Sequence', required=True, select=True),
        'code': fields.char('Code', size=52, required=True, help="The code that can be used in the salary rules"),
        'number_of_days': fields.float('Number of Days'),
        'number_of_hours': fields.float('Number of Hours'),
        'contract_id': fields.many2one('hr.contract', 'Contract', required=True, help="The contract for which applied this input"),
    }
    _order = 'payslip_id, sequence'
    _defaults = {
        'sequence': 10,
    }
hr_payslip_worked_days()

class hr_payslip_input(osv.osv):
    '''
    Payslip Input
    '''

    _name = 'hr.payslip.input'
    _description = 'Payslip Input'
    _columns = {
        'name': fields.char('Description', size=256, required=True),
        'payslip_id': fields.many2one('hr.payslip', 'Pay Slip', required=True, ondelete='cascade', select=True),
        'sequence': fields.integer('Sequence', required=True, select=True),
        'code': fields.char('Code', size=52, required=True, help="The code that can be used in the salary rules"),
        'amount': fields.float('Amount', help="It is used in computation. For e.g. A rule for sales having 1% commission of basic salary for per product can defined in expression like result = inputs.SALEURO.amount * contract.wage*0.01."),
        'contract_id': fields.many2one('hr.contract', 'Contract', required=True, help="The contract for which applied this input"),
    }
    _order = 'payslip_id, sequence'
    _defaults = {
        'sequence': 10,
        'amount': 0.0,
    }

hr_payslip_input()

class hr_salary_rule(osv.osv):

    _name = 'hr.salary.rule'
    _columns = {
        'name':fields.char('Name', size=256, required=True, readonly=False),
        'sequence': fields.integer('Sequence', required=True, help='Use to arrange calculation sequence', select=True),
        'quantity': fields.char('Quantity', size=256, help="It is used in computation for percentage and fixed amount.For e.g. A rule for Meal Voucher having fixed amount of 1€ per worked day can have its quantity defined in expression like worked_days.WORK100.number_of_days."),
        'category_id':fields.many2one('hr.salary.rule.category', 'Category'),
        'active':fields.boolean('Active', help="If the active field is set to false, it will allow you to hide the salary rule without removing it."),
        'appears_on_payslip': fields.boolean('Appears on Payslip', help="Used for the display of rule on payslip"),
        'parent_rule_id':fields.many2one('hr.salary.rule', 'Parent Salary Rule', select=True),
        'company_id':fields.many2one('res.company', 'Company', required=False),
        'condition_select': fields.selection([('none', 'Always True'),('range', 'Range'), ('python', 'Python Expression')], "Condition Based on", required=True),
        'condition_range':fields.char('Range Based on',size=1024, readonly=False, help='This will be used to compute the % fields values; in general it is on basic, but you can also use categories code fields in lowercase as a variable names (hra, ma, lta, etc.) and the variable basic.'),
        'condition_python':fields.text('Python Condition', required=True, readonly=False, help='Applied this rule for calculation if condition is true. You can specify condition like basic > 1000.'),#old name = conditions
        'condition_range_min': fields.float('Minimum Range', required=False, help="The minimum amount, applied for this rule."),
        'condition_range_max': fields.float('Maximum Range', required=False, help="The maximum amount, applied for this rule."),
        'amount_select':fields.selection([
            ('percentage','Percentage (%)'),
            ('fix','Fixed Amount'),
            ('code','Python Code'),
        ],'Amount Type', select=True, required=True, help="The computation method for the rule amount."),
        'amount_fix': fields.float('Fixed Amount', digits_compute=dp.get_precision('Payroll'),),
        'amount_percentage': fields.float('Percentage (%)', digits_compute=dp.get_precision('Payroll Rate'), help='For example, enter 50.0 to apply a percentage of 50%'),
        'amount_python_compute':fields.text('Python Code'),
        'amount_percentage_base':fields.char('Percentage based on',size=1024, required=False, readonly=False, help='result will be affected to a variable'),
        'child_ids':fields.one2many('hr.salary.rule', 'parent_rule_id', 'Child Salary Rule'),
        'register_id':fields.many2one('hr.contribution.register', 'Contribution Register', help="Eventual third party involved in the salary payment of the employees."),
        'input_ids': fields.one2many('hr.rule.input', 'input_id', 'Inputs'),
        'note':fields.text('Description'),
     }
    _defaults = {
        'amount_python_compute': '''

# Variables disponibles:
#----------------------
# payslip: object containing the payslips
# empleado: Objeto hr.employee
# edad: Edad del trabajador
# sexo: Sexo del trabajador  (Femenino / Masculino)
# contract: hr.contract object
# rules: object containing the rules code (previously computed)
# categories: object containing the computed salary rule categories (sum of amount of all rules belonging to that category).
# worked_days: object containing the computed worked days.
# inputs: object containing the computed inputs.
# nomina: objeto que contiene la nomina
# ultimo_periodo: indica si el periodo que se esta ejecutando es el ultimo del mes (verdadero o falso)
# lunes: numero de lunes en el periodo
# jornada_dia: valor de la jornada diaria
# jornada_semana: valor de la jornada semanal
# categoria_concepto: suma de totales de conceptos por categoria
# concepto: resultado del calculo de los concepto (notacion concepto['codigo_concepto'] o concepto.get('codigo_concepto',0))
# cantidad_horas: contiene la cantidad de horas por concepto (notacion cantidad_horas['codigo_concepto'] o cantidad_horas.get('codigo_concepto',0))
# pierde_descanso: cantidad de horas necesarias para perder el descanso
# parametros: parametros de las nominas (notacion parametros['nombre_parametro'])
# historial: a traves de esta variable se puede consultar el calculo de conceptos de procesos anteriores
#           (notacion historial(param, payslip, lista_de_codigos_de_conceptos, lista_de_tipos_de_procesos, num_periodos))


#CALCULOS DE VACACIONES
#sueldo_promedio: devuelve el sueldo promedio calculado en base a los dos ultimos periodos en el caso de la nomina mensual
#                 y a los 4 ultimos periodos en caso de la nomina diaria para el calculo del bono vacacional

#PAGO DE GUARDERIA
#monto_guarderia: devuelve el monto definido para cada hijo de empleado registrado en el sistema y que este en guarderia

#PAGO DE UTILIDADES
# sueldo_mensual: devuelve el sueldo mensual del empleado para la estimación de las utilidades del los meses restantes de año
# periodos_estimados: numero de meses a estimar (si se cancela utilidades en octubre se debe estimar de octubre a diciembre, es decir, 3 meses)
# parametros['utilidades']: parametro definido en la configuracion de nomina para el número de dias que se cancelan de utilidades
# acumulado_utilidades: total acumulado para pago de utilidades del empleado hasta el periodo anterio al pago (periodo cerrado)

# Note: returned value have to be set in the variable 'result'

result = contract.wage * 0.10''',
        'condition_python':
'''
# Variables disponibles:
#----------------------
# payslip: object containing the payslips
# empleado: Objeto hr.employee
# edad: Edad del trabajador
# sexo: Sexo del trabajador (Femenino / Masculino)
# contract: hr.contract object
# rules: object containing the rules code (previously computed)
# categories: object containing the computed salary rule categories (sum of amount of all rules belonging to that category).
# worked_days: object containing the computed worked days
# inputs: object containing the computed inputs
# nomina: objeto que contiene la nomina
# ultimo_periodo: indica si el periodo que se esta ejecutando es el ultimo del mes (verdadero o falso)
# lunes: numero de lunes en el periodo
# jornada_dia: valor de la jornada diaria
# jornada_semana: valor de la jornada semanal
# categoria_concepto: suma de totales de conceptos por categoria
# concepto: resultado del calculo de los concepto (notacion concepto['codigo_concepto'] o concepto.get('codigo_concepto',0))
# cantidad_horas: contiene la cantidad de horas por concepto (notacion cantidad_horas['codigo_concepto'] o cantidad_horas.get('codigo_concepto',0))
# pierde_descanso: cantidad de horas necesarias para perder el descanso
# parametros: parametros de las nominas (notacion parametros['nombre_parametro'])
# historial: a traves de esta variable se puede consultar el calculo de conceptos de procesos anteriores
#           (notacion historial(param, payslip, lista_de_codigos_de_conceptos, lista_de_tipos_de_procesos, num_periodos))

# CALCULOS DE VACACIONES
# sueldo_promedio: devuelve el sueldo promedio calculado en base a los dos ultimos periodos en el caso de la nomina mensual
#                 y a los 4 ultimos periodos en caso de la nomina diaria para el calculo del bono vacacional

# PAGO DE GUARDERIA
# monto_guarderia: devuelve el monto definido para cada hijo de empleado registrado en el sistema y que este en guarderia

# PAGO DE UTILIDADES
# sueldo_mensual: devuelve el sueldo mensual del empleado para la estimación de las utilidades del los meses restantes de año
# periodos_estimados: numero de meses a estimar (si se cancela utilidades en octubre se debe estimar de octubre a diciembre, es decir, 3 meses)
# parametros['utilidades']: parametro definido en la configuracion de nomina para el número de dias que se cancelan de utilidades
# acumulado_utilidades: total acumulado para pago de utilidades del empleado hasta el periodo anterio al pago (periodo cerrado)

# Note: returned value have to be set in the variable 'result'

result = rules.NET > categories.NET * 0.10''',
        'condition_range': 'contract.wage',
        'sequence': 5,
        'appears_on_payslip': True,
        'active': True,
        'company_id': lambda self, cr, uid, context: \
                self.pool.get('res.users').browse(cr, uid, uid,
                    context=context).company_id.id,
        'condition_select': 'none',
        'amount_select': 'fix',
        'amount_fix': 0.0,
        'amount_percentage': 0.0,
        'quantity': '1.0',
     }

    def _recursive_search_of_rules(self, cr, uid, rule_ids, context=None):
        """
        @param rule_ids: list of browse record
        @return: returns a list of tuple (id, sequence) which are all the children of the passed rule_ids
        """
        children_rules = []
        for rule in rule_ids:
            if rule.child_ids:
                children_rules += self._recursive_search_of_rules(cr, uid, rule.child_ids, context=context)
        return [(r.id, r.sequence) for r in rule_ids] + children_rules

    #TODO should add some checks on the type of result (should be float)
    def compute_rule(self, cr, uid, rule_id, localdict, context=None):
        """
        :param rule_id: id of rule to compute
        :param localdict: dictionary containing the environement in which to compute the rule
        :return: returns a tuple build as the base/amount computed, the quantity and the rate
        :rtype: (float, float, float)
        """
        rule = self.browse(cr, uid, rule_id, context=context)
        if rule.amount_select == 'fix':
            try:
                return rule.amount_fix, eval(rule.quantity, localdict), 100.0
            except:
                raise osv.except_osv(_('Error'), _('Wrong quantity defined for salary rule %s (%s)')% (rule.name, rule.code))
        elif rule.amount_select == 'percentage':
            try:
                return eval(rule.amount_percentage_base, localdict), eval(rule.quantity, localdict), rule.amount_percentage
            except:
                raise osv.except_osv(_('Error'), _('Wrong percentage base or quantity defined for salary rule %s (%s)')% (rule.name, rule.code))
        else:
            try:
                eval(rule.amount_python_compute, localdict, mode='exec', nocopy=True)
                return localdict['result'], 'result_qty' in localdict and localdict['result_qty'] or 1.0, 'result_rate' in localdict and localdict['result_rate'] or 100.0
            except:
                raise osv.except_osv(_('Error'), _('Wrong python code defined for salary rule %s (%s) ')% (rule.name, rule.code))

    def satisfy_condition(self, cr, uid, rule_id, localdict, context=None):
        """
        @param rule_id: id of hr.salary.rule to be tested
        @param contract_id: id of hr.contract to be tested
        @return: returns True if the given rule match the condition for the given contract. Return False otherwise.
        """
        rule = self.browse(cr, uid, rule_id, context=context)

        if rule.condition_select == 'none':
            return True
        elif rule.condition_select == 'range':
            try:
                result = eval(rule.condition_range, localdict)
                return rule.condition_range_min <=  result and result <= rule.condition_range_max or False
            except:
                raise osv.except_osv(_('Error'), _('Wrong range condition defined for salary rule %s (%s)')% (rule.name, rule.code))
        else: #python code
            try:
                eval(rule.condition_python, localdict, mode='exec', nocopy=True)
                return 'result' in localdict and localdict['result'] or False
            except:
                raise osv.except_osv(_('Error'), _('Wrong python condition defined for salary rule %s (%s)')% (rule.name, rule.code))

hr_salary_rule()

class hr_rule_input(osv.osv):
    '''
    Salary Rule Input
    '''

    _name = 'hr.rule.input'
    _description = 'Salary Rule Input'
    _columns = {
        'name': fields.char('Description', size=256, required=True),
        'code': fields.char('Code', size=52, required=True, help="The code that can be used in the salary rules"),
        'input_id': fields.many2one('hr.salary.rule', 'Salary Rule Input', required=True)
    }

hr_rule_input()

class hr_payslip_line(osv.osv):
    '''
    Payslip Line
    '''

    _name = 'hr.payslip.line'
    _inherit = 'hr.salary.rule'
    _description = 'Payslip Line'
    _order = 'contract_id, sequence'

    def _calculate_total(self, cr, uid, ids, name, args, context):
        if not ids: return {}
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            res[line.id] = float(line.quantity) * line.amount * line.rate / 100
        return res

    _columns = {
        'code':fields.char('Código', size=64, required=True),
        'slip_id':fields.many2one('hr.payslip', 'Pay Slip', ondelete='cascade'),
        'salary_rule_id':fields.many2one('hr.salary.rule', 'Rule'),
        'employee_id':fields.many2one('hr.employee', 'Employee', required=True),
        'contract_id':fields.many2one('hr.contract', 'Contract', required=False, select=True),
        'rate': fields.float('Rate (%)', digits_compute=dp.get_precision('Payroll Rate')),
        'amount': fields.float('Amount', digits_compute=dp.get_precision('Payroll')),
        'quantity': fields.float('Quantity', digits_compute=dp.get_precision('Payroll')),
        'total': fields.function(_calculate_total, method=True, type='float', string='Total', digits_compute=dp.get_precision('Payroll'),store=True ),
    }

    _defaults = {
        'quantity': 1.0,
        'rate': 100.0,
    }

hr_payslip_line()

class hr_employee(osv.osv):
    '''
    Employee
    '''

    _inherit = 'hr.employee'
    _description = 'Employee'

    def _calculate_total_wage(self, cr, uid, ids, name, args, context):
        if not ids: return {}
        res = {}
        current_date = datetime.now().strftime('%Y-%m-%d')
        for employee in self.browse(cr, uid, ids, context=context):
            if not employee.contract_ids:
                res[employee.id] = {'basic': 0.0}
                continue
            cr.execute( 'SELECT SUM(wage) '\
                    'FROM hr_contract '\
                    'WHERE employee_id = %s '\
                    'AND date_start <= %s '\
                    'AND (date_end > %s OR date_end is NULL)',
                     (employee.id, current_date, current_date))
            result = dict(cr.dictfetchone())
            res[employee.id] = {'basic': result['sum']}
        return res

    _columns = {
        'slip_ids':fields.one2many('hr.payslip', 'employee_id', 'Payslips', required=False, readonly=True),
        'total_wage': fields.function(_calculate_total_wage, method=True, type='float', string='Total Basic Salary', digits_compute=dp.get_precision('Payroll'), help="Sum of all current contract's wage of employee."),
    }

hr_employee()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
