#-*- coding:utf-8 -*-

from time import strftime
from datetime import *
import datetime
from dateutil import relativedelta
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import decimal_precision as dp
import logging
from tools.safe_eval import safe_eval as eval
import calendar

class hr_hours(osv.osv):

    _name = 'hr.hours'
    _description = 'Horario dia'

    _columns = {
        'name': fields.char('Nombre', size=256, required=True),
        'hour_start': fields.time('Hora entrada', required=True),
        'hour_end': fields.time('Hora salida', required=True),
    }
    _sql_constraints = [('unique_name','unique(name)', _('No pueden existir dos horarios con el mismo nombre'))]

hr_hours()


class hr_hours_per_day(osv.osv):

    _name = 'hr.hours.per.day'
    _description = 'Horario'

    _columns = {
        'name': fields.char('Nombre', size=256, required=True),
        'monday_hours_id': fields.many2one('hr.hours', 'Lunes', required=True),
        'tuesday_hours_id': fields.many2one('hr.hours', 'Martes', required=True),
        'wednesday_hours_id': fields.many2one('hr.hours', 'Miercoles', required=True),
        'thursday_hours_id': fields.many2one('hr.hours', 'Jueves', required=True),
        'friday_hours_id': fields.many2one('hr.hours', 'Viernes', required=True),
        'saturday_hours_id': fields.many2one('hr.hours', 'Sabado', required=True),
        'sunday_hours_id': fields.many2one('hr.hours', 'Domingo', required=True),
    }
    _sql_constraints = [('unique_name','unique(name)', _('No pueden existir dos horarios con el mismo nombre'))]

hr_hours_per_day()


class hr_shift(osv.osv):

    _name = 'hr.shift'
    _description = 'Turnos'

    _columns = {
        'name': fields.char('Nombre', size=256),
        'jor_week': fields.float('Jornada semanal'),
        'jor_day': fields.float('Jornada diaria'),
        'rest': fields.float('Dias promedio descanso'),
        'lose_rest': fields.float('Pierde descanso'),
        'work_hours': fields.float('Horas laborales'),  
        'employee_ids': fields.one2many('hr.employee', 'shift_id', 'Empleados'),
        'monday_hours_day_id': fields.many2one('hr.hours.per.day', 'Lunes', required=True),   
        'tuesday_hours_day_id': fields.many2one('hr.hours.per.day', 'Martes', required=True),   
        'wednesday_hours_day_id': fields.many2one('hr.hours.per.day', 'Miercoles', required=True),   
        'thursday_hours_day_id': fields.many2one('hr.hours.per.day', 'Jueves', required=True),   
        'friday_hours_day_id': fields.many2one('hr.hours.per.day', 'Viernes', required=True),   
        'saturday_hours_day_id': fields.many2one('hr.hours.per.day', 'Sabado', required=True),   
        'sunday_hours_day_id': fields.many2one('hr.hours.per.day', 'Domingo', required=True), 
    }
    _sql_constraints = [('unique_name','unique(name)', _('No pueden existir dos turnos con el mismo nombre'))]

hr_shift()

class hr_hours_per_day(osv.osv):

    _inherit = 'hr.hours.per.day'
    _description = 'Horario'

    _columns = {
        'hours_per_day_ids': fields.many2many('hr.shift', 'hr_shift_hours_rel',  'hours_id', 'shift_id','Horario por dia'),
    }

hr_hours_per_day()

class hr_rotation(osv.osv):

    _name = 'hr.rotation'
    _description = 'Rotaciones'

    _columns = {
        'name': fields.char('Nombre', size=256),
        'rotation_line_ids': fields.one2many('hr.rotation.line', 'rotation_id', 'Turnos'),
        'employee_ids': fields.one2many('hr.employee', 'rotation_id', 'Empleados')
    }
    _sql_constraints = [('unique_name','unique(name)', _('No pueden existir dos rotaciones con el mismo nombre'))]

    def next_shift_in_rotation(self, cr, uid, ids=None, context=None):
        ''' Runs through scheduler.
        Metodo usado para realizar la rotacion automatica de todos los empleados en cada rotacion al proximo turno
        '''
        if context is None:
            context = {}
         
        employee_obj = self.pool.get('hr.employee')

        employee_ids = employee_obj.search(cr, uid, [('rotation_id', '<>', False)])

        date = datetime.datetime.now()
        period_ids = self.pool.get('hr.period').search(cr, uid, [('date_start','=',date+timedelta(days=1)),('date_end','=',date+timedelta(days=7))])
        period = self.pool.get('hr.period').browse(cr, uid, period_ids)
        period_id = period[0].id if period else False
        nro_week = period[0].period_in_year if period else False
        next_shift = False
        for employee in employee_obj.browse(cr, uid, employee_ids):
            next_shift = False
            rotation_brw = employee.rotation_id
            sequence = employee.sequence
            
            rotation_line_brw = rotation_brw.rotation_line_ids
            for rotation_line in rotation_line_brw:
                if rotation_line.sequence > sequence:
                    vals_employee = {
                        'shift_id': rotation_line.shift_id.id,
                        'sequence': rotation_line.sequence,
                    }
                    context['cron'] = 1 #Si la sobreescriura del employee viene del cron no se debe generar nueva linea de rotacion
                    next_shift = employee_obj.write(cr, uid, employee.id, vals_employee, context)
                
                    vals_shift_week = {
                        'name': 'Cambio de Turno '+str(date),
                        'nro_week': nro_week,
                        'employee_id': employee.id,
                        'period_id': period_id,
                        'shift_id': rotation_line.shift_id.id,
                        'monday_hours_day_id': rotation_line.shift_id.monday_hours_day_id.id,   
                        'tuesday_hours_day_id': rotation_line.shift_id.tuesday_hours_day_id.id, 
                        'wednesday_hours_day_id': rotation_line.shift_id.wednesday_hours_day_id.id, 
                        'thursday_hours_day_id': rotation_line.shift_id.thursday_hours_day_id.id, 
                        'friday_hours_day_id': rotation_line.shift_id.friday_hours_day_id.id, 
                        'saturday_hours_day_id': rotation_line.shift_id.saturday_hours_day_id.id, 
                        'sunday_hours_day_id': rotation_line.shift_id.sunday_hours_day_id.id, 
                    }
                    shift_week = self.pool.get('hr.shift.week').create(cr, uid, vals_shift_week)
                    
                    break
                
            #Si sale del for y next_shift=False es porque no hay ningun rotation_line con prioridad mayor a la actual, es decir el turno en 
            #el que esta el empleado es el ultimo, por lo que debe regresar al primer turno de la secuencia
            
            if not next_shift:
                vals_employee = {
                    'shift_id': rotation_line_brw[0].shift_id.id,
                    'sequence': rotation_line_brw[0].sequence,
                }
                context['cron'] = 1 #Si la sobreescriura del employee viene del cron no se debe generar nueva linea de rotacion
                next_shift = employee_obj.write(cr, uid, employee.id, vals_employee, context)
                
                vals_shift_week = {
                    'name': 'Cambio de Turno '+str(date),
                    'nro_week': nro_week,
                    'employee_id': employee.id,
                    'period_id': period_id,
                    'shift_id': rotation_line_brw[0].shift_id.id,
                    'monday_hours_day_id': rotation_line_brw[0].shift_id.monday_hours_day_id.id,   
                    'tuesday_hours_day_id': rotation_line_brw[0].shift_id.tuesday_hours_day_id.id, 
                    'wednesday_hours_day_id': rotation_line_brw[0].shift_id.wednesday_hours_day_id.id, 
                    'thursday_hours_day_id': rotation_line_brw[0].shift_id.thursday_hours_day_id.id, 
                    'friday_hours_day_id': rotation_line_brw[0].shift_id.friday_hours_day_id.id, 
                    'saturday_hours_day_id': rotation_line_brw[0].shift_id.saturday_hours_day_id.id, 
                    'sunday_hours_day_id': rotation_line_brw[0].shift_id.sunday_hours_day_id.id, 
                }
                shift_week = self.pool.get('hr.shift.week').create(cr, uid, vals_shift_week)    
            
        return next_shift

hr_rotation()

class hr_rotation_line(osv.osv):

    _name = 'hr.rotation.line'
    _description = 'Lineas de Rotacion'

    _columns = {
        'shift_id': fields.many2one('hr.shift', 'Turno'),
        'name': fields.related('shift_id', 'name', type='string', relation='hr.shift', string="Nombre"),
        'rotation_id': fields.many2one('hr.rotation', 'Rotaciones'),
        'sequence': fields.integer('Secuencia', required=True, help='Usado para establer la secuencia de los turnos para el cron', select=True),
    }

    _order = 'sequence'
    _sql_constraints = [('unique_shift_rotation_sequence','unique(shift_id, rotation_id, sequence)', _('La combinacion rotacion-turno-secuencia debe ser unica')),
                        (('unique_rotation_sequence'),'unique(rotation_id, sequence)', _('No pueden existir dos turnos en una rotacion con la misma secuencia'))]

hr_rotation_line()

class hr_shift_week(osv.osv):

    _name = 'hr.shift.week'
    _description = 'Turnos trabajados por los empleados semana a semana'
    _order = 'id desc'

    _columns = {
        'name': fields.char('Nombre', size=256),
        'nro_week': fields.integer('Nro se semana', help='Corresponde al numero de semana en el año', select=True),
        'employee_id': fields.many2one('hr.employee', 'Empleado', required=True),
        'period_id': fields.many2one('hr.period', 'Periodo'),
        'shift_id': fields.many2one('hr.shift', 'Turno', required=True),
        'monday_hours_day_id': fields.many2one('hr.hours.per.day', 'Lunes', required=True),   
        'tuesday_hours_day_id': fields.many2one('hr.hours.per.day', 'Martes', required=True),   
        'wednesday_hours_day_id': fields.many2one('hr.hours.per.day', 'Miercoles', required=True),   
        'thursday_hours_day_id': fields.many2one('hr.hours.per.day', 'Jueves', required=True),   
        'friday_hours_day_id': fields.many2one('hr.hours.per.day', 'Viernes', required=True),   
        'saturday_hours_day_id': fields.many2one('hr.hours.per.day', 'Sabado', required=True),   
        'sunday_hours_day_id': fields.many2one('hr.hours.per.day', 'Domingo', required=True),
    }

hr_shift_week()

class hr_employee(osv.osv):
    '''
    Employee
    '''

    _inherit = 'hr.employee'
    _description = 'Employee'

    _columns = { 
        'shift_id': fields.many2one('hr.shift', 'Turno'),
        'rotation_id': fields.many2one('hr.rotation', 'Rotación'),
        'sequence': fields.integer('Secuencia del turno', help='Usado para mostrar la secuencia del turno \
                                                                en el que se encuentra el empleado', select=True),
        'shift_week_ids': fields.one2many('hr.shift.week', 'employee_id', 'Turnos trabajados'),
    }

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
            
        shift_id = False
        rotation_id = False
        
        if 'shift_id' in vals and 'rotation_id' in vals:
            shift_id = vals['shift_id']
            rotation_id = vals['rotation_id']

            rotation = self.pool.get('hr.rotation').browse(cr, uid, rotation_id)
            rotation_line = [rotation_line for rotation_line in rotation.rotation_line_ids if rotation_line.shift_id.id == shift_id]
            if not rotation_line:
                raise osv.except_osv(_('Acción Denegada!'), _('El turno seleccionado no pertenece a la rotacion seleccionada'))
            else:
                vals['sequence'] = rotation_line[0].sequence
                
        employee = super(hr_employee, self).create(cr, uid, vals, context)
        
        if employee and shift_id and rotation_id:
            date = datetime.datetime.now()
            period_ids = self.pool.get('hr.period').search(cr, uid, [('date_start','=',date+timedelta(days=1)),('date_end','=',date+timedelta(days=7))])
            period = self.pool.get('hr.period').browse(cr, uid, period_ids)
            period_id = period[0].id if period else False
            nro_week = period[0].period_in_year if period else False
            vals_shift_week = {
                'name': 'Primer turno empleado '+str(date),
                'nro_week': nro_week,
                'employee_id': employee,
                'period_id': period_id,
                'shift_id': shift_id,
                'monday_hours_day_id': rotation_line[0].shift_id.monday_hours_day_id.id,   
                'tuesday_hours_day_id': rotation_line[0].shift_id.tuesday_hours_day_id.id, 
                'wednesday_hours_day_id': rotation_line[0].shift_id.wednesday_hours_day_id.id, 
                'thursday_hours_day_id': rotation_line[0].shift_id.thursday_hours_day_id.id, 
                'friday_hours_day_id': rotation_line[0].shift_id.friday_hours_day_id.id, 
                'saturday_hours_day_id': rotation_line[0].shift_id.saturday_hours_day_id.id, 
                'sunday_hours_day_id': rotation_line[0].shift_id.sunday_hours_day_id.id, 
            }
            shift_week = self.pool.get('hr.shift.week').create(cr, uid, vals_shift_week)

        return True

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}

        if ('rotation_id' not in vals) and ('shift_id' not in vals):
            return super(hr_employee, self).write(cr, uid, ids, vals, context) 

        if not type(ids) is list:
            ids = [ids]

        employee_brw = self.browse(cr, uid, ids, context)

        for employee in employee_brw:
            if 'rotation_id' in vals and 'shift_id' not in vals:
                shift_brw = employee.shift_id
                rotation_id = vals['rotation_id']
                if not shift_brw:
                    raise osv.except_osv(_('Acción Denegada!'), _('Debe seleccionar tanto la rotación como el turno de inicio del empleado'))
                shift_id = shift_brw.id

            if 'shift_id' in vals and 'rotation_id' not in vals:
                rotation_brw = employee.rotation_id
                shift_id = vals['shift_id']
                if not rotation_brw:
                    raise osv.except_osv(_('Acción Denegada!'), _('Debe seleccionar tanto la rotación como el turno de inicio del empleado'))
                rotation_id = rotation_brw.id

            if 'shift_id' in vals and 'rotation_id' in vals:
                shift_id = vals['shift_id']
                rotation_id = vals['rotation_id']

            if rotation_id == False or shift_id == False:
                raise osv.except_osv(_('Acción Denegada!'), _('Debe seleccionar tanto la rotación como el turno de inicio del empleado'))

            rotation = self.pool.get('hr.rotation').browse(cr, uid, rotation_id)
            rotation_line = [rotation_line for rotation_line in rotation.rotation_line_ids if rotation_line.shift_id.id == shift_id]
            if not rotation_line:
                raise osv.except_osv(_('Acción Denegada!'), _('El turno seleccionado no pertenece a la rotacion seleccionada'))
            else:
                vals['sequence'] = rotation_line[0].sequence
            
            date = datetime.datetime.now()
            period_ids = self.pool.get('hr.period').search(cr, uid, [('date_start','=',date+timedelta(days=1)),('date_end','=',date+timedelta(days=7))])
            period = self.pool.get('hr.period').browse(cr, uid, period_ids)
            period_id = period[0].id if period else False
            nro_week = period[0].period_in_year if period else False
            
            if not 'cron' in context:
                vals_shift_week = {
                        'name': 'Cambio de Turno '+str(date),
                        'nro_week': nro_week,
                        'employee_id': employee.id,
                        'period_id': period_id,
                        'shift_id': shift_id,
                        'monday_hours_day_id': rotation_line[0].shift_id.monday_hours_day_id.id,   
                        'tuesday_hours_day_id': rotation_line[0].shift_id.tuesday_hours_day_id.id, 
                        'wednesday_hours_day_id': rotation_line[0].shift_id.wednesday_hours_day_id.id, 
                        'thursday_hours_day_id': rotation_line[0].shift_id.thursday_hours_day_id.id, 
                        'friday_hours_day_id': rotation_line[0].shift_id.friday_hours_day_id.id, 
                        'saturday_hours_day_id': rotation_line[0].shift_id.saturday_hours_day_id.id, 
                        'sunday_hours_day_id': rotation_line[0].shift_id.sunday_hours_day_id.id, 
                    }
                shift_week = self.pool.get('hr.shift.week').create(cr, uid, vals_shift_week)    
                
            super(hr_employee, self).write(cr, uid, ids, vals, context)

        return True

hr_employee()
