#!/usr/bin/python
# -*- encoding: utf-8 -*-

from time import strftime
from datetime import *
import datetime
from dateutil import relativedelta
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import logging
import base64
import os
import math

class pago_nomina_txt(osv.osv):
    _inherit = 'hr.process'
    
    _columns = {
        'date_generate_txt': fields.date('Fecha generación txt'),
        'note': fields.text('Nota'),
    }

    def generar_txt_pago_nomina(self, cr, uid, ids, context):
        process_brw = self.browse(cr, uid, ids[0], context)
        if process_brw.date_generate_txt and not process_brw.note:
            raise osv.except_osv(_('Error!'),_('El archivo txt ya fue generado por primera vez. Por favor ingrese una nota explicando porque se esta \
                                            generando de nuevo'))
        fecha = datetime.datetime.now().strftime('%Y-%m-%d')
        result_dict = {}
        for payslip in process_brw.hr_payslip_ids:
            if not payslip.employee_id.check:
                bank = payslip.employee_id.employee_account_bank_id.bank_id
                if bank not in result_dict:
                    result_dict[bank] = [payslip]
                else:
                    result_dict[bank] = result_dict[bank] + [payslip]
        id_att = False
        for key in result_dict.keys():
            if key.name.upper().find(('banco del tesoro').upper()) >= 0: # Se deben agregar tatos de estos como tipos de archivos txt se tengan         
                                                                        # (debe ser un archivo por cada banco)
                send_txt, total = self.txt_btesoro(cr, uid, ids, result_dict[key], context) 
                id_att = self._write_attachment(cr, uid, ids[0], send_txt, 'BT'+'-'+str(total), context)
        
        if id_att:
            self.write(cr, uid, ids, {'date_generate_txt':fecha}, context)
            self._logger.info("%s: Se genero el txt de pago." % (process_brw.name,))
        else:
            raise osv.except_osv(_('Error!'),_('El archivo no se genero por estar vacío. Favor revisar datos de cuantas bancarias de trabajadores'))
        return True

    def _write_attachment(self, cr, uid, ids, root, bank, context):
        '''
        Encrypt txt, save it to the db and view it on the client as an attachment
        '''
        process_brw = self.browse(cr, uid, ids, context)
        fecha = datetime.datetime.now().strftime('%Y-%m-%d')
        name = bank + '-' + process_brw.name + '-' + fecha +'.'+ 'txt'
        id_att = self.pool.get('ir.attachment').create(cr, uid, {
            'name': name,
            'voucher_support': name,
            'res_name': name,
            'datas': base64.encodestring(root),
            'datas_fname': name,
            'res_model': 'hr.process',
            'res_id': str(ids),
        }, context=context)
        return id_att

    def txt_btesoro(self, cr, uid, ids, employee_list, context):
        """Metodo del Banco del Tesoro"""
        txt_string = ''
        total = 0.0
        for payslip in employee_list:
            employee = []
            if len(payslip.employee_id.identification_id) > 9:
                raise osv.except_osv(_('Error!'),_('El número de cédula del empleado %s tiene mas dígitos de los debidos')%(payslip.employee_id.name))
            if len(payslip.employee_id.employee_account_bank_id.num_account) <> 20:
                raise osv.except_osv(_('Error!'),_('El número de cuenta bancaria del empleado %s no tiene los 20 dígitos requeridos')%(payslip.employee_id.name))
            nac = 'V' if payslip.employee_id.country_id.code == 'VE' else 'E'
            while len(nac)+len(payslip.employee_id.identification_id) < 10:
                nac = nac+'0'
            cedula = nac+payslip.employee_id.identification_id
            txt_string =  txt_string + str(cedula) + str(',')
            num_cuenta = payslip.employee_id.employee_account_bank_id.num_account
            txt_string =  txt_string + str(num_cuenta) + str(',')
            amount_allocation = sum([line.total for line in payslip.line_ids if line.type_calc == 'allocation'])
            amount_deduction = sum([line.total for line in payslip.line_ids if line.type_calc == 'deduction'])
            monto = str(round(amount_allocation - amount_deduction, 2))
            if monto.find(',') < 0 and monto.find('.') < 0: # Verificar si el monto no tiene coma o punto y colocarselo es caso de que no tenga
                monto = monto + '.'
            # Verificar que tenga dos decimales, si tienen menos de dos se agregan ceros al final para completar
            while ((monto.find(',') > 0 and (len(monto) - monto.find(',')) < 3) or (monto.find('.') > 0 and (len(monto) - monto.find('.')) < 3)):
                monto = monto + '0'
            monto = monto.replace(',','')
            monto = monto.replace('.','')
            while len(monto) < 15:
                monto = '0'+monto
            txt_string =  txt_string + str(monto) +'\n'
            total = total + round(amount_allocation - amount_deduction, 2)

        return txt_string, total
        
pago_nomina_txt()
