from report import report_sxw
from report.report_sxw import rml_parse
from datetime import datetime
import time
import logging

class LiquidacionVacacionesParser(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(LiquidacionVacacionesParser, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'time':time,
            'get_employee_holidays_dates':self.get_employee_holidays_dates,
            'get_fecha_ingreso':self.get_fecha_ingreso,
            'get_fecha_inicio':self.get_fecha_inicio,
            'get_fecha_fin':self.get_fecha_fin,
            'get_periodo':self.get_periodo
        })
        self.context = context
        self.employee_holidays_date_start = None
        self.employee_holidays_date_end = None
        self.employee_holidays_period = None
        self._logger = logging.getLogger('hr.payslip.report.holidays')

    def get_employee_holidays_dates(self, employee, process):
        holiday_id = self.pool.get('hr.employee.holidays').search(self.cr, self.uid, [('employee_id','=',employee.id),('hr_process_id','=',process.id)])
        if not holiday_id:
            self._logger.warning("No existe un registro en hr.employee.holidays para el empleado con codigo %s y proceso=%s" %(employee.cod_number,process.name))
        else:
            holiday = self.pool.get("hr.employee.holidays").browse(self.cr, self.uid, holiday_id)[0]
            self._logger.info(holiday.date_start)
            fecha = datetime.strptime(holiday.date_start,"%Y-%m-%d")
            self.employee_holidays_date_start = fecha.strftime("%d/%m/%Y")

            fecha = datetime.strptime(holiday.date_end,"%Y-%m-%d")
            self.employee_holidays_date_end = fecha.strftime("%d/%m/%Y")

            year0 = datetime.strptime(holiday.corresp_to_start,"%Y-%m-%d").strftime("%Y")
            year1 = datetime.strptime(holiday.corresp_to_end,"%Y-%m-%d").strftime("%Y")
            self.employee_holidays_period = year0 + ' al ' + year1

    def get_fecha_ingreso(self,employee):
        fecha = datetime.strptime(employee.contract_id.date_start,"%Y-%m-%d")
        return fecha.strftime("%d/%m/%Y")

    def get_fecha_inicio(self):
        return self.employee_holidays_date_start

    def get_fecha_fin(self):
        return self.employee_holidays_date_end

    def get_periodo(self):
        return self.employee_holidays_period

report_sxw.report_sxw(
                      'report.hr.payslip.liquidacion.vacaciones',
                      'hr.payslip',
                      'sisb_nomina/report/report_liquidacion_vacaciones.rml',
                      parser=LiquidacionVacacionesParser, header=False)