#!/usr/bin/env python
#-*- coding:utf-8 -*-

##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    d$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from report import report_sxw
from tools import amount_to_text_en

class payslip_report(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(payslip_report, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'get_payslip_lines': self.get_payslip_lines,
            'get_utilidades': self.get_utilidades_acum,
            'get_faov': self.get_faov_acum,
            'get_sso': self.get_sso_acum,
            'get_islr': self.get_islr_acum,
            'get_rpe': self.get_rpe_acum,
            'get_saldo': self.get_saldo_prestamo,
            'get_devengado': self.get_devengado_acum,
            'get_allocation': self.get_allocation,
            'get_deduction': self.get_deduction,
            'get_salary': self.get_salary,
        })

    def get_payslip_lines(self, obj):
        res = []
        for id in range(15):
            if id < len(obj.line_ids):
                res.append((obj.line_ids[id].name, obj.line_ids[id].quantity, obj.line_ids[id].total, obj.line_ids[id].type_calc))
            else:
                res.append(('','','','allocation'))
        return res

    def get_utilidades_acum(self, obj):
        return obj.utilidades_acumulado or 0.00

    def get_sso_acum(self, obj):
        return obj.sso_acumulado or 0.00

    def get_faov_acum(self, obj):
        return obj.faov_acumulado or 0.00

    def get_islr_acum(self, obj):
        return obj.islr_acumulado or 0.00

    def get_rpe_acum(self, obj):
        return obj.rpe_acumulado or 0.00

    def get_devengado_acum(self, obj):
        return obj.devengado_acumulado or 0

    def get_saldo_prestamo(self, obj):
        return obj.saldo_prestamo or 0

    def get_allocation(self, obj):
        res = ""
        if obj[3] == 'allocation':
            res = obj[2]
        return res

    def get_deduction(self, obj):
        res = ""
        if obj[3] == 'deduction':
            res = obj[2]
        return res
        
    def get_salary(self, obj):
        res = ""
        employee_id = obj.employee_id
                
        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina
        parameter_obj = self.pool.get('hr.parameters')
        param_concept_salary_id = parameter_obj.search(self.cr, self.uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', employee_id.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(self.cr, self.uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo Concepto Salario')
                                                %(employee_id.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(self.cr, self.uid, param_concept_salary_id)[0].concept_wage_id
                
        attr_ids = self.pool.get('hr.attribute').search(self.cr, self.uid, [('employee_id', '=', employee_id.id),('concept_id','=',concept_salary_brw.id),
                                                    ('date_start', '<=', obj.date_to),'|',('date_end', '>=', obj.date_from),('date_end', '=', False),
                                                    '|',('active', '=', True),('active','=',False)], 
                                                            order = 'date_start desc')
        if not isinstance(attr_ids, list):
            attr_ids = [attr_ids]
            
        attr_brw = self.pool.get('hr.attribute').browse(self.cr, self.uid, attr_ids[0])
        
        return attr_brw.amount

report_sxw.report_sxw('report.hoja.pago', 'hr.payslip', 'sisb_nomina/report/report_payslip.rml', parser=payslip_report)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
