# -*- coding: utf-8 -*-


from osv import fields,osv
import tools

class report_payslip_line(osv.osv):
    _name = "report.payslip.line"
    _description = "Payslip Lines"
    _auto = False
    _columns = {
        'state': fields.selection([('draft','Borrador'), ('load','Cargado'), ('execute','En ejecucion'),
                                 ('review','Procesado'), ('done','Aprobado'), ('cancel','Cancelado')],
                                 'Estado', readonly=True),
        'employee_id': fields.many2one('hr.employee', 'Empleado', readonly=True),
        'gender': fields.selection([
                ('male', 'Masculino'),
                ('female', 'Femenino'),
            ], 'Sexo'),
        'department_id': fields.many2one('hr.department', 'Departamento', readonly=True),
        'concept_id': fields.many2one('hr.concept', 'Concepto', readonly=True),
        'type': fields.selection([
                ('fijo', 'Fijo'),
                ('variable', 'Variable'),
                ('aporte_patronal', 'Aporte patronal'),
            ], "Tipo de concepto", readonly=True),
        'type_calc': fields.selection([
                ('allocation', 'Asignacion'),
                ('deduction', 'Deduccion')
            ], "Tipo de cálculo", readonly=True),
        'appears_on_payslip': fields.boolean('Aparece en hoja de pago'),
        'payroll_id': fields.many2one('hr.payroll', 'Nomina', readonly=True),
        'process_id': fields.many2one('hr.process', 'Proceso', readonly=True),
        'type_process': fields.selection([
                ('nomina', 'Nómina'),
                ('vacaciones', 'Vacaciones'),
                ('utilidades', 'Utilidades'),
                ('prestaciones', 'Prestaciones sociales'),
                ('guarderias', 'Pago de guarderías'),
                ('anticipos', 'Pago de anticipos'),
                ('bono_alimentacion', 'Pago bono alimentación'),
                ('pago_especial', 'Pagos especiales'),
            ], "Tipo de proceso", readonly=True),
        'sede_id': fields.many2one('hr.sede', 'Sede', readonly=True),
        'period_id': fields.many2one('hr.period', 'Periodo', readonly=True),
        'month_id': fields.many2one('hr.month', 'Mes', readonly=True),
        'quantity': fields.float('Cantidad', digits=(16,2), readonly=True),
        'total': fields.float('Total', digits=(16,2), readonly=True),
    }
    _order = 'month_id desc,period_id desc,department_id,concept_id'
        
    def init(self, cr):
        tools.sql.drop_view_if_exists(cr, 'report_payslip_line')
        cr.execute("""
            create or replace view report_payslip_line as (
                select hpl.id as id,
                    hp.state as state,
                    he.id as employee_id,
                    he.gender as gender, 
                    hd.id as department_id, 
                    hc.id as concept_id, 
                    hpl.type_calc as type_calc, 
                    hc.type as type,
                    hpl.appears_on_payslip as appears_on_payslip,
                    hpr.id as payroll_id,
                    hs.id as sede_id,
                    hp.id as process_id, 
                    hp.type as type_process, 
                    hpe.id as period_id, 
                    hm.id as month_id,
                    hpl.quantity as quantity, 
                    hpl.total as total
                from hr_payslip_line hpl
                    join hr_employee he on he.id = hpl.employee_id
                    join hr_department_employee hde on he.id = hde.employee_id
                    join hr_department hd on hd.id = he.department_id
                    join hr_concept hc on hc.id = hpl.concept_id
                    join hr_process hp on hp.id = hpl.process_id
                    join hr_period_process_rel hppr on hppr.process_id = hp.id
                    join hr_period hpe on hpe.id = hppr.period_id
                    join hr_payroll hpr on hpr.id = hde.id_payroll
                    join hr_sede hs on hs.name = hde.sede
                    join hr_payroll_calendar_rel hpcr on hpcr.hr_payroll_id = hpr.id
                    join hr_payment_calendar hpcal on hpcal.id = hpcr.hr_calendar_id
                    and hpcal.id = hpe.hr_payment_calendar_id
                    join hr_month hm on hm.id = hpe.hr_month_id
                    where 
                    case
                        when hde.date_end is not null and hpl.write_date is not null then
                        hpl.write_date between hde.date_start and hde.date_end
                        when hde.date_end is null and hpl.write_date is not null then
                        hpl.write_date >= hde.date_start
                        when hde.date_end is not null and hpl.write_date is null then
                        hpl.create_date between hde.date_start and hde.date_end
                        else hpl.create_date >= hde.date_start
                    end
            )
        """)
report_payslip_line()
