from report import report_sxw
from report.report_sxw import rml_parse
from operator import itemgetter
import time

class ProcessConceptsParser(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(ProcessConceptsParser, self).__init__(cr, uid, name, 
          context)
        self.localcontext.update({
            'report_lines': self.get_report_lines,
            'total_asignaciones': self.get_total_asignaciones,
            'total_deducciones': self.get_total_deducciones,
            'company': self.get_company,
            'rif': self.get_rif,
            'time': time
        })
        self.context = context
        self.total_asignaciones = 0
        self.total_deducciones = 0
        self.result = []

    def get_company(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, 
          self.uid).company_id.name

    def get_rif(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, 
          self.uid).company_id.partner_id.vat

    def get_report_lines(self, process):
        self.total_asignaciones = 0
        self.total_deducciones = 0
        self.result = []
        cpt_obj = self.pool.get('hr.concept')
        #aportes patronales
        conceptos_aportes_patronales_ids = cpt_obj.search(self.cr,
          self.uid, [('type', '=', 'aporte_patronal')])
          
        codigos_aportes_patronales = []
        conceptos_aportes_patronales = []
        
        if conceptos_aportes_patronales_ids:
            conceptos_aportes_patronales = cpt_obj.browse(self.cr, 
              self.uid, conceptos_aportes_patronales_ids)
            codigos_aportes_patronales = [ap.code \
              for ap in conceptos_aportes_patronales]

        # Se buscan los payslip.lines que son visibles en los payslips
        payslips = [payslip for payslip in process.hr_payslip_ids]
        payslip_lines_visibles = []
        for payslip in payslips:
            payslip_lines_visibles.extend([payslip_line.id \
              for payslip_line in payslip.line_ids])

        payslip_line_obj = self.pool.get('hr.payslip.line')
        
        # Primero se obtienen las asignaciones
        payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
          [('process_id', '=', process.id),
          ('id', 'in', payslip_lines_visibles), 
          ('type_calc', '=', 'allocation')])
          
        payslip_lines = payslip_line_obj.browse(self.cr, self.uid, 
          payslip_lines_ids)
          
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            if not (code, concept) in result:
                result[(code, concept)] = total
            else:
                result[(code, concept)] += total
            self.total_asignaciones += total
            
        self.result.extend([(code, concept, 'allocation', 
          result[(code, concept)]) for (code, concept) in result])
          
        # Luego se obtienen las deducciones
        payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
          [('process_id','=', process.id),
          ('id', 'in', payslip_lines_visibles),
          ('type_calc','=', 'deduction')])
        
        payslip_lines = payslip_line_obj.browse(self.cr, self.uid, 
          payslip_lines_ids)
          
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            if not (code, concept) in result:
                result[(code, concept)] = total
            else:
                result[(code, concept)] += total
            self.total_deducciones += total
            
        self.result.extend([(code, concept, 'deduction', 
          result[(code, concept)]) for (code, concept) in result])

        payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
          [('process_id', '=', process.id),('code', 'in', 
          codigos_aportes_patronales)])
        
        payslip_lines = payslip_line_obj.browse(self.cr, self.uid, 
          payslip_lines_ids)
          
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            
            if not (code, concept) in result:
                result[(code, concept)] = total
            else:
                result[(code, concept)] += total
                
        self.result.extend([(code, concept, 'allocation', 
          result[(code, concept)]) for (code, concept) in result])
        return self.result

    def get_total_asignaciones(self):
        return self.total_asignaciones

    def get_total_deducciones(self):
        return self.total_deducciones
    
report_sxw.report_sxw(
                      'report.hr.process.concepts',
                      'hr.process',
                      'sisb_nomina/report/report_process_concepts.rml',
                      parser=ProcessConceptsParser, header=False
)
