from report import report_sxw
from report.report_sxw import rml_parse
from operator import itemgetter
import time
from datetime import datetime
import random

class ProcessPayslipsParser(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(ProcessPayslipsParser, self).__init__(cr, uid, name, context)
        self.localcontext.update({
            'company':self.company,
            'rif':self.rif,
            'set_info_trabajadores':self.set_info_trabajadores,
            'info_trabajadores':self.info_trabajadores,
            'departamentos_sede':self.departamentos_sede,
            'nominas_proceso':self.nominas_proceso,
            'sedes_nomina':self.sedes_nomina,
            'sedes_proceso':self.sedes_proceso,
            'periodo_nomina':self.periodo_nomina,
            'time':time,
            'sumar_asignacion_sede':self.sumar_asignacion_sede,
            'sumar_deduccion_sede':self.sumar_deduccion_sede,
            'total_asignaciones_sede':self.get_total_asignaciones_sede,
            'total_deducciones_sede':self.get_total_deducciones_sede,
            'reset_asignaciones_deducciones_sede':self.reset_asignaciones_deducciones_sede,
            'calc_total_asignaciones_trabajador':self.calc_total_asignaciones_trabajador,
            'calc_total_deducciones_trabajador':self.calc_total_deducciones_trabajador,
            'total_asignaciones_trabajador':self.total_asignaciones_trabajador,
            'total_deducciones_trabajador':self.total_deducciones_trabajador,
            'total_neto_trabajador': self.total_neto_trabajador,
            'lineas': self.lineas,
        })
        self.context = context
        self.total_asignaciones_trabajador = 0
        self.total_deducciones_trabajador = 0
        self.total_asignaciones_sede = 0
        self.total_deducciones_sede = 0
        self.info_trabajadores = []  #Trabajadores del proceso
        self.sedes = set()
        self.nominas = set()
        self.nomina_sedes = {}
        self.sede_departamentos = {}
        self.nomina_id = {}
        

    def company(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, self.uid).company_id.name

    def rif(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, self.uid).company_id.partner_id.vat[2:]

    def periodo_nomina(self, process, nomina):
        id_nomina =  self.nomina_id[nomina]
        payroll = self.pool.get('hr.payroll').browse(self.cr, self.uid, id_nomina)
        p_id = [p for p in process.hr_period_ids]
        prl_period_ids = [ p for p in payroll.hr_payment_calendar_id.hr_period_ids]
        period = set(p_id).intersection( set(prl_period_ids) )
        period = period.pop()
        return period.date_start[8:11] + '/' + period.date_start[5:7] + '/' + period.date_start[:4] + " al " + period.date_end[8:11] + '/' + period.date_end[5:7] + '/' + period.date_end[:4]

    def _get_datos_historicos(self, process, payslip):
        """Se devuelve en un diccionario los datos historicos del trabajador
        (sede, departamento, cargo y nomina). Estos datos se corresponden con
        los que estaban vigentes en el momento que se proceso la nomina"""
        historico = {}
        procesado = datetime.strptime(process.date_review,"%Y-%m-%d")
        movimientos_ids = self.pool.get('hr.department.employee').search(self.cr, self.uid, [('employee_id','=',payslip.employee_id.id)])
        movimientos = self.pool.get('hr.department.employee').browse(self.cr, self.uid, movimientos_ids)
        for movimiento in movimientos:
            inicio = datetime.strptime(movimiento.date_start,"%Y-%m-%d")
            fin = movimiento.date_end
            if not fin:
                fin = datetime.now()
                fin = fin.strftime("%Y-%m-%d")
                fin = datetime.strptime(fin, "%Y-%m-%d")
            else:
                fin = datetime.strptime(fin,"%Y-%m-%d")
            if inicio <= procesado <= fin:
                historico['sede'] = movimiento.sede or ''
                historico['departamento'] = movimiento.department_id or ''
                historico['cargo'] = movimiento.job or ''
                historico['nomina'] = movimiento.payroll or ''
                historico['id_nomina'] =  movimiento.id_payroll or ''
                return historico

    def _get_salary(self, process, payslip):
        """Determinar el salario que se pagaba al empleado en la fecha en que
        se proceso la nomina"""
        parameter_obj = self.pool.get('hr.parameters')
        param_concept_salary_id = parameter_obj.search(self.cr, self.uid, [('type', '=', 'concepto_sueldo')])
        if not param_concept_salary_id:
            return 0
        else:
            concept_salary = parameter_obj.browse(self.cr, self.uid, param_concept_salary_id)[0].concept_wage_id
            procesado = datetime.strptime(process.date_review,"%Y-%m-%d")
            atributos_ids = self.pool.get('hr.attribute').search(self.cr, self.uid,[('concept_id','=',concept_salary.id),('employee_id','=',payslip.employee_id.id)])
            atributos = self.pool.get('hr.attribute').browse(self.cr, self.uid, atributos_ids)
            for atributo in atributos:
                inicio = datetime.strptime(atributo.date_start,"%Y-%m-%d")
                fin = atributo.date_end
                if not fin:
                    fin = datetime.now()
                    fin = fin.strftime("%Y-%m-%d")
                    fin = datetime.strptime(fin, "%Y-%m-%d")
                else:
                    fin = datetime.strptime(fin,"%Y-%m-%d")

                if inicio <= procesado <= fin:
                    return atributo.amount
            return 0

    def set_departamentos_sede(self, nomina, sede, departamento):
        if not (nomina, sede) in self.sede_departamentos:
            self.sede_departamentos.update({(nomina,sede):[departamento]})
        else:
            departamentos = set(self.sede_departamentos[(nomina,sede)])
            departamentos.add(departamento)
            self.sede_departamentos.update({(nomina,sede):departamentos})
        
    def set_nomina_sedes(self, nomina, sede):
        if not nomina in self.nomina_sedes:
            self.nomina_sedes.update({nomina:[sede]})
        else:
            sedes = set(self.nomina_sedes[nomina])
            sedes.add(sede)
            self.nomina_sedes.update({nomina:sedes})

    def set_nomina_id(self, nomina, id):
        if not nomina in self.nomina_id:
            self.nomina_id.update({nomina: id})

    def set_info_trabajadores(self, process):
        info = {}
        payslips = [payslip for payslip in process.hr_payslip_ids]
        for payslip in payslips:
            """ Datos fijos"""
            codigo_empleado = payslip.employee_id.cod_number
            nombres = payslip.employee_id.first_name  + " "
            nombres += payslip.employee_id.second_name or ""
            apellidos = payslip.employee_id.first_surname + " "
            apellidos += payslip.employee_id.second_surname or ""
            cedula = payslip.employee_id.identification_id
            fecha_ingreso_empresa = str(payslip.employee_id.contract_id.date_start)

            """Datos Historicos"""
            # Cargo, sede, departamento y nomina se obtienen desde hr_department_employee
            historico = self._get_datos_historicos(process, payslip)
            sede = historico['sede']
            self.sedes.add(sede)
            departamento = historico['departamento']
            nomina = historico['nomina']
            self.set_departamentos_sede(nomina, sede, departamento)
            cargo = historico['cargo']
            id_nomina = historico['id_nomina']
            self.set_nomina_id(nomina, id_nomina)
            self.nominas.add(nomina)
            self.set_nomina_sedes(nomina, sede)
            salario = self._get_salary(process, payslip)
            info.update({(nomina, sede, departamento, codigo_empleado):[nombres, apellidos, cedula, cargo, salario, fecha_ingreso_empresa, payslip]})

        self.info_trabajadores = [(key[0], key[1], key[2], key[3], info[key][0], info[key][1], info[key][2], info[key][3], info[key][4], info[key][5], info[key][6]) for key in info.iterkeys()]
        #Se organizan nomina, sede, departamento y codigo de empleado
        self.info_trabajadores.sort(key=itemgetter(0,1,2,3))

    def nominas_proceso(self):
        return self.nominas

    def sedes_proceso(self):
        return self.sedes

    def sedes_nomina(self, nomina):
        return self.nomina_sedes[nomina]

    def departamentos_sede(self, nomina, sede):
        return self.sede_departamentos[(nomina,sede)]
    
    def sumar_asignacion_sede(self, asignacion):
        self.total_asignaciones_sede += asignacion

    def sumar_deduccion_sede(self, deduccion):
        self.total_deducciones_sede += deduccion

    def get_total_asignaciones_sede(self):
        return self.total_asignaciones_sede

    def get_total_deducciones_sede(self):
        return self.total_deducciones_sede

    def calc_total_asignaciones_trabajador(self, payslip):
        self.total_asignaciones_trabajador = sum([pl.total for pl in payslip.line_ids if pl.type_calc == 'allocation'])

    def calc_total_deducciones_trabajador(self, payslip):
        self.total_deducciones_trabajador = sum([pl.total for pl in payslip.line_ids if pl.type_calc == 'deduction'])

    def total_asignaciones_trabajador(self):
        return self.total_asignaciones_trabajador

    def total_deducciones_trabajador(self):
        return self.total_deducciones_trabajador

    def total_neto_trabajador(self):
        return self.total_asignaciones_trabajador - self.total_deducciones_trabajador

    def reset_asignaciones_deducciones_sede(self):
        self.total_asignaciones_sede = 0
        self.total_deducciones_sede = 0

    def info_trabajadores(self, nomina, sede, departamento):
        trabajadores = [info_trabajador for info_trabajador in self.info_trabajadores if info_trabajador[0] == nomina
                        and info_trabajador[1] == sede and info_trabajador[2] == departamento]
        return trabajadores
    
    def lineas(self, lines):
        l = [x for x in lines]
        codes = [x.code for x in lines]
        c = self.sort_string(codes)
        l.sort(key=lambda x: c.index(x.code))
        return l

    def sort_string(self, s):
        def sort_string_to_a_list(s):
            return sorted(s, lambda x,y: cmp(x.lower(), y.lower()) or cmp(x,y))
        sorted_list = sort_string_to_a_list(s)
        return sorted_list


report_sxw.report_sxw(
                      'report.hr.process.payslips',
                      'hr.process',
                      'sisb_nomina/report/report_process_payslips.rml',
                      parser=ProcessPayslipsParser, header=False
)
