# -*- coding: utf-8 -*-
##############################################################################
#
# Facturar pago de guarderias
#
##############################################################################

from osv import fields, osv
from tools.translate import _


class guarderia_invoice(osv.osv_memory):
    
    _name = "guarderia.invoice"
    _description = "Facturas pago guarderias"
    
    def _get_journal_id(self, cr, uid, context=None):
        if context is None:
            context = {}

        model = context.get('active_model')
        if not model or model != 'hr.payslip':
            return []

        model_pool = self.pool.get(model)
        journal_obj = self.pool.get('account.journal')
        res_ids = context and context.get('active_ids', [])
        vals = []
        browse_payslips = model_pool.browse(cr, uid, res_ids, context=context)
        
        for hc in browse_payslips:
            journal_type = ('purchase',)
            value = journal_obj.search(cr, uid, [('type','in',journal_type )])
            for jr_type in journal_obj.browse(cr, uid, value, context=context):
                t1 = jr_type.id,jr_type.name
                if t1 not in vals:
                    vals.append(t1)
        if not vals:
            raise osv.except_osv(_('Mensaje !'), _('No hay diario configurado para facturas de proveedor!'))
        
        return vals
        
    _columns = {
        'journal_id': fields.selection(_get_journal_id, 'Diario destino',required=True),
        'group': fields.boolean("Agrupado por hoja de pago"),
        'invoice_date': fields.date('Fecha de la Factura'),
    }
    
    def view_init(self, cr, uid, fields_list, context=None):
        if context is None:
            context = {}
        res = super(guarderia_invoice, self).view_init(cr, uid, fields_list, context=context)
        payslip_obj = self.pool.get('hr.payslip')
        count = 0
        active_ids = context.get('active_ids',[])
        #Solo se crean facturas para las hojas de pago de procesos de pago guarderia
        sucursal_ant = False
        for ps in payslip_obj.browse(cr, uid, active_ids, context=context):
            if ps.type_process <> 'guarderias':
                raise osv.except_osv(_('Mensaje!'), _('Solo se crean facturas para las hojas de pago de procesos de pago de guarderias.\
                                                    La hoja de pago %s es de tipo %s'%(ps.name, ps.type_process)))
            if ps.state <> 'done':
                raise osv.except_osv(_('Mensaje!'), _('Solo se crean facturas para las hojas de pago en estado Aprobado.\
                                                    La hoja de pago %s esta en estado %s'%(ps.name, ps.state)))
        return res
        
    def open_invoice(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        invoice_ids = []
        data_pool = self.pool.get('ir.model.data')
        res = self.create_invoice(cr, uid, ids, context=context)
        invoice_ids += res.values()
        inv_type = context.get('inv_type', False)
        action_model = False
        action = {}
        if not invoice_ids:
            raise osv.except_osv(_('Error'), _('No se pudo crear factura'))
        if inv_type == "in_invoice":
            action_model,action_id = data_pool.get_object_reference(cr, uid, 'account', "action_invoice_tree2")
        else:
            raise osv.except_osv(_('Error'), _('No se puede crear facturas que no sean al proveedor'))
        if action_model:
            action_pool = self.pool.get(action_model)
            action = action_pool.read(cr, uid, action_id, context=context)
            action['domain'] = "[('id','in', ["+','.join(map(str,invoice_ids))+"])]"
        return action
        
    def create_invoice(self, cr, uid, ids, context=None):
        journal_mapping_type_inv = {
            'sale': 'out_invoice',
            'purchase': 'in_invoice',
            'sale_refund': 'out_refund',
            'purchase_refund': 'in_refund',
        }
        if context is None:
            context = {}
        payslip_pool = self.pool.get('hr.payslip')
        invoice_data_obj = self.read(cr, uid, ids, ['journal_id', 'group', 'invoice_date'])
        journal_pool = self.pool.get('account.journal')

        context['date_inv'] = invoice_data_obj[0]['invoice_date']
        active_ids = context.get('active_ids', [])
        message = 'Ya se generaron facturas para las hojas de pago de los trabajadores: '
        inv_exist = False
        for payslip in payslip_pool.browse(cr, uid, active_ids):
            if payslip.account_invoice_ids:
                message = message + payslip.employee_id.name + ', '
                inv_exist = True
        if inv_exist:
            raise osv.except_osv(_('Error'), _(message))
        
        active_hoja_carga = payslip_pool.browse(cr, uid, context.get('active_id',False), context=context)
        journal_type = journal_pool.browse(cr, uid, invoice_data_obj[0]['journal_id']).type
        inv_type = journal_mapping_type_inv[journal_type]
        context['inv_type'] = inv_type
        res = payslip_pool.action_invoice_create(cr, uid, active_ids,
              journal_id = invoice_data_obj[0]['journal_id'],
              group = invoice_data_obj[0]['group'],
              type = inv_type,
              context=context)
        
        return res

guarderia_invoice()

class hr_payslip(osv.osv):
    
    _inherit = 'hr.payslip'
    _description = 'Pay Slip'

    _columns = {
        'account_invoice_ids': fields.one2many('account.invoice', 'payslip_id', 'Facturas para pago de guarderias'),
    }
    
    def _get_address_invoice(self, cr, uid, partner_id):
        """ Gets invoice address of a partner
        @return {'contact': address, 'invoice': address} for invoice
        """
        partner_obj = self.pool.get('res.partner')
        return partner_obj.address_get(cr, uid, [partner_id.id],
                ['contact', 'invoice'])
                
    def new_invoice(self, cr, uid, payslip, partner, lines, journal_id, type='in_invoice', context=None):
        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        address_obj = self.pool.get('res.partner.address')
        obj_hr_acct = self.pool.get('hr.account')
        
        payment_term_id = False
        address_contact_id, address_invoice_id = \
                    self._get_address_invoice(cr, uid, partner).values()
        invoice_vals = {
            'name': payslip.name,
            'origin': (payslip.name or ''),
            'type': type,
            'account_id': partner.property_account_payable.id,
            'partner_id': partner.id,
            'address_invoice_id': address_invoice_id,
            'address_contact_id': address_contact_id,
            'comment': 'Factura generada desde el asistente de nomina para pago de guarderias',
            'payment_term': payment_term_id,
            'fiscal_position': partner.property_account_position.id,
            'date_invoice': context.get('date_inv',False),
            'company_id': payslip.employee_id.company_id.id,
            'user_id': uid,
            'process_id': payslip.hr_process_id.id,
            'payslip_id': payslip.id,
        }
        
        if journal_id:
            invoice_vals['journal_id'] = journal_id
        invoice_id = invoice_obj.create(cr, uid, invoice_vals, context=context)
        
        for l in lines:
            concept = l.salary_rule_id.concept_id if l.salary_rule_id else l.attribute_id.concept_id
            department = l.employee_id.department_id
            payroll = l.employee_id.hr_payroll_id
            hr_account_id = obj_hr_acct.search(cr, uid, 
                      [('concept_id', '=', concept.id),
                      ('analytic_account_id', '=', department.departament_analytic_account_id.id),
                      ('payroll_id', '=', payroll.id)])
            account = obj_hr_acct.browse(cr, uid, hr_account_id)[0]
            
            invoice_line_id = invoice_line_obj.create(cr, uid, {
                    'name': l.name,
                    'origin': (payslip.name or ''),
                    'invoice_id': invoice_id,
                    'uom_id': False,
                    'product_id': False,
                    'account_id': account.account_id.id,
                    'price_unit': l.total,
                    'discount': 0.0,
                    'quantity': 1,
                    'invoice_line_tax_id': False,
                    'account_analytic_id': account.analytic_account_id.id,
                    
                }, context=context)

        invoice_obj.button_compute(cr, uid, [invoice_id], context=context,
                set_total=(type in ('in_invoice', 'in_refund')))
        
        return invoice_id
                    
    def action_invoice_create(self, cr, uid, ids, journal_id=False, group=False, type='in_invoice', context=None):
        """ Crea las facturas de para pago de guarderias
        @param journal_id: Id del diario
        @param group: Si se agrupa por hoja de pago
        @param type: Tipo de factura que debe crearse
        @return: Ids de la factura creada para la(s) hoja(s) de pago
        """
        obj_hr_acct = self.pool.get('hr.account')
        
        if context is None:
            context = {}

        res = {}
        invoices_group = {}
        invoices = {}
        inv_type = type
                  
        for slip in self.browse(cr, uid, ids, context=context):
                    
            partners = {}
            payment_term_id = False
            lines = []
            for slip_line in slip.line_ids:
                """ Comprobar configuracion contable"""
                concept = slip_line.salary_rule_id.concept_id if slip_line.salary_rule_id else slip_line.attribute_id.concept_id
                department = slip_line.employee_id.department_id
                payroll = slip_line.employee_id.hr_payroll_id
                
                account_id = obj_hr_acct.search(cr, uid, 
                    [('concept_id', '=', concept.id),
                    ('analytic_account_id', '=', department.departament_analytic_account_id.id),
                    ('payroll_id', '=', payroll.id)])

                if not account_id:
                    raise osv.except_osv(_('Error!'), 
                        _('No existe una configuración contable para el concepto\
                        %s, cuenta analítica %s y nómina %s. Para verificar ir a\
                        Talento Humano -> Configuración -> Contabilidad nómina.')
                        % (concept.name, department.departament_analytic_account_id.complete_name, 
                        payroll.name))
                        
                if (slip, slip_line.employee_family_id.guarderia_id.partner_id) in partners:
                    lista = partners[slip, slip_line.employee_family_id.guarderia_id.partner_id] + [slip_line]
                    partners[slip, slip_line.employee_family_id.guarderia_id.partner_id] =  lista
                else:
                    partners[slip, slip_line.employee_family_id.guarderia_id.partner_id] = [slip_line]

            invoices[slip] = partners
            
        for s in invoices:
            payslip = s
            if group:
                for inv in invoices[s]:
                    partner = inv[1]
                    lines_inv = invoices[s][inv]
                    new_inv = self.new_invoice(cr, uid, payslip, partner, lines_inv, journal_id, type, context)
                    res[payslip.id, new_inv] = new_inv
            else:
                for inv in invoices[s]:
                    partner = inv[1]
                    lines_inv = invoices[s][inv]
                    for line in lines_inv:
                        new_inv = self.new_invoice(cr, uid, payslip, partner, [line], journal_id, type, context)
                        res[payslip.id, new_inv] = new_inv
                        
        return res

hr_payslip()

class account_invoice(osv.osv):

    _inherit = "account.invoice"
    _description = "Account Invoice"

    _columns = {
        'process_id': fields.many2one('hr.process', 'Proceso de Nómina'),
        'payslip_id': fields.many2one('hr.payslip', 'Hoja de pago'),
    }

account_invoice()
