#-*- coding:utf-8 -*-

import netsvc
import time
from tools.translate import _
from osv import fields, osv
import calendar
from datetime import date
from datetime import datetime
from datetime import timedelta
import unicodedata

def elimina_tildes(s):
    """
    Funcion para eliminar las tildes de algun texto utilizando el modulo unicodedata.
    """
    return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))
    
class hr_salary_increase(osv.osv_memory):
    
    _name = 'hr.salary.increase'
    _description = 'Salary Increase'
    
    _columns = {
        'type_increase': fields.selection(
                            [('employee', 'Por empleado'),
                            ('sede', 'Por sede'),
                            ('department', 'Por departamento'),
                            ('all', 'Todos los empleados'),
                            ], 'Tipo de incremento'),
        'employee_id': fields.many2one('hr.employee', 'Empleado'),
        'sede_id': fields.many2one('hr.sede', 'Sede'),
        'department_ids': fields.many2many('hr.department', 'hr_salary_increase_department_rel', 'hr_increase_id', 'hr_department_id', 'Departamentos'),
        'payroll_id': fields.many2one('hr.payroll', 'Nómina'),
        'type_amount': fields.selection(
                            [('percentage', 'Porcentaje'),
                            ('amount', 'Monto'),
                            ], 'Tipo monto'),
        'amount': fields.float('', digits=(16,2)),
            
    }
    
    _defaults = {
        'type_increase': 'employee',
        'type_amount': 'percentage',
    }
    
    def _check_amount(self, cr, uid, ids, context=None):
        for hr_salary_increase in self.browse(cr, uid, ids, context=context):
            if hr_salary_increase.amount <= 0.0:
                return False
        return True

    _constraints = [(_check_amount, "El monto debe ser mayor que 0", ['amount'])]
    
    def make_increase(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        context.update({'wizard_incremento':True})
        attribute_obj = self.pool.get('hr.attribute')
        attributes = self.get_attributes(cr, uid, ids, context)
        
        nuevos_attr_ids = []
        for attrs in attributes:
            attrs_id = attribute_obj.create(cr, uid, attrs, context)
            nuevos_attr_ids.append(attrs_id)
        
        parameter_obj = self.pool.get('hr.parameters')
        param_correos_inc_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'correos_inc_salary')])
        if param_correos_inc_salary_id:
            nuevos_attr_brw = attribute_obj.browse(cr, uid, nuevos_attr_ids)
            correos_inc_salary_brw = parameter_obj.browse(cr, uid, param_correos_inc_salary_id)[0].correos
            subject = u'Incrementos salariales'
            message_email = u'Se le realizo un cambio de salario al(a los) trabajadores(s): <br/>'
            for attr in nuevos_attr_brw:
                message_email = message_email + '%s (codigo %s). Salario nuevo %s. <br/>'\
                                    %(elimina_tildes(attr.employee_id.name),attr.employee_id.cod_number,attr.amount)
            self.pool.get('hr.contract').send_email_notification(cr, uid, subject, message_email, correos_inc_salary_brw, context)
                    
        return {
            'type': 'ir.actions.act_window_close',
            'active_id': ids
        }
        
    def get_attributes(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        attr_dic = {}
        salary_increase_brw = self.browse(cr, uid, ids)[0]
        employee_list = []
        
        if salary_increase_brw.type_increase == 'employee':
            if salary_increase_brw.employee_id.state in ('activo','vacaciones'):
                employee_list.append(salary_increase_brw.employee_id)
            else:
                raise osv.except_osv(_('ERROR!'), _('El empleado seleccionado para incremento salarial debe estar en \
                                            estado Activo o Vacaciones'))
            
        elif salary_increase_brw.type_increase == 'sede':
            employee_list += list(set(salary_increase_brw.sede_id.employee_ids) & set(salary_increase_brw.payroll_id.employee_ids))
            
            
        elif salary_increase_brw.type_increase == 'department':
            employee_depart_ids = []
            for dep in salary_increase_brw.department_ids:
                employee_depart_ids += dep.member_ids
            employee_list += list(set(employee_depart_ids) & set(salary_increase_brw.payroll_id.employee_ids))
            
        elif salary_increase_brw.type_increase == 'all':
            employee_ids = self.pool.get('hr.employee').search(cr, uid, [])
            employee_list += self.pool.get('hr.employee').browse(cr, uid, employee_ids)
        
        employee_list = [x for x in employee_list if x.state in ('activo','vacaciones')]
        for employee in employee_list:
            if employee.id <> 1:
                parameter_obj = self.pool.get('hr.parameters')
                param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', employee.hr_payroll_id.id)])
                if not param_concept_salary_id:
                    param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
                if not param_concept_salary_id:
                    raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                                                        la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                        registrar un parametro de tipo concepto_sueldo')
                                                        %(employee.hr_payroll_id.name))
                    
                concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id
        
                exist = False
                attribute_brw = employee.attribute_ids
                for attribute in attribute_brw:
                    if attribute.concept_id.code == concept_salary_brw.code: #'SLDA': #Si el concepto es sueldo atributo
                        exist = True
                        if salary_increase_brw.type_amount == 'percentage':
                            new_amount = attribute.amount * (1+salary_increase_brw.amount/100)
                        else:
                            new_amount = attribute.amount + salary_increase_brw.amount
                            
                        key = employee.cod_number
                        attr_dic[key] = {
                            'name': attribute.name,
                            'employee_id': employee.id,
                            'date_start': datetime.now().strftime('%Y-%m-%d'),
                            'amount': new_amount,
                            'concept_id': attribute.concept_id.id,
                            'notes': 'Creado por el asistente se aumento de sueldo',
                            'active': True,
                        }
                        
                        break
                
                if not exist:
                    raise osv.except_osv(_('ERROR!'), _('El empleado %s codigo %s no tiene un atributo de inicio asociado al concepto\
                                                        Sueldo Atributo')%(employee.name, employee.cod_number))
                
        result = [value for code, value in attr_dic.items()]
        return result
    
hr_salary_increase()
