#-*- coding:utf-8 -*-

import netsvc
import time
from tools.translate import _
from osv import fields, osv
import calendar
from datetime import *
import datetime

class prestaciones_employee(osv.osv_memory):
    
    _name = 'prestaciones.employee'
    _description = 'Prestaciones sociales empleados'
    
    _columns = {
        'employee_id': fields.many2one('hr.employee', 'Empleado', readonly=True),
        'acumulado_prestaciones': fields.float('Devengado acumulado', digits=(16,2), readonly=True),
        'anticipos': fields.float('Saldo anticipos', digits=(16,2), readonly=True),
        'disponible': fields.float('Saldo disponible', digits=(16,2), readonly=True),
        'fecha_ingreso': fields.date('Fecha de ingreso', digits=(16,2), readonly=True),
        'tipo_cuenta_abono': fields.selection([('cont_emp', 'Contabilidad de la Empresa'),
                            ('fidei_ind', 'Fideicomiso individual'),('fondo_prest', 'Fondo de prestaciones de antiguedad')], 
                            "Tipo cuenta para abono"),
    }
    
    def acumulado_prestaciones(self, cr, uid, employee):
        if not employee:
            return 0.0   
        
        parameter_obj = self.pool.get('hr.parameters')
        concept_obj = self.pool.get('hr.concept')
        today = datetime.datetime.now().strftime('%Y-%m-%d')
        
        #Busqueda del concepto que se definio como Conceptos que inciden en Prestaciones Sociales
        param_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'conceptos_prest'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_pres_id:
            param_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'conceptos_prest'),('payroll_id', '=', None)])
        if not param_pres_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Conceptos que inciden en Prestaciones Sociales para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Conceptos que inciden en Prestaciones Sociales')
                            %(employee.hr_payroll_id.name))
            
        concept_prest_ids = parameter_obj.browse(cr, uid, param_pres_id)[0].concept_prest_ids
        
        #Busqueda del concepto que se definio como Concepto de anticipo de prestaciones
        param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_ant_pres_id:
            param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', None)])
        if not param_ant_pres_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Concepto anticipo prestaciones para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto anticipo prestaciones')
                            %(employee.hr_payroll_id.name))
            
        concept_ant_prest_id = parameter_obj.browse(cr, uid, param_ant_pres_id)[0].concept_ant_presta_id
        
        total_anticipos_mes = 0.0
        period = self.pool.get('hr.period').search(cr, uid, [('date_start','<=', today), 
                                    ('date_end','>=', today), 
                                    ('type','=',employee.hr_payroll_id.hr_payment_calendar_id.type)])
        if not period:
            raise osv.except_osv(_('ERROR!'), _('No hay un periodo para la fecha'))
        
        period = self.pool.get('hr.period').browse(cr, uid, period[0])

        #Verificar si ya se corrio proceso de prestaciones del mes actual
        proceso_prestaciones = True
        process_prest_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', [period.id]),
                                ('type','=','prestaciones'), ('state','=','done')])
        if not process_prest_ids:            
            periodos_mes = [x.id for x in period.hr_month_id.hr_period_ids]
            process_ids = self.pool.get('hr.process').search(cr, uid, [('type','=', 'anticipos'),('hr_period_ids','=', periodos_mes),
                            ('state','=','done')])
            process = self.pool.get('hr.process').browse(cr, uid, process_ids)
            for proc in process:
                payslip_id =  self.pool.get('hr.payslip').search(cr, uid, [('employee_id','=', employee.id),('hr_process_id','=', proc.id)])
                if payslip_id:
                    payslip_line_ids = self.pool.get('hr.payslip.line').search(cr, uid, [('slip_id','=', payslip_id)])
                    for line in self.pool.get('hr.payslip.line').browse(cr, uid, payslip_line_ids):
                        if line.code == concept_ant_prest_id.code:
                            total_anticipos_mes = (total_anticipos_mes + line.total) if line.type_calc == 'allocation' \
                                else (total_anticipos_mes - line.total)
        
        codes = [x.code for x in concept_prest_ids]
        total = 0.0

        previous_payslip_id = self.pool.get('hr.payslip').search(cr, uid, [('type_process','=', 'prestaciones'),
                                ('employee_id', '=', employee.id), ('hr_process_id.state', '=', 'done')], order='date_from desc')
        if previous_payslip_id:
            if isinstance(previous_payslip_id, list):
                previous_payslip_id = previous_payslip_id[0]
                
            payslip_line_ids = self.pool.get('hr.payslip.line').search(cr, uid, [('slip_id','=', previous_payslip_id),('code','in',codes)])
            
            for line in self.pool.get('hr.payslip.line').browse(cr, uid, payslip_line_ids):
                total = (total + line.total) if line.type_calc == 'allocation' else (total - line.total)
                
        return total - total_anticipos_mes

    def anticipos(self, cr, uid, employee): 
        parameter_obj = self.pool.get('hr.parameters')
        concept_obj = self.pool.get('hr.concept')
        
        #Busqueda del concepto que se definio como Concepto de anticipo de prestaciones
        param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_ant_pres_id:
            param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', None)])
        if not param_ant_pres_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Concepto anticipo prestaciones para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Concepto anticipo prestaciones')
                            %(employee.hr_payroll_id.name))
            
        concept_ant_prest_id = parameter_obj.browse(cr, uid, param_ant_pres_id)[0].concept_ant_presta_id

        total = 0.0

        process_ids = self.pool.get('hr.process').search(cr, uid, [('type','=', 'anticipos'),('state','=','done')])
        process = self.pool.get('hr.process').browse(cr, uid, process_ids)
        for proc in process:
            payslip_id =  self.pool.get('hr.payslip').search(cr, uid, [('employee_id','=', employee.id),('hr_process_id','=', proc.id)])
            if payslip_id:
                payslip_line_ids = self.pool.get('hr.payslip.line').search(cr, uid, [('slip_id','=', payslip_id)])
                for line in self.pool.get('hr.payslip.line').browse(cr, uid, payslip_line_ids):
                    if line.code == concept_ant_prest_id.code:
                        total = (total + line.total) if line.type_calc == 'allocation' else (total - line.total)
        return total

    def default_get(self, cr, uid, fields, context=None):
        """ To get default values for the object.
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param fields: List of fields for which we want default values
        @param context: A standard dictionary
        @return: A dictionary which of fields with values.
        """
        if context is None:
            context = {}
        
        record_id = context and context.get('active_id', False) or False
        res = super(prestaciones_employee, self).default_get(cr, uid, fields, context=context)
        employee_id = self.pool.get('hr.employee').browse(cr, uid, record_id, context=context)
        
        parameter_obj = self.pool.get('hr.parameters')
        
        #Busqueda del parametro que define el porcentaje maximo para otorgar anticipo de prestaciones-----------------------------
        param_porct_prest = parameter_obj.search(cr, uid, [('type', '=', 'porct_ant_prest'),('payroll_id', '=', employee_id.hr_payroll_id.id)])
        if not param_porct_prest:
            param_porct_prest = parameter_obj.search(cr, uid, [('type', '=', 'porct_ant_prest'),('payroll_id', '=', None)])
        if not param_porct_prest:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Maximo porcentaje para anticipo de Prestaciones para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Maximo porcentaje para anticipo de Prestaciones')
                            %(employee_id.hr_payroll_id.name))
            
        portc_prest = parameter_obj.browse(cr, uid, param_porct_prest)[0].value

        if not employee_id.contract_id:
            raise osv.except_osv(_('ERROR!'), _('Al empleado %s no se le ha registrado un contrato')
                            %(employee_id.name))

        acumulado_prestaciones = self.acumulado_prestaciones(cr, uid, employee_id)
        anticipos = self.anticipos(cr, uid, employee_id)
        disponible = (acumulado_prestaciones)*(portc_prest/100)
        
        if 'employee_id' in fields:
            res.update({
                'employee_id': employee_id.id,
                'tipo_cuenta_abono': employee_id.tipo_cuenta_abono,
                'fecha_ingreso': employee_id.start_rel_lab,
                'acumulado_prestaciones': acumulado_prestaciones,
                'anticipos': anticipos,
                'disponible': disponible,
            })
        return res

prestaciones_employee()
