# -*- coding: utf-8 -*-
##############################################################################
#
#    Industrias Diana, C.A.
#    Copyright (C) 2004-2010 Industrias Diana, C.A. (<http://www.industriasdiana.gob.ve>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from lxml import etree
import netsvc
from osv import osv, fields
import decimal_precision as dp
from tools.translate import _

class account_amortization_config(osv.osv):
    _name = 'account.amortization.config'
    
    _columns = {
    'name': fields.char('Ref', size=64),
    'journal_id': fields.many2one('account.journal','Journal'),
    'sequence_id': fields.many2one('ir.sequence', 'Entry Sequence', help="This field contains the informatin related to the numbering of the journal entries of this journal.", required=True),
    }
    
account_amortization_config()

class account_amortization(osv.osv):
    _name = 'account.amortization'
    
    def unlink(self, cr, uid, ids, context=None):
        am_brw = self.pool.get('account.amortization').browse(cr, uid, ids, context)
        for am in am_brw:
            if am.state <> 'concilied':
                super(account_amortization, self).unlink(cr, uid, ids, context)
            else:
                raise osv.except_osv(_('Error !'), _('No se puede borrar amortizaciones conciliadas'))
        return True
    
    def _get_journal(self, cr, uid, context=None):
        """Funcion que trae el diario configurado para
        las amortizaciones """
        journal_obj = self.pool.get('account.amortization.config')
        journal_ids = journal_obj.search(cr, uid, [])
        journal_brw = journal_obj.browse(cr, uid, journal_ids, context)
        return journal_brw[0].journal_id.id

    def _get_period(self, cr, uid, context=None):
        """ Funcion que trae el periodo en curso """
        period_obj = self.pool.get('account.period')
        return period_obj.find(cr, uid)[0]
    
    def _get_documents(self, cr, uid, ids, name, arg, context=None):
        """ Funcion que busca todos los documentos
        asociados a una factura y los devuelve en un diccionario"""
        res = {}
        doc_ids = []
        invoice_ids = []
        
        amortization_brw = self.browse(cr, uid, ids, context)
        for header in amortization_brw:
            for lines in header.line_ids:
                invoice_ids.append(lines.invoice_id.id)
            doc_ids = self.pool.get('account.invoice').search(cr, uid,[('parent_id','in',invoice_ids)])
            res[header.id] = doc_ids
        return res
    
    def _compute_lines(self, cr, uid, ids, name, args, context=None):
        """ Esta Funcion busca los asientos que se han reconciliado con
        la factura"""
        result = {}
        invoice_ids = []
        line_ids = []
        amortization_brw = self.browse(cr, uid, ids, context)
        for header in amortization_brw:
            for line in header.line_ids: 
                invoice_ids.append(line.invoice_id.id)
            invoice_brw = self.pool.get('account.invoice').browse(cr, uid, invoice_ids, context)
            for invoice in invoice_brw:
                src = []
                lines = []
                if invoice.move_id:
                    for m in invoice.move_id.line_id:
                        temp_lines = []
                        if m.reconcile_id:
                            temp_lines = map(lambda x: x.id, m.reconcile_id.line_id)
                        elif m.reconcile_partial_id:
                            temp_lines = map(lambda x: x.id, m.reconcile_partial_id.line_partial_ids)
                        lines += [x for x in temp_lines if x not in lines]
                        src.append(m.id)

                lines = filter(lambda x: x not in src, lines)
                if lines:
                    line_ids.append(lines[0])
            result[header.id] = line_ids
        return result
    
    _order = 'id desc'
    
    _columns = {
    'name': fields.char("Ref.", size=120),
    'partner_id': fields.many2one('res.partner', 'Partner', required=True, states={'draft':[('readonly',False)]}),
    'period_id': fields.many2one('account.period', 'Periodo', readonly=True, states={'draft':[('readonly',False)]}),
    'journal_id': fields.many2one('account.journal', 'Diario', readonly=True, states={'draft':[('readonly',False)]}),
    'date_amortization': fields.date('Fecha Registro'),
    'date_process': fields.date('Fecha Procesado'),
    'line_ids': fields.one2many('account.amortization.lines', 'amortization_id', 'Lineas'),
    'notes_ids': fields.function(_get_documents, method=True, type='many2many', relation='account.invoice', string='Notas de credito'),
    'payment_ids': fields.function(_compute_lines, method=True, relation='account.move.line', type="many2many", string='Payments'),
    'type': fields.selection([('pagar','CxP'),('cobrar','CxC')]),
    'state': fields.selection([
            ('draft','Draft'),
            ('concilied','Concilied'),
            ('cancel','Cancelled')
            ],'State', select=True, readonly=True),
    }
    
    _defaults = {
    'state':'draft',
    'date_amortization': lambda *a: time.strftime('%Y-%m-%d'),
    'journal_id': _get_journal,
    'period_id': _get_period,
    }
    
    def validate_withholdings (self, cr, uid, notes_ids, context=None):
        inv_brw = self.pool.get('account.invoice').browse(cr, uid, notes_ids, context)
        for inv in inv_brw:
            if inv.wh_iva_id:
                if inv.wh_iva_id.state <> 'done':
                    raise osv.except_osv(_('Error !'), _('The Credit Note has not checked the IVA withholdings'))
            if inv.islr_wh_doc_id:
                if inv.islr_wh_doc_id.state <> 'done':
                    raise osv.except_osv(_('Error !'), _('The Credit Note has not checked the ISLR withholdings'))
        return True
    
    def apply_amortization(self, cr, uid, ids, context=None):
        """ Funcion que busca y valida que las facturas agregadas
        a las lineas de la amortizacion posean notas de credito para
        luego comenzar a conciliarlas """
        line_ids = []
        for doc in self.browse(cr, uid, ids, context):
            if doc.line_ids:
                line_ids.append(doc.id)
                for lines in doc.line_ids:
                    if context.get('type',False) == 'cobrar': 
                        notes_ids = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',lines.invoice_id.id),('type','=','out_refund'),('state','<>','paid')])
                    else:
                        notes_ids = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',lines.invoice_id.id),('type','=','in_refund'),('state','<>','paid')])
                    if not notes_ids:
                            raise osv.except_osv(_('Integrity Error !'), _('This bill %s has no associated credit notes'%(str(lines.invoice_id.number))))
                    if self.validate_withholdings(cr, uid, notes_ids, context):
                        if not self._process_amortization(cr, uid, notes_ids, context):
                            raise osv.except_osv(_('Error !'), _('The reconciliation process is not complete, please check the documents'))
            else:
                raise osv.except_osv(_('Integrity Error !'), _('You can not process a write without lines'))
        number = self.pool.get('ir.sequence').get(cr, uid, 'account.amortization.code')
        self.write(cr, uid, ids, {'state':'concilied','date_process':time.strftime('%Y-%m-%d'),'type':context['type'],'name':number})
        return True
    
    def _process_amortization(self, cr, uid, ids, context=None):
        """ Funcion que se encarga de la conciliacion
        automatica de las notas de credito, si la misma
        es parcial crea un asiento intermedio y hace una
        conciliacion parcial de lo contrario ejecuta
        la conciliacion por el monto completo.
        @ids: Id de la Nota de Credito.
        @conciliation_ids: Id de las lineas a conciliar.
        @credit_note_amount: Monto de la nota de credito.
        @credit_note_account: Id de la cuenta contable de la nota de credito.
        @invoice_amount: Monto de la factura a la que se le aplicara la nota de credito.
        @invoice_account: Id de la cuenta contable de la factura a la que se le aplica la nota de credito. 
        @aml_obj: Variable con la clase account.move.line instanciada """
        
        am_obj = self.pool.get("account.move")
        aml_obj = self.pool.get("account.move.line")
        seq_obj = self.pool.get('ir.sequence')
        ai_obj = self.pool.get('account.invoice')
        aj_obj = self.pool.get('account.journal')
        conciliation_ids = []
        child_move_ids = []
        credit_note_amount = 0.00
        
        credit_note_brw = ai_obj.browse(cr, uid, ids, context)
        
        for credit_note in credit_note_brw:
            
            credit_note_amount += credit_note.amount_total
            credit_note_account = credit_note.account_id.id
            credit_note_journal = credit_note.journal_id.id
            credit_note_date = credit_note.date_invoice
            credit_note_date_due = credit_note.date_due
            credit_note_period = credit_note.period_id.id
            credit_note_partner = credit_note.partner_id.id 
            credit_note_name = credit_note.name
            credit_note_reference = credit_note.reference
            
            company_currency = credit_note.journal_id.company_id.currency_id.id or False
            current_currency = credit_note.currency_id.id or False
            
            child_move_brw = credit_note.move_id
            child_move_ids.append(credit_note.move_id.id)
            
            for invoice in [credit_note.parent_id]:
                invoice_amount  = invoice.amount_total
                invoice_account = invoice.account_id.id
                parent_move_brw = invoice.move_id
                
        if (credit_note_amount == invoice_amount):
            #~ Conciliacion Total
            for move_reg in [parent_move_brw]:
                for lines in move_reg.line_id:
                    if context.get('type_invoice',False) == 'out_invoice':
                        if lines.account_id.id == credit_note_account and lines.debit > 0.00:
                            conciliation_ids.append(lines.id)
                    else:
                        if lines.account_id.id == credit_note_account and lines.credit > 0.00:
                            conciliation_ids.append(lines.id)
            for move_reg in [child_move_brw]:
                for lines in move_reg.line_id:
                    if context.get('type_invoice',False) == 'out_invoice':
                        if lines.account_id.id == credit_note_account and lines.credit > 0.00:
                            conciliation_ids.append(lines.id)
                    else:
                        if lines.account_id.id == credit_note_account and lines.debit > 0.00:
                            conciliation_ids.append(lines.id)
            aml_obj.reconcile_partial(cr, uid, conciliation_ids, 'auto', context=context)
        elif (credit_note_amount < invoice_amount):
            #~ Creacion del Asiento Intermedio para la conciliacion parcial de la nota de credito
            am_conf = self.pool.get('account.amortization.config')
            am_conf_id = am_conf.search(cr, uid, [])
            am_brw = am_conf.browse(cr, uid, am_conf_id, context=context)
            for journal in am_brw:
                journal_sequence_id = journal.journal_id.sequence_id.id
                journal_id = journal.journal_id.id
            
            if not credit_note_reference:
                ref = "Asiento Intermedio"
            else:
                ref = "Asiento intermedio "+ credit_note_reference
            move = {
                    'name': seq_obj.get_id(cr, uid, journal_sequence_id),
                    'journal_id': journal_id,
                    'date': credit_note_date,
                    'ref': ref,
                    'period_id': credit_note_period or False
                    }
            move_id = am_obj.create(cr, uid, move, context)
            #~ Asiento Inverso para la Conciliacion.
            #create the first line manually
            debit = 0.0
            credit = credit_note_amount
            move_line = {
                'name': credit_note_name or '/',
                'debit': debit,
                'credit': credit,
                'account_id': credit_note_account,
                'move_id': move_id,
                'journal_id': journal_id,
                'period_id': credit_note_period,
                'partner_id': credit_note_partner,
                'currency_id': company_currency <> current_currency and  current_currency or False,
                'date': credit_note_date,
                'date_maturity': credit_note_date_due,
            }
            account_credit = aml_obj.create(cr, uid, move_line)
            debit = credit_note_amount
            credit = 0.00
            move_line = {
                'name': credit_note_name or '/',
                'debit': debit,
                'credit': credit,
                'account_id': credit_note_account,
                'move_id': move_id,
                'journal_id': journal_id, 
                'period_id': credit_note_period,
                'partner_id': credit_note_partner,
                'currency_id': company_currency <> current_currency and  current_currency or False,
                'date': credit_note_date,
                'date_maturity': credit_note_date_due,
            }
            account_debit = aml_obj.create(cr, uid, move_line)
            isok = am_obj.button_validate(cr, uid, [move_id], context)
            #~ Conciliacion parcial contra la factura
            for move_reg in [parent_move_brw]:
                for lines in move_reg.line_id:
                    if context.get('type_invoice',False) == 'out_invoice':
                        if lines.account_id.id == credit_note_account and lines.debit > 0.00:
                            conciliation_ids.append(lines.id)
                            conciliation_ids.append(account_credit)
                    else:
                        if lines.account_id.id == credit_note_account and lines.credit > 0.00:
                            conciliation_ids.append(lines.id)
                            conciliation_ids.append(account_debit)
            aml_obj.reconcile_partial(cr, uid, conciliation_ids, 'auto', context=context)
            conciliation_ids = []
            child_move_brw = am_obj.browse(cr, uid, child_move_ids, context)
            #~ Conciliacion contra las notas de credito
            for move_reg in child_move_brw:
                for lines in move_reg.line_id:
                    if context.get('type_invoice',False) == 'out_invoice':
                        if lines.account_id.id == credit_note_account and lines.credit > 0.00:
                            conciliation_ids.append(lines.id)
                            conciliation_ids.append(account_debit)
                            aml_obj.reconcile_partial(cr, uid, conciliation_ids, 'auto', context=context)
                            conciliation_ids = []
                    else:
                        if lines.account_id.id == credit_note_account and lines.debit > 0.00:
                            conciliation_ids.append(lines.id)
                            conciliation_ids.append(account_credit)
                            aml_obj.reconcile_partial(cr, uid, conciliation_ids, 'auto', context=context)
                            conciliation_ids = []
                            
        elif (credit_note_amount > invoice_amount):
            raise osv.except_osv(_('Integrity Error !'), _('Credit notes exceed the amount of the invoice'))
        return True
              
account_amortization()

class account_amortization_lines(osv.osv):
    _name = 'account.amortization.lines'
    
    def on_change_validate(self, cr, uid, ids, invoice_id, context=None):
        line_ids = []
        result = {}
        print "Aqui ta el ids ",ids
        print "Aqui ta la factura", invoice_id
        if context.get('type',False) == 'cobrar': 
            notes_ids = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',invoice_id),('type','=','out_refund'),('state','<>','paid')])
        else:
            notes_ids = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',invoice_id),('type','=','in_refund'),('state','<>','paid')])
        print "Aqui tan las Notas ",notes_ids
        if not notes_ids:
            warning = {
            'title': 'Precio erroneo!',
            'message':
                "This invoice has not been credit notes."
            }
            result.update({'warning': warning})
            result.update({'value': {'invoice_id':False,}})
        else:
            result.update({'value': {'invoice_id':invoice_id,}})
        return result
    
    _columns = {
    'name': fields.char('Referencia',size=120),
    'invoice_id':fields.many2one('account.invoice', 'Facturas', required=True),
    'amortization_id':fields.many2one('account.amortization','Amortizacion', required=True, ondelete='cascade'),
    }
    
account_amortization_lines()
