# -*- encoding: utf-8 -*-

import time
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import logging
from osv import osv, fields
import netsvc
import pooler
from tools.translate import _
import decimal_precision as dp
from osv.orm import browse_record, browse_null

class res_company(osv.osv):
    
#Esta Clase hereda de res.company y agrega dos nuevos campos uno que es utilizado como parametro de las cotizaciones

    _name = "res.company"
    _inherit = "res.company"

    _columns = {
        'nro_minimo_cotizaciones': fields.integer('Numero Minimo de Cotizaciones'),
        'grupo_superusuarios_compras' : fields.many2one('res.groups','Rol superusuario de compras',ondelete='set null'),
    }
    
    _defaults = {
       'nro_minimo_cotizaciones': 3,
    }

res_company()

class purchase_requisition(osv.osv):
    """Esta clase hereda de purchase.requisition en ella se agregan los campos
    para el solicitante de la requisicion y se sobrescribe el campo exclusive
    para solo permitir requisiciones exclusivas y el user_id para crear por 
    defecto vacio el campo""" 
    
    _name = "purchase.requisition"
    _inherit = "purchase.requisition"
    
    _columns = {
        'solicitante_requisicion_id': fields.many2one('res.users','Solicitante de Requisicion',ondelete='set null'),
        'exclusive': fields.selection([('exclusive','Purchase Requisition (exclusive)')]),
        'user_id': fields.many2one('res.users', 'Responsible',states={'done': [('readonly', True)]}),
        'state': fields.selection([('draft','Draft'),('in_progress','In Progress'),('cancel','Cancelled'),('approved','Approved'),('done','Done')], 'State', required=True),
    }
    
    _defaults = {
        'solicitante_requisicion_id': lambda self, cr, uid, context: uid,
        'user_id': None,
        'exclusive': 'exclusive',
    }
    
    def copy(self, cr, uid, id, defaults, context=None): #Sobreescritura del metodo copy para no permitir duplicar objetos stock_picking
        raise osv.except_osv(_('Acción Denegada!'),_('No se permite duplicar solicitudes de compra!'))
            
    def tender_approved(self, cr, uid, ids, context=None):
        brw_comp = self.browse(cr, uid, ids[0])
        n_cotizaciones = brw_comp.company_id.nro_minimo_cotizaciones
        brw_ids = self.pool.get('purchase.order').search(cr, uid, [('requisition_id', '=', int(ids[0]))])
        brw_order = self.pool.get('purchase.order').browse(cr, uid, brw_ids, context)
        providers = []
        for o in brw_order:
            if o.partner_id not in providers:
                providers.append(o.partner_id)
        if len(providers) < n_cotizaciones:
                texto = 'No puede aprobar una solicitud sin haber licitado por lo menos '+str(n_cotizaciones)+' proveedores.'
                raise osv.except_osv('Error!',texto)
        self.write(cr, uid, ids, {'state':'approved',}, context=context)
        return True
        
    def tender_in_progress(self, cr, uid, ids, context=None):
        """ Valida que este asignado un responsable a la requisicion 
        de compra
        """
        brw = self.search(cr, uid, [('user_id', '<>', ''), ("id" , "=" , int(ids[0]))])
        if brw == []:
            raise osv.except_osv('Error !','Debe asignar un comprador')
        else:
            self.write(cr, uid, ids, {'state':'in_progress'} ,context=context)
            for po in self.browse(cr, uid, ids, context=context):
                #Notificaciones internas
                notificaciones = {
                    'name': ('[%s] Atencion, notificacion de compra asignada.') % (po.name,),
                    'act_to': int(po.user_id),
                    'act_from': int(uid),
                    'body': ("Se le ha asignado la compra con referencia: '%s'.") % (po.name,),
                    'active': True,
                    'priority': '2',
                    'state': 'waiting'
                }
                request_obj = self.pool.get('res.request')
                request_id = request_obj.create(cr, uid, notificaciones, context=context)
                #Notificaciones por correo
                smtp_server_id = self.pool.get('email.smtpclient').search(cr, \
                uid, [('active','=',True),('state','=','confirm')], context=False)
                smtp_server_id = smtp_server_id and smtp_server_id[0] or False
                if smtp_server_id :
                    server_name = self.pool.get('email.smtpclient').browse(cr, uid, smtp_server_id)
                    email_to = self.pool.get('res.users').browse(cr, uid, int(po.user_id)).user_email
                    email = {
                            'server_id': smtp_server_id,
                            'name': ('[%s] Atencion, notificacion de compra asignada.') % (po.name,),
                            'server': server_name.server,
                            'body': ("Se le ha asignado la compra con referencia: '%s'.") % (po.name,),
                            'to': email_to,
                            'from': uid,
                            }
                    smtp_obj = self.pool.get('email.smtpclient')
                    if email['to']:
                        smtp_obj.send_email(cr, uid, smtp_server_id, email['to'], email['name'], email['body'])
        return True
    
    def tender_forced(self, cr, uid, ids, context=None):
        """Funcion que utiliza el boton "forzar realizado" del jefe de compras
        para la excepcion cuando existen un numero menor de cotizaciones a las
        requeridas"""
        brw_comp = self.read(cr, uid, ids[0])
        if brw_comp["description"] is False:
            raise osv.except_osv(_('Error !'),_('Debe llenar el campo Nota'))
            
        brw_order = self.pool.get('purchase.order').search(cr, uid, [('requisition_id', '=', int(ids[0]))])
        
        if len(brw_order):
            self.write(cr, uid, ids, {'state':'done', 'date_end':time.strftime('%Y-%m-%d %H:%M:%S')}, context=context)
            return True
        else:
            raise osv.except_osv('Error!', 'Debe haber al menos una cotización para continuar.')
            return False
            
    def tender_forced_approved(self, cr, uid, ids, context=None):
        """Funcion que utiliza el boton "forzar aprobado" del jefe de compras
        para la excepcion cuando existen un numero menor de cotizaciones a las
        requeridas"""
        brw_comp = self.read(cr, uid, ids[0])
        if brw_comp["description"] is False:
            raise osv.except_osv(_('Error !'),_('Debe llenar el campo Nota'))
            
        brw_order = self.pool.get('purchase.order').search(cr, uid, [('requisition_id', '=', int(ids[0]))])
        
        if len(brw_order):
            self.write(cr, uid, ids, {'state':'approved',}, context=context)
            return True
        else:
            raise osv.except_osv('Error!', 'Debe haber al menos una cotización para continuar.')
            return False    

    def tender_done(self, cr, uid, ids, context=None):
        """Funcion para el boton "realizado" que valida la cantidad minima de cotizaciones
        y cambia el estado de la solicitud a done si se cumple con la condicion requerida"""
        brw_comp = self.browse(cr, uid, ids[0])
        n_cotizaciones = brw_comp.company_id.nro_minimo_cotizaciones
        brw_ids = self.pool.get('purchase.order').search(cr, uid, [('requisition_id', '=', int(ids[0]))])
        brw_order = self.pool.get('purchase.order').browse(cr, uid, brw_ids, context)
        providers = []
        for o in brw_order:
            if o.partner_id not in providers:
                providers.append(o.partner_id)
        if len(providers) < n_cotizaciones:
                texto = 'No puede confirmar una cotizacion sin haber solicitado por lo menos '+str(n_cotizaciones)+' proveedores.'
                raise osv.except_osv('Error!',texto)
        self.write(cr, uid, ids, {'state':'done', 'date_end':time.strftime('%Y-%m-%d %H:%M:%S')}, context=context)
        return True

    def tender_reset(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state': 'done'})
        return True
        
    def write(self, cr, user, ids, vals, context=None):
        id_req = super(purchase_requisition, self).write(cr, user, ids, vals, context)
        
        """ 
            Esto es absurdo, pero funciona. 
            El problema es que unas veces llega ids como lista y otras
            como integer
        """
        if not type(ids) == type(list()):
            tmp = []
            tmp.append(ids)
            ids = tmp
            
        for req_id in ids :
            order_obj = self.pool.get('purchase.order')
            order_ids = order_obj.search(cr, user, [('requisition_id', '=', req_id)])
            for order in order_obj.browse(cr,user, order_ids, context=context):
                if not order.state is 'done':
                    if not vals.get('user_id') is None:
                        order_obj.write(cr, user,order.id, {'create_uid': vals.get('user_id')})
        return id_req
        
    def tender_cancel(self, cr, uid, ids, context=None):
        if super(purchase_requisition, self).tender_cancel(cr, uid, ids, context=context):
            abas_ids = self.pool.get('procurement.order').search(cr, 1, [('requisition_id','=',ids)])
            return self.pool.get('procurement.order').action_cancel(cr, uid, abas_ids)
   
    
purchase_requisition()

class purchase_requisition_line(osv.osv):
    _inherit = 'purchase.requisition.line'
    
    _columns = {
        'state' : fields.related('requisition_id', 'state', 
            type='string', relation='purchase.requisition', 
            readonly=True, string="Estado"),
        'categ_id' : fields.related('product_id', 'product_tmpl_id', 'categ_id',
            type='many2one', relation='product.category', 
            readonly=True, string="Categoría"),
        'type' : fields.related('product_id', 'product_tmpl_id', 'type',
            type='string', relation='product.template', 
            readonly=True, string="Tipo de producto"),
        'description' : fields.text('Descripcion'),
    }
    
purchase_requisition_line()

class procurement_order(osv.osv):

    _inherit = 'procurement.order'
    _columns = {
        'requisition_id' : fields.many2one('purchase.requisition','Latest Requisition')
    }
    _logger = logging.getLogger('diana.orden.abastecimiento')

class purchase_order(osv.osv):
    _logger = logging.getLogger('diana.orden.de.compra')
    
    """Clase que hereda de purchase.order, y genera un campo funcional para
    calcular la cantidad de producto de cada una de las cotizaciones para mostrarlo
    en la vista purchase.requisition"""
    
    _inherit = "purchase.order"

    def _get_total_qty (self, cr, uid, ids, field_name, arg, context=None):
        """
        FunciÓn que cuenta las lÍneas de producto de las cotizaciones y las suma
        """
        res = {}
        for order in self.browse(cr, uid, ids, context=context):
            res[order.id] = 0
            for line in order.order_line:
                res[order.id] += line.product_qty
        return res
    
    _columns = {
        'requisition_id' : fields.many2one('purchase.requisition','Purchase Requisition', required=False, domain=[('state','in',('in_progress',))]),
        'qty' : fields.function(_get_total_qty, method=True, type='integer', string='Cantidad',
            store=False),
        'approved' : fields.boolean('Aprobado'),
        'via_envio': fields.selection([('Terrestre','Terrestre'),('Maritima','Maritima'),('Aereo','Aereo')],'Via de Envio', required= False ),
        'cond_pago': fields.many2one('account.payment.term','Condicion de Pago'),
        'user_id' : fields.related('requisition_id', 'user_id', \
            type='many2one', relation='purchase.requisition', \
            readonly='True', string="Responsable"),
        'req_state' : fields.related('requisition_id', 'state', \
            type='string', relation='purchase.requisition', \
            readonly='True', string="Estado Req."),
    }
    
    _defaults = {
        'name': lambda obj, cr, uid, context: obj.pool.get('ir.sequence').get(cr, uid, 'purchase.order.draft'),
        'invoice_method' : 'picking', 
        'approved' : False,
        'via_envio': None,
        'cond_pago': None,
    }
    
    def action_approve(self, cr, uid, ids, context=None):
        for po in self.browse(cr, uid, ids, context=context): # Recorrer la lista de purchase_order
            if not (po.requisition_id.state == 'approved' or po.requisition_id.state == 'done'):
                raise osv.except_osv(_('Error !'),_('Debe aprobar la solicitud de compra previamente.'))
            if po.requisition_id and (po.requisition_id.exclusive == 'exclusive'):
                    for order in po.requisition_id.purchase_ids:
                        if order.id <> po.id:
                            wf_service = netsvc.LocalService("workflow")
                            wf_service.trg_validate(uid, 'purchase.order', order.id, 'purchase_cancel', cr)
        
        if self.write(cr, uid, ids, {'approved' : True}) :
            po_obj = self.browse(cr, uid, ids, context=context)
            po_ids = self.search(cr, uid, [('requisition_id','=', po_obj[0].requisition_id.id)])
            proc_ord_ids = self.pool.get('procurement.order').search(cr, uid, [('purchase_id', 'in', po_ids)])
            self.pool.get('procurement.order').write(cr, 1, proc_ord_ids, {'purchase_id' : po_obj[0].id, 'state':'running'})
                
    def procurement_backorder(self, cr, uid, po_obj, new_procurement, context=None): 
        if context is None:
            context = {}
        procurement_obj = self.pool.get('procurement.order')   
        for item in new_procurement:
            new_product_qty = new_procurement.get(item) #diferencia de cantidades solicitadas y cantidades aprobadas  
            wf_service = netsvc.LocalService("workflow")
            if len(item) == 3: #Si item tiene 3 campos se trata de un servicio
                product_id = item[0]
                product_uom = item[1]
                note = item[2]
            else: #Si item tiene 2 campos se trata de un producto
                product_id = item[0]
                product_uom = item[1]
                note = False

            """
            Se crea la nueva solicitud de abastecimiento con la cantidad faltante solicitada
            """
            #aqui se colocan los mismos datos que se generaron al momento de la primera solicitud de abastecimiento
            procure_id = procurement_obj.create(cr, 1, {
                'name':po_obj[0].requisition_id.name,
                'origin': po_obj[0].requisition_id.name,
                'date_planned': datetime.strftime(datetime.strptime(po_obj[0].requisition_id.date_end[0:10], '%Y-%m-%d') + timedelta(days=60), '%Y-%m-%d'),
                'product_id': product_id,
                'product_qty': new_product_qty,
                'product_uom': product_uom,
                'location_id': po_obj[0].location_id.id,
                'note': note,
                'procure_method':'make_to_order',
                })
                
            wf_service.trg_validate(1, 'procurement.order', procure_id, 'button_confirm', cr) #Pasa el objeto procurement.order del estado draft al estado confirm
            wf_service.trg_validate(1, 'procurement.order', procure_id, 'button_check', cr) #Pasa el objeto procurement.order del estado confirm al estado check
            
            
            pur_order_obj = self.pool.get('purchase.order')
            pur_req_obj = self.pool.get('purchase.requisition')
            product = self.pool.get('product.product').browse(cr, uid, product_id)
            proc = self.pool.get('procurement.order').browse(cr, uid, procure_id)

            if procure_id:
                message = _("Se genero un backorder con documento origen '%s', con cantidad faltante del producto '%s'.") % (proc.name, product.name_template)
                self.log(cr, uid, proc.requisition_id.id, message, context=context)
                
            if all ([proc.requisition_id, proc.purchase_id]):
                req = self.pool.get('purchase.requisition').browse(cr, uid, proc.requisition_id.id) # Solicitud de compra (purchase.requisition)
                pur = self.pool.get('purchase.order').browse(cr, uid, proc.purchase_id.id) # Pedido de compra (purchase.order)
                """ 
                Aca se actualiza el origen
                """
                values = {
                    'origin': po_obj[0].requisition_id.name,
                    'notes':proc.note,
                    }
                if proc.product_id.product_tmpl_id.type == 'service':
                    values['invoice_method']= 'order'
                    
                self.pool.get('purchase.order').write(cr, 1, pur.id, values) 
                """
                Agregar concepto de retencion de ISRL del producto a la solicitud 
                de presupuesto que se genera 
                """
                for i in pur.order_line:
                    pol_name = self.pool.get('purchase.order.line').browse(cr, uid, i.id, context=context).name
                    self.pool.get('purchase.order.line').write(cr, uid, i.id, {
                        'concept_id': i.product_id and 
                            i.product_id.concept_id and 
                            i.product_id.concept_id.id or False,
                        'name':proc.note or pol_name,
                    })
                """
                Se consulta el warehouse de la ubicacion desde donde se realizo la solicitud de abastecimiento (procurement_order)
                para asignarle este warehouse a la solicitud de compra (purchase.requisition). Luego se actualiza la 
                solicitud de compra (purchase.requisition)
                """
                wh_ = self.pool.get('stock.warehouse').search(cr, uid, [('lot_input_id', '=', proc.location_id.id)]) 
                
                self.pool.get('purchase.requisition').write(cr, 1, req.id, { 
                    'warehouse_id': wh_[0],
                    'origin': proc.origin,
                    'user_id': po_obj[0].requisition_id.user_id.id,
                    'solicitante_requisicion_id': po_obj[0].requisition_id.solicitante_requisicion_id.id,
                    'state': 'in_progress',
                    #'description':proc.note,
                })
                
                po_line_id = self.pool.get('purchase.requisition.line').search(cr, uid, [('requisition_id', '=', req.id)])
                self.pool.get('purchase.requisition.line').write(cr, uid, po_line_id, { 
                    'description':proc.note,
                })
        
    def wkf_confirm_order(self, cr, uid, ids, context=None):
        po_obj = self.browse(cr, uid, ids, context=context)

        for po in po_obj:
            if po.amount_total <= 0:
                raise osv.except_osv(_('Error !'),_('El total presupuestado no puede ser menor o igual a cero.'))

        id_purc_o_l2 = self.pool.get('purchase.order.line').search(cr, uid, [('order_id', '=', po_obj[0].id)])
        for po2 in id_purc_o_l2:
            purc_o_l2 = self.pool.get('purchase.order.line').browse(cr, uid, po2) #Cantidad que se aprueba purchase.order.line
            if purc_o_l2.product_qty > 0 and purc_o_l2.price_unit <= 0:
                raise osv.except_osv(_('Error !'),_('Verifique el precio unitario, no debe ser igual a cero.'))
                
        if po_obj[0].requisition_id.id and not (po_obj[0].requisition_id.state == 'approved' or po_obj[0].requisition_id.state == 'done') :
            raise osv.except_osv(_('Error !'),_('Debe aprobar la solicitud de compra previamente.'))
            
        res = super(purchase_order, self).wkf_confirm_order(cr, uid, ids, context=context)
        
        name = self.pool.get('ir.sequence').get(cr, uid, 'purchase.order')
        self.write(cr, uid, ids, {'name' : name})
        
        po_ids = self.search(cr, uid, [('requisition_id','=', po_obj[0].requisition_id.id)])
        proc_ord_ids = self.pool.get('procurement.order').search(cr, 1, [('purchase_id', 'in', po_ids)])
        self.pool.get('procurement.order').write(cr, uid, proc_ord_ids, {'purchase_id':po_obj[0].id, 'state':'done'})
        
        #---------------------------------------
        id_purc_o_l = self.pool.get('purchase.order.line').search(cr, uid, [('order_id', '=', po_obj[0].id)])
        id_purc_r_l = self.pool.get('purchase.requisition.line').search(cr, uid, [('requisition_id', '=', po_obj[0].requisition_id.id)])
        
        products_pol = {}
        products_prl = {}
        new_procurement = {}
        for each_purc_o_l in id_purc_o_l:
            purc_o_l = self.pool.get('purchase.order.line').browse(cr, uid, each_purc_o_l) #Cantidad que se aprueba purchase.order.line
            
            if purc_o_l.product_id.product_tmpl_id.type == 'product': #Si es un producto se guarda en el diccionario el product_id y el product_uom con que se solicito
                if products_pol.get((purc_o_l.product_id.id, purc_o_l.product_uom.id)): 
                    products_pol[(purc_o_l.product_id.id, purc_o_l.product_uom.id)] = products_pol.get((purc_o_l.product_id.id, purc_o_l.product_uom.id)) + purc_o_l.product_qty
                else:
                    products_pol[(purc_o_l.product_id.id, purc_o_l.product_uom.id)] = purc_o_l.product_qty
            else: #Si es un servicio se guarda en el diccionario el product_id, el product_uom con que se solicito 
                  #y la descripcion para luego colocarsela en la nota de la nueva solicitud de abastecimineto en caso de que se adquiera una cantidad menor a la solicitada
                if products_pol.get((purc_o_l.product_id.id, purc_o_l.product_uom.id, purc_o_l.name)):
                    products_pol[(purc_o_l.product_id.id, purc_o_l.product_uom.id, purc_o_l.name)] = products_pol.get((purc_o_l.product_id.id, purc_o_l.product_uom.id, purc_o_l.name)) + purc_o_l.product_qty
                else:
                    products_pol[(purc_o_l.product_id.id, purc_o_l.product_uom.id, purc_o_l.name)] = purc_o_l.product_qty
            
        for each_purc_r_l in id_purc_r_l:
            purc_r_l = self.pool.get('purchase.requisition.line').browse(cr, uid, each_purc_r_l) #Cantidad que se solicita purchase.requisition.line
            
            if purc_r_l.product_id.product_tmpl_id.type == 'product':
                if products_prl.get((purc_r_l.product_id.id, purc_r_l.product_uom_id.id)):
                    products_prl[(purc_r_l.product_id.id, purc_r_l.product_uom_id.id)] = products_prl.get((purc_r_l.product_id.id, purc_r_l.product_uom_id.id)) + purc_r_l.product_qty
                else:
                    products_prl[(purc_r_l.product_id.id, purc_r_l.product_uom_id.id)] = purc_r_l.product_qty
            else:
                if products_prl.get((purc_r_l.product_id.id, purc_r_l.product_uom_id.id, purc_r_l.description)):
                    products_prl[(purc_r_l.product_id.id, purc_r_l.product_uom_id.id, purc_r_l.description)] = products_prl.get((purc_r_l.product_id.id, purc_r_l.product_uom_id.id, purc_r_l.description)) + purc_r_l.product_qty
                else:
                    products_prl[(purc_r_l.product_id.id, purc_r_l.product_uom_id.id, purc_r_l.description)] = purc_r_l.product_qty
        
        for item in products_pol:
            if products_pol.get(item) < products_prl.get(item):
                new_procurement[item] = products_prl.get(item) - products_pol.get(item)

        if new_procurement:
            self.procurement_backorder(cr, uid, po_obj, new_procurement, context)
        return True
        

    def purchase_editar(self, cr, uid, ids, context=None):
        """Funcion que llama la vista de purchase.order al hacer click sobre el boton
        de editar"""
        if self.browse(cr, uid, ids, context=context)[0].approved :
            raise osv.except_osv(_('Error !'),
            _('La cotización ya fue aprobada. No se pueden realizar cambios.'))
        return {
            'name':_("Purchase Order"),
            'view_mode': 'form',
            'view_id': False,
            'view_type': 'form',
            'res_model': 'purchase.order',
            'res_id': ids,
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'new',
            'ref': 'purchase.purchase_order_tree_view',
        }

    def button_dummy(self, cr, uid, ids, context=None):
        return False
        
    def merge(self, cr, uid, ids, context=None):
        """
        To merge similar type of purchase orders.
        Orders will only be merged if:
        * Purchase Orders are in draft
        * Purchase Orders belong to the same partner
        * Purchase Orders are have same stock location, same pricelist
        Lines will only be merged if:
        * Order lines are exactly the same except for the quantity and unit

         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param ids: the ID or list of IDs
         @param context: A standard dictionary

         @return: new purchase order id

        """
        wf_service = netsvc.LocalService("workflow")
        def make_key(br, fields):
            list_key = []
            for field in fields:
                field_val = getattr(br, field)
                if field in ('product_id', 'move_dest_id', 'account_analytic_id'):
                    if not field_val:
                        field_val = False
                if isinstance(field_val, browse_record):
                    field_val = field_val.id
                elif isinstance(field_val, browse_null):
                    field_val = False
                elif isinstance(field_val, list):
                    field_val = ((6, 0, tuple([v.id for v in field_val])),)
                list_key.append((field, field_val))
            list_key.sort()
            return tuple(list_key)
    # compute what the new orders should contain

        new_orders = {}

        for porder in [order for order in self.browse(cr, uid, ids, context=context) if order.state == 'draft']:
            order_key = make_key(porder, ('partner_id', 'location_id', 'pricelist_id'))
            new_order = new_orders.setdefault(order_key, ({}, []))
            new_order[1].append(porder.id)
            order_infos = new_order[0]
            if not order_infos:
                order_infos.update({
                    'origin': porder.origin,
                    'date_order': time.strftime('%Y-%m-%d'),
                    'partner_id': porder.partner_id.id,
                    'partner_address_id': porder.partner_address_id.id,
                    'dest_address_id': porder.dest_address_id.id,
                    'warehouse_id': porder.warehouse_id.id,
                    'location_id': porder.location_id.id,
                    'pricelist_id': porder.pricelist_id.id,
                    'state': 'draft',
                    'order_line': {},
                    'notes': '%s' % (porder.notes or '',),
                    'fiscal_position': porder.fiscal_position and porder.fiscal_position.id or False,
                })
            else:
                if porder.notes:
                    order_infos['notes'] = (order_infos['notes'] or '') + ('\n%s' % (porder.notes,))
                if porder.origin:
                    order_infos['origin'] = (order_infos['origin'] or '') + ' ' + porder.origin
            for order_line in porder.order_line:
                line_key = make_key(order_line, ('name', 'date_planned', 'taxes_id', 'price_unit', 'notes', 'product_id', 'move_dest_id', 'account_analytic_id'))
                o_line = order_infos['order_line'].setdefault(line_key, {})
                if o_line:
                    # merge the line with an existing line
                    o_line['product_qty'] += order_line.product_qty * order_line.product_uom.factor / o_line['uom_factor']
                else:
                    # append a new "standalone" line
                    for field in ('product_qty', 'product_uom'):
                        field_val = getattr(order_line, field)
                        if isinstance(field_val, browse_record):
                            field_val = field_val.id
                        o_line[field] = field_val
                    o_line['uom_factor'] = order_line.product_uom and order_line.product_uom.factor or 1.0
        allorders = []
        orders_info = {}
        for order_key, (order_data, old_ids) in new_orders.iteritems():
            # skip merges with only one order
            if len(old_ids) < 2:
                allorders += (old_ids or [])
                continue
            # cleanup order line data
            for key, value in order_data['order_line'].iteritems():
                del value['uom_factor']
                value.update(dict(key))
            order_data['order_line'] = [(0, 0, value) for value in order_data['order_line'].itervalues()]
            order_data['requisition_id'] = porder.requisition_id.id
            # create the new order
            neworder_id = self.create(cr, uid, order_data)
            orders_info.update({neworder_id: old_ids})
            allorders.append(neworder_id)
            # make triggers pointing to the old orders point to the new order
            for old_id in old_ids:
                wf_service.trg_redirect(uid, 'purchase.order', old_id, neworder_id, cr)
                wf_service.trg_validate(uid, 'purchase.order', old_id, 'purchase_cancel', cr)
        return orders_info
        
    def do_merge(self, cr, uid, ids, context=None):
        po_obj = self.browse(cr, uid, ids, context=context)
        for item in po_obj:
            po_ids = self.search(cr, uid, [('requisition_id','=', item.requisition_id.id)])
            po_obj_ids = self.browse(cr, uid, po_ids, context=context)
            pur_req = self.pool.get('purchase.requisition').browse(cr, uid, item.requisition_id.id, context)

            if pur_req.state <> 'approved':
                raise osv.except_osv(_('Error !'),
                    _('No se permite fusionar Ordenes de Compra cuyas Solicitudes de Compra no hayan sido aprobadas previamente')) 
            for po in po_obj_ids:
                if (bool(po.id <> item.id) & bool(po.state <> 'cancel')) | (bool(po.id == item.id) & bool(po.state <> 'draft')):
                    raise osv.except_osv(_('Error !'),
                        _('No se permite fusionar Ordenes de Compra que no hayan sido aprobadas previamente')) 
            
        #~ res = super(purchase_order, self).do_merge(cr, uid, ids, context=context)
        res = self.merge(cr, uid, ids, context=context)
        for porder in res.keys():
            po_new_obj = self.browse(cr, uid, porder, context)
            pol_new_ids = self.pool.get('purchase.order.line').search(cr, uid, [('order_id','=', porder)])
            
            if type(pol_new_ids) == type(list()):
                pol_new_ids = pol_new_ids[0]
                
            pol_new_brw = self.pool.get('purchase.order.line').browse(cr, uid, pol_new_ids)
            
            if pol_new_brw.product_id.product_tmpl_id.type == 'service':
                invoice_method = 'order'
            else:
                invoice_method = 'picking'
                
            self.write(cr, uid, porder, {'invoice_method': invoice_method})    
            
            po_old_names = []
            for po_old in res.get(porder):
                order_brw = self.browse(cr, uid, po_old, context)
                po_old_names.append(order_brw.name)
                req_id = order_brw.requisition_id.id
            
            for po_old_id in res.get(porder):
                po_old_obj = self.browse(cr, uid, po_old_id, context)
                pur_req_obj = self.pool.get('purchase.requisition').browse(cr, uid, po_old_obj.requisition_id.id, context)
                nota = pur_req_obj.description and '%s.\n '%(pur_req_obj.description,) or ''
                nota += """La Orden de Compra %s correspondiente a esta Solicitud de Compra ha sido fusionada con la(s)
                        Orden(es) de Compra %s para generar la Orden de Compra %s """ \
                    %(po_old_obj.name, ', '.join(set(po_old_names)-set(po_old_obj.name,)), po_new_obj.name)
                self.pool.get('purchase.requisition').write(cr, uid, po_old_obj.requisition_id.id, { 
                    'description': nota})
        return res
        
    def action_invoice_create(self, cr, uid, ids, *args):
        res = False
        property_obj = self.pool.get('ir.property')
        fp_obj =  self.pool.get('account.fiscal.position')
        journal_obj = self.pool.get('account.journal')
        for o in self.browse(cr, uid, ids):
            il = []
            todo = []
            for ol in o.order_line:
                if (not ol.product_id) or (ol.product_qty ==  0.0):
                    continue
                todo.append(ol.id)
                if ol.product_id:
                    acc_id = ol.product_id.product_tmpl_id.property_account_expense.id
                    if not acc_id:
                        acc_id = ol.product_id.categ_id.property_account_expense_categ.id
                    if not acc_id:
                        raise osv.except_osv(_('Error !'), _('There is no expense account defined for this product: "%s" (id:%d)') % (ol.product_id.name, ol.product_id.id,))
                else:
                    prop = property_obj.get(cr, uid, 'property_account_expense_categ', 'product.category')
                    acc_id = prop and prop.id or False
                fpos = o.fiscal_position or False
                acc_id = fp_obj.map_account(cr, uid, fpos, acc_id)
                if not acc_id:
                    raise osv.except_osv(_('Error !'),
                        _('There is no expense account defined in default Properties for Product Category or Fiscal Position is not defined !'))
                il.append(self.inv_line_create(cr, uid, acc_id, ol))
            acc_id = o.partner_id.property_account_payable.id
            journal_ids = journal_obj.search(cr, uid, [('type', '=','purchase'),('company_id', '=', o.company_id.id)], limit=1)
            if not journal_ids:
                raise osv.except_osv(_('Error !'),
                    _('There is no purchase journal defined for this company: "%s" (id:%d)') % (o.company_id.name, o.company_id.id))
            inv = {
                'name': o.partner_ref or o.name,
                'reference': o.partner_ref or o.name,
                'account_id': acc_id,
                'type': 'in_invoice',
                'partner_id': o.partner_id.id,
                'currency_id': o.pricelist_id.currency_id.id,
                'address_invoice_id': o.partner_address_id.id,
                'address_contact_id': o.partner_address_id.id,
                'journal_id': len(journal_ids) and journal_ids[0] or False,
                'origin': o.name,
                'invoice_line': il,
                'fiscal_position': o.fiscal_position.id or o.partner_id.property_account_position.id,
                'payment_term': o.partner_id.property_payment_term and o.partner_id.property_payment_term.id or False,
                'company_id': o.company_id.id,
            }
            inv_id = self.pool.get('account.invoice').create(cr, uid, inv, {'type':'in_invoice', 'journal_type': 'purchase'})
            self.pool.get('account.invoice').button_compute(cr, uid, [inv_id], {'type':'in_invoice'}, set_total=True)
            self.pool.get('purchase.order.line').write(cr, uid, todo, {'invoiced':True})
            self.write(cr, uid, [o.id], {'invoice_ids': [(4, inv_id)]})
            res = inv_id
        return res
        
    def action_picking_create(self,cr, uid, ids, *args):
        picking_id = False
        for order in self.browse(cr, uid, ids):
            loc_id = order.partner_id.property_stock_supplier.id
            istate = 'none'
            if order.invoice_method=='picking':
                istate = '2binvoiced'
            pick_name = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.in')
            picking_id = self.pool.get('stock.picking').create(cr, uid, {
                'name': pick_name,
                'origin': order.name+((order.origin and (':'+order.origin)) or ''),
                'type': 'in',
                'address_id': order.dest_address_id.id or order.partner_address_id.id,
                'invoice_state': istate,
                'purchase_id': order.id,
                'company_id': order.company_id.id,
                'move_lines' : [],
            })
            todo_moves = []
            for order_line in order.order_line:
                if (not order_line.product_id) or (order_line.product_qty ==  0.0):
                    continue
                if order_line.product_id.product_tmpl_id.type in ('product', 'consu'):
                    dest = order.location_id.id
                    move = self.pool.get('stock.move').create(cr, uid, {
                        'name': order.name + ': ' +(order_line.name or ''),
                        'product_id': order_line.product_id.id,
                        'product_qty': order_line.product_qty,
                        'product_uos_qty': order_line.product_qty,
                        'product_uom': order_line.product_uom.id,
                        'product_uos': order_line.product_uom.id,
                        'date': order_line.date_planned,
                        'date_expected': order_line.date_planned,
                        'location_id': loc_id,
                        'location_dest_id': dest,
                        'picking_id': picking_id,
                        'move_dest_id': order_line.move_dest_id.id,
                        'state': 'draft',
                        'purchase_line_id': order_line.id,
                        'company_id': order.company_id.id,
                        'price_unit': order_line.price_unit
                    })
                
                    if order_line.move_dest_id:
                        self.pool.get('stock.move').write(cr, uid, [order_line.move_dest_id.id], {'location_id':order.location_id.id})
                    todo_moves.append(move)
            self.pool.get('stock.move').action_confirm(cr, uid, todo_moves)
            self.pool.get('stock.move').force_assign(cr, uid, todo_moves)
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
        return picking_id
        
purchase_order()

class purchase_order_line(osv.osv):
    """Clase que hereda de purchase order line, esta contiene el metodo
    purchase_order_line_edit"""
    _inherit = "purchase.order.line"
    
    def save_order_line(self, cr, uid, ids, values, context=None):
        return  {'type': 'ir.actions.act_window_close'}
    
    
    #Muestra la vista al hacer click sobre el boton editar agregado por nosotros
    def purchase_order_line_edit(self, cr, uid, ids, context=None):
        return {
            'name':_("Purchase Order Line"),
            'view_mode': 'form',
            'view_id': False,
            'view_type': 'form',
            'res_model': 'purchase.order.line',
            'res_id': ids,
            'type': 'ir.actions.act_window',
            'nodestroy': False,
            'target': 'new',
            'domain': '[]',
            'ref': 'purchase_order_line_form',
    }
    
purchase_order_line()

class purchase_requisition_partner(osv.osv_memory):
    _inherit = "purchase.requisition.partner"
    _logger = logging.getLogger('sisb.purchase.requisition.partner')
    
    def create_order(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        record_ids = context and context.get('active_ids', False)
        if record_ids:
            data =  self.read(cr, uid, ids)
            company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
            order_obj = self.pool.get('purchase.order')
            order_line_obj = self.pool.get('purchase.order.line')
            partner_obj = self.pool.get('res.partner')
            tender_line_obj = self.pool.get('purchase.requisition.line')
            pricelist_obj = self.pool.get('product.pricelist')
            prod_obj = self.pool.get('product.product')
            tender_obj = self.pool.get('purchase.requisition')
            acc_pos_obj = self.pool.get('account.fiscal.position')
            partner_id = data[0]['partner_id']

            supplier_data = partner_obj.browse(cr, uid, partner_id, context=context)

            address_id = partner_obj.address_get(cr, uid, [partner_id], ['delivery'])['delivery']
            list_line=[]
            purchase_order_line={}
            for tender in tender_obj.browse(cr, uid, record_ids, context=context):
                for line in tender.line_ids:
                    partner_list = sorted([(partner.sequence, partner) for partner in  line.product_id.seller_ids if partner])
                    partner_rec = partner_list and partner_list[0] and partner_list[0][1] or False
                    uom_id = line.product_uom_id.id

                    if tender.date_start:
                        newdate = datetime.strptime(tender.date_start, '%Y-%m-%d %H:%M:%S') - relativedelta(days=company.po_lead)
                    else:
                        newdate = datetime.today() - relativedelta(days=company.po_lead)
                    delay = partner_rec and partner_rec.delay or 0.0
                    if delay:
                        newdate -= relativedelta(days=delay)

                    partner = partner_rec and partner_rec.name or supplier_data
                    pricelist_id = partner.property_product_pricelist_purchase and partner.property_product_pricelist_purchase.id or False
                    price = pricelist_obj.price_get(cr, uid, [pricelist_id], line.product_id.id, line.product_qty, False, {'uom': uom_id})[pricelist_id]
                    product = prod_obj.browse(cr, uid, line.product_id.id, context=context)
                    location_id = self.pool.get('stock.warehouse').read(cr, uid, [tender.warehouse_id.id], ['lot_input_id'])[0]['lot_input_id'][0]
                    
                    purchase_order_line= {
                            'name': product.partner_ref,
                            'product_qty': line.product_qty,
                            'product_id': line.product_id.id,
                            'product_uom': uom_id,
                            'price_unit': price,
                            'date_planned': newdate.strftime('%Y-%m-%d %H:%M:%S'),
                            'notes': line.description,
                            #~ 'notes': product.description_purchase,
                    }
                    if product.product_tmpl_id.type == 'service':
                        invoice_method = 'order'
                        purchase_order_line.update({
                            'name' : line.description,
                            })
                    else:
                        invoice_method = 'picking'
                    taxes_ids = line.product_id.product_tmpl_id.supplier_taxes_id
                    taxes = acc_pos_obj.map_tax(cr, uid, partner.property_account_position, taxes_ids)
                    purchase_order_line.update({
                            'taxes_id': [(6,0,taxes)]
                        })
                    list_line.append(purchase_order_line)
                    
                purchase_id = order_obj.create(cr, uid, {
                            'origin': tender.purchase_ids and tender.purchase_ids[0].origin or tender.name,
                            'partner_id': partner_id,
                            'partner_address_id': address_id,
                            'pricelist_id': pricelist_id,
                            'location_id': tender.purchase_ids and tender.purchase_ids[0].location_id.id or line.product_id.product_tmpl_id.property_stock_production.id,
                            'company_id': tender.company_id.id,
                            'fiscal_position': partner.property_account_position and partner.property_account_position.id or False,
                            'requisition_id':tender.id,
                            'notes':tender.description,
                            'warehouse_id':tender.warehouse_id.id and tender.warehouse_id.id,
                            'location_id':location_id,
                            'company_id':tender.company_id.id,
                            'invoice_method' : invoice_method,
                })
                order_ids=[]
                for order_line in list_line:
                    order_line.update({
                            'order_id': purchase_id
                        })
                    order_line_obj.create(cr,uid,order_line)
        return {'type': 'ir.actions.act_window_close'}
    
purchase_requisition_partner()

class purchase_order_group(osv.osv_memory):
    _inherit = "purchase.order.group"
    
    def merge_orders(self, cr, uid, ids, context=None):
        res = super(purchase_order_group, self).merge_orders(cr, uid, ids, context)
        res['target'] = 'edit'
        return res
        
purchase_order_group()
