# -*- coding: utf-8 -*-
"""
Módulo para traspasos a sucursales
"""
from osv import fields, osv
import time
from tools.translate import _
import logging

class rutas(osv.osv):
    _name = 'rutas'
    _description = 'Rutas para traspasos entre sucursales'

    _columns = {
        'name':fields.char('Referencia de Ruta', size=128, required=True),
        'orig_a': fields.many2one('stock.location', 'Ubicacion origen',
          required=True),
        'dest_a': fields.many2one('stock.location', 'Ubicacion destino',
          required=True),
        'orig_b': fields.many2one('stock.location', 'Ubicacion origen',
          required=True),
        'dest_b': fields.many2one('stock.location', 'Ubicacion destino',
          required=True),
        'user_ids': fields.many2many('res.users', 'per_ubicacion_rel',
                            'rutas_id', 'user_id', 'Usuarios por Ubicacion'),
    }

rutas()

class traspasos(osv.osv):
    _name = 'traspasos'
    _description = 'Traspasos a Sucursales'
    _logger = logging.getLogger('sisb.traspasos')

    def _get_shipping(self, cr, uid, ids, field_name, arg, context=None):
        if not type(ids) == type([]):
            ids = [ids]
        d = {}
        l = []
        a = []
        b = []
        shipping_obj = self.pool.get('shipping.move')
        picking_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        for traspaso in self.browse(cr, uid, ids, context=context):
            shipping_ids = shipping_obj.search(cr, uid, [('traspaso_id','=',traspaso.id)])
            if traspaso.state not in ['done','cancel']:
                shipping_brw = shipping_obj.browse(cr, uid, shipping_ids)
                picking_ids = picking_obj.search(cr, uid, [('origin','=',traspaso.name)])
                for s in shipping_brw:
                    l.append(s.picking_id.id)
                for p in picking_ids:
                    picking_brw = picking_obj.browse(cr, uid, p)
                    picking_state = picking_brw.state
                    name = picking_brw.name
                    if p in l:
                        continue
                    else:
                        move_ids = move_obj.search(cr, uid, [('picking_id', '=', p)])
                        if not move_ids:
                            continue
                        else:
                            move_brw = move_obj.browse(cr, uid, move_ids)
                            for m in move_brw:
                                if m.location_id.id == traspaso.ruta_id.orig_a.id and m.location_dest_id.id == traspaso.ruta_id.dest_a.id:
                                    a.append(m.id)
                                if m.location_id.id == traspaso.ruta_id.orig_b.id and m.location_dest_id.id == traspaso.ruta_id.dest_b.id:
                                    b.append(m.id)
                                if len(a) > 0 and len(b) == 0:
                                    if picking_state == 'cancel':
                                        estado = 'cancel'
                                    else:
                                        estado = 'draft'
                                if len(a) == 0 and len(b) > 0:
                                    if picking_state == 'cancel':
                                        estado = 'cancel'
                                    elif picking_state == 'done':
                                        estado = 'done'
                                    else:
                                        estado = 'auto'
                                if len(a) == 0 and len(b) == 0:
                                    estado = False
                            if estado:
                                if shipping_obj.search(cr, uid, [('picking_id','=',p)]):
                                    if field_name:
                                        msg = 'El albarán: {0} ya se encuentra asignado a otro traspaso, por favor verifique.'.format(name)
                                        picking_obj.log(cr, uid, p, msg)
                                else:
                                    shipping_id = shipping_obj.create(cr, uid, {
                                        'picking_id': p,
                                        'traspaso_id': traspaso.id,
                                        'state': estado,
                                    })
                                    shipping_ids.append(shipping_id)
                            else:
                                picking_brw = picking_obj.browse(cr, uid, p)
                                if field_name:
                                    msg = 'El albarán: {0} no pertenece a la ruta, por favor verifique.'.format(name)
                                    picking_obj.log(cr, uid, p, msg)
            d[traspaso.id] = shipping_ids
        return d

    _columns = {
        'name':fields.char('Referencia de Traspaso', size=128),
        'date': fields.date('Fecha', required=True),
        'state': fields.selection([
            ('draft', 'Borrador'),
            ('auto', 'Espera'),
            ('done', 'Procesado'),
            ('exception', 'Excepción'),
            ('cancel', 'Cancelado')], 'Estado', readonly=True),
        'ruta_id': fields.many2one('rutas', 'Ruta para traspaso', required=True),
        'line_ids': fields.one2many('traspasos.lineas',
          'traspaso_id', 'Traspasos Lineas', states={'draft': [('readonly', False)],
          'auto': [('readonly', True)],'done': [('readonly', True)],'exception': [('readonly', True)],
          'cancel': [('readonly', True)]}),
        'company_id': fields.many2one('res.company','Company',required=True),
        'shipping_ids': fields.function(_get_shipping, method=True,
          type='one2many', relation='shipping.move', string='Movimientos'),
    }

    _order = "id desc"

    _defaults = {
        'date': lambda *args: time.strftime('%Y-%m-%d'),
        'state': 'draft',
        'company_id': lambda self, cr, uid, c:
          self.pool.get('res.company')._company_default_get(cr, uid,
            'traspasos', context=c)
    }

    # Para generar de forma correcta la secuencia
    def create(self, cr, uid, values, context=None):
        ids = super(traspasos, self).create(cr, uid, values, context)
        self.write(cr, uid, ids, {'name': self.pool.get('ir.sequence').get(cr, uid, 'traspasos')})
        return ids

    # Para eliminar las lineas del traspaso cuando se elimine la cabecera
    def unlink(self,cr, uid, ids, context=None):
        if self.search(cr, uid, [('id','=',ids[0]),('state','!=','draft')]):
            raise osv.except_osv(_('Error !'), _('No es posible eliminar el traspaso.'))
        return super(traspasos, self).unlink(cr, uid, ids, context)

    # Al "procesar" el traspaso se ejecuta esta función
    def solicitar_traspaso(self,cr, uid, ids, context):
        d = {}
        lineas_traspaso_obj = self.pool.get('traspasos.lineas')
        lineas_ids = lineas_traspaso_obj.search(cr, uid, [('traspaso_id', 'in',
                                                list(set(ids)))])
        if not lineas_ids:
             raise osv.except_osv(_('Error !'), _('Por favor agregue lineas.'))
        context['tipo'] = True
        d = self.generar_albaran(cr, uid, ids, context=context)
        if not d['result']:
            raise osv.except_osv(_('Error !'), _(d['msg']))
        return True

    def generar_albaran(self, cr, uid, ids, context):
        """
        Si context['tipo'] = True, el albarán generado corresponde al segmento A
        de la ruta especificada, si es False el alabarán corresponde al segmento
        B de la ruta, ademas crea la linea correspondiente en shipping_move.
        """
        if not type(ids) == type([]):
            ids = [ids]
        shipping_obj = self.pool.get('shipping.move')
        picking_obj = self.pool.get('stock.picking')
        stock_obj = self.pool.get('stock.location')
        move_obj = self.pool.get('stock.move')
        lineas_traspaso_obj = self.pool.get('traspasos.lineas')
        traspasos_lineas_ids = lineas_traspaso_obj.search(cr, uid, [('traspaso_id', 'in',
                                                list(set(ids)))])
        traspasos_lineas_brw = lineas_traspaso_obj.browse(cr, uid, traspasos_lineas_ids)
        traspasos_brw = self.browse(cr, uid, ids)
        for t in traspasos_brw:
            ruta = t.ruta_id
            doc = t.name
            state = t.state
        picking_ids = picking_obj.search(cr, uid, [('origin','=',doc)])
        """ Si el traspaso no tiene pickings asociados es porque se va a generar
        albarán del primer segmento de la ruta, por ello se asignan las variables
        pertinentes, ademas se pregunta por el valor del "tipo" en el context ya
        que la funcion pudiese ser llamada desde otro modelo (hoja de carga) para
        generar el albarán del segmento B de la ruta """
        if not picking_ids:
            if context['tipo']:
                    origen = ruta.orig_a.id
                    destino = ruta.dest_a.id
                    address = ruta.dest_a.address_id.id
        else:
            move_ids = move_obj.search(cr, uid, [('picking_id', 'in',
                                        list(set(picking_ids)))])
            move_brw = move_obj.browse(cr, uid, move_ids)
            for s in move_brw:
                if s.location_id.id == ruta.orig_a.id and s.location_dest_id.id == ruta.dest_a.id and s.state == 'done':
                    verificado = True
                else:
                    verificado = False
            """
            El context['tipo'] = False, vendra del llamado a la función desde el
            boton del módulo instalable o desde la hoja de carga.
            Se debe revisar "len(picking_ids) == 1" ya que del albaran A pueden
            surgir varios si el mismo se procesa por partes.
            """
            if len(picking_ids) == 1 and not context['tipo'] and verificado and state == 'auto':
                origen = ruta.orig_b.id
                destino = ruta.dest_b.id
                address = ruta.dest_b.address_id.id
            else:
                if not verificado:
                    return {'result': False, 'msg': 'Verifique el origen y destino del albarán, no pertenece a la ruta asociada al traspaso.',}
                else:
                    return {'result': False, 'msg': 'Verifique el estado del albarán.',}
        if not address:
            return {'result': False, 'msg': 'La ubicación destino debe poseer dirección, no se ha generado el albaran.',}
        pick_name = self.pool.get('ir.sequence').get(cr, uid,
                  'stock.picking.internal')
        picking_id = picking_obj.create(cr, uid, {
            'name': pick_name,
            'origin': traspasos_brw[0].name or '',
            'address_id': address,
            'type': 'internal',
            'invoice_state': 'none',
            'traspaso_id': traspasos_brw[0].id,
            'company_id': traspasos_brw[0].company_id.id,
            'move_lines': [],
        })
        if not picking_id:
            self.write(cr, uid, ids, {'state': 'exception'}, context=context)
            return {'result': False, 'msg': 'No se pudo procesar.',}
        else:
            picking_brw = picking_obj.browse(cr, uid, picking_id)
            for l in traspasos_lineas_brw:
                values = {
                    'name': l.product_id.name_get()[0][1],
                    'product_id': l.product_id.id,
                    'product_qty': l.product_qty,
                    'product_uos_qty': l.product_qty,
                    'product_uom': l.product_uom.id,
                    'product_uos': l.product_uom.id,
                    'date': l.date_planned,
                    'date_expected': l.date_planned,
                    'location_id': origen,
                    'location_dest_id': destino,
                    'picking_id': picking_id,
                    'state': 'draft',
                    'traspaso_line_id': l.id,
                    'company_id': traspasos_brw[0].company_id.id,
                }
                move_id = move_obj.create(cr, uid, values)
            msg = 'Se ha generado el albarán: {0}'.format(picking_brw.name)
            picking_obj.log(cr, uid, picking_id, msg)
        self.write(cr, uid, ids, {'state': 'auto',})
        return {'result': True,}

    def comprobar_estado(self, cr, uid, ids=False, context=None):
        self._logger.info("traspasos starting function")
        if not ids:
            ids = self.search(cr, uid, [('state','in',['auto','exception'])])
        picking_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        shipping_obj = self.pool.get('shipping.move')
        
        #~ ARREGLO LOS SHIPPING_MOVE DE CADA PICKING
        for t in self.browse(cr, uid, ids, context=context):
            
            shipping_list = []
            d_a = {}
            d_b = {}
        
            #~ ARMO LA LISTA CON LOS PICKING_IDS DEL TRASPASO ENCONTRADOS EN SHIPPING_MOVE
            shipping_brw = t.shipping_ids
            for s in shipping_brw:
                shipping_list.append(s.picking_id.id)
            picking_brw = picking_obj.browse(cr, uid, shipping_list)
            
            #~ RECORRO LOS PICKING ASOCIADOS AL TRASPASO EXISTENTES EN SHIPING_MOVE
            for pick in picking_brw:
                move_ids = move_obj.search(cr, uid, [('picking_id','=',pick.id),])
                move_brw = move_obj.browse(cr, uid, move_ids)
                #~ GUARDO EL ID DEL SHIPPING PARA ESTE PICKING PARA EL WRITE
                ship_id = shipping_obj.search(cr, uid, [('picking_id','=', pick.id)])
                #~ VERIFICO SI PERTENECE AL SEGMENTO A DE LA RUTA
                if move_brw[0].location_id.id == t.ruta_id.orig_a.id and move_brw[0].location_dest_id.id == t.ruta_id.dest_a.id:
                    #~ SOBREESCRIBO EL ESTADO DE LOS SHIPPING DE ACUERDO AL ESTADO DEL PICKING
                    if pick.state == 'cancel':
                        shipping_obj.write(cr, uid, ship_id, {'state': 'cancel',})
                    elif pick.state == 'done':
                        continue
                    else:
                        shipping_obj.write(cr, uid, ship_id, {'state': 'draft',})
                #~ VERIFICO SI PERTENECE AL SEGMENTO B DE LA RUTA
                elif move_brw[0].location_id.id == t.ruta_id.orig_b.id and move_brw[0].location_dest_id.id == t.ruta_id.dest_b.id:
                    #~ SOBREESCRIBO EL ESTADO DE LOS SHIPPING DE ACUERDO AL ESTADO DEL PICKING
                    if pick.state == 'cancel':
                        shipping_obj.write(cr, uid, ship_id, {'state': 'cancel',})
                    elif pick.state == 'done':
                        shipping_obj.write(cr, uid, ship_id, {'state': 'done',})
                    else:
                        shipping_obj.write(cr, uid, ship_id, {'state': 'auto',})
                else:
                    shipping_obj.write(cr, uid, ship_id, {'state': 'exception',})
            #~ VALIDO SEGUN LOS ESTADOS DE LOS SHIPPING EL CAMBIO DE ESTADO DEL TRASPASO
            move_a = move_obj.search(cr, uid, [('picking_id','in',shipping_list),
              ('location_id','=',t.ruta_id.orig_a.id),
              ('location_dest_id','=',t.ruta_id.dest_a.id),
              ('picking_id','in',shipping_list),('state','=','done')])
            states_a = move_obj.search(cr, uid, [('picking_id','in',shipping_list),
              ('location_id','=',t.ruta_id.orig_a.id),
              ('location_dest_id','=',t.ruta_id.dest_a.id),
              ('picking_id','in',shipping_list),('state','in',['draft','confirmed','assigned'])])
            cancel_a = move_obj.search(cr, uid, [('picking_id','in',shipping_list),
              ('location_id','=',t.ruta_id.orig_a.id),
              ('location_dest_id','=',t.ruta_id.dest_a.id),
              ('picking_id','in',shipping_list),('state','=','cancel')])
            states_b = move_obj.search(cr, uid, [('picking_id','in',shipping_list),
              ('location_id','=',t.ruta_id.orig_b.id),
              ('location_dest_id','=',t.ruta_id.dest_b.id),
              ('picking_id','in',shipping_list),('state','in',['draft','confirmed','assigned'])])
            move_b = move_obj.search(cr, uid, [('picking_id','in',shipping_list),
              ('location_id','=',t.ruta_id.orig_b.id),
              ('location_dest_id','=',t.ruta_id.dest_b.id),
              ('picking_id','in',shipping_list),('state','!=','cancel')])
            cancel_b = move_obj.search(cr, uid, [('picking_id','in',shipping_list),
              ('location_id','=',t.ruta_id.orig_b.id),
              ('location_dest_id','=',t.ruta_id.dest_b.id),
              ('picking_id','in',shipping_list),('state','=','cancel')])
            move_brw_a = move_obj.browse(cr, uid, move_a)
            move_brw_b = move_obj.browse(cr, uid, move_b)
            for m_a in move_brw_a:
                d_a[m_a.product_id.id] = d_a.get(m_a.product_id.id, 0) + m_a.product_qty
            for m_b in move_brw_b:
                d_b[m_b.product_id.id] = d_b.get(m_b.product_id.id, 0) + m_b.product_qty
            flag = True
            shipping_ids = shipping_obj.search(cr, uid, [('traspaso_id', '=', t.id),('state','in',['draft','auto'])])
            shipping_brw = shipping_obj.browse(cr, uid, shipping_ids)
            
            if cancel_a and len(move_a) == 0 and len(states_a) == 0 and len(move_b) == 0:
                self.write(cr, uid, t.id, {'state': 'cancel',})
            elif states_a or states_b:
                self.write(cr, uid, t.id, {'state': 'auto',})
            else:
                if not (states_a and states_b):
                    if not (len(d_a.items()) > 0 and len(d_b.items()) > 0 and len(shipping_brw) > 0):
                        if len(d_a.items()) == len(d_b.items()):
                            for v in d_a:
                                if not d_a[v] == d_b[v]:
                                    flag = False
                            if flag:
                                self.write(cr, uid, t.id, {'state': 'done',})
                            else:
                                self.write(cr, uid, t.id, {'state': 'exception',})
                        elif len(d_a.items()) != len(d_b.items()) and len(cancel_b) > 0:
                            self.write(cr, uid, t.id, {'state': 'exception',})
        self._logger.info("traspasos ending function")
        return True

    def cancelar(self, cr, uid, ids, context=None):
        if not type(ids) == type([]):
            ids = [ids]
        traspaso_brw = self.browse(cr, uid, ids)
        for traspaso in traspaso_brw:
            if traspaso.state == 'draft':
                self.write(cr, uid, traspaso.id, {'state': 'cancel'})

traspasos()

class traspasos_lineas(osv.osv):
    _name = 'traspasos.lineas'
    _description = 'Lineas de Traspasos'

    _columns = {
        'name': fields.char('Lineas de Traspasos', size=129),
        'product_id': fields.many2one('product.product', 'Producto',
          required=True),
        'product_qty': fields.float('Cantidad', required=True,
          states={'draft': [('readonly', False)]}, readonly=True,
          digits=(16, 2)),
        'product_uom': fields.many2one('product.uom', 'Unidad de Medida',
          required=True, states={'draft': [('readonly', False)]},
          readonly=True),
        'date_planned': fields.date('Fecha Planificada', required=True),
        'traspaso_id': fields.many2one('traspasos',
          'Traspaso', required=True, ondelete='cascade'),
    }

    def _check_dates(self, cr, uid, ids, context=None):
        for despacho in self.browse(cr, uid, ids, context=context):
            if despacho.date_planned < time.strftime('%Y-%m-%d'):
                return False
        return True

    def _check_qtys(self, cr, uid, ids, context=None):
        for despacho in self.browse(cr, uid, ids, context=context):
            if despacho.product_qty <= 0:
                return False
        return True

    _constraints = [(_check_dates, "La fecha debe ser posterior a la actual.", ['date_planned']),
      (_check_qtys, "La cantidad ingresada debe ser mayor a cero.", ['product_qty'])]

    def on_change_product_id(self, cr, uid, ids, product_id, uom=False):
        """ On change of product id.
        @param product_id: Changed Product id
        @param uom: Changed Product uom
        @return: Dictionary of values
        """
        context = {}
        if product_id:
            product_uom = self.pool.get('product.product').browse(cr,
              uid, product_id)
            result = {
                'product_id': product_id,
                'product_uom': product_uom.uom_id.id,
            }
        else:
            result = {'product_id': product_id, 'product_uom': uom,}
        return {'value': result}

traspasos_lineas()

# Para llevar el control del estado del traspaso
class shipping_move(osv.osv):
    _name = 'shipping.move'
    _description = 'Movimientos de Traspasos'

    _columns = {
        'picking_id': fields.many2one('stock.picking', 'albaranes', required=True),
        'traspaso_id': fields.many2one('traspasos', 'traspasos', required=True),
        'state': fields.selection([
            ('draft', 'Listo para enviar'),
            ('auto', 'Espera'),
            ('done', 'Procesado'),
            ('exception', 'Excepción'),
            ('cancel', 'Cancelado'),
            ], 'Estado',),
    }

    def validate(self, cr, uid, ids, context):
        if not type(ids) == type([]):
            ids = [ids]
        tlineas_obj = self.pool.get('traspasos.lineas')
        picking_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        shipping_brw = self.browse(cr, uid, ids)
        for t in shipping_brw:
            move_ids = move_obj.search(cr, uid, [('picking_id', '=', t.picking_id.id)])
            move_brw = move_obj.browse(cr, uid, move_ids)
            tlineas_ids = tlineas_obj.search(cr, uid, [('traspaso_id', '=', t.traspaso_id.id)])
            tlineas_brw = tlineas_obj.browse(cr, uid, tlineas_ids)
            for s in move_brw:
                if t.picking_id.state != 'done':
                    raise osv.except_osv(_('Error !'), _('Aún no puede ser procesado, verifique el estado del albarán: {0}'.format(t.picking_id.name)))
                if s.location_id.id == t.traspaso_id.ruta_id.orig_a.id and s.location_dest_id.id == t.traspaso_id.ruta_id.dest_a.id and t.picking_id.state == 'done':
                    validate = True
                else:
                    validate = False
            if not validate:
                return False
            if not t.traspaso_id.ruta_id.dest_b.address_id.id:
                raise osv.except_osv(_('Error !'), _('La ubicación destino debe poseer dirección, no se ha generado el albaran.'))
            if validate == True and t.picking_id.state == 'done':
                istate = 'none'
                picking_name = self.pool.get('ir.sequence').get(cr, uid,
                          'stock.picking.internal')
                picking_id = picking_obj.create(cr, uid, {
                    'name': picking_name,
                    'origin': t.traspaso_id.name or '',
                    'address_id': t.traspaso_id.ruta_id.dest_b.address_id.id,
                    'type': 'internal',
                    'invoice_state': istate,
                    'traspaso_id': t.traspaso_id.id,
                    'company_id': t.traspaso_id.company_id.id,
                    'move_lines': [],
                })
                if not picking_id:
                    raise osv.except_osv(_('Error !'), _('No se pudo procesar'))
                picking_brw = picking_obj.browse(cr, uid, picking_id)
                for l in move_brw:
                    values = {
                        'name': t.traspaso_id.name,
                        'product_id': l.product_id.id,
                        'product_qty': l.product_qty,
                        'product_uos_qty': l.product_qty,
                        'product_uom': l.product_uom.id,
                        'product_uos': l.product_uom.id,
                        'date': l.date_expected,
                        'date_expected': l.date_expected,
                        'location_id': t.traspaso_id.ruta_id.orig_b.id,
                        'location_dest_id': t.traspaso_id.ruta_id.dest_b.id,
                        'picking_id': picking_id,
                        'state': 'draft',
                        'traspaso_line_id': tlineas_brw[0].id,
                        'company_id': t.traspaso_id.company_id.id,
                    }
                    move_id = move_obj.create(cr, uid, values)
                    
            msg = 'Se ha generado el albarán: {0}'.format(picking_brw.name)
            picking_obj.log(cr, uid, picking_id, msg)
        return self.write(cr, uid, ids, {'state': 'done',})

    def validate_hoja_de_carga(self, cr, uid, ids, context):
        if not type(ids) == type([]):
            ids = [ids]
        tlineas_obj = self.pool.get('traspasos.lineas')
        move_obj = self.pool.get('stock.move')
        shipping_brw = self.browse(cr, uid, ids)
        for t in shipping_brw:
            move_ids = move_obj.search(cr, uid, [('picking_id', '=', t.picking_id.id)])
            move_brw = move_obj.browse(cr, uid, move_ids)
            tlineas_ids = tlineas_obj.search(cr, uid, [('traspaso_id', '=', t.traspaso_id.id)])
            tlineas_brw = tlineas_obj.browse(cr, uid, tlineas_ids)
            for s in move_brw:
                if t.picking_id.state != 'done':
                    return {'result': False, 'msg': 'Verifique el estado del albarán, no se ha generado el albaran.',}
                if s.location_id.id == t.traspaso_id.ruta_id.orig_a.id and s.location_dest_id.id == t.traspaso_id.ruta_id.dest_a.id and t.picking_id.state == 'done':
                    validate = True
                else:
                    validate = False
            if not validate:
                return {'result': False, 'msg': 'Verifique permiso a ubicaciones de origen y destino del albarán, no se ha generado el albaran.',}
            if not t.traspaso_id.ruta_id.dest_b.address_id.id:
                return {'result': False, 'msg': 'La ubicación destino del segmento B de la ruta debe poseer dirección, no se ha generado el albarán.',}
        return {'result': True,}

shipping_move()

# Relacion usuarios y rutas permitidas
class res_users(osv.osv):
    _inherit = 'res.users'

    _columns = {
        'rutas_ids': fields.many2many('rutas',
          'per_ubicacion_rel', 'user_id', 'rutas_id',
          'Usuarios por ubicacion'),
    }

res_users()
