# -*- coding: utf-8 -*-
"""
Módulo utilizado para adaptar el módulo de ventas de Industrias Diana
"""
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import re
import time
import netsvc
from osv import fields, osv
from tools import config
from tools.translate import _

from product._common import rounding

class res_partner(osv.osv):
    _inherit = 'res.partner'
    
    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        if not args:
           args=[]
        if name:
           ids = self.search(cr, user, [('ref','=',name)]+ args, limit=limit, context=context)
           if not len(ids):
               ids = self.search(cr, user, ['|',('name',operator,name),('ref',operator,name)] + args, limit=limit, context=context)
           if not len(ids):
               ptrn=re.compile('(\[(.*?)\])')
               res = ptrn.search(name)
               if res:
                   ids = self.search(cr, user, [('ref','=', res.group(2))] + args, limit=limit, context=context)
        else:
           ids = self.search(cr, user, args, limit=limit, context=context)
        result = self.name_get(cr, user, ids, context=context)
        return result

res_partner()

class sale_order(osv.osv):
    _inherit = "sale.order"

    _defaults = {
        'name':'/',
    }
    
    #Validaciones tanto para crear como para actualizar
    def validaciones_registros(self,values):
        
        if values.get('picking_policy',False):
            if values['picking_policy'] != 'one':
                raise osv.except_osv(_('Error!'),_('Debe indicar la politica de envio completo!'))
                
        if values.get('order_policy',False):
            if values['order_policy'] == 'picking':
                raise osv.except_osv(_('Error!'),_('La politica de facturacion no puede ser factura desde albaran!'))

        return True
        
        
    def create(self, cr, uid, values, context=None):

        self.validaciones_registros(values)

        res = super(sale_order, self).create(cr, uid, values, context)
        values = {
            'name':self.pool.get('ir.sequence').get(cr, uid, 'sale.order')
            }
        seq = self.write(cr, uid, res, values, context)
        
        if not seq:
            raise osv.except_osv(_('Error!'),_('Hubo error en el proceso de creacion de la secuencia. Contactar el administrador!'))
    
        return res
        
    def write(self, cr, user, ids, vals, context=None):
        #validar cambio de cliente habiendo lineas registradas (puede haber cambio de tarifa)
        #SI hubo cambio de partner_id y ya habia registrado lineas de pedido
        #no permitir cambios
        
        if vals.get('partner_id',False) and vals.get('pricelist_id',False):
            so_brow = self.browse(cr,user,ids)
            if so_brow[0].order_line:
                raise osv.except_osv(_('Error!'),_('Hubo cambio de cliente y tarifas. Debe: 1ro eliminar las lineas del pedido y luego presionar guardar y editar para realizar el registro!'))
        
        #############
        
        self.validaciones_registros(vals)
        
        record = super(sale_order, self).write(cr, user, ids, vals, context)
        
        return record
    
    def copy(self, cr, uid, id, defaults, context=None): #Sobreescritura del metodo copy para no permitir duplicar objetos stock_picking
        raise osv.except_osv(_('Acción Denegada!'),_('No se permite duplicar Pedidos!'))
        return False
    
    def check_limit(self, cr, uid, ids, context={}):
        if context is None:
            context = {}
        
        ##############################################################################################################
        #Se verifica si hay lineas en el pedido de venta
        ##############################################################################################################
        obj_so_lines = self.pool.get('sale.order.line')
        so_lines = obj_so_lines.search(cr, uid, [('order_id', '=', ids[0])])
        
        if not so_lines:
            raise osv.except_osv(_('Error !'), _('Por favor ingresar las líneas del pedido de venta antes de continuar!'))
            return False
        ###############################################################################################################
        
        #lineas del pedido
        br_so_lines = obj_so_lines.browse(cr, uid, so_lines, context)
        
        #############################################################################################################
        #Se verifica esta asociado el producto en cada linea del pedido
        #############################################################################################################
        existe_producto = True
        for linea in br_so_lines:
            if not linea.product_id:
                existe_producto = False
                break
        if not existe_producto:
            raise osv.except_osv(_('Error !'), _('No se puede procesar el pedido. No hay producto asociado en alguna de las lineas.!'))
            return False
        #############################################################################################################
        
        #############################################################################################################
        #Validar existencia en almacen
        #############################################################################################################
        
        #Se valida la existencia en la ubicacion de permanencia
        obj_product = self.pool.get('product.product')
        uom_obj = self.pool.get('product.uom')
        for linea in br_so_lines:
            if linea.product_id and linea.product_id.type != 'service':
                c= context.copy()
                c.update({ 'location': (linea.order_id.shop_id.warehouse_id.lot_stock_id.id,)})
                c.update({ 'states': ('done',), 'what': ('in', 'out') })
                
                stock = obj_product.get_product_available(cr, uid, [linea.product_id.id], context=c)
                converted_qty = linea.product_uom_qty
                if linea.product_uom.id <> linea.product_id.uom_id.id:
                    converted_qty = uom_obj._compute_qty(cr, uid, linea.product_uom.id, linea.product_uom_qty, linea.product_id.uom_id.id)
                
                if converted_qty > stock[linea.product_id.id]:
                    raise osv.except_osv(_('Error !'), _('No hay existencia suficiente del producto %s para procesar el pedido!')%linea.product_id.name)
                    return False
        #############################################################################################################
        
        
        
        #Se verificar si se forza el cambio de estado
        prueba = self.pool.get('sisb.log.forzar.presupuesto').search(cr, uid, [('order_id', '=', ids[0])])
        if prueba:
            return True
        else:
            
            #############################################################################################################
            #Validar limite de credito
            #############################################################################################################
            #Se modifica para tomar en cuenta sólo el saldo del cliente 
            #sin restar la deuda que la empresa tiene, en caso de que el cliente sea proveedor tambien
            
            so = self.browse(cr, uid, ids[0], context)
            partner_id = so.partner_id.id
            limite_credito = so.partner_id.credit_limit
            deuda = so.partner_id.credit
            monto = so.amount_total
            saldo=0.0
            
            #Excluir los presupuestos, documentos cancelados y pedidos facturados (asumiendo que toda pedido de venta esta asociado a las facturas generadas por ese pedido)
            otros_so = self.search(cr, uid, [('partner_id ', '=', partner_id),('state','not in',('draft','cancel')),('invoiced_rate','<',100.00)])
            so_list = self.browse(cr, uid, otros_so, context)
            
            for so_en_proceso in so_list:
                saldo += so_en_proceso.amount_total - so_en_proceso.amount_total*so_en_proceso.invoiced_rate/100
            
            
            #Incluir las facturas de cliente en borrador y que no estan asociados a una orden de venta
            #lista de facturas asociadas a la orden de venta
            sale_order_partner = self.search(cr, uid, [('partner_id ', '=', partner_id),('state','not in',('draft','cancel'))])
            sop_list = self.browse(cr, uid, sale_order_partner, context)
            
            list_inv = []
            for so_item in sop_list:
                for inv in so_item.invoice_ids:
                    list_inv.append(inv.id)
            
            obj_ai = self.pool.get('account.invoice')
            fact_draft = obj_ai.search(cr, uid, [('partner_id','=',so.partner_id.id),('type','=','out_invoice'),('state','=','draft'),('id','not in',list_inv)])
            fact_draft_list = obj_ai.browse(cr, uid, fact_draft, context)
            
            acum_draft = 0.0
            for fd_item in fact_draft_list:
                acum_draft +=  fd_item.amount_total
            
            #Verificar si se va a tomar en cuenta la fecha de vencimiento del asiento contable
            if(saldo + deuda + monto + acum_draft) > limite_credito:
                if not so.partner_id.over_credit:
                    msg = 'No se puede confirmar la orden de venta por %s. \n' \
                        'Monto por Facturar %s. \n' \
                        'Monto Facturado sin pagar %s. \n' \
                        'Monto de facturas sin orden de venta asociadas en borrador %s. \n' \
                        'La suma de ellas supera el límite de credito de %s. Revise las cuentas contables del cliente o su límite de crédito !' % (monto, saldo, deuda, acum_draft, limite_credito)
                    raise osv.except_osv(_('Credit Over Limits !'), _(msg))
                    return False
                else:
                    self.pool.get('res.partner').write(cr, uid, [partner_id], {'credit_limit':saldo + monto})
                    return True
            else:
                return True
            ###############################################################################################################
                
    def _make_invoice(self, cr, uid, order, lines, context=None):
        #Sobreescritura del método para crear factura desde orden de venta
        #con la finalidad de registrar el journal_id asociado a la sucursal (tienda)
        invoice = super(sale_order, self)._make_invoice(cr, uid, order, lines,  context=context)
        if not order.shop_id.journal_id.id: 
            return invoice #Toma el journal de la factura de venta por defecto
        else:
            if self.pool.get('account.invoice').write(cr, uid, invoice, {'journal_id':order.shop_id.journal_id.id, 'address_shipping_id':order.partner_shipping_id.id }):
                return invoice
            else:
                raise osv.except_osv(_('Error!'),_('There were problems in the process'))
                return False
    
    def action_ship_create(self, cr, uid, ids, *args):
        #~ sale_order_confirm = super(sale_order, self).action_ship_create(cr, uid, ids, args)
        ####################################################
        #Se sobreescribe para no saltar la secuencia en los alabranes de salida
        wf_service = netsvc.LocalService("workflow")
        picking_id = False
        move_obj = self.pool.get('stock.move')
        proc_obj = self.pool.get('procurement.order')
        company = self.pool.get('res.users').browse(cr, uid, uid).company_id
        for order in self.browse(cr, uid, ids, context={}):
            proc_ids = []
            output_id = order.shop_id.warehouse_id.lot_output_id.id
            #######
            es_encadenado = order.shop_id.warehouse_id.lot_output_id.chained_location_type != 'none' \
                            and order.shop_id.warehouse_id.lot_output_id.chained_auto_packing != 'transparent' or False
            #######
            picking_id = False
            for line in order.order_line:
                proc_id = False
                date_planned = datetime.now() + relativedelta(days=line.delay or 0.0)
                date_planned = (date_planned - timedelta(days=company.security_lead)).strftime('%Y-%m-%d %H:%M:%S')

                if line.state == 'done':
                    continue
                move_id = False
                if line.product_id and line.product_id.product_tmpl_id.type in ('product', 'consu'):
                    location_id = order.shop_id.warehouse_id.lot_stock_id.id
                    if not picking_id:
                        if es_encadenado:
                            pick_name = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.intermedio')
                        else:
                            pick_name = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.out')

                        picking_id = self.pool.get('stock.picking').create(cr, uid, {
                            'name': pick_name,
                            'origin': order.name,
                            'type': 'out',
                            'state': 'auto',
                            'move_type': order.picking_policy,
                            'sale_id': order.id,
                            'address_id': order.partner_shipping_id.id,
                            'note': order.note,
                            'invoice_state': (order.order_policy=='picking' and '2binvoiced') or 'none',
                            'company_id': order.company_id.id,
                        })
                    move_id = self.pool.get('stock.move').create(cr, uid, {
                        'name': line.name[:64],
                        'picking_id': picking_id,
                        'product_id': line.product_id.id,
                        'date': date_planned,
                        'date_expected': date_planned,
                        'product_qty': line.product_uom_qty,
                        'product_uom': line.product_uom.id,
                        'product_uos_qty': line.product_uos_qty,
                        'product_uos': (line.product_uos and line.product_uos.id)\
                                or line.product_uom.id,
                        'product_packaging': line.product_packaging.id,
                        'address_id': line.address_allotment_id.id or order.partner_shipping_id.id,
                        'location_id': location_id,
                        'location_dest_id': output_id,
                        'sale_line_id': line.id,
                        'tracking_id': False,
                        'state': 'draft',
                        #'state': 'waiting',
                        'note': line.notes,
                        'company_id': order.company_id.id,
                    })

                if line.product_id:
                    proc_id = self.pool.get('procurement.order').create(cr, uid, {
                        'name': line.name,
                        'origin': order.name,
                        'date_planned': date_planned,
                        'product_id': line.product_id.id,
                        'product_qty': line.product_uom_qty,
                        'product_uom': line.product_uom.id,
                        'product_uos_qty': (line.product_uos and line.product_uos_qty)\
                                or line.product_uom_qty,
                        'product_uos': (line.product_uos and line.product_uos.id)\
                                or line.product_uom.id,
                        'location_id': order.shop_id.warehouse_id.lot_stock_id.id,
                        'procure_method': line.type,
                        'move_id': move_id,
                        'property_ids': [(6, 0, [x.id for x in line.property_ids])],
                        'company_id': order.company_id.id,
                    })
                    proc_ids.append(proc_id)
                    self.pool.get('sale.order.line').write(cr, uid, [line.id], {'procurement_id': proc_id})
                    if order.state == 'shipping_except':
                        for pick in order.picking_ids:
                            for move in pick.move_lines:
                                if move.state == 'cancel':
                                    mov_ids = move_obj.search(cr, uid, [('state', '=', 'cancel'),('sale_line_id', '=', line.id),('picking_id', '=', pick.id)])
                                    if mov_ids:
                                        for mov in move_obj.browse(cr, uid, mov_ids):
                                            move_obj.write(cr, uid, [move_id], {'product_qty': mov.product_qty, 'product_uos_qty': mov.product_uos_qty})
                                            proc_obj.write(cr, uid, [proc_id], {'product_qty': mov.product_qty, 'product_uos_qty': mov.product_uos_qty})

            val = {}

            if picking_id:
                wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)

            for proc_id in proc_ids:
                wf_service.trg_validate(uid, 'procurement.order', proc_id, 'button_confirm', cr)

            if order.state == 'shipping_except':
                val['state'] = 'progress'
                val['shipped'] = False

                if (order.order_policy == 'manual'):
                    for line in order.order_line:
                        if (not line.invoiced) and (line.state not in ('cancel', 'draft')):
                            val['state'] = 'manual'
                            break
            self.write(cr, uid, [order.id], val)
            
        ################################################################
        forced_uid = 1
        for so_id in ids:
            wf_service = netsvc.LocalService("workflow")
            picking_obj = self.pool.get('stock.picking')
            
            politica_factura = self.browse(cr, uid, so_id).order_policy
            pick_auto = self.browse(cr, uid, so_id).shop_id.picking_auto
            loc_output = self.browse(cr, uid, so_id).shop_id.warehouse_id.lot_output_id.id
            name_so = self.browse(cr, uid, so_id).name
            
            loc_output_cli = self.browse(cr, uid, so_id).shop_id.warehouse_id.lot_output_id.chained_location_type
            #aqui se procesa si la configuracion es para procesar automatico el primer albaran que esta encadenado a clientes
            proceso_auto = pick_auto and loc_output_cli == 'customer' or False

            if proceso_auto:
                picking_list = picking_obj.search(cr, forced_uid, [('sale_id', '=', so_id)])
                br_picking_list = picking_obj.browse(cr, forced_uid, picking_list)
                
                #determinar el albaran que se debe procesar automatico
                #se toma el primer stock_move del picking porque se supone que todos se generan con las misma ubicacion de salida configurada en el almacen
                procesar_1pick = None
                cambiar_2pick = None
                for pick in br_picking_list:
                    #se debe procesar el primer albaran generado
                    if pick.move_lines[0].location_dest_id.id == loc_output:
                        procesar_1pick = pick.id
                    else:
                        cambiar_2pick = pick.id
                
                """
                Si existe un segundo albaran, se asocia los stock move al procurement.order
                en lugar de los stock move del primer albaran
                precondicion: se debe verificar que tienen el mismo numero de stock move asociados
                para hacer la sustitucion del primer albaran por el segundo
                Esto es para mantener el control de lo enviado y entregado en el pedido de venta
                Esto impide que en un mismo pedido se coloque productos y servicios
                de este modo se garantiza que se actualice el campo entregado al despachar el último albaran.
                """
                if cambiar_2pick and name_so:
                    proc_obj = self.pool.get('procurement.order')
                    move_obj = self.pool.get('stock.move')
                    
                    move_list_1pick = proc_obj.search(cr, uid, [('origin', '=', name_so),('state','!=','cancel')])
                    move_list_2pick = move_obj.search(cr, uid, [('picking_id', '=', cambiar_2pick)])
                    
                    if len(move_list_1pick) == len(move_list_2pick):
                        for it in move_obj.browse(cr,uid,move_list_2pick):
                            if len(it.move_history_ids2) == 1:
                                move_ids = [x.id for x in it.move_history_ids2]
                                move_list_update = proc_obj.search(cr, uid, [('origin', '=', name_so),('move_id','in',move_ids)])
                                if move_list_update:
                                    proc_obj.write(cr, uid, move_list_update, {'move_id': it.id})
                                else:
                                    raise osv.except_osv(_('Invalid action !'), _('Hubo problemas en el proceso de actualizar la planificacion del pedido de venta !. No esta asociado el primer albaran en el abastecimiento.'))
                            else:
                                raise osv.except_osv(_('Invalid action !'), _('Hubo problemas en el proceso de actualizar la planificacion del pedido de venta !. Hay más de un stock move que lo precede.'))
                    else:
                        raise osv.except_osv(_('Invalid action !'), _('Hubo problemas en el proceso de actualizar la planificacion del pedido de venta !. Hubo diferencias en los albaranes creados.'))
                
                ####################################################################################
                
                ########Actualizar costo###########
                #para registrar el costo del producto
                if procesar_1pick:
                    move_obj = self.pool.get('stock.move')
                    product_obj = self.pool.get('product.product')
                    currency_obj = self.pool.get('res.currency')
                    
                    s_move_id = move_obj.search(cr,uid,[('picking_id','=',procesar_1pick)])
                    brow_s_move = move_obj.browse(cr,uid,s_move_id)
                    
                    for move_id in brow_s_move:
                        product = product_obj.browse(cr, forced_uid, move_id.product_id.id)
                        product_price = product.standard_price
                        product_currency = move_id.company_id.currency_id.id
                        move_obj.write(cr, uid, [move_id.id],
                                {'price_unit': product_price,
                                 'price_currency_id': product_currency})
                

                ########################################
                
                
                if br_picking_list:
                    #Si se factura desde albaran
                    if politica_factura == 'picking':
                        if procesar_1pick:
                            picking_obj.write(cr, uid, procesar_1pick, {'invoice_state': 'none'})
                        
                        if cambiar_2pick:
                            picking_obj.write(cr, uid, cambiar_2pick, {'invoice_state': '2binvoiced'})
                    
                    if procesar_1pick:
                        reservado = picking_obj.action_assign(cr, forced_uid, [procesar_1pick])
                        verificar_reserva = picking_obj.test_assigned(cr, forced_uid, [procesar_1pick])
    
                        if reservado and verificar_reserva: #Si hay disponibilidad y pasa a reservado
                            picking_obj.action_move(cr, forced_uid, [procesar_1pick])
                            wf_service.trg_validate(forced_uid, 'stock.picking', procesar_1pick,'button_done', cr)
                            wf_service.trg_write(uid, 'stock.picking', procesar_1pick, cr)
                        else:
                            message = _('Proceso incompleto. El albaran de apartado no se completo por existencias incompletas.')
                            self.log(cr, uid, procesar_1pick, message, context=context)
        return True
    
    def action_cancel_draft(self, cr, uid, ids, *args):
        sale_order_cancel_to_draft = super(sale_order, self).action_cancel_draft(cr, uid, ids, args)
        if sale_order_cancel_to_draft:
            obj_forzar_presup = self.pool.get('sisb.log.forzar.presupuesto')
            pedido = obj_forzar_presup.search(cr,uid,[('order_id','in',ids)])
            if pedido:
                a = obj_forzar_presup.unlink(cr, uid, pedido)
        return sale_order_cancel_to_draft
        
    
    def onchange_partner_id(self, cr, uid, ids, part, shop_id=False):
        so_onchange_partner_id = super(sale_order, self).onchange_partner_id( cr, uid, ids, part)
        address_obj = self.pool.get('res.partner.address')
        if so_onchange_partner_id['value'].get('partner_order_id',False):
            #si existe un contacto, verificar si es el unico contacto, de lo contrario asignar False
            contacto = address_obj.search(cr,uid,[('partner_id','=',part),('type','=','contact')])
            if len(contacto) > 1:
                so_onchange_partner_id['value']['partner_order_id'] = False
                
        if so_onchange_partner_id['value'].get('partner_shipping_id',False):
            #si existe un contacto, verificar si es el unico conctacto, de lo contrario asignar False
            contacto = address_obj.search(cr,uid,[('partner_id','=',part),('type','=','delivery')])
            if len(contacto) > 1:
                so_onchange_partner_id['value']['partner_shipping_id'] = False
        
        #Seleccionar el plazo de pago por defecto de la sucursal si está vacio el del cliente
        payment_term_id = so_onchange_partner_id['value'].get('payment_term','No')
        if payment_term_id != 'No' and not payment_term_id:
            if shop_id:
                #Consultar la sucursal seleccionada
                shop = self.pool.get('sale.shop').browse(cr, uid, shop_id)
                so_onchange_partner_id['value']['payment_term'] = shop.payment_default_id and shop.payment_default_id.id or False

            if not so_onchange_partner_id['value']['payment_term']:
                #Remover la llave
                so_onchange_partner_id['value'].pop('payment_term')
            
        return so_onchange_partner_id
        
    def onchange_shop_id(self, cr, uid, ids, shop_id):
        so_onchange_shop_id = super(sale_order, self).onchange_shop_id( cr, uid, ids, shop_id)
        if shop_id:
            #Se asigna el plazo de pago por defecto de la sucursal
            shop = self.pool.get('sale.shop').browse(cr, uid, shop_id)
            so_onchange_shop_id['value']['payment_term'] = shop.payment_default_id.id or False
            
        return so_onchange_shop_id
        
sale_order()


class sale_order_line(osv.osv):
    _inherit = 'sale.order.line'
    
    #para evitar cambiar el precio del producto
    def price_unit_change(self, cr, uid, ids, pricelist, product, price_unit=False, qty=0, 
            uom=False, qty_uos=0, uos=False, name='', partner_id=False, 
            lang=False, update_tax=True, date_order=False, packaging=False, fiscal_position=False, flag=False):
                
        result = {}

        if not pricelist:
            warning = {
                'title': 'No hay lista de precio!',
                'message':
                    'Antes de seleccionar el producto, debe indicar una lista de precio.!'
                }
            result = {'warning': warning}
        elif product and qty and uom and date_order and partner_id:
            #se verifica si es un producto
            product_type = self.pool.get('product.product').browse(cr, uid, product).type
            product_type = True if (product_type == 'product') else False
            if product_type:
                res=super(sale_order_line, self).product_id_change(cr, uid, ids, pricelist, product, qty,
                                uom, qty_uos, uos, name, partner_id,
                                lang, update_tax,date_order,fiscal_position=fiscal_position,flag=flag)
                price = res['value']['price_unit']
                discount = res['value']['discount'] or 0.00
                if price:
                    #Se compara el cambio realizado, si es diferente al precio
                    #entonces se restaura el precio y el descuento y se envia un mensaje al usuario
                    if abs(price - price_unit) > 0.00:
                        warning = {
                            'title': 'Precio erroneo!',
                            'message':
                                "Debe mantener el precio definido para el producto, para cambiarlo debe modificar la lista de precio vigente."
                            }

                        result.update({'value': {'price_unit':price,'discount':discount}})
                        result.update({'warning': warning})
                else:
                    warning = {
                        'title': 'Precio erroneo!',
                        'message':
                            "No se encuentra una lista de precio que coincida con el producto indicado."
                        }
                    result = {'warning': warning}
        return result
    
    #Validaciones tanto para crear como para actualizar
    def validaciones_registros(self,values):
        
        if values.get('type',False):
            if values['type'] != 'make_to_stock':
                raise osv.except_osv(_('Error!'),_('Debe indicar el metodo de abastecimiento desde stock !'))

        return True
        
    
    def create(self, cr, uid, values, context=None):
        
        self.validaciones_registros(values)

        res = super(sale_order_line, self).create(cr, uid, values, context)
        
        return res
        
    def write(self, cr, user, ids, vals, context=None):
        
        self.validaciones_registros(vals)
        
        record = super(sale_order_line, self).write(cr, user, ids, vals, context)
        
        return record
    
    def get_product_category_account(self,cr, uid, categ_id, fieldname,context=None):
        if context is None:
            context = {}
        categ_brw=self.pool.get('product.category').browse(cr, uid, categ_id, context=context)
        account_id = getattr(categ_brw,fieldname) and getattr(categ_brw,fieldname).id or False
        if not account_id and categ_brw.parent_id:
            account_id = self.get_product_category_account(cr, uid, categ_brw.parent_id.id, fieldname,context=context)
        return account_id
    
    
    def invoice_line_create(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        def _get_line_qty(line):
            if (line.order_id.invoice_quantity=='order') or not line.procurement_id:
                if line.product_uos:
                    return line.product_uos_qty or 0.0
                return line.product_uom_qty
            else:
                return self.pool.get('procurement.order').quantity_get(cr, uid,
                        line.procurement_id.id, context=context)

        def _get_line_uom(line):
            if (line.order_id.invoice_quantity=='order') or not line.procurement_id:
                if line.product_uos:
                    return line.product_uos.id
                return line.product_uom.id
            else:
                return self.pool.get('procurement.order').uom_get(cr, uid,
                        line.procurement_id.id, context=context)

        create_ids = []
        sales = {}
        for line in self.browse(cr, uid, ids, context=context):
            if not line.invoiced:
                if line.product_id:
                    a = line.product_id.product_tmpl_id.property_account_income.id
                    if not a:
                        a = line.product_id.categ_id.property_account_income_categ.id
                    if not a:
                        a = self.get_product_category_account(cr,uid,line.product_id.categ_id.id,"property_account_income_categ",context=context)
                    if not a:
                        raise osv.except_osv(_('Error !'),
                                _('There is no income account defined ' \
                                        'for this product: "%s" (id:%d)') % \
                                        (line.product_id.name, line.product_id.id,))
                else:
                    #Tomar la cuenta de haber por defecto del diario de la factura en lugar del diario por defecto de oerp
                    #~ prop = self.pool.get('ir.property').get(cr, uid,
                            #~ 'property_account_income_categ', 'product.category',
                            #~ context=context) 
                    prop = line.order_id.shop_id.journal_id.default_credit_account_id
                    a = prop and prop.id or False

                uosqty = _get_line_qty(line)
                uos_id = _get_line_uom(line)
                pu = 0.0
                if uosqty:
                    pu = round(line.price_unit * line.product_uom_qty / uosqty,
                            self.pool.get('decimal.precision').precision_get(cr, uid, 'Sale Price'))
                fpos = line.order_id.fiscal_position or False
                a = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, a)
                if not a:
                    raise osv.except_osv(_('Error !'),
                                _('There is no income category account defined in default Properties for Product Category or Fiscal Position is not defined !'))
                inv_id = self.pool.get('account.invoice.line').create(cr, uid, {
                    'name': line.name,
                    'origin': line.order_id.name,
                    'account_id': a,
                    'price_unit': pu,
                    'quantity': uosqty,
                    'discount': line.discount,
                    'uos_id': uos_id,
                    'product_id': line.product_id.id or False,
                    'invoice_line_tax_id': [(6, 0, [x.id for x in line.tax_id])],
                    'note': line.notes,
                    'account_analytic_id': line.order_id.project_id and line.order_id.project_id.id or False,
                })
                cr.execute('insert into sale_order_line_invoice_rel (order_line_id,invoice_id) values (%s,%s)', (line.id, inv_id))
                self.write(cr, uid, [line.id], {'invoiced': True})
                sales[line.order_id.id] = True
                create_ids.append(inv_id)
        # Trigger workflow events
        wf_service = netsvc.LocalService("workflow")
        for sid in sales.keys():
            wf_service.trg_write(uid, 'sale.order', sid, cr)
            
        return create_ids
        
    def product_id_change(self, cr, uid, ids, pricelist, product, qty=0,
            uom=False, qty_uos=0, uos=False, name='', partner_id=False,
            lang=False, update_tax=True, date_order=False, packaging=False, fiscal_position=False, flag=False):
   
        def get_real_price(res_dict, product_id, qty, uom, pricelist):
            item_obj = self.pool.get('product.pricelist.item')
            price_type_obj = self.pool.get('product.price.type')
            product_obj = self.pool.get('product.product')
            template_obj = self.pool.get('product.template')
            field_name = 'list_price'
            #Se consulta con el id correspondiente de product.pricelist.item
            #para ello se sobreescreibe el metodo price_get_multi() de product_pricelist
            #es para corregir un bug que hay en el modulo product_visible_discount
            if res_dict.get('item_id',False) and res_dict['item_id'].get(pricelist,False) and res_dict.get('base',False):
                item = res_dict['base']
                item_base = item_obj.read(cr, uid, [item], ['base'])
                if item_base and item_base[0]['base'] > 0:
                    field_name = price_type_obj.browse(cr, uid, item_base[0]['base']).field
            
            product = product_obj.browse(cr, uid, product_id, context)
            product_tmpl_id = product.product_tmpl_id.id

            product_read = template_obj.read(cr, uid, product_tmpl_id, [field_name], context)
            
            factor = 1.0
            if uom and uom != product.uom_id.id:
                product_uom_obj = self.pool.get('product.uom')
                uom_data = product_uom_obj.browse(cr, uid,  product.uom_id.id)
                factor = uom_data.factor
            return product_read[field_name] * factor

        res=super(sale_order_line, self).product_id_change(cr, uid, ids, pricelist, product, qty,
            uom, qty_uos, uos, name, partner_id,
            lang, update_tax,date_order,fiscal_position=fiscal_position,flag=flag)

        context = {'lang': lang, 'partner_id': partner_id}
        result = res['value']
        
        pricelist_obj = self.pool.get('product.pricelist')
        product_obj = self.pool.get('product.product')
        product_uom_obj = self.pool.get('product.uom')
        if product:
            if result.get('price_unit',False):
                price = result['price_unit']
            else:
                return res

            product = product_obj.browse(cr, uid, product, context)
            list_price = pricelist_obj.price_get(cr, uid, [pricelist],
                    product.id, qty or 1.0, partner_id, {'uom': uom,'date': date_order })
            pricelists = pricelist_obj.read(cr,uid,[pricelist],['visible_discount'])
            old_uom = product.uos_id or product.uom_id
            new_list_price = get_real_price(list_price, product.id, qty, uom, pricelist)
            
            #determinar si es visible el descuento y asignar el precio real del producto
            result['precio_real'] = new_list_price
            result['visible_desc'] = pricelists[0]['visible_discount']
            
            son_iguales = abs(new_list_price - result['price_unit']) < 0.001
            if not son_iguales:
                if(len(pricelists)>0 and pricelists[0]['visible_discount'] and list_price[pricelist] != 0):
                    price = list_price[pricelist]
                    discount = (new_list_price - price) / new_list_price * 100
                    result['price_unit'] = new_list_price
                    result['discount'] = discount
                else:
                    result['discount'] = 0.0
            else:
                if(len(pricelists)>0 and pricelists[0]['visible_discount'] and list_price[pricelist] != 0):
                    price = list_price[pricelist]
                    new_list_price = result['price_unit']
                    discount = (new_list_price - price) / new_list_price * 100
                    result['price_unit'] = new_list_price
                    result['discount'] = discount
                else:
                    result['discount'] = 0.0
            
        #Por haber hecho una asignacion (result = res['value']) al modificar result se modifica res
        return res
        
    #validar no colocar un descuento mayor al permitido
    def discount_change(self, cr, uid, ids, pricelist, product, price_unit=False, discount=False, qty=0, 
            uom=False, qty_uos=0, uos=False, name='', partner_id=False, 
            lang=False, update_tax=True, date_order=False, packaging=False, fiscal_position=False, flag=False):
        
        result = {}
        if not pricelist:
            warning = {
                'title': 'No hay lista de precio!',
                'message':
                    'Antes de seleccionar el producto, debe indicar una lista de precio.!'
                }
            result = {'warning': warning}
        elif product and qty and uom and date_order and partner_id:                
            #se verifica si es un producto
            product_type = self.pool.get('product.product').browse(cr, uid, product).type
            product_type = True if (product_type == 'product') else False
            if product_type:
                pricelist_obj = self.pool.get('product.pricelist')
                discount_max = 0.00
                res = self.product_id_change(cr, uid, ids, pricelist, product, qty,
                        uom, qty_uos, uos, name, partner_id,
                        lang, update_tax,date_order,fiscal_position=fiscal_position,flag=flag)
                #Confirmar que se tiene toda la información para validar el descuento
                if not res['value'].has_key('price_unit') or not res['value'].has_key('discount') \
                        or not res['value'].has_key('visible_desc') or not res['value'].has_key('precio_real'):
                    #no debería entar aca
                    warning = {
                        'title': 'Error en la aplicacion!',
                        'message':
                            'Contactar con el administrador. La informacion para validar el descuento esta incompleta.!'
                        }
                    result = {'warning': warning}
                    return result

                #Determinar si tiene el descuento visible la tarifa del pedido para determinar el descuento
                if res['value']['visible_desc']:
                    price = res['value']['price_unit']
                    discount_tarifa = res['value']['discount']
                else:
                    price = res['value']['price_unit']
                    discount_tarifa = 0.00
                    #No se permite hacer un descuento sobre descuento, para permitir modificar el descuento debe estar visible
                    warning = {
                        'title': 'Descuento no permitido!',
                        'message':
                            "El descuento ya se encuentra aplicado en el precio, la tarifa no tiene el descuento visible."
                        }

                    result.update({'value': {'price_unit':res['value']['price_unit'],'discount':res['value']['discount']}})
                    result.update({'warning': warning})
                    return result
                
                #Descuento maximo para empezar es el registrado en la tarifa registrada en el pedido
                discount_max = discount_tarifa
                if price:
                    #Se compara el cambio realizado, si es mayor al descuento máximo que se pueda aplicar
                    #entonces se restaura el precio y el descuento y se envia un mensaje al usuario
                    #Buscar el descuento maximo a aplicar
                    price_list = pricelist_obj.search(cr, uid, [('type','=','sale')])
                    #Saco de la lista la tarifa registrada en el pedido
                    if pricelist in price_list:
                        price_list.remove(pricelist)
                    for pricelist_id in price_list:
                        #verificar si hay una unica lista de precio vigente
                        # product.pricelist.version:
                        product_pricelist_version_obj = self.pool.get('product.pricelist.version')
                        plversions_search_args = [
                            ('pricelist_id', '=', pricelist_id),
                            '|',
                            ('date_start', '=', False),
                            ('date_start', '<=', date_order),
                            '|',
                            ('date_end', '=', False),
                            ('date_end', '>=', date_order),
                        ]
                        plversion_ids = product_pricelist_version_obj.search(cr, uid, plversions_search_args)
                        if 1 != len(plversion_ids):
                            continue

                        resultado = self.product_id_change(cr, uid, ids, pricelist_id, product, qty,
                                uom, qty_uos, uos, name, partner_id,
                                lang, update_tax,date_order,fiscal_position=fiscal_position,flag=flag)
                        #Confirmar que se tiene toda la información para validar el descuento
                        if not res['value'].has_key('price_unit') or not res['value'].has_key('discount') \
                                or not res['value'].has_key('visible_desc') or not res['value'].has_key('precio_real'):
                            continue

                        #Determinar si tiene el descuento visible la tarifa del pedido para determinar el descuento
                        if resultado['value']['visible_desc']:
                            discount_tarifa = resultado['value']['discount']
                        else:
                            #Se toma para calcular el descuento el precio real segun la tarifa
                            price = resultado['value']['price_unit']
                            price_product = resultado['value']['precio_real']
                            discount_tarifa = (price_product - price) / price_product * 100

                        if discount_tarifa > discount_max:
                            discount_max = discount_tarifa
                            
                    if (discount - discount_max) > 0.001:
                        warning = {
                            'title': 'Descuento no permitido!',
                            'message':
                                "Debe mantener el descuento definido para el producto, o menor o igual a %s porciento."%(discount_max)
                            }

                        result.update({'value': {'price_unit':res['value']['price_unit'],'discount':res['value']['discount']}})
                        result.update({'warning': warning})
                else:
                    warning = {
                        'title': 'Precio erroneo!',
                        'message':
                            "No se encuentra una lista de precio que coincida con el producto indicado."
                        }
                    result = {'warning': warning}
        return result
            
sale_order_line()

class sale_shop(osv.osv):
    _inherit = "sale.shop"
    #journal_id no es un campo requerido debido a la utilización de tiendas q no utilice diario de factura particular
    _columns = {
        'journal_id': fields.many2one('account.journal', 'Journal Sale'), 
        'refund_journal_id': fields.many2one('account.journal', 'Journal Refund'), 
        'picking_auto': fields.boolean('Albaran de apartado automatico', help="Utilizado en Pedidos de Venta. Si se activa, el sistema intentara procesar automaticamente el albaran de apartado si la ubicacion de salida del almacen esta encadenado a Clientes."),
    }

sale_shop()

class sisb_log_forzar_presupuesto(osv.osv):
    _name = "sisb.log.forzar.presupuesto"
    _description = "Log Force Confirm Sale Order"
    
    _columns = {
        'order_id': fields.many2one('sale.order', 'Pedido de Venta', ondelete='cascade', select=True),
        'motivo': fields.text('Motivo'),
        'responsable_uid': fields.many2one('res.users', 'Responsable de forzar el confirmar el presupuesto'),
    }
    
    _sql_constraints = [
        ('order_id_uniq', 'unique (order_id)', 'The order_id must be unique !')
    ]
    
sisb_log_forzar_presupuesto()

class stock_picking(osv.osv):
    _inherit = "stock.picking"
    
    _order = "id desc"
    
stock_picking()
#~ 
class stock_move(osv.osv):
    _inherit = "stock.move"
    
    def _create_product_valuation_moves(self, cr, uid, move, context=None):
        super(stock_move, self)._create_product_valuation_moves(cr, uid, move, context)
        #Para registrar la cuenta analitica de la sucursal que genera el movimiento
        #esto se registra si el albaran de entrada o salida es generado desde un
        #pedido de venta
        if move.product_id.valuation == 'real_time':
            verificar = move.picking_id and move.picking_id.sale_id or False
            if verificar:
                cta_analitica = move.picking_id.sale_id.shop_id.project_id and move.picking_id.sale_id.shop_id.project_id.id or False
                if cta_analitica:
                    #consultar el account.move por la referencia y actualizar si se encuentra
                    id_account_move = self.pool.get('account.move').search(cr, uid, [('ref', '=', move.picking_id.name)])
                    if id_account_move:
                        id_account_move_line_analytic = self.pool.get('account.move.line').search(cr, uid, [('move_id', 'in', id_account_move),('analytic_account_id','=',False)])
                        if id_account_move_line_analytic:
                            #Actualizar la cuenta analitica 
                            valor = {
                                'analytic_account_id': cta_analitica
                            }
                            self.pool.get('account.move.line').write(cr,uid,id_account_move_line_analytic,valor)
    
    
    def _get_accounting_data_for_valuation(self, cr, uid, move, context=None):
        journal_id, acc_src, acc_dest, acc_variation = super(stock_move, self)._get_accounting_data_for_valuation(cr, uid, move, context)
        #################################
        #para asientos de salida en ventas
        if move.location_dest_id.usage == 'customer':
            if move.location_id.account_location_variant_id:
                acc_dest = move.location_id.account_location_variant_id.id

        #para asientos de devolucion en ventas
        if move.location_id.usage == 'customer':
            if move.location_dest_id.account_location_variant_id:
                acc_src = move.location_dest_id.account_location_variant_id.id
           
        ################

        if acc_dest == acc_variation:
            raise osv.except_osv(_('Error!'),  _('Can not create Journal Entry, Output Account defined on this product and Variant account on category of this product are same.'))

        if acc_src == acc_variation:
            raise osv.except_osv(_('Error!'),  _('Can not create Journal Entry, Input Account defined on this product and Variant account on category of this product are same.'))

        if not acc_src:
            raise osv.except_osv(_('Error!'),  _('There is no stock input account defined for this product or its category: "%s" (id: %d)') % \
                                    (move.product_id.name, move.product_id.id,))
        if not acc_dest:
            raise osv.except_osv(_('Error!'),  _('There is no stock output account defined for this product or its category: "%s" (id: %d)') % \
                                    (move.product_id.name, move.product_id.id,))
        if not journal_id:
            raise osv.except_osv(_('Error!'), _('There is no journal defined on the product category: "%s" (id: %d)') % \
                                    (move.product_id.categ_id.name, move.product_id.categ_id.id,))
        if not acc_variation:
            raise osv.except_osv(_('Error!'), _('There is no inventory variation account defined on the product category: "%s" (id: %d)') % \
                                    (move.product_id.categ_id.name, move.product_id.categ_id.id,))
                                    
        return journal_id, acc_src, acc_dest, acc_variation
        
stock_move()


class account_invoice(osv.osv):
    _inherit = "account.invoice"
    
    
    def validar_concept_id_account_invoice_line(self, cr, uid, ids):
        """Funcion para validar que la factura no se le apruebe si unas de las
        lineas de la factura no tiene retencion asociada al menos que sea una factura de venta."""
        ai_brw = self.browse(cr, uid, ids)
        if ai_brw[0].type not in ('out_invoice','out_refund'):
            for ail in ai_brw[0].invoice_line:
                if not ail.concept_id :
                    raise osv.except_osv(_('Error con Retenciones'),_("Debe registrar el tipo de retencion en cada linea de la factura."))
        return True
        
    def validar_diario_ventas(self, cr, uid, ids):
        ai_brw = self.browse(cr, uid, ids)
        if ai_brw[0].type in ('out_invoice','out_refund') and uid != 1:
            jour_user = self.lista_journal_permitidos(cr, uid)
            if ai_brw[0].journal_id.id not in jour_user:
                raise osv.except_osv(_('Error con el Diario'),_("El usuario no tiene permiso para aprobar la factura con el diario seleccionado."))
        
        return True
    
    def action_date_assign(self, cr, uid, ids, *args):
        #Se hereda la función action_date_assign para llamar funcion de validacion
        self.validar_diario_ventas(cr, uid, ids)
        self.validar_concept_id_account_invoice_line(cr, uid, ids)
        return super(account_invoice, self).action_date_assign(cr, uid, ids, args)
    
    def lista_journal_permitidos(self, cr, uid):
            
        cr.execute("""select distinct aj.id from stock_location slp
            left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
            inner join diana_groups_location_rel dlr on dlr.location_id =slp.id
            inner join stock_warehouse sw on sw.lot_stock_id = slh.id
            inner join sale_shop ss on ss.warehouse_id = sw.id
            inner join account_journal aj on aj.sale_shop_id = ss.id 
            where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)""", (uid,))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
    
    def _get_journal(self, cr, uid, context=None):

        if context is None:
            context = {}

        type_inv = context.get('type', 'out_invoice')
        
        if type_inv in ('out_invoice','out_refund') and uid != 1:
            
            type_inv = context.get('type', 'out_invoice')
            user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
            company_id = context.get('company_id', user.company_id.id)
            type2journal = {'out_invoice': 'sale', 'in_invoice': 'purchase', 'out_refund': 'sale_refund', 'in_refund': 'purchase_refund'}
            refund_journal = {'out_invoice': False, 'in_invoice': False, 'out_refund': True, 'in_refund': True}
            journal_obj = self.pool.get('account.journal')
            list_res = journal_obj.search(cr, uid, [('type', '=', type2journal.get(type_inv, 'sale')),
                                                ('company_id', '=', company_id),
                                                ('refund_journal', '=', refund_journal.get(type_inv, False))])

            #Filtrar journal de acuerdo al usuario conectado y sus permisos en ubicaciones
            jour_user = self.lista_journal_permitidos(cr, uid)

            res = list(set(jour_user) & set(list_res))

            res = res and res[0] or False

        else:
            res = super(account_invoice, self)._get_journal(cr, uid, context)
        

        return res
        
    _order = "number desc"
    
    _columns = {
        'sale_ids': fields.many2many('sale.order', 'sale_order_invoice_rel', 'invoice_id', 'order_id', 'Sale Orders', help='This is the list of sale orders that generated this invoice'),
        'address_shipping_id': fields.many2one('res.partner.address', 'Invoice Address', readonly=True, states={'draft':[('readonly',False)]}),
    }
    
    _defaults = {
        'journal_id': _get_journal,
    }
    
    def onchange_partner_id(self, cr, uid, ids, type, partner_id,\
            date_invoice=False, payment_term=False, partner_bank_id=False, company_id=False):
        
        result = super(account_invoice, self).onchange_partner_id(cr, uid, ids, type, partner_id, date_invoice, payment_term, partner_bank_id, company_id)
        
        if partner_id:
            partner = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])
            shipping_addr_id = partner['delivery']
            
            result['value']['address_shipping_id'] = shipping_addr_id
            
        else:
            result['value']['address_shipping_id'] = False
        
        return result
    
    def onchange_company_id(self, cr, uid, ids, company_id, part_id, type, invoice_line, currency_id):
        
        res = super(account_invoice, self).onchange_company_id(cr, uid, ids, company_id, part_id, type, invoice_line, currency_id)
        if company_id and type in ('out_invoice','out_refund') and uid != 1:
            dom = {}
            #Filtrar journal de acuerdo al usuario conectado y sus permisos en ubicaciones
            jour_user = self.lista_journal_permitidos(cr, uid)
            if res.get('domain', False) and res['domain'].get('journal_id', False):
                jour_res = res['domain']['journal_id'][0][2]
                journal_ids = list(set(jour_user) & set(jour_res))
                if journal_ids:
                    #reconstruir el diccionario
                    dom = {'journal_id':  [('id', 'in', journal_ids)]}
                    if res.get('value', False) and res['value'].get('journal_id', False):
                        res['value']['journal_id'] = journal_ids[0]
                else:
                    dom = {'journal_id':  [('id', 'in', jour_user)]}
                    res.update({'warning': {'message': 'Verificar la configuracion del usuario y los permisos en los almacenes de la sucursal que corresponde.'}})
                
            else:
                dom = {'journal_id':  [('id', 'in', jour_user)]}
                
            res['domain'] = dom

        return res

    def fields_view_get(self, cr, uid, view_id=None, view_type=False, context=None, toolbar=False, submenu=False):
        
        res = super(account_invoice, self).fields_view_get(cr, uid, view_id, view_type, context, toolbar, submenu)
        
        type_inv = context.get('type', 'out_invoice')
        
        if type_inv in ('out_invoice','out_refund') and uid != 1:
            #Filtrar journal de acuerdo al usuario conectado y sus permisos en ubicaciones
            jour_user = []
            journal_obj = self.pool.get('account.journal')
            
            jour_user = self.lista_journal_permitidos(cr, uid)

            if res.get('fields', False) and res['fields'].get('journal_id', False):
                type = context.get('journal_type', 'sale')
                for field in res['fields']:
                    if field == 'journal_id':
                        journal_select = journal_obj._name_search(cr, uid, '', [('type', '=', type),('id','in',jour_user)], context=context, limit=None, name_get_uid=1)
                        if journal_select:
                            res['fields'][field]['selection'] = journal_select

        return res
    
    #Se sobreescribe split_invoice mientras no se registre el origen a la factura creada en la ovl.
    #Tomada de la version 689 de ovl/6.0
    def split_invoice(self, cr, uid, ids):
        '''
        Split the invoice when the lines exceed the maximum set for the company
        '''
        for inv in self.browse(cr, uid, ids):
            inv_id =False
            if inv.company_id.lines_invoice < 1:
                raise osv.except_osv(_('Error !'), _('Please set an invoice lines value in:\nAdministration->Company->Configuration->Invoice lines'))
            if inv.type in ["out_invoice","out_refund"]:
                if len(inv.invoice_line)> inv.company_id.lines_invoice:
                    lst = []
                    invoice = self.read(cr, uid, inv.id, ['name', 'type', 'number', 
                        'reference', 'comment', 'date_due', 'partner_id', 'address_contact_id', 
                        'address_invoice_id', 'partner_contact', 'partner_insite', 'partner_ref',
                         'payment_term', 'account_id', 'currency_id', 'invoice_line', 'tax_line',
                          'journal_id', 'period_id','user_id','origin','address_shipping_id'])
                    invoice.update({
                        'state': 'draft',
                        'number': False,
                        'invoice_line': [],
                        'tax_line': [],
                    })
                    # take the id part of the tuple returned for many2one fields
                    for field in ('address_contact_id', 'address_invoice_id', 'partner_id',
                            'account_id', 'currency_id', 'payment_term', 'journal_id', 'period_id','user_id','address_shipping_id'):
                        invoice[field] = invoice[field] and invoice[field][0]
                    
                    if hasattr(inv,'sale_ids'):
                        if self.browse(cr,uid,inv.id,context={}).sale_ids:
                            invoice.update({
                                'sale_ids':[(6,0,[i.id for i in self.browse(cr,uid,inv.id,context={}).sale_ids])]
                            })
                        
                    inv_id = self.create(cr, uid, invoice)
                    cont = 0
                    lst = inv.invoice_line
                    while cont < inv.company_id.lines_invoice:
                        lst.pop(0)
                        cont += 1
                    for il in lst:
                        self.pool.get('account.invoice.line').write(cr,uid,il.id,{'invoice_id':inv_id})
                    self.button_compute(cr, uid, [inv.id], set_total=True)
            if inv_id:
                wf_service = netsvc.LocalService("workflow")
                self.button_compute(cr, uid, [inv_id], set_total=True)
#                wf_service.trg_validate(uid, 'account.invoice', inv_id, 'invoice_open', cr)
        return True
        
    def action_move_create(self, cr, uid, ids, *args):
        res = super(account_invoice, self).action_move_create(cr, uid, ids, args)
        if res:
            """
            Se asigna cuenta analitica si no la tiene registrada en la linea de la factura
            siempre y cuando el asiento provenga de una factura con un diario con sucursal asociada
            """
            ai_brw = self.browse(cr, uid, ids)
            for inv_item in ai_brw:
                move = inv_item.move_id and inv_item.move_id.line_id or []
                for lines_move in move:
                    if lines_move.account_id.id == inv_item.account_id.id and not lines_move.analytic_account_id:
                        cuenta_analitica = inv_item.journal_id.sale_shop_id and \
                                                    inv_item.journal_id.sale_shop_id.project_id and \
                                                    inv_item.journal_id.sale_shop_id.project_id.id or False
                        if cuenta_analitica:
                            valor = {
                                'analytic_account_id': cuenta_analitica
                            }
                            self.pool.get('account.move.line').write(cr,uid,[lines_move.id],valor)
                    
        return res
        
account_invoice()


class account_invoice_line(osv.osv):
    _inherit = "account.invoice.line"
    
    _columns = {
        'sale_lines': fields.many2many('sale.order.line', 'sale_order_line_invoice_rel', 'invoice_id', 'order_line_id', 'Sale Orders Lines', readonly=True),
    }
    
    def price_unit_change(self, cr, uid, ids, product, type='out_invoice', price_unit=False, p_invice_id=False, context=None):
        result_precio = {}
        
        if type == 'out_invoice' and product:
            product_type = self.pool.get('product.product').browse(cr, uid, product).type
            product_type = True if (product_type != 'service') else False
            if product_type:
                #Consultar la linea del pedido y extraer el precio si es el mismo producto
                linea_so = p_invice_id[0][2]

                if linea_so: #si tiene asociado un pedido de venta
                    sol = self.pool.get('sale.order.line').browse(cr, uid, linea_so[0])

                    if sol and sol.product_id.id == product and abs(sol.price_unit - price_unit) > 0.00:
                        warning = {
                            'title': 'Precio erroneo!',
                            'message': "Debe mantener el precio y descuento definido en el pedido de venta para el producto, no se puede cambiar."
                            }
                        result_precio.update({'value': {'price_unit': sol.price_unit, 'discount': sol.discount,}})
                        result_precio.update({'warning': warning})
                    
        return result_precio
        
    def discount_change(self, cr, uid, ids, product, type='out_invoice', price_unit=False, discount=False, p_invice_id=False, context=None):
        result_precio = {}
        
        if type == 'out_invoice' and product:
            product_type = self.pool.get('product.product').browse(cr, uid, product).type
            product_type = True if (product_type != 'service') else False
            if product_type:
                #Consultar la linea del pedido y extraer el precio si es el mismo producto
                linea_so = p_invice_id[0][2]

                if linea_so: #si tiene asociado un pedido de venta
                    sol = self.pool.get('sale.order.line').browse(cr, uid, linea_so[0])

                    if sol and sol.product_id.id == product and abs(sol.discount - discount) > 0.00:
                        warning = {
                            'title': 'Descuento erroneo!',
                            'message': "Debe mantener el precio y descuento definido en el pedido de venta para el producto, no se puede cambiar."
                            }
                        result_precio.update({'value': {'price_unit': sol.price_unit, 'discount': sol.discount,}})
                        result_precio.update({'warning': warning})
                    
        return result_precio
        
    def get_product_category_account(self,cr, uid, categ_id, fieldname,context=None):
        if context is None:
            context = {}
        categ_brw=self.pool.get('product.category').browse(cr, uid, categ_id, context=context)
        account_id = getattr(categ_brw,fieldname) and getattr(categ_brw,fieldname).id or False
        if not account_id and categ_brw.parent_id:
            account_id = self.get_product_category_account(cr, uid, categ_brw.parent_id.id, fieldname,context=context)
        return account_id
    
    def move_line_get(self, cr, uid, invoice_id, context=None):
        res = []
        
        inv = self.pool.get('account.invoice').browse(cr, uid, invoice_id, context=context)
        company_currency = inv.company_id.currency_id.id
        def sisb_get_price(cr, uid, inv, company_currency,i_line):
            cur_obj = self.pool.get('res.currency')
            if inv.currency_id.id != company_currency:
                price = cur_obj.compute(cr, uid, company_currency, inv.currency_id.id, i_line.product_id.product_tmpl_id.standard_price * i_line.quantity, context={'date': inv.date_invoice})
            else:
                price = i_line.product_id.product_tmpl_id.standard_price * i_line.quantity
            return price
                
        if inv.type in ('out_invoice','out_refund'):
            ##################
            #move_line_get original
            res = []
            tax_obj = self.pool.get('account.tax')
            cur_obj = self.pool.get('res.currency')
            if context is None:
                context = {}
            inv = self.pool.get('account.invoice').browse(cr, uid, invoice_id, context=context)
            company_currency = inv.company_id.currency_id.id

            for line in inv.invoice_line:
                mres = self.move_line_get_item(cr, uid, line, context)
                if not mres:
                    continue
                res.append(mres)
                tax_code_found= False
                for tax in tax_obj.compute_all(cr, uid, line.invoice_line_tax_id,
                        (line.price_unit * (1.0 - (line['discount'] or 0.0) / 100.0)),
                        line.quantity, inv.address_invoice_id.id, line.product_id,
                        inv.partner_id)['taxes']:

                    if inv.type in ('out_invoice', 'in_invoice'):
                        tax_code_id = tax['base_code_id']
                        tax_amount = line.price_subtotal * tax['base_sign']
                    else:
                        tax_code_id = tax['ref_base_code_id']
                        tax_amount = line.price_subtotal * tax['ref_base_sign']

                    if tax_code_found:
                        if not tax_code_id:
                            continue
                        res.append(self.move_line_get_item(cr, uid, line, context))
                        res[-1]['price'] = 0.0
                        res[-1]['account_analytic_id'] = False
                    elif not tax_code_id:
                        continue
                    tax_code_found = True

                    res[-1]['tax_code_id'] = tax_code_id
                    res[-1]['tax_amount'] = cur_obj.compute(cr, uid, inv.currency_id.id, company_currency, tax_amount, context={'date': inv.date_invoice})
                
            ####################
            #Asiento del costo
            
            for i_line in inv.invoice_line:
                if i_line.product_id and i_line.product_id.type != 'service':
                    if inv.type == 'out_invoice':
                        # debit account dacc will be the output account
                        # first check the product, if empty check the category                
                                
                        #ojo este comportamiento esta sobreescrito para facturas del modulo de ventas
                        
                        # Se selecciona la cuenta variante de la ubicacion de salida
                        # del almacen de la sucursal
                        
                        dacc = inv.journal_id.sale_shop_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id.id or False
                        
                        if not dacc:
                            raise osv.except_osv(_('Error !'),
                                    _('No hay cuenta de variacion en la ubicacion de salida de la sucursal asociada al journal' \
                                            ' para este producto: "%s" (id:%d)') % \
                                            (i_line.product_id.name, i_line.product_id.id,))
                        #####

                    else:
                        # = out_refund
                        # debit account dacc will be the input account
                        # first check the product, if empty check the category             
                        
                        #ojo este comportamiento esta sobreescrito para facturas del modulo de ventas           
                       
                        # Se selecciona la cuenta variante de la ubicacion de salida
                        # del almacen de la sucursal
                        dacc = inv.journal_id.sale_shop_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id and \
                            inv.journal_id.sale_shop_id.warehouse_id.lot_output_id.account_location_variant_id.id or False
                        
                        
                        if not dacc:
                            raise osv.except_osv(_('Error !'),
                                    _('No hay cuenta de variacion en la ubicacion de salida de la sucursal asociada al journal ' \
                                            'para este producto: "%s" (id:%d)') % \
                                            (i_line.product_id.name, i_line.product_id.id,))
                                            
                        #####
                                            
                    # in both cases the credit account cacc will be the expense account
                    # first check the product, if empty check the category                                    
                    cacc = i_line.product_id.product_tmpl_id.property_account_expense and i_line.product_id.product_tmpl_id.property_account_expense.id
                    if not cacc:
                        cacc = i_line.product_id.categ_id.property_account_expense_categ and i_line.product_id.categ_id.property_account_expense_categ.id
                    
                    if not cacc:
                            cacc = self.get_product_category_account(cr,uid,i_line.product_id.categ_id.id,"property_account_expense_categ",context=context)
                            
                    if not cacc:
                        raise osv.except_osv(_('Error !'),
                                _('There is no account expense defined ' \
                                        'for this product: "%s" (id:%d)') % \
                                        (i_line.product_id.name, i_line.product_id.id,))
                                        
                    ####
                    
                    
                    if dacc and cacc:
                        res.append({
                            'type': 'src',
                            'name': i_line.name[:64],
                            'price_unit': i_line.product_id.product_tmpl_id.standard_price,
                            'quantity': i_line.quantity,
                            'price': sisb_get_price(cr, uid, inv, company_currency, i_line),
                            'account_id': dacc,
                            'product_id': i_line.product_id.id,
                            'uos_id': i_line.uos_id.id,
                            'account_analytic_id': i_line.account_analytic_id.id,
                            'taxes': i_line.invoice_line_tax_id,
                            })
                        
                        res.append({
                            'type': 'src',
                            'name': i_line.name[:64],
                            'price_unit': i_line.product_id.product_tmpl_id.standard_price,
                            'quantity': i_line.quantity,
                            'price': -1 * sisb_get_price(cr, uid, inv, company_currency, i_line),
                            'account_id': cacc,
                            'product_id': i_line.product_id.id,
                            'uos_id': i_line.uos_id.id,
                            'account_analytic_id': i_line.account_analytic_id.id,
                            'taxes': i_line.invoice_line_tax_id,
                            })

        else:
            
            res = super(account_invoice_line,self).move_line_get(cr, uid, invoice_id, context=context)

        return res
        
account_invoice_line()


class account_journal(osv.osv):
    _inherit = "account.journal"
    
    
    _columns = {
        'sale_shop_id': fields.many2one('sale.shop', 'Sucursal Asociada'), 
    }
     
account_journal()


class product_pricelist(osv.osv):
    _inherit = "product.pricelist"
    
    #Se reescribe para poder saber el id del campo base con el que se aplica la formula
    def price_get_multi(self, cr, uid, pricelist_ids, products_by_qty_by_partner, context=None):
        """multi products 'price_get'.
           @param pricelist_ids:
           @param products_by_qty:
           @param partner:
           @param context: {
             'date': Date of the pricelist (%Y-%m-%d),}
           @return: a dict of dict with product_id as key and a dict 'price by pricelist' as value
        """

        def _create_parent_category_list(id, lst):
            if not id:
                return []
            parent = product_category_tree.get(id)
            if parent:
                lst.append(parent)
                return _create_parent_category_list(parent, lst)
            else:
                return lst
        # _create_parent_category_list

        if context is None:
            context = {}

        date = time.strftime('%Y-%m-%d')
        if 'date' in context:
            date = context['date']

        currency_obj = self.pool.get('res.currency')
        product_obj = self.pool.get('product.product')
        product_template_obj = self.pool.get('product.template')
        product_category_obj = self.pool.get('product.category')
        product_uom_obj = self.pool.get('product.uom')
        supplierinfo_obj = self.pool.get('product.supplierinfo')
        price_type_obj = self.pool.get('product.price.type')
        product_pricelist_version_obj = self.pool.get('product.pricelist.version')

        # product.pricelist.version:
        if pricelist_ids:
            pricelist_version_ids = pricelist_ids
        else:
            # all pricelists:
            pricelist_version_ids = self.pool.get('product.pricelist').search(cr, uid, [], context=context)

        pricelist_version_ids = list(set(pricelist_version_ids))
        plversions_search_args = [
            ('pricelist_id', 'in', pricelist_version_ids),
            '|',
            ('date_start', '=', False),
            ('date_start', '<=', date),
            '|',
            ('date_end', '=', False),
            ('date_end', '>=', date),
        ]

        plversion_ids = product_pricelist_version_obj.search(cr, uid, plversions_search_args)
        if len(pricelist_version_ids) != len(plversion_ids):
            msg = "At least one pricelist has no active version !\nPlease create or activate one."
            raise osv.except_osv(_('Warning !'), _(msg))

        # product.product:
        product_ids = [i[0] for i in products_by_qty_by_partner]
        #products = dict([(item['id'], item) for item in product_obj.read(cr, uid, product_ids, ['categ_id', 'product_tmpl_id', 'uos_id', 'uom_id'])])
        products = product_obj.browse(cr, uid, product_ids, context=context)
        products_dict = dict([(item.id, item) for item in products])

        # product.category:
        product_category_ids = product_category_obj.search(cr, uid, [])
        product_categories = product_category_obj.read(cr, uid, product_category_ids, ['parent_id'])
        product_category_tree = dict([(item['id'], item['parent_id'][0]) for item in product_categories if item['parent_id']])

        results = {}
        for product_id, qty, partner in products_by_qty_by_partner:
            for pricelist_id in pricelist_version_ids:
                price = False
                base = False
                tmpl_id = products_dict[product_id].product_tmpl_id and products_dict[product_id].product_tmpl_id.id or False

                categ_id = products_dict[product_id].categ_id and products_dict[product_id].categ_id.id or False
                categ_ids = _create_parent_category_list(categ_id, [categ_id])
                if categ_ids:
                    categ_where = '(categ_id IN (' + ','.join(map(str, categ_ids)) + '))'
                else:
                    categ_where = '(categ_id IS NULL)'

                cr.execute(
                    'SELECT i.*, pl.currency_id '
                    'FROM product_pricelist_item AS i, '
                        'product_pricelist_version AS v, product_pricelist AS pl '
                    'WHERE (product_tmpl_id IS NULL OR product_tmpl_id = %s) '
                        'AND (product_id IS NULL OR product_id = %s) '
                        'AND (' + categ_where + ' OR (categ_id IS NULL)) '
                        'AND price_version_id = %s '
                        'AND (min_quantity IS NULL OR min_quantity <= %s) '
                        'AND i.price_version_id = v.id AND v.pricelist_id = pl.id '
                    'ORDER BY sequence',
                    (tmpl_id, product_id, plversion_ids[0], qty))
                res1 = cr.dictfetchall()
                uom_price_already_computed = False
                for res in res1:
                    if res:
                        base = res['base'] 
                        if res['base'] == -1:
                            if not res['base_pricelist_id']:
                                price = 0.0
                            else:
                                fff = self.price_get(cr, uid,
                                        [res['base_pricelist_id']], product_id,
                                        qty, context=context)
                                base = fff['base']
                                price_tmp = fff[res['base_pricelist_id']]
                                ptype_src = self.browse(cr, uid, res['base_pricelist_id']).currency_id.id
                                price = currency_obj.compute(cr, uid, ptype_src, res['currency_id'], price_tmp, round=False)
                        elif res['base'] == -2:
                            # this section could be improved by moving the queries outside the loop:
                            where = []
                            if partner:
                                where = [('name', '=', partner) ]
                            sinfo = supplierinfo_obj.search(cr, uid,
                                    [('product_id', '=', tmpl_id)] + where)
                            price = 0.0
                            if sinfo:
                                qty_in_product_uom = qty
                                product_default_uom = product_template_obj.read(cr, uid, [tmpl_id], ['uom_id'])[0]['uom_id'][0]
                                supplier = supplierinfo_obj.browse(cr, uid, sinfo, context=context)[0]
                                seller_uom = supplier.product_uom and supplier.product_uom.id or False
                                supplier_currency = supplier.company_id and supplier.company_id.currency_id.id or False
                                if seller_uom and product_default_uom and product_default_uom != seller_uom:
                                    uom_price_already_computed = True
                                    qty_in_product_uom = product_uom_obj._compute_qty(cr, uid, product_default_uom, qty, to_uom_id=seller_uom)
                                cr.execute('SELECT * ' \
                                        'FROM pricelist_partnerinfo ' \
                                        'WHERE suppinfo_id IN %s' \
                                            'AND min_quantity <= %s ' \
                                        'ORDER BY min_quantity DESC LIMIT 1', (tuple(sinfo),qty_in_product_uom,))
                                res2 = cr.dictfetchone()
                                if res2:
                                    price = currency_obj.compute(cr, uid, supplier_currency,\
                                            res['currency_id'], res2['price'], round=False, context=context)
                        else:
                            price_type = price_type_obj.browse(cr, uid, int(res['base']))
                            price = currency_obj.compute(cr, uid,
                                    price_type.currency_id.id, res['currency_id'],
                                    product_obj.price_get(cr, uid, [product_id],
                                        price_type.field,context=context)[product_id], round=False, context=context)

                            uom_price_already_computed = True

                        if price is not False:
                            price_limit = price

                            price = price * (1.0+(res['price_discount'] or 0.0))
                            price = rounding(price, res['price_round'])
                            price += (res['price_surcharge'] or 0.0)
                            if res['price_min_margin']:
                                price = max(price, price_limit+res['price_min_margin'])
                            if res['price_max_margin']:
                                price = min(price, price_limit+res['price_max_margin'])
                            break

                    else:
                        # False means no valid line found ! But we may not raise an
                        # exception here because it breaks the search
                        price = False

                if price:
                    if 'uom' in context and not uom_price_already_computed:
                        product = products_dict[product_id]
                        uom = product.uos_id or product.uom_id
                        price = self.pool.get('product.uom')._compute_price(cr, uid, uom.id, price, context['uom'])

                if results.get(product_id):
                    results[product_id][pricelist_id] = price
                    results[product_id]['base'] = base
                else:
                    results[product_id] = {pricelist_id: price, 'base': base}

        return results

product_pricelist()




