# -*- encoding: utf-8 -*-
##############################################################################
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###############################################################################

from osv import osv
from osv import fields
import sys
from tools.translate import _
import decimal_precision as dp
import netsvc

class sisb_wizard_purchase_retornar(osv.osv_memory):
    """
    Wizard para colocar solicitudes de compra en estado in_progress y su solicitud de 
    presupuesto approbada en estado cancel. Esto se podra realizar si y solo, si en caso
    de haber facturas asociadas, esten en estado draft o cancel, y en el caso de haber
    albaranes esten en estado draft, cancel, assigned o confirm. En estos casos
    tanto las facturas como los albaranes son llevados a estado cancel
    """
    _name = "sisb.wizard.purchase.retornar"
    
    _description = "wizard para corregir solicitudes de compra"
    _columns = {
       'requisition_id': fields.many2one('purchase.requisition','Solicitud de Compra a Corregir', required=False),
       'concept_id': fields.text('Razon', size=64, help="Razon por la cual se realiza la correccion", required=True),
    }
    
    def requisition_corregir(self, cr, uid, ids, context=None):    
        pr_obj = self.pool.get('purchase.requisition').browse(cr, uid, context.get('active_ids')[0])
        
        po_id = self.pool.get('purchase.order').search(cr, uid, [('requisition_id','=',context.get('active_ids')[0]),'|',('state','=','approved'),('state','=','done')])
        
        if not po_id:
            raise osv.except_osv(_('Invalid action!'), 
                _('No se puede realizar la operacion. La Solicitud de Compra no tiene las condiciones para ser reversada!'))
                    
        po_obj = self.pool.get('purchase.order').browse(cr, uid, po_id)
            
        for po_brw in po_obj:
            inv_bool = [True for inv_brw in po_brw.invoice_ids if inv_brw.state not in  ('draft','cancel')]            
            """
            Si el inv_bool es True => hay facturas distintas de draft o cancel, no puede seguir
            de lo contrario si es False, [], entonces continuar
            """
            if inv_bool:
                raise osv.except_osv(_('Invalid action!'), 
                    _('No se puede reversar la Solicitud de Compra ya que existe una factura aprobada!'))

            for sp_brw in po_brw.picking_ids:
                win_ids = self.pool.get('workflow.instance').search(cr, uid, [('res_type','=','stock.picking'),('res_id','=',sp_brw.id)])
                wit_ids = self.pool.get('workflow.workitem').search(cr, uid, [('inst_id','in',win_ids)])
                wit_obj = self.pool.get('workflow.workitem').browse(cr, uid, wit_ids)
                for item in wit_obj:
                    wia_ids = self.pool.get('workflow.activity').search(cr, uid, [('id','in',[item.act_id.id])])
                    pick_bool = [True for pick_brw in [item.act_id] if pick_brw.name not in ('draft','cancel','confirmed','assigned')]
                    if pick_bool:
                        raise osv.except_osv(_('Invalid action!'), 
                            _('No se puede reversar la Solicitud de Compra ya que existe un albaran aprobado!'))
                
            if po_brw.invoice_ids:
                invoice_ids = [x.id for x in po_brw.invoice_ids]
                cancel_invoice = self.pool.get('account.invoice').action_cancel(cr, uid, invoice_ids)
        
            if po_brw.picking_ids:
                picking_ids = [x.id for x in po_brw.picking_ids]
                cancel_picking = self.pool.get('stock.picking').action_cancel(cr, uid, picking_ids, context=context)
                    
        win_ids = self.pool.get('workflow.instance').search(cr, uid, [('res_type','=','purchase.order'),('res_id','=',po_id[0])])            
        wit_ids = self.pool.get('workflow.workitem').search(cr, uid, [('inst_id','in',win_ids)])
        
        act_workitem = self.pool.get('workflow.workitem').write(cr, uid, wit_ids, {'act_id':29})                 
        if act_workitem:
            act_instance = self.pool.get('workflow.instance').write(cr, uid, win_ids, {'state':'active'})             
            if act_instance:
                pol_id = self.pool.get('purchase.order.line').search(cr, uid, [('order_id','=',po_id[0])])
                act_pol = self.pool.get('purchase.order.line').write(cr, uid, pol_id, {'state':'draft'})               
                if act_pol:
                    po_brw = self.pool.get('purchase.order').browse(cr, uid, po_id[0])
                    act_pr = self.pool.get('purchase.requisition').write(cr, uid, po_brw.requisition_id.id, {'state':'in_progress'})               
                    if act_pr:
                        act_po = self.pool.get('purchase.order').write(cr, uid, po_id[0], {'state':'cancel', 'shipped':False, 'approved':False})               
                        
        return {'type': 'ir.actions.act_window_close'}
        
    def default_get(self, cr, uid, fields, context=None):
        """ To get default values for the object.
        """
        if context is None:
            context = {}
        record_id = context and context.get('active_id', False) or False

        res = super(sisb_wizard_purchase_retornar, self).default_get(cr, uid, fields, context=context)

        if 'requisition_id' in fields:
            res.update({'requisition_id':context.get('active_ids')[0]})
        return res

sisb_wizard_purchase_retornar()
