#-*- coding:utf-8 -*-

from time import strftime
from datetime import date
from datetime import datetime
from datetime import timedelta
import re
from dateutil import relativedelta
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import decimal_precision as dp
import logging
from tools.safe_eval import safe_eval as eval
import calendar
from itertools import groupby
from operator import itemgetter
import logging

class hr_sede(osv.osv):
    '''
    Sedes
    '''
    _name = 'hr.sede'
    _description = 'Sedes'

    _columns = {
        'code':fields.char('Código sede', size=64, required=True),
        'cod_bono_alim':fields.char('Código bono alimentación', size=64, required=True),
        'name':fields.char('Descripcion', size=256, required=True),
        'employee_ids':fields.one2many('hr.employee', 'hr_sede_id', 'Empleados'),
        'salary_rule_ids':fields.one2many('hr.salary.rule', 'hr_sede_id', 'Reglas'),
        'amount_bono_alim': fields.float('Monto bono de alimentación'),
        'fact_amount_daily': fields.integer('Factor para monto diario', help='Valor por el cual se divide el monto del bono de alimentación\
                            para obtener el valor diario'),
        'num_payment_1': fields.integer('Cantidad tickets 1'),
        'amount_payment_1': fields.float('Monto ticket 1'),
        'num_payment_2': fields.integer('Cantidad tickets 2'),
        'amount_payment_2': fields.float('Monto ticket 2'),
        'active':fields.boolean('Activo'),
        'analytic_id': fields.many2one('account.analytic.account', 'Cuenta analítica'),
    }

    _sql_constraints = [('unique_code','unique(code)', _('El codigo de la sede debe ser unico'))]

    def _check_amount(self, cr, uid, ids, context=None):
        for sede in self.browse(cr, uid, ids, context=context):
            if sede.amount_bono_alim <> (sede.num_payment_1*sede.amount_payment_1 + sede.num_payment_2*sede.amount_payment_2):
                return False
        return True

    _constraints = [(_check_amount, "Las cantidades de tickets y los montos no coinciden con el monto total a \
                    cancelar por bono de alimentación", ['amount_bono_alim','num_payment_1','amount_payment_1',
                                    'num_payment_2','amount_payment_2'])]

hr_sede()

class hr_year(osv.osv):

    _name = 'hr.year'
    _description = 'Year'

    _columns = {
        'name': fields.char('Año', size=256, readonly=True),
        'year': fields.integer('Año', required=True, readonly=True),
        'hr_month_ids': fields.one2many('hr.month', 'hr_year_id', 'Meses en el año', readonly=True),
        'diario_mensual': fields.boolean('Ambos peridos generados para el año', readonly=True),
    }

    _defaults = {
        'diario_mensual':False,
    }

hr_year()

class hr_payment_calendar(osv.osv):

    _name = 'hr.payment.calendar'
    _description = 'Paid Calendar'

    _columns = {
        'name': fields.char('Calendario de Pago', size=256, readonly=True),
        'hr_year_id': fields.many2one('hr.year', 'Año', readonly=True),
        'hr_period_ids': fields.one2many('hr.period', 'hr_payment_calendar_id', 'Periodos'),
        'active':fields.boolean('Activo', readonly=True),
        'type': fields.selection([('diario', 'Diario'),('mensual', 'Mensual')], "Tipo periodo", required=True, readonly=True),
    }

    _defaults = {
        'active': True,
    }

    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('ERROR!'), _('No se permite eliminar calendarios'))

hr_payment_calendar()


class hr_month(osv.osv):

    _name = 'hr.month'
    _description = 'Period Month'

    _columns = {
        'name': fields.char('Mes', size=256, readonly=True, required=True),
        'month_year': fields.integer('Mes del año', readonly=True),
        'weeks': fields.integer('Semanas', required=True, readonly=True),
        'monday': fields.integer('Lunes en el mes', required=True, readonly=True),
        'monday_first_fortnight': fields.integer('Lunes en la primera quincena', required=True, readonly=True),
        'monday_second_fortnight': fields.integer('Lunes en la segunda quincena', required=True, readonly=True),
        'saturday': fields.integer('Sabados', required=True, readonly=True),
        'sunday': fields.integer('Domingos', required=True, readonly=True),
        'saturday_first_fortnight': fields.integer('Sabados en la primera quincena', readonly=True),
        'saturday_second_fortnight': fields.integer('Sabados en la segunda quincena', readonly=True),
        'sunday_first_fortnight': fields.integer('Domingos en la primera quincena', readonly=True),
        'sunday_second_fortnight': fields.integer('Domingos en la segunda quincena', readonly=True),
        'hr_year_id': fields.many2one('hr.year', 'Periodo', required=True, readonly=True),
        'hr_period_ids': fields.one2many('hr.period', 'hr_month_id', 'Periodos en el mes', readonly=True),
        'interes_prest': fields.float('Tasa de interes para prestaciones')
    }
    
    _order = 'hr_year_id desc, id' 
    
hr_month()

class hr_payroll(osv.osv):

    _name = 'hr.payroll'
    _description = 'Nomina'
    
    def _get_latest_calendar(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        obj_payment_calendar = self.pool.get('hr.payment.calendar')
        for payroll in self.browse(cr, uid, ids, context=context):
            calendars = payroll.hr_payment_calendar_ids
            if len(calendars) > 1:
                raise osv.except_osv(_('ERROR!'), _('Una nómina no puede tener mas de un calendario es estado activo'))
            if calendars:
                res[payroll.id] = calendars[0].id
            else:
                res[payroll.id] = False
        return res

    _columns = {
        'code':fields.char('Código', size=64, required=True),
        'name':fields.char('Descripcion', size=256, required=True),
        'hr_payroll_structure_ids':fields.one2many('hr.payroll.structure', 'hr_payroll_id', 'Estructura'),
        'employee_ids': fields.one2many('hr.employee', 'hr_payroll_id', 'Empleados'),
        'hr_payment_calendar_ids': fields.many2many('hr.payment.calendar', 'hr_payroll_calendar_rel', 'hr_payroll_id', 'hr_calendar_id', 'Calendarios',),
        'hr_payment_calendar_id':fields.function(_get_latest_calendar, method=True, string='Calendario de pago', 
                                    type='many2one', relation="hr.payment.calendar", store=True),
        'parameters_ids': fields.one2many('hr.parameters', 'payroll_id', 'Parametros'),
        'account_id': fields.many2one('account.account', 'Cuenta contable'),
        'journal_id': fields.many2one('account.journal', 'Diario'),
        'out_in_holidays': fields.boolean('Empleados salen de nómina en vacaciones?', help='Este campo indica si los empleados de esta nómina \
                                        salen de ella durante su período de vacaciones, es decir, no se les cancelará sueldo hasta que se \
                                        reincorporen'),
    }

    _sql_constraints = [('unique_code','unique(code)', _('El codigo de la nomina debe ser unico'))]

hr_payroll()

class hr_payment_calendar(osv.osv):

    _inherit = 'hr.payment.calendar'
    _description = 'Paid Calendar'

    _columns = {
        'hr_payroll_ids': fields.many2many('hr.payroll', 'hr_payroll_calendar_rel', 'hr_calendar_id', 'hr_payroll_id', 'Nóminas',),
    }

hr_payment_calendar()

class hr_period(osv.osv):

    _name = 'hr.period'
    _description = 'Periods'

    _columns = {
        'name': fields.char('Periodos', size=256, readonly=True),
        'period_in_year': fields.integer('Período en el año', required=True, readonly=True),
        'period_in_month': fields.integer('Período en el mes', required=True, readonly=True),
        'date_start': fields.date('Fecha inicio', required=True, readonly=True),
        'date_end': fields.date('Fecha fin', required=True, readonly=True),
        'hr_month_id': fields.many2one('hr.month', 'Mes', readonly=True, ondelete='cascade'),
        'last_period_month': fields.boolean('Ultimo periodo en el mes'),
        'hr_payment_calendar_id': fields.many2one('hr.payment.calendar', 'Calendario de Pago', readonly=True),
        'type': fields.selection([('diario', 'Diario'),('mensual', 'Mensual')], "Tipo periodo", readonly=True),
        'state': fields.selection([('draft','Borrador'), ('active','Activo'), ('close','Cerrado')], 'State'),
        'period_account_id': fields.many2one('account.period','Período contable', required=True),
    }

    _defaults = {
        'state': 'draft',
    }

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}

        if 'state' in vals:
            if vals['state'] in ('cancel','draft'):
                raise osv.except_osv(_('ERROR!'), _('Los períodos solo pueden ser cambiados a estado activo. \
                                De los demás estados se encarga el sistema'))

            actual_period_brw = self.browse(cr, uid, ids)[0]

            if actual_period_brw.period_in_year == 1: #Si el periodo a activar es el primero del calendario se debe pasar a cerrado el ultimo
                                                        #periodo del calendario anterior y a inactivo dicho calendario
                old_payment_calendar_ids = self.pool.get('hr.payment.calendar').search(cr, uid, [('id','<>',actual_period_brw.hr_payment_calendar_id.id),\
                                            ('active','=',True),('type','=',actual_period_brw.hr_payment_calendar_id.type)])
                if old_payment_calendar_ids:
                    if self.pool.get('hr.payment.calendar').write(cr, uid, old_payment_calendar_ids, {'active':False}, context):
                        old_period_ids = self.search(cr, uid, [('state','=','active'),('hr_payment_calendar_id','in',old_payment_calendar_ids)])
                        process_in_old_period_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', old_period_ids),
                                        ('state','in',['draft','load','review'])])
                    if process_in_old_period_ids:
                        raise osv.except_osv(_('ERROR!'), _('No se puede cerrar el período anterior. Existen procesos en él \
                                                            que no han sido aprobados'))

                    if super(hr_period, self).write(cr, uid, old_period_ids, {'state':'close'}, context):
                        return super(hr_period, self).write(cr, uid, ids, vals, context)
                else:
                    return super(hr_period, self).write(cr, uid, ids, vals, context)

            else:
                old_period_ids = self.search(cr, uid, [('state','=','active'),('hr_payment_calendar_id','=',actual_period_brw.hr_payment_calendar_id.id)])

                if old_period_ids:

                    process_in_old_period_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', old_period_ids[0]),
                            ('state','in',['draft','load','review'])])
                    if process_in_old_period_ids:
                        raise osv.except_osv(_('ERROR!'), _('No se puede cerrar el período anterior. Existen procesos en él \
                                    que no han sido aprobados'))

                    old_period_brw = self.browse(cr, uid, old_period_ids[0])

                    if actual_period_brw.period_in_year <> old_period_brw.period_in_year + 1: #Los periodos se deben ir activando de manera secuencial segun su numero en el año
                        raise osv.except_osv(_('Acción Denegada!'), _('El período que desea activar no es el que corresponde'))


                    if super(hr_period, self).write(cr, uid, old_period_ids[0], {'state':'close'}, context):
                        return super(hr_period, self).write(cr, uid, ids, vals, context)
        else:
            return super(hr_period, self).write(cr, uid, ids, vals, context)
        return False

hr_period()

class hr_process(osv.osv):

    _name = 'hr.process'
    _description = 'Procesos de pago a empleados'
    _order = 'id desc'
    _logger = logging.getLogger('hr.process')

    def _select_types(self, cr, uid, context=None):
        if context is None:
            context = {}
        res = [('nomina', 'Nómina'),
            ('vacaciones', 'Vacaciones'),
            ('utilidades', 'Utilidades'),
            ('prestaciones', 'Prestaciones sociales'),
            ('guarderias', 'Pago de guarderías'),
            ('anticipos', 'Pago de anticipos'),
            ('bono_alimentacion', 'Pago bono alimentación'),
            ('pago_especial', 'Pagos especiales'),
        ]

        parameter_obj = self.pool.get('hr.parameters')
        param_type_process_ids = parameter_obj.search(cr, uid, [('type', '=', 'tipo_proceso')])
        for param_type_process in parameter_obj.browse(cr, uid, param_type_process_ids):
            res.append((param_type_process.type_process, param_type_process.type_process))

        return res

    def _get_dates_utilities(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        process_brw = self.browse(cr, uid, ids)
        for process in process_brw:
            if not process.type == 'utilidades':
                return res
            else:
                period = process.hr_period_ids[0] #Se selecciona el primer periodo del proceso, si hay mas de uno es indiferente
                payroll_ids = [p.id for p in process.hr_payroll_ids]
                year = datetime.strptime(period.date_start,"%Y-%m-%d").year
                start = datetime(year, 01, 01)
                sede_ids = [x.id for x in process.hr_sede_ids]
                process_old_ids = self.search(cr, uid, [('type', '=', 'utilidades'), ('date_start', '=', start),
                                                    ('hr_payroll_ids', '=', payroll_ids), ('hr_sede_ids', 'in', sede_ids), 
                                                    ('state', '=', 'done')])
                if not process_old_ids:
                    #Verificar si existe un proceso de utilidades del año anterior cuya fecha fin sea el 31 de diciembre,
                    #es decir, si se cancelaron todas las utilidades correspondientes al año anterior
                    end_last_year = datetime(year-1, 12, 31)
                    process_old_ids = self.search(cr, uid, [('type', '=', 'utilidades'), ('date_end', '=', end_last_year),
                                                    ('hr_payroll_ids', '=', payroll_ids), ('hr_sede_ids', 'in', sede_ids),
                                                    ('state', '=', 'done')])
                    if process_old_ids:
                        #Si existe ese proceso entonces el proceso actual es el de pago inicial de utilidades
                        if period.hr_month_id.month_year == 1:
                            raise osv.except_osv(_('Acción Denegada!'), _('No se puede hacer pago inicial de utilidades en el primer \
                                                            período de año.'))
                        last_day = max(calendar.monthcalendar(year, period.hr_month_id.month_year-1)[-1])
                        end = datetime(year, period.hr_month_id.month_year-1, last_day)
                        res[ids[0]] = {
                            'date_start': start,
                            'date_end': end,
                        }
                    else:
                        #Si no existe, el periodo de pago de utilidades correspondiente es el complemento de los ultimos meses
                        #del año anterior, o en su defecto es la primera vez que se cancela utilidades a traves del sistema
                        #y el periodo a cancelar es el del año actual

                        process_old_ids = self.search(cr, uid, [('type', '=', 'utilidades'), ('hr_payroll_ids', '=', payroll_ids), 
                                                            ('hr_sede_ids', 'in', sede_ids),('state', '=', 'done')])
                        if not process_old_ids: #Pago de utilidades por primera vez en el sistema
                            parameter_obj = self.pool.get('hr.parameters')

                            #Busqueda de la fechas inicio y fin del primer proceso de utilidades-----------------------------
                            param_ant_util_id = parameter_obj.search(cr, uid, [('type', '=', 'proc_uno_util')])
                            if not param_ant_util_id:
                                raise osv.except_osv(_('ERROR!'), _('No hay un parámetro definido para Fechas de inicio y fin del primer \
                                                Proceso de utilidades. Ir a la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo Fechas de inicio y fin del primer Proceso de utilidades'))

                            fecha_inicio = parameter_obj.browse(cr, uid, param_ant_util_id)[0].date_start_util
                            fecha_fin = parameter_obj.browse(cr, uid, param_ant_util_id)[0].date_end_util
                            
                            res[ids[0]] = {
                                'date_start': fecha_inicio,
                                'date_end': fecha_fin,
                            }
                            
                        else: #Pago del complemento del año anterior
                            start = datetime(year-1, 01, 01)
                            res[ids[0]] = {
                                'date_start': start,
                                'date_end': end_last_year,
                            }
                else:
                    last_day = max(calendar.monthcalendar(year, period.hr_month_id.month_year-1)[-1])
                    end = datetime(year, period.hr_month_id.month_year-1, last_day)
                    end_last_process = datetime.strptime(self.browse(cr, uid, process_old_ids[0]).date_end,"%Y-%m-%d")
                    if end <= end_last_process:
                        raise osv.except_osv(_('Acción Denegada!'), _('No se puede hacer pago de utilidades en este período. El último \
                                                                    período en el que se cancelo utilidades corresponde al mismo mes'))
                    res[ids[0]] = {
                        'date_start': start,
                        'date_end': end,
                    }
        return res

    _columns = {
        'name':fields.char('Descripcion', size=256, required=True),
        'date': fields.date('Fecha creación', required=True),
        'date_done': fields.date('Fecha aprobación'),
        'date_review':fields.date('Fecha procesado'),
        'hr_payslip_ids':fields.one2many('hr.payslip', 'hr_process_id', 'Calculos de Nómina'),
        'state': fields.selection([('draft','Borrador'), ('load','Cargado'), ('execute','En ejecucion'),
                                 ('review','Procesado'), ('done','Aprobado'), ('cancel','Cancelado')],
                                 'State', readonly=True),
        'hr_sede_ids': fields.many2many('hr.sede', 'hr_process_sede_rel', 'hr_process_id', 'hr_sede_id', 'Sedes',
                                        readonly=True, states={'draft':[('readonly',False)]}),
        'cargado': fields.boolean('Cargado'),
        'type': fields.selection(_select_types, "Tipo proceso", required=True, size=64),
        'account_move_ids': fields.one2many('account.move', 'hr_process_id', 'Asientos contables'),
        'account_voucher_ids': fields.one2many('account.voucher', 'hr_process_id', 'Ordenes de pago'),
        'date_start': fields.function(_get_dates_utilities, string='Utilidades desde', method=True, type='date', store=True, multi='dates',
                                help='Usado solo para los procesos de pago de utilidades \
                                para establecer la fecha de inicio del periodo que se esta cancelando'),
        'date_end': fields.function(_get_dates_utilities, string='Hasta', method=True, type='date', store=True, multi='dates',
                                help='Usado solo para los procesos de pago de utilidades \
                                para establecer la fecha de fin del periodo que se esta cancelando'),
        'month_id': fields.many2one('hr.month', 'Mes a pagar', help='Corresponde al mes a cancelar bono de alimentación'),
        'hr_period_ids': fields.many2many('hr.period', 'hr_period_process_rel', 'process_id', 'period_id', 'Periodos',
                                            readonly=True, states={'draft':[('readonly',False)]}),
        'hr_payroll_ids': fields.many2many('hr.payroll','hr_process_payroll_rel', 'hr_process_id', 'hr_payroll_id', 'Nóminas',
                                            readonly=True, states={'draft':[('readonly',False)]}),
        'hr_employee_ids': fields.many2many('hr.employee','hr_process_employee_rel', 'hr_process_id', 'hr_employee_id', 'Empleados',
                                            readonly=True, states={'draft':[('readonly',False)]}), #Campo usado para el caso en el q solo se quiera generar pago de uno o algunos empleados de una nomina
        'filtro': fields.char('Filtro para los periodos', size=256),
        'account_invoice_ids': fields.one2many('account.invoice', 'process_id', 'Facturas para pago de guarderias'),
    }

    _defaults = {
        'date': lambda *args: strftime('%Y-%m-%d'),
        'state': 'draft',
        'cargado': False,
    }

    def on_change_type(self, cr, uid, ids, type, filtro, context):
        """ Funcion usada para setear el valor de filtro segun el tipo de proceso que
            se escoja. Si el proceso es de prestaciones sociles el valor de filtro debe ser
            'close' para que en el domain de la vista el campo hr_period_ids se llene solo
            con los periodos en estado 'close', en cualquier otro tipo de proceso el campo
            hr_periop_ids se debe llenar con periodos en estado 'active'
        """
        if context is None:
            context = {}
        if type == 'prestaciones':
            result = {
                'filtro': 'close',
                'hr_period_ids': False,
                }
            return {'value': result}
        else:
            result = {
                'filtro': 'active',
                'hr_period_ids': False,
                }
            return {'value': result}

    def create(self, cr, uid, vals, context=None):
        if not (vals['type'] == 'utilidades'):
             vals['date_start'] = False
             vals['date_end'] = False
        return super(hr_process, self).create(cr, uid, vals, context)

    #Chequear _check_payroll al tratar de guardar un proceso sin una nomina o con mismas caracteristicas de otro ya guardado
    def _check_payroll(self, cr, uid, ids, context=None):
        for process in self.browse(cr, uid, ids, context=context):
            '''Verificar:
                * Que por lo menos de haya escogido una nomina
                * Que no se genere mas de un proceso con la misma nomina y periodo
                * En el caso de haber escogido empleados, verificar que pertenezcan a las nominas seleccionadas
            '''
            if not process.hr_payroll_ids:
                return False

            # No se debe dejar correr ningun proceso si no se ha generado el proceso de prestaciones del mes anterior
            if process.type <> 'prestaciones':
                for period in process.hr_period_ids:
                    if period.period_in_month == 1:
                        last_period = False
                        month = period.hr_month_id
                        if month.month_year == 1: # Cunsultar el ultimo periodo del calendario anterior
                            year_old = self.pool.get('hr.year').search(cr, uid, [('year', '=', month.hr_year_id.year-1)])
                            if year_old: # Si no existe año anterior El sistema esta comenzando y no hay año anterior generado
                                year_old = year_old[0]
                                calendar_old_id = self.pool.get('hr.payment.calendar').search(cr, uid, [('type', '=', period.hr_payment_calendar_id.type),
                                                                ('hr_year_id', '=', year_old),('active', '=', False)])
                                calendar_old_brw = self.pool.get('hr.payment.calendar').browse(cr, uid, calendar_old_id)
                                if not calendar_old_brw:
                                    raise osv.except_osv(_('Error!'),_('No existe un calendario anterior para obtener el ultimo período'))

                                last_period = (calendar_old_brw[0].hr_period_ids[-1]).id

                        else:
                            last_month = self.pool.get('hr.month').search(cr, uid, [('month_year','=', month.month_year - 1),
                                                                    ('hr_year_id','=', month.hr_year_id.id)])
                            last_period = self.pool.get('hr.period').search(cr, uid, [('hr_month_id','=', last_month),
                                                        ('last_period_month','=', True)])
                        if last_period:
                            payrolls = [x.id for x in process.hr_payroll_ids]
                            process_in_last_period_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', last_period),
                                                    ('hr_payroll_ids','=', payrolls),('hr_sede_ids','=',False),('hr_employee_ids','=',False),
                                                    ('type','=','prestaciones'),('state','=','done')])
                            if not process_in_last_period_ids:
                                if process.hr_sede_ids:
                                    for sede in process.hr_sede_ids:
                                        process_in_last_period_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', last_period),
                                                    ('hr_payroll_ids','=', payrolls),('hr_sede_ids','=',sede.id),('hr_employee_ids','=',False),
                                                    ('type','=','prestaciones'),('state','=','done')])
                                        if not process_in_last_period_ids:
                                            if process.hr_employee_ids:
                                                for employee in process.hr_employee_ids:
                                                    #Si no tiene hojas de pago anteriores es porque es nuevo y no se le debe verificar prestaciones
                                                    old_payslip_ids = self.pool.get('hr.payslip').search(cr, uid, [('employee_id','=',employee.id),
                                                        ('date_from','<',process.date)])
                                                    if old_payslip_ids:
                                                        process_in_last_period_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', last_period),
                                                            ('type','=','prestaciones'), ('state','=','done')])
                                                        payslip_prest = self.pool.get('hr.payslip').search(cr, uid, [('employee_id','=',employee.id),
                                                            ('hr_process_id','in',process_in_last_period_ids)])
                                                        if not payslip_prest:
                                                            return False
                                            else:
                                                return False
                    
                                else:
                                    for sede in self.pool.get('hr.sede').search(cr, uid, []):
                                        process_in_last_period_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', last_period),
                                                    ('hr_payroll_ids','=', payrolls),('hr_sede_ids','=',sede),('hr_employee_ids','=',False),
                                                    ('type','=','prestaciones'),('state','=','done')])
                                        if not process_in_last_period_ids:
                                            if process.hr_employee_ids:
                                                for employee in process.hr_employee_ids:
                                                    #Si no tiene hojas de pago anteriores es porque es nuevo y no se le debe verificar prestaciones
                                                    old_payslip_ids = self.pool.get('hr.payslip').search(cr, uid, [('employee_id','=',employee.id),
                                                        ('date_from','<',process.date)])
                                                    if old_payslip_ids:
                                                        process_in_last_period_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', last_period),
                                                            ('type','=','prestaciones'), ('state','=','done')])
                                                        payslip_prest = self.pool.get('hr.payslip').search(cr, uid, [('employee_id','=',employee.id),
                                                            ('hr_process_id','in',process_in_last_period_ids)])
                                                        if not payslip_prest:
                                                            return False
                                            else:
                                                return False
                                                
            if process.hr_employee_ids:
                employees_in_payrolls = []
                for payroll in process.hr_payroll_ids:
                    employees_in_payrolls = employees_in_payrolls + payroll.employee_ids

                payrolls_in_employees = []
                for employee in process.hr_employee_ids:
                    payrolls_in_employees.append(employee.hr_payroll_id)

                employees_in_sedes = []
                sede_set = []
                if process.hr_sede_ids:
                    for sede in process.hr_sede_ids:
                        employees_in_sedes = employees_in_sedes + sede.employee_ids
                    sede_set = list(set(employees_in_sedes) & set(process.hr_employee_ids))

                employee_set_payroll = list(set(employees_in_payrolls) & set(process.hr_employee_ids))
                payroll_set = list(set(payrolls_in_employees) & set(process.hr_payroll_ids))


                # Retorna error si todas las nominas seleccionadas no tienen por lo menos un empleado y si algun
                # empleado seleccionado no tiene su nomina seleccionada
                if not (len(employee_set_payroll) == len(process.hr_employee_ids) and len(payroll_set) == len(process.hr_payroll_ids)
                        and len(sede_set) == len(process.hr_sede_ids)):
                    return False

            if process.type == 'bono_alimentacion':
                return True

            args = [('id','<>', process.id), ('type','=', process.type)]
            
            criterio = []
            for x in range(max(len(process.hr_period_ids),len(process.hr_payroll_ids),len(process.hr_sede_ids))+1):
                if len(process.hr_period_ids) == x:
                    criterio.append(('hr_period_ids', process.hr_period_ids))
                if len(process.hr_payroll_ids) == x:
                    criterio.append(('hr_payroll_ids', process.hr_payroll_ids))
                if len(process.hr_sede_ids) == x and process.hr_sede_ids:
                    criterio.append(('hr_sede_ids', process.hr_sede_ids))
                      
            for c in criterio:
                l = []
                v = [(c[0], '=', [x.id if not isinstance(x, bool) else x for x in c[1]])]
                l = l + v
                args = args + l  
            
            process_ids = self.pool.get('hr.process').search(cr, uid, args)
            process_brw = self.pool.get('hr.process').browse(cr, uid, process_ids)
            
            if process_brw:
                # En caso de haber procesos con misma nomina-periodo-sede verificar si son del mismo tipo y si el nuevo
                # proceso tienen empleados, verificar que no tenga una hoja de pago en los procesos anteriores
                for p in process_brw:
                    if process.hr_employee_ids:
                        employee_set = []
                        employees_old_process = []
                        for payslip in p.hr_payslip_ids:
                            employees_old_process.append(payslip.employee_id)

                        employee_set = list((set(employees_old_process) & set(process.hr_employee_ids)))
                        if ((not process.type == 'anticipos') and process.type == p.type) and employee_set:
                            return False

                    elif ((not process.type == 'anticipos') and process.type == p.type):
                        return False

        return True

    def _check_periods(self, cr, uid, ids, context=None):
        types = {}

        for process in self.browse(cr, uid, ids, context=context):
            if not process.hr_period_ids:
                return False

            if process.type == 'guarderias': #Si el proceso es de guarderia se verifica que el periodo de pago sea el primero del mes
                for period in process.hr_period_ids:
                    if not period.period_in_month == 1:
                        return False
            if process.type == 'prestaciones': #Si el proceso es de prestaciones se verifica que los periodos de pago sea el primero del mes
                for period in process.hr_period_ids:
                    if not period.last_period_month == 1:
                        return False

            if process.hr_payroll_ids:
                calendar_ids = [k.id for k in [y.hr_payment_calendar_id for y in [x for x in process.hr_payroll_ids]]]
                period_ids = self.pool.get('hr.period').search(cr, uid, [('hr_payment_calendar_id','in', calendar_ids)])
                for period in process.hr_period_ids:
                    if period.id not in period_ids: #Si el periodo seleccionado no esta dentro de las nominas seleccionadas
                        return False
                    if not period.type in types:
                        types[period.type] = 1
                    else:
                        types[period.type] += 1

                for calendar_id in calendar_ids: #El calendario dentro de cada nomina debe tener un periodo en el process
                    calendar_brw = self.pool.get('hr.payment.calendar').browse(cr, uid, calendar_id)
                    if calendar_brw.type not in types:
                        return False

        for t in types.values():
            if t > 1: #Si hay mas de un periodo del mismo tipo en el proceso no es valido
                return False
        return True

    def _check_month(self, cr, uid, ids, context=None):
        types = {}

        # Verficar que haya un proceso de pago de bono de alimentacion ya registrado en estado done en el mismo mes escogido para el proceso actual
        for process in self.browse(cr, uid, ids, context=context):
            if process.type == 'bono_alimentacion':
                periods_brw = process.month_id
                if process.hr_sede_ids:
                    for sede in process.hr_sede_ids:
                        process_old = self.search(cr, uid, [('id','<>',process.id),('month_id','=',process.month_id.id),('hr_sede_ids','=',sede.id)])
                        if process_old:
                            return False
                else:
                    process_old = self.search(cr, uid, [('id','<>',process.id),('month_id','=', process.month_id.id)])
                    if process_old:
                        return False
        return True

    _constraints = [
        (_check_payroll, "Chequear: * El proceso debe tener por lo menos una nómima asociada. \
                      * Si se escogen empleados dentro del proceso, estos deben estar dentro de las nóminas seleccionadas. \
                      * No pueden haber dos procesos del mismo tipo con la misma nómina y período.\
                      * Para correr cualquier proceso de inicio de mes se dede haber generado el proceso de Prestaciones \
                      Sociales del mes anterior",
                      ['hr_payroll_ids', 'hr_period_ids']),
        (_check_periods, "Chequear: * Los períodos seleccionados deben estar dentro de los calendarios de pago de las nóminas seleccionadas.\n \
                      * El proceso no puede tener más de un período para la misma nómina (semanal o quincenal.) \
                      * En el caso de Pago de Guardería, éste se hace sólo en el primer período de cada mes \
                      * En el caso de cálculo de Prestaciones Sociales, éste se hace sólo en el último período de cada mes",
                      ['hr_period_ids']),
        (_check_month, "Ya existe un proceso de pago de bono de alimentación registrado para el mes seleccionado", ['month_id'])
        ]
    _sql_constraints = [('unique_name','unique(name)', "No pueden existir dos procesos de nomina con el mismo nombre")]

    def calculo_acumulados_process(self, cr, uid, payslip, context):
        ''' 
        Metodo usado para el recalculo de los devengados de las hojas de 
        pago de los procesos en estado aprobado
        @param payslip: objeto Browseable que contiene la hoja de pago
        (hr.payslip)
        '''
        if not payslip.employee_id.hr_payroll_id.hr_payment_calendar_id:
            raise osv.except_osv(_('ERROR!'),
              _('La nomina %s no tiene calendario de pago. Debe asignarle un \
                calendario de pago antes de ejecutar cualquier proceso') % (payslip.employee_id.hr_payroll_id.name))

        concept_obj = self.pool.get('hr.concept')
        parameter_obj = self.pool.get('hr.parameters')

        concepts = [
                ('sso', ' Retencion S.S.O.', 'Concepto S.S.O.'),
                ('islr', ' Retencion I.S.L.R.', 'Concepto I.S.L.R.'),
                ('rpe', ' Regimen Prestacional de Empleo', 'Concepto R.P.E.'),
                ('faov', ' Retencion F.A.O.V.', 'Concepto F.A.O.V.'),
                ('utilidades', 's que inciden en Utilidades', 'Conceptos que inciden en Utilidades'),
                ('devengado', 's que inciden en I.S.R.L. para Devengado', 'Conceptos que inciden en I.S.R.L. para Devengado'),
            ]

        # Consigue el período y la nómina anterior
        period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))
        if period:
            period = period[0]
                    
        previous_payslip_id = self.pool.get('hr.payslip').search(cr, uid, [('id','<>',payslip.id),
                                ('type_process', 'in', ['nomina', 'utilidades', 'vacaciones', 'prestaciones']),
                                ('employee_id', '=', payslip.employee_id.id), ('state', '=', 'done'),
                                ('date_done','<=', payslip.hr_process_id.date_done)], order='date_done desc')

        if previous_payslip_id:
            previous_payslip_id = previous_payslip_id[0]
        # Revisar búsqueda
        previous_payslip = self.pool.get('hr.payslip').browse(cr, uid, previous_payslip_id)
        values = {}

        for cpt in concepts:
            """
            FIXME: Estandarizar los nombres/tipos de los parametros para evitar
            estas comparaciones y hacer mas simple el metodo
            """
            args = 'concepto_' + cpt[0]
            key = cpt[0] + '_acumulado'
            brw_key = 'concept_' + cpt[0] + '_id'

            if cpt[0] == 'utilidades':
                args = 'conceptos_util'
                brw_key = 'concept_util_ids'

            if cpt[0] == 'devengado':
                args = 'conceptos_deven'
                brw_key = 'concept_deven_ids'

            param_id = parameter_obj.search(cr, uid, [('type', '=', args),
              ('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])

            if not param_id:
                param_id = parameter_obj.search(cr, uid, [('type', '=', args),('payroll_id', '=', None)])
            if not param_id:
                raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para concepto%s para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo %s')
                            %(cpt[1], payslip.employee_id.hr_payroll_id.name, cpt[2]))

            concept_brw = parameter_obj.browse(cr, uid, param_id)[0][brw_key]
            if cpt[0] not in ['utilidades', 'devengado']:
                if period and period.period_in_year == 1 and payslip.type_process == 'nomina':
                    values[key] = 0
                else:
                    try:
                        values[key] = previous_payslip[cpt[0] + '_acumulado']
                    except Exception, e:
                        values[key] = 0
                for line in payslip.line_ids:
                    if line.code == concept_brw.code:
                        values[key] = values[key] + line.total
            else:
                self._logger.debug('Calculando acumulado %s..' % cpt[0])
                if period and period.period_in_year == 1 and payslip.type_process == 'nomina':
                    values[key] = 0
                else:
                    try:
                        values[key] = previous_payslip[cpt[0] + '_acumulado']
                    except Exception, e:
                        values[key] = 0
                for line in payslip.line_ids:
                    if line.code in [concept.code for concept in concept_brw]:
                        for concept in concept_brw:
                            if line.code == concept.code:
                                values[key] = (values[key] + line.total) if concept.type_calc == 'allocation' \
                                  else (values[key] - line.total)
        res = self.pool.get('hr.payslip').write(cr, uid, payslip.id, values)
        return True                    
    
    def recalculo_acumulados(self, cr, uid, ids, context=None):
        ''' 
        Metodo usado para hacer el llamado a la funcion de recalculo 
        de los devengados de las hojas de 
        pago de los procesos en estado aprobado
        @param ids: ids de los objetos hr.process a recalcular
        (hr.process)
        '''
        if not isinstance(ids, list):
            ids = [ids]
        process_brw = self.browse(cr, uid, ids, context)
        for process in process_brw: 
            for payslip in process.hr_payslip_ids:
                self.calculo_acumulados_process(cr, uid, payslip, context)
        return True
    
    def write(self, cr, uid, ids, vals, context=None):
        """Si aun existe la accion planificada con el nombre del proceso
        entonces el proceso aun esta en ejecucion y no se permite su
        modificacion por parte del usuario"""
        if context is None:
            context = {}
        if type(ids) != list:
            ids = [ids]

        process = self.browse(cr, uid, ids)[0]
        cron = self.pool.get('ir.cron')
        cron_proceso_nomina = cron.search(cr, uid, [('name', '=', process.name)])

        if cron_proceso_nomina:
            raise osv.except_osv(_('Accion no permitida!'), _('No se permite modificar procesos que se encuentren en ejecucion'))
        else:
            return super(hr_process, self).write(cr, uid, ids, vals, context)

    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if type(ids) != list:
            ids = [ids]

        process_brw = self.browse(cr, uid, ids)
        for process in process_brw:
            if process.state == 'done':
                raise osv.except_osv(_('Accion no permitida!'), _('No se permite eliminar Procesos ya aprobados'))
            elif process.state == 'execute':
                raise osv.except_osv(_('Accion no permitida!'), _('No se permite eliminar Procesos que se encuentren en ejecucion'))
            for payslip in process.hr_payslip_ids:
                self.pool.get('hr.payslip').unlink(cr, uid, payslip.id, context)
            super(hr_process, self).unlink(cr, uid, process.id, context)
        return True

    def cancel(self, cr, uid, ids, context=None):
        process_brw = self.browse(cr, uid, ids)
        if not isinstance(process_brw, list):
            process_brw = [process_brw]
        variable_concept_ids = self.pool.get('hr.variable.concepts').search(cr, uid, [('hr_process_id', 'in', ids)])
        if not isinstance(variable_concept_ids, list):
            variable_concept_ids = [variable_concept_ids]
        if variable_concept_ids:
            for variable_concept_id in variable_concept_ids:
                self.pool.get('hr.variable.concepts').write(cr, uid, variable_concept_id, {'state': 'cancel'}, context=context)

        self.pool.get('hr.payslip').write(cr, uid, [x.id for x in process_brw[0].hr_payslip_ids], {'state':'cancel'})
        return self.write(cr, uid, ids, {'state': 'cancel'}, context=context)

    def _get_employee(self, cr, uid, ids, **params):
        """
        Los parametros a recibir pueden ser:
        states = Lista de estados permitidos de los trabajadores
        payrolls = Lista de nominas a las que deben pertenecer los trabajadores
        sedes = Lista de sedes a las que deben pertenecer los trabajadores
        """
        process = self.browse(cr, uid, ids)
        employees = self.pool.get('hr.employee')
        sentencia = []
        if 'states' in params and params['states']:
            sentencia.append(('state','in',params['states']))
        if 'employee_ids' in params and params['employee_ids']:
            sentencia.append(('id','in',params['employee_ids']))
        if 'payroll_ids' in params and params['payroll_ids']:
                sentencia += [('hr_payroll_id', 'in', [p.id for p in params['payroll_ids']])]
        if 'sede_ids' in params and params['sede_ids']:
            sentencia += [('hr_sede_id', 'in', [s.id for s in params['sede_ids']])]
        
        employee_ids = employees.search(cr, uid, sentencia)
        employee_brw = employees.browse(cr, uid, employee_ids)
        employee = []
        for e in employee_brw:
            if not e.contract_id:
                raise osv.except_osv(_('ERROR!'), _('El empleado %s no tiene un contrato registrado')
                                                %(e.name))
                                                
            period = list(set(process[0].hr_period_ids) & set(e.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
            date_end = e.contract_id.date_end >= period.date_start if e.contract_id.date_end else True
            if e.start_rel_lab > period.date_end or not date_end:
                continue
            else:
                employee.append(e)
        return employee
        
    def cargar_payslips(self, cr, uid, ids, context=None):
        res = {}
        employee_in_process = False
        hr_payslip = self.pool.get('hr.payslip')

        process = self.browse(cr, uid, ids, context=context)
        
        employee_brw = self._get_employee(cr, uid, ids, states = ['activo','vacaciones','reposo'], 
                            employee_ids = [x.id for x in process[0].hr_employee_ids] or False, 
                            payroll_ids = process[0].hr_payroll_ids or False, 
                            sede_ids = process[0].hr_sede_ids or False)

        if not employee_brw:
            raise osv.except_osv(_('ERROR!'), _('El(los) trabajador(es) no cumple(n) con las especificaciones \
                                                necesarias para el proceso'))

        for employee in employee_brw:
            if employee.no_remunerado and process[0].type in ('nomina','vacaciones','utilidades','guarderias',\
                                        'anticipos','bono_alimentacion','pago_especial'):
                continue
                
            hijos = False
            if not employee.contract_id:
                raise osv.except_osv(_('ERROR!'), _('El empleado %s no tiene un contrato registrado')
                                                %(employee.name))

            period = list(set(process[0].hr_period_ids) & set(employee.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]

            if process[0].type == 'vacaciones':   #Verificar con cada empleado si le toca pago de vacaciones
                period_end_year = datetime.strptime(period.date_end,"%Y-%m-%d").year
                
                contract_year = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").year
                contract_month = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").month
                contract_day = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").day

                nueva_f = datetime(period_end_year, contract_month, contract_day)
                if contract_year == period_end_year or not (nueva_f >= datetime.strptime(period.date_start,"%Y-%m-%d")
                        and nueva_f <= datetime.strptime(period.date_end,"%Y-%m-%d")):
                    continue

            elif process[0].type == 'guarderias':
                if not employee.employee_family_ids: #Verificar si el empleado tiene familiar registrado
                    continue
                for employee_family in employee.employee_family_ids:
                    if employee_family.relationship == 'hijo_a' and employee_family.guarderia_id: #Por cada familiar verificar si es hijo y si tiene una guarderia
                        hijos = True
                        break
                if not hijos:
                    continue
                    
            elif process[0].type == 'nomina':
                if employee.hr_payroll_id.out_in_holidays:
                    hol_employee_id = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id', '=', employee.id),
                                ('date_start', '<=', period.date_start),('date_end', '>=', period.date_end),
                                ('state', '=', 'done')])

                    if hol_employee_id: #Esta corriendo el periodo de disfrute de vacaciones por lo tanto no le corresponde pago de nomina
                        continue
                        
            elif process[0].type == 'anticipos':
                parameter_obj = self.pool.get('hr.parameters')

                #Busqueda del concepto que se definio como Concepto de anticipo de prestaciones-----------------------------
                param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', employee.hr_payroll_id.id)])
                if not param_ant_pres_id:
                    param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', None)])
                if not param_ant_pres_id:
                    raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Concepto anticipo prestaciones para la nomina %s. Ir a \
                                    la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                    registrar un parametro de tipo Concepto anticipo prestaciones')
                                    %(employee.hr_payroll_id.name))

                concept_ant_prest_id = parameter_obj.browse(cr, uid, param_ant_pres_id)[0].concept_ant_presta_id
                concept_loan_ids = self.pool.get('hr.concept.loan').search(cr, uid, [('concept_allocation_id', '<>', False)])
                concept_loan_brw = self.pool.get('hr.concept.loan').browse(cr, uid, concept_loan_ids)
                concept_allocation_brw = [x.concept_allocation_id for x in concept_loan_brw]
                
                concept_allocation_ids = [concept_ant_prest_id.id] + [x.id for x in concept_allocation_brw]
                exist_attr = False
                for attr in employee.attribute_ids:
                    date_end = attr.date_end >= period.date_start if attr.date_end else True
                    if attr.concept_id.id in concept_allocation_ids and attr.date_start <= period.date_end and date_end:
                        exist_attr = True
                        break
                if not exist_attr:
                    continue
                    
            employee_in_process = True
            old_payslip_id = hr_payslip.search(cr, uid, [('employee_id', '=', employee.id),('hr_process_id', '=', ids[0])])
            if not old_payslip_id:

                res = {
                    'employee_id': employee.id,
                    'hr_process_id': ids[0],
                    'company_id': employee.company_id.id,
                    'line_ids':[],
                    'input_line_ids': [],
                    'worked_days_line_ids': [],
                    'name':_('Hoja de pago %s - proceso %s') % (self.pool.get('hr.employee').name_get(cr, uid, [employee.id])[0][1], process[0].name),
                    'contract_id': False,
                    'payroll_structure_id': False,
                    'hr_payroll_id': employee.hr_payroll_id.id,
                    'state': 'draft',
                    'date_from': period.date_start, 
                    'date_to': period.date_end,
                }
                
                new_hr_payslip = hr_payslip.create(cr, uid, res)
                
        if not employee_in_process and process[0].type == 'vacaciones':
            raise osv.except_osv(_('Advertencia!'), _('No hay empleados con vacaciones pendientes en el periodo seleccionado'))

        if not employee_in_process and process[0].type == 'guarderias':
            raise osv.except_osv(_('Advertencia!'), _('No hay empleados con hijos para pago de guardería'))

        if not employee_in_process and process[0].type == 'anticipos':
            raise osv.except_osv(_('Advertencia!'), _('No hay empleados pendientes por pago de anticipos'))

        variable_concept_ids = self.pool.get('hr.variable.concepts').search(cr, uid, [('hr_process_id', 'in', ids)])
        if not isinstance(variable_concept_ids, list):
            variable_concept_ids = [variable_concept_ids]
        if variable_concept_ids:
            for variable_concept_id in variable_concept_ids:
                self.pool.get('hr.variable.concepts').write(cr, uid, variable_concept_id, {'state': 'load'}, context=context)
        return self.write(cr, uid, ids, {'state': 'load'}, context=context)

    def execute_compute_sheet(self, cr, uid, ids, context=None):
        """
        Este metodo controla la ejecucion de la accion planificada de los procesos
        de hr_process.
        """
        payslip = self.pool.get('hr.payslip')
        process = payslip.browse(cr, uid, ids, context)[0].hr_process_id
        self._logger.info("%s: Inicio la ejecucion del proceso." % (process.name,))
        subject = ""
        message = ""
        excepcion = False
        #Se ejecuta el proceso para cada payslip
        try:
            payslip.calcular_payslips(cr, uid, ids, context)
            subject = "%s: Finalizo la ejecucion del proceso." % (process.name,)
            message = "La ejecucion del proceso '%s' se realizo con exito." % (process.name,)
            self._logger.info("%s: Finalizo con exito la ejecucion del proceso." % (process.name,))
        except Exception, e:
            excepcion = True
            subject = process.name + ": Error en la ejecucion del proceso."
            message = "La ejecucion del proceso '%s' devolvio el siguiente error: %s." % (process.name, e.value)
            self._logger.exception("%s: Error la ejecucion del proceso." % (process.name,))
        finally:
            self.send_email_notification(cr, uid, ids, subject, message, context)
            self.send_request(cr, uid, ids, subject, message, context=None)

            #La accion planificada se borra. Su nombre es el nombre del proceso de nomina.
            cron = self.pool.get('ir.cron')
            accion_planificada = cron.search(cr, uid, [('name','=', process.name)])
            cron.unlink(cr, uid, accion_planificada)
            self._logger.info("%s Se elimino la accion planificada que ejecuto el proceso" %(process.name,))

            #Se devuelve el proceso de nomina a estado 'review' (Procesado)
            self.write(cr, uid, process.id, {'state': 'review'}, context=context)
            if not excepcion:
                self.write(cr, uid, process.id, {'date_review':strftime('%Y-%m-%d')}, context=context)

            #Si existen conceptos variables se devuelven a estado 'review' (Procesado)
            v_concepts = self.pool.get('hr.variable.concepts')
            variable_concept_ids = v_concepts.search(cr, uid, [('hr_process_id', '=', process.id)])
            if not isinstance(variable_concept_ids, list):
                variable_concept_ids = [variable_concept_ids]
            if variable_concept_ids:
                for variable_concept_id in variable_concept_ids:
                    v_concepts.write(cr, uid, variable_concept_id, {'state': 'review'}, context=context)

            return True

    def procesar_nomina(self, cr, uid, ids, context=None):
        """
        Este metodo coloca el proceso de nomina y sus conceptos variables
        en estado 'execute'. Crea una accion planificada para ejecutar
        el proceso de nomina en segundo plano. Error en caso de que no exista
        el parametro de nomina 'Correo para notificaciones de procesos de pago' o en
        caso de que no existan hojas de pago para calcular
        """
        parameter_obj = self.pool.get('hr.parameters')
        parametro_correo = parameter_obj.search(cr, uid, [('type', '=', 'correos')])
        if not parametro_correo:
            raise osv.except_osv(_('ERROR!'), _("No hay un parametro definido 'Correo para notificaciones de procesos de pago' . Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo 'Correo para notificaciones de procesos de pago'")
                                 )
        process = self.browse(cr, uid, ids, context=context)
        payslip_ids = [payslip.id for payslip in process[0].hr_payslip_ids]

        if not payslip_ids:
            raise osv.except_osv(_('ERROR!'),_('El proceso no tiene hojas de pago para calcular'))
        else:
            #Se colocan en estado 'execute' (En ejecucion) el proceso y los conceptos variables asociados.
            self.write(cr, uid, ids, {'state': 'execute'}, context=context)
            variable_concept_ids = self.pool.get('hr.variable.concepts').search(cr, uid, [('hr_process_id', 'in', ids)])
            if not isinstance(variable_concept_ids, list):
                variable_concept_ids = [variable_concept_ids]
            if variable_concept_ids:
                for variable_concept_id in variable_concept_ids:
                    self.pool.get('hr.variable.concepts').write(cr, uid, variable_concept_id, {'state': 'execute'}, context=context)

            vals = {
                    'name': process[0].name,
                    'active': True,
                    'user_id': uid,
                    'interval_number': 1,
                    'interval_type':'minutes',
                    'numbercall': 1,
                    'doall': False,
                    'model': 'hr.process',
                    'function': 'execute_compute_sheet',
                    'args':'(' + str(payslip_ids) + ',)',
               }
            self.pool.get('ir.cron').create(cr, uid, vals, context)
            subject = "%s: Se ha iniciado la ejecucion del proceso." % (process[0].name,)
            self._logger.info("%s Se creo la accion planificada para la ejecucion del proceso." %(process[0].name,))
            self.send_request(cr, uid, ids, subject, message="", context=None)

        return True
    
    def asientos_nomina(self, cr, uid, process_brw, lines_brw, value, reserva, context=None):
        ''' 
        Metodo para la creacion de los asientos contables
        @param process_brw: objeto Browseable que contiene el proceso
        de nomina (hr.process), value: entero que indica el valor por el 
        cual de multiplica el total de cada linea, reserva: valor booleano
        para saber que tipo de asiento se creara (nomina o reserva)
        @return : Lista de ids de asientos creados
        '''
        obj_moves = self.pool.get('account.move')
        obj_acm = self.pool.get('account.move.line')
        obj_period = self.pool.get('account.period')
        obj_company = self.pool.get('res.company')
        obj_hr_acct = self.pool.get('hr.account')
        
        total_dict = {}
        contrapartida_dict = {}
        
        payrolls = []
        # Creamos las tuplas para totalizar los account.move.lines
        
        # Comprobar la existencia de la configuracion contable para todos los conceptos
        self._logger.info("Comprobar la existencia de la configuracion contable para todos los conceptos")
        for line in lines_brw:
            concept = line.salary_rule_id.concept_id if line.salary_rule_id else line.attribute_id.concept_id
            department = line.employee_id.department_id
            payroll = line.employee_id.hr_payroll_id
            
            account_id = obj_hr_acct.search(cr, uid, 
                      [('concept_id', '=', concept.id),
                      ('analytic_account_id', '=', department.departament_analytic_account_id.id),
                      ('payroll_id', '=', payroll.id)])

            if not account_id:
                raise osv.except_osv(_('Error!'), 
                  _('No existe una configuración contable para el concepto\
                  %s, cuenta analítica %s y nómina %s. Para verificar ir a\
                  Talento Humano -> Configuración -> Contabilidad nómina.')
                  % (concept.name, department.departament_analytic_account_id.complete_name, 
                  payroll.name))
                  
        for line in lines_brw:
            concept = line.salary_rule_id.concept_id if line.salary_rule_id else line.attribute_id.concept_id
            department = line.employee_id.department_id
            payroll = line.employee_id.hr_payroll_id
            payrolls.append(payroll.id)
            
            account_id = obj_hr_acct.search(cr, uid, 
                      [('concept_id', '=', concept.id),
                      ('analytic_account_id', '=', department.departament_analytic_account_id.id),
                      ('payroll_id', '=', payroll.id)])

            account = obj_hr_acct.browse(cr, uid, account_id)[0]
            
            concept_id = account.concept_id
            payroll_id = account.payroll_id
            account_id = account.account_id
            type_calc = concept.type_calc
            partner_id = line.employee_id.company_id.partner_id
            
            if not account_id.user_type.code in ['expense']:
                analytic_account_id = line.employee_id.hr_sede_id.analytic_id
            else:
                analytic_account_id = account.analytic_account_id

            if (concept_id, analytic_account_id, payroll_id, account_id, type_calc, partner_id) not in total_dict:
                total_dict[concept_id, analytic_account_id, payroll_id, account_id, type_calc, partner_id] = line.total*value
            else:
                total_dict[concept_id, analytic_account_id, payroll_id, account_id, type_calc, partner_id] = \
                total_dict[concept_id, analytic_account_id, payroll_id, account_id, type_calc, partner_id] + line.total*value

            # Lineas de contrapartidas
            if not account.account_payroll_id.user_type.code in ['expense']:
                analytic_account_id = line.employee_id.hr_sede_id.analytic_id
            else:
                analytic_account_id = account.analytic_account_id
                
            if (None, analytic_account_id, payroll_id, account.account_payroll_id, None, partner_id) not in contrapartida_dict:
                if concept_id.type_calc == 'allocation':
                    contrapartida_dict[None, analytic_account_id, payroll_id, account.account_payroll_id, None, partner_id] = line.total*value * -1
                elif concept_id.type_calc == 'deduction':
                    contrapartida_dict[None, analytic_account_id, payroll_id, account.account_payroll_id, None, partner_id] = line.total*value
            else:
                if concept_id.type_calc == 'allocation':
                    contrapartida_dict[None, analytic_account_id, payroll_id, account.account_payroll_id, None, partner_id] = \
                            contrapartida_dict[None, analytic_account_id, payroll_id, account.account_payroll_id, None, partner_id] + (line.total*value * -1)
                elif concept_id.type_calc == 'deduction':
                    contrapartida_dict[None, analytic_account_id, payroll_id, account.account_payroll_id, None, partner_id] = \
                            contrapartida_dict[None, analytic_account_id, payroll_id, account.account_payroll_id, None, partner_id] + line.total*value
                    
        payrolls = list(set(payrolls))
        company_id = obj_company._company_default_get(cr, uid, 
          'account.move', context=context)
        
        cont = len(total_dict) + len(contrapartida_dict)
        linea = 0
        moves_dict = {}
        res = []
        
        # Creamos los asientos para cada nomina
        for pr in self.pool.get('hr.payroll').browse(cr, uid, payrolls):
            period = list(set(process_brw.hr_period_ids) & set(pr.hr_payment_calendar_id.hr_period_ids))[0]
            fecha = obj_period.browse(cr, uid, period.period_account_id.id).date_stop
            move = { 
                'ref' : 'Asiento de nomina ' + process_brw.name + '/' + pr.name if not reserva \
                                    else 'Asiento de reserva ' + '/' + pr.name + ' mes ' + period.hr_month_id.name,
                'to_check': 1,
                'journal_id': pr.journal_id.id,
                'period_id': period.period_account_id.id,
                'date': fecha,
                'state': 'draft',
                'company_id': int(company_id),
                'hr_process_id': process_brw.id,
            }

            moves_dict[pr.id] = obj_moves.create(cr, uid, move, context=context)
        
        # Creamos los apuntes
        total_dict.update(contrapartida_dict)
        for key in total_dict.keys():
            concept, analytic_account, payroll, account, type_calc, partner = key
            period = list(set(process_brw.hr_period_ids) & set(payroll.hr_payment_calendar_id.hr_period_ids))[0]
            fecha = obj_period.browse(cr, uid, period.period_account_id.id).date_stop
            if concept:
                debit = total_dict[key] if type_calc == 'allocation' else 0.0
                credit = total_dict[key] if type_calc == 'deduction' else 0.0
            else:
                debit = total_dict[key] if total_dict[key] > 0 else 0.0
                credit = total_dict[key]*-1 if total_dict[key] < 0 else 0.0

            moves_lines = {
                'journal_id': payroll.journal_id.id,
                'credit': round(credit, 2),
                'debit': round(debit, 2),
                'account_id': account.id,
                'period_id': period.period_account_id.id,
                'move_id': moves_dict[payroll.id],
                'name': concept.name if concept else account.code,
                'state': 'draft',
                'analytic_account_id': analytic_account.id,
                'partner_id': partner.id if account.type in ('payable', 'receivable') else 0,
                'fecha': fecha,
            }
            linea += 1
            
            if linea == cont:
                # Si es el ultimo apunte se crea nativamente para que 
                # tambien se valida el asiento.
                moves_lines_id = obj_acm.create(cr, uid, moves_lines, context=context)
            elif moves_lines['partner_id'] > 0:
                cr.execute("insert into account_move_line (account_id,name,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,partner_id,create_uid,create_date,analytic_account_id) values (%s,'%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s','%s',%s,False,%s,%s,now(),%s)"%(moves_lines['account_id'], moves_lines['name'],moves_lines['journal_id'],1,moves_lines['credit'],moves_lines['state'],moves_lines['period_id'],moves_lines['debit'],moves_lines['fecha'],moves_lines['fecha'],moves_lines['move_id'],moves_lines['partner_id'],uid,moves_lines['analytic_account_id']))
            else:
                cr.execute("insert into account_move_line (account_id,name,centralisation,journal_id,company_id,currency_id,credit,state,period_id,debit,date,date_created,move_id,blocked,create_uid,create_date,analytic_account_id) values (%s,'%s','normal',%s,%s,NULL,%s,'%s',%s,%s,'%s','%s',%s,False,%s,now(),%s)"%(str(moves_lines['account_id']), str(moves_lines['name']),str(moves_lines['journal_id']),str(1),str(moves_lines['credit']),str(moves_lines['state']),str(moves_lines['period_id']),str(moves_lines['debit']),str(moves_lines['fecha']),str(moves_lines['fecha']),str(moves_lines['move_id']),str(uid),moves_lines['analytic_account_id']))
                    
        #Colocar en estado asentado el asiento contable que se acaba de generar
        res = [moves_dict[x] for x in moves_dict]
        self.pool.get('account.move').button_validate(cr, uid, res) 
        for r in res:
            self._logger.info("Se validaron: %s.", (self.pool.get('account.move').browse(cr, uid, r).ref))
        return res
        
    def crear_asientos_reserva(self, cr, uid, process_brw, context=None):    
        ''' 
        Metodo para la creacion de los asientos contables de reserva 
        de vacaciones y utilidades
        @param process_brw: objeto Browseable que contiene el proceso
        de nomina (hr.process)
        @return : Lista de ids de asientos creados
        '''
        obj_payslip_line = self.pool.get('hr.payslip.line')
        obj_parameter = self.pool.get('hr.parameters')
                
        total_dict = {}
        contrapartida_dict = {}
        
        #Busqueda del parametro que indica cuales son los conceptos para los asientos de reserva
        param_reserva_id = obj_parameter.search(cr, uid, [('type', '=', 'conceptos_reserva')])
        if not param_reserva_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para los Conceptos del asiento de reserva. Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo Conceptos del asiento de reserva'))

        concept_reserva_brw = obj_parameter.browse(cr, uid, param_reserva_id)[0].concept_reserva_ids
        concept_reserva_ids = [x.id for x in concept_reserva_brw]
        
        # Obtenemos todas las lineas de pago
        slip_lines_ids = obj_payslip_line.search(cr, uid, [('process_id', '=', process_brw.id), 
                                        ('concept_id', 'in', concept_reserva_ids)], context=context)
        
        slip_lines_brw = obj_payslip_line.browse(cr, uid, slip_lines_ids)
        return self.asientos_nomina(cr, uid, process_brw, slip_lines_brw, 30, True, context)
        
    def crear_asientos(self, cr, uid, process_brw, context=None):
        ''' 
        Metodo para la creacion de los asientos contables de la nomina
        @param process_brw: objeto Browseable que contiene el proceso
        de nomina (hr.process)
        @return : Lista de ids de asientos creados
        '''
        obj_concept = self.pool.get('hr.concept')
        obj_payslip_line = self.pool.get('hr.payslip.line')
        
        total_dict = {}
        contrapartida_dict = {}

        # Obtenemos los codigos de los conceptos de aportes patronales
        aportes_ids = obj_concept.search(cr, uid, 
                                [('type', '=', 'aporte_patronal')])
        
        # Obtenemos todas las lineas de pago
        slip_lines_ids = obj_payslip_line.search(cr, uid, [
                                  ('process_id', '=', process_brw.id), 
                                  '|', ('concept_id', 'in', aportes_ids), 
                                  ('appears_on_payslip', '=', True)], context=context)
          
        slip_lines_brw = obj_payslip_line.browse(cr, uid, slip_lines_ids)
        return self.asientos_nomina(cr, uid, process_brw, slip_lines_brw, 1, False, context)

    def crear_orden_pago_guarderia(self, cr, uid, process_brw, context=None):
        total_dict = {}
        account_vaucher_obj = self.pool.get('account.voucher')
        account_vaucher_line_obj = self.pool.get('account.voucher.line')

        for paylip_brw in process_brw.hr_payslip_ids:
            #Busqueda del diario que se definio como diario para las ordenes de pago en los parametros de la nomina-----------------------------
            parameter_obj = self.pool.get('hr.parameters')
            param_journal_id = parameter_obj.search(cr, uid, [('type', '=', 'diario_orden_p'),('payroll_id', '=', paylip_brw.employee_id.hr_payroll_id.id)])
            if not param_journal_id:
                param_journal_id = parameter_obj.search(cr, uid, [('type', '=', 'diario_orden_p'),('payroll_id', '=', None)])
            if not param_journal_id:
                raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el diario de ordenes de pago para la nomina %s. Ir a \
                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                registrar un parametro de tipo Diario para ordenes de pago')
                                %(paylip_brw.employee_id.hr_payroll_id.name))

            journal_brw = parameter_obj.browse(cr, uid, param_journal_id)[0].journal_id

            period = list(set(process_brw.hr_period_ids) & set(paylip_brw.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
            '''Se llenará un diccionario de la siguiente forma: La llave del diccionario contiene el empleado y el total
            a pagar en guarderia por ese empleado, y en el valor del diccionario contiene el detalle de cada employee_family
            con su respectivo monto de guarderia. Ejemplo: {(empleado Carlos, guarderia, 800):[(hijo_1, 600),(hijo_2, 200)]}'''
            dict_guarderias = {}
            dict_hijos = {}
            list_employee_fam = []
            for paylip_line_brw in paylip_brw.line_ids:
                if paylip_line_brw.employee_family_id.guarderia_id not in dict_guarderias:
                    list_employee_fam = [(paylip_line_brw.employee_family_id, round(paylip_line_brw.total, 2))]
                    dict_hijos[paylip_line_brw.employee_family_id.guarderia_id] = list_employee_fam
                    dict_guarderias[paylip_line_brw.employee_family_id.guarderia_id] = round(paylip_line_brw.total, 2)
                else:
                    list_employee_fam.append((paylip_line_brw.employee_family_id, round(paylip_line_brw.total, 2)))
                    dict_hijos[paylip_line_brw.employee_family_id.guarderia_id] = list_employee_fam
                    dict_guarderias[paylip_line_brw.employee_family_id.guarderia_id] = \
                        dict_guarderias[paylip_line_brw.employee_family_id.guarderia_id] + round(paylip_line_brw.total, 2)

                for item in dict_guarderias:
                    total_dict[paylip_brw.employee_id, period, item, dict_guarderias[item]] = dict_hijos[item]

        for employee, period, guarderia, total in total_dict:
            account_id = guarderia.partner_id.property_account_payable

            vals_av = {
                'type': 'payment',
                'name': 'Pago guarderia ' + str(self.pool.get('hr.employee').name_get(cr, uid, [employee.id])[0][1]) + ' / Periodo ' + str(period.name) + ' / ' + str(guarderia.name),
                'amount': total,
                'partner_id': guarderia.partner_id.id,
                'pay': 'asiento',
                'period_id': period.period_account_id.id,
                'journal_id': journal_brw.id,
                'hr_process_id': process_brw.id,
                'account_id': journal_brw.default_credit_account_id.id,
            }
            idav = account_vaucher_obj.create(cr, uid, vals_av, context)

            list_employee_fam = total_dict[(employee,period,guarderia,total)]
            for hijo, guar_monto in list_employee_fam:
                vals_avl = {
                    'voucher_id': idav,
                    'account_id': account_id.id,
                    'amount': guar_monto,
                    'type': 'dr',
                    'partner_id': guarderia.partner_id.id,
                }
                idavl = account_vaucher_line_obj.create(cr, uid, vals_avl, context)
                
            self._logger.info("Se creo orden de %s" %(vals_av['name'],))
        return True

    def crear_orden_pago_nomina(self, cr, uid, process_brw, context=None):
        total_dict = {}
        account_vaucher_obj = self.pool.get('account.voucher')
        account_vaucher_line_obj = self.pool.get('account.voucher.line')

        for paylip_brw in process_brw.hr_payslip_ids:

            period = list(set(process_brw.hr_period_ids) & set(paylip_brw.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
            payroll = paylip_brw.employee_id.hr_payroll_id
            employee_id = paylip_brw.employee_id
            account_id = paylip_brw.employee_id.hr_payroll_id.account_id
            journal_id = paylip_brw.employee_id.journal_id
            partner_id = paylip_brw.employee_id.company_id.partner_id
            amount_allocation = sum([line.total for line in paylip_brw.line_ids if line.type_calc == 'allocation'])
            amount_deduction = sum([line.total for line in paylip_brw.line_ids if line.type_calc == 'deduction'])
            if not employee_id.check: #Si al empleado se le deposita a una cuenta bancaria
                if (payroll, False, period, account_id, journal_id, partner_id) not in total_dict:
                    total_dict[(payroll, False, period, account_id, journal_id, partner_id)] = amount_allocation - amount_deduction

                else:
                    total_dict[(payroll, False, period, account_id, journal_id, partner_id)] = \
                            total_dict[(payroll, False, period, account_id, journal_id, partner_id)] + (amount_allocation - amount_deduction)
            else: #Si al empleado se le cancela con cheque
                total_dict[(payroll, employee_id, period, account_id, journal_id, partner_id)] = amount_allocation - amount_deduction


        for payroll, employee, period, account, journal, partner in total_dict:
            if employee == False:
                name = payroll.name
            else:
                name = employee.name
            total = total_dict[(payroll, employee, period, account, journal, partner)]
            vals_av = {
                'type': 'payment',
                'name': 'Pago nomina ' + str(name) + ' ' + str(process_brw.name) + ' / Periodo ' + str(period.name),
                'amount': round(total, 2),
                'partner_id': partner.id,
                'pay': 'check' if employee else 'asiento',
                'period_id': period.period_account_id.id,
                'journal_id': journal.id,
                'account_id': journal.default_credit_account_id.id,
                'hr_process_id': process_brw.id,
            }
            idav = account_vaucher_obj.create(cr, uid, vals_av, context)
            vals_avl = {
                'voucher_id': idav,
                'account_id': account.id,
                'amount': round(total, 2),
                'type': 'dr',
                'partner_id': partner.id,
            }
            idavl = account_vaucher_line_obj.create(cr, uid, vals_avl, context)
            self._logger.info("Se creo orden de %s" %(vals_av['name'],))
        return True

    def loan_update(self, cr, uid, payslip, aprobar_proceso, context):
        ''' Esta funcion verifica las lineas dentro de la hoja de pago payslip para que en caso de provenir
            de un pretamo, este sea acualizado dependiendo de si se esta aprobando un proceso
            o se esta devolviendo
        '''
        if aprobar_proceso: # Se esta aprobando el proceso
            for paylip_line_brw in payslip.line_ids:
                ''' Si la linea viene de un atributo y este a su vez proviene de un prestamo (hr.loan) el prestamo se debe sobreescribir
                 para actualizar la deuda restante y el numero de cuotas pagadas por el empleado. Ademas se debe pasar a inactivo el atributo 
                 de asignacion que se pudo haber creado con el prestamo'''
                 
                if paylip_line_brw.attribute_id.allocation_loan_ids:
                    period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
                    for loan in paylip_line_brw.attribute_id.allocation_loan_ids:
                        attr = loan.attr_allocation_id
                        if attr:
                            date_end_attr = attr.date_end if attr.date_end else False  
                            if date_end_attr and date_end_attr <= period.date_end:
                                self.pool.get('hr.attribute').write(cr, uid, attr.id, {'active':False}, context)
                                
                if paylip_line_brw.attribute_id.loan_ids:
                    vals_loan = {}
                    for loan in paylip_line_brw.attribute_id.loan_ids:
                        vals_loan['paid_quotas'] = loan.paid_quotas + 1
                        vals_loan['paid_amount'] = loan.paid_amount + paylip_line_brw.total
                        self.pool.get('hr.loan').write(cr, uid, loan.id, vals_loan, context)
                        
                        '''Si con este descuento faltara solo una cuota para terminar de pagar el prestamo, se debe verificar si existe un atributo
                        de saldo remanente para modificar el atributo para la cancelacion de la ultima cuota'''
                        if (loan.num_quotas == vals_loan['paid_quotas']+1):
                            new_amount = loan.created_attr_id.amount + loan.remaining_attr_id.amount if loan.remaining_attr_id else loan.created_attr_id.amount
                            self.pool.get('hr.attribute').write(cr, uid, loan.created_attr_id.id, {'amount':new_amount,}, context)

                        '''Si con este descuento se termina de cancelar el prestamo el atributo correspondiente se pasa
                        a inactivo'''
                        if (loan.num_quotas == vals_loan['paid_quotas']) or (loan.amount == vals_loan['paid_amount']):
                            vals_loan['date_end'] = datetime.now().strftime('%Y-%m-%d')
                            vals_loan['active'] = False 
                            self.pool.get('hr.loan').write(cr, uid, loan.id, vals_loan, context)
                            self.pool.get('hr.attribute').write(cr, uid, loan.created_attr_id.id,
                                        {'active':False,'date_end':datetime.now().strftime('%Y-%m-%d')}, context)
        else: # Se esta devolviendo el proceso
            for paylip_line_brw in payslip.line_ids:
                ''' Si la linea viene de un atributo y este a su vez proviene de un prestamo (hr.loan) el prestamo se debe sobreescribir
                 para actualizar la deuda restante y el numero de cuotas pagadas por el empleado. Ademas se debe devolver a activo el atributo 
                 de asignacion que se pudo haber creado con el prestamo '''
                
                if paylip_line_brw.attribute_id.allocation_loan_ids:
                    period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
                    for loan in paylip_line_brw.attribute_id.allocation_loan_ids:
                        attr = loan.attr_allocation_id
                        if attr:
                            date_end_attr = attr.date_end if attr.date_end else False  
                            if date_end_attr and date_end_attr <= period.date_end:
                                self.pool.get('hr.attribute').write(cr, uid, attr.id, {'active':True}, context)
                                
                loan_ids = self.pool.get('hr.loan').search(cr, uid, [('employee_id','=',payslip.employee_id.id),
                                                                ('created_attr_id','=', paylip_line_brw.attribute_id.id), ('active','=',False)]) \
                            + self.pool.get('hr.loan').search(cr, uid, [('employee_id','=',payslip.employee_id.id),
                                                                ('created_attr_id','=', paylip_line_brw.attribute_id.id)])

                if loan_ids:
                    vals_loan = {}
                    for loan in self.pool.get('hr.loan').browse(cr, uid, loan_ids, context):
                        vals_loan['paid_quotas'] = loan.paid_quotas - 1
                        vals_loan['paid_amount'] = loan.paid_amount - paylip_line_brw.total
                        vals_loan['date_end'] = None
                        vals_loan['active'] = True
                        self.pool.get('hr.loan').write(cr, uid, loan.id, vals_loan, context)
                        '''Si el atributo de habia pasado a inactivo porque la deuda se habia cancelado conpleta,
                        el atributo debe ser pasado nuevamente a activo'''
                        if not loan.created_attr_id.active:
                            self.pool.get('hr.attribute').write(cr, uid, loan.created_attr_id.id, {'active':True,'date_end':None}, context)

    def aprobar_proceso(self, cr, uid, ids, context=None):
        parameter_obj = self.pool.get('hr.parameters')
        process_brw = self.browse(cr, uid, ids, context=context)[0]
        
        if self.pool.get('hr.payslip').search(cr, uid, [('hr_process_id','=',process_brw.id),('total','<=',0.0)]):
            raise osv.except_osv(_('Error!'), _('No se puede aprobar el proceso. Existen hojas de pago con total <= 0.0'))
        if self.pool.get('hr.payslip').search(cr, uid, [('hr_process_id','=',process_brw.id),('state','=','except')]):
            raise osv.except_osv(_('Error!'), _('No se puede aprobar el proceso. Existen hojas de pago en Estado Excepción'))
        if self.pool.get('hr.payslip').search(cr, uid, [('hr_process_id','=',process_brw.id),('state','<>','review')]):
            raise osv.except_osv(_('Error!'), _('No se puede aprobar el proceso. Existen hojas de pago en Estado Borrador'))

        if process_brw.type in ('nomina','anticipos','utilidades','pago_especial'):
            self.crear_asientos(cr, uid, process_brw, context)
            self.crear_orden_pago_nomina(cr, uid, process_brw, context)

        if process_brw.type == 'vacaciones':
            self.crear_asientos(cr, uid, process_brw, context)
            self.crear_orden_pago_nomina(cr, uid, process_brw, context)
            for payslip in process_brw.hr_payslip_ids:
                #Busqueda del concepto que se definio como Dias de disfrute de vacaciones
                param_dias_vac_id = parameter_obj.search(cr, uid, [('type', '=', 'dias_disfrute_vacaciones'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
                if not param_dias_vac_id:
                    param_dias_vac_id = parameter_obj.search(cr, uid, [('type', '=', 'dias_disfrute_vacaciones'),('payroll_id', '=', None)])
                if not param_dias_vac_id:
                    raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Dias de disfrute de vacaciones %s. Ir a \
                                    la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                    registrar un parametro de tipo Dias de disfrute de vacaciones')
                                    %(payslip.employee_id.hr_payroll_id.name))

                dias_vac = parameter_obj.browse(cr, uid, param_dias_vac_id)[0].value
        
                #Se deben pasar a state done los periodos de vacaciones asociados a cada empleado en el proceso de vacaciones
                hol_employee_ids = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id', '=', payslip.employee_id.id),
                                                        ('hr_process_id', '=', process_brw.id)])
                hol_employee_brw = self.pool.get('hr.employee.holidays').browse(cr, uid, hol_employee_ids)[0]
                
                period = list(set(payslip.hr_process_id.hr_period_ids) & set(payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
                period_start_year = datetime.strptime(period.date_start,"%Y-%m-%d").year
                contract_date = datetime.strptime(payslip.employee_id.start_rel_lab,"%Y-%m-%d")
                contract_year = contract_date.year

                #Se cancelan 1 dia adicional por año a partir del 2º año
                dias_adicionales = period_start_year-contract_year-1 if period_start_year-contract_year > 1 else 0
                dias_adicionales = dias_adicionales if dias_adicionales < 15 else 15
                                
                holiday_status_id = self.pool.get('hr.holidays.status').search(cr, uid, [('name', '=', 'Vacaciones Legales')])[0]
                wf_service = netsvc.LocalService("workflow")
                #Crear la asignacion de dias de vacaciones y dias adicionales
                vals_holidays = {
                    'name': hol_employee_brw.name,
                    'employee_id': hol_employee_brw.employee_id.id,
                    'holiday_status_id': holiday_status_id,
                    'number_of_days_temp': dias_vac,
                    'number_of_days': dias_vac,
                    'state': 'draft',
                    'type': 'add',
                    'employee_holidays_add_id': hol_employee_brw.id,
                }
                
                holiday_id = self.pool.get('hr.holidays').create(cr, uid, vals_holidays)
                wf_service.trg_validate(uid, 'hr.holidays', holiday_id, 'confirm', cr)
                wf_service.trg_validate(uid, 'hr.holidays', holiday_id, 'validate', cr)
                    
                if dias_adicionales:
                    vals_holidays.update({
                        'name': hol_employee_brw.name + '- Dias adicionales',
                        'number_of_days_temp':dias_adicionales,
                        'number_of_days':dias_adicionales,
                    })
                    
                    holiday_id = self.pool.get('hr.holidays').create(cr, uid, vals_holidays)
                    wf_service.trg_validate(uid, 'hr.holidays', holiday_id, 'confirm', cr)
                    wf_service.trg_validate(uid, 'hr.holidays', holiday_id, 'validate', cr)

                self.pool.get('hr.employee.holidays').write(cr, uid, hol_employee_ids, {'state':'done'}, context)

        if process_brw.type == 'prestaciones':
            self.crear_asientos_reserva(cr, uid, process_brw, context)
            self.crear_asientos(cr, uid, process_brw, context)
            for payslip in process_brw.hr_payslip_ids:
                hol_employee_ids = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id','=', payslip.employee_id.id),('state','=','done'),
                                                                            ('prest_calc','=',False)])
                if hol_employee_ids:
                    self.pool.get('hr.employee.holidays').write(cr, uid, hol_employee_ids, {'prest_calc':True}, context)
            
        for payslip_brw in process_brw.hr_payslip_ids:                                
            self.loan_update(cr, uid, payslip_brw, True, context) # El True le indica a la funcion que se esta aprobando un proceso
            
            if process_brw.type in ['nomina', 'vacaciones']:
                self._logger.info("Calculando devengados: Hoja de pago %s"%(payslip_brw.employee_id.name))
                self.pool.get('hr.payslip').calculo_acumulados(cr, uid, payslip_brw, context)
                
        variable_concept_ids = self.pool.get('hr.variable.concepts').search(cr, uid, [('hr_process_id', 'in', ids)])
        if not isinstance(variable_concept_ids, list):
            variable_concept_ids = [variable_concept_ids]
        if variable_concept_ids:
            for variable_concept_id in variable_concept_ids:
                self.pool.get('hr.variable.concepts').write(cr, uid, variable_concept_id, {'state': 'done'}, context=context)

        self.pool.get('hr.payslip').write(cr, uid, [x.id for x in process_brw.hr_payslip_ids], {'state':'done'})
        self._logger.info("%s: Se aprobo el proceso" %(process_brw.name,))
        return self.write(cr, uid, ids, {'state': 'done', 'date_done': datetime.now().strftime('%Y-%m-%d')}, context=context)

    def eliminar_asientos(self, cr, uid, move_ids, context=None):
        diarios = []
        for account_move in move_ids:
            analytic_lines = []
            diarios.append(account_move.journal_id.id)
            for move_line in account_move.line_id:
                analytic_lines = analytic_lines + self.pool.get('account.analytic.line').search(cr, uid, [('move_id', '=', move_line.id)])
            analytic_lines = str(analytic_lines).replace('[','(').replace(']',')')
            cr.execute("delete from account_analytic_line where id in %s"%(analytic_lines))
            cr.execute("delete from account_move where id = %s"%(account_move.id))
        return True
        
    def devolver_proceso(self, cr, uid, ids, context=None):
        process_brw = self.browse(cr, uid, ids, context=context)[0]

        ''' Validaciones necesarias:
            1- Verificar que el proceso que se desea revertir sea el ultimo que se ha corrido
            2- Verificar que los periodos contables correspondientes esten abiertos
            3- Verificar que las ordenes de pago generadas al aprobar el proceso esten en estado borrador
            4- Verificar que no se haya generado el txt de la nomina
            5- Si es un proceso de guarderias y se han creado facturas no se puede devolver el proceso
        '''
        if process_brw.type == 'guarderias':
            if process_brw.account_invoice_ids:
                raise osv.except_osv(_('Error!'), _('No se puede realizar la operación. Al proceso ya se le generaron facturas \
                                                    de pago de guarderias'))
            
        for period in process_brw.hr_period_ids:
            if period.state == 'close' and process_brw.type <> 'prestaciones':
                raise osv.except_osv(_('Error!'), _('No se puede realizar la operación. El período %s presente en este proceso \
                                        ya se encuentra cerrado')%(period.name))

            if process_brw.type == 'prestaciones': # Si el proceso es de calculo de prestaciones no permitir que se devuelva el proceso si ya se han
                                                # creado procesos para el prox periodo
                if (period.type == 'diario' and period.period_in_year < 52) or \
                    (period.type == 'mensual' and period.period_in_year < 24):
                    next_period_id = self.pool.get('hr.period').search(cr, uid, [('period_in_year','=', period.period_in_year+1),
                                                                        ('hr_payment_calendar_id','=', period.hr_payment_calendar_id.id)])

                elif (period.type == 'diario' and period.period_in_year == 52) or \
                    (period.type == 'mensual' and period.period_in_year == 24):
                    next_year_id = self.pool.get('hr.year').search(cr, uid, [('year', '=', period.hr_month_id.hr_year_id.year+1)])
                    if next_year_id:
                        next_calendar_id = self.pool.get('hr.payment.calendar').search(cr, uid, [('hr_year_id', '=', next_year_id),
                                                                                            ('type', '=', period.type)])
                        next_period_id = self.pool.get('hr.period').search(cr, uid, [('period_in_year','=', 1),
                                                                        ('hr_payment_calendar_id','=', next_calendar_id)])
                if next_period_id:
                    next_period = self.pool.get('hr.period').browse(cr, uid, next_period_id)
                    process_ids = self.pool.get('hr.process').search(cr, uid, [('hr_period_ids','=', next_period[0].id)])
                    if process_ids:
                        raise osv.except_osv(_('Error!'), _('No se puede realizar la operación. Ya se han creado procesos en el próximo \
                                            período %s')%(next_period[0].name))

            if period.period_account_id.state == 'done':
                raise osv.except_osv(_('Error!'), _('No se puede realizar la operación. El período contable %s correspondiente a este proceso \
                                        ya se encuentra cerrado')%(period.period_account_id.name))

        for account_voucher in process_brw.account_voucher_ids:
            if not (account_voucher.state == 'draft'):
                raise osv.except_osv(_('Error!'), _('No se puede realizar la operación. La orden de pago %s generada al aprobar este proceso \
                                        no se encuentra en estado Borrador (ya fue procesada)')%(account_voucher.name))

        # FALTA VALIDACION DEL TXT

        ''' Si pasa todas las validaciones se procede a eliminar los asientos contables:
            1- Ir al diario de cada asiento y habilitarlo para poder eliminar sus asientos
            2- Eliminar las lineas de asientos analiticos generados junto a los asientos contable
            3- Eliminar los asientos contables
            4- Ir de nuevo a los diarios para deshabilitarle la opcion que permite eliminar sus asientos
        '''
        self.eliminar_asientos(cr, uid, process_brw.account_move_ids, context)
        self._logger.info("%s: Se eliminaron los asientos" %(process_brw.name,))

        voucher_ids = [x.id for x in process_brw.account_voucher_ids]
        self.pool.get('account.voucher').unlink(cr, uid, voucher_ids)

        variable_concept_ids = self.pool.get('hr.variable.concepts').search(cr, uid, [('hr_process_id', 'in', ids)])
        if not isinstance(variable_concept_ids, list):
            variable_concept_ids = [variable_concept_ids]
        if variable_concept_ids:
            for variable_concept_id in variable_concept_ids:
                self.pool.get('hr.variable.concepts').write(cr, uid, variable_concept_id, {'state': 'review'}, context=context)

        for payslip_brw in process_brw.hr_payslip_ids:
            self.loan_update(cr, uid, payslip_brw, False, context) # El False le indica a la funcion que se esta devolviendo un proceso

            if process_brw.type == 'prestaciones':
                hol_employee_ids = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id','=', payslip_brw.employee_id.id),('state','=','done'),
                                                                            ('prest_calc','=',True)])
                if hol_employee_ids:
                    self.pool.get('hr.employee.holidays').write(cr, uid, hol_employee_ids, {'prest_calc':False}, context)

            if process_brw.type == 'vacaciones':
                period = list(set(process_brw.hr_period_ids) & set(payslip_brw.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
                #Se deben pasar a state done los periodos de vacaciones asociados a cada empleado en el proceso de vacaciones
                hol_employee_id = self.pool.get('hr.employee.holidays').search(cr, uid, [('employee_id', '=', payslip_brw.employee_id.id),
                                                    ('hr_process_id', '=', process_brw.id),('state', '=', 'done')])
                if hol_employee_id:
                    hol_employee_brw = self.pool.get('hr.employee.holidays').browse(cr, uid, hol_employee_id)[0]
                    holiday_add_id = [x.id for x in hol_employee_brw.holidays_add_ids]
                    if holiday_add_id:
                        wf_service = netsvc.LocalService("workflow")
                        wf_service.trg_validate(uid, 'hr.holidays', holiday_add_id[0], 'refuse', cr)
                        self.pool.get('hr.holidays').set_to_draft(cr, uid, holiday_add_id)
                        if self.pool.get('hr.holidays').unlink(cr, uid, holiday_add_id):
                            self.pool.get('hr.employee.holidays').write(cr, uid, hol_employee_id, {'state':'draft'}, context)
                    else:
                        self.pool.get('hr.employee.holidays').write(cr, uid, hol_employee_id, {'state':'draft'}, context)

        self.pool.get('hr.payslip').write(cr, uid, [x.id for x in process_brw.hr_payslip_ids], {'state':'review'})
        self._logger.info("%s: Se devolvio el proceso" %(process_brw.name,))
        return self.write(cr, uid, ids, {'state': 'review','date_done': False}, context=context)

    def send_email_notification(self, cr, uid, ids, subject, message, context=None):
        """
        Metodo que envia un correo electronico al grupo de emails especificados
        en el parametro 'Correo para notificaciones de procesos de pago.'
        """
        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),
                                        ('state','=','confirm')], context=False)
        if smtp_server:
            parameter_obj = self.pool.get('hr.parameters')
            parametro_correo = parameter_obj.search(cr, uid, [('type', '=', 'correos')])
            correos = parameter_obj.browse(cr, uid, parametro_correo)[0].correos

            email_to = correos.split(',')
            if email_to:
                smtp_server_id = smtp_server[0]
                server_name = smtp_client.browse(cr, uid, smtp_server_id).server
                email = {
                        'server_id': smtp_server_id,
                        'name': (subject),
                        'server': server_name,
                        'body': (message),
                        'to': email_to,
                        'from': uid,
                        }
                smtp_client.send_email(cr, uid, smtp_server_id, email['to'], email['name'], email['body'])

    def send_request(self, cr, uid, ids, subject, message, context=None):
        """
        Metodo que envia una notificacion interna de OpenERP a un usuario cuando
        ocurre algun evento de hr_process
        """
        notificaciones = {
                'name': (subject),
                'act_to': uid,
                'act_from': uid,
                'body': (message),
                'active': True,
                'priority': '2',
                'state': 'waiting'
            }
        request_obj = self.pool.get('res.request')
        request_obj.create(cr, uid, notificaciones, context)

hr_process()

class hr_payroll(osv.osv):

    _inherit = 'hr.payroll'
    _description = 'Nomina'

    _columns = {
        'hr_process_ids':fields.many2many('hr.process', 'hr_process_payroll_rel', 'hr_payroll_id', 'hr_process_id', 'Procesos de nomina'),
    }

hr_payroll()

class hr_department_employee(osv.osv):
    '''
        Clase para definir la relacion empleado-departamento para que s guarde el rastro de los departamentos
        en los que ha estado el empleado
    '''
    _description = "Departamentos donde ha estado asignado un empleado"
    _name = 'hr.department.employee'
    _rec_name = 'department'

    _columns = {
        'employee_id': fields.many2one('hr.employee', 'Empleado'),
        'date_start': fields.date('Fecha inicio'),
        'date_end': fields.date('Fecha fin'),
        'active': fields.boolean('Activo'),
        'sede': fields.char("Sede", size=256),
        'department_id':fields.char('Departamento', size=256),
        'job': fields.char("Cargo", size=256),
        'payroll': fields.char('Nomina', size=256),
        'id_payroll': fields.integer('id del payroll')
    }

    _defaults = {
        'date_start': lambda *args: strftime('%Y-%m-%d'),
        'active': True,
    }

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        current_date = datetime.now().strftime('%Y-%m-%d')
        old_dep_emp_id = self.search(cr, uid, [('employee_id', '=', vals['employee_id']),('active', '=', True)])
        if old_dep_emp_id:
            old_dep_emp_brw = self.browse(cr, uid, old_dep_emp_id)
            super(hr_department_employee, self).write(cr, uid, old_dep_emp_id, {'active':False, 'date_end':current_date}, context)
        return super(hr_department_employee, self).create(cr, uid, vals, context)

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        raise osv.except_osv(_('Acción Denegada!'), _('No se permite modificar. Debe eliminar y crear uno nuevo'))

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context=None, count=False):
        if not context:
            context={}

        args.append(('active', 'in', ['True', 'False']))
        return super(hr_department_employee, self).search(cr, uid, args, offset, limit, order, context, count)

hr_department_employee()

class hr_employee_account_bank(osv.osv):

    _name = 'hr.employee.account.bank'
    _description = 'Cuenta bancaria del empleado'

    def _get_employee(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        obj_employee = self.pool.get('hr.employee')
        for acc in self.browse(cr, uid, ids, context=context):
            employee_ids = obj_employee.search(cr, uid, [('employee_account_bank_id','=',acc.id),], context=context)
            if employee_ids:
                res[acc.id] = employee_ids[-1:][0]
            else:
                res[acc.id] = False
        return res

    _columns = {
        'bank_id': fields.many2one('res.bank', 'Banco', required=True),
        'name': fields.related('bank_id', 'name', type='string', relation='res.bank', string="Nombre"),
        'num_account': fields.char('Número de cuenta', size=64, required=True),
        'employee_ids': fields.one2many('hr.employee', 'employee_account_bank_id', 'Employes'),
        'employee_id':fields.function(_get_employee, method=True, string='Empleado', type='many2one', relation="hr.employee"),
    }

    _sql_constraints = [('unique_bank_id_num_account','unique(bank_id, num_account)', _('Ya existe un registro con el mismo Banco y Número de cuenta')),]

    def _check_len_num_account(self, cr, uid, ids, context=None):
        for account in self.browse(cr, uid, ids, context=context):
            if len(account.num_account) <> 20:
                return False
        return True

    _constraints = [(_check_len_num_account, "La longitud del número de cuenta esta errada", ['num_account'])]

hr_employee_account_bank()

class hr_employee_holidays(osv.osv):
    '''
    Modelo para guardar los periodos (fechas) en que al empleado le han tocado vacaciones
    '''
    _name = 'hr.employee.holidays'
    _description = 'Vacaciones empleados'

    _columns = {
        'name': fields.char("Nombre", size=64),
        'employee_id': fields.many2one('hr.employee', 'Employee'),
        'date_start': fields.date('Inicio disfrute', readonly=True),
        'date_end': fields.date('Fin disfrute', readonly=True),
        'corresp_to_start': fields.date('Vacaciones correspondientes a periodo inicio', readonly=True),
        'corresp_to_end': fields.date('Vacaciones correspondientes a periodo fin', readonly=True),
        'state': fields.selection([
            ('draft', 'No aprobado'),
            ('done', 'Aprobado'),
        ], 'State', select=True, readonly=True,
            help='* Cuando el periodo de disfrute es creado su estado es \'Draft\'.\
            \n* Al ser aprobado el proceso de vacaciones este periodo de disfrute pasa a estado done \'Done\'.'),
        'hr_process_id': fields.many2one('hr.process', 'Proceso', help='Indica en proceso de vacaciones en que se calcularon las vacaciones'),
        'prest_calc': fields.boolean('Prestaciones calculadas', help='Indica si se calcularon las prestaciones de este periodo de vacaciones'),
        'holidays_add_ids': fields.one2many('hr.holidays', 'employee_holidays_add_id', 'Periodo de vacaciones asignado'),
        'holidays_remove_ids': fields.one2many('hr.holidays', 'employee_holidays_remove_id', 'Periodo de vacaciones disfrutados'),
    }

    _order = "date_start desc"

    _defaults = {
        'state':'draft',
        'prest_calc':False,
    }
    
    def _check_dates(self, cr, uid, ids, context=None):
        for hol in self.browse(cr, uid, ids, context=context):
            if hol.corresp_to_start > hol.corresp_to_end:
                return False
        return True

    _constraints = [(_check_dates, "Periodo inicio debe ser menor a periodo fin.", ['corresp_to_start', 'corresp_to_end'])]
    
    _sql_constraints = [('unique_name','unique(name)', _('Name must be unique'))]
    
    def create(self, cr, uid, vals, context=None):
        if not 'menu' in context: #Se premite la creacion solo desde los procesos
            return super(hr_employee_holidays, self).create(cr, uid, vals, context)
        else:
            holidays_employee_ids = self.search(cr, uid, [('employee_id','=',vals['employee_id'])], order='corresp_to_start desc')
            if holidays_employee_ids and vals['corresp_to_start'] >= self.browse(cr, uid, holidays_employee_ids[0]).corresp_to_start:
                raise osv.except_osv(_('Warning!'),_('No se permite crear registros de vacaciones a futuro, solo historicos'))
            holidays_employee_ids = self.search(cr, uid, [('employee_id','=',vals['employee_id']), 
                '|','&',('corresp_to_start','<=',vals['corresp_to_start']),('corresp_to_end','>=',vals['corresp_to_start']),
                '&',('corresp_to_start','<=',vals['corresp_to_end']),('corresp_to_end','>=',vals['corresp_to_end'])])
            if holidays_employee_ids:
                raise osv.except_osv(_('Warning!'),_('Ya existe un registro de asignacion de vacaciones solapado con estas fechas'))
        
            return super(hr_employee_holidays, self).create(cr, uid, vals, context)
        
        
        
    def unlink(self, cr, uid, ids, context=None):
        if not 'menu' in context: #Se premite la eliminacion solo desde los procesos
            return super(hr_employee_holidays, self).unlink(cr, uid, ids, context)
        else:
            raise osv.except_osv(_('Warning!'),_('No se permite eliminar registros de vacaciones'))

    def write(self, cr, uid, ids, vals, context=None):
        if not 'menu' in context: #Se premite la modificacion solo desde los procesos
            return super(hr_employee_holidays, self).write(cr, uid, ids, vals, context)
        else:
            if not isinstance(ids, list):
                ids = [ids]
            hol_emp = self.browse(cr, uid, ids)
            for h in hol_emp:
                employee = vals['employee_id'] if 'employee_id' in vals else h.employee_id.id
                corresp_to_start = vals['corresp_to_start'] if 'corresp_to_start' in vals else h.corresp_to_start
                corresp_to_end = vals['corresp_to_end'] if 'corresp_to_end' in vals else h.corresp_to_end
                holidays_employee_ids = self.search(cr, uid, [('id', '<>',h.id),('employee_id','=',employee)], order='corresp_to_start desc')
                if holidays_employee_ids and corresp_to_start >= self.browse(cr, uid, holidays_employee_ids[0]).corresp_to_start:
                    raise osv.except_osv(_('Warning!'),_('No se permite crear registros de vacaciones a futuro, solo historicos'))
                holidays_employee_ids = self.search(cr, uid, [('employee_id','=',employee), 
                    '|','&',('corresp_to_start','<=',corresp_to_start),('corresp_to_end','>=',corresp_to_start),
                    '&',('corresp_to_start','<=',corresp_to_end),('corresp_to_end','>=',corresp_to_end)])
                if holidays_employee_ids:
                    raise osv.except_osv(_('Warning!'),_('Ya existe un registro de asignacion de vacaciones solapado con estas fechas'))
            
                super(hr_employee_holidays, self).write(cr, uid, ids, vals, context)
        
        return True
        
hr_employee_holidays()

class hr_holidays_status(osv.osv):
    _inherit = "hr.holidays.status"
    _description = "Leave Type"

    def get_days(self, cr, uid, ids, employee_id, return_false, context, hr_holiday=None):
        if hr_holiday:
            employee_holidays_add_id = hr_holiday.employee_holidays_remove_id.id
        else:
            employee_holidays_add_id = None
        cr.execute("""SELECT id, type, number_of_days, holiday_status_id FROM hr_holidays
                    WHERE employee_id = %s AND state='validate' AND holiday_status_id in %s AND (employee_holidays_add_id = %s OR
                    employee_holidays_remove_id = %s)""",
            [employee_id, tuple(ids), employee_holidays_add_id, employee_holidays_add_id])
        result = sorted(cr.dictfetchall(), key=lambda x: x['holiday_status_id'])
        grouped_lines = dict((k, [v for v in itr]) for k, itr in groupby(result, itemgetter('holiday_status_id')))
        res = {}
        for record in self.browse(cr, uid, ids, context=context):
            res[record.id] = {}
            max_leaves = leaves_taken = 0
            if not return_false:
                if record.id in grouped_lines:
                    leaves_taken = -sum([item['number_of_days'] for item in grouped_lines[record.id] if item['type'] == 'remove'])
                    max_leaves = sum([item['number_of_days'] for item in grouped_lines[record.id] if item['type'] == 'add'])
            res[record.id]['max_leaves'] = max_leaves
            res[record.id]['leaves_taken'] = leaves_taken
            res[record.id]['remaining_leaves'] = max_leaves - leaves_taken
        return res

hr_holidays_status()

class hr_holidays(osv.osv):
    _inherit = "hr.holidays"
    _description = "Leave"

    def _type_holidays_status(self, cr, uid, ids, name, args, context=None):
        result = {}
        for hol in self.browse(cr, uid, ids, context=context):
            result[hol.id] = hol.holiday_status_id.name
        return result

    _columns = {
        'employee_holidays_add_id': fields.many2one('hr.employee.holidays', 'Periodo de vacaciones'),
        'employee_holidays_remove_id': fields.many2one('hr.employee.holidays', 'Periodo de vacaciones'),
        'type_holidays_status': fields.function(_type_holidays_status, method=True, type='char', string='Holidays Status', store=True, size=64),
    }

    def onchange_holidays_status(self, cr, uid, ids, holiday_status_id):
        result = {}
        type_holidays_status = self.pool.get('hr.holidays.status').browse(cr, uid, holiday_status_id).name
        result['value'] = {
            'type_holidays_status': type_holidays_status,
            'employee_holidays_remove_id': False,
            'employee_holidays_add_id': False,
        }
        return result

    def onchange_employee(self, cr, uid, ids, employee_id):
        result = {}
        result['value'] = {
            'employee_holidays_remove_id': False,
            'employee_holidays_add_id': False,
        }
        return result
    
    def onchange_date_from(self, cr, uid, ids, date_from, date_to):
        result = {}
        
        if date_from and date_to:
            feriados = self.pool.get('hr.holidays.calendar').search(cr, uid, 
                                [('date','>=',datetime(datetime.strptime(date_from,'%Y-%m-%d %H:%M:%S').year,01,01)),
                                ('date','<=',datetime(datetime.strptime(date_to,'%Y-%m-%d %H:%M:%S').year,12,31))])
            feriados = [f.date for f in self.pool.get('hr.holidays.calendar').browse(cr, uid, feriados)]
            num_dias = self.pool.get('hr.payslip').habiles(datetime.strptime(date_from,'%Y-%m-%d %H:%M:%S'), 
                        fecha_fin=datetime.strptime(date_to,'%Y-%m-%d %H:%M:%S'), count=True, holidays=feriados)
                            
            result['value'] = {
                'number_of_days_temp': num_dias,
                'date_to': date_to,
            }
            return result
            
        result['value'] = {
            'number_of_days_temp': 0,
            'date_to': None,
        }
        return result

    def onchange_num_days(self, cr, uid, ids, date_from, num_days):
        result = {}
            
        if date_from and num_days:
            feriados = self.pool.get('hr.holidays.calendar').search(cr, uid, 
                                [('date','>=',datetime(datetime.strptime(date_from,'%Y-%m-%d %H:%M:%S').year,01,01)),
                                ('date','<=',datetime(datetime.strptime(date_from,'%Y-%m-%d %H:%M:%S').year+1,12,31))])
            feriados = [f.date for f in self.pool.get('hr.holidays.calendar').browse(cr, uid, feriados)]
            date_to = self.pool.get('hr.payslip').habiles(datetime.strptime(date_from,'%Y-%m-%d %H:%M:%S'), 
                        fecha_fin=None, days=num_days, count=False, holidays=feriados)
            if isinstance(date_to, list):
                date_to = date_to[-1]
                result['value'] = {
                    'date_to': str(date_to),
                    'number_of_days_temp': num_days,
                }
                return result
            
        result['value'] = {
            'number_of_days_temp': 0,
            'date_to': None,
        }
        return result
                
    def check_holidays(self, cr, uid, ids, context=None):
        if not context:
            context={}
        holi_status_obj = self.pool.get('hr.holidays.status')
        for record in self.browse(cr, uid, ids):
            if record.holiday_type == 'employee' and record.type == 'remove':
                if record.employee_id and not record.holiday_status_id.limit:
                    leaves_rest = holi_status_obj.get_days(cr, uid, [record.holiday_status_id.id],
                                    record.employee_id.id, None, context, record)[record.holiday_status_id.id]['remaining_leaves']
                    if leaves_rest < record.number_of_days_temp:
                        raise osv.except_osv(_('Warning!'),_('No se pueden validar los dias para el empleado %s. Dias disponibles (%s).')
                                                                % (record.employee_id.name, leaves_rest))
        return True
        
    def create(self, cr, uid, vals, context=None):
        if not context:
            context = {}
        if 'employee_holidays_add_id' in vals:
            holiday_old = self.search(cr, uid, [('name','=',vals['name']),('employee_id','=',vals['employee_id']), 
                                        ('employee_holidays_add_id','=',vals['employee_holidays_add_id'])])
            if holiday_old:
                raise osv.except_osv(_('Acción Denegada!'), _('Ya existe un registro de asignacion \
                                        de vacaciones para el periodo de vacaciones seleccionado'))
                
        if 'type' in vals and vals['type'] <> 'add':
            holiday_old = self.search(cr, uid, [('employee_id','=',vals['employee_id']), 
                                        '|','&',('date_from','<=',vals['date_from']),('date_to','>=',vals['date_from']),
                                        '&',('date_from','<=',vals['date_to']),('date_to','>=',vals['date_to'])])
            if holiday_old:
                raise osv.except_osv(_('Acción Denegada!'), _('Ya existe un registro de asignacion de \
                                        vacaciones solapado con estas fechas'))
            
            if (not 'number_of_days_temp' in vals or not vals['number_of_days_temp']) \
                and (not 'date_to' in vals or not vals['date_to'] == None):
                raise osv.except_osv(_('Warning!'),_('Debe ingresar Fecha final o Numero de dias'))
                
        return super(hr_holidays, self).create(cr, uid, vals, context)

hr_holidays()

class hr_professions(osv.osv):
    '''
    Profesiones
    '''
    _name = 'hr.professions'
    _description = 'Professions'

    _columns = {
        'code': fields.char('Código', size=64),
        'name': fields.char('Nombre', size=256),
    }

hr_professions()

class hr_type_job(osv.osv):
    '''
    Nombres de cargos
    '''
    _name = 'hr.type.job'
    _description = 'Types Job'

    _columns = {
        'name': fields.char('Nombre', size=256),
    }
    
    _sql_constraints = [('unique_name','unique(name)', _('El nombre del cargo debe ser unico'))]
    
hr_type_job()

class hr_job(osv.osv):
    _inherit = "hr.job"
    _description = "Job Description"

    def _get_name(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        job_brw = self.browse(cr, uid, ids)
        for job in job_brw:
            res[job.id] = job.type_job_id.name
        return res
        
    _columns = {
        'type_job_id': fields.many2one('hr.type.job', 'Tipo de cargo'),
        'name': fields.function(_get_name, string='Nombre', method=True, type='char', size=64, store=False),
    }
    
    _sql_constraints = [('unique_type_job_dapartment','unique(type_job_id, department_id)', _('La combinacion cargo y departamento debe ser unica'))]
    
hr_job()

class hr_employee(osv.osv):
    '''
    Employee
    '''
    _inherit = 'hr.employee'
    _description = 'Employee'

    def _get_department(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina
        department_id = self.pool.get('hr.department.employee').search(cr, uid, [('employee_id', '=', ids[0]),('active', '=', True)])
        if department_id:
            department = self.pool.get('hr.department.employee').browse(cr, uid, department_id)
            res[ids[0]] = department[0].department_id.id
        else:
            res[ids[0]] = False
        return res

    def _get_salary(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina
        parameter_obj = self.pool.get('hr.parameters')
        if not isinstance(ids, list):
            ids = [ids]
        employee_brw = self.browse(cr, uid, ids)
        for employee in employee_brw:
            attrs = False
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', employee.hr_payroll_id.id)])
            if not param_concept_salary_id:
                param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
            if not param_concept_salary_id:
                raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo concepto_sueldo')
                                                %(employee.hr_payroll_id.name))
            concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id
            
            for attr in concept_salary_brw.attribute_ids:
                if attr.employee_id.id == employee.id:
                    attrs = True
                    res[employee.id] = attr.amount
            if not attrs: 
                res[employee.id] = 0.0
        return res  

    def _get_date_start_rel_lab(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        if not isinstance(ids, list):
            ids = [ids]
        employee_brw = self.browse(cr, uid, ids)
        for employee in employee_brw:
            contracts = sorted(employee.contract_ids, key=lambda x: x.date_start, reverse=True)
            for i in range(0,len(contracts)):
                if (i < len(contracts)-1) and \
                    (datetime.strptime(contracts[i+1].date_end,"%Y-%m-%d") + timedelta(days=1) == datetime.strptime(contracts[i].date_start,"%Y-%m-%d")):
                    continue
                else:
                    res[employee.id] = contracts[i].date_start
        return res
        
    def _complete_name(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for employee in self.browse(cr, uid, ids, context=context):
            name = ''
            if employee.first_name:
                name = employee.first_name
            if employee.second_name:
                name = name + ' ' + employee.second_name
            if employee.first_surname:
                name = name +  ' ' + employee.first_surname
            if employee.second_surname:
                name = name + ' ' + employee.second_surname
            res[employee.id] = name
        return res

    def _get_latest_contract(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        obj_contract = self.pool.get('hr.contract')
        today = datetime.now() 
        for emp in self.browse(cr, uid, ids, context=context):
            contract_ids = obj_contract.search(cr, uid, [('employee_id','=',emp.id),
                                                        '|',('date_end','>=',today),('date_end','=',False)], 
                                                            order='date_start', context=context)
            if not contract_ids:
                contract_ids = obj_contract.search(cr, uid, [('employee_id','=',emp.id),
                                                                ('date_end','<=',today)], 
                                                            order='date_start', context=context)
            if contract_ids:
                res[emp.id] = contract_ids[-1:][0]
            else:
                res[emp.id] = False
        return res
        
    _columns = {
        'first_name': fields.char("Primer nombre", size=256),
        'second_name': fields.char("Segundo nombre", size=256),
        'first_surname': fields.char("Primer apellido", size=256),
        'second_surname': fields.char("Segundo apellido", size=256),
        'name': fields.function(_complete_name, method=True, string='Nombre', store=True, type="char", size=256),
        'cod_number':fields.char('Código', size=64),
        'type_employee': fields.selection([
                ('empleado', 'Empleado'),
                ('obrero', 'Obrero'),
                ('vendedor', 'Vendedor'),
            ], 'Tipo trabajador', select=True),
        'employee_family_ids': fields.one2many('hr.employee.family', 'employee_id', 'Familiares'),
        'rif': fields.char('Rif', size=32),
        'hr_sede_id': fields.many2one('hr.sede', 'Sede'),
        'hr_payroll_id': fields.many2one('hr.payroll', 'Nómina'),
        'payslip_ids': fields.one2many('hr.payslip', 'employee_id', 'Hojas de pago'),
        'attribute_ids': fields.one2many('hr.attribute', 'employee_id', 'Atributos'),
        'size_shirt': fields.char('Talla franela', size=64),
        'size_pant': fields.char('Talla pantalón', size=64),
        'size_shoes': fields.char('Talla zapatos', size=64),
        'salary': fields.function(_get_salary, string='Salario', method=True, type='float', store=False),
        'employee_account_bank_id': fields.many2one('hr.employee.account.bank', 'Cuenta bancaria'),
        'journal_id': fields.many2one('account.journal', 'Diario'),
        'check': fields.boolean('Cobra por cheque?'),
        'department_employee_ids': fields.one2many('hr.department.employee', 'employee_id', 'Movimientos trabajador'),
        'employee_holidays_ids': fields.one2many('hr.employee.holidays', 'employee_id', 'Periodos de vacaciones'),
        'tipo_cuenta_abono': fields.selection([('cont_emp', 'Contabilidad de la Empresa'),
                            ('fidei_ind', 'Fideicomiso individual'),('fondo_prest', 'Fondo de prestaciones de antiguedad')],
                            "Tipo cuenta para abono", help='Campo usado para establecer el tipo de abono que se le hace al empleado en cuanto \
                                                            a sus prestaciones sociales'),
        'cod_bank': fields.many2one('res.bank', 'Banco', help='Código del banco para el abono de prestaciones'),
        'cod_fdo_fidei': fields.char('Código fondo fideicomiso', size=64),
        'type_bono_alimen': fields.selection([
            ('tarjeta', 'Tarjeta'),
            ('tickets', 'Tickets'),
            ], 'Tipo pago bono alimentacion', select=True),
        'cod_tar_bono_alimen': fields.char("Número tarjeta bono alimentación", size=64),
        'home_address': fields.char("Dirección de habitación", size=256),
        'academy_grade': fields.selection([
            ('primaria', 'Primaria'),
            ('bachiller', 'Bachiller'),
            ('universitario', 'Universitario'),
            ('maestria', 'Maestría'),
            ('especializacion', 'Especialización'),
            ('doctorado', 'Doctorado'),
            ], 'Grado académico', select=True),
        'profession_id': fields.many2one('hr.professions', 'Profesión'),
        'parent_id': fields.related('job_id', 'department_id', 'manager_id', relation='hr.employee', string='Manager', type='many2one', store=True, select=True, readonly=True, help="It is linked with manager of Department"),
        'department_id':fields.related('job_id', 'department_id', relation='hr.department', string="Departamento", type='many2one', store=True, select=True, readonly=True),
        'state': fields.selection([
                ('vacaciones', 'Vacaciones'),
                ('reposo', 'Reposo'),
                ('egresado', 'Egresado'),
                ('activo', 'Activo'),
                ('borrador', 'Borrador'),
            ], 'Estado', select=True),
        'contract_id':fields.function(_get_latest_contract, method=True, string='Contract', type='many2one', relation="hr.contract", help='Latest contract of the employee'),
        'start_rel_lab': fields.function(_get_date_start_rel_lab, string='Inicio relacion laboral', method=True, type='date', store=False),
        'no_remunerado':fields.boolean('No remunerado', help="Campo que se puede tildar en el caso de los trabajadores en estado Reposo que no deben \
                                                            entrar en los procesos de pago de nomina"),
        #~ Los Campos a continuacion se encuentran en el modulo hr_payroll_structure_account del 6.0
        'property_bank_account': fields.property(
            'account.account',
            type='many2one',
            relation='account.account',
            string="Bank Account",
            method=True,
            domain="[('type', '=', 'liquidity')]",
            view_load=True,
            help="Select Bank Account from where Salary Expense will be Paid, to be used for payslip verification."),
        'salary_account':fields.property(
            'account.account',
            type='many2one',
            relation='account.account',
            string="Salary Account",
            method=True,
            domain="[('type', '=', 'other')]",
            view_load=True,
            help="Expense account when Salary Expense will be recorded"),
        'employee_account':fields.property(
            'account.account',
            type='many2one',
            relation='account.account',
            string="Employee Account",
            method=True,
            domain="[('type', '=', 'other')]",
            view_load=True,
            help="Employee Payable Account"),
        'analytic_account':fields.property(
            'account.analytic.account',
            type='many2one',
            relation='account.analytic.account',
            string="Analytic Account",
            method=True,
            view_load=True,
            help="Analytic Account for Salary Analysis"),
    }

    _defaults = {
        'name':'-',
        'state': 'borrador',
    }

    _sql_constraints = [('unique_cod_number','unique(cod_number)', _('El codigo del trabajador debe ser unico')),
                        ('unique_identification_id','unique(identification_id)', _('El numero de cedula del trabajador debe ser unico')),
                        ('unique_employee_account_bank','unique(employee_account_bank_id)', ('La cuenta bancaria seleccionada ya esta asignada a otro empleado'))]

    def set_to_active(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        today = datetime.now().strftime('%Y-%m-%d')
        for id in ids:
            employee_brw = self.browse(cr, uid, id, context)
            if not employee_brw.contract_id or not employee_brw.salary:
                raise osv.except_osv(_('Error!'),_('El trabajador debe tener un contrato activo y su salario antes de ser aprobado'))
            date_end = employee_brw.contract_id.date_end >= today if employee_brw.contract_id.date_end else True
            if employee_brw.contract_id.date_start > today or not date_end:
                raise osv.except_osv(_('Error!'),_('El trabajador no tiene un contrato activo'))
            self.write(cr, uid, id, {'state':'activo'})
        return True
        
    def set_to_active_to_rest(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        today = datetime.now().strftime('%Y-%m-%d')
        for id in ids:
            employee_brw = self.browse(cr, uid, id, context)
            if employee_brw.state == 'activo':
                self.write(cr, uid, id, {'state':'reposo'})
            if employee_brw.state == 'reposo':
                self.write(cr, uid, id, {'state':'activo'})
        return True
        
    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        employee_id = super(hr_employee, self).create(cr, uid, vals, context)
        if 'job_id' in vals or 'hr_sede_id' in vals or 'hr_payroll_id' in vals:
            if employee_id:
                job_brw = self.pool.get('hr.job').browse(cr, uid, vals['job_id'])
                sede_brw = self.pool.get('hr.sede').browse(cr,uid,vals['hr_sede_id'])
                payroll_brw =  self.pool.get('hr.payroll').browse(cr, uid, vals['hr_payroll_id'])
                vals_depart_emp = {
                    'employee_id': employee_id,
                    'date_start': datetime.now().strftime('%Y-%m-%d'),
                    'department_id': job_brw.department_id.name,
                    'sede': sede_brw.name,
                    'job': job_brw.name,
                    'payroll':payroll_brw.name,
                    'id_payroll':payroll_brw.id,
                }
                depart_emp_id = self.pool.get('hr.department.employee').create(cr, uid, vals_depart_emp, context)
        return employee_id

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = [ids]
        for id in ids:
            
            prop_liq_aprob = self.pool.get('hr.linea.liquidacion').search(cr, uid, [('employee_id','=',id),('state','=','aprobado')])
            if prop_liq_aprob:
                raise osv.except_osv(_('ERROR!'),_('El empleado tiene una Liquidación aprobada, no se puede modificar!'))
            
            if 'active' in vals and not vals['active']:
                employee = self.browse(cr, uid, id)
                for attr in employee.attribute_ids:
                    self.pool.get('hr.attribute').write(cr, uid, attr.id, {'active' : False})
                    
            if super(hr_employee, self).write(cr, uid, ids, vals, context):
                employee = self.browse(cr, uid, id)
                if 'job_id' in vals or 'hr_sede_id' in vals or 'hr_payroll_id' in vals:
                    vals_depart_emp = {
                       'employee_id': id,
                       'date_start': datetime.now().strftime('%Y-%m-%d'),
                       'department_id': employee.job_id.department_id.name,
                       'sede': employee.hr_sede_id.name,
                       'job': employee.job_id.name,
                       'payroll':employee.hr_payroll_id.name,
                       'id_payroll':employee.hr_payroll_id.id,
                    }
                            
                    self.pool.get('hr.department.employee').create(cr, uid, vals_depart_emp, context)
            else:
                return False
        
        return True

    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            employee = self.browse(cr, uid, id)
            if not employee.state == 'borrador':
                raise osv.except_osv(_('Error!'),_('El trabajador %s no esta en estado Borrador!')%(employee.name))
            old_dep_emp_id = self.pool.get('hr.department.employee').search(cr, uid, [('employee_id', '=', id)])
            if old_dep_emp_id:
                self.pool.get('hr.department.employee').unlink(cr, uid, old_dep_emp_id, context)
            contract_ids = self.pool.get('hr.contract').search(cr, uid, [('employee_id', '=', id)])
            if contract_ids:
                self.pool.get('hr.contract').unlink(cr, uid, contract_ids, context)
            shift_week_ids = self.pool.get('hr.shift.week').search(cr, uid, [('employee_id', '=', id)])
            if shift_week_ids:
                self.pool.get('hr.shift.week').unlink(cr, uid, shift_week_ids, context)
            unl = super(hr_employee, self).unlink(cr, uid, id, context)
            if not unl:
                return False
        return True

    def _get(self, employee, codes, period):
        res = {}
        for c in codes:
            res[c] = 0.0
        for attr in employee.attribute_ids:
            if attr.code in codes:
                if attr.concept_id.date_end: #Predominan las fechas en el concepto sobre las fechas en los atributos
                    date_end_concept = attr.concept_id.date_end >= period.date_start
                    if attr.concept_id.date_start <= period.date_end and date_end_concept:
                        if attr.type == 'date':
                            res[attr.code] = attr.date_value
                        elif not attr.code in res:
                            res[attr.code] = attr.amount
                        else:
                            res[attr.code] = res[attr.code] + attr.amount
                else:
                    date_end_attr = attr.date_end >= period.date_start if attr.date_end else True
                    if attr.date_start <= period.date_end and date_end_attr:
                        if attr.type == 'date':
                            res[attr.code] = attr.date_value
                        elif not attr.code in res:
                            res[attr.code] = attr.amount
                        else:
                            res[attr.code] = res[attr.code] + attr.amount
        return res

    def on_change_check(self, cr, uid, ids, check):
        result = {'employee_account_bank_id': False}
        return {'value': result}

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        if not args:
            args=[]
        if name:
            ids = self.search(cr, user, [('cod_number','=',name)]+ args, limit=limit, context=context)
            if not len(ids):
                ids = self.search(cr, user, ['|',('name',operator,name),('cod_number',operator,name)] + args, limit=limit, context=context)
            if not len(ids):
                ptrn=re.compile('(\[(.*?)\])')
                res = ptrn.search(name)
                if res:
                    ids = self.search(cr, user, [('cod_number','=', res.group(2))] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, user, args, limit=limit, context=context)
        result = self.name_get(cr, user, ids, context=context)
        return result

hr_employee()

class hr_guarderias(osv.osv):
    '''
        Clase para definir los partner que son guarderías
    '''
    _description = "Guarderias"
    _name = 'hr.guarderias'

    _columns = {
        'partner_id': fields.many2one('res.partner', 'Partner'),
        'name': fields.related('partner_id', 'name', type='string', relation='res.partner', string="Nombre"),
    }

    _sql_constraints = [('unique_partner','unique(partner_id)', _('El partner debe ser unico'))]

hr_guarderias()

class hr_employee_family(osv.osv):
    '''
    Employee Family
    '''
    _name = 'hr.employee.family'
    _description = 'Employee Family'

    def _get_age(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        employee_fam_obj = self.pool.get('hr.employee.family')
        current_date = datetime.now()
        for em in self.browse(cr, uid, ids, context=context):
            edad = current_date.year - datetime.strptime(em.birthday,"%Y-%m-%d").year
            res[em.id] = edad
        return res

    def search_age(self, cr, uid, obj, name, args, context=None):
        current_date = datetime.now()
        edad = int(args[0][2])
        oper = '<=' if args[0][1] == '>=' else '>='
        date_birth = datetime(current_date.year-edad, current_date.month, current_date.day)
        ids = self.search(cr, uid, [('birthday',oper,date_birth)])
        return [('id','in',ids)]

    _columns = {
        'name':fields.char('Nombre(s) y Apellido(s)', size=256, required=True),
        'identification_id':fields.char('Cédula', size=32),
        'gender': fields.selection([('male', 'Masculino'),('female', 'Femenino')], 'Sexo'),
        'relationship':fields.selection(
                    [('hijo_a', 'Hijo(a)'),
                    ('padre/madre', 'Padre/Madre'),
                    ('esposo_a', 'Esposo(a)'),
                    ('concubino_a', 'Concubino(a)'),], 'Parentesco'),
        'birthday': fields.date('Fecha de Nacimiento'),
        'employee_id': fields.many2one('hr.employee', 'Empleado'),
        'hcm':fields.boolean('HCM'),
        'seg_vida':fields.boolean('Seguro de Vida'),
        'size_shirt': fields.char('Talla franela', size=64),
        'size_pant': fields.char('Talla pantalón', size=64),
        'size_shoes': fields.char('Talla zapatos', size=64),
        'guarderia_id': fields.many2one('hr.guarderias', 'Guarderia'),
        'guarderia_amount': fields.integer('Monto guarderia', digits=(16,2)),
        'edad': fields.function(_get_age, string='Edad', fnct_search=search_age, method=True, type='integer', store=False),

    }
    
    _sql_constraints = [('check_birthday',"CHECK(birthday > '1900-01-01')",'La fecha de nacimiento debe ser mayor a 01-01-1900')
                    ]
    
hr_employee_family()

class hr_concept_category(osv.osv):
    """
    HR Concept Category
    """

    _name = 'hr.concept.category'
    _description = 'Concept Category'

    _columns = {
        'name':fields.char('Name', size=64, required=True, readonly=False),
        'code':fields.char('Code', size=64, required=True, readonly=False),
        'concept_ids':fields.one2many('hr.concept', 'category_id', 'Conceptos'),
        'note': fields.text('Description'),
    }

hr_concept_category()

class hr_concept(osv.osv):
    '''
    Payroll Concepts
    '''
    _name = 'hr.concept'
    _description = 'Payroll Concepts'

    _columns = {
        'code':fields.char('Código', size=64, required=True),
        'name':fields.char('Descripcion', size=256, required=True),
        'date_start': fields.date('Valido desde', required=True),
        'date_end': fields.date('Valido hasta'),
        'attribute_ids': fields.one2many('hr.attribute', 'concept_id', 'Conceptos'),
        'salary_rule_ids':fields.one2many('hr.salary.rule', 'concept_id', 'Reglas'),
        'active':fields.boolean('Activo'),
        'type_calc': fields.selection([(_('allocation'), 'Asignacion'),(_('deduction'), 'Deduccion')], "Tipo de calculo", required=True, size=64),
        'type': fields.selection([('fijo', 'Fijo'),('variable', 'Variable'),('aporte_patronal', 'Aporte patronal')], "Tipo de concepto", required=True, size=64),
        'type_amount': fields.selection([(_('hours'), 'Horas'),(_('days'), 'Dias')], "Tipo de valor", help="Indica el tipo de valor que se esta \
                                                                                usando para el cálculo del concepto (en horas o dias)", size=64),
        'category_id':fields.many2one('hr.concept.category', 'Categoria'),
        'concept_loan_ids': fields.one2many('hr.concept.loan', 'concept_id', 'Conceptos de prestamos'),
    }

    _defaults = {
        'active': True,
    }

    _sql_constraints = [('unique_code','unique(code)', _('Code must be unique'))]

    def _check_dates(self, cr, uid, ids, context=None):
        for concept in self.browse(cr, uid, ids, context=context):
            if concept.date_end and concept.date_start > concept.date_end or False:
                return False
        return True

    _constraints = [(_check_dates, "'Valido desde' must be before 'Valido hasta'.", ['date_start', 'date_end'])]

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        if not args:
            args=[]
        if name:
            ids = self.search(cr, user, [('code','=',name)]+ args, limit=limit, context=context)
            if not len(ids):
                ids = self.search(cr, user, ['|',('name',operator,name),('code',operator,name)] + args, limit=limit, context=context)
            if not len(ids):
                ptrn=re.compile('(\[(.*?)\])')
                res = ptrn.search(name)
                if res:
                    ids = self.search(cr, user, [('code','=', res.group(2))] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, user, args, limit=limit, context=context)
        result = self.name_get(cr, user, ids, context=context)
        return result
        
    def get_cal_concept(self, params, payslip, code, tipo, num_periods, mes=None):
        '''
        Devuelve total acumulado para el concepto con codigo code y un diccionario
        con detallado por periodo para dicho concepto.
        code: Codigo del concepto a consultar
        mes: Se pasa solo en el proceso de pago de utilidades
        tipo: Tipo de proceso del que se desea consultar el concepto
        num_periods: numero de periodos hacia atras que se desean consultar
        '''
        total = 0.0
        if payslip.hr_process_id.type == 'utilidades':
            year = datetime.strptime(payslip.hr_process_id.date_end,"%Y-%m-%d").year
            start = datetime(year, 01, 01)
            end = datetime(year, 12, 31)
            date_start = datetime.strptime(payslip.hr_process_id.date_start,"%Y-%m-%d")
            date_end = datetime.strptime(payslip.hr_process_id.date_end,"%Y-%m-%d")

            if date_start == start and date_end == end:
                #Si el calculo que se esta realizando es de utilidades del año completo, los conceptos a consultar son todos los del
                #calendario de pago pasado
                year_old = self.pool.get('hr.year').search(params[0], params[1], [('year', '=', year)])
                if not year_old:
                    periods = []

                year_old = year_old[0]

                calendar_old = self.pool.get('hr.payment.calendar').search(params[0], params[1], [('type', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.type),
                                                ('hr_year_id', '=', year_old),('active', '=', False)])

                calendar_old_brw = self.pool.get('hr.payment.calendar').browse(params[0], params[1], calendar_old)

                if not calendar_old_brw:
                    raise osv.except_osv(_('Error!'),_('No existe un calendario anterior para obtener sueldos promedios'))

                periods = calendar_old_brw[0].hr_period_ids
                
            else:
                periods_employee_brw = payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids
                periods_process_brw = payslip.hr_process_id.hr_period_ids
                period_brw = list(set(periods_employee_brw) & set(periods_process_brw))[0] #Corresponde al periodo que se esta ejecutando en el proceso actual

                if period_brw.period_in_year > num_periods:
                    periods = periods_employee_brw[period_brw.period_in_year-num_periods-period_brw.period_in_month:period_brw.period_in_year-period_brw.period_in_month]
                else:
                    year_old = self.pool.get('hr.year').search(params[0], params[1], [('year', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_year_id.year-1)])
                    if not year_old:
                        raise osv.except_osv(_('Error!'),_('Se esta tratando de consultar un periodo del año pasado el cual no existe'))

                    year_old = year_old[0]

                    calendar_old = self.pool.get('hr.payment.calendar').search(params[0], params[1], [('type', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.type),
                                                    ('hr_year_id', '=', year_old),('active', '=', False)])
                    calendar_old_brw = self.pool.get('hr.payment.calendar').browse(params[0], params[1], calendar_old)
                    if not calendar_old_brw:
                        raise osv.except_osv(_('Error!'),_('No existe un calendario anterior para obtener sueldos promedios'))

                    periods = calendar_old_brw[0].hr_period_ids[period_brw.period_in_year-num_periods-period_brw.period_in_month:] + \
                            periods_employee_brw[0:period_brw.period_in_year-period_brw.period_in_month]

        elif payslip.hr_process_id.type == 'prestaciones':
            periods_employee_brw = payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids
            periods_process_brw = payslip.hr_process_id.hr_period_ids
            period_brw = list(set(periods_employee_brw) & set(periods_process_brw))[0] #Corresponde al periodo que se esta ejecutando en el proceso actual
            if period_brw.period_in_year >= num_periods:
                periods = periods_employee_brw[period_brw.period_in_year-num_periods:period_brw.period_in_year]

            else:
                year_old = self.pool.get('hr.year').search(params[0], params[1], [('year', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_year_id.year-1)])

                if not year_old:
                    periods = periods_employee_brw[0:period_brw.period_in_year]
                else:
                    year_old = year_old[0]

                    calendar_old = self.pool.get('hr.payment.calendar').search(params[0], params[1], [('type', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.type),
                                                    ('hr_year_id', '=', year_old),('active', '=', False)])
                    calendar_old_brw = self.pool.get('hr.payment.calendar').browse(params[0], params[1], calendar_old)
                    if not calendar_old_brw:
                        raise osv.except_osv(_('Error!'),_('No existe un calendario anterior para obtener sueldos promedios'))

                    periods = calendar_old_brw[0].hr_period_ids[period_brw.period_in_year-num_periods:] + periods_employee_brw[0:period_brw.period_in_year]

        elif mes:
            periods = mes.hr_period_ids

        elif payslip.hr_process_id.type == 'vacaciones':
            periods_employee_brw = payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids
            periods_process_brw = payslip.hr_process_id.hr_period_ids
            period_brw = list(set(periods_employee_brw) & set(periods_process_brw))[0] #Corresponde al periodo que se esta ejecutando en el proceso actual

            if period_brw.period_in_year > num_periods:
                periods = periods_employee_brw[period_brw.period_in_year-num_periods-1:period_brw.period_in_year-1]
            else:
                year_old = self.pool.get('hr.year').search(params[0], params[1], [('year', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_year_id.year-1)])
                if not year_old:
                    periods = periods_employee_brw[0:period_brw.period_in_year]
                else:
                    year_old = year_old[0]

                    calendar_old = self.pool.get('hr.payment.calendar').search(params[0], params[1], [('type', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.type),
                                                    ('hr_year_id', '=', year_old),('active', '=', False)])
                    calendar_old_brw = self.pool.get('hr.payment.calendar').browse(params[0], params[1], calendar_old)
                    if not calendar_old_brw:
                        raise osv.except_osv(_('Error!'),_('No existe un calendario anterior para obtener sueldos promedios'))

                    periods = calendar_old_brw[0].hr_period_ids[period_brw.period_in_year-num_periods-1:] + periods_employee_brw[0:period_brw.period_in_year-1]

        else:
            periods_employee_brw = payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids
            periods_process_brw = payslip.hr_process_id.hr_period_ids
            period_brw = list(set(periods_employee_brw) & set(periods_process_brw))[0] #Corresponde al periodo que se esta ejecutando en el proceso actual

            if period_brw.period_in_year > num_periods:
                periods = periods_employee_brw[period_brw.period_in_year-num_periods-1:period_brw.period_in_year-1]
            else:
                year_old = self.pool.get('hr.year').search(params[0], params[1], [('year', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_year_id.year-1)])
                if not year_old:
                    periods = periods_employee_brw[0:period_brw.period_in_year]
                else:
                    year_old = year_old[0]

                    calendar_old = self.pool.get('hr.payment.calendar').search(params[0], params[1], [('type', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.type),
                                                    ('hr_year_id', '=', year_old),('active', '=', False)])
                    calendar_old_brw = self.pool.get('hr.payment.calendar').browse(params[0], params[1], calendar_old)
                    if not calendar_old_brw:
                        raise osv.except_osv(_('Error!'),_('No existe un calendario anterior para obtener sueldos promedios'))

                    periods = calendar_old_brw[0].hr_period_ids[period_brw.period_in_year-num_periods-1:] + periods_employee_brw[0:period_brw.period_in_year-1]

        for period in periods:
            process_ids = self.pool.get('hr.process').search(params[0], params[1], [('hr_period_ids','=', period.id)])
            process = self.pool.get('hr.process').browse(params[0], params[1], process_ids)
            for proc in process:
                if proc.type in tipo and proc.state == 'done':
                    payslip_period_id =  self.pool.get('hr.payslip').search(params[0], params[1], [('employee_id','=', payslip.employee_id.id),('hr_process_id','=', proc.id)])
                    if payslip_period_id:
                        payslip_line_ids = self.pool.get('hr.payslip.line').search(params[0], params[1], [('slip_id','=', payslip_period_id)])
                        for line in self.pool.get('hr.payslip.line').browse(params[0], params[1], payslip_line_ids):
                            if line.code in code:
                                total = (total + line.total) if line.type_calc == 'allocation' else (total - line.total)
        return total

    def get_cal_concept_anticipos(self, params, payslip, code, tipo, num_periods):
        '''
        Devuelve total acumulado para el concepto con codigo code y un diccionario
        con detallado por periodo para dicho concepto.
        code: Codigo del concepto a consultar
        tipo: Tipo de proceso del que se desea consultar el concepto
        num_periods: numero de periodos hacia atras que se desean consultar
        '''
        total = 0.0
        fecha = False

        if payslip.hr_process_id.type == 'prestaciones':
            periods_employee_brw = payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_period_ids
            periods_process_brw = payslip.hr_process_id.hr_period_ids
            period_brw = list(set(periods_employee_brw) & set(periods_process_brw))[0] #Corresponde al periodo que se esta ejecutando en el proceso actual

            if period_brw.period_in_year >= num_periods:
                periods = periods_employee_brw[period_brw.period_in_year-num_periods:period_brw.period_in_year]

            else:
                year_old = self.pool.get('hr.year').search(params[0], params[1], [('year', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.hr_year_id.year-1)])

                if not year_old:
                    periods = periods_employee_brw[period_brw.period_in_year-num_periods:period_brw.period_in_year]
                else:
                    year_old = year_old[0]

                    calendar_old = self.pool.get('hr.payment.calendar').search(params[0], params[1], [('type', '=', payslip.employee_id.hr_payroll_id.hr_payment_calendar_id.type),
                                                    ('hr_year_id', '=', year_old),('active', '=', False)])
                    calendar_old_brw = self.pool.get('hr.payment.calendar').browse(params[0], params[1], calendar_old)
                    if not calendar_old_brw:
                        raise osv.except_osv(_('Error!'),_('No existe un calendario anterior para obtener sueldos promedios'))

                    periods = calendar_old_brw[0].hr_period_ids[period_brw.period_in_year-num_periods:] + periods_employee_brw[0:period_brw.period_in_year]

        for period in periods:
            process_ids = self.pool.get('hr.process').search(params[0], params[1], [('hr_period_ids','=', period.id)])
            process = self.pool.get('hr.process').browse(params[0], params[1], process_ids)
            for proc in process:
                if proc.type in tipo and proc.state == 'done':
                    payslip_period_id =  self.pool.get('hr.payslip').search(params[0], params[1], [('employee_id','=', payslip.employee_id.id),('hr_process_id','=', proc.id)])
                    if payslip_period_id:
                        payslip_line_ids = self.pool.get('hr.payslip.line').search(params[0], params[1], [('slip_id','=', payslip_period_id)])
                        for line in self.pool.get('hr.payslip.line').browse(params[0], params[1], payslip_line_ids):
                            if line.code in code:
                                total = (total + line.total) if line.type_calc == 'allocation' else (total - line.total)
                                fecha = line.process_id.date_done
        return [total,fecha]

hr_concept()

class hr_payroll_structure(osv.osv):
    """
    Salary structure used to defined
    - Basic
    - Allowances
    - Deductions
    """

    _inherit = 'hr.payroll.structure'
    _description = 'Estructura'

    def _select_types(self, cr, uid, context=None):
        if context is None:
            context = {}
        res = [('nomina', 'Nómina'),
                ('vacaciones', 'Vacaciones'),
                ('utilidades', 'Utilidades'),
                ('prestaciones', 'Prestaciones sociales'),
                ('guarderias', 'Pago de guarderías'),
                ('anticipos', 'Pago de anticipos'),
                ('bono_alimentacion', 'Pago bono alimentación'),
                ('pago_especial', 'Pagos especiales'),
                ('liquidacion', 'Liquidaciones'),
            ]

        parameter_obj = self.pool.get('hr.parameters')
        param_type_process_ids = parameter_obj.search(cr, uid, [('type', '=', 'tipo_proceso')])
        for param_type_process in parameter_obj.browse(cr, uid, param_type_process_ids):
            res.append((param_type_process.type_process, param_type_process.type_process))

        return res

    _columns = {
        'concept_ids': fields.many2many('hr.concept', 'hr_payroll_structure_concept_rel', 'payroll_structure_id', 'concept_id', 'Conceptos'),
        'hr_payroll_id': fields.many2one('hr.payroll', 'Nomina'),
        'type': fields.selection(_select_types, "Tipo estructura", required=True, size=64),
    }

    def get_all_attributes(self, cr, uid, structure_ids, employee_id, period_id, context=None):

        """
        @param structure_ids: list of structure
        @return: returns a list of tuple (id, sequence) of attribute that are maybe to apply
        """
        all_attributes = []
        for struct in self.browse(cr, uid, structure_ids, context=context):
            for concept in struct.concept_ids:
                if concept.type == 'fijo':
                    period_brw = self.pool.get('hr.period').browse(cr, uid, period_id)
                    date_end = concept.date_end >= period_brw.date_start if concept.date_end else True
                    if concept.date_start <= period_brw.date_end and date_end:
                        for attribute in concept.attribute_ids:
                            if employee_id.id == attribute.employee_id.id:
                                if attribute.concept_id.date_end: #Predominan las fechas en el concepto sobre las fechas en los atributos
                                    date_end_concept = attribute.concept_id.date_end >= period_brw.date_start
                                    if attribute.concept_id.date_start <= period_brw.date_end and date_end_concept:
                                        all_attributes.append(attribute)
                                else:
                                    date_end_attr = attribute.date_end >= period_brw.date_start if attribute.date_end else True
                                    if attribute.date_start <= period_brw.date_end and date_end_attr:
                                        all_attributes.append(attribute)
        return all_attributes

hr_payroll_structure()

class hr_attribute(osv.osv):
    '''
    Payroll Attributes
    '''
    _name = 'hr.attribute'
    _description = 'Payroll Attributes'

    _columns = {
        'name': fields.char('Nombre', size=256, required=True),
        'code': fields.related('concept_id', 'code', type='string', relation='hr.concept', string="Código"),
        'employee_id': fields.many2one('hr.employee', 'Empleado'),
        'type': fields.selection([('amount', 'Cantidad'),('date', 'Fecha')], "Tipo atributo", required=True),
        'amount': fields.float('Monto', digits_compute=dp.get_precision('Payroll')),
        'date_value': fields.date('Fecha atributo'),
        'date_start': fields.date('Valido desde'),
        'date_end': fields.date('Valido hasta'),
        'concept_id': fields.many2one('hr.concept', 'Concepto', required=True),
        'appears_on_payslip': fields.boolean('Appears on Payslip', help="Used for the display of rule on payslip"),
        'save_on_database': fields.boolean('Save result in database', help="Used for save calculation of attribute on database"),
        'notes': fields.text('Notas'),
        'active': fields.boolean('Activo'),
        'loan_ids': fields.one2many('hr.loan', 'created_attr_id', 'Préstamos'),
        'allocation_loan_ids': fields.one2many('hr.loan', 'attr_allocation_id', 'Préstamos'),
        
    }

    _defaults = {
        'active': True,
        'save_on_database': True,
        'type': 'amount',
    }

    def _check_concept_attribute_or_rule(self, cr, uid, ids, context=None):
        for attribute in self.browse(cr, uid, ids, context=context):
            if attribute.concept_id.salary_rule_ids:
                return False
        return True

    _constraints = [(_check_concept_attribute_or_rule, "El Concepto ya tiene asociada una regla", ['concept_id']),
                    ]

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        current_date = datetime.now().strftime('%Y-%m-%d')
        employee = self.pool.get('hr.employee').browse(cr, uid, vals['employee_id'])
        
        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina-----------------------------
        parameter_obj = self.pool.get('hr.parameters')
        param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo concepto_sueldo')
                                                %(employee.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id

        if vals['concept_id'] == concept_salary_brw.id:
            # Si el atributo a guardar es sueldo se debe desactivar el atributo anterior
            old_attribute_id = self.search(cr, uid, [('employee_id', '=', vals['employee_id']),
                                                    ('concept_id', '=', vals['concept_id']),('active', '=', True)])
            if old_attribute_id:
                old_attribute_brw = self.browse(cr, uid, old_attribute_id)
                if super(hr_attribute, self).write(cr, uid, old_attribute_id, {'active':False, 'date_end':current_date}, context):
                    message = 'El atributo %s del empleado %s de monto %s fue pasado a inactivo'\
                                %(old_attribute_brw[0].name,
                                self.pool.get('hr.employee').name_get(cr, uid, [old_attribute_brw[0].employee_id.id])[0][1],
                                old_attribute_brw[0].amount)
                    self.log(cr, uid, old_attribute_id[0], message, context=context)
            
            if not 'wizard_incremento' in context:
                param_correos_inc_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'correos_inc_salary')])
                if param_correos_inc_salary_id:
                    correos_inc_salary_brw = parameter_obj.browse(cr, uid, param_correos_inc_salary_id)[0].correos
                    subject = u'Incrementos salariales'
                    message_email = u'Se le realizó un cambio de salario al trabajador %s (código %s). <br/>'\
                            %(employee.name,employee.cod_number)
                    if old_attribute_id:
                        message_email = message_email + 'Su salario cambio de %s a %s. <br/>'%(old_attribute_brw[0].amount,vals['amount'])
                    else:
                        message_email = message_email + 'Su salario asignado es de %s. <br/>'%(vals['amount'])
                    self.pool.get('hr.contract').send_email_notification(cr, uid, subject, message_email, correos_inc_salary_brw, context)
        
        return super(hr_attribute, self).create(cr, uid, vals, context)

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        if type(ids) == list:
            ids = ids[0]
            
        attribute = self.browse(cr, uid, ids, context)
        
        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina
        parameter_obj = self.pool.get('hr.parameters')
        param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', attribute.employee_id.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo Concepto Salario')
                                                %(employee.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id
        
        if ('employee_id' in vals and vals['employee_id'] <> attribute.employee_id.id) or ('concept_id' in vals and vals['concept_id'] <> attribute.concept_id.id):
            raise osv.except_osv(_('ERROR!'), _('No se permite modificar empleado o concepto a los atributos'))
        if ('amount' in vals) and (attribute.employee_id.state <> 'borrador') and (attribute.concept_id.id == concept_salary_brw.id):
            raise osv.except_osv(_('ERROR!'), _('No se permite modificar los atributos de Concepto salario'))
        if ('active' in vals and vals['active']) and attribute.concept_id.id == concept_salary_brw.id and attribute.employee_id.salary and \
                attribute.employee_id.state <> 'borrador':
            raise osv.except_osv(_('ERROR!'), _('El trabajador ya tiene un salario activo'))
         
        current_date = datetime.now().strftime('%Y-%m-%d')
        sentencia = []
        if 'employee_id' in vals:
            sentencia = [('employee_id', '=', vals['employee_id'])]
        else:
            sentencia = [('employee_id', '=', attribute.employee_id.id)]
        if 'concept_id' in vals:
            sentencia += [('concept_id', '=', vals['concept_id'])]
        else:
            sentencia += [('concept_id', '=', attribute.concept_id.id)]
        if 'date_end' in vals:
            sentencia += [('date_end', '=', vals['date_end'])]
        elif attribute.date_end:
            sentencia += [('date_end', '=', attribute.date_end)]
        if 'active' in vals:
            sentencia += [('active', '=', vals['active'])]
        else:
            sentencia += [('active', '=', attribute.active)]
        old_attribute_id = self.search(cr, uid, [('id', '<>', attribute.id),('amount', '=', attribute.amount),('date_start','=',attribute.date_start),
                                                ('name','=',attribute.name)]+sentencia)
        #~ if old_attribute_id:
            #~ raise osv.except_osv(_('Acción Denegada!'), _('No se permite attributos duplicados. \
                                                #~ Ya existe un atributo con las mismas caracterìsticas'))
        #else:
        if 'active' in vals:
            if not 'date_end' in vals and not attribute.date_end:
                vals['date_end'] = current_date
        return super(hr_attribute, self).write(cr, uid, ids, vals, context)
        #~ return False

    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        attribute_brw = self.browse(cr, uid, ids, context)
        for attribute in attribute_brw:
            if attribute.active:
                raise osv.except_osv(_('Acción Denegada!'), _('No se permite eliminar atributos en estado activo'))
            else:
                return super(hr_attribute, self).unlink(cr, uid, ids, context)
        return False


hr_attribute()

class hr_salary_rule(osv.osv):

    _inherit = 'hr.salary.rule'
    _description = 'Salary Rules'

    _columns = {
        'hr_sede_id': fields.many2one('hr.sede', 'Sede'),
        'concept_id': fields.many2one('hr.concept', 'Concepto'),
        'code': fields.related('concept_id', 'code', type='string', relation='hr.concept', string="Código"),
        'hr_payroll_ids': fields.many2many('hr.payroll','hr_rule_payroll_rel', 'salary_rule_id', 'payroll_id', 'Nóminas a las que aplica'),
        'save_on_database': fields.boolean('Save result in database', help="Used for save calculation of rule on database"),
    }

    _defaults = {
        'save_on_database': True,
    }

    def _check_concept_attribute_or_rule(self, cr, uid, ids, context=None):
        for hr_salary_rule in self.browse(cr, uid, ids, context=context):
            if hr_salary_rule.concept_id.attribute_ids:
                return False
        return True

    _constraints = [(_check_concept_attribute_or_rule, "El Concepto ya tiene asociado un atributo", ['concept_id'])]

hr_salary_rule()

class hr_payslip(osv.osv):
    '''
    Pay Slip
    '''
    _inherit = 'hr.payslip'
    _description = 'Pay Slip'

    def _compute_sum(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        payslip_brw = self.browse(cr, uid, ids)

        for slip in payslip_brw:
            total = 0.0
            asignaciones = 0.0
            deducciones = 0.0
            for line in slip.line_ids:
                if line.appears_on_payslip:
                    if (line.salary_rule_id and line.salary_rule_id.concept_id.type_calc == 'allocation') or \
                        (line.attribute_id and line.attribute_id.concept_id.type_calc == 'allocation'):
                        asignaciones += (line.quantity * line.amount)
                    else:
                        deducciones += (line.quantity * line.amount)

            total = round(asignaciones - deducciones, 2)
            res[slip.id] = {
                'total_allocations': asignaciones,
                'total_deductions': deducciones,
                'total': total,
                }
        return res

    def _get_new_employee(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        line_brw = self.browse(cr, uid, ids)
        for line in line_brw:
            old_payslip_ids = self.pool.get('hr.payslip').search(cr, uid, [('id','<>',line.id),
                ('employee_id','=',line.employee_id.id),('date_from','<',line.date_from)])
            if not old_payslip_ids:
                res[line.id] = {
                    'new_employee': True,
                }
        return res

    _columns = {
        'hr_process_id': fields.many2one('hr.process', 'Proceso de Nómina'),
        'type_process': fields.related('hr_process_id', 'type', type='char', size=256, relation='hr.process', string="Tipo", store=True),
        'date_done': fields.related('hr_process_id', 'date_done', type='date', relation='hr.process', string="Fecha aprobacion", store=True),
        'total_allocations': fields.function(_compute_sum, method=True, string='Asignaciones',
                                            type='float', store=True, multi='totals'),
        'total_deductions': fields.function(_compute_sum, method=True, string='Deducciones',
                                            type='float', store=True, multi='totals'),
        'total': fields.function(_compute_sum, method=True, string='Total',
                                            type='float', store=True, multi='totals'),
        'devengado_acumulado':fields.float('Devengado acumulado', digits=(16,2), readonly=True),
        'saldo_prestamo':fields.float('Saldo prestamos', digits=(16,2), readonly=True),
        'sso_acumulado':fields.float('S.S.O. acumulado', digits=(16,2), readonly=True),
        'islr_acumulado':fields.float('I.S.L.R. acumulado', digits=(16,2), readonly=True),
        'utilidades_acumulado':fields.float('Devengado utilidades acumulado', digits=(16,2), readonly=True),
        'rpe_acumulado':fields.float('R.P.E. acumulado', digits=(16,2), readonly=True),
        'faov_acumulado':fields.float('F.A.O.V. acumulado', digits=(16,2), readonly=True),
        'department_id': fields.related('employee_id', 'department_id', type='many2one', relation='hr.department', string="Departamento", store=True),
        'new_employee': fields.function(_get_new_employee, method=True, string='Nuevo trabajador', type='boolean', store=True),
    }
    
    _order = 'id desc, hr_process_id, department_id'
    
    _defaults = {
        'total_allocations': 0.0,
        'total_deductions': 0.0,
        'total': 0.0,
        'new_employee': False
    }

    _sql_constraints = [('unique_process_employee','unique(hr_process_id, employee_id)', _('The combination of process and employee must be unique'))]
    
    def _check_hojas_pago(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        payslips_brw = self.browse(cr, uid, ids, context=context)
        fecha = datetime.now() - timedelta(days=360)
        if payslips_brw[0].hr_process_id.type == 'vacaciones':
            for payslip in payslips_brw:
                if payslip.type_process == 'vacaciones':
                    old_payslip = self.search(cr, uid, [('id','<>', payslip.id),('employee_id','=',payslip.employee_id.id),
                            ('state','=','done'),('type_process','=','vacaciones'),('date_from','>',fecha)])
                    if old_payslip:
                        return False
                    
        return True
        
    _constraints = [(_check_hojas_pago, "No se puede crear hoja de pago de vacaciones para el empleado. \
                    Se le pago bono vacacional en un lapso menor a un año", ['type_process'])]
    
    def create(self, cr, uid, vals, context=None):
        if 'hr_process_id' in vals:
            process = self.pool.get('hr.process').browse(cr, uid, vals['hr_process_id'])
            employee = self.pool.get('hr.employee').browse(cr, uid, vals['employee_id'])
            period = list(set(process.hr_period_ids) & set(employee.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
            vals.update({
                'date_from': period.date_start, 
                'date_to': period.date_end
            })
        else:
            vals.update({
                'date_from': datetime.now(), 
                'date_to': datetime.now(),
            })
        return super(hr_payslip, self).create(cr, uid, vals, context)
        
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = []
        for id in ids:
            payslip = self.browse(cr, uid, id)
            if payslip.state == 'done':
                raise osv.except_osv(_('Error!'),_('La hoja de pago %s no se puede eliminar, \
                                esta en estado Aprobado!')%(payslip.name))
        return super(hr_payslip, self).unlink(cr, uid, ids, context)
            
hr_payslip()

class hr_variable_concepts(osv.osv):
    '''
    Variable Concepts
    '''
    _name = 'hr.variable.concepts'
    _description = 'Variable Concepts'
    _columns = {
        'name': fields.char('Concepto Variable', size=256, readonly=True),
        'hr_process_id': fields.many2one('hr.process', 'Proceso de Nómina', required=True),
        'variable_concept_line_ids': fields.one2many('hr.variable.concept.line', 'variable_concept_id', 'Lineas de conceptos variables'),
        'state': fields.selection([('draft','Borrador'), ('load','Cargado'), ('execute','En ejecucion'),
                             ('review','Procesado'), ('done','Aprobado'), ('cancel','Cancelado')],
                             'State', readonly=True),
    }
    _order = 'id desc'
    
    _defaults = {
        'state':'draft'
    }
    
    #_sql_constraints = [('unique_process','unique(hr_process_id)', "Ya existen Conceptos Variables asociados al proceso de nomina")]

    def create(self, cr, uid, vals, context=None):
        """
        Evita que se creen conceptos variables asociados a un proceso de nomina
        que se encuentre en ejecucion.
        """
        process = self.pool.get('hr.process').browse(cr, uid, [vals['hr_process_id']])[0]
        if process.state == 'execute':
            raise osv.except_osv(_('Accion no permitida!'), _('No se permite crear Conceptos Variables asociados a un proceso de nomina que se encuentre en ejecucion'))
        else:
            return super(hr_variable_concepts, self).create(cr, uid, vals, context)

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}

        variable_concepts = self.browse(cr, uid, ids)
        if not isinstance(variable_concepts, list):
            variable_concepts = [variable_concepts]
        process = variable_concepts[0].hr_process_id
        cron = self.pool.get('ir.cron')
        cron_proceso_nomina = cron.search(cr, uid, [('name', '=', process.name)])

        if cron_proceso_nomina:
            raise osv.except_osv(_('Accion no permitida!'), _('No se permite modificar Conceptos Variables que se encuentren en ejecucion'))
        if not ('hr_process_id' in vals):
            return super(hr_variable_concepts, self).write(cr, uid, ids, vals, context)

        context['procesar'] = True # Para que en la sobreescritura del unlink permita eliminar la linea para recalcularla

        for var_concept in variable_concepts:
            for var_concept_line in var_concept.variable_concept_line_ids:
                self.pool.get('hr.variable.concept.line').unlink(cr, uid, [var_concept_line.id], context)

        record = super(hr_variable_concepts, self).write(cr, uid, ids, vals, context)
        return record

    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        record = False
        
        context['procesar'] = True #Para que en la sobreescritura del unlink permita eliminar la linea para recalcularla

        var_concept_brw = self.browse(cr, uid, ids)

        for var_concept in var_concept_brw:
            if var_concept.state == 'done':
                raise osv.except_osv(_('Accion no permitida!'), _('No se permite eliminar Conceptos Variables en estado Procesado'))
            elif var_concept.state == 'execute':
                raise osv.except_osv(_('Accion no permitida!'), _('No se permite eliminar Conceptos Variables que se encuentren en ejecucion'))
            for var_concept_line in var_concept.variable_concept_line_ids:
                self.pool.get('hr.variable.concept.line').unlink(cr, uid, [var_concept_line.id], context)

        record = super(hr_variable_concepts, self).unlink(cr, uid, ids, context)
        return record

hr_variable_concepts()

class hr_variable_concept_line(osv.osv):
    '''
    Variable Concept Line
    '''
    _name = 'hr.variable.concept.line'
    _description = 'Variable Concept Lines'
    _order = 'id desc'
    _logger = logging.getLogger('hr.variable.concept.line')

    _columns = {
        'name': fields.char('Linea Concepto Variable', size=256, readonly=True),
        'variable_concept_id': fields.many2one('hr.variable.concepts', 'Concepto Variable'),
        'payslip_line_ids': fields.one2many('hr.payslip.line', 'variable_concept_line_id', 'Lineas de nómina'),
        'employee_id': fields.many2one('hr.employee', 'Empleado'),
        'payroll_id': fields.many2one('hr.payroll', 'Nómina'),
        'concept_id': fields.many2one('hr.concept', 'Concepto',required=True),
        'type': fields.selection([('qty', 'Cantidad'),('amount', 'Monto')], "Tipo Valor", required=True),
        'value': fields.float('Valor', digits=(16,2),required=True),
    }

    def _check_employee(self, cr, uid, ids, context=None):
        for var_concept_line in self.browse(cr, uid, ids, context=context):
            if var_concept_line.employee_id and (
                (not var_concept_line.employee_id.hr_payroll_id in var_concept_line.variable_concept_id.hr_process_id.hr_payroll_ids)
                or (var_concept_line.variable_concept_id.hr_process_id.hr_sede_ids and 
                (not var_concept_line.employee_id.hr_sede_id in var_concept_line.variable_concept_id.hr_process_id.hr_sede_ids))):
                return False
        return True

    def _check_payroll(self, cr, uid, ids, context=None):
        for var_concept_line in self.browse(cr, uid, ids, context=context):
            if var_concept_line.payroll_id and (not var_concept_line.payroll_id in var_concept_line.variable_concept_id.hr_process_id.hr_payroll_ids):
                return False
        return True

    def _check_employee_or_payroll(self, cr, uid, ids, context=None):
        for var_concept_line in self.browse(cr, uid, ids, context=context):
            if (var_concept_line.employee_id and var_concept_line.payroll_id) or (not var_concept_line.employee_id and not var_concept_line.payroll_id):
                return False
        return True

    def _check_employee_in_process(self, cr, uid, ids, context=None):
        for var_concept_line in self.browse(cr, uid, ids, context=context):
            if var_concept_line.variable_concept_id.hr_process_id.hr_employee_ids:
                if var_concept_line.employee_id and \
                    not (var_concept_line.employee_id in var_concept_line.variable_concept_id.hr_process_id.hr_employee_ids):
                    return False
        return True

    _constraints = [(_check_employee_or_payroll, "El concepto variable debe tener un Empleado ó una Nómina (no ambos)", ['employee_id, payroll_id']),
            (_check_employee, "El empleado no se encuantra dentro de la nómina y/o sede a procesar", ['employee_id']),
            (_check_payroll, "La nómina seleccionada no se encuantra dentro de las nóminas del proceso", ['payroll_id']),
            (_check_employee_in_process, "El proceso seleccionado es para ciertos empleados. El empleado seleccionado no se encuentra dentro de ellos", ['employee_id']),
            ]

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
            
        employee_list = []
        record_id = super(hr_variable_concept_line, self).create(cr, uid, vals, context)
        variable_concept_brw = self.pool.get('hr.variable.concepts').browse(cr, uid, vals['variable_concept_id'])
        payslips = variable_concept_brw.hr_process_id.hr_payslip_ids
        
        if record_id:
            if 'employee_id' in vals and vals['employee_id']:
                employee_list.append(vals['employee_id'])
                payslips = [p for p in payslips if p.employee_id.id == vals['employee_id']]
            else:
                employee_payroll_brw = self.pool.get('hr.payroll').browse(cr, uid, vals['payroll_id']).employee_ids
                if variable_concept_brw.hr_process_id.hr_sede_ids:
                    employee_sedes = [x.employee_ids for x in variable_concept_brw.hr_process_id.hr_sede_ids]
                    employee_sedes_list = []
                    for e in employee_sedes:
                        employee_sedes_list = e + employee_sedes_list
                    employee_list = list(set(employee_sedes_list) & set(employee_payroll_brw))
                    employee_list = [y.id for y in employee_list]
                else:
                    for employee in employee_payroll_brw:
                        employee_list.append(employee.id)
                payslips = [p for p in payslips if p.employee_id.hr_payroll_id.id == vals['payroll_id']]

        concept = self.pool.get('hr.concept').browse(cr, uid, vals['concept_id'])
        line = {}
        line_id = False
        for p in payslips:
            line_dict = self.get_payslip_lines(cr, uid, record_id, p.id, context=context)
            line = [l for l in line_dict if l['code'] == concept.code]
            if not line:
                employee_list.remove(p.employee_id.id)
                continue
            else:
                line = line[0]
                
            line['variable_concept_line_id'] = record_id
            line['type_calc'] = concept.type_calc
            line['type'] = concept.type
            line['process_id'] = variable_concept_brw.hr_process_id.id
            line['concept_id'] = concept.id
            line['department_id'] = p.employee_id.department_id.id
            line['sede_id'] = p.employee_id.hr_sede_id.id
            
            line['total'] = float(line['quantity']) * round(line['amount'],2) * line['rate'] / 100
            
            if line['amount'] <> 0.0:
                save_on_database = False
                if 'salary_rule_id' in line:
                    rule = self.pool.get('hr.salary.rule').browse(cr, uid, line['salary_rule_id'])
                    save_on_database = rule.save_on_database
                else: #Es un attribute
                    attribute = self.pool.get('hr.attribute').browse(cr, uid, line['attribute_id'])
                    save_on_database = attribute.save_on_database
                if save_on_database: #Se debe guardar en BD
                    """
                    Si se creo para empleado y viene para empleado, se sobreescribe.
                    Si se creo para empleado y viene por nomina, no pasa nada.
                    Siempre que se haya creado para nómina se sobreescribe.
                    """
                    context['procesar'] = True #Para que en la sobreescritura del unlink permita eliminar la linea para recalcularla
                    old_line = [l for l in p.line_ids if l.code == concept.code]
                    if not old_line or (old_line[0].variable_concept_line_id.payroll_id and vals['employee_id']):
                        if old_line:
                            self.pool.get('hr.payslip.line').unlink(cr, uid, [old_line[0].id])
                            self._logger.debug('Se elimino payslip_line id %s' % (old_line[0].id))
                        line_id = self.pool.get('hr.payslip.line').create(cr, uid, line, context)
                        self._logger.debug('Se creo payslip_line id %s' % (line_id))
                        
                    elif (old_line[0].variable_concept_line_id.employee_id and vals['employee_id']) or \
                      (old_line[0].variable_concept_line_id.payroll_id and vals['payroll_id']):
                        line_id = self.pool.get('hr.payslip.line').write(cr, uid, [old_line[0].id], line, context)
                        self._logger.debug('Se sobreescribio payslip_line id %s' % (old_line[0].id))
            self.pool.get('hr.payslip').write(cr, uid, p.id, {'state': 'draft'})
            employee_list.remove(p.employee_id.id)
        
        for employee in employee_list:
            employee_brw = self.pool.get('hr.employee').browse(cr, uid, employee)
            if not employee_brw.state in ['activo','vacaciones','reposo']:
                if 'employee_id' in vals and vals['employee_id']:
                    raise osv.except_osv(_('ERROR!'),_('El empleado %s se encuentra en estado %s'%(employee_brw.name, employee_brw.state)))
                else:
                    continue
            period = list(set(variable_concept_brw.hr_process_id.hr_period_ids) & set(employee_brw.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
            date_end = employee_brw.contract_id.date_end >= period.date_start if employee_brw.contract_id.date_end else True
            if employee_brw.start_rel_lab > period.date_end or not date_end:
                if 'employee_id' in vals and vals['employee_id']:
                    raise osv.except_osv(_('ERROR!'),
                      _('Las fechas en el contrato para el empleado \
                      %s no estan dentro del periodo del proceso' % (employee_brw.name)))
                else:
                    continue
            res = {
                'employee_id': employee,
                'hr_process_id': variable_concept_brw.hr_process_id.id,
                'company_id': employee_brw.company_id.id,
                'line_ids':[],
                'input_line_ids': [],
                'worked_days_line_ids': [],
                'name':_('Hoja de pago %s - proceso %s') % (self.pool.get('hr.employee').name_get(cr, uid, [employee])[0][1], variable_concept_brw.hr_process_id.name),
                'contract_id': False,
                'payroll_structure_id': False,
                'hr_payroll_id': employee_brw.hr_payroll_id.id,
                'date_from': period.date_start,
                'date_to': period.date_end,
            }

            new_payslip_id = self.pool.get('hr.payslip').create(cr, uid, res, context)
            self._logger.debug('Se creo payslip id %s' % (new_payslip_id))

            line_dict = self.get_payslip_lines(cr, uid, record_id, new_payslip_id, context=context)
            line = [l for l in line_dict if l['code'] == concept.code]
            if not line:
                continue
            else:
                line = line[0]
                                
            line['variable_concept_line_id'] = record_id
            line['type_calc'] = concept.type_calc
            line['type'] = concept.type
            line['process_id'] = variable_concept_brw.hr_process_id.id
            line['concept_id'] = concept.id
            line['department_id'] = employee_brw.department_id.id
            line['sede_id'] = employee_brw.hr_sede_id.id
            
            line['total'] = float(line['quantity']) * round(line['amount'],2) * line['rate'] / 100
            
            line_id = self.pool.get('hr.payslip.line').create(cr, uid, line, context)
            self._logger.debug('Se creo payslip_line id %s' % (line_id))

        if not line_id and ('employee_id' in vals and vals['employee_id']):
            super(hr_variable_concept_line, self).unlink(cr, uid, record_id)#Si no se creo el payslip_line (posiblemente por la condicion de la regla) se borra el variable_concept_line
            self._logger.debug('Se elimino variable_concept_line id %s' % (record_id))
            raise osv.except_osv(_('ERROR!'),_('No se creo la línea de hoja de pago. Revisar estructura del concepto'))
        
        return record_id

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        record = False
        var_concept_line_brw = self.browse(cr, uid, ids)
        for var_concept_line in var_concept_line_brw:
            vals.update({'variable_concept_id': var_concept_line.variable_concept_id.id,})
            record = self.unlink(cr, uid, var_concept_line.id, context)
            if record:
                self.create(cr, uid, vals, context)
        return record

    def unlink(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
            
        if context is None:
            context = {}
        
        """
        TODO: Revisar estas declaraciones en context.
        """
        #Para que en la sobreescritura del unlink permita eliminar la linea para recalcularla
        context['procesar'] = True 
        if 'unlink_slip_line' in context:
            """
            Si viene del llamado que se hace al eliminar una linea de 
            hoja de pago solo se debe eliminar la linea de concepto 
            variable
            """
            return super(hr_variable_concept_line, self).unlink(cr, uid, ids, context)

        slip_line_obj = self.pool.get('hr.payslip.line')
        slip_line_ids = slip_line_obj.search(cr, uid, [('variable_concept_line_id', 'in', ids),])
        line_brw  = slip_line_obj.browse(cr, uid, slip_line_ids)
        slip_ids = [x.slip_id.id for x in line_brw]
        slip_line_obj.unlink(cr, uid, slip_line_ids, context)
        self.pool.get('hr.payslip').write(cr, uid, slip_ids, {'state': 'draft',})
        
        return super(hr_variable_concept_line, self).unlink(cr, uid, ids, context)

    def _recursive_search_of_rules(self, cr, uid, rule_ids, context=None):
        """
        @param rule_ids: list of browse record
        @return: returns a list of tuple (id, sequence) which are all the children of the passed rule_ids
        """
        children_rules = []
        for rule in rule_ids:
            if rule.child_ids:
                children_rules += self._recursive_search_of_rules(cr, uid, rule.child_ids, context=context)
        return [(r.id, r.sequence) for r in rule_ids] + children_rules

    def get_rules(self, cr, uid, structure_ids, concept_id, employee, period_id, context=None):
        all_rules = []

        for struct in self.pool.get('hr.payroll.structure').browse(cr, uid, structure_ids, context=context):
            #Reglas de los conceptos fijos ques estan dentro de la estructura + las reglas dentro del concepto variable
            concept_ids = struct.concept_ids + [concept_id]
            for concept in concept_ids:
                period_brw = self.pool.get('hr.period').browse(cr, uid, period_id)
                date_end = concept.date_end >= period_brw.date_start if concept.date_end else True
                if concept.date_start <= period_brw.date_end and date_end:
                    salary_rule_ids = []
                    for salary_rule in concept.salary_rule_ids:
                        #Si la regla aplica para una nomina en especial
                        if salary_rule.hr_payroll_ids and employee.hr_payroll_id in salary_rule.hr_payroll_ids:
                            salary_rule_ids.append(salary_rule)
                        #Si la regla no tiene hr_payroll_ids aplica para todas las nominas
                        elif not salary_rule.hr_payroll_ids:
                            salary_rule_ids.append(salary_rule)
                    all_rules += self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, salary_rule_ids, context=context)
        return all_rules

    def get_attributes(self, cr, uid, concept, employee, period_id, context=None):
        all_attributes = []
        if concept.type == 'variable':
            period_brw = self.pool.get('hr.period').browse(cr, uid, period_id)
            date_end = concept.date_end >= period_brw.date_start if concept.date_end else True
            if concept.date_start <= period_brw.date_end and date_end:
                for attribute in concept.attribute_ids:
                    if employee.id == attribute.employee.id:
                        all_attributes.append(attribute)
        return all_attributes

    def get_payslip_lines(self, cr, uid, ids, payslip_id, context):
        def _sum_salary_rule_category(localdict, category, amount):
            if category.parent_id:
                localdict = _sum_salary_rule_category(localdict, category.parent_id, amount)
            localdict['categories'].dict[category.code] = category.code in localdict['categories'].dict and localdict['categories'].dict[category.code] + amount or amount
            return localdict

        class BrowsableObject(object):
            def __init__(self, pool, cr, uid, employee_id, dict):
                self.pool = pool
                self.cr = cr
                self.uid = uid
                self.employee_id = employee_id
                self.dict = dict

            def __getattr__(self, attr):
                return attr in self.dict and self.dict.__getitem__(attr) or 0.0

        class InputLine(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(amount) as sum\
                            FROM hr_payslip as hp, hr_payslip_input as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()[0]
                return res or 0.0

        class WorkedDays(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""
            def _sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                result = 0.0
                self.cr.execute("SELECT sum(number_of_days) as number_of_days, sum(number_of_hours) as number_of_hours\
                            FROM hr_payslip as hp, hr_payslip_worked_days as pi \
                            WHERE hp.employee_id = %s AND hp.state = 'done'\
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pi.payslip_id AND pi.code = %s",
                           (self.employee_id, from_date, to_date, code))
                return self.cr.fetchone()

            def sum(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[0] or 0.0

            def sum_hours(self, code, from_date, to_date=None):
                res = self._sum(code, from_date, to_date)
                return res and res[1] or 0.0

        class Payslips(BrowsableObject):
            """a class that will be used into the python code, mainly for usability purposes"""

            def sum(self, code, from_date, to_date=None):
                if to_date is None:
                    to_date = datetime.now().strftime('%Y-%m-%d')
                self.cr.execute("SELECT sum(case when hp.credit_note = False then (pl.total) else (-pl.total) end)\
                            FROM hr_payslip as hp, hr_payslip_line as pl \
                            WHERE hp.employee_id = %s AND hp.state = 'done' \
                            AND hp.date_from >= %s AND hp.date_to <= %s AND hp.id = pl.slip_id AND pl.code = %s",
                            (self.employee_id, from_date, to_date, code))
                res = self.cr.fetchone()
                return res and res[0] or 0.0

        #we keep a dict with the result because a value can be overwritten by another rule with the same code
        result_dict = {}
        rules = {}
        categories_dict = {}
        payroll_dict = {}
        period_dict = {}
        parameters_dict = {}
        categories_concept_dict = {}
        blacklist = []
        payslip_obj = self.pool.get('hr.payslip')
        inputs_obj = self.pool.get('hr.payslip.worked_days')
        obj_rule = self.pool.get('hr.salary.rule')
        payslip = payslip_obj.browse(cr, uid, payslip_id, context=context)
        worked_days = {}

        for worked_days_line in payslip.worked_days_line_ids:
            worked_days[worked_days_line.code] = worked_days_line
        inputs = {}

        for input_line in payslip.input_line_ids:
            inputs[input_line.code] = input_line

        categories_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, categories_dict)
        input_obj = InputLine(self.pool, cr, uid, payslip.employee_id.id, inputs)
        worked_days_obj = WorkedDays(self.pool, cr, uid, payslip.employee_id.id, worked_days)
        payslip_obj = Payslips(self.pool, cr, uid, payslip.employee_id.id, payslip)
        rules_obj = BrowsableObject(self.pool, cr, uid, payslip.employee_id.id, rules)
        get_method = getattr(self.pool.get('hr.employee'), '_get')
        get_concept = getattr(self.pool.get('hr.concept'), 'get_cal_concept')
        localdict = {
            'categories': categories_obj,
            'rules': rules_obj,
            'payslip': payslip,
            'worked_days': worked_days_obj,
            'inputs': input_obj,
            'atributo': get_method,
            'historial': get_concept,
            'param': [cr, uid],
            'concepto': {},
            'cantidad_horas': {},
            'categoria_concepto': {},
            'nomina': payroll_dict,
            'datetime': datetime,
        }
        
        employee = payslip.employee_id
        edad = (datetime.now() - datetime.strptime(employee.birthday,"%Y-%m-%d")).days/365
        localdict.update({'empleado': employee,
                'sexo': 'Femenino' if employee.gender == 'female' else 'Masculino',
                'edad': edad,
                'jornada_dia': employee.shift_id.jor_day,
                'jornada_semana': employee.shift_id.jor_week,})

        # Busqueda del parametro que indica la cantidad de dias para perder descanso. Si el parametro no existe
        # al empleado se le calculara el descanso para que luego verifiquen -----------------------------
        parameter_obj = self.pool.get('hr.parameters')
        param_perder_descanso_id = parameter_obj.search(cr, uid, [('type', '=', 'perdida_descanso'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_perder_descanso_id:
            param_perder_descanso_id = parameter_obj.search(cr, uid, [('type', '=', 'perdida_descanso'),('payroll_id', '=', None)])

        if param_perder_descanso_id:
            pierde_descanso = parameter_obj.browse(cr, uid, param_perder_descanso_id)[0].value
            pierde_descanso = pierde_descanso*employee.shift_id.jor_day #Para llevarlo a horas
            localdict['pierde_descanso'] = pierde_descanso
        else:
            localdict['pierde_descanso'] = 1000 #Si no tiene lo calcula igual para que luego verifiquen
        #-------------------------------------------------------------------------------------------------------------------------

        structure_ids = []
        structure_brw = [j for j in payslip.employee_id.hr_payroll_id.hr_payroll_structure_ids]
        for str_brw in structure_brw: #El tipo de la estructura debe ser del mismo tipo del proceso
            if str_brw.type == payslip.hr_process_id.type:
                structure_ids.append(str_brw.id)

        #Agregar parametros de la nomina
        for parameter in employee.hr_payroll_id.parameters_ids:
            parameters_dict[parameter.type] = parameter.value or 0.0
        localdict.update({'parametros': parameters_dict,})

        #Busqueda del concepto que se definio como concepto sueldo en los parametros de la nomina-----------------------------
        parameter_obj = self.pool.get('hr.parameters')
        param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', payslip.employee_id.hr_payroll_id.id)])
        if not param_concept_salary_id:
            param_concept_salary_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_sueldo'),('payroll_id', '=', None)])
        if not param_concept_salary_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto salario para la nomina %s. Ir a \
                                                la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar un parametro de tipo concepto_sueldo')
                                                %(payslip.employee_id.hr_payroll_id.name))

        concept_salary_brw = parameter_obj.browse(cr, uid, param_concept_salary_id)[0].concept_wage_id

        #Busqueda del concepto que se definio como Concepto para registro de fecha para calculos de prestaciones-----------------------------
        param_fp_id = parameter_obj.search(cr, uid, [('type', '=', 'fecha_prest')])
        concept_fp_id = 0
        if param_fp_id:
            concept_fp_id = parameter_obj.browse(cr, uid, param_fp_id)[0].concept_prest_id

        var_concept_line_brw = self.browse(cr, uid, ids)

        #Calculo de los lunes en el periodo actual (para seguro social obligatorio) y regimen prestacional
        period = list(set(payslip.hr_process_id.hr_period_ids) & set(employee.hr_payroll_id.hr_payment_calendar_id.hr_period_ids))[0]
        mondays_in_period = 1
        if period.type == 'mensual':
            mondays_in_period = period.hr_month_id.monday_first_fortnight if period.period_in_month == 1 else period.hr_month_id.monday_second_fortnight
        localdict.update({'lunes': mondays_in_period})
        
        #Numero de periodos en el mes anterior
        periodos_mes_ant = 0
        if period.hr_month_id.month_year == 1:
            year_ant = self.pool.get('hr.year').search(cr, uid,[('year','=',period.hr_month_id.hr_year_id.year-1)])
            if year_ant:
                if isinstance(year_ant,list):
                    year_ant = year_ant[0]
                month_ant = self.pool.get('hr.month').search(cr, uid, [('month_year','=',12),('hr_year_id','=',year_ant)])
        else:
            month_ant = self.pool.get('hr.month').search(cr, uid, [('month_year','=',period.hr_month_id.month_year-1),
                                            ('hr_year_id','=',period.hr_month_id.hr_year_id.id)])
        
        if month_ant:
            if isinstance(month_ant,list):
                month_ant = month_ant[0]
            periodos_mes_ant_ids = self.pool.get('hr.period').search(cr, uid, [('hr_month_id','=',month_ant),('type','=',period.type)])
            periodos_mes_ant = len(periodos_mes_ant_ids)
        
        localdict['periodos_mes_ant'] = periodos_mes_ant
        localdict['ultimo_periodo'] = period.last_period_month
        localdict['periodo'] = period

        if payslip.hr_process_id.month_id:
            localdict['mes'] = payslip.hr_process_id.month_id

        #Verificar con cada empleado si esta de aniversario
        if not employee.contract_id:
            raise osv.except_osv(_('ERROR!'), _('El empleado %s no tiene un contrato registrado')
                    %(employee.name))

        period_end_year = datetime.strptime(period.date_end,"%Y-%m-%d").year

        contract_year = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").year
        contract_month = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").month
        contract_day = datetime.strptime(employee.start_rel_lab,"%Y-%m-%d").day

        if contract_year < period_end_year:
            nueva_f = datetime(period_end_year, contract_month, contract_day)
            localdict['aniversario'] = True if (nueva_f >= datetime.strptime(period.date_start,"%Y-%m-%d") and
                        nueva_f <= datetime.strptime(period.date_end,"%Y-%m-%d")) else False
        else:
            localdict['aniversario'] = False
        
        #Fecha de ingreso segun fecha de prestaciones
        attr_date_prest = list(set(concept_fp_id.attribute_ids) & set(payslip.employee_id.attribute_ids)) if concept_fp_id else False
        date_prest = datetime.strptime(attr_date_prest[0].date_value,"%Y-%m-%d") if attr_date_prest \
                        else datetime.strptime(payslip.employee_id.start_rel_lab,"%Y-%m-%d")
                        
        if date_prest.year < period_end_year:
            nueva_f = datetime(period_end_year, date_prest.month, date_prest.day)
            
            period_start = datetime.strptime(period.date_start,"%Y-%m-%d")
            period_end = datetime.strptime(period.date_end,"%Y-%m-%d")
            
            if period.period_in_month == 1:
                period_start = datetime(period_start.year, period_start.month, 01)
            elif period.last_period_month:
                last_day = calendar.monthrange(period_start.year,period_start.month)[1]
                period_end = datetime(period_start.year, period_start.month, last_day)
                
            localdict['aniversario_prest'] = True if (nueva_f >= period_start and nueva_f <= period_end) else False
        else:
            localdict['aniversario_prest'] = False
            
        #Dias trabajados por el empleado en el periodo a pagar
        if period.date_start > employee.start_rel_lab:
            if period.type == 'diario':
                localdict['dias_trabajados'] = 6
            else:
                localdict['dias_trabajados'] = 15
        else:
            if period.type == 'diario':
                dias = datetime.strptime(period.date_start,"%Y-%m-%d") - datetime.strptime(employee.start_rel_lab,"%Y-%m-%d")
                localdict['dias_trabajados'] = dias.days + 6
            else:
                dias = datetime.strptime(period.date_start,"%Y-%m-%d") - datetime.strptime(employee.start_rel_lab,"%Y-%m-%d")
                localdict['dias_trabajados'] = dias.days + 15
                    
        if var_concept_line_brw.concept_id.salary_rule_ids:
            rule_ids = self.get_rules(cr, uid, structure_ids, var_concept_line_brw.concept_id, payslip.employee_id, period.id, context=context)
            #run the rules by sequence
            sorted_rule_ids = [id for id, sequence in sorted(rule_ids, key=lambda x:x[1])]

            rules_dict = {}
            hours_dict = {}
            
            #Incluir en el localdict los montos de los conceptos ya cargados
            for line in payslip.line_ids:
                rules_dict[line.code] = line.total
                localdict['concepto'] = rules_dict
            
            for attribute in employee.attribute_ids:
                if attribute.concept_id.id == concept_salary_brw.id:
                    rules_dict['SUELDO'] = attribute.amount
                    localdict['concepto'] = rules_dict
                    
            for rule in obj_rule.browse(cr, uid, sorted_rule_ids, context=context):
                qty_ = 1.0
                key = rule.code
                localdict['result'] = None
                localdict['result_qty'] = 1.0
                #check if the rule can be applied
                if obj_rule.satisfy_condition(cr, uid, rule.id, localdict, context=context) and rule.id not in blacklist:
                    if rule.concept_id.type == 'variable':
                        variable = True
                    else:
                        variable = False
                    #compute the amount of the rule
                    amount, qty, rate = obj_rule.compute_rule(cr, uid, rule.id, localdict, context=context)
                    if var_concept_line_brw.type == 'qty':
                        amount_ = amount
                        qty_ = var_concept_line_brw.value
                    else:
                        amount_ = var_concept_line_brw.value
                    #check if there is already a rule computed with that code
                    previous_amount = rule.code in localdict and localdict[rule.code] or 0.0
                    #set/overwrite the amount computed for this rule in the localdict
                    tot_rule = amount * qty * rate / 100.0
                    localdict[rule.code] = tot_rule
                    rules[rule.code] = rule
                    #sum the amount for its salary category
                    localdict = _sum_salary_rule_category(localdict, rule.category_id, tot_rule - previous_amount)
                    #create/overwrite the rule in the temporary results
                    result_dict[key] = {
                        'salary_rule_id': rule.id,
                        'name': rule.name,
                        'code': rule.code,
                        'category_id': rule.category_id.id,
                        'sequence': rule.sequence,
                        'appears_on_payslip': rule.appears_on_payslip,
                        'condition_select': rule.condition_select,
                        'condition_python': rule.condition_python,
                        'condition_range': rule.condition_range,
                        'condition_range_min': rule.condition_range_min,
                        'condition_range_max': rule.condition_range_max,
                        'amount_select': rule.amount_select,
                        'amount_fix': rule.amount_fix,
                        'amount_python_compute': rule.amount_python_compute,
                        'amount_percentage': rule.amount_percentage,
                        'amount_percentage_base': rule.amount_percentage_base,
                        'register_id': rule.register_id.id,
                        'amount': amount_,
                        'employee_id': employee.id,
                        'quantity': qty_,
                        'rate': rate,
                        'slip_id': payslip_id,
                        'variable': variable,
                        'state': 'draft',
                    }
                    rules_dict[rule.code] = amount_ or 0.0
                    localdict['concepto'] = rules_dict

                    ''' Para el concepto de Descanso se debe tener la cuanta de las horas en la semana que el emplaedo falto,
                    para ello se agrega un diccionario en el cual se lleva la cuenta de la cantidad en horas por concepto.
                    Luego en la condicion del concepto Descanso se pregunta si esta cantidad de horas es mayor
                    a la cantidad que se suministra en los parametro para la perdida del descanso
                    '''
                    if rule.concept_id.type_amount and rule.concept_id.type_amount == 'hours':
                        hours = qty
                    else:
                        hours = qty*employee.shift_id.jor_day
                    hours_dict[rule.code] = hours
                    localdict['cantidad_horas'] = hours_dict

                    if rule.concept_id.category_id.code not in categories_concept_dict:
                        categories_concept_dict[rule.concept_id.category_id.code] = amount or 0.0
                    else:
                        categories_concept_dict[rule.concept_id.category_id.code] = categories_concept_dict[rule.concept_id.category_id.code] + amount
                    localdict['categoria_concepto'] = categories_concept_dict

                else:
                    #blacklist this rule and its children
                    blacklist += [id for id, seq in self.pool.get('hr.salary.rule')._recursive_search_of_rules(cr, uid, [rule], context=context)]
            
        else:
            attribute_ids = self.get_attributes(cr, uid, var_concept_line_brw.concept_id, payslip.employee_id, context=context)
            for attribute in attribute_ids:
                quantity = var_concept_line_brw.type == 'qty' and var_concept_line_brw.value or attribute.amount
                amount = var_concept_line_brw.type <> 'qty' and var_concept_line_brw.value or attribute.amount
                key = attribute.code
                if not key in result_dict:
                    result_dict[key] = {
                        'attribute_id': attribute.id,
                        'appears_on_payslip': attribute.appears_on_payslip,
                        'name': attribute.name,
                        'code': attribute.code,
                        'amount': amount,
                        'employee_id': employee.id,
                        'quantity': quantity,
                        'rate': 100.00,
                        'slip_id': payslip_id,
                        'variable':True,
                        'state': 'draft',
                    }
                    rules_dict[attribute.code] = attribute.amount or 0.0
                    localdict['concepto'] = rules_dict
                else:
                    result_dict[key+str(attribute.id)] = {
                        'attribute_id': attribute.id,
                        'appears_on_payslip': attribute.appears_on_payslip,
                        'name': attribute.name,
                        'code': attribute.code,
                        'amount': amount,
                        'employee_id': employee.id,
                        'quantity': quantity,
                        'rate': 100.00,
                        'slip_id': payslip_id,
                        'variable':True,
                        'state': 'draft',
                    }
                    rules_dict[attribute.code] = rules_dict[attribute.code] + (attribute.amount or 0.0)
                    localdict['concepto'] = rules_dict
                    
            if attribute.concept_id.category_id.code not in categories_concept_dict:
                categories_concept_dict[attribute.concept_id.category_id.code] = amount or 0.0
            else:
                categories_concept_dict[attribute.concept_id.category_id.code] = categories_concept_dict[attribute.concept_id.category_id.code] + amount
            localdict['categoria_concepto'] = categories_concept_dict
        
        result = [value for code, value in result_dict.items()]
        
        return result

hr_variable_concept_line()

class hr_payslip_line(osv.osv):
    '''
    Payslip Line
    '''

    _inherit = 'hr.payslip.line'
    _description = 'Payslip Line'

    def _get_concept_type_calc(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        line_brw = self.browse(cr, uid, ids)
        for line in line_brw:
            if line.salary_rule_id:
                concept = line.salary_rule_id.concept_id.id
                type_calc =  line.salary_rule_id.concept_id.type_calc
                type_amount = line.salary_rule_id.concept_id.type_amount
                type = line.salary_rule_id.concept_id.type
            else:
                concept = line.attribute_id.concept_id.id
                type_calc =  line.attribute_id.concept_id.type_calc
                type_amount = line.attribute_id.concept_id.type_amount
                type = line.attribute_id.concept_id.type

            res[line.id] = {
                'concept_id': concept or False,
                'type_calc': type_calc or ' ',
                'type_amount': type_amount or ' ',
                'type': type or ' ',
            }
        return res
        
    _columns = {
        'attribute_id': fields.many2one('hr.attribute', 'Atributo'),
        'concept_id':fields.function(_get_concept_type_calc, method=True, string='Concepto', type='many2one', relation='hr.concept', store=True, multi='concept'),
        'type_calc':fields.function(_get_concept_type_calc, method=True, string='Tipo de calculo', store=True, type="char", size=64, multi='concept'),
        'type_amount':fields.function(_get_concept_type_calc, method=True, string='Unidad', store=True, type="char", size=64, multi='concept'),
        'type':fields.function(_get_concept_type_calc, method=True, string='Tipo de concepto', store=True, type="char", size=64, multi='concept'),
        'process_id':fields.related('slip_id', 'hr_process_id', type='many2one', relation='hr.process', string='Proceso', readonly=True, store=True),
        'process_state':fields.related('process_id', 'state', type='char', relation='hr.process', string='Estado', readonly=True),
        'state': fields.selection([
            ('draft', 'Borrador'),
            ('done', 'Aprobado'),
            ('cancel', 'Cancelado'),
        ], 'State', select=True, readonly=True,
            help='* When the payslip line is created the state is \'Draft\'.\
            \n* If the payslip line is in exception, the state is \'Exception\'. \
            \n* If the payslip line is confirmed then state is set to \'Done\'.\
            \n* When user cancel payslip line the state is \'Rejected\'.'),
        'variable_concept_line_id': fields.many2one('hr.variable.concept.line', 'Linea de Concepto Variable'),
        'variable':fields.boolean('Variable'),
        'employee_family_id': fields.many2one('hr.employee.family', 'Información hijo (guardería)'), #Campo usado para crear la orden de pago de las guarderias
        'department_id':fields.related('employee_id', 'department_id', type='many2one', relation='hr.department', string='Departamento', readonly=True, store=True),
        'sede_id':fields.related('employee_id', 'hr_sede_id', type='many2one', relation='hr.sede', string='Sede', readonly=True, store=True),
    }

    _order = 'type_calc, code'

    def _default_validate_create(self, cr, uid, context=None):
        if context is None:
            context = {}
        if 'create' in context:
            raise osv.except_osv(_('Acceso Denegado!'),_('No se permite crear Lineas de Hoja de Pago desde esta opción!'))
        return 0

    _defaults = {
        'variable': False,
        'employee_id':_default_validate_create, #Esta funcion se usa para bloquear la creacion de lineas de hoja de pago desde la vista en tree
    }

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        if 'create' in context: #Se esta tratando de modificar un payslip_line desde las vista de las lineas
            raise osv.except_osv(_('Acceso Denegado!'),_('No se permite modificar Lineas de Hoja de Pago directamente. \
                                                    Se debe recalcular la Hoja de Pago o modificar la línea de concepto variable, \
                                                    según sea el caso!'))
        else:
            return super(hr_payslip_line, self).write(cr, uid, ids, vals, context)

    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        record = False
        var_concept_line_obj = self.pool.get('hr.variable.concept.line')
        payslip_line_brw = self.browse(cr, uid, ids)
        
        for payslip_line in payslip_line_brw:
            if 'procesar' in context or payslip_line.linea_liq_id:
                return super(hr_payslip_line, self).unlink(cr, uid, ids, context)
            
            if (not payslip_line.variable_concept_line_id) or \
                (payslip_line.variable_concept_line_id.variable_concept_id.state == 'done'):
                raise osv.except_osv(_('Acceso Denegado!'),_('No se permite eliminar Líneas de Hoja de Pago de conceptos fijos \
                                    o de Procesos que ya esten aprobados'))
            
            var_concept_line = payslip_line.variable_concept_line_id
            context['unlink_slip_line'] = True #Se usa para que en caso de tener que eliminar el var_concept_line
                                                #asociado al payslip_line, en la sobreescritura del unlink solo elimine
                                                #ese var_concept_line
            if payslip_line.variable_concept_line_id.employee_id: #Viene de un concepto variable de un solo empleado
                payslip_brw = payslip_line.slip_id
                if payslip_brw and super(hr_payslip_line, self).unlink(cr, uid, payslip_line.id, context):
                    self.pool.get('hr.payslip').write(cr, uid, payslip_brw.id, {'state': 'draft'})
                var_concept_line_obj.unlink(cr, uid, var_concept_line.id, context)

            else:
                # Viene de un concepto variable de una nomina por lo que se elimina solo la linea de hoja de pago
                payslip_brw = payslip_line.slip_id
                if payslip_brw and super(hr_payslip_line, self).unlink(cr, uid, payslip_line.id, context):
                    self.pool.get('hr.payslip').write(cr, uid, payslip_brw.id, {'state': 'draft'})
                if len(var_concept_line.payslip_line_ids) == 0:
                    var_concept_line_obj.unlink(cr, uid, var_concept_line.id, context)
        return True

hr_payslip_line()

class hr_holidays_calendar(osv.osv):

    _name = 'hr.holidays.calendar'
    _description = 'Holidays Calendar'

    _columns = {
        'name':fields.char('Nombre', size=256, required=True),
        'date': fields.date('Fecha', required=True),
    }

    _sql_constraints = [('unique_date','unique(date)', _('Solo se puede registrar un feriado por dia'))]

    def day_of_week(self, fecha):
        dia = ['Sabado','Domingo','Lunes','Martes','Miercoles','Jueves','Viernes']
        val1 = fecha.month
        val2 = fecha.year
        if fecha.month == 1:
            val1 = 13
            val2 = val2-1

        if fecha.month == 2:
            val1 = 14
            val2 = val2-1

        val3 = ((val1+1)*3)/5
        val4 = val2/4
        val5 = val2/100
        val6 = val2/400
        val7 = fecha.day+(val1*2)+val3+val2+val4-val5+val6+2
        val8 = val7/7
        val0 = val7-(val8*7)
        return dia[val0]

    def calculate_period_holidays(self, cr, uid, start, days, aditional_days):
        holidays = []
        (MON, TUE, WED, THU, FRI, SAT, SUN) = range(7)
        workdays = (MON, TUE, WED, THU, FRI)
        weeks, days = divmod(days, len(workdays))
        result = start + timedelta(weeks=weeks) - timedelta(days=1)
        
        '''A la fecha resultante (periodo de 15 de vacaciones) se le deben sumar los dias adicionales
            que le tocan al empleado disfrutar por cada año de servicio a partir del segundo año
        '''
        i = 0
        while i < aditional_days:
            result = result + timedelta(days=1)
            if not self.day_of_week(result) in ['Sabado','Domingo']:
                i = i + 1

        '''Luego al periodo resultante se le deben sumar los dias feriados que esten dentro de el
        '''
        holidays_ids = self.search(cr, uid, [('date', '>=', start),('date', '<=', result)])
        for h in self.browse(cr, uid, holidays_ids):
            date = datetime.strptime(h.date,"%Y-%m-%d")
            if not self.day_of_week(date) in ['Sabado','Domingo']:
                holidays.append(datetime.strptime(h.date,"%Y-%m-%d"))

        lo, hi = min(start, result), max(start, result)
        count = len([h for h in holidays if h >= lo and h <= hi])
        days += count * (-1 if days < 0 else 1)
        for _ in range(days):
            result += timedelta(days=1)
            ''' Cada vez que se sume un dia se debe verificar si este es feriado y si es asi agregarlo a la lista holidays
            '''
            holidays_ids = self.search(cr, uid, [('date', '=', result)])
            for h in self.browse(cr, uid, holidays_ids):
                date = datetime.strptime(h.date,"%Y-%m-%d")
                if not self.day_of_week(date) in ['Sabado','Domingo']:
                    holidays.append(datetime.strptime(h.date,"%Y-%m-%d"))
            while result in holidays or result.weekday() not in workdays:
                result += timedelta(days=1)
                holidays_ids = self.search(cr, uid, [('date', '=', result)])
                for h in self.browse(cr, uid, holidays_ids):
                    date = datetime.strptime(h.date,"%Y-%m-%d")
                    if not self.day_of_week(date) in ['Sabado','Domingo']:
                        holidays.append(datetime.strptime(h.date,"%Y-%m-%d"))
        return result

    def mondays_in_holidays(self, cr, uid, date_start, date_end):
        mondays = 0
        date = date_start
        while date < date_end:
            day = self.day_of_week(date)
            if day == 'Lunes':
                mondays += 1
            date = date + timedelta(days=1)
        return mondays

hr_holidays_calendar()

class hr_department(osv.osv):
    _description = "Department"
    _inherit = 'hr.department'

    _columns = {
       'departament_analytic_account_id': fields.many2one('account.analytic.account', 'Cuenta analítica'),
    }

    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        if context == {}: #Se esta tratando de crear un department desde el wizard de aumentos salariales masivos
            raise osv.except_osv(_('Acceso Denegado!'),_('No se permite crear/modificar Departamentos desde esta opción.'))
        else:
            return super(hr_department, self).create(cr, uid, vals, context)

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        if not '_terp_view_name' in context: #Se esta tratando de modificar un department desde el wizard de aumentos salariales masivos
            raise osv.except_osv(_('Acceso Denegado!'),_('No se permite crear/modificar Departamentos desde esta opción.'))
        else:
            return super(hr_department, self).write(cr, uid, ids, vals, context)

hr_department()

class hr_account(osv.osv):

    _description = "Asientos contables nomina"
    _name = 'hr.account'

    _columns = {
        'name':fields.char('Name', size=256),
        'concept_id': fields.many2one('hr.concept', 'Concepto'),
        'analytic_account_id': fields.many2one('account.analytic.account', 'Cuenta analítica'),
        'payroll_id': fields.many2one('hr.payroll', 'Nómina'),
        'account_id': fields.many2one('account.account', 'Cuenta contable'),
        'account_payroll_id': fields.many2one('account.account', 'Cuenta contrapartida'),
    }

    def _check_unique_relation(self, cr, uid, ids, context=None):
        for hr_account in self.browse(cr, uid, ids, context=context):
            if self.search(cr, uid, [('id', '<>', hr_account.id),('concept_id', '=', hr_account.concept_id.id),
                                    ('analytic_account_id', '=', hr_account.analytic_account_id.id),('payroll_id', '=', hr_account.payroll_id.id)]):
                return False
        return True

    _constraints = [(_check_unique_relation, "Ya existe la misma relación Concepto-Cuenta Analítica-Nómina", ['concept_id,analytic_account_id,payroll_id'])]

hr_account()

class hr_concept_loan(osv.osv):

    _description = "Conceptos de prestamos"
    _name = 'hr.concept.loan'

    _columns = {
        'concept_id': fields.many2one('hr.concept', 'Concepto', required=True),
        'concept_allocation_id': fields.many2one('hr.concept', 'Concepto de asignación'),
        'name': fields.related('concept_id', 'name', type='string', relation='hr.concept', string="Nombre"),
        'partner_id': fields.many2one('res.partner', 'Proveedor'),
    }

    _sql_constraints = [('unique_concept','unique(concept_id)', _('El concepto debe ser unico'))]

hr_concept_loan()

class hr_loan(osv.osv):

    _description = "Préstamos"
    _name = 'hr.loan'
        
    def _compute_quota(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        loan_brw = self.browse(cr, uid, ids)
        if not isinstance(loan_brw, list):
            loan_brw = [loan_brw]
        for loan in loan_brw:
            amount_quota = loan.amount/loan.num_quotas
            res[ids[0]] = amount_quota
        return res

    def _compute_remaining(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        loan_brw = self.browse(cr, uid, ids)
        if not isinstance(loan_brw, list):
            loan_brw = [loan_brw]
        for loan in loan_brw:
            remaining_debt = loan.amount - loan.paid_amount
            res[ids[0]] = remaining_debt
        return res

    _columns = {
        'employee_id': fields.many2one('hr.employee', 'Empleado', required=True),
        'concept_id': fields.many2one('hr.concept.loan', 'Concepto', required=True),
        'concept_allocation_id': fields.related('concept_id', 'concept_allocation_id', type='many2one', 
                                            relation='hr.concept.loan', string="Concepto de asignación"),        
        'amount': fields.float('Monto préstamo', digits=(16,2), required=True),
        'amount_quota': fields.function(_compute_quota, method=True, string='Monto cuota', type='float', store=False),
        'num_quotas': fields.integer('Número de cuotas a pagar', digits=(16,2), required=True),
        'date_start': fields.date('Fecha inicio'),
        'date_end': fields.date('Fecha fin', readonly=True),
        'date_payment': fields.date('Fecha de pago'),
        'created_attr_id': fields.many2one('hr.attribute', 'Atributo'),
        'attr_allocation_id': fields.many2one('hr.attribute', 'Atributo de asignación'),
        'remaining_attr_id': fields.many2one('hr.attribute', 'Atributo remanente'),
        'paid_quotas': fields.integer('Número de cuotas pagadas', digits=(16,2), readonly=True),
        'paid_amount': fields.float('Monto cancelado', digits=(16,2), readonly=True),
        'remaining_debt': fields.function(_compute_remaining, method=True, string='Saldo', type='float', store=True),
        'active': fields.boolean('Activo'),
        'note': fields.text('Nota'),
    }

    _defaults = {
        'paid_quotas': 0,
        'paid_amount': 0.0,
        'active': True,
    }
                    
    def _check_dates(self, cr, uid, ids, context=None):
        current_date = datetime.now().strftime('%Y-%m-%d')
        for loan in self.browse(cr, uid, ids, context=context):
            if loan.date_start < current_date or loan.date_payment < current_date:
                return False
        return True

    def validar_prestamo(self, cr, uid, employee_id, concept_loan_id, amount, num_quotas, loan=None, context=None):
        concept_loan = self.pool.get('hr.concept.loan').browse(cr, uid, concept_loan_id)
        employee = self.pool.get('hr.employee').browse(cr, uid, employee_id)
        parameter_obj = self.pool.get('hr.parameters')
        prestaciones_employee_obj = self.pool.get('prestaciones.employee')
        
        #Busqueda del concepto que se definio como Concepto de anticipo de prestaciones-----------------------------
        param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_ant_pres_id:
            param_ant_pres_id = parameter_obj.search(cr, uid, [('type', '=', 'cpt_ant_presta'),('payroll_id', '=', None)])
        if not param_ant_pres_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Concepto anticipo prestaciones para la nomina %s. Ir a \
                    la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                    registrar un parametro de tipo Concepto anticipo prestaciones')
                    %(employee.hr_payroll_id.name))

        concept_ant_prest_id = parameter_obj.browse(cr, uid, param_ant_pres_id)[0].concept_ant_presta_id

        #Busqueda del parametro que define el porcentaje maximo para otorgar anticipo de prestaciones-----------------------------
        param_porct_prest = parameter_obj.search(cr, uid, [('type', '=', 'porct_ant_prest'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_porct_prest:
            param_porct_prest = parameter_obj.search(cr, uid, [('type', '=', 'porct_ant_prest'),('payroll_id', '=', None)])
        if not param_porct_prest:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para Maximo porcentaje para anticipo de Prestaciones para la nomina %s. Ir a \
                            la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                            registrar un parametro de tipo Maximo porcentaje para anticipo de Prestaciones')
                            %(employee.hr_payroll_id.name))

        portc_prest = parameter_obj.browse(cr, uid, param_porct_prest)[0].value

        #Busqueda del concepto que se definio como concepto Prestamo sobre utilidades-----------------------------------------
        param_prest_util_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_prest_util'),('payroll_id', '=', employee.hr_payroll_id.id)])
        if not param_prest_util_id:
            param_prest_util_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_prest_util'),('payroll_id', '=', None)])
        if not param_prest_util_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parámetro definido para el concepto Prestamo sobre utilidades para la nómina %s. Ir a \
                            la pestaña Configuración -> Configuraciones de nómina -> Parámetros de nómina y \
                            registrar un parámetro de tipo Concepto Prestamo sobre utilidades')%(employee.hr_payroll_id.name))

        concept_prest_util_brw = parameter_obj.browse(cr, uid, param_prest_util_id)[0].concept_prest_util_id

        #-----------------------------------------------------------------------------------------------------------------------

        if concept_ant_prest_id == concept_loan.concept_id: #Si se trata de un anticipo de prestaciones
            acumulado = prestaciones_employee_obj.acumulado_prestaciones(cr, uid, employee)
            anticipos = prestaciones_employee_obj.anticipos(cr, uid, employee)
            disponible = ((acumulado-anticipos)*portc_prest)/100
            if amount > disponible:
                raise osv.except_osv(_('ERROR!'), _('El monto solicitado de anticipo de Prestaciones Sociales es mayor al nonto \
                        disponible. El empleado dispone de %s (correspondiente al %s del monto total %s acumulado)')
                        %(disponible,str(portc_prest)+'%',acumulado-anticipos))

        elif concept_loan.concept_id <> concept_prest_util_brw: #Si el prestamo solicitado es de los que se descuenta de las nominas habituales
                #se debe verificar que las cuotas a pagar no superen el 30% del sueldo del trabajador
            conceptos_prestamos = []
            total_prest = 0.0
            concept_prest_ids = self.pool.get('hr.concept.loan').search(cr, uid, [])
            #Para cada uno de los conceptos que se difenieron como conceptos de prestamo verificar que no sean prestamo ni de anticipo
            #de prestaciones ni prestamo de utilidades
            for concept_prest in self.pool.get('hr.concept.loan').browse(cr, uid, concept_prest_ids):
                if concept_prest.concept_id <> concept_prest_util_brw and concept_prest.concept_id <> concept_ant_prest_id:
                    conceptos_prestamos.append(concept_prest.concept_id.id)
            if loan:
                attrs_ids = self.pool.get('hr.attribute').search(cr, uid, [('employee_id','=',employee.id),('active','=',True),
                                                                        ('concept_id','in',conceptos_prestamos),('loan_ids','<>',loan.id)])
            else:
                attrs_ids = self.pool.get('hr.attribute').search(cr, uid, [('employee_id','=',employee.id),('active','=',True),
                                                                        ('concept_id','in',conceptos_prestamos)])
                                                                        
            for attrs in self.pool.get('hr.attribute').browse(cr, uid, attrs_ids):
                total_prest = total_prest + attrs.amount
            if employee.hr_payroll_id.hr_payment_calendar_id.type == 'mensual':
                if (total_prest + (amount/num_quotas)) > ((employee.salary/2)*0.30):
                    raise osv.except_osv(_('ERROR!'), _('El monto solicitado mas los montos de los prestamos ya registrado para el empleado (%s)\
                                                    superan el 30%% del sueldo quincenal (%s) del empleado. \
                                                    El prestamo no se puede registrar')%(total_prest,(employee.salary/2)*0.30))
                
            if employee.hr_payroll_id.hr_payment_calendar_id.type == 'diario':
                if (total_prest + (amount/num_quotas)) > ((employee.salary*7)*0.30):
                    raise osv.except_osv(_('ERROR!'), _('El monto solicitado mas los montos de los prestamos ya registrado para el empleado (%s)\
                                                    superan el 30%% del salario (%s) del empleado. \
                                                    El prestamo no se puede registrar')%(total_prest,(employee.salary*7)*0.30))

        return True
        
    def create(self, cr, uid, vals, context=None):
        if context is None:
            context = {}
        concept_loan = self.pool.get('hr.concept.loan').browse(cr, uid, vals['concept_id'])
        employee = self.pool.get('hr.employee').browse(cr, uid, vals['employee_id'])
        parameter_obj = self.pool.get('hr.parameters')
        prestaciones_employee_obj = self.pool.get('prestaciones.employee')
        
        if vals['amount'] <= 0 or vals['num_quotas'] <= 0:
            raise osv.except_osv(_('ERROR!'), _('El monto del préstamo y el número de cuotas a pagar \
                                                    deben ser mayor a cero'))

        #~ self.validar_prestamo(cr, uid, vals['employee_id'], vals['concept_id'], vals['amount'], vals['num_quotas'], None, context)
        
        
        attribute_obj = self.pool.get('hr.attribute')
        attr_dic = {
            'name': concept_loan.concept_id.name,
            'employee_id': vals['employee_id'],
            'date_start': vals['date_start'],
            'amount': round(vals['amount']/vals['num_quotas'],2),
            'concept_id': concept_loan.concept_id.id,
            'notes': 'Creado por el asistente de Prestamos',
            'appears_on_payslip': True,
            'active': True,
            }

        attr_id = attribute_obj.create(cr, uid, attr_dic, context)
        
        # Creacion del atributo de asignacion
        attr_id_2 = 0
        if concept_loan.concept_allocation_id:
            if (not 'date_payment' in vals) or (not vals['date_payment']):
                raise osv.except_osv(_('ERROR'), _('Debe ingresar Fecha de pago de asignación del préstamo'))
            asignacion_attr_dic = {
                'name': 'Asignacion '+concept_loan.concept_allocation_id.name,
                'employee_id': vals['employee_id'],
                'date_start': vals['date_payment'],
                'date_end': vals['date_payment'],
                'amount': vals['amount'],
                'concept_id': concept_loan.concept_allocation_id.id,
                'notes': 'Creado por el asistente de Prestamos',
                'appears_on_payslip': True,
                'active': True,
            }
            attr_id_2 = attribute_obj.create(cr, uid, asignacion_attr_dic, context)
                
        attr_id_3 = 0
        if attr_dic['amount']*vals['num_quotas'] <> vals['amount']:
            # Se debe crear un atributo por la diferencia que se puede dar entre el monto otorgado en el prestamo y el monto a pagar por el empleado
            # debido al redondeo por los decimales. Luego esta cantidad remanente se suma algebraicamente en la ultima cuota del prestamo
            attr_rema_dic = {
                'name': 'Remanente '+concept_loan.concept_id.name,
                'employee_id': vals['employee_id'],
                'date_start': vals['date_start'],
                'amount': vals['amount']-(attr_dic['amount']*vals['num_quotas']),
                'concept_id': concept_loan.concept_id.id,
                'notes': 'Creado por el asistente de Prestamos',
                'appears_on_payslip': False,
                'active': False,
            }
            attr_id_3 = attribute_obj.create(cr, uid, attr_rema_dic, context)

        if attr_id:
            vals['created_attr_id'] = attr_id
            if attr_id_2:
                vals['attr_allocation_id'] = attr_id_2
            if attr_id_3:
                vals['remaining_attr_id'] = attr_id_3
            return super(hr_loan, self).create(cr, uid, vals, context)
        
        return False

    def write(self, cr, uid, ids, vals, context=None):
        if context is None:
            context = {}
        if type(ids) == list:
            ids = ids[0]
        loan_brw = self.pool.get('hr.loan').browse(cr, uid, ids, context)
        attr_brw = loan_brw.created_attr_id
        attr_brw_2 = loan_brw.attr_allocation_id
        attr_brw_3 = loan_brw.remaining_attr_id
        
        if ('amount' in vals and vals['amount'] <= 0) or ('num_quotas' in vals and vals['num_quotas'] <= 0):
            raise osv.except_osv(_('ERROR!'), _('El monto del préstamo y el número de cuotas a pagar \
                                                    deben ser mayor a cero'))
                                                    
        # No se permite modificar prestamos que ya se han comenzado a pagar a menos que la modificacion sea
        # sobre los campo de cuotas pagadas y saldo pagado
        if (loan_brw.paid_quotas > 0 and not ('paid_quotas' in vals or 'paid_amount' in vals)) and (not 'desactivar_prestamo' in context):
            raise osv.except_osv(_('Acción Denegada!'), _('No se permite modificar prestamos que ya se han comenzado a cancelar'))
        
        concept_loan = self.pool.get('hr.concept.loan').browse(cr, uid, vals['concept_id']) if 'concept_id' in vals else loan_brw.concept_id
        amount = vals['amount'] if 'amount' in vals else loan_brw.amount
        num_quotas = vals['num_quotas'] if 'num_quotas' in vals else loan_brw.num_quotas
        #~ self.validar_prestamo(cr, uid, loan_brw.employee_id.id, concept_loan.id, amount, num_quotas, loan_brw, context)
        
        attr_vals = {}
        allocation_attr_vals = {}
        remaining_attr_vals = {}
        for v in vals:
            if v == 'paid_quotas' or v == 'paid_amount':
                return super(hr_loan, self).write(cr, uid, ids, vals, context)
                
            if concept_loan.concept_allocation_id and loan_brw.attr_allocation_id:
                if (('date_payment' in vals) and (not vals['date_payment'])) or \
                    ((not 'date_payment' in vals) and (not loan_brw.date_payment)):
                    raise osv.except_osv(_('ERROR'), _('Debe ingresar Fecha de pago de asignación del préstamo'))
                
                allocation_attr_vals = {
                    'name': 'Asignacion '+concept_loan.concept_allocation_id.name,
                    'employee_id': vals['employee_id'] if 'employee_id' in vals else loan_brw.employee_id.id,
                    'date_start': vals['date_payment'] if 'date_payment' in vals else loan_brw.date_payment,
                    'date_end': vals['date_payment'] if 'date_payment' in vals else loan_brw.date_payment,
                    'amount': vals['amount'] if 'amount' in vals else loan_brw.amount,
                    'concept_id': concept_loan.concept_allocation_id.id,
                    'notes': 'Creado por el asistente de Prestamos',
                    'appears_on_payslip': True,
                }
                
            if v == 'concept_id':
                attr_vals['name'] = concept_loan.concept_id.name
                attr_vals['concept_id'] = concept_loan.concept_id.id                
                remaining_attr_vals['name'] = 'Remanente ' + concept_loan.concept_id.name
                remaining_attr_vals['concept_id'] = concept_loan.concept_id.id
                
            elif v == 'amount':
                attr_vals[v] = round(vals['amount']/vals['num_quotas'],2) if 'num_quotas' in vals else round(vals['amount']/loan_brw.num_quotas,2)
                remaining_attr_vals[v] = vals['amount']-(attr_vals['amount']*vals['num_quotas']) if 'num_quotas' in vals else \
                                            vals['amount']-(attr_vals['amount']*loan_brw.num_quotas)
            elif v == 'num_quotas':
                attr_vals['amount'] = round(vals['amount']/vals['num_quotas'],2) if 'amount' in vals else round(loan_brw.amount/vals['num_quotas'],2)
                remaining_attr_vals['amount'] = vals['amount']-(attr_vals['amount']*vals['num_quotas']) if 'amount' in vals else \
                                            loan_brw.amount-(attr_vals['amount']*vals['num_quotas'])
            else:
                attr_vals[v] = vals[v]
                remaining_attr_vals[v] = vals[v]

        if 'paid_amount' in vals:
            if loan_brw.amount == vals['paid_amount']:
                vals['active'] = False
            else:
                vals['active'] = True

        if self.pool.get('hr.attribute').write(cr, uid, [attr_brw.id], attr_vals, context):
            if attr_brw_2 and allocation_attr_vals:
                self.pool.get('hr.attribute').write(cr, uid, [attr_brw_2.id], allocation_attr_vals, context)
                
            elif not attr_brw_2 and allocation_attr_vals:
                attr_dic = {
                    'name': 'Asignacion '+ concept_loan.concept_allocation_id.name,
                    'employee_id': loan_brw.employee_id.id,
                    'date_start': vals['date_payment'] if 'date_payment' in vals else loan_brw.date_payment,
                    'date_end': vals['date_payment'] if 'date_payment' in vals else loan_brw.date_payment,
                    'amount': allocation_attr_vals['amount']  if 'amount' in vals else loan_brw.amount,
                    'concept_id': concept_loan.concept_allocation_id.id,
                    'notes': 'Creado por el asistente de Prestamos',
                    'appears_on_payslip': False,
                }
                
                attr_id_2 = self.pool.get('hr.attribute').create(cr, uid, attr_dic, context)
                if attr_id_2:
                    vals['attr_allocation_id'] = attr_id_2
                    
            elif attr_brw_2 and not allocation_attr_vals:
                self.pool.get('hr.attribute').write(cr, uid, attr_brw_2.id, {'active':False}, context)
                self.pool.get('hr.attribute').unlink(cr, uid, [attr_brw_2.id], context)
                vals['attr_allocation_id'] = False
                
            if attr_brw_3 and (('amount' in remaining_attr_vals and remaining_attr_vals['amount'] <> 0) or not 'amount' in remaining_attr_vals):
                self.pool.get('hr.attribute').write(cr, uid, [attr_brw_3.id], remaining_attr_vals, context)
                
            elif not attr_brw_3 and ('amount' in remaining_attr_vals and remaining_attr_vals['amount'] <> 0):
                attr_dic = {
                    'name': 'Remanente '+ concept_loan.concept_id.name,
                    'employee_id': loan_brw.employee_id.id,
                    'date_start': loan_brw.date_start,
                    'amount': remaining_attr_vals['amount'],
                    'concept_id': concept_loan.concept_id.id,
                    'notes': 'Creado por el asistente de Prestamos',
                    'appears_on_payslip': False,
                    'active': False,
                }
                attr_id_3 = self.pool.get('hr.attribute').create(cr, uid, attr_dic, context)
                if attr_id_3:
                    vals['remaining_attr_id'] = attr_id_3
            
            elif attr_brw_3 and ('amount' in remaining_attr_vals and remaining_attr_vals['amount'] == 0):
                self.pool.get('hr.attribute').unlink(cr, uid, [attr_brw_3.id], context)
                vals['remaining_attr_id'] = False
                
            return super(hr_loan, self).write(cr, uid, ids, vals, context)
        
        return False

    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        for id in ids:
            loan_brw = self.pool.get('hr.loan').browse(cr, uid, id, context)

            # No se permite eliminar prestamos que ya se han comenzado a pagar
            if loan_brw.paid_quotas > 0:
                raise osv.except_osv(_('Acción Denegada!'), _('No se permite eliminar prestamos que ya se han comenzado a cancelar'))

            attr_brw = loan_brw.created_attr_id
            attr_brw_2 = loan_brw.attr_allocation_id
            attr_brw_3 = loan_brw.remaining_attr_id

            #Se debe desactivar el atributo para luego poder eliminarlo
            self.pool.get('hr.attribute').write(cr, uid, attr_brw.id, {'active':False}, context)
            self.pool.get('hr.attribute').unlink(cr, uid, [attr_brw.id], context)
            if attr_brw_2:
                self.pool.get('hr.attribute').write(cr, uid, attr_brw_2.id, {'active':False}, context)
                self.pool.get('hr.attribute').unlink(cr, uid, [attr_brw_2.id], context)
            if attr_brw_3:
                self.pool.get('hr.attribute').unlink(cr, uid, [attr_brw_3.id], context)
        return super(hr_loan, self).unlink(cr, uid, ids, context)

    def desactivar_prestamo(self, cr, uid, ids, context):
        if not context:
            context = {}
        
        context.update({'desactivar_prestamo':True})
        
        if not isinstance(ids, list):
            ids = [ids]
            
        current_date = datetime.now().strftime('%Y-%m-%d')
        for id in ids:
            loan_brw = self.browse(cr, uid, id)
            created_attr_brw = loan_brw.created_attr_id
            attr_allocation_brw = loan_brw.attr_allocation_id
            remaining_attr_brw = loan_brw.remaining_attr_id
            
            if created_attr_brw:
                note = 'Atributo desactivado por desactivacion del prestamo' 
                note = note if created_attr_brw.notes == False else note + '\n' + created_attr_brw.notes
                self.pool.get('hr.attribute').write(cr, uid, created_attr_brw.id, 
                                                {'active':False, 'notes': note}, context)
            if attr_allocation_brw:
                note = 'Atributo desactivado por desactivacion del prestamo' 
                note = note if attr_allocation_brw.notes == False else note + '\n' + attr_allocation_brw.notes
                self.pool.get('hr.attribute').write(cr, uid, attr_allocation_brw.id, 
                                                {'active':False, 'notes': note}, context)
            if remaining_attr_brw:
                note = 'Atributo desactivado por desactivacion del prestamo' 
                note = note if remaining_attr_brw.notes == False else note + '\n' + remaining_attr_brw.notes
                self.pool.get('hr.attribute').write(cr, uid, remaining_attr_brw.id, 
                                                {'active':False, 'notes': note}, context)
        return self.write(cr, uid, ids, {'active':False, 'date_end': current_date}, context)
    
hr_loan()

class hr_parameters(osv.osv):

    _name = 'hr.parameters'
    _description = 'Parameters'

    _columns = {
        'type':fields.selection([('vacaciones','Dias vacaciones'), #establece los dias a pagar de vacaciones
                ('utilidades','Dias utilidades'), #establece los dias a pagar de utilidades
                ('s_minimo','Sueldo mínimo vigente'),
                ('concepto_sueldo','Concepto Salario'), #Concepto que contiene el sueldo de los empleados (mensual para nomina quincenal o diario para nomina diaria)
                ('concepto_vac','Concepto promedio sueldo vacaciones'), #Concepto que calcula periodicamente el sueldo que luego se usa para calculo de vacaciones
                ('concepto_bono_vac','Concepto Bono Vacacional'), #Concepto bono vanacional
                ('concepto_bono_util','Concepto Utilidades'), #Concepto Utilidades
                ('concepto_prest_util','Concepto Prestamo sobre Utilidades'),
                ('concepto_sso','Concepto S.S.O.'),
                ('concepto_islr','Concepto I.S.L.R.'),
                ('concepto_rpe','Concepto R.P.E.'),
                ('concepto_faov','Concepto F.A.O.V.'),
                ('cpt_util_bruta','Concepto utilidad bruta pagada'),
                ('cpt_ant_presta','Concepto anticipo prestaciones'),
                ('conceptos_util','Conceptos que inciden en Utilidades'),
                ('conceptos_prest','Conceptos que inciden en Prestaciones Sociales'),
                ('conceptos_deven','Conceptos que inciden en I.S.R.L. para Devengado'),
                ('guarderia','Monto máximo a pagar por Guardería'),
                ('diario_orden_p','Diario para ordenes de pago'), #Diario que se usa para la generacion de ordenes de pago
                ('perdida_descanso','Número de días para pérdida del Descanso'),
                ('date_prest','Fecha máxima para cálculo días adicionales de prestaciones'),
                ('porct_ant_prest','Maximo porcentaje para anticipo de Prestaciones'),
                ('max_min_tick_alim','Denominacion maxima y minima para los ticket de alimentacion'),
                ('partner_ca','Proveedor y conceptos de Caja de Ahorro'),
                ('partner_faov','Proveedor y conceptos FAOV'),
                ('correos','Correos para notificaciones de procesos de pago'),
                ('dias_adic_prest','Concepto Dias adicionales prestaciones'),
                ('fecha_prest','Concepto para registro de fecha para calculos de prestaciones'),
                ('concepto_dias_prest','Concepto calculo de prestaciones mensual'),
                ('proc_uno_util','Fechas de inicio y fin de primer Proceso de utilidades'),
                ('conceptos_reserva','Conceptos del asiento de reserva'),
                ('notify_end_contract','Notificaciones de fin de contrato'),
                ('correos_inc_salary','Correos para notificaciones de aumentos de sueldo'),
                ('dias_disfrute_vacaciones','Dias para disfrute de vacaciones'),
                ('estructuras_liq','Estructuras para liquidaciones'),
                ('concepto_vac_venc','Concepto Vacaciones vencidas'),
                ('concepto_dias_vac_ven','Concepto Dias Vacaciones vencidas'),
                ('concepto_dias_adic_vac_ven','Concepto Dias adicionales Vacaciones vencidas'),
                ('concepto_vac_no_disf','Concepto Vacaciones no disfrutadas'),
                ('concepto_util_liq','Concepto Utilidades Legales para Liquidaciones'),
                ('concepto_util_frac_liq','Concepto Utilidades fraccionadas para Liquidaciones'),
                ('concepto_sueldo_prest','Concepto sueldo prestaciones'),
                ('fecha_ultima_ley','Fecha ultima Ley del Trabajo'),
                ('concepto_prest_liq','Concepto Prestaciones de Antiguedad por Liquidacion'),
                ('gerente_th','Gerente Talento Humano'),
                ],'Nombre', size=256,),
        'concept_wage_id': fields.many2one('hr.concept', 'Concepto sueldo'),
        'concept_holidays_id': fields.many2one('hr.concept', 'Concepto promedio sueldo vacaciones'),
        'concept_bon_hol_id': fields.many2one('hr.concept', 'Concepto Bono Vacacional'),
        'concept_util_id': fields.many2one('hr.concept', 'Concepto Utilidades'),
        'concept_prest_util_id': fields.many2one('hr.concept', 'Concepto Prestamo sobre Utilidades'),
        'concept_sso_id': fields.many2one('hr.concept', 'Concepto S.S.O.'),
        'concept_islr_id': fields.many2one('hr.concept', 'Concepto I.S.L.R.'),
        'concept_rpe_id': fields.many2one('hr.concept', 'Concepto R.P.E.'),
        'concept_faov_id': fields.many2one('hr.concept', 'Concepto F.A.O.V.'),
        'concept_util_bruta_id': fields.many2one('hr.concept', 'Concepto utilidad bruta pagada'),
        'concept_ant_presta_id': fields.many2one('hr.concept', 'Concepto anticipo prestaciones'),
        'concept_util_ids': fields.many2many('hr.concept', 'hr_concept_utilities_rel', 'hr_parameter_id', 'hr_concept_id', 'Conceptos que inciden en Utilidades'),
        'concept_deven_ids': fields.many2many('hr.concept', 'hr_concept_devengado_rel', 'hr_parameter_id', 'hr_concept_id', 'Conceptos que inciden en I.S.R.L. para Devengado'),
        'concept_prest_ids': fields.many2many('hr.concept', 'hr_concept_prestaciones_rel', 'hr_parameter_id', 'hr_concept_id', 'Conceptos que inciden en Prestaciones Sociales'),
        'payroll_id': fields.many2one('hr.payroll', 'Nómina'),
        'value': fields.float('Valor', digits=(16,2)),
        'journal_id': fields.many2one('account.journal', 'Diario'), #Parametro que define el diario que se usara en las ordenes de pago
        'date_prest': fields.date('A partir de:', help='Fecha a partir de la cual se acumulan días adicionales de prestaciones'),
        'max_year': fields.integer('Màximo número de años' ,help='Número de años máximo para acumulado de días adicionales de prestaciones'),
        'min_tickets_alim': fields.integer('Minimo' ,help='Denominacion minima para los ticket de alimentacion'),
        'max_tickets_alim': fields.integer('Maximo' ,help='Denominacion maxima para los ticket de alimentacion'),
        'correos': fields.char('Direcciones de correo', help='Escriba las direcciones de correo a las que se hará las notificaciones. \
                                                                Separe cada direccion con una coma(,).', size=256),
        'partner_id': fields.many2one('res.partner', 'Proveedor'),
        'concept_ca_o_pago_ids': fields.many2many('hr.concept', 'hr_concept_ca_o_pago_rel', 'hr_parameter_id', 'hr_concept_id', 'Conceptos',
                                                    help='Conceptos para generar las ordenes de pago de Caja de Ahorro'),
        'concept_faov_o_pago_ids': fields.many2many('hr.concept', 'hr_concept_faov_o_pago_rel', 'hr_parameter_id', 'hr_concept_id', 'Conceptos',
                                                    help='Conceptos para generar las ordenes de pago del Fondo de Ahorro Habitacional\
                                                            siempre que se ejecute un proceso de pago.'),
        'concept_prest_id': fields.many2one('hr.concept', 'Concepto'),
        'date_start_util': fields.date('Utilidades desde:'),
        'date_end_util': fields.date('Hasta:'),
        'concept_reserva_ids': fields.many2many('hr.concept', 'hr_concept_reserva_rel', 'hr_parameter_id', 'hr_concept_id', 'Conceptos del asiento de reserva'),
        'user_id': fields.many2one('res.users', 'Usuario al que se notificará'),
        'days_end_contract': fields.float('Días para finalizar contratos', digits=(16,2)),
        'estruc_liq_ids': fields.many2many('hr.payroll.structure', 'hr_estruc_liq_rel', 'hr_parameter_id', 'hr_structure_id', 'Estructuras para Liquidaciones'),
        'concept_vac_ven_id': fields.many2one('hr.concept', 'Concepto Vacaciones vencidas'),
        'concepto_dias_vac_ven_id': fields.many2one('hr.concept', 'Concepto Dias Vacaciones vencidas'),
        'concepto_dias_adic_vac_ven_id': fields.many2one('hr.concept', 'Concepto Vacaciones no disfrutadas'),
        'concept_vac_no_disf_id': fields.many2one('hr.concept', 'Concepto Vacaciones no disfrutadas'),
        'concept_util_liq_id': fields.many2one('hr.concept', 'Concepto Utilidades Legales para Liquidaciones'),
        'concept_util_frac_liq_id': fields.many2one('hr.concept', 'Concepto Utilidades fraccionadas para Liquidaciones'),
        'concepto_sueldo_prest_id': fields.many2one('hr.concept', 'Concepto sueldo prestaciones'),
        'concepto_prest_liq_id': fields.many2one('hr.concept', 'Concepto Prestaciones de Antiguedad por Liquidacion'),
        'gerente': fields.many2one('hr.employee', 'Gerente Talento Humano', size=256),
    }

    _defaults = {
        'type': 'vacaciones',
    }

    _sql_constraints = [('unique_type_payroll','unique(type, payroll_id)', _('The combination of type and payroll must be unique')),
                        ]

    def _check_type_payroll(self, cr, uid, ids, context=None): #Cuando payroll_id es None el _sql_constraints lo permite por lo que se debe validar por aca
        for hr_parameters in self.browse(cr, uid, ids, context=context):
            if hr_parameters.type == 'concepto_sueldo' or hr_parameters.type == 'concepto_vac' or \
                hr_parameters.type == 'conceptos_util' or hr_parameters.type == 'conceptos_deven' \
                or hr_parameters.type == 'diario_orden_p' or hr_parameters.type == 'perdida_descanso'\
                or hr_parameters.type == 'concepto_bono_vac' or hr_parameters.type == 'concepto_bono_util'\
                or hr_parameters.type == 'concepto_prest_util' or hr_parameters.type == 'concepto_sso'\
                or hr_parameters.type == 'concepto_islr' or hr_parameters.type == 'concepto_rpe'\
                or hr_parameters.type == 'concepto_faov' or hr_parameters.type == 'concepto_rpe'\
                or hr_parameters.type == 'cpt_util_bruta' or hr_parameters.type == 'cpt_ant_presta'\
                or hr_parameters.type == 'conceptos_prest' or hr_parameters.type == 'guarderia'\
                or hr_parameters.type == 'date_prest' or hr_parameters.type == 'porct_ant_prest'\
                or hr_parameters.type == 'max_min_tick_alim' or hr_parameters.type == 'partner_ca'\
                or hr_parameters.type == 'partner_faov' or hr_parameters.type == 'correos':
                if self.search(cr, uid, [('id', '<>', hr_parameters.id),('type', '=', hr_parameters.type),('payroll_id', '=', hr_parameters.payroll_id.id)]):
                    return False

            if hr_parameters.type == 'guarderia':
                if self.search(cr, uid, [('id', '<>', hr_parameters.id),('type', '=', hr_parameters.type)]):
                    return False
        return True

    _constraints = [(_check_type_payroll, "Ya existe un parámetro registrado con las mismas características", ['type,payroll_id'])]

hr_parameters()


class account_move(osv.osv):

    _inherit = "account.move"
    _description = "Account Entry"

    _columns = {
        'hr_process_id': fields.many2one('hr.process', 'Proceso de Nómina'),
    }

account_move()

class account_voucher(osv.osv):

    _inherit = "account.voucher"
    _description = "Account Voucher"

    _columns = {
        'hr_process_id': fields.many2one('hr.process', 'Proceso de Nómina'),
    }

account_voucher()
